#!/bin/bash
#---------------------------------------------------
# (C) Karsten Feilke, 12.06.2007 - D-Link 
#---------------------------------------------------
# Package components:
# (1) This script (create-environment.sh)
# (2) Packages directory
#	- All external Source-Packages except for
#     	  proprietary Intel package:
#	  "IPL-ixp400NpeLibrary-2_3.zip"

#     -> Run this script to get further information
#	 about the proprietary / missing package.
#---------------------------------------------------
#
# Documentation (Horst-Box Development DVD)
# 
# (1) Change the environment depending variables 
#     to your needs
# (2) Check if a ext2 or ext3 partition has been
#     mounted to /"USER_DIRECTORY"/Work-Environment
#     ( This is not needed after installation )
# (3) Execute this script
# (4) Go to /"USER_DIRECTORY"/Work-Environment/build_env/
# (5) Type: "make"
#
# Documentation (General):
#
# (1) Change the environment depending variables
#     to your needs
# (2) Check if subversion is installed / configured
# (2) Check if the project directory exists
# (3) This script will get the "build_env" from a
#     repository, defined by "SVN_PREFIX" from your
#     local subversion. 
# (4) Then this script will run the Update_Webcache.sh
#     script in the Packages directory. This script
#     will check if all external packages like
#     kernel-source, binutils... exist in Package
#     directory (local cache). If packages do not
#     exist, the script downloads them from the web.
#     Future executions do not need the web, all files
#     will be stored in the local cache directory.
# (5) The script will create the "/archive/dl" directory
#     in the actual working directory.
#     For example: /"Project_directory"/build_env/archive/dl/
# (6) The script will copy all external packages to the
#     "/archive/dl" directory
# (7) The build process will later check this directory
#     automatically
# (8) This script will terminate
# (9) Go to the /"Project_directory"/build_env/ and
#     type "make" to build the Horst-Box Image. 
#---------------------------------------------------

#---------------------------------------------------
# Environment depending variables (SVN + Directory) 
# - Change them to your need -		      
#---------------------------------------------------

# Your SVN username
SVN_USER=dlink
# Your CVSD Password
SVN_PASSWORD=dlink	
# The Repository (/home/dlink/SVN-ROOT/ "SVN_PREFIX" /build_env/)
SVN_PREFIX=Source-4.1
# Checkout the "build_env" - (typically do not change) 
SVN_BUILD_ENV=build_env
# Output directory (This directory must exist)
PROJECT_DIR=/home/$(whoami)/Work-Environment

#----------------------------------------------------
# Beginning of internal section ( do not change )
# Create workdirectory and checkout "build_env" (SVN)
# / if necessary get the external packages
#----------------------------------------------------

# -- Test if proprietary Intel-Package exists -- #

if test -f Packages/Build_Env_Webcache/IPL_ixp400NpeLibrary-2_3.zip; then
  echo "Proprietary Intel-Package "IPL_ixp400NpeLibrary-2_3.zip" is present (OK)";
 else echo "----------------------------------------------------------------------------------------------------------------"
  echo "| Missing external Intel-Package: IPL_ixp400NpeLibrary-2_3.zip                                                 |"
  echo "|--------------------------------------------------------------------------------------------------------------|"			
  echo "| We are not authorized to publish this proprietary package at the moment                                      |"
  echo "| Please goto http://downloadfinder.intel.com and search for 'IPL_ixp400NpeLibrary-2_3.zip'                    |"
  echo "| Afterwards copy the Package to 								               |";				
  echo "| /home/dlink/Development/Scripts/Create-Work-Environment/Packages/Build_Env_Webcache/ and restart this script |";
  echo "----------------------------------------------------------------------------------------------------------------"
  exit
fi

# ----------- Test if user is "root" ----------- #

parameter1=${1:-"--user-mode"}
if [ $parameter1 = "--user-mode" ]; then
	if [ `whoami` = "root" ]; then
		echo "---------------------------------------------------------------"
		echo "| It is not recommended to run this script as root            |"
		echo "| Please log in as normal user and restart the script         |"
		echo "| Or start the script with the following option:              |"
		echo "|                                                             |"
		echo "| ./create-environment.sh --root-mode                         |"
		echo "| ( Data will be stored in: /home/root/Work-Environment/ !! ) |"
		echo "---------------------------------------------------------------"
		exit
	fi
else if [ $parameter1 != "--root-mode" ]; then
	echo "Unknown parameter: \"$parameter1\""
	exit
	fi
fi

# ------------  End of Root-Test ---------------#


THIS_DIR=$(pwd)					
# Sample: PROJECT_DIR/Source-4.0/build_env
WORKING_DIR=$PROJECT_DIR/$SVN_PREFIX	
# Create the directory
mkdir $WORKING_DIR -p

# Get the build_env source from local SVN
cd $WORKING_DIR
echo "svn checkout file:///home/dlink/SVN-ROOT/$SVN_PREFIX/$SVN_BUILD_ENV/ ....."
svn checkout file:///home/dlink/SVN-ROOT/$SVN_PREFIX/$SVN_BUILD_ENV/ --username $SVN_USER --password $SVN_PASSWORD

BUILD_ENV_DIR=$WORKING_DIR/$SVN_BUILD_ENV
# The compile process will search for external data in "/archive/dl" 
# If all the packages have been copied to this directory the internet will not be used
# anymore.
mkdir $BUILD_ENV_DIR/archive/dl/ -p

# Copy other external Packages like: BINUTILS, KERNEL, ASTERISK...
echo "Copying external packages to $BUILD_ENV_DIR/archive/dl/ ... [300MB]"
cp $THIS_DIR/Packages/Build_Env_Webcache/* $BUILD_ENV_DIR/archive/dl/

# All done!

