/*
 * (C) Copyright 2006
 * Sven Kolberg, Maxina GmbH & Co KG, s.kolberg@maxina.de
 * Henry Margies, Maxina GmbH & Co KG, h.margies@maxina.de
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include "codec.h"

static struct codec* codec_list[32] = {0};
static int registered_codecs = 0;


/**
 * Puts a codec to the codec list
 */
void register_codec(struct codec* c)
{
	/* check for valid pointer */
	if (!c)
		return;

	/* check if we still have place for new codec */
	if (registered_codecs >= sizeof(codec_list)/sizeof(struct codec*))
		return;

	c->fmt = ((c->dstfmt&0x7) << 3) | (c->srcfmt & 0x7);

	/* put codec to our list */
	codec_list[registered_codecs++]	= c;

	printf("codec registered (%s)\n", c->name);
}


/**
 * Returns the codec which fits to the given format
 */
struct codec* find_codec(unsigned char fmt)
{
	int i;

	/* extract format information */
	fmt &= 0x3f;

	/* go through all elements */
	for (i=0; i<registered_codecs; i++) {
		
		/* check for wanted codec */
		if (fmt == codec_list[i]->fmt)
			return codec_list[i];
	}

	return NULL;
}
