/*
 * (C) Copyright 2006
 * Sven Kolberg, Maxina GmbH & Co KG, s.kolberg@maxina.de
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have progress a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include "console.h"
#include "codec.h"
#include "channel.h"
#include "protocol.h"

/**
 * simple loopback codec
 */

/**
 * Initialize Codec
 */
static void codec_loopback_open(struct channel* channel)
{
	/* nothing to do */
}

/**
 * Deinitialize codec 
 */
static void codec_loopback_close(struct channel* channel)
{
	/* nothing to do */
}


/**
 * Process data 
 */
static void codec_loopback_proceed(struct channel* channel, 
			struct ring_buf* dst, struct ring_buf* src)
{
	/* just loop back data */
	memcpy(dst->data, src->data, src->size);
	dst->size    = src->size;
	dst->samples = src->samples;
}

/*
 * Config-Structure 
 */
static struct codec loopback_codec = {

	.name = "loopback codec",

	.srcfmt = CODEC_SLINEAR,
	.dstfmt = CODEC_SLINEAR,

	.open    = codec_loopback_open,
	.close   = codec_loopback_close,
	.proceed = codec_loopback_proceed,

};


/**
 * register the loopback codec
 */
void register_codec_loopback(void)
{
	register_codec(&loopback_codec);
}
