/*
 * (C) Copyright 2006
 * Sven Kolberg, Maxina GmbH & Co KG, s.kolberg@maxina.de
 * Henry Margies, Maxina GmbH & Co KG, h.margies@maxina.de
 * 
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef __HEADER_CONSOLE__
#define __HEADER_CONSOLE__

#ifdef DEBUG
	#define PDEBUG(fmt, args...)	\
		circ_printf("[%s:%d(%s)] "fmt, __FILE__,__LINE__,__PRETTY_FUNCTION__, ##args)
#else
	#define PDEBUG(fmt, args...)    {}
#endif

#define ASSERT(x)	if(!(x)) { 					\
		printf(	"Assertion ( " #x " ) is wrong!\n"		\
			"in File: %s Function: %s Line %i\n", 		\
			__FILE__, __PRETTY_FUNCTION__, __LINE__);	\
			gdb_halt();}

/**
 * Purpose:
 * 	The Output through the console in U-Boot is a crapy piece of ...
 * 	In other words: it is blocking. it will wait until the last character has been send through 
 * 	the Serial Port, wich takes hours..., this is critical in Interrupt routines
 *
 * Solution:
 * 	Use a circular buffer wich will be emptied by an Interrupt handler or by a "user-space" routine
 */

/**
 * The console printf function
 * \params fmt - The Format string
 * \params ... - The Variables
 */
int circ_printf(const char *fmt, ...);

/**
 * The console output function which should be called
 * \params blocking set to zero if it should return once the buffer was emptied, set to nonzero if it should stay endless in this loop
 * \returns nothing
 */
void circ_output(void);

/**
 * Set breakpoint to this function to break on assertions 
 */
void gdb_halt(void);

#endif /* __HEADER_CONSOLE__ */
