/*
 * (C) Copyright 2006
 * Sven Kolberg, Maxina GmbH & Co KG, s.kolberg@maxina.de
 * Henry Margies, Maxina GmbH & Co KG, h.margies@maxina.de
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>

#include "dma_buffer.h"
#include "console.h"

/* dma buffer for tx */
unsigned char dma_tx_buf[DMA_BUFFER_SIZE*2] = {0};

/* dma buffer for rx */
unsigned char dma_rx_buf[DMA_BUFFER_SIZE*2] = {0};


/**
 * Init DMA for autobuffer mode, but dont enable it yet .
 * MAP DMA1 to SPORT0 RX
 * MAP DMA2 to SPORT0 TX
 * 
 * The organisation of the Buffer is as follows:
 *         iRxBuffer1 ------>  +-----------------------------+
 *                             |                             |
 *                             | DMA_BUFFER_SIZE	     |
 *                             |                             |
 *                             +-----------------------------+
 *                             |                             |
 *                             | DMA_BUFFER_SIZE	     |
 *                             |                             |
 *                             +-----------------------------+                            
 *
 */
void init_dma_wc(void)
{
	/* Set up DMA1 to receive, map DMA1 to Sport0 RX */
	*pDMA1_PERIPHERAL_MAP = 0x1000;
	*pDMA1_IRQ_STATUS |= 0x2;

	/* Start address of data buffer */
	*pDMA1_START_ADDR = dma_rx_buf;

	/* DMA inner loop count */
	*pDMA1_X_COUNT = DMA_BUFFER_SIZE;

	/* Inner loop address increment */
	*pDMA1_X_MODIFY	= 1;
	*pDMA1_Y_MODIFY = 1;
	*pDMA1_Y_COUNT = 2;	
	*pDMA1_X_COUNT = DMA_BUFFER_SIZE;
	
	/* Configure DMA1
	   8-bit transfers, Interrupt on completion, Autobuffer mode */
	*pDMA1_CONFIG = WNR | WDSIZE_8 | DI_EN | 0x1000 | DI_SEL | DMA2D; 

	/* Set up DMA2 to transmit, map DMA2 to Sport0 TX */
	*pDMA2_PERIPHERAL_MAP = 0x2000;
	
	/* Configure DMA2 8-bit transfers, Autobuffer mode */
	*pDMA2_CONFIG = WDSIZE_8 | 0x1000 | DMA2D;

	/* Start address of data buffer */
	*pDMA2_START_ADDR = dma_tx_buf;

	/* DMA inner loop count */
	*pDMA2_X_COUNT = DMA_BUFFER_SIZE;

	/* Inner loop address increment */
	*pDMA2_X_MODIFY	= 1;
	*pDMA2_Y_MODIFY = 1;
	*pDMA2_Y_COUNT = 2;
	*pDMA2_X_COUNT = DMA_BUFFER_SIZE;
}



/**
 * Enable DMA and SPORT0
 * before calling this routine, the clock of the SPORT-interface must be applied
 * after calling this routine the sport0_rx_isr should trigger with a rate of 1kHz
 */
void enable_dma_sport0(void)
{
	/* enable DMAs */
	*pDMA2_CONFIG |= DMAEN;
	*pDMA1_CONFIG |= DMAEN;
	__builtin_bfin_ssync();

	/* enable sport0 Tx and Rx */
	*pSPORT0_TCR1 |= TSPEN;
	*pSPORT0_RCR1 |= RSPEN;
	__builtin_bfin_ssync();
}

