/*
 * (C) Copyright 2006
 * Sven Kolberg, Maxina GmbH & Co KG, s.kolberg@maxina.de
 * Henry Margies, Maxina GmbH & Co KG, h.margies@maxina.de
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have progress a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <malloc.h>

#include "codec.h"
#include "console.h"
#include "protocol.h"
#include "profile.h"

#ifdef USE_ECHO_AEC

#include "echocancel/aec.h"
//#include "aec.h"

/* Include header for alaw and ulaw conversion */
#include <filter.h>


/* The Function wich will handle a frame */
static void echoc_proceed(struct channel* channel, 
			struct ring_buf* dst, struct ring_buf* src)
{
	short lineardata[16];
	int i;
	struct aec *aec = (struct aec *) channel->codec_data;

	dst->size    = src->size;
	dst->samples = src->samples;

	/* convert from alaw to linear */
	mu8_expand(src->data, lineardata, src->size);

	STATIC_PROFILE_START(doAEC);
	/* work on the data ... */
	for (i=0; i<src->size; i+=2) {

		/* i   == mic 
		 * i+1 == speaker 
		 */

		/* Echocancel */
		lineardata[i] = 
			aec_doAEC(aec, lineardata[i], lineardata[i+1]);

	}
	STATIC_PROFILE_STOP(doAEC);
	if (profile_doAEC.cycles > 1000 * 500)
		profile_doAEC.user_counter++;

	/* convert from linear to alaw */
	mu8_compress(lineardata, dst->data, dst->size);

}

/**
 * Initialize echo canceller for a new channel
 */
static void echoc_open(struct channel* channel)
{
	struct aec *aec;

	/* Malloc space for the AEC echocanceler */
	aec = (struct aec *) malloc(sizeof (struct aec));
	ASSERT(aec);

	/* initialize all to zero */
	memset(aec, 0, sizeof(struct aec));
	
	/* Init the structure */
	aec_init(aec);

	/* save aec to channel */
	channel->codec_data = aec;
}

/**
 * Free all memory which was allocated in open
 */
static void echoc_close(struct channel* channel)
{
	if(channel->codec_data)
		free(channel->codec_data);
}

/*
 * Config-Structure 
 */
static struct codec echoc = {

	.name = "adaptive echo canceller",

	.srcfmt = ECHOC_FMT_ALAW,
	.dstfmt = ECHOC_FMT_ALAW,

	.open    = echoc_open,
	.close   = echoc_close,
	.proceed = echoc_proceed,

};

void register_echoc_aec(void)
{
	register_codec(&echoc);
}


#endif
