/*
 * (C) Copyright 2006
 * Sven Kolberg, Maxina GmbH & Co KG, s.kolberg@maxina.de
 * Henry Margies, Maxina GmbH & Co KG, h.margies@maxina.de
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have progress a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <malloc.h>

#include "codec.h"
#include "console.h"
#include "protocol.h"

#ifdef USE_ECHO_SPEEX

#define NN	128
#define TAIL	1024


/* Include header for alaw and ulaw conversion */
#include <filter.h>

/* Speex includec */
#include <speex/speex_echo.h>

/* The Function wich will handle a frame */
static void echoc_proceed(struct channel* channel, 
			struct ring_buf* dst, struct ring_buf* src)
{
	unsigned short lineardata[16];
	int i;

	dst->size    = src->size;
	dst->samples = src->samples;

	/* convert from alaw to linear */
	a8_expand(src->data, lineardata, src->size);

	/* work on the data ... */
	for (i=0; i<src->size; i+=2) {

		/* Echocancel */
		lineardata[i] =   lineardata[i]; 
		lineardata[i+1] = lineardata[i+1];

	}

	/* convert from linear to alaw */
	a8_compress(lineardata, dst->data, dst->size);
}

/**
 * Initialize echo canceller for a new channel
 */
static void echoc_open(struct channel* channel)
{
	SpeexEchoState *st;

	st = speex_echo_state_init(NN, TAIL);

	/* save speex cfg to channel */
	channel->codec_data = st;
}

/**
 * Free all memory which was allocated in open
 */
static void echoc_close(struct channel* channel)
{
	if(channel->codec_data)
		free(channel->codec_data);
}

/*
 * Config-Structure 
 */
static struct codec echoc = {

	.name = "speex echo canceller",

	.srcfmt = ECHOC_FMT_ALAW,
	.dstfmt = ECHOC_FMT_ALAW,

	.open    = echoc_open,
	.close   = echoc_close,
	.proceed = echoc_proceed,

};

void register_echoc_speex(void)
{
	register_codec(&echoc);
}


#endif
