 /* aec.c
 *
 * Copyright (C) DFS Deutsche Flugsicherung (2004, 2005). 
 * All Rights Reserved.
 *
 * Copyright (C) Sven Kolberg (2006).
 * Conversion from floatingpoint to fixpoint
 *
 * Acoustic Echo Cancellation NLMS-pw algorithm
 *
 * Version 0.3.4 H-infinity LMS-pw algorithm
 * Version 0.3.1 Allow change of stability parameter delta
 * Version 0.3 filter created with www.dsptutor.freeuk.com
 */
#include "iir.h"
#include "aec.h"

#include <vector.h>
#include <common.h>

#if WIDEB==2
	#define RATE (2*8000)
#else
	#define RATE (8000)
#endif

/* dB Values */

fixpoint M0dB 	=  to_fixpoint(1.0f	, NFIXPOINT_ACC);		
fixpoint M3dB 	=  to_fixpoint(0.71f	, NFIXPOINT_ACC);
fixpoint M6dB 	=  to_fixpoint(0.50f	, NFIXPOINT_ACC);
fixpoint M9dB 	=  to_fixpoint(0.35f	, NFIXPOINT_ACC);
fixpoint M12dB 	=  to_fixpoint(0.25f	, NFIXPOINT_ACC);
fixpoint M18dB 	=  to_fixpoint(0.125f	, NFIXPOINT_ACC);
fixpoint M24dB 	=  to_fixpoint(0.063f	, NFIXPOINT_ACC);


/* dB values for 16bit PCM */
/* MxdB_PCM = 32767 * 10 ^(x / 20) */
fixpoint M10dB_PCM = to_fixpoint(10362, NFIXPOINT_ACC);		
fixpoint M20dB_PCM = to_fixpoint(3277 , NFIXPOINT_ACC);	
fixpoint M25dB_PCM = to_fixpoint(1843 , NFIXPOINT_ACC);	
fixpoint M30dB_PCM = to_fixpoint(1026 , NFIXPOINT_ACC); 
fixpoint M35dB_PCM = to_fixpoint(583  , NFIXPOINT_ACC); 
fixpoint M40dB_PCM = to_fixpoint(328  , NFIXPOINT_ACC); 
fixpoint M45dB_PCM = to_fixpoint(184  , NFIXPOINT_ACC); 
fixpoint M50dB_PCM = to_fixpoint(104  , NFIXPOINT_ACC); 
fixpoint M55dB_PCM = to_fixpoint(58   , NFIXPOINT_ACC); 
fixpoint M60dB_PCM = to_fixpoint(33   , NFIXPOINT_ACC); 
fixpoint M65dB_PCM = to_fixpoint(18   , NFIXPOINT_ACC); 
fixpoint M70dB_PCM = to_fixpoint(10   , NFIXPOINT_ACC); 
fixpoint M75dB_PCM = to_fixpoint(6    , NFIXPOINT_ACC); 
fixpoint M80dB_PCM = to_fixpoint(3    , NFIXPOINT_ACC); 
fixpoint M85dB_PCM = to_fixpoint(2    , NFIXPOINT_ACC); 
fixpoint M90dB_PCM = to_fixpoint(1    , NFIXPOINT_ACC); 

fixpoint MAXPCM = to_fixpoint(32767, NFIXPOINT_ACC);

/* Design constants (Change to fine tune the algorithms */

/* The following values are for hardware AEC and studio quality 
 * microphone */


/* minimum energy in xf. Range: M70dB_PCM to M50dB_PCM. Should be equal
 * to microphone ambient Noise level */
fixpoint NoiseFloor = to_fixpoint(58   , NFIXPOINT_ACC);


/* Ageing multiplier for LMS memory vector w */
fixpoint Leaky = to_fixpoint(0.9999f, NFIXPOINT_ACC);

/* Double Talk Detector Speaker/Microphone Threshold. Range <=1
 * Large value (M0dB) is good for Single-Talk Echo cancellation, 
 * small value (M12dB) is good for Doulbe-Talk AEC */
fixpoint GeigelThreshold = to_fixpoint(0.50f    , NFIXPOINT_ACC);

/* Double Talk Detector hangover in taps. 
 * Small Value (0ms = No DTD) is good for Single-Talk Echo cancellation,
 * Large Value (30ms) is good for Double-Talk Echo cancellation.
 */
int Thold  = 30 * WIDEB * 8;  

/* for Non Linear Processor. Range >0 to 1. Large value (M0dB) is good
 * for Double-Talk, small value (M12dB) is good for Single-Talk */
fixpoint NLPAttenuation = to_fixpoint(0.25f    , NFIXPOINT_ACC); 

// Adrian soft decision DTD 
// left point. X is ratio, Y is stepsize
fixpoint STEPX1 = to_fixpoint(1.0, NFIXPOINT_ACC) , STEPY1 = to_fixpoint(1.0, NFIXPOINT_ACC);
// right point. STEPX2=2.0 is good double talk, 3.0 is good single talk.
fixpoint STEPX2 = to_fixpoint(2.5, NFIXPOINT_ACC), STEPY2 = to_fixpoint(0, NFIXPOINT_ACC);
sfixpoint ALPHAFAST = to_fixpoint(1.0f / 100.0f, SFIXPOINT_ACC);
sfixpoint ALPHASLOW = to_fixpoint(1.0f / 20000.0f, SFIXPOINT_ACC);



/**
 * Vector Dot Product 
 * \params a
 * \params b
 */
fixpoint AEC_dotp(struct aec *aec, sfixpoint a[], fixpoint b[])
{
	fixpoint sum;
	sum.lval = 0;
	int i;
#if 0
	for(i=0; i< NLMS_LEN; i++)
	{
		sum = 	FP_ADD_FFF(
				sum, 
				FP_MUL_SFF( 
					a[i], 
					b[i]
				)
			);
	}
#elif 0
	unsigned long  l = 0, m = 0, h = 0;

	for(i=0; i< NLMS_LEN; i++)
	{
	//	l += ((unsigned long long)a[i].val.l * (unsigned long long) b[i].val.l);
		m += ((unsigned long long)a[i].val.l * (long long) b[i].val.h);
		m += ((long long)a[i].val.h * (unsigned long long) b[i].val.l);
		h += ((long long)a[i].val.h * (long long) b[i].val.h);
	}
	
	sum.lval  = 	(l + (m << 16) + (h << 32)) >> ( SFIXPOINT_ACC + NFIXPOINT_ACC - NFIXPOINT_ACC);
#elif 0
	unsigned long long  m = 0, h = 0;

	for(i=0; i< NLMS_LEN; i++)
	{
	//	l += ((unsigned long long)a[i].val.l * (unsigned long long) b[i].val.l);
		m += ((unsigned long)a[i].val.l * (long) b[i].val.h);
		m += ((long )a[i].val.h * (unsigned long) b[i].val.l);
		h += ((long )a[i].val.h * (long) b[i].val.h);
	}
	
	sum.lval  = /*	(l >> ( SFIXPOINT_ACC + NFIXPOINT_ACC - NFIXPOINT_ACC)) +*/
			(m << (16 - (SFIXPOINT_ACC + NFIXPOINT_ACC - NFIXPOINT_ACC))) +
			(h << (32 - (SFIXPOINT_ACC + NFIXPOINT_ACC - NFIXPOINT_ACC))) ;
#else
	sum.lval = cvecdot_fr32((long *) a , (long *) b, NLMS_LEN);
#endif
	return sum;
}

fixpoint AEC_getambient(struct aec *aec)
{
	return aec->dfast;
}

void AEC_setambient(struct aec *aec, fixpoint Min_xf)
{
	// subtract old data
	aec->dotp_xf_xf = FP_SUB_DDD (
				aec->dotp_xf_xf,
				aec->delta
			);
	
	// calculate power for the current sample
	aec->delta = 	FP_MUL_FFD (
				INT_TO_FP_fixpoint(NLMS_LEN-1),
				FP_MUL_FFF (
						Min_xf,
						Min_xf
				)
			);
	
	// add new delta
	aec->dotp_xf_xf = 	FP_ADD_DDD (
				aec->dotp_xf_xf,
				aec->delta
			);
			
}

void AEC_setgain(struct aec *aec, fixpoint gain_) {
	aec->gain = gain_;
}

/**
 * Initialize the AEC
 */
void aec_init(struct aec *aec)
{
	FIR_HP_300Hz_init(&aec->cutoff);

	aec->j 	   = NLMS_EXT;
	aec->delta = INT_TO_FP_dfixpoint(0);

	AEC_setambient(aec, NoiseFloor);
	
	aec->gain = INT_TO_FP_fixpoint(1);

	aec->dfast = aec->dslow = M75dB_PCM;

	aec->xfast = aec->xslow = M80dB_PCM;

	IIR1_init(&aec->Fx,  (fixpoint) to_fixpoint(2000.0f/RATE, NFIXPOINT_ACC));
	IIR1_init(&aec->Fe,  (fixpoint) to_fixpoint(2000.0f/RATE, NFIXPOINT_ACC));
	IIR1_init(&aec->Fxx, (fixpoint) to_fixpoint(2000.0f/RATE, NFIXPOINT_ACC));
	IIR1_init(&aec->Fdd, (fixpoint) to_fixpoint(2000.0f/RATE, NFIXPOINT_ACC));

 //	const float M = (STEPY2 - STEPY1) / (STEPX2 - STEPX1);
	aec->M = 	FP_DIV_FFF(
			FP_SUB_FFF(
				STEPY2,
				STEPY1
			),
			FP_SUB_FFF(
				STEPX2,
				STEPX1
			)
		);

	aec->fdwdisplay = -1;
}

// for logging
fixpoint ratio;

/**
 * Double talk detection
 */

fixpoint AEC_dtd(struct aec *aec, fixpoint _d, fixpoint _x)
{
	fixpoint stepsize;

	aec->dfast = FP_ADD_FFF(
			aec->dfast,
			FP_MUL_SFF(
				ALPHAFAST,
				FP_SUB_FFF(
					FP_ABS_fixpoint( _d ),
					aec->dfast
				)
			)
		);

	aec->xfast = FP_ADD_FFF(
			aec->xfast,
			FP_MUL_SFF(
				ALPHAFAST,
				FP_SUB_FFF(
					FP_ABS_fixpoint( _x ),
					aec->xfast
				)
			)
		);
	aec->dslow = FP_ADD_FFF(
			aec->dslow,
			FP_MUL_SFF(
				ALPHASLOW,
				FP_SUB_FFF(
					FP_ABS_fixpoint( _d ),
					aec->dslow
				)
			)
		);

	aec->xslow = FP_ADD_FFF(
			aec->xslow,
			FP_MUL_SFF(
				ALPHASLOW,
				FP_SUB_FFF(
					FP_ABS_fixpoint( _x ),
					aec->xslow
				)
			)
		);

//	if (FP_IS_LESS_fixpoint(xfast,M55dB_PCM)) {
	if (FP_IS_LESS_fixpoint(aec->xfast,M55dB_PCM)) {
		return (fixpoint) to_fixpoint(0, 0);   // no Spk signal
	}
//	if (FP_IS_LESS_fixpoint(dfast,M70dB_PCM)) {
	if (FP_IS_LESS_fixpoint(aec->dfast,M55dB_PCM)) {
	  return (fixpoint) to_fixpoint(0, 0);   // no Mic signal
	}
	
	// ratio of NFRs
//	ratio = (dfast * xslow) / (dslow * xfast);
	ratio = FP_DIV_FFF(
			FP_MUL_FFF(
				aec->dfast,
				aec->xslow
			),
			FP_MUL_FFF(
				aec->dslow,
				aec->xfast
			)
		);
	// begrenzte lineare Kennlinie

	if (       FP_IS_LESS_fixpoint(ratio,STEPX1)) {
		stepsize = STEPY1;
	} else if (FP_IS_LESS_fixpoint(STEPX2,ratio)) {
		stepsize = STEPY2;
	} else {
		// Punktrichtungsform einer Geraden
		//stepsize = M * (ratio - STEPX1) + STEPY1;
		//
		stepsize = 	FP_ADD_FFF (
					FP_MUL_FFF (
						aec->M,
						FP_SUB_FFF (
							ratio,
							STEPX1
						)
					),
					STEPY1
				);
	}
	return stepsize;

}

inline fixpoint AEC_nlms_pw(struct aec *aec, fixpoint d, fixpoint x_, fixpoint stepsize)
{
	fixpoint e;
	fixpoint ef;

	aec->x[aec->j] = x_; 
	aec->xf[aec->j] = IIR1_highpass(&aec->Fx, x_);   // pre-whitening of x

	// calculate error value 
	// (mic signal - estimated mic signal from spk signal)

	e = FP_SUB_FFF (
		d,
		AEC_dotp(
			aec,
			aec->w,
			aec->x+aec->j
		)
	);
	
	ef  = IIR1_highpass(&aec->Fe, e); //pre-whitening of e


	// optimize: iterative dotp(xf, xf)
	//dotp_xf_xf += (fp_to_float_fixpoint(xf[j]) * fp_to_float_fixpoint(xf[j]) - fp_to_float_fixpoint(xf[j + NLMS_LEN - 1]) * fp_to_float_fixpoint(xf[j + NLMS_LEN - 1]));
	
	aec->dotp_xf_xf = 	FP_ADD_DDD(
				aec->dotp_xf_xf,
				FP_SUB_DDD(
					FP_MUL_FFD(
						aec->xf[aec->j],
						aec->xf[aec->j]
					),
					FP_MUL_FFD(
						aec->xf[aec->j + NLMS_LEN - 1],
						aec->xf[aec->j + NLMS_LEN - 1]
					)
				)
			);

	if (FP_IS_LESS_fixpoint( (fixpoint) to_fixpoint (0,0), stepsize)) {
		aec->w_valid = 1;
		// calculate variable step size
		//REAL mikro_ef = fp_to_float_fixpoint(stepsize) * fp_to_float_fixpoint(ef) / fp_to_float_dfixpoint(dotp_xf_xf);
		sfixpoint mikro_ef = FP_DIV_FDS(
					FP_MUL_FFF (
						stepsize,
						ef
					),
					aec->dotp_xf_xf
				);

  
		// update tap weights (filter learning)
#if 0
/* not optimized */
		int i;
		for (i = 0; i < NLMS_LEN; i += 2) {
			// optimize: partial loop unrolling
			aec->w[i] = FP_ADD_SSS (
					aec->w[i],
					FP_MUL_SFS(
						mikro_ef,
						aec->xf[i + aec->j]
					)
				);
			aec->w[i + 1] = 	FP_ADD_SSS (
						aec->w[i+1],
						FP_MUL_SFS(
							mikro_ef,
							aec->xf[i + aec->j + 1]
						)
					);

		}
#elif 1
		long *imlong = (long *) &mikro_ef;
		vect_mul_update_32(	(long *) aec->w,
					(long *) &aec->xf[aec->j],
					*imlong,
					NLMS_LEN);
#else

#endif

	} else {
		// ageing of w[] (leaky NLMS)
		if (aec->w_valid) {
			aec->w_valid = 0;
			memset(aec->w, 0, sizeof(aec->w));  // brutal forgetting
		}
	}

	if (--aec->j < 0) {
		// optimize: decrease number of memory copies
		aec->j = NLMS_EXT;
		memmove(aec->x + aec->j + 1, aec->x, (NLMS_LEN - 1) * sizeof(fixpoint));
		memmove(aec->xf + aec->j + 1, aec->xf, (NLMS_LEN - 1) * sizeof(fixpoint));
	}
	// Saturation
	if        (FP_IS_LESS_fixpoint(MAXPCM,e) ){
		return MAXPCM;
	} else if (FP_IS_LESS_fixpoint(e, FP_NEG_fixpoint(MAXPCM))) {
		return FP_NEG_fixpoint(MAXPCM);
	} else {
		return e;
	}
}

int aec_doAEC(struct aec *aec, int d_, int x_)
{

	fixpoint _d = INT_TO_FP_fixpoint(d_);
	fixpoint _x = INT_TO_FP_fixpoint(x_);
	fixpoint stepsize;

	// Mic Highpass Filter - to remove DC
	_d = IIR_HP_highpass(&aec->acMic, _d);

	// Mic Highpass Filter - cut-off below 300Hz
	_d = FIR_HP_300Hz_highpass(&aec->cutoff, _d);
	
	// Amplify, for e.g. Soundcards with -6dB max. volume
	_d = 	FP_MUL_FFF(
			_d,
			aec->gain
		);
	
	// Spk Highpass Filter - to remove DC
	_x = IIR_HP_highpass(&aec->acSpk, _x);


	// Double Talk Detector
	stepsize = AEC_dtd(aec, _d, _x);
	
	//Acoustic Echo Cancellation
	_d = AEC_nlms_pw(aec, _d, _x, stepsize); 

	return  FP_TO_INT_fixpoint(_d);
}

