#ifndef __ADAPTIVE_ECHO_CANCELATION_H__
#define __ADAPTIVE_ECHO_CANCELATION_H__  

#include "iir.h"
#include "../profile.h" 
/* maximum NLMS filter length in taps. A longer filter length gives 
 * better Echo Cancellation, but maybe slower convergence speed and
 * needs more CPU power (Order of NLMS is linear) */
#define NLMS_LEN  (40*WIDEB*8)

// Extention in taps to reduce mem copies
#define NLMS_EXT  (10*8)

// block size in taps to optimize DTD calculation 
#define DTD_LEN   16


struct aec {
	// Time domain Filters
	struct IIR_HP acMic, acSpk;          // DC-level remove Highpass)
	struct FIR_HP_300Hz cutoff;          // 150Hz cut-off Highpass
	fixpoint gain;                    // Mic signal amplify

	struct IIR1 Fx, Fe;                  // pre-whitening Highpass for x, e
	struct IIR1 Fxx, Fdd;                  // pre-whitening Highpass
	// Geigel DTD (Double Talk Detector)

	int hangover;
	// optimize: less calculations for max()
	int dtdCnt;
	int dtdNdx;
	

	// Leaky NLMS-pw
	fixpoint x[NLMS_LEN + NLMS_EXT];  // tap delayed loudspeaker signal
	fixpoint xf[NLMS_LEN + NLMS_EXT]; // pre-whitening tap delayed signal
	sfixpoint w[NLMS_LEN];             // tap weights
	fixpoint ws[NLMS_LEN];            // tap weights sums

	int j;                        // optimize: less memory copies
	dfixpoint dotp_xf_xf;

	dfixpoint delta;

	int w_valid;	      	      	// vector w is valid, not zero
	      
	// w vector display
	int fdwdisplay;               // TCP file descriptor
	int dumpcnt;                  // wdisplay output counter
	
	fixpoint dfast, xfast;    
	fixpoint dslow, xslow;

	fixpoint M;
};

fixpoint AEC_getambient(struct aec *aec);
void AEC_setambient(struct aec *aec, fixpoint Min_xf);
void aec_init(struct aec *aec);
int aec_doAEC(struct aec *aec, int d_, int x_);


#endif /* __ADAPTIVE_ECHO_CANCELATION_H__ */
