 /* aec.cpp
 *
 * Copyright (C) DFS Deutsche Flugsicherung (2004, 2005). 
 * All Rights Reserved.
 *
 * Acoustic Echo Cancellation NLMS-pw algorithm
 *
 * Version 0.3.4 H-infinity LMS-pw algorithm
 * Version 0.3.1 Allow change of stability parameter delta
 * Version 0.3 filter created with www.dsptutor.freeuk.com
 */
#include "iir.h"








int RATE=WIDEB*8000;

// use double if your CPU does software-emulation of float

/* dB Values */
fixpoint M0dB 	=  to_fixpoint(1.0f	, NFIXPOINT_ACC);		
fixpoint M3dB 	=  to_fixpoint(0.71f	, NFIXPOINT_ACC);
fixpoint M6dB 	=  to_fixpoint(0.50f	, NFIXPOINT_ACC);
fixpoint M9dB 	=  to_fixpoint(0.35f	, NFIXPOINT_ACC);
fixpoint M12dB 	=  to_fixpoint(0.25f	, NFIXPOINT_ACC);
fixpoint M18dB 	=  to_fixpoint(0.125f	, NFIXPOINT_ACC);
fixpoint M24dB 	=  to_fixpoint(0.063f	, NFIXPOINT_ACC);

/* dB values for 16bit PCM */
/* MxdB_PCM = 32767 * 10 ^(x / 20) */
fixpoint M10dB_PCM = to_fixpoint(10362.0f, NFIXPOINT_ACC);		
fixpoint M20dB_PCM = to_fixpoint(3277.0f , NFIXPOINT_ACC);	
fixpoint M25dB_PCM = to_fixpoint(1843.0f , NFIXPOINT_ACC);	
fixpoint M30dB_PCM = to_fixpoint(1026.0f , NFIXPOINT_ACC); 
fixpoint M35dB_PCM = to_fixpoint(583.0f  , NFIXPOINT_ACC); 
fixpoint M40dB_PCM = to_fixpoint(328.0f  , NFIXPOINT_ACC); 
fixpoint M45dB_PCM = to_fixpoint(184.0f  , NFIXPOINT_ACC); 
fixpoint M50dB_PCM = to_fixpoint(104.0f  , NFIXPOINT_ACC); 
fixpoint M55dB_PCM = to_fixpoint(58.0f   , NFIXPOINT_ACC); 
fixpoint M60dB_PCM = to_fixpoint(33.0f   , NFIXPOINT_ACC); 
fixpoint M65dB_PCM = to_fixpoint(18.0f   , NFIXPOINT_ACC); 
fixpoint M70dB_PCM = to_fixpoint(10.0f   , NFIXPOINT_ACC); 
fixpoint M75dB_PCM = to_fixpoint(6.0f    , NFIXPOINT_ACC); 
fixpoint M80dB_PCM = to_fixpoint(3.0f    , NFIXPOINT_ACC); 
fixpoint M85dB_PCM = to_fixpoint(2.0f    , NFIXPOINT_ACC); 
fixpoint M90dB_PCM = to_fixpoint(1.0f    , NFIXPOINT_ACC); 

fixpoint MAXPCM = to_fixpoint(32767.0f, NFIXPOINT_ACC);

/* Design constants (Change to fine tune the algorithms */

/* The following values are for hardware AEC and studio quality 
 * microphone */

/* maximum NLMS filter length in taps. A longer filter length gives 
 * better Echo Cancellation, but maybe slower convergence speed and
 * needs more CPU power (Order of NLMS is linear) */
#define NLMS_LEN  (40*WIDEB*8)

/* minimum energy in xf. Range: M70dB_PCM to M50dB_PCM. Should be equal
 * to microphone ambient Noise level */
fixpoint NoiseFloor = to_fixpoint(58.0f   , NFIXPOINT_ACC);


/* Ageing multiplier for LMS memory vector w */
fixpoint Leaky = to_fixpoint(0.9999f, NFIXPOINT_ACC);

/* Double Talk Detector Speaker/Microphone Threshold. Range <=1
 * Large value (M0dB) is good for Single-Talk Echo cancellation, 
 * small value (M12dB) is good for Doulbe-Talk AEC */
fixpoint GeigelThreshold = to_fixpoint(0.50f    , NFIXPOINT_ACC);

/* Double Talk Detector hangover in taps. 
 * Small Value (0ms = No DTD) is good for Single-Talk Echo cancellation,
 * Large Value (30ms) is good for Double-Talk Echo cancellation.
 */
int Thold  = 30 * WIDEB * 8;  

/* for Non Linear Processor. Range >0 to 1. Large value (M0dB) is good
 * for Double-Talk, small value (M12dB) is good for Single-Talk */
fixpoint NLPAttenuation = to_fixpoint(0.25f    , NFIXPOINT_ACC); 

// Adrian soft decision DTD 
// left point. X is ratio, Y is stepsize
fixpoint STEPX1 = to_fixpoint(1.0, NFIXPOINT_ACC) , STEPY1 = to_fixpoint(1.0, NFIXPOINT_ACC);
// right point. STEPX2=2.0 is good double talk, 3.0 is good single talk.
fixpoint STEPX2 = to_fixpoint(2.5, NFIXPOINT_ACC), STEPY2 = to_fixpoint(0, NFIXPOINT_ACC);
sfixpoint ALPHAFAST = to_fixpoint(1.0f / 100.0f, SFIXPOINT_ACC);
sfixpoint ALPHASLOW = to_fixpoint(1.0f / 20000.0f, SFIXPOINT_ACC);


// Extention in taps to reduce mem copies
#define NLMS_EXT  (10*8)

// block size in taps to optimize DTD calculation 
#define DTD_LEN   16

struct AEC {
	// Time domain Filters
	struct IIR_HP acMic, acSpk;          // DC-level remove Highpass)
	struct FIR_HP_300Hz cutoff;          // 150Hz cut-off Highpass
	fixpoint gain;                    // Mic signal amplify

	struct IIR1 Fx, Fe;                  // pre-whitening Highpass for x, e
	struct IIR1 Fxx, Fdd;                  // pre-whitening Highpass
	// Geigel DTD (Double Talk Detector)

	int hangover;
	// optimize: less calculations for max()
	int dtdCnt;
	int dtdNdx;
	

	// Leaky NLMS-pw
	fixpoint x[NLMS_LEN + NLMS_EXT];  // tap delayed loudspeaker signal
	fixpoint xf[NLMS_LEN + NLMS_EXT]; // pre-whitening tap delayed signal
	sfixpoint w[NLMS_LEN];             // tap weights
	fixpoint ws[NLMS_LEN];            // tap weights sums

	int j;                        // optimize: less memory copies
	dfixpoint dotp_xf_xf;

	dfixpoint delta;

	int w_valid;	      	      	// vector w is valid, not zero
	      
	// w vector display
	int fdwdisplay;               // TCP file descriptor
	int dumpcnt;                  // wdisplay output counter
	
	fixpoint dfast, xfast;    
	fixpoint dslow, xslow;

	fixpoint M;
};

#define acMic 		(aec->acMic)
#define acSpk 		(aec->acSpk)
#define cutoff 		(aec->cutoff)
#define gain 		(aec->gain)
#define Fx		(aec->Fx)
#define Fe		(aec->Fe)
#define Fxx		(aec->Fxx)
#define Fdd		(aec->Fdd)
#define max_max_x       aec->max_max_x
#define hangover        (aec->hangover)
#define max_x           (aec->max_x)
#define dtdCnt          (aec->dtdCnt)
#define dtdNdx          (aec->dtdNdx)
#define xfast1          (aec->xfast1)
#define max_max_u       (aec->max_max_u)
#define max_u           aec->max_u
#define c_max_u         (aec->c_max_u)
#define i_max_u         (aec->i_max_u)
#define x               aec->x
#define xf              aec->xf
#define w               aec->w
#define ws              aec->ws
#define j               (aec->j)
#define dotp_xf_xf      (aec->dotp_xf_xf)
#define delta           (aec->delta)
#define w_valid         (aec->w_valid)
#define aes_y2          (aec->aes_y2)
#define fdwdisplay      (aec->fdwdisplay)
#define dumpcnt         (aec->dumpcnt)
#define dfast           (aec->dfast)
#define xfast           (aec->xfast)
#define dslow           (aec->dslow)
#define xslow           (aec->xslow)
#define efast           (aec->efast)      
#define M		(aec->M)


/**
 * Vector Dot Product 
 * \params a
 * \params b
 */
fixpoint AEC_dotp(struct AEC *aec, sfixpoint a[], fixpoint b[])
{
	fixpoint sum;
	sum.lval = 0;
	int i;
#if 0
	for(i=0; i< NLMS_LEN; i++)
	{
		sum = 	FP_ADD_FFF(
				sum, 
				FP_MUL_SFF( 
					a[i], 
					b[i]
				)
			);
	}
#elif 0
	unsigned long  l = 0, m = 0, h = 0;

	for(i=0; i< NLMS_LEN; i++)
	{
	//	l += ((unsigned long long)a[i].val.l * (unsigned long long) b[i].val.l);
		m += ((unsigned long long)a[i].val.l * (long long) b[i].val.h);
		m += ((long long)a[i].val.h * (unsigned long long) b[i].val.l);
		h += ((long long)a[i].val.h * (long long) b[i].val.h);
	}
	
	sum.lval  = 	(l + (m << 16) + (h << 32)) >> ( SFIXPOINT_ACC + NFIXPOINT_ACC - NFIXPOINT_ACC);
#else
	unsigned long long  l = 0, m = 0, h = 0;

	for(i=0; i< NLMS_LEN; i++)
	{
	//	l += ((unsigned long long)a[i].val.l * (unsigned long long) b[i].val.l);
		m += ((unsigned long)a[i].val.l * (long) b[i].val.h);
		m += ((long )a[i].val.h * (unsigned long) b[i].val.l);
		h += ((long )a[i].val.h * (long) b[i].val.h);
	}
	
	sum.lval  = /*	(l >> ( SFIXPOINT_ACC + NFIXPOINT_ACC - NFIXPOINT_ACC)) +*/
			(m << (16 - (SFIXPOINT_ACC + NFIXPOINT_ACC - NFIXPOINT_ACC))) +
			(h << (32 - (SFIXPOINT_ACC + NFIXPOINT_ACC - NFIXPOINT_ACC))) ;

#endif
	return sum;
}

fixpoint AEC_getambient(struct AEC *aec)
{
	return dfast;
}

void AEC_setambient(struct AEC *aec, fixpoint Min_xf)
{
	// subtract old data
	dotp_xf_xf = 	FP_SUB_DDD (
				dotp_xf_xf,
				delta
			);
	
	// calculate power for the current sample
	delta = 	FP_MUL_FFD (
				INT_TO_FP_fixpoint(NLMS_LEN-1),
				FP_MUL_FFF (
						Min_xf,
						Min_xf
				)
			);
	
	// add new delta
	dotp_xf_xf = 	FP_ADD_DDD (
				dotp_xf_xf,
				delta
			);
			
}

void AEC_setgain(struct AEC *aec, fixpoint gain_) {
	gain = gain_;
}

/**
 * Initialize the AEC
 */
void AEC_init(struct AEC *aec)
{
	hangover = 0;
	dtdCnt = dtdNdx = 0;
	
	FIR_HP_300Hz_init(&cutoff);
	
	memset(x, 0, sizeof(x));
	memset(xf, 0, sizeof(xf));
	memset(w, 0, sizeof(w));
	j = NLMS_EXT;
	delta = INT_TO_FP_dfixpoint(0);

	AEC_setambient(aec, NoiseFloor);
	
	gain = INT_TO_FP_fixpoint(1);

	dfast = dslow = M75dB_PCM;

	xfast = xslow = M80dB_PCM;

	IIR1_init(&Fx,  (fixpoint) to_fixpoint(2000.0f/RATE, NFIXPOINT_ACC));
	IIR1_init(&Fe,  (fixpoint) to_fixpoint(2000.0f/RATE, NFIXPOINT_ACC));
	IIR1_init(&Fxx, (fixpoint) to_fixpoint(2000.0f/RATE, NFIXPOINT_ACC));
	IIR1_init(&Fdd, (fixpoint) to_fixpoint(2000.0f/RATE, NFIXPOINT_ACC));

 //	const float M = (STEPY2 - STEPY1) / (STEPX2 - STEPX1);
	M = 	FP_DIV_FFF(
			FP_SUB_FFF(
				STEPY2,
				STEPY1
			),
			FP_SUB_FFF(
				STEPX2,
				STEPX1
			)
		);

	fdwdisplay = -1;
	dumpcnt = 0;
	memset(ws, 0, sizeof(ws));
}

// for logging
fixpoint ratio;

/**
 * Double talk detection
 */

fixpoint AEC_dtd(struct AEC *aec, fixpoint _d, fixpoint _x)
{
	fixpoint stepsize;
	
	dfast = FP_ADD_FFF(
			dfast,
			FP_MUL_SFF(
				ALPHAFAST,
				FP_SUB_FFF(
					FP_ABS_fixpoint( _d ),
					dfast
				)
			)
		);

	xfast = FP_ADD_FFF(
			xfast,
			FP_MUL_SFF(
				ALPHAFAST,
				FP_SUB_FFF(
					FP_ABS_fixpoint( _x ),
					xfast
				)
			)
		);
	dslow = FP_ADD_FFF(
			dslow,
			FP_MUL_SFF(
				ALPHASLOW,
				FP_SUB_FFF(
					FP_ABS_fixpoint( _d ),
					dslow
				)
			)
		);

	xslow = FP_ADD_FFF(
			xslow,
			FP_MUL_SFF(
				ALPHASLOW,
				FP_SUB_FFF(
					FP_ABS_fixpoint( _x ),
					xslow
				)
			)
		);

	if (FP_IS_LESS_fixpoint(xfast,M55dB_PCM)) {
	  return (fixpoint) to_fixpoint(0, 0);   // no Spk signal
	}
	
	if (FP_IS_LESS_fixpoint(dfast,M70dB_PCM)) {
	  return (fixpoint) to_fixpoint(0, 0);   // no Mic signal
	}
	
	// ratio of NFRs
//	ratio = (dfast * xslow) / (dslow * xfast);
	ratio = FP_DIV_FFF(
			FP_MUL_FFF(
				dfast,
				xslow
			),
			FP_MUL_FFF(
				dslow,
				xfast
			)
		);
	
	// begrenzte lineare Kennlinie

	if (       FP_IS_LESS_fixpoint(ratio,STEPX1)) {
		stepsize = STEPY1;
	} else if (FP_IS_LESS_fixpoint(STEPX2,ratio)) {
		stepsize = STEPY2;
	} else {
		// Punktrichtungsform einer Geraden
		//stepsize = M * (ratio - STEPX1) + STEPY1;
		//
		stepsize = 	FP_ADD_FFF (
					FP_MUL_FFF (
						M,
						FP_SUB_FFF (
							ratio,
							STEPX1
						)
					),
					STEPY1
				);
	}

	return stepsize;

}

inline fixpoint AEC_nlms_pw(struct AEC *aec, fixpoint d, fixpoint x_, fixpoint stepsize)
{
	x[j] = x_; 
	xf[j] = IIR1_highpass(&Fx, x_);   // pre-whitening of x
	

	// calculate error value 
	// (mic signal - estimated mic signal from spk signal)
	fixpoint e;
	e = FP_SUB_FFF (
		d,
		AEC_dotp(
			aec,
			w,
			x +j
		)
	);
	

	fixpoint ef  = IIR1_highpass(&Fe, e); //pre-whitening of e

	// optimize: iterative dotp(xf, xf)
	//dotp_xf_xf += (fp_to_float_fixpoint(xf[j]) * fp_to_float_fixpoint(xf[j]) - fp_to_float_fixpoint(xf[j + NLMS_LEN - 1]) * fp_to_float_fixpoint(xf[j + NLMS_LEN - 1]));
	dotp_xf_xf = 	FP_ADD_DDD(
				dotp_xf_xf,
				FP_SUB_DDD(
					FP_MUL_FFD(
						xf[j],
						xf[j]
					),
					FP_MUL_FFD(
						xf[j + NLMS_LEN - 1],
						xf[j + NLMS_LEN - 1]
					)
				)
			);

	if (FP_IS_LESS_fixpoint( (fixpoint) to_fixpoint (0,0), stepsize)) {
		w_valid = 1;
		// calculate variable step size
		//REAL mikro_ef = fp_to_float_fixpoint(stepsize) * fp_to_float_fixpoint(ef) / fp_to_float_dfixpoint(dotp_xf_xf);
		sfixpoint mikro_ef = FP_DIV_FDS(
					FP_MUL_FFF (
						stepsize,
						ef
					),
					dotp_xf_xf
				);


		  
		// update tap weights (filter learning)
		int i;
		for (i = 0; i < NLMS_LEN; i += 2) {
			// optimize: partial loop unrolling
			w[i] = FP_ADD_SSS (
					w[i],
					FP_MUL_SFS(
						mikro_ef,
						xf[i + j]
					)
				);
			w[i + 1] = 	FP_ADD_SSS (
						w[i+1],
						FP_MUL_SFS(
							mikro_ef,
							xf[i + j + 1]
						)
					);

		}
	} else {
		// ageing of w[] (leaky NLMS)
		if (w_valid) {
			w_valid = 0;
			memset(w, 0, sizeof(w));  // brutal forgetting
		}
	}
	
	if (--j < 0) {
		// optimize: decrease number of memory copies
		j = NLMS_EXT;
		memmove(x + j + 1, x, (NLMS_LEN - 1) * sizeof(fixpoint));
		memmove(xf + j + 1, xf, (NLMS_LEN - 1) * sizeof(fixpoint));
	}
	
	// Saturation
	if        (FP_IS_LESS_fixpoint(MAXPCM,e) ){
		return MAXPCM;
	} else if (FP_IS_LESS_fixpoint(e, FP_NEG_fixpoint(MAXPCM))) {
		return FP_NEG_fixpoint(MAXPCM);
	} else {
		return e;
	}

}

int AEC_doAEC(struct AEC *aec, int d_, int x_)
{
	fixpoint _d = INT_TO_FP_fixpoint(d_);
	fixpoint _x = INT_TO_FP_fixpoint(x_);
	
	// Mic Highpass Filter - to remove DC
	_d = IIR_HP_highpass(&acMic, _d);

	// Mic Highpass Filter - cut-off below 300Hz
	_d = FIR_HP_300Hz_highpass(&cutoff, _d);

	// Amplify, for e.g. Soundcards with -6dB max. volume
	_d = 	FP_MUL_FFF(
			_d,
			gain
		);
	
	// Spk Highpass Filter - to remove DC
	_x = IIR_HP_highpass(&acSpk, _x);

	// Double Talk Detector
	fixpoint stepsize = 	AEC_dtd(aec, 
					_d, 
					_x
				);
	//Acoustic Echo Cancellation
	_d = AEC_nlms_pw(aec, _d, _x, stepsize);

	return  FP_TO_INT_fixpoint(_d);
}

#undef acMic 	   
#undef acSpk 	   
#undef cutoff 	   
#undef gain 	   
#undef Fx	   
#undef Fe	   
#undef Fxx	   
#undef Fdd	   
#undef max_max_x  
#undef hangover   
#undef max_x      
#undef dtdCnt     
#undef dtdNdx     
#undef xfast1     
#undef max_max_u  
#undef max_u      
#undef c_max_u    
#undef i_max_u    
#undef x          
#undef xf         
#undef w          
#undef ws         
#undef j          
#undef dotp_xf_xf 
#undef delta      
#undef w_valid    
#undef aes_y2     
#undef fdwdisplay 
#undef dumpcnt    
#undef dfast      
#undef xfast      
#undef dslow      
#undef xslow      
#undef efast      
#undef M
