/**
 * Copyright (C) (SFIXPOINT_ACC06)
 * Sven Kolberg
 *
 * Fixpoint arithmetrik with presetable fractionsize
 * optimized for 16bit multiplikation
 */ 

#include "fixpoint.h"

/**
 * Fixpoint arithmetrik
 */
#define FIXPOINT_DATATYPE(name, accuracy)		\
	typedef union {					\
		long lval;				\
		struct {				\
			unsigned short l;		\
			signed short h;			\
		} val;					\
	} name;

#define to_fixpoint(value, accuracy)			\
	{((value) * (1<<accuracy))}

/**
 * Fixpoint negation
 * \param a fixpoint
 * \returns -a
 */
#define FIXPOINT_NEG(name, accuracy)				\
	inline name FP_NEG_##name(name a)			\
	{							\
		a.lval = -a.lval;				\
		return a;					\
	}


/**
 * Fixpoint Frac
 */
#define FIXPOINT_FRAC(name, accuracy)				\
	inline name FP_FRAC_##name(name a)			\
	{							\
		a.lval = a.lval & 0x0fff;			\
		return a;					\
	}

/**
 * float to fixpoint
 */
#define FIXPOINT_FLOAT_TO_FIXPOINT_L(name,accuracy)		\
	inline name float_to_fp_##name(float a)			\
	{							\
		a *= 1<<accuracy;				\
		name b;						\
		b.lval = (long) a;				\
		return b;					\
	}

#define FIXPOINT_FLOAT_TO_FIXPOINT_R(name,accuracy)		\
	inline name float_to_fp_##name(float a)			\
	{							\
		a /= 1<<(-accuracy);				\
		name b;						\
		b.lval = (long) a;				\
		return b;					\
	}

/**
 * fixpoint to float
 */
#define FIXPOINT_FIXPOINT_TO_FLOAT_L(name, accuracy)		\
	inline float fp_to_float_##name(name a)			\
	{							\
		float b;					\
		b = (float) a.lval;				\
		b /= 1<<accuracy;				\
		return b;					\
	}

#define FIXPOINT_FIXPOINT_TO_FLOAT_R(name, accuracy)		\
	inline float fp_to_float_##name(name a)			\
	{							\
		float b;					\
		b = (float) a.lval;				\
		b *= 1<<(-accuracy);				\
		return b;					\
	}

#define FIXPOINT_PRINT(name, accuracy)				\
	inline void print_fp_##name(name a)			\
	{							\
		fprintf(stderr,"l:%04x h:%04x   %08x\n", a.val.l, a.val.h, a.lval);	\
	}

#define FIXPOINT_ABS(name, accuracy)				\
	inline name FP_ABS_##name(name a)			\
	{							\
		if (a.lval > 0) return a;			\
		else{						\
			a.lval = -a.lval;			\
			return a;				\
		}						\
	}

#define FIXPOINT_LESS(name, accuracy)				\
	inline int FP_IS_LESS_##name(name a, name b)		\
	{							\
		return (a.lval < b.lval);			\
	}

#define FIXPOINT_INT_TO_FIXPOINT(name, accuracy)		\
	inline name INT_TO_FP_##name(int a)			\
	{							\
		name c;						\
		c.lval = ((long) a) << accuracy;		\
		return c;					\
	}

#define FIXPOINT_TO_INT(name, accuracy)				\
	inline int FP_TO_INT_##name (name a)			\
	{							\
		return a.lval >>accuracy;			\
	}

/**
 * Fixpoint add
 * \params a fixpoint
 * \params b fixpoint
 * \returns the sum of a and b 
 */
#define FIXPOINT_ADD_LL(pre, namea, acca, nameb, accb, namec,  accc)	\
	inline namec FP_ADD_##pre(namea a, nameb b) 			\
	{							\
		namec c;					\
		c.lval = (a.lval<<(accc - acca)) + (b.lval<<(accc - accb)); \
		return c;					\
	}

/**
 * Fixpoint add
 * \params a fixpoint
 * \params b fixpoint
 * \returns the sum of a and b 
 */
#define FIXPOINT_ADD_LR(pre, namea, acca, nameb, accb, namec,  accc)	\
	inline namec FP_ADD_##pre(namea a, nameb b) 			\
	{							\
		namec c;					\
		c.lval = (a.lval<<(accc - acca)) + (b.lval>>(accb - accc)); \
		return c;					\
	}

/**
 * Fixpoint add
 * \params a fixpoint
 * \params b fixpoint
 * \returns the sum of a and b 
 */
#define FIXPOINT_ADD_RL(pre, namea, acca, nameb, accb, namec,  accc)	\
	inline namec FP_ADD_##pre(namea a, nameb b) 			\
	{							\
		namec c;					\
		c.lval = (a.lval>>(acca - accc)) + (b.lval<<(accc - accb)); \
		return c;					\
	}

/**
 * Fixpoint add
 * \params a fixpoint
 * \params b fixpoint
 * \returns the sum of a and b 
 */
#define FIXPOINT_ADD_RR(pre, namea, acca, nameb, accb, namec,  accc)	\
	inline namec FP_ADD_##pre(namea a, nameb b) 			\
	{							\
		namec c;					\
		c.lval = (a.lval>>(acca - accc)) + (b.lval>>(accb - accc)); \
		return c;					\
	}

/**
 * Fixpoint minus
 * \params a fixpoint
 * \params b fixpoint
 * \returns the difference between a and b
 */
#define FIXPOINT_SUB_LL(pre, namea, acca, nameb, accb, namec, accc)	\
	inline namec FP_SUB_##pre(namea a,nameb b)	\
	{							\
		namec c;					\
		c.lval = (a.lval<<(accc - acca)) - (b.lval<<(accc-accb));			\
		return c;					\
	}

/**
 * Fixpoint minus
 * \params a fixpoint
 * \params b fixpoint
 * \returns the difference between a and b
 */
#define FIXPOINT_SUB_LR(pre, namea, acca, nameb, accb, namec, accc)	\
	inline namec FP_SUB_##pre(namea a,nameb b)	\
	{							\
		namec c;					\
		c.lval = (a.lval<<(accc - acca)) - (b.lval>>(accb-accc));			\
		return c;					\
	}


/**
 * Fixpoint minus
 * \params a fixpoint
 * \params b fixpoint
 * \returns the difference between a and b
 */
#define FIXPOINT_SUB_RL(pre, namea, acca, nameb, accb, namec, accc)	\
	inline namec FP_SUB_##pre(namea a,nameb b)	\
	{							\
		namec c;					\
		c.lval = (a.lval>>(acca - accc)) - (b.lval<<(accc-accb));			\
		return c;					\
	}


/**
 * Fixpoint minus
 * \params a fixpoint
 * \params b fixpoint
 * \returns the difference between a and b
 */
#define FIXPOINT_SUB_RR(pre, namea, acca, nameb, accb, namec, accc)	\
	inline namec FP_SUB_##pre(namea a,nameb b)	\
	{							\
		namec c;					\
		c.lval = (a.lval>>(acca - accc)) - (b.lval>>(accb-accc));			\
		return c;					\
	}


/**
 * Fixpoint multiply optimized for 16 mulitplication
 * \params a fixpoint
 * \params b fixpoint
 * \returns the produkt of a and b
 */
#define FIXPOINT_MUL_R(pre, namea, acca, nameb, accb, namec,  accc) 	\
	inline namec FP_MUL_##pre(namea a, nameb b)			\
	{								\
		unsigned long long c;						\
		namec co;						\
		c      =(((unsigned long long)a.val.l * (unsigned long long) b.val.l)<<0)+		\
			(((unsigned long long)a.val.l * (long long) b.val.h)<<16)+ 		\
			(((long long)a.val.h * (unsigned long long) b.val.l)<<16)+			\
			(((long long)a.val.h * (long long) b.val.h)<<32);		\
                co.lval = c >> (acca + accb  -accc);                    \
		return co;						\
	}

/**
 * Fixpoint multiply optimized for 16 mulitplication
 * \params a fixpoint
 * \params b fixpoint
 * \returns the product of a and b
 */
#define FIXPOINT_MUL_L(pre, namea, acca, nameb, accb, namec,  accc) 	\
	inline namec FP_MUL_##pre(namea a, nameb b)			\
	{								\
		unsigned long long c;						\
		namec co;						\
		c      =(((unsigned long long)a.val.l * (unsigned long long) b.val.l)<<0)+		\
			(((unsigned long long)a.val.l * (long long) b.val.h)<<16)+ 		\
			(((long long)a.val.h * (unsigned long long) b.val.l)<<16)+			\
			(((long long)a.val.h * (long long) b.val.h)<<32);		\
                co.lval = c << (-acca - accb + accc);                    \
		return co;						\
	}

/**
 * Fixpoint divide optimized for nothing yet FIXME
 * XXX CRASHS IF USES long long
 * \params a fixpoint
 * \params b fixpoint
 * \returns the quotient of a and b
 */
#define FIXPOINT_DIV_L(pre, namea, acca, nameb, accb, namec,  accc)	\
	inline namec FP_DIV_##pre(namea a, nameb b)		\
	{							\
		long val;					\
		namec c;						\
		val = ((long long)a.lval) << (-acca+accb+accc);		\
		c.lval = val / b.lval;				\
		return c;					\
	}

/**
 * Fixpoint divide optimized for nothing yet FIXME
 * \params a fixpoint
 * \params b fixpoint
 * \returns the quotient of a and b
 */
#define FIXPOINT_DIV_R(pre, namea, acca, nameb, accb, namec,  accc)	\
	inline namec FP_DIV_##pre(namea a, nameb b)		\
	{							\
		long long val;					\
		namec c;						\
		val = ((long long)a.lval) >> (acca-accb-accc);		\
		c.lval = val / b.lval;				\
		return c;					\
	}



#define FIXPOINT_CONVERSION(namefrom, accuracyfrom, nameto, accuracyto)	\
	inline nameto fp_convert(namefrom a)			\
	{							\
		nameto b;					\
		b.lval = a.lval << (accuracyto - accuracyfrom);	\
		return b;					\
	}


/**
 * define the needed datatypes and accuracys
 */

#define NFIXPOINT_ACC 12
#define DFIXPOINT_ACC -8
#define SFIXPOINT_ACC 28

	FIXPOINT_NEG			(fixpoint, NFIXPOINT_ACC)
//	FIXPOINT_FLOAT_TO_FIXPOINT_L	(fixpoint, NFIXPOINT_ACC)
//	FIXPOINT_FIXPOINT_TO_FLOAT_L	(fixpoint, NFIXPOINT_ACC)
//	FIXPOINT_PRINT			(fixpoint, NFIXPOINT_ACC)
	FIXPOINT_ABS			(fixpoint, NFIXPOINT_ACC)
	FIXPOINT_LESS			(fixpoint, NFIXPOINT_ACC)
	FIXPOINT_INT_TO_FIXPOINT        (fixpoint, NFIXPOINT_ACC)
	FIXPOINT_TO_INT                 (fixpoint, NFIXPOINT_ACC)
	FIXPOINT_FRAC			(fixpoint, NFIXPOINT_ACC)

	FIXPOINT_NEG			(dfixpoint, DFIXPOINT_ACC)
//	FIXPOINT_FLOAT_TO_FIXPOINT_R	(dfixpoint, DFIXPOINT_ACC)
//	FIXPOINT_FIXPOINT_TO_FLOAT_R	(dfixpoint, DFIXPOINT_ACC)
//	FIXPOINT_PRINT			(dfixpoint, DFIXPOINT_ACC)
	FIXPOINT_ABS			(dfixpoint, DFIXPOINT_ACC)
	FIXPOINT_LESS			(dfixpoint, DFIXPOINT_ACC)
	FIXPOINT_INT_TO_FIXPOINT        (dfixpoint, DFIXPOINT_ACC)
	FIXPOINT_TO_INT                 (dfixpoint, DFIXPOINT_ACC)
	FIXPOINT_FRAC			(dfixpoint, DFIXPOINT_ACC)

	FIXPOINT_NEG			(sfixpoint, SFIXPOINT_ACC)
//	FIXPOINT_FLOAT_TO_FIXPOINT_L	(sfixpoint, SFIXPOINT_ACC)
//	FIXPOINT_FIXPOINT_TO_FLOAT_L	(sfixpoint, SFIXPOINT_ACC)
//	FIXPOINT_PRINT			(sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_ABS			(sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_LESS			(sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_INT_TO_FIXPOINT        (sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_TO_INT                 (sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_FRAC			(sfixpoint, SFIXPOINT_ACC)

/**
 * define the needed operations
 */
	FIXPOINT_ADD_RR			(FFF, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC)
	FIXPOINT_SUB_RR			(FFF, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC)
	FIXPOINT_MUL_R			(FFF, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC)
	FIXPOINT_DIV_L			(FFF, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC)

	FIXPOINT_ADD_RR			(DDD, dfixpoint, DFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC)
	FIXPOINT_SUB_RR			(DDD, dfixpoint, DFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC)
	FIXPOINT_MUL_L			(DDD, dfixpoint, DFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC)
	FIXPOINT_DIV_R			(DDD, dfixpoint, DFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC)

	FIXPOINT_ADD_RR			(SSS, sfixpoint, SFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_SUB_RR			(SSS, sfixpoint, SFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_MUL_R			(SSS, sfixpoint, SFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_DIV_L			(SSS, sfixpoint, SFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC)

	FIXPOINT_ADD_LL			(FDS, fixpoint, NFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_SUB_LL			(FDS, fixpoint, NFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_MUL_L			(FDS, fixpoint, NFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_DIV_L			(FDS, fixpoint, NFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC)

	FIXPOINT_ADD_RL			(SFF, sfixpoint, SFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC)
	FIXPOINT_SUB_RL			(SFF, sfixpoint, SFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC)
	FIXPOINT_MUL_R			(SFF, sfixpoint, SFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC)
	FIXPOINT_DIV_L			(SFF, sfixpoint, SFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC)

	FIXPOINT_ADD_RR			(FFD, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC)
	FIXPOINT_SUB_RR			(FFD, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC)
	FIXPOINT_MUL_R			(FFD, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC)
	FIXPOINT_DIV_L			(FFD, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC)

	FIXPOINT_ADD_RR			(SFS, sfixpoint, SFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_SUB_RR			(SFS, sfixpoint, SFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_MUL_R			(SFS, sfixpoint, SFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_DIV_L			(SFS, sfixpoint, SFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC)

/**
 * define the needed Conversations between the different datatypes
 */
	FIXPOINT_CONVERSION(fixpoint, NFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC)

/**
 * Fixpoint Expf
 */
#if 0
#define FIXPOINT_EXPF(name, namea, accuracy)	\
	inline namea FP_EXPF_##name(namea a)			\
	{							\
		namea r = (namea) to_fixpoint(1, accuracy);		\
		namea e = (namea) to_fixpoint(2.7182818284, accuracy);	\
								\
		/* First multiply e int(a) times */		\
		int i;						\
		for(i=0;i<FP_TO_INT_##namea(a);i++)		\
			r = FP_MUL_##name (r,e);		\
								\
		namea frac = FIXPOINT_FRAC_##namea(r);		\
		namea rf = frac;				\
		/* Second, calculate the fraction */		\
		for(i=0;i< 20;i++)				\
			rf = FP_MUL_##name(			\
				rf,				\
				FP_ADD_##name(			\
					(fixpoint) to_fixpoint(1, accuracy),	\
					FP_DIV_##name(				\
						frac,				\
						to_fixpoint(i,accuracy)		\
					)			\
				)				\
			);					\
								\
		r = FP_MUL_##name(r, FP_ADD_##name(rf, to_fixpoint(1, accuracy)));	\
		return r;					\
	}
#else
	inline fixpoint FP_EXPF_FFF(fixpoint a)
	{				
		fixpoint r = to_fixpoint(1, NFIXPOINT_ACC);		
		fixpoint e = to_fixpoint(2.7182818284, NFIXPOINT_ACC);	
		int neg = 0;
		
		if(FP_IS_LESS_fixpoint(a, (fixpoint) to_fixpoint(1, NFIXPOINT_ACC)))
		{
			neg = 1;
			a = FP_ABS_fixpoint(a);
		}

		/* First multiply e int(a) times */		
		int i;						
		for(i=0;i<FP_TO_INT_fixpoint(a);i++)		
			r = FP_MUL_FFF (r,e);		
			
		fixpoint frac = FP_FRAC_fixpoint(a);		
		fixpoint rf = to_fixpoint(1, NFIXPOINT_ACC);

		/* Second, calculate the fraction */	
		for(i=20;i> 0;i--)				
			rf = FP_ADD_FFF ( 
				(fixpoint) to_fixpoint(1, NFIXPOINT_ACC),
				FP_MUL_FFF(			
					FP_DIV_FFF(				
						frac,				
						(fixpoint) to_fixpoint(i,NFIXPOINT_ACC)		
					),
					rf
				)				
			);					
								
		r = FP_MUL_FFF(r, rf );
		if (neg)
		{
			r = FP_DIV_FFF(
				(fixpoint) to_fixpoint(1, NFIXPOINT_ACC),
				r
			);
		}
		return r;					
	}
#endif

	
//	FIXPOINT_EXPF			(FFF, fixpoint, NFIXPOINT_ACC)
/**
 * define the testcases to check the operations
 */
#define FIXPOINT_TEST(name, namea, nameb, namec)					\
	void fixpoint_test_##name(float vala, float valb)				\
	{										\
		namea a = float_to_fp_##namea(vala);					\
		nameb b = float_to_fp_##nameb(valb);					\
		print_fp_##namea(a);							\
		print_fp_##nameb(b);							\
											\
		/* Test the 4 Basic arithmetrik operations */				\
		/* addition */								\
		fprintf (stderr, "%f + %f = %f (%f)\n", 				\
				vala,							\
				valb, 							\
				fp_to_float_##namec(FP_ADD_##name(a, b)),		\
				vala + valb );						\
											\
		/* subtraction */							\
		fprintf (stderr, "%f - %f = %f (%f)\n", 				\
				vala,							\
				valb, 							\
				fp_to_float_##namec(FP_SUB_##name(a, b)),		\
				vala - valb);						\
											\
		/* division */								\
		fprintf (stderr, "%f / %f = %f (%f)\n", 				\
				vala,							\
				valb, 							\
				fp_to_float_##namec(FP_DIV_##name(a, b)),		\
				vala / valb);						\
											\
		/* multiplication */							\
		fprintf (stderr, "%f * %f = %f (%f)\n", 				\
				vala,							\
				valb, 							\
				fp_to_float_##namec(FP_MUL_##name(a, b)),		\
				vala * valb);						\
											\
		/* exponentiation */							\
		fprintf (stderr, "e hoch %f = %f (%f)\n",				\
				vala,							\
				fp_to_float_##namec(FP_EXPF_##name(a)),			\
				expf(vala));						\
	}

/* define Test cases */
/*	FIXPOINT_TEST(FFF, fixpoint, fixpoint, fixpoint)
	FIXPOINT_TEST(DDD, dfixpoint, dfixpoint, dfixpoint)
	FIXPOINT_TEST(SSS, sfixpoint, sfixpoint, sfixpoint)
	FIXPOINT_TEST(FDS, fixpoint, dfixpoint, sfixpoint)
	FIXPOINT_TEST(SFF, sfixpoint, fixpoint, fixpoint)
	FIXPOINT_TEST(FFD, fixpoint, fixpoint, dfixpoint)
	FIXPOINT_TEST(SFS, sfixpoint, fixpoint, sfixpoint)*/


/*
 * run the test
 */
void fixpoint_test(void)
{
	float a = (-2.0f * 3.14159 * 2000/8000); //7.996713241; //;
	float b = 134.357;

/*	fixpoint_test_FFF(a,b);
	fixpoint_test_DDD(a,b);
	fixpoint_test_SSS(a,b);
	fixpoint_test_FDS(a,b);
	fixpoint_test_SFF(a,b);
	fixpoint_test_FFD(a,b);
	fixpoint_test_SFS(a,b);
	*/
}
