/**
 * Copyright (C) (SFIXPOINT_ACC06)
 * Sven Kolberg
 *
 * Fixpoint arithmetrik with presetable fractionsize
 * optimized for 16bit multiplikation
 */ 

#ifndef __HEADER_FIXPOINT__
#define __HEADER_FIXPOINT__

/**
 * Fixpoint arithmetrik
 */
#define FIXPOINT_DATATYPE(name, accuracy)		\
	typedef union {					\
		long lval;				\
		struct {				\
			unsigned short l;		\
			signed short h;			\
		} val;					\
	} name;

#define to_fixpoint(value, accuracy)			\
	{((value) * (1<<accuracy))}

/**
 * Fixpoint negation
 * \param a fixpoint
 * \returns -a
 */
#define FIXPOINT_NEG(name, accuracy)				\
	inline name FP_NEG_##name(name a);

/**
 * Fixpoint Frac
 */
#define FIXPOINT_FRAC(name, accuracy)				\
	inline name FP_FRAC_##name(name a);

/**
 * float to fixpoint
 */
#define FIXPOINT_FLOAT_TO_FIXPOINT_L(name,accuracy)		\
	inline name float_to_fp_##name(float a);

#define FIXPOINT_FLOAT_TO_FIXPOINT_R(name,accuracy)		\
	inline name float_to_fp_##name(float a);

/**
 * fixpoint to float
 */
#define FIXPOINT_FIXPOINT_TO_FLOAT_L(name, accuracy)		\
	inline float fp_to_float_##name(name a); 

#define FIXPOINT_FIXPOINT_TO_FLOAT_R(name, accuracy)		\
	inline float fp_to_float_##name(name a);

#define FIXPOINT_PRINT(name, accuracy)				\
	inline void print_fp_##name(name a);

#define FIXPOINT_ABS(name, accuracy)				\
	inline name FP_ABS_##name(name a);

#define FIXPOINT_LESS(name, accuracy)				\
	inline int FP_IS_LESS_##name(name a, name b);

#define FIXPOINT_INT_TO_FIXPOINT(name, accuracy)		\
	inline name INT_TO_FP_##name(int a);

#define FIXPOINT_TO_INT(name, accuracy)				\
	inline int FP_TO_INT_##name (name a);

/**
 * Fixpoint add
 * \params a fixpoint
 * \params b fixpoint
 * \returns the sum of a and b 
 */
#define FIXPOINT_ADD_LL(pre, namea, acca, nameb, accb, namec,  accc)	\
	inline namec FP_ADD_##pre(namea a, nameb b);

/**
 * Fixpoint add
 * \params a fixpoint
 * \params b fixpoint
 * \returns the sum of a and b 
 */
#define FIXPOINT_ADD_LR(pre, namea, acca, nameb, accb, namec,  accc)	\
	inline namec FP_ADD_##pre(namea a, nameb b);

/**
 * Fixpoint add
 * \params a fixpoint
 * \params b fixpoint
 * \returns the sum of a and b 
 */
#define FIXPOINT_ADD_RL(pre, namea, acca, nameb, accb, namec,  accc)	\
	inline namec FP_ADD_##pre(namea a, nameb b);

/**
 * Fixpoint add
 * \params a fixpoint
 * \params b fixpoint
 * \returns the sum of a and b 
 */
#define FIXPOINT_ADD_RR(pre, namea, acca, nameb, accb, namec,  accc)	\
	inline namec FP_ADD_##pre(namea a, nameb b);

/**
 * Fixpoint minus
 * \params a fixpoint
 * \params b fixpoint
 * \returns the difference between a and b
 */
#define FIXPOINT_SUB_LL(pre, namea, acca, nameb, accb, namec, accc)	\
	inline namec FP_SUB_##pre(namea a,nameb b);

/**
 * Fixpoint minus
 * \params a fixpoint
 * \params b fixpoint
 * \returns the difference between a and b
 */
#define FIXPOINT_SUB_LR(pre, namea, acca, nameb, accb, namec, accc)	\
	inline namec FP_SUB_##pre(namea a,nameb b);

/**
 * Fixpoint minus
 * \params a fixpoint
 * \params b fixpoint
 * \returns the difference between a and b
 */
#define FIXPOINT_SUB_RL(pre, namea, acca, nameb, accb, namec, accc)	\
	inline namec FP_SUB_##pre(namea a,nameb b);

/**
 * Fixpoint minus
 * \params a fixpoint
 * \params b fixpoint
 * \returns the difference between a and b
 */
#define FIXPOINT_SUB_RR(pre, namea, acca, nameb, accb, namec, accc)	\
	inline namec FP_SUB_##pre(namea a,nameb b);

/**
 * Fixpoint multiply optimized for 16 mulitplication
 * \params a fixpoint
 * \params b fixpoint
 * \returns the produkt of a and b
 */
#define FIXPOINT_MUL_R(pre, namea, acca, nameb, accb, namec,  accc) 	\
	inline namec FP_MUL_##pre(namea a, nameb b);

/**
 * Fixpoint multiply optimized for 16 mulitplication
 * \params a fixpoint
 * \params b fixpoint
 * \returns the product of a and b
 */
#define FIXPOINT_MUL_L(pre, namea, acca, nameb, accb, namec,  accc) 	\
	inline namec FP_MUL_##pre(namea a, nameb b);

/**
 * Fixpoint divide optimized for nothing yet FIXME
 * \params a fixpoint
 * \params b fixpoint
 * \returns the quotient of a and b
 */
#define FIXPOINT_DIV_L(pre, namea, acca, nameb, accb, namec,  accc)	\
	inline namec FP_DIV_##pre(namea a, nameb b);

/**
 * Fixpoint divide optimized for nothing yet FIXME
 * \params a fixpoint
 * \params b fixpoint
 * \returns the quotient of a and b
 */
#define FIXPOINT_DIV_R(pre, namea, acca, nameb, accb, namec,  accc)	\
	inline namec FP_DIV_##pre(namea a, nameb b);

#define FIXPOINT_CONVERSION(namefrom, accuracyfrom, nameto, accuracyto)	\
	inline nameto fp_convert(namefrom a);	


/**
 * define the needed datatypes and accuracys
 */

#define NFIXPOINT_ACC 12
#define DFIXPOINT_ACC -8
#define SFIXPOINT_ACC 28

	FIXPOINT_DATATYPE		(fixpoint, NFIXPOINT_ACC)
	FIXPOINT_NEG			(fixpoint, NFIXPOINT_ACC)
//	FIXPOINT_FLOAT_TO_FIXPOINT_L	(fixpoint, NFIXPOINT_ACC)
//	FIXPOINT_FIXPOINT_TO_FLOAT_L	(fixpoint, NFIXPOINT_ACC)
//	FIXPOINT_PRINT			(fixpoint, NFIXPOINT_ACC)
	FIXPOINT_ABS			(fixpoint, NFIXPOINT_ACC)
	FIXPOINT_LESS			(fixpoint, NFIXPOINT_ACC)
	FIXPOINT_INT_TO_FIXPOINT        (fixpoint, NFIXPOINT_ACC)
	FIXPOINT_TO_INT                 (fixpoint, NFIXPOINT_ACC)
	FIXPOINT_FRAC			(fixpoint, NFIXPOINT_ACC)

	FIXPOINT_DATATYPE		(dfixpoint, DFIXPOINT_ACC)
	FIXPOINT_NEG			(dfixpoint, DFIXPOINT_ACC)
//	FIXPOINT_FLOAT_TO_FIXPOINT_R	(dfixpoint, DFIXPOINT_ACC)
//	FIXPOINT_FIXPOINT_TO_FLOAT_R	(dfixpoint, DFIXPOINT_ACC)
//	FIXPOINT_PRINT			(dfixpoint, DFIXPOINT_ACC)
	FIXPOINT_ABS			(dfixpoint, DFIXPOINT_ACC)
	FIXPOINT_LESS			(dfixpoint, DFIXPOINT_ACC)
	FIXPOINT_INT_TO_FIXPOINT        (dfixpoint, DFIXPOINT_ACC)
	FIXPOINT_TO_INT                 (dfixpoint, DFIXPOINT_ACC)
	FIXPOINT_FRAC			(dfixpoint, DFIXPOINT_ACC)

	FIXPOINT_DATATYPE		(sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_NEG			(sfixpoint, SFIXPOINT_ACC)
//	FIXPOINT_FLOAT_TO_FIXPOINT_L	(sfixpoint, SFIXPOINT_ACC)
//	FIXPOINT_FIXPOINT_TO_FLOAT_L	(sfixpoint, SFIXPOINT_ACC)
//	FIXPOINT_PRINT			(sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_ABS			(sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_LESS			(sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_INT_TO_FIXPOINT        (sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_TO_INT                 (sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_FRAC			(sfixpoint, SFIXPOINT_ACC)

/**
 * define the needed operations
 */
	FIXPOINT_ADD_RR			(FFF, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC)
	FIXPOINT_SUB_RR			(FFF, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC)
	FIXPOINT_MUL_R			(FFF, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC)
	FIXPOINT_DIV_L			(FFF, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC)

	FIXPOINT_ADD_RR			(DDD, dfixpoint, DFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC)
	FIXPOINT_SUB_RR			(DDD, dfixpoint, DFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC)
	FIXPOINT_MUL_L			(DDD, dfixpoint, DFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC)
	FIXPOINT_DIV_R			(DDD, dfixpoint, DFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC)

	FIXPOINT_ADD_RR			(SSS, sfixpoint, SFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_SUB_RR			(SSS, sfixpoint, SFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_MUL_R			(SSS, sfixpoint, SFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_DIV_L			(SSS, sfixpoint, SFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC)

	FIXPOINT_ADD_LL			(FDS, fixpoint, NFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_SUB_LL			(FDS, fixpoint, NFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_MUL_L			(FDS, fixpoint, NFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_DIV_L			(FDS, fixpoint, NFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC)

	FIXPOINT_ADD_RL			(SFF, sfixpoint, SFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC)
	FIXPOINT_SUB_RL			(SFF, sfixpoint, SFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC)
	FIXPOINT_MUL_R			(SFF, sfixpoint, SFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC)
	FIXPOINT_DIV_L			(SFF, sfixpoint, SFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC)

	FIXPOINT_ADD_RR			(FFD, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC)
	FIXPOINT_SUB_RR			(FFD, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC)
	FIXPOINT_MUL_R			(FFD, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC)
	FIXPOINT_DIV_L			(FFD, fixpoint, NFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC)

	FIXPOINT_ADD_RR			(SFS, sfixpoint, SFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_SUB_RR			(SFS, sfixpoint, SFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_MUL_R			(SFS, sfixpoint, SFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC)
	FIXPOINT_DIV_L			(SFS, sfixpoint, SFIXPOINT_ACC, fixpoint, NFIXPOINT_ACC, sfixpoint, SFIXPOINT_ACC)

/**
 * define the needed Conversations between the different datatypes
 */
	FIXPOINT_CONVERSION(fixpoint, NFIXPOINT_ACC, dfixpoint, DFIXPOINT_ACC)

/**
 * Fixpoint Expf
 */
inline fixpoint FP_EXPF_FFF(fixpoint a);

	
#endif /* __HEADER_FIXPOINT__ */

