/* aec.h
 *
 * Copyright (C) DFS Deutsche Flugsicherung (2004, 2005). 
 * All Rights Reserved.
 * Author: Andre Adrian
 *
 * Acoustic Echo Cancellation NLMS-pw algorithm
 *
 * Version 0.3.1 Allow change of stability parameter delta
 * Version 0.3 filter created with www.dsptutor.freeuk.com
 */

#include "fixpoint.h"

#include "iir.h"
#include "../profile.h"
#include <common.h>

#define x 	(iir_hp->x)
void IIR_HP_init(struct IIR_HP *iir_hp)
{
	x.lval = 0;
}

fixpoint IIR_HP_highpass(struct IIR_HP *iir_hp, fixpoint in)
{
	/* Highpass = Signal - Lowpass. Lowpass = Exponential Smoothing */
	fixpoint a0 = to_fixpoint(0.01f, NFIXPOINT_ACC);

	x = FP_ADD_FFF(
		x, 
		FP_MUL_FFF(
			a0,
			FP_SUB_FFF(
				in,
				x
			)
		)
	);
	return FP_SUB_FFF(in,x);
}
#undef x


#if WIDEB==1
/* 17 taps FIR Finite Impulse Response filter
 * Coefficients calculated with
 * www.dsptutor.freeuk.com/KaiserFilterDesign/KaiserFilterDesign.html
 */
#define z	(fir_hp_300hz)->z
#define a	(fir_hp_300hz)->a

void FIR_HP_300Hz_init(struct FIR_HP_300Hz *fir_hp_300hz)
{
	memset(z, 0, sizeof(fixpoint) *18);
	sfixpoint b[18] = {
		to_fixpoint(-0.034870606, SFIXPOINT_ACC), 
		to_fixpoint(-0.039650206, SFIXPOINT_ACC),
		to_fixpoint(-0.044063766, SFIXPOINT_ACC),
		to_fixpoint(-0.04800318 , SFIXPOINT_ACC),
		to_fixpoint(-0.051370874, SFIXPOINT_ACC),
		to_fixpoint(-0.054082647, SFIXPOINT_ACC),
		to_fixpoint(-0.056070227, SFIXPOINT_ACC),
		to_fixpoint(-0.057283327, SFIXPOINT_ACC),
		to_fixpoint(0.8214126   , SFIXPOINT_ACC),
		to_fixpoint(-0.057283327, SFIXPOINT_ACC),
		to_fixpoint(-0.056070227, SFIXPOINT_ACC),
		to_fixpoint(-0.054082647, SFIXPOINT_ACC),
		to_fixpoint(-0.051370874, SFIXPOINT_ACC),
		to_fixpoint(-0.04800318 , SFIXPOINT_ACC),
		to_fixpoint(-0.044063766, SFIXPOINT_ACC),
		to_fixpoint(-0.039650206, SFIXPOINT_ACC),
		to_fixpoint(-0.034870606, SFIXPOINT_ACC),
		to_fixpoint(0.0         , SFIXPOINT_ACC)
	};

	memcpy(a,b, sizeof(sfixpoint) * 18);
}

fixpoint FIR_HP_300Hz_highpass(struct FIR_HP_300Hz *fir_hp_300hz, fixpoint in)
{

	memmove(z+1, z, 17 * sizeof(fixpoint));


	z[0] = in;

	fixpoint sum;
	sum.lval = 0;

	int j;
#if 0
	for (j=0;j<18;j++)
	{
		sum = FP_ADD_fixpoint(sum, FP_MUL_SFF(a[j], z[j]));
	
	}
#else
{
	unsigned long l = 0, m = 0, h = 0;

	for (j = 0; j < 18; j ++) 
	{
		l+=(unsigned long long)a[j].val.l * (unsigned long long) z[j].val.l;
		m+=(unsigned long long)a[j].val.l * (long long)          z[j].val.h;
		m+=(long long)a[j].val.h * (unsigned long long) z[j].val.l;
		h+=(long long)a[j].val.h * (long long) z[j].val.h;
	}
	sum.lval  = 	(l >> ( SFIXPOINT_ACC + NFIXPOINT_ACC - NFIXPOINT_ACC)) +
			(m << (16 - (SFIXPOINT_ACC + NFIXPOINT_ACC - NFIXPOINT_ACC))) +
			(h << (32 - (SFIXPOINT_ACC + NFIXPOINT_ACC - NFIXPOINT_ACC))) ;
}

#endif

#warning untested section
	return sum;
}
#undef a
#undef z

#else

#define z	(fir_hp_300hz)->z
#define a	(fir_hp_300hz)->a

void FIR_HP_300Hz_init(struct FIR_HP_300Hz *fir_hp_300hz)
{
	memset(z, 0, sizeof(fixpoint) * 36);

	sfixpoint b[36] = {
	  // Kaiser Window FIR Filter, Filter type: High pass
	  // Passband: 150.0 - 4000.0 Hz, Order: 34
	  // Transition band: 34.0 Hz, Stopband attenuation: 10.0 dB
	  to_fixpoint(-0.016165324, SFIXPOINT_ACC), 	to_fixpoint(-0.017454365, SFIXPOINT_ACC), 	to_fixpoint(-0.01871232 , SFIXPOINT_ACC), 	to_fixpoint(-0.019931411, SFIXPOINT_ACC), 
	  to_fixpoint(-0.021104068, SFIXPOINT_ACC), 	to_fixpoint(-0.022222936, SFIXPOINT_ACC), 	to_fixpoint(-0.02328091 , SFIXPOINT_ACC), 	to_fixpoint(-0.024271343, SFIXPOINT_ACC), 
	  to_fixpoint(-0.025187887, SFIXPOINT_ACC), 	to_fixpoint(-0.02602462 , SFIXPOINT_ACC), 	to_fixpoint(-0.026776174, SFIXPOINT_ACC),	to_fixpoint(-0.027437767, SFIXPOINT_ACC), 
	  to_fixpoint(-0.028004972, SFIXPOINT_ACC), 	to_fixpoint(-0.028474221, SFIXPOINT_ACC), 	to_fixpoint(-0.028842418, SFIXPOINT_ACC), 	to_fixpoint(-0.029107114, SFIXPOINT_ACC), 
	  to_fixpoint(-0.02926664 , SFIXPOINT_ACC), 	to_fixpoint(0.8524841   , SFIXPOINT_ACC), 	to_fixpoint(-0.02926664 , SFIXPOINT_ACC), 	to_fixpoint(-0.029107114, SFIXPOINT_ACC), 
	  to_fixpoint(-0.028842418, SFIXPOINT_ACC), 	to_fixpoint(-0.028474221, SFIXPOINT_ACC), 	to_fixpoint(-0.028004972, SFIXPOINT_ACC),	to_fixpoint(-0.027437767, SFIXPOINT_ACC), 
	  to_fixpoint(-0.026776174, SFIXPOINT_ACC), 	to_fixpoint(-0.02602462 , SFIXPOINT_ACC), 	to_fixpoint(-0.025187887, SFIXPOINT_ACC), 	to_fixpoint(-0.024271343, SFIXPOINT_ACC), 
	  to_fixpoint(-0.02328091 , SFIXPOINT_ACC), 	to_fixpoint(-0.022222936, SFIXPOINT_ACC), 	to_fixpoint(-0.021104068, SFIXPOINT_ACC), 	to_fixpoint(-0.019931411, SFIXPOINT_ACC), 
	  to_fixpoint(-0.01871232 , SFIXPOINT_ACC), 	to_fixpoint(-0.017454365, SFIXPOINT_ACC), 	to_fixpoint(-0.016165324, SFIXPOINT_ACC), 	to_fixpoint(0.0         , SFIXPOINT_ACC)    
	};
	memcpy(a,b, sizeof(sfixpoint) * 36);

}

//REAL FIR_HP_300Hz_highpass(struct FIR_HP_300Hz *fir_hp_300hz, REAL in)
fixpoint FIR_HP_300Hz_highpass(struct FIR_HP_300Hz *fir_hp_300hz, fixpoint in_)
{
	/* z wird nur weitergeschoben und nicht manipuliert */
	memmove(z + 1, z, 35 * sizeof(fixpoint));
	z[0] = in_;

	fixpoint sum;
	sum.lval = 0;
	int j;

	/**
	 * Bilde skalarprodukt von a un z
	 */
#if 1
	for (j = 0; j < 36; j ++) {
		sum = FP_ADD_FFF(sum, FP_MUL_SFF(a[j], z[j]));
	}
#else
{
	unsigned long l = 0, m = 0, h = 0;

	for (j = 0; j < 36; j ++) 
	{
		l+=(unsigned long long)a[j].val.l * (unsigned long long) z[j].val.l;
		m+=(unsigned long long)a[j].val.l * (long long)          z[j].val.h;
		m+=(long long)a[j].val.h * (unsigned long long) z[j].val.l;
		h+=(long long)a[j].val.h * (long long) z[j].val.h;
	}
	sum.lval  = 	(l >> ( SFIXPOINT_ACC + NFIXPOINT_ACC - NFIXPOINT_ACC)) +
			(m << (16 - (SFIXPOINT_ACC + NFIXPOINT_ACC - NFIXPOINT_ACC))) +
			(h << (32 - (SFIXPOINT_ACC + NFIXPOINT_ACC - NFIXPOINT_ACC))) ;
}
#endif

	return sum;
}
#undef z
#undef a
#endif


/* Recursive single pole IIR Infinite Impulse response High-pass filter
 *
 * Reference: The Scientist and Engineer's Guide to Digital Processing
 *
 * 	output[N] = A0 * input[N] + A1 * input[N-1] + B1 * output[N-1]
 *
 *      X  = exp(-2.0 * pi * Fc)
 *      A0 = (1 + X) / 2
 *      A1 = -(1 + X) / 2
 *      B1 = X
 *      Fc = cutoff freq / sample rate
 */
#define in0		(iir1->in0)
#define out0            (iir1->out0)
#define a0              (iir1->a0)
#define a1              (iir1->a1)
#define b1              (iir1->b1)  

#define M_PI		3.1415927
void IIR1_init(struct IIR1 * iir1, fixpoint Fc)
{
//	b1 = float_to_fp_fixpoint(expf(-2.0f * M_PI * Fc));
//	a0 = float_to_fp_fixpoint((1.0f + expf(-2.0f * M_PI * Fc)) / 2.0f);
	b1 = FP_EXPF_FFF(
			FP_MUL_FFF (
				(fixpoint) to_fixpoint(-2.0f * M_PI, NFIXPOINT_ACC),
				Fc
			)
		);

	a0 = FP_MUL_FFF (
			FP_EXPF_FFF (
				FP_MUL_FFF (
					(fixpoint) to_fixpoint(-2.0f * M_PI, NFIXPOINT_ACC),
					Fc
				)
			),
			(fixpoint) to_fixpoint (0.5, NFIXPOINT_ACC)
		);
	a1 = FP_NEG_fixpoint(a0);
	in0.lval = 0;
	out0.lval = 0;
}

fixpoint IIR1_highpass(struct IIR1 * iir1, fixpoint in)
{
//	REAL out = a0 * fixpoint_to_float(in) + a1 * in0 + b1 * out0;
	fixpoint out = FP_ADD_FFF(
				FP_MUL_FFF(a0,in),
				FP_ADD_FFF(
					FP_MUL_FFF(a1,in0),
					FP_MUL_FFF(b1,out0)
				)
			);
	in0 = in;
	out0 = out;
	return out;
}

#undef 	M_PI
#undef in0 
#undef out0
#undef a0  
#undef a1  
#undef b1  

/* Recursive two pole IIR Infinite Impulse Response filter
 * Coefficients calculated with
 * http://www.dsptutor.freeuk.com/IIRFilterDesign/IIRFiltDes102.html
 */
#define x		iir2->x
#define y		iir2->y

void IIR2_init(struct IIR2 * iir2)
{
	x[0].lval = 0;
	x[1].lval = 0;
	y[0].lval = 0;
	y[1].lval = 0;
}

fixpoint IIR2_highpass( struct IIR2 * iir2, fixpoint in_)
{
	static const sfixpoint a[] = { 
		to_fixpoint(0.29289323f ,SFIXPOINT_ACC), 
		to_fixpoint(-0.58578646f,SFIXPOINT_ACC), 
		to_fixpoint(0.29289323f ,SFIXPOINT_ACC) 
	};
	static const sfixpoint b[] = { 
		to_fixpoint(1.3007072E-16f, SFIXPOINT_ACC), 
		to_fixpoint(0.17157288f, SFIXPOINT_ACC) 
	};

	fixpoint out = 	FP_SUB_FFF(
				FP_ADD_FFF( 
					FP_MUL_SFF(a[0], in_),
					FP_ADD_FFF(
						FP_MUL_SFF(a[1], x[0]),
						FP_MUL_SFF(a[2], x[1])
					)
				),
				FP_ADD_FFF(
					FP_MUL_SFF(b[0], y[0]),
					FP_MUL_SFF(b[1], y[1])
				)
			);
	x[1] = x[0];
	x[0] = in_;
	y[1] = y[0];
	y[0] = out;
	return out;
}

#undef x
#undef y

