/* aec.h
 *
 * Copyright (C) DFS Deutsche Flugsicherung (2004, 2005). 
 * All Rights Reserved.
 * Author: Andre Adrian
 *
 * Acoustic Echo Cancellation NLMS-pw algorithm
 *
 * Version 0.3.1 Allow change of stability parameter delta
 * Version 0.3 filter created with www.dsptutor.freeuk.com
 */

#ifndef __FIR_AEC_HEADER__
#define __FIR_AEC_HEADER__

#include "fixpoint.h"

/**
 * Exponential Smoothin or IIR Infinite Impulse Response Filter
 */
struct IIR_HP {
	fixpoint x;
};

void IIR_HP_init(struct IIR_HP *iir_hp);
fixpoint IIR_HP_highpass(struct IIR_HP *iir_hp, fixpoint in);

#if WIDEB==1

/* 17 taps FIR Finite Impulse Response filter
 * Coefficients calculated with
 * www.dsptutor.freeuk.com/KaiserFilterDesign/KaiserFilterDesign.html
 */
struct FIR_HP_300Hz {
	fixpoint z[18];
	sfixpoint a[36];
};
	
void FIR_HP_300Hz_init(struct FIR_HP_300Hz *fir_hp_300hz);
fixpoint FIR_HP_300Hz_highpass(struct FIR_HP_300Hz *fir_hp_300hz, fixpoint in);

#else

/* 36 taps FIR Finite Impulse Response filter
 * Coefficients calculated with
 * www.dsptutor.freeuk.com/KaiserFilterDesign/KaiserFilterDesign.html
 */
struct FIR_HP_300Hz {
	fixpoint z[36];
	sfixpoint a[36];
};

void FIR_HP_300Hz_init(struct FIR_HP_300Hz *fir_hp_300hz);
fixpoint FIR_HP_300Hz_highpass(struct FIR_HP_300Hz *fir_hp_300hz, fixpoint in_);

#endif

struct IIR1 {
	fixpoint in0, out0;
	fixpoint a0, a1, b1;
};

void IIR1_init(struct IIR1 * iir1, fixpoint Fc);
fixpoint IIR1_highpass(struct IIR1 * iir1, fixpoint in);

struct IIR2 {
	fixpoint x[2], y[2];
};

void IIR2_init(struct IIR2 * iir2);
fixpoint IIR2_highpass( struct IIR2 * iir2, fixpoint in_);

#endif /* __FIR_AEC_HEADER__ */

