/******************************************************************************
  Copyright(c) 2006 Sven Kolberg Germany. 

 This file is subject to the terms and conditions of the GNU Library General
 Public License. See the file "COPYING.LIB" in the main directory of this
 archive for more details.

 ******************************************************************************
  File name       :  vecsdot32.asm 
 
  Purpose         :  Real vector dot product for int data types.

  Description     :  This function multiplies each element of
		     input vector `a[]` in 20.12 to each element 
		     of input vector `b[]` in 4.28 and returns the sum in 20.12.
		 
  Registers used  :
  Operands in  R0 & R1 
  R0 - Index to vector A
  R1 - Index to vector B
  R2 - No of elements   (16 bits)
  R3,I0,I1,P2,A0,A1

  Notes          : Input vectors should be aligned on a 4-byte address
		   boundary and be allocated in different banks to achieve 
		   the cycle count given below.

  CYCLE COUNT    : ? + N/2 where N is the number of elements

**************************************************************/
.text;
.global __vecdot_fr32;
.align 2;

__vecdot_fr32:
	    	I0 = R0;	// address of input complex vector1
	    	I1 = R1;	// address of input complex vector2
	    	P2 = R2;	// set loop counter

	    /* Start of Algorithm */
		R0 = 0;

	    LSETUP(ST_VECDOT_32, END_VECDOT_32) LC0 = P2;
ST_VECDOT_32:	R1 = [I0++];			// load a
		R3 = [I1++];			// load b

		A0 = R1.H * R3.H (IS), A1 = R1.H * R3.L (M, IS);
		A1 += R3.H * R1.L (M, IS);
		A0 = A0 << 4;
		A1 = A1 >>> 12;
		A0 += A1;
		A1 = R1.L * R3.L (FU);
		A1 = A1 >>> 28;
		A0 += A1;
		R2 = A0;

END_VECDOT_32:	R0 = R0 + R2;
		RTS;

RETURN_ZERO:
		R0 = 0;
		RTS;
__vecdot_fr32.end:

#if 0
__vecdot_fr32:
	    I0 = R0;                            // ADDRESS OF INPUT COMPLEX VECTOR1
	    CC = R2 <= 0;                       // CHECKS IF NO OF ELEMNETS IS ZERO
	    IF CC JUMP RETURN_ZERO;             // IF TRUE THEN RETURN ZERO AND EXIT

	    I1 = R1;                            // ADDRESS OF INPUT COMPLEX VECTOR2
	    P2 = R2;                            // SET LOOP COUNTER
						// GET INPUTS FROM VECTOR1 AND VECTOR2
	    CC = R2 == 1;                       
	    IF CC JUMP SINGLE;                  // IF N == 1, JUMP

	    R1 = R0 | R1;                       // CHECK THAT INPUT VECTORS ARE 32-BIT ALIGNED
	    R3 = 3;
	    R3 = R1 & R3;
	    CC = R3 == 0;
	    IF !CC JUMP NOT_ALIGNED;		// should always be alligned

NOT_ALIGNED:
SINGLE:
	    /* Start of Algorithm */
	    A1 = A0 = 0 || R1 = [I0++];		// Set A0 and A1 to zero load the first operand
	    R3 = [I1++];			// load the operand

	    LSETUP(ST_VECDOT_32, END_VECDOT_32) LC0 = P2;
ST_VECDOT_32:	A1 = R1.L * R3.L (FU);		// multiply unsigned x unsigned
		A1 = A1 >> 16;			// allign A1 for the next mac
		A1 += R1.H * R3.L (M, IS);	// multiply signed x unsigned
		A1 += R3.H * R1.L (M, IS);	// multiply signed x unsigned
		A1 = A1 >>> 16;			// align A1 for the next mac
		A1 += R1.H * R3.H (IS);		// multiply signed x signed
		A0 += A1; 
		R1 = [I0++];
END_VECDOT_32:	R3 = [I1++];
		A0 = A0 << 4;			// align A1 to a 20.12 fixpoint format

		R0 = A0;			// return the mac value
		RTS;

RETURN_ZERO:
		R0 = 0;
		RTS;
__vecdot_fr32.end:

#endif
