/*
 * (C) Copyright 2006
 * Sven Kolberg, Maxina GmbH & Co KG, s.kolberg@maxina.de
 * Henry Margies, Maxina GmbH & Co KG, h.margies@maxina.de
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have progress a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifdef PROFILE 

#include <common.h>

#include "profile.h"
#include "console.h"

#define MIN_profile_cycles ((profile_cycles ) 0)
#define MAX_profile_cycles ((profile_cycles ) ~0)

/* The horstbox DSP runs at 500Mhz */
#define CCLK_MHZ 500

/* start of list */
struct profile_struct *profile_head = NULL;

/* inactivity counter */
profile_cycles pause_time;

/**
 * read back cpu cycles 
 */
profile_cycles _read_cycles(void) 
{
	profile_cycles val;

	unsigned long l;
	unsigned long h;
	asm volatile ("%R0 = CYCLES ; %0=%R0;" : "=d" (l):	: "R0" );
	asm volatile ("%R0 = CYCLES2; %0=%R0;" : "=d" (h):	: "R0" );

	val = (((profile_cycles) h)<< 32) + (profile_cycles) l;

	return val;
}


/**
 * enable cycle counter 
 */
static void enable_cycle_counter(void)
{
	asm volatile ("R0 = SYSCFG; BITSET(R0,1); SYSCFG = R0;" : : );
}

/**
 * read back enable register 
 */
static unsigned long read_cycle_enable_register(void)
{
	unsigned long val;
	asm volatile ("%R0 = SYSCFG; %0 = %R0; " : "=d" (val): : "R0");

	return val;
}

/**
 * init the profiling point and add it to the profiling list
 */
void profile_init(struct profile_struct *p, char *name) 
{ 
	struct profile_struct **cur_profile; 
	strncpy(p->name, name, sizeof(p->name)); 
	p->start_cycles = 0; 
	p->cycles = 0; 
	p->max_cycles = MIN_profile_cycles; 
	p->min_cycles = MAX_profile_cycles; 
	p->total_cycles = 0; 
	p->num_calls = 0; 
	p->user_counter = 0;
	p->active = 0; 
	p->init = 1;
	p->next = NULL;

	/* append to the list */ 
	cur_profile = &profile_head; 
	do { 
		if(*cur_profile) cur_profile = & (*cur_profile)->next; 
		else {
			(*cur_profile) = p; 
			break; 
		}
	}while(1);
	
	circ_printf("starting profiling for %s(%x)\n", name, p);

	enable_cycle_counter();
}

/**
 * remove the profile from the registration 
 */
void profile_destroy(struct profile_struct *p)
{
	struct profile_struct **cur_profile;

	/* remove from the list */ 
	cur_profile = &profile_head;
	while (*cur_profile) {
		if (*cur_profile == p) {
			*cur_profile = (*cur_profile)->next;
			break;
		}
		cur_profile = & (*cur_profile)->next;
	}
	
	PDEBUG("stop profiling for %s\n", name);
}
		

/**
 * print profile information 
 */
void print_profile(struct profile_struct *p)
{
	circ_printf(
		"name            : %s(%x)\n"
		"total_cycles    : %12lu\n"
		"number of calls : %12lu\n"
		"user counter    : %12lu\n"
		"max cycles      : %12lu (%lu us)\n"
		"min cycles      : %12lu (%lu us)\n"
		"avg time        : %12lu (%lu us)\n"
		, (unsigned long) p->name, p
		, (unsigned long) p->total_cycles
		, (unsigned long) p->num_calls
		, (unsigned long) p->user_counter
		, (unsigned long) p->max_cycles
		, (unsigned long) p->max_cycles / CCLK_MHZ
		, (unsigned long) p->min_cycles
		, (unsigned long) p->min_cycles / CCLK_MHZ
		, (unsigned long) p->total_cycles / p->num_calls
		, (unsigned long) p->total_cycles / (CCLK_MHZ * p->num_calls));
}

/**
 * print all profile infomations
 */
void print_all_profiles(void)
{
	struct profile_struct *cur_profile;
	cur_profile = profile_head;
	while (cur_profile) {
		print_profile(cur_profile);
		cur_profile = cur_profile->next;
	}
}


#endif /* PROFILE */
