/*
 * (C) Copyright 2006
 * Sven Kolberg, Maxina GmbH & Co KG, s.kolberg@maxina.de
 * Henry Margies, Maxina GmbH & Co KG, h.margies@maxina.de
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */


/**
 * The Management Channel Statemachine
 * The slot 0 and slot 1 are used for sending management commands from the 
 * IXP to the Blackfin
 *
 * The format is as follows:
 *
 *   1111 11
 *   5432 1098 76543210
 *  +----+----+--------+
 *  +  a |  b |      c |
 *  +----+----+--------+
 *
 *  a - 4bit command
 *  b - 4bit address
 *  c - 8bit value
 *
 *  Aviable Command are:
 *  	CMGMT_READ	-	read value form address b, ignore c
 *  	CMGMT_WRITE	-	write value c to address b
 *  	CMGMT_STATUS	-	query status information
 *
 *  Aviable Addresses b are:
 *  	CFG_Channel0	
 *  	CFG_Channel1
 *  	CFG_Channel2
 *  	CFG_Channel3
 *  	CFG_Channel4
 *  	CFG_Channel5
 *  	CFG_Channel6
 *  	CFG_Channel7
 *  	CFG_Channel8
 *  	CFG_Channel9
 *
 * The Format c of the Channel registers is as follows:
 *       7 6 543 210
 *  	+-+-+---+---+
 *  	| | |   |   |
 *  	+-+-+---+---+
 *  	 | |  |   |
 *  	 | |  |   +-----Source Format
 *  	 | |  +---------Destination Format
 *  	 | +------------Channel Enable
 *  	 +--------------unused
 *
 * In the reverse direction each command will be Acknowledged:
 * 	CMGMT_READ	will be acknowledged by
 * 		a = CMGMT_READ_ACK
 * 		b = The register which should be read
 * 		c = The value of register b
 * 	CMGMT_WRITE	will be acknowledged by
 * 		a = CMGM_WRITE_ACK
 * 		b = The register which was writen
 * 		c = The conntent of the Register b
 * 	CMGMT_STATUS	will be acknowledged by
 * 		a = CMGMG_STATUS_ACK
 * 		b = The Version Number
 */


#define CMGMT_READ		0x7
#define CMGMT_WRITE		0x8
#define CMGMT_STATUS		0x9

#define CMGMT_READ_ACK		0x7
#define CMGMT_WRITE_ACK		0x8
#define CMGMT_STATUS_ACK	0x9
#define CMGMT_UNKNOWN_CMD	0xf


//#define CODEC_CHANNEL_ENABLE (1<<6)
//#define CODEC_DESTINATION_FORMAT(x) 	((x>>3) & 0x7)
//#define CODEC_SOURCE_FORMAT(x)		(x & 0x7)


/**
 * The Errorlist
 */
#define WRITE_OK				0x1
#define WRITE_ERR_CODEC_NOT_AVIABLE		0x2
#define WRITE_ERR_CHANNEL_NOT_ASSIGNED		0x3
#define WRITE_ERR_UNASSIGNED_REG		0x4

#define CODEC_CHANNEL_ENABLE 		(1<<6)
#define CODEC_DESTINATION_FORMAT(x) 	((x>>3) & 0x7)
#define CODEC_SOURCE_FORMAT(x)		(x & 0x7)

/**
 * The Codecs which we can handle
 */
#define CODEC_SLINEAR	0
#define CODEC_GSM	1

#define ECHOC_FMT_ALAW	2

/* Control Signals send over the Logical Channel */
#define LCT_START		0x0d
#define LCT_CONTINUE		0x12
#define LCT_IDLE		0xe0

void handle_data(unsigned char *rx, unsigned char *tx, unsigned int frames);

