/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Henning Holtschneider <henning@loca.net>                      *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include "lan_common.h"

int main(int argc, char** argv)
{
  NVCL *nvram;
  HLST *avp;
  char *startIp, *endIp, *ipAddress;
  struct in_addr ip_addr;
  uint32_t h_addr, startIpByte, endIpByte, ip_1, ip_2, ip_3;
  char newStartIp[16], newEndIp[16];
	
#ifndef NO_SYSLOG
  openlog("adjust_dhcp", 0, LOG_DAEMON);
#endif

  /* connect to nvram */
  nvram = nvcl_open(0,0,0,0);

  if (!nvram) {

    /* report error */
#ifndef NO_SYSLOG
    closelog();
#endif
    return -1;

  }

  avp = create_from_nvram(nvram);

  if (!avp) {

    /* report error */
    nvcl_close(nvram);
#ifndef NO_SYSLOG
    closelog();
#endif
    return -2;

  }

  /* extract last address byte from DHCP startIp */
  startIp = get_list_value(avp, "lan_dhcp.startIp");

  if(inet_aton(startIp, &ip_addr)) {

    h_addr 	= ntohl(ip_addr.s_addr);
    startIpByte	= (h_addr & 0x000000FF);

  } else {

    nvcl_close(nvram);
#ifndef NO_SYSLOG
    log_msg(LOG_ERR, "invalid lan_dhcp.startIp");
    closelog();
#endif
    return -3;

  }

  /* extract last address byte from DHCP endIp */
  endIp = get_list_value(avp, "lan_dhcp.endIp");

  if(inet_aton(endIp, &ip_addr)) {

    h_addr 	= ntohl(ip_addr.s_addr);
    endIpByte	= (h_addr & 0x000000FF);

  } else {

    nvcl_close(nvram);
#ifndef NO_SYSLOG
    log_msg(LOG_ERR, "invalid lan_dhcp.endIp");
    closelog();
#endif
    return -4;

  }

  /* save first three bytes from HBX IP address */
  ipAddress = get_list_value(avp, "lan_ip.ipAddress");

  if(inet_aton(ipAddress, &ip_addr)) {

    h_addr = ntohl(ip_addr.s_addr);
    ip_1 = (h_addr & 0xFF000000)>>24;
    ip_2 = (h_addr & 0x00FF0000)>>16;
    ip_3 = (h_addr & 0x0000FF00)>>8;

  } else {

    nvcl_close(nvram);
#ifndef NO_SYSLOG
    log_msg(LOG_ERR, "invalid lan_dhcp.endIp");
    closelog();
#endif
    return -4;

  }


  /* put together new DHCP range */
  sprintf(newStartIp, "%d.%d.%d.%d",
	  ip_1, ip_2, ip_3, startIpByte);

  sprintf(newEndIp, "%d.%d.%d.%d",
	  ip_1, ip_2, ip_3, endIpByte);

  /* save to nvram */
  update_to_nvram(nvram, avp, "lan_dhcp.startIp", newStartIp);
  update_to_nvram(nvram, avp, "lan_dhcp.endIp", newEndIp);

  nvcl_close(nvram);
  destroy_hlst(avp);

#ifndef NO_SYSLOG
  closelog();
#endif

  return 0;
}
