/*
 * Copyright (C) 2003-2005 Maxina GmbH - Jordan Hrycaj
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 * Author: Jordan Hrycaj <jordan@mjh.teddy.net.com>
 *
 * $Id: ascii_convert.c 2008 2006-04-07 10:56:40Z jordan $
 */ 

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define _GNU_SOURCE
#include <getopt.h>

/* ----------------------------------------------------------------------- *
 * Private: definitions & declarations
 * ----------------------------------------------------------------------- */

#ifndef HORST_DECODE
# define HORST_DECODE "alpha2horst_decode"
#endif

typedef struct _input {
    FILE *fp ;
    const char *d ;
#   ifdef NEET_UNGET_C
    char *back ;
#   endif /* NEET_UNGET_C */
    int pos ;
} INPUT ;

static const
struct option long_options [] = {
    {"decode",  0, 0, 'd'},
    {"encode",  0, 0, 'e'},
    {"raw",     0, 0, 'r'},
    {0, 0, 0, 0}
};
static const
char short_options [] = "+der";
 
/* ----------------------------------------------------------------------- *
 * Private: ascii to alpha encoding
 * ----------------------------------------------------------------------- */

/* starts at 0x20 */
static long ascii2alpha_table [] = {

  142725, /* 0x20 SPACE */
  142726, /* 0x21   !   */
  142727, /* 0x22   "   */
  142728, /* 0x23   #   */
  142729, /* 0x24   $   */
  142730, /* 0x25   %   */
  142731, /* 0x26   &   */
  142732, /* 0x27   '   */
  142733, /* 0x28   (   */
  142734, /* 0x29   )   */
  142742, /* 0x2a   *   */
  142743, /* 0x2b   +   */
  142744, /* 0x2c   ,   */
  142745, /* 0x2d   -   */
  142746, /* 0x2e   .   */
  142747, /* 0x2f   /   */

  25,     /* 0x30   0   */
  26,     /* 0x31   1   */
  27,     /* 0x32   2   */
  28,     /* 0x33   3   */
  29,     /* 0x34   4   */
  30,     /* 0x35   5   */
  31,     /* 0x36   6   */
  32,     /* 0x37   7   */
  33,     /* 0x38   8   */
  34,     /* 0x39   9   */
  142842, /* 0x3a   :   */
  142843, /* 0x3b   ;   */
  142844, /* 0x3c   <   */
  142845, /* 0x3d   =   */
  142846, /* 0x3e   >   */
  142847, /* 0x3f   ?   */

  142925, /* 0x40   @   */
  42,     /* 0x41   A   */
  43,     /* 0x42   B   */
  44,     /* 0x43   C   */
  45,     /* 0x44   D   */
  46,     /* 0x45   E   */
  47,     /* 0x46   F   */
  48,     /* 0x47   G   */
  49,     /* 0x48   H   */
  50,     /* 0x49   I   */
  51,     /* 0x4a   J   */
  52,     /* 0x4b   K   */
  53,     /* 0x4c   L   */
  54,     /* 0x4d   M   */
  55,     /* 0x4e   N   */
  56,     /* 0x4f   O   */

  57,     /* 0x50   P   */
  58,     /* 0x51   Q   */
  59,     /* 0x52   R   */
  60,     /* 0x53   S   */
  61,     /* 0x54   T   */
  62,     /* 0x55   U   */
  63,     /* 0x56   V   */
  64,     /* 0x57   W   */
  65,     /* 0x58   X   */
  66,     /* 0x59   Y   */
  67,     /* 0x5a   Z   */
  143043, /* 0x5b   [   */
  143044, /* 0x5c   \   */
  143045, /* 0x5d   ]   */
  143046, /* 0x5e   ^   */
  143047, /* 0x5f   _   */

  143125, /* 0x60   `   */
  74,     /* 0x61   a   */
  75,     /* 0x62   b   */
  76,     /* 0x63   c   */
  77,     /* 0x64   d   */
  78,     /* 0x65   e   */
  79,     /* 0x66   f   */
  80,     /* 0x67   g   */
  81,     /* 0x68   h   */
  82,     /* 0x69   i   */
  83,     /* 0x6a   j   */
  84,     /* 0x6b   k   */
  85,     /* 0x6c   l   */
  86,     /* 0x6d   m   */
  87,     /* 0x6e   n   */
  88,     /* 0x6f   o   */

  89,     /* 0x70   p   */
  90,     /* 0x71   q   */
  91,     /* 0x72   r   */
  92,     /* 0x73   s   */
  93,     /* 0x74   t   */
  94,     /* 0x75   u   */
  95,     /* 0x76   v   */
  96,     /* 0x77   w   */
  97,     /* 0x78   x   */
  98,     /* 0x79   y   */
  99,     /* 0x7a   z   */

  143243, /* 0x7b   {   */
  143244, /* 0x7c   |   */
  143245, /* 0x7d   }   */
  0,      /* 0x7e   ~   */

};


/* ----------------------------------------------------------------------- *
 * Private: alpha to ascii decoding
 * ----------------------------------------------------------------------- */

static char alpha_25to99 [] = {

  '0', /* 25 */
  '1', /* 26 */
  '2', /* 27 */
  '3', /* 28 */
  '4', /* 29 */
  '5', /* 30 */
  '6', /* 31 */
  '7', /* 32 */
  '8', /* 33 */
  '9', /* 34 */

  '?', /* 35 */
  '?', /* 36 */
  '?', /* 37 */
  '?', /* 38 */
  '?', /* 39 */
  '?', /* 40 */
  '?', /* 41 */

  'A', /* 42 */
  'B', /* 43 */
  'C', /* 44 */
  'D', /* 45 */
  'E', /* 46 */
  'F', /* 47 */
  'G', /* 48 */
  'H', /* 49 */
  'I', /* 50 */
  'J', /* 51 */
  'K', /* 52 */
  'L', /* 53 */
  'M', /* 54 */
  'N', /* 55 */
  'O', /* 56 */
  'P', /* 57 */
  'Q', /* 58 */
  'R', /* 59 */
  'S', /* 60 */
  'T', /* 61 */
  'U', /* 62 */
  'V', /* 63 */
  'W', /* 64 */
  'X', /* 65 */
  'Y', /* 66 */
  'Z', /* 67 */

  '?', /* 68 */
  '?', /* 69 */
  '?', /* 70 */
  '?', /* 71 */
  '?', /* 72 */
  '?', /* 73 */
	  
  'a', /* 74 */
  'b', /* 75 */
  'c', /* 76 */
  'd', /* 77 */
  'e', /* 78 */
  'f', /* 79 */
  'g', /* 80 */
  'h', /* 81 */
  'i', /* 82 */
  'j', /* 83 */
  'k', /* 84 */
  'l', /* 85 */
  'm', /* 86 */
  'n', /* 87 */
  'o', /* 88 */
  'p', /* 89 */
  'q', /* 90 */
  'r', /* 91 */
  's', /* 92 */
  't', /* 93 */
  'u', /* 94 */
  'v', /* 95 */
  'w', /* 96 */
  'x', /* 97 */
  'y', /* 98 */
  'z', /* 99 */
} ;

static char alpha_1427_25to47 [] = {

   ' ', /* 142725 */
   '!', /* 142726 */
   '"', /* 142727 */
   '#', /* 142728 */
   '$', /* 142729 */
   '%', /* 142730 */
   '&', /* 142731 */
  '\'', /* 142732 */
   '(', /* 142733 */
   ')', /* 142734 */

   '?', /* 142735 */
   '?', /* 142736 */
   '?', /* 142737 */
   '?', /* 142738 */
   '?', /* 142739 */
   '?', /* 142740 */
   '?', /* 142741 */

   '*', /* 142742 */
   '+', /* 142743 */
   ',', /* 142744 */
   '-', /* 142745 */
   '.', /* 142746 */
   '/', /* 142747 */
};

static char alpha_1428_42to47 [] = {
  ':', /* 142842 */
  ';', /* 142843 */
  '<', /* 142844 */
  '=', /* 142845 */
  '>', /* 142846 */
  '?', /* 142847 */
};

static char alpha_1429_25to25 [] = {
  '@', /* 142925 */
};

static char alpha_1430_43to47 [] = {
   '[', /* 143043 */
  '\\', /* 143044 */
   ']', /* 143045 */
   '^', /* 143046 */
   '_', /* 143047 */
};

static char alpha_1431_25to25 [] = {
  '`', /* 143125 */
};

static char alpha_1432_43to45 [] = {
  '{', /* 143243 */
  '|', /* 143244 */
  '}', /* 143245 */
} ;

static struct {
  int  from ;
  int    to ;
  char *map ;
} alpha_1427to1432 [] = {
  { 25, 47, alpha_1427_25to47 }, /* 1427 */
  { 42, 47, alpha_1428_42to47 }, /* 1428 */
  { 25, 25, alpha_1429_25to25 }, /* 1429 */
  { 43, 47, alpha_1430_43to47 }, /* 1430 */
  { 25, 25, alpha_1431_25to25 }, /* 1431 */
  { 43, 45, alpha_1432_43to45 }  /* 1432 */
};

/* ----------------------------------------------------------------------- *
 * Private: input
 * ----------------------------------------------------------------------- */

static
int nextc (INPUT *in) {
    if (in->d) {
#       ifdef NEET_UNGET_C
        if (in->back) {
            int a = in->back ;
            in->back = 0 ;
            return a ;
        }
#       endif /* NEET_UNGET_C */
        return in->d [in->pos] ? in->d [in->pos ++] : EOF ;
    }

    return getc (in->fp);
}

#ifdef NEET_UNGET_C
static
void txenc (int c, INPUT *in) {
    if (in->d)
        in->back = c ;
    else
        ungetc (c, in->fp);
}
#endif /* NEET_UNGET_C */

/* ----------------------------------------------------------------------- *
 * Private: encoding/decoding functions
 * ----------------------------------------------------------------------- */

static void ascii2alpha (FILE *out, INPUT *in, int raw) {
  int c ;

  for (;;) {

    switch (c = nextc (in)) {
    case  EOF:
        return ;
    case  ' ':
    case '\t':
        if (raw == 0)
            c = '_' ;
        break ;
    case 0x7f:
        c = '?' ;
        break ;
    case '\r':
        switch (c = nextc (in)) {
        case EOF:
            return ;
        case '\n':
            if ((c = nextc (in)) == EOF)
                return ;
        default:
            putc ('?', out); /* '\r' */
            putc ('?', out); /* '\n' */
        }
        break ;
    case '\n':
        switch (c = nextc (in)) {
        case EOF:
            return ;
        default:
            putc ('?', out);
        }
        break ;
    default:
        if (c < ' ' || (c & ~0x7f))
            c = '?' ;
    }

    fprintf (out, "%u", ascii2alpha_table [c - 0x20]);
  }
}


static void echar (int n, FILE *out) {
   while (n --) putc ('?', out);
}


static void alpha2ascii (FILE *out, INPUT *in, int raw) {

  for (;;) {
     int c, d, dd, inx, from ;

     /* parse double digit */
     switch (c = nextc (in)) {
     case EOF:
         return ;
     case '\n':
         if ((c = nextc (in)) == EOF)
             return ;
     case '\r':
         switch (c = nextc (in)) {
         case EOF:
             return ;
         case '\n':
             if ((c = nextc (in)) == EOF)
                 return ;
             echar (2, out);
             continue;
         }
     }
     if (c < '0' || '9' < c)      { echar (1, out); continue; }
     if ((d = nextc (in)) == EOF) { echar (1, out); return;   }
     if (d < '0' || '9' < d)      { echar (2, out); continue; }

     dd = (c - '0') * 10 + (d - '0'); 

     if (25 <= dd && dd <= 99) {
        putc (alpha_25to99 [dd - 25], out);
        continue ;
     }

     if (dd != 14) {
        echar (2, out);
        continue ;
     }

     /* parse "27" .. "32" */
     if ((c = nextc (in)) == EOF) { echar (2, out); return;   }
     if (c < '0' || '9' < c)      { echar (3, out); continue; }
     if ((d = nextc (in)) == EOF) { echar (3, out); return;   }
     if (d < '0' || '9' < d)      { echar (4, out); continue; }

     dd = (c - '0') * 10 + (d - '0'); 

     if (dd < 27 || 32 < dd) {
        echar (4, out);
        continue ;
     }

     inx = dd - 27 ;
     
     /* parse double digit */
     if ((c = nextc (in)) == EOF) { echar (4, out); return;   }
     if (c < '0' || '9' < c)      { echar (5, out); continue; }
     if ((d = nextc (in)) == EOF) { echar (5, out); return;   }
     if (d < '0' || '9' < d)      { echar (6, out); continue; }
   
     dd   = (c - '0') * 10 + (d - '0'); 
     from = alpha_1427to1432 [inx].from ;

     if (dd < from || alpha_1427to1432 [inx].to < dd) {
        echar (6, out);
        continue ;
     }

     switch (c = alpha_1427to1432 [inx].map [dd - from]) {
     case ' ':
     case '\t':
        c = '_' ;
     }
     putc (c, out);
  }
}

/* ----------------------------------------------------------------------- *
 * Public: MAIN
 * ----------------------------------------------------------------------- */

static char *pname, mode = 'e' ;

static
void usage (void) {
    fprintf (stderr,
             ("Usage: %s [options] text ...\n"
              "       %s [options] [-]\n"
              "Options: --decode,-d  decode data\n"
              "         --encode,-e  encode data\n"
              "         --raw,-r     don't filter spaces\n"
              "Default mode is -%c\n"), pname, pname, mode) ;
    exit (2);
}

main (int argc, char **argv) {
    int c, option_inx, raw = 0 ;

    if ((pname = strrchr (argv [0], '/')) == 0)
        pname = argv [0] ;
    else
        pname ++ ;

    if (strcmp (pname, HORST_DECODE) == 0)
        mode = 'd' ;

    /* parse opotions */
    while ((c = getopt_long (argc, argv, short_options, 
                             long_options, &option_inx)) != -1)
        switch (c) {
        case 'e':
        case 'd':
            mode = c ;
            continue ;
        case 'r':
            raw = 1 ;
            continue ;
        case 'h':
        case '?':
            usage ();
        }


    do {
        INPUT in ;
        memset (&in, 0, sizeof (in));

        if (optind < argc &&
            (argv [optind] [0] != '-' ||
             argv [optind] [1] != '\0'))
            in.d = argv [optind] ;
        else
            in.fp = stdin ;

        if (mode == 'e')
            ascii2alpha (stdout, &in, raw);
        else
            alpha2ascii (stdout, &in, raw);
    }
    while (++ optind < argc) ;

    exit (0);
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
