/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Roman Schmidt                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/


#include <string.h>
#include <errno.h>

#include "lan_cfg_samba.h"

/**********************************************************
 * Function for building the samba smb.conf
 **********************************************************/

#define SAMBA_CONF	"/etc/smb.conf"
#define MOUNTPATH	"/run/mnt"

/**
 * General section 
 */
static void samba_general(struct info_t* info)
{
        char *workgroup;
		char *gateway;
		char *netip;
		char *netmask;
	
	workgroup 	= get_list_value(info->list, "lan_samba_conf.workgroup");
	gateway 	= get_list_value(info->list, "lan_ip.ipAddress");
	netip 		= get_list_value(info->list, "net.ipAddr");
	netmask 	= get_list_value(info->list, "net.netmask");

	fprintf(info->output,
		"\n[global]\n"
		" workgroup = %s\n"
		" netbios name = HorstBox\n"
		" server string = HorstBox\n"
		" gateway address = %s\n"
		" wins support = yes\n"
		" security = SHARE\n"
		" guest account = nobody\n"
		" encrypt passwords = yes\n"
		" interfaces = %s/%s\n"
		//" bind interfaces only = yes\n"
		"\n"
		, workgroup, gateway, netip, netmask );
	
}


/**
 * Shares section 
 */
static void samba_account_entry(struct info_t *info, const char *id){

	char *name, *mpath;
	char *is_active, *is_phyactive, *is_readonly;
	char *readonly, *guest;
	char *guest_user, *share_user;

	is_active = get_indexed_value(info->list, 
			"lan_samba_conf.shares.isactive", id);
	
	is_phyactive = get_indexed_value(info->list,
			"lan_samba_conf.shares.isphyactive", id);

	/* check if account is enabled */
	if( !is_active || strcmp(is_active,"true") )
		return;
	
	if( !is_phyactive || strcmp(is_phyactive,"true") )
		return;

	name 	= get_indexed_value(info->list, 
				"lan_samba_conf.shares.name", id);

	mpath 	= get_indexed_value(info->list, 
				"lan_samba_conf.shares.mntpoint", id);
	
	/* make sure that only subdirectories of /usb/mnt are contained */
	if( strncmp(mpath, MOUNTPATH, strlen(MOUNTPATH)) )
		return;

	is_readonly = get_indexed_value(info->list,
			"lan_samba_conf.shares.isreadonly", id);
	
	readonly = strcmp(is_readonly, "true") ? "no" : "yes"; 

	guest_user = get_list_value(info->list,"lan_samba_conf.guest");
	share_user = get_indexed_value(info->list,
				"lan_samba_conf.shares.user", id);
	
	guest = strcmp(guest_user, share_user) ? "no" : "yes";
	
	fprintf(info->output,
	 	"\n[%s]\n"
		" path = %s\n"
		" read only = %s\n"
		" guest ok = %s\n"
		,name, mpath, readonly, guest );

	if(! strcmp(guest,"no") )
	{
		fprintf(info->output,
			" valid users = %s\n"
			, share_user);
	}
}

static void samba_accounts(struct info_t* info)
{
	for_each(info->list, "lan_samba_conf.shares.id.",
		(cb_t) samba_account_entry, info);
}


/**
 * Main function
 */

int create_samba_conf(HLST* list)
{
	struct info_t info;

	info.output = fopen(SAMBA_CONF, "w+");
	if (!info.output) {
		log_msg(LOG_ERR, "Error creating %s (%s)", SAMBA_CONF,
				strerror(errno));
		return -1;
	}

	info.list   = list;

	samba_general(&info);
	samba_accounts(&info);

	fclose(info.output);

	return 0;
}

