/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Henry Margies <h.margies@maxina.de> - Maxina GmbH, Neu-       *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/

/*
 * $Log$
 * Revision 1.46  2006/12/18 21:35:41  susi
 * change behaviour of failover account
 *
 * Revision 1.45  2006/12/18 18:59:51  susi
 * Add support for sip failover account. A beep is played, if sip account is not available
 * and failover account is used to dial.
 *
 * Revision 1.44  2006/12/15 14:08:52  henry
 * isdn bug fixed
 *
 * Revision 1.43  2006/12/05 13:58:27  roman
 * removed the agi script (using daemon for watching channel state, now)
 *
 * Revision 1.42  2006/11/24 09:22:35  roman
 * SIP parameter verification function added
 *
 * Revision 1.41  2006/11/08 17:14:24  henry
 * changed pri of fallback for sip and isdn
 *
 * Revision 1.40  2006/11/07 15:17:51  roman
 * - added the agi module and agi call to signalize a call
 *
 * Revision 1.39  2006/08/24 10:47:15  henry
 * fallback for analog and ISDN
 *
 * Revision 1.38  2006/08/04 09:09:00  henry
 * international callerid will be modified to national
 *
 * Revision 1.37  2006/08/03 09:08:44  henry
 * todo added :)board_config/phone/phone_cfg_extension.c
 *
 * Revision 1.36  2006/07/21 15:03:15  henry
 * support for freenet added
 *
 * Revision 1.35  2006/06/20 10:27:10  henry
 * swap, conf, disc added
 *
 * Revision 1.34  2006/06/12 14:22:52  henry
 * bugfix
 *
 * Revision 1.33  2006/06/07 17:33:23  henry
 * makes much more configurable
 *
 * Revision 1.32  2006/05/17 16:21:50  henry
 * callerid support for isdn reactivated
 *
 * Revision 1.31  2006/05/17 14:27:52  henry
 * max_calls added
 *
 * Revision 1.30  2006/05/16 17:13:24  henry
 * offline online check
 *
 * Revision 1.29  2006/05/16 10:09:29  henry
 * bugfix #184
 *
 * Revision 1.28  2006/05/14 14:59:24  susi
 * added method to create phone_stdcall and make an entry within the dialplan
 *
 * Revision 1.27  2006/05/09 22:54:05  henry
 * outgoings call are limited now
 *
 * Revision 1.26  2006/05/06 13:07:52  henry
 * somebody has to change this still
 *
 * Revision 1.25  2006/05/01 16:46:26  henry
 * set caller id maybe not needed
 *
 * Revision 1.24  2006/03/28 14:59:21  henry
 * new sip configs
 *
 * Revision 1.23  2006/03/20 18:09:21  henry
 * bug fixed for empty manipulation prefix
 *
 * Revision 1.22  2006/03/20 10:10:00  henry
 * sip fallback disabled
 *
 * Revision 1.21  2006/02/28 12:08:53  henry
 * .
 *
 * Revision 1.20  2006/02/27 15:18:22  henry
 * failover implemented
 *
 * Revision 1.19  2006/02/20 13:03:41  henry
 * changed rule to support 3 digits for isdn and sip devices
 *
 * Revision 1.18  2006/02/19 16:27:12  henry
 * support for more than 10 devices added
 *
 * Revision 1.17  2006/02/16 19:04:42  henry
 * dnd support added
 *
 * Revision 1.16  2006/02/03 12:06:03  henry
 * added callerid for unknown caller
 *
 * Revision 1.15  2006/02/02 10:20:38  jordan
 * changed callerpres for isdn
 *
 * Revision 1.14  2006/02/02 09:04:56  jordan
 * callerid blocking for isdn added
 *
 * Revision 1.13  2006/01/30 20:52:49  henry
 * callerid blocking added
 *
 * Revision 1.12  2006/01/30 19:15:43  henry
 * caller id blocking added
 *
 * Revision 1.11  2006/01/30 16:38:44  henry
 * fixed setCid cmd
 *
 * Revision 1.10  2006/01/24 17:26:33  henry
 * changed fxs1 and 2
 *
 * Revision 1.9  2006/01/23 22:01:15  henry
 * bugs removed... cleanups.. hostip added
 *
 * Revision 1.8  2006/01/17 12:17:04  henry
 * new standard call handling
 *
 * Revision 1.7  2006/01/15 18:46:33  henry
 * for_seq removed, bug fixed
 *
 * Revision 1.6  2006/01/09 19:48:45  henry
 * new debug entries
 *
 * Revision 1.5  2006/01/02 19:35:27  henry
 * added support for call pickup
 *
 * Revision 1.4  2006/01/02 13:53:44  henry
 * bug in goto to debug extension
 *
 * Revision 1.3  2006/01/02 12:41:54  henry
 * reworked to support msn
 *
 * Revision 1.2  2005/12/22 17:46:21  henry
 * call parking and music on hold demo added
 *
 * Revision 1.1  2005/12/20 00:53:50  henry
 * new phone_config written in c
 *
 *
 */

#include <string.h>
#include <errno.h>

#include "phone_cfg_extension.h"

/**********************************************************
 * Function for creation of asterisks extenstion.conf
 **********************************************************/

#define UNKNOWN		0
#define ANALOG		1
#define ISDN		2
#define SIP		3
#define EXTERNAL	4

#define EXTENSION_CONF	"/etc/asterisk/extensions.conf"

/**
 * Helper
 */
static int get_type(const char* type)
{
	if (!strcmp(type, "analog"))
		return ANALOG;
	
	if (!strcmp(type, "isdn"))
		return ISDN;
	
	if (!strcmp(type, "sip"))
		return SIP;
	
	if (!strcmp(type, "external"))
		return EXTERNAL;

	return UNKNOWN;
}


/**
 * This function creates the general section
 */
static void ext_general(struct info_t* info)
{
	fprintf(info->output, 
		"[general]\n"
		"\n"
		" static=yes\n"
		" writeprotect=yes\n");
}


/**
 * This functions create the global section
 */
static void ext_globals_device(struct info_t* info, const char* id)
{
	char* type;
	char* number;

	type = get_indexed_value(info->list, "phone_devices.type", id);

	if (!type)
		return;

	number = get_indexed_value(info->list, "phone_devices.number", id);
	
	switch (get_type(type)) {

		case ANALOG:
			if (!strcmp(number, "21"))
				fprintf(info->output, " phone_%s=Zap/g4\n", id);
			else
				fprintf(info->output, " phone_%s=Zap/g3\n", id);
			break;

		case ISDN:
			fprintf(info->output, " phone_%s=Zap/g2/%s\n", 
				id, number);
			break;

		case SIP:
			fprintf(info->output, " phone_%s=SIP/%s\n", 
				id, number);
			break;

		case EXTERNAL:
			fprintf(info->output, " phone_%s=local/%s@intern\n", 
				id, number);
			break;

		default:
			log_msg(LOG_ERR, "Invalid type <%s>", type);
			break;
	}
}


static void ext_globals(struct info_t* info)
{
	char* maxcalls;

	maxcalls = get_list_value(info->list, "phone_accounts.max_calls");

	fprintf(info->output, "\n[globals]\n\n" 
			" max_calls = %s\n"
			" DYNAMIC_FEATURES=>swap#conf#disc\n",
			(maxcalls)?maxcalls:"40");

	for_each(info->list, "phone_devices.id.", 
		(cb_t)ext_globals_device, info);
}

/**
 * Functions for local section
 */
static void ext_locals_devices(struct info_t* info, const char* dev_id)
{
	char* number;
	char *ts_names[] = {   	"phone_accounts.timeslot_0", 
				"phone_accounts.timeslot_10", 
				"phone_accounts.timeslot_30", 
				"phone_accounts.timeslot_45", 
				"phone_accounts.timeslot_60", 
				NULL};
	int   ts_times[] = { 	10, 20, 15, 15, 30, 0};
	char* acc_id = info->p1;
	int i, p, t;
	int devid = atoi(dev_id);
	int accid = atoi(acc_id);
	
	number = get_indexed_value(info->list, "phone_devices.number",dev_id);

	fprintf(info->output, 
			" exten => %03d%03d,1,DBget(temp=DND/%s%s)\n"
			" exten => %03d%03d,2,GotoIf(${temp}?3:4)\n"
			" exten => %03d%03d,3,Busy()\n",
			accid, devid, (*number=='3')?"":"*",
			number, 
			accid, devid, accid, devid);
			   
	for (i=0, p=4, t=0; ts_names[i]; i++) {	
		char* ts = get_indexed_value(info->list,ts_names[i], acc_id);
		
		if (id_is_in(ts, dev_id)) {
			t += ts_times[i];
			continue;
		}

		if (t) 
			fprintf(info->output, 
				" exten => %03d%03d,%d,Dial(${phone_%s}, %d, t)\n",
				accid, devid, p++, dev_id, t);
		t = 0;
		fprintf(info->output, 
				" exten => %03d%03d,%d,Wait(%d)\n", 
				accid, devid, p++, ts_times[i]);

	}
	if (!t)
		return;

	fprintf(info->output, " exten => %03d%03d,%d,Dial(${phone_%s}, %d, t)\n", 
				accid, devid, p, dev_id, t);
	
}

static void ext_locals_accounts(struct info_t* info, const char* id)
{
	info->p1 = (char*)id;

	for_each(info->list, "phone_devices.id.", 
		(cb_t)ext_locals_devices, info);
}


static void ext_locals(struct info_t* info)
{
	fprintf(info->output, "\n[local]\n\n");

	for_each(info->list, "phone_accounts.id.", 
		(cb_t)ext_locals_accounts, info);

}

/**
 * Functions for incoming section
 */
static void ext_incoming_fallback(struct info_t* info)
{
	/* if there is an incoming call on an account which is 
	 * not configured, ring all ISDN and analog phones */
	fprintf(info->output, "\n[fallback]\n\n");
	fprintf(info->output, " exten => s,1,ModifyCallerID(doit)\n");
	fprintf(info->output, " exten => s,2,Dial(Zap/g2/&Zap/g3&Zap/g4,90,t)\n");
	fprintf(info->output, " exten => s,3,Busy()\n");
	fprintf(info->output, " exten => _[0-9a-zA-Z#*].,1,ModifyCallerID(doit)\n");
	fprintf(info->output, " exten => _[0-9a-zA-Z#*].,2,Dial(Zap/g2/&Zap/g3&Zap/g4,90,t)\n");
	fprintf(info->output, " exten => _[0-9a-zA-Z#*].,3,Busy()\n");

}


static void ext_incoming_devices(struct info_t* info, const char* dev_id)
{
	char* acc_id = info->p1;

	int devid = atoi(dev_id);
	int accid = atoi(acc_id);

	if (info->i2)
		fprintf(info->output, "&");
	else 
		info->i2 = 1;

	fprintf(info->output, "local/%03d%03d@local", accid, devid);

}


static void ext_incoming_accounts(struct info_t* info, const char* id)
{
	char* type;
	char* ext;
	int   t = info->i1;

	type = get_indexed_value(info->list, "phone_accounts.type", id);

	if (!type || (get_type(type) != t))
		return;
	
	if(get_type(type) == SIP && !sip_parms_ok(info, id))
		return;


	switch (t) {
		case ANALOG:
			ext = "s";
			break;
		case ISDN:
			ext = get_indexed_value(info->list, 
					"phone_accounts.number", id);
			break;
		case SIP:
			ext = get_indexed_value(info->list, 
					"phone_accounts.username", id);
			break;
			
	}

	ext = replace_char(strdup(ext), '@', '_');

	fprintf(info->output, " exten => %s,1,ModifyCallerID(doit)\n", ext);
	fprintf(info->output, " exten => %s,2,Dial(", ext);

	info->p1 = (char*)id;
	info->i2 = 0;
	info->i3++;
	for_each(info->list, "phone_devices.id.", 
		(cb_t)ext_incoming_devices, info);
	
	fprintf(info->output, ", 90,t)\n");
	fprintf(info->output, " exten => %s,3,Busy()\n", ext);

	free(ext);

}

static void ext_incoming(struct info_t* info)
{
	ext_incoming_fallback(info);

	info->i1 = ANALOG;
	info->i3 = 0;
	fprintf(info->output, "\n[analog-in]\n\n");
	for_each(info->list, "phone_accounts.id.", 
		(cb_t)ext_incoming_accounts, info);
	if (!info->i3)
		fprintf(info->output, " exten => s,1,GoTo(fallback,${EXTEN},1)\n");
		

	info->i1 = ISDN;
	info->i3 = 0;
	fprintf(info->output, "\n[isdn-in]\n\n");
	for_each(info->list, "phone_accounts.id.", 
		(cb_t)ext_incoming_accounts, info);
	if (!info->i3)
		fprintf(info->output, " exten => _[0-9#*].,1,GoTo(fallback,${EXTEN},1)\n");

	info->i1 = SIP;
	info->i3 = 0;
	fprintf(info->output, "\n[sip-in]\n\n");
	for_each(info->list, "phone_accounts.id.", 
		(cb_t)ext_incoming_accounts, info);
	if (!info->i3)
		fprintf(info->output, " exten => _[0-9a-zA-Z#*].,1,GoTo(fallback,${EXTEN},1)\n");

}

/**
 * Function for autodial section
 */
static void ext_autodial(struct info_t* info)
{
	char* autodial = get_list_value(info->list, "phone_autodial.number");

	fprintf(info->output, 
		"\n[autodial]\n\n"
		" exten => 0,1,System(nvram -d phone_autodial.number)\n"
		" exten => 0,2,Playback(beep)\n"
		" exten => 0,3,Wait(2)\n"
		" exten => 0,4,Busy()\n"
		" exten => 1,1,SayDigits(%s)\n"
		" exten => 1,2,Wait(2)\n"
		" exten => 1,3,Busy()\n"
		" exten => _[0-9#*].,1,System(nvram phone_autodial.number `IFS='#' \\; echo ${EXTEN} | (read x y \\; echo $x)`)\n"
		" exten => _[0-9#*].,2,Playback(beep)\n"
		" exten => _[0-9#*].,3,Wait(2)\n"
		" exten => _[0-9#*].,4,Busy()\n",
		(autodial)?autodial:"");

}


/**
 * Function for stdcall section
 */
static void ext_stdcall(struct info_t* info)
{
	char* stdcall;
	char*  number;

	fprintf(info->output, "\n[stdcall]\n\n");

	stdcall = get_list_value(info->list, "phone_dials.standardaccount");

	if (!stdcall || !*stdcall)
		return;

	number = get_indexed_value(info->list, "phone_accounts.number", stdcall);

	fprintf(info->output, 
		" exten => _[0-9#*].,1,Goto(account-%s,${EXTEN},1)\n",
		stdcall); 
	
}


/**
 * Function for phone_stdcall section
 */
static void ext_phone_stdcall_entry(struct info_t* info, const char* id)
{
	char* phone_stdcall;
	char* internal_number;
	char* device_type; 
        
	phone_stdcall 	= get_indexed_value(info->list, "phone_devices.stdaccount",id);
	internal_number = get_indexed_value(info->list, "phone_devices.number", id);
	device_type	= get_indexed_value(info->list, "phone_devices.type", id);

	/* should happen for external devices because of no stdaccount */
	if (!phone_stdcall || !internal_number) 
		return;
	
	/* means always use global stdcall */ 
	if (!strcmp(phone_stdcall, "-1")) {

		phone_stdcall = get_list_value(info->list, "phone_dials.standardaccount");

		if (!phone_stdcall || !*phone_stdcall)
			return;
	}

	if(!strcmp(device_type,"isdn")) {

		fprintf(info->output, 
			 " exten => _[0-9#*]./%s,1,Goto(account-%s,${EXTEN},1)\n",
			 internal_number, phone_stdcall);

	} else	{
		/* analog or sip */ 
	
		fprintf(info->output, 
			 " exten => _[0-9#*]./*%s,1,Goto(account-%s,${EXTEN},1)\n",
			 internal_number, phone_stdcall);
	}
}


static void ext_phone_stdcall(struct info_t* info)
{
	fprintf(info->output, "\n[phone_stdcall]\n");

	for_each(info->list, "phone_devices.id.", 
		(cb_t)ext_phone_stdcall_entry, info);
}




/**
 * Debug Entries 
 */
static void ext_debug(struct info_t* info)
{
	fprintf(info->output,
		"\n[debug]\n"
		" exten => _93XX,1,Wait(2)\n"
		" exten => _93XX,2,Record(/tmp/prompt${EXTEN:2}:wav)\n"
		" exten => _93XX,3,Wait(1)\n"
		" exten => _93XX,4,Playback(/tmp/prompt${EXTEN:2})\n"
		" exten => _93XX,5,Wait(2)\n"
		" exten => _93XX,6,Hangup()\n"
		" exten => 94,1,Wait(2)\n"
		" exten => 94,2,Monitor(wav|/tmp/record)\n"
		" exten => 94,3,BackGround(demo-thanks)\n"
		" exten => 94,4,Busy()\n"
		" exten => 95,1,Playback(/tmp/record-in)\n"
		" exten => 95,2,Playback(/tmp/record-out)\n"
		" exten => 96,1,MusicOnHold(default)\n"
		" exten => 97,1,Wait(2)\n"
		" exten => 97,2,Playback(teltest)\n"
		" exten => 97,3,Busy()\n");

}


/**
 * Function for intern-calls section
 */
static void ext_intern_call(struct info_t* info)
{
	fprintf(info->output, 
		"\n[intern-call]\n"
		" exten => 21,1,Dial(Zap/g4, 60, tT)\n"
		" exten => 22,1,Dial(Zap/g3, 60, tT)\n"
		" exten => _3XX,1,Dial(Zap/g2/${EXTEN}, 60, tT)\n"
		" exten => _4XX,1,Dial(SIP/${EXTEN}, 60, tT)\n"
		" exten => _[0-9].,2,Busy()\n");
}

/**
 * Function for extern-calls section
 */
static void ext_list_analog_account(struct info_t* info, const char* id)
{
	char* block;
	char* block_pre;
	char* number;
	char* prefix = NULL;
	
	number = get_indexed_value(info->list, "phone_accounts.number", id);
	block = get_indexed_value(info->list, "phone_accounts.block_id", id);
	block_pre = get_indexed_value(info->list, "phone_accounts.block_pre", id);
	
	if (block && !strcmp(block, "true"))
		prefix = block_pre;
	
	fprintf(info->output,
		" exten => _[0-9#*].,4,System(test `cat /proc/si3050/status` = Online)\n"
		" exten => _[0-9#*].,5,NoOp(${SYSTEMSTATUS})\n"
		" exten => _[0-9#*].,6,GotoIf($[${SYSTEMSTATUS} != SUCCESS]?104)\n"
		" exten => _[0-9#*].,7,Dial(Zap/g5/%s${EXTEN},60, T)\n"
		" exten => _[0-9#*].,104,Busy()\n",
		(prefix)?prefix:"");
}

static void ext_list_isdn_account(struct info_t* info, const char* id)
{
	char* number;
	char* block;
	int   callingpres = 0;
	
	number = get_indexed_value(info->list, "phone_accounts.number", id);
	block = get_indexed_value(info->list, "phone_accounts.block_id", id);
	
	if (block && !strcmp(block, "true"))
		callingpres = 32;
	
	fprintf(info->output, 
		" exten => _[0-9#*].,4,System(echo off > /proc/hfc/alarm)\n"
		" exten => _[0-9#*].,5,Wait(1)\n"
		" exten => _[0-9#*].,6,CallingPres(%d)\n"
		" exten => _[0-9#*].,7,Set(CALLERID(number)=%s)\n"
		" exten => _[0-9#*].,8,Dial(Zap/g1/${EXTEN},60, T)\n"
		" exten => _[0-9#*].,10,Busy()\n",
		callingpres, number);
}


static void ext_list_sip_account(struct info_t* info, const char* id)
{
	char *server, *failover, *user, *c, *account;

	server = get_indexed_value(info->list, "phone_accounts.server", id);
	user   = get_indexed_value(info->list, "phone_accounts.username", id);
	failover = get_list_value(info->list, "phone_dials.sipfailover");
	
	account = strdup(user);
	while((c=strchr(account, '@'))) {
		*c = '_';
	}
	
	/* TODO ADD BEEP if fail over */

	/* if the account id is same as failover id, don't configure a failover, just make busy, if account is unavailable */
	if(strcmp(failover,id))
	{
		fprintf(info->output, 
			" exten => _[0-9#*].,4,System('/usr/share/board_cfg/phone/phone_sipavail.sh')\n"
			" exten => _[0-9#*].,5,ChanIsAvail(SIP/${EXTEN}@%s|j)\n"
			" exten => _[0-9#*].,6,Dial(SIP/${EXTEN}@%s,60, T)\n"
			" exten => _[0-9#*].,7,Busy()\n"
	//		" exten => _[0-9#*].,105,NoOp(\"Ping Failed\")\n"
			" exten => _[0-9#*].,106,NoOp(\"ChanAvail Failed\")\n"
			" exten => _[0-9#*].,107,BackGround(beep)\n"
			" exten => _[0-9#*].,108,Goto(account-%s,${EXTEN},1)\n"
			" exten => _[0-9#*].,109,Busy()\n",
			server, /*id, */ /*id*/ account, failover);
	}
	else{
		fprintf(info->output, 
			" exten => _[0-9#*].,4,System('/usr/share/board_cfg/phone/phone_sipavail.sh')\n"
			" exten => _[0-9#*].,5,ChanIsAvail(SIP/${EXTEN}@%s|j)\n"
			" exten => _[0-9#*].,6,Dial(SIP/${EXTEN}@%s,60, T)\n"
			" exten => _[0-9#*].,7,Busy()\n"
			" exten => _[0-9#*].,106,NoOp(\"ChanAvail Failed\")\n"
			" exten => _[0-9#*].,107,Busy()\n",
			server, /*id, */ /*id*/ account);
	}

	free(account);
}


static void ext_list_accounts(struct info_t* info, const char* id)
{
	char* type;

	type = get_indexed_value(info->list, "phone_accounts.type", id);

	if (!type)
		return;
	
	if (get_type(type) == SIP && !sip_parms_ok(info, id))
		return;


	/* limit outgoing calls */
	fprintf(info->output, "\n[account-%s]\n"
		" exten => _[0-9#*].,1,Set(GROUP()=OUTBOUND_GROUP)\n"
		" exten => _[0-9#*].,2,GotoIf($[${GROUP_COUNT()} <= ${max_calls}]?5)\n"
		" exten => _[0-9#*].,3,Busy()\n", id);

	switch (get_type(type)) {
		case ANALOG:
			ext_list_analog_account(info, id);
			break;

		case ISDN:
			ext_list_isdn_account(info, id);
			break;
		case SIP:
			ext_list_sip_account(info, id);
			break;

		default:
			return;
			
	}
}

static void ext_extern_accounts(struct info_t* info, const char* id)
{
	char* type;
	char  counter[32];
	int   pre;

	type = get_indexed_value(info->list, "phone_accounts.type", id);

	if (!type)
		return;

	switch (get_type(type)) {
		case ANALOG:
			counter[0] = '\0';
			pre = 5;
			break;

		case ISDN:
			snprintf(counter, sizeof(counter), "%d", info->i1++);
			pre = 6;
			break;
		case SIP:
			snprintf(counter, sizeof(counter), "%d", info->i2++);
			pre = 7;
			break;

		default:
			return;
			
	}
	fprintf(info->output, 
		" exten => _%d%sX.,1,GoTo(account-%s,${EXTEN:%d},1)\n",
		pre, counter, id, strlen(counter)+1);
}

static void ext_extern_call(struct info_t* info)
{

	for_each(info->list, "phone_accounts.id.", 
		(cb_t)ext_list_accounts, info);

	fprintf(info->output, "\n[extern-call]\n");
	
	info->i1=0;
	info->i2=0;
	
	for_each(info->list, "phone_accounts.id.", 
		(cb_t)ext_extern_accounts, info);
	
												
}

/**
 * Function for quickdial section
 */
static void ext_quickdial_entry(struct info_t* info, const char* id)
{
	char* name, *number;
	
	name = get_indexed_value(info->list, "phone_quickdials.quickdial", id);
	number = get_indexed_value(info->list, "phone_quickdials.number", id);
	if (!name || !number)
		return;

	fprintf(info->output, " exten => %s,1,GoTo(intern,%s,1)\n", name, number);
}


static void ext_quickdial(struct info_t* info)
{
	fprintf(info->output, "\n[quickdial]\n");

	for_each(info->list, "phone_quickdials.id.", 
		(cb_t)ext_quickdial_entry, info);
}


/**
 * Function for intern section
 */
static void ext_intern_autodial(struct info_t* info)
{
	char* autodial;
	autodial = get_list_value(info->list, "phone_autodial.number");
	if (autodial && *autodial) {
		fprintf(info->output, 
			" exten => s,1,GoTo(intern,%s,1)\n"
			" exten => s,2,Busy()\n",
			autodial);
	} else {
		fprintf(info->output, " exten => s,1,Busy()\n");
	}
	
}

static void ext_intern_devices(struct info_t* info, char* prefix)
{
	if (!prefix || !*prefix)
		prefix = "*";

	fprintf(info->output, 
		"\n ; Direct Device Access\n"
		" exten => _%s[234].,1,GoTo(intern-call,${EXTEN:%d},1)\n",
		prefix, strlen(prefix));
}

static void ext_intern_accounts(struct info_t* info, char* prefix)
{
	if (!prefix || !*prefix)
		prefix = "*";

	fprintf(info->output, 
		"\n ; Direct Account Access\n"
		" exten => _%s[567].,1,GoTo(extern-call,${EXTEN:%d},1)\n",
		prefix, strlen(prefix));
}

static void ext_intern_parking(struct info_t* info, char* prefix)
{
	if (!prefix || !*prefix)
		prefix = "*";

	fprintf(info->output, 
		"\n ; Call Parking\n"
		" exten => _%s8.,1,GoTo(parkedcalls,${EXTEN:%d},1)\n"
		" exten => %s8,1,PickUp(1)\n",
		prefix, strlen(prefix)+1);
}

static void ext_intern_dnd(struct info_t* info, char* prefix)
{
	if (!prefix || !*prefix)
		prefix = "*";

	fprintf(info->output, 
		"\n ; DND\n"
		" exten => %s91,1,DBput(DND/${CALLERIDNUM}=1)\n"
		" exten => %s91,2,Busy()\n"
		" exten => %s92,1,DBdel(DND/${CALLERIDNUM})\n"
		" exten => %s92,2,Busy()\n",
		prefix, prefix, prefix, prefix);
}

static void ext_intern_quickdial(struct info_t* info, char* prefix)
{
	int size;

	if (!prefix || !*prefix)
		prefix = "*";

	size = strlen(prefix);
	
	fprintf(info->output, 
		"\n ; Quick Dials\n"
		" exten => _%sX.,1,GoTo(quickdial,${EXTEN:%d},1)\n"
		" exten => _%sX,1,GoTo(quickdial,${EXTEN:%d},1)\n",
		prefix, size, prefix, size);
}

static void ext_intern_debug(struct info_t* info, char* prefix)
{
	int size;

	if (!prefix || !*prefix)
		prefix = "*";

	size = strlen(prefix);
	
	fprintf(info->output, 
		"\n ; Debug\n"
		" exten => _%s9[3456789],1,GoTo(debug,${EXTEN:%d},1)\n"
		" exten => _%s9[3456789].,1,GoTo(debug,${EXTEN:%d},1)\n",
		prefix, size, prefix, size);
}

static void ext_intern_autodial_cfg(struct info_t* info, char* prefix)
{
	int size;

	if (!prefix || !*prefix)
		prefix = "*";

	size = strlen(prefix);
	
	fprintf(info->output, 
		"\n ; Dial on Timeout\n"
		" exten => _%s53*X.#,1,GoTo(autodial,${EXTEN:%d},1)\n"
		" exten => _#53#,1,GoTo(autodial,0,1)\n"
		" exten => _%s#53#,1,GoTo(autodial,1,1)\n",
		prefix, size+3, prefix);

}


static void ext_intern(struct info_t* info)
{
	char prefix_string[] = "**";
	char* autodial;
	int   prefix_size    = 1;


	fprintf(info->output, "\n[intern]\n");

	ext_intern_autodial(info);

	ext_intern_devices(info, 
		get_list_value(info->list, "phone_prefix.devices"));

	ext_intern_accounts(info,
		get_list_value(info->list, "phone_prefix.accounts"));

	ext_intern_parking(info,
		get_list_value(info->list, "phone_prefix.parking"));

	ext_intern_dnd(info,
		get_list_value(info->list, "phone_prefix.dnd"));

	ext_intern_quickdial(info,
		get_list_value(info->list, "phone_prefix.quickdial"));

	ext_intern_debug(info,
		get_list_value(info->list, "phone_prefix.debug"));

	ext_intern_autodial_cfg(info,
		get_list_value(info->list, "phone_prefix.autodial"));


	
}


/**
 * Function for dial plan functionality
 */
static void ext_dialplan_period(struct info_t* info, const char* id)
{
	char *hour_from, *hour_to, *min_from, *min_to;
	char* per;
	char* prefix = info->p1;
	char  timestring[32];
	int   i;
	
	per = get_indexed_value(info->list, "phone_dials_config.period", id);

	/* without timetable */
	if (!per || strcmp(per, "userdefined")) {
		fprintf(info->output, 
			" exten => _%s,1,Goto(9)\n"
			" exten => _%s.,1,Goto(9)\n", prefix, prefix);
		return;
	}

	/* with time */
	hour_from = get_indexed_value(info->list, "phone_dials_config.fromhours", id);
	hour_to	  = get_indexed_value(info->list, "phone_dials_config.tohours", id);
	
	min_from = get_indexed_value(info->list, "phone_dials_config.fromminutes", id);
	min_to	 = get_indexed_value(info->list, "phone_dials_config.tominutes", id);
	snprintf(timestring, sizeof(timestring), "%s:%s-%s:%s", 
			hour_from, min_from, hour_to, min_to);

	for (i=0;i<7;i++) {
		char use_lbl[128];
		char* days[] = {"mon", "tue", "wed", 
				"thu", "fri", "sat", "sun"};
		char* use;

		snprintf(use_lbl, sizeof(use_lbl),
				"phone_dials_config.%s", days[i]);
		use = get_indexed_value(info->list, use_lbl, id);

		if (use && strcmp(use, "true")) {
			fprintf(info->output, 
					" exten => _%s,%d,NoOp()\n"
					" exten => _%s.,%d,NoOp()\n",
					prefix, i+1, prefix, i+1);
			continue;
		}
		fprintf(info->output, 
				" exten => _%s,%d,GotoIfTime(%s|%s||?9)\n"
				" exten => _%s.,%d,GotoIfTime(%s|%s||?9)\n",
				prefix, i+1, timestring, days[i],
				prefix, i+1, timestring, days[i]);

	}

}

static void ext_dialplan_entry(struct info_t* info, const char* id)
{
	char *prefix, *rule, *mani, *account, *mmod, *mpre;

	rule = get_indexed_value(info->list, "phone_dials_config.rule", id);
	prefix = get_indexed_value(info->list,"phone_dials_config.prefix", id);
	info->p1 = prefix;

	ext_dialplan_period(info, id);

	fprintf(info->output, 
			" exten => _%s,8,GoTo(stdcall,${EXTEN},1)\n"
			" exten => _%s.,8,GoTo(stdcall,${EXTEN},1)\n",
			prefix, prefix);

	/* block call */
	if (rule && strcmp(rule, "connect")) {
		fprintf(info->output,
			" exten => _%s,9,Busy()\n"
			" exten => _%s.,9,Busy()\n", prefix, prefix);
		return;
			      
	}

	account = get_indexed_value(info->list, "phone_dials_config.account", id);
	mani	= get_indexed_value(info->list, "phone_dials_config.manipulation", id);

	/* without number manipulation */
	if (mani && strcmp(mani, "true")) {
		fprintf(info->output,
			" exten => _%s,9,GoTo(account-%s,${EXTEN},1)\n"
			" exten => _%s.,9,GoTo(account-%s,${EXTEN},1)\n",
			prefix, account, prefix, account);

		return;

	}

	/* with manipulation */
	mmod = get_indexed_value(info->list, "phone_dials_config.manipulationModifier", id);
	mpre = get_indexed_value(info->list, "phone_dials_config.manipulationPrefix", id);

	fprintf(info->output, 
		" exten => _%s,9,GoTo(account-%s,%s${EXTEN}:%d,1)\n"
		" exten => _%s.,9,GoTo(account-%s,%s${EXTEN:%d},1)\n",
		prefix, account, mmod, (mpre)?strlen(mpre):0,
		prefix, account, mmod, (mpre)?strlen(mpre):0);

}


static void ext_dialplan(struct info_t* info)
{
	fprintf(info->output, "\n ; Dial Plan\n");
	for_each(info->list, "phone_dials_config.id.", 
		(cb_t)ext_dialplan_entry, info);

/*	added by Susanne for testing */
/*	fprintf(info->output, " exten => _[0-9#*].,1,GoTo(phone_stdcall,${EXTEN},1)\n"); */
	fprintf(info->output, " include => phone_stdcall\n");
/*	end */

/*	fprintf(info->output, " exten => _[0-9#*].,1,GoTo(stdcall,${EXTEN},1)\n"); */
	fprintf(info->output, " include => stdcall\n");
}


/**
 * Main function
 */

int create_extension_conf(HLST* list)
{
	struct info_t info;

	info.output = fopen(EXTENSION_CONF, "w+");
	if (!info.output) {
		log_msg(LOG_ERR, "Error creating %s (%s)", EXTENSION_CONF,
				strerror(errno));
		return -1;
	}

	info.list   = list;

	ext_general(&info);
	ext_globals(&info);
	ext_locals(&info);
	ext_incoming(&info);
	ext_autodial(&info);
	ext_stdcall(&info);
	ext_phone_stdcall(&info);		/* added by Susanne for testing */
	ext_debug(&info);
	ext_intern_call(&info);
	ext_extern_call(&info);
	ext_quickdial(&info);
	ext_intern(&info);
	ext_dialplan(&info);

	fclose(info.output);

	return 0;
}

