/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Henry Margies <h.margies@maxina.de> - Maxina GmbH, Neu-       *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/

/*
 * $Log$
 * Revision 1.6  2006/11/22 09:08:55  henry
 * further tests added, whether to use dsp or not
 *
 * Revision 1.5  2006/11/17 14:57:11  henry
 * hbx_dsp codec added
 *
 * Revision 1.4  2006/11/07 15:17:51  roman
 * - added the agi module and agi call to signalize a call
 *
 * Revision 1.3  2006/10/30 10:02:59  henry
 * .
 *
 * Revision 1.2  2006/10/30 09:53:25  henry
 * dsp added
 *
 * Revision 1.1  2006/10/24 15:54:57  henry
 * dsp_gsm added
 *
 *
 */

#include <string.h>
#include <errno.h>

#include "phone_cfg_modules.h"

/**********************************************************
 * Function for creation of asterisks modules.conf
 **********************************************************/

#define MODULES_CONF	"/etc/asterisk/modules.conf"



/**
 * General section 
 */
static void modules_modules(struct info_t* info)
{
	char* v;
	int   has_dsp = 0;


	v = get_list_value(info->list, "sys_hardware.use_dsp");
	if (v && !strcmp(v, "true"))
		has_dsp = 1;//0;

	fprintf(info->output,
		"\n[modules]\n"
		"autoload=no\n\n"
		"load => func_callerid.so\n"
		"load => res_musiconhold.so\n"
		"load => res_features.so\n"
		"load => res_agi.so\n"
		"load => chan_zap.so\n"
		"load => chan_sip.so\n"
		"load => chan_local.so\n"
		"load => codec_alaw.so\n"
		"load => codec_ulaw.so\n"
		"load => codec_gsm.so\n"
		"load => codec_g726.so\n"
		"load => codec_ilbc.so\n"
		";load => codec_speex.so\n"
		";load => codec_g723.so\n"
		"load => pbx_config.so\n"
		"load => pbx_spool.so\n"
		"load => pbx_functions.so\n"
		"load => format_gsm.so\n"
		"load => format_wav.so\n"
		"load => format_pcm_alaw.so\n"
		"load => format_pcm.so\n"
		"load => app_dial.so\n"
		"load => app_pickup.so\n"
		"load => app_playback.so\n"
		"load => app_record.so\n"
		"load => app_setcallerid.so\n"
		"load => app_modifycallerid.so\n"
		"load => app_callingpres.so\n"
		"load => app_cdr.so\n"
		"load => app_setcidnum.so\n"
		"load => app_db.so\n"
		"load => app_system.so\n"
		"load => app_chanisavail.so\n"
		"load => cdr_manager.so\n"
		"load => cdr_csv.so\n"
		"load => res_monitor.so\n");

	if (has_dsp)
		fprintf(info->output, "load => codec_dsp_gsm.so\n");
}


static void modules_global(struct info_t* info)
{
	fprintf(info->output, "\n[global]\n");
	
}


/**
 * Main function
 */

int create_modules_conf(HLST* list)
{
	struct info_t info;

	info.output = fopen(MODULES_CONF, "w+");
	if (!info.output) {
		log_msg(LOG_ERR, "Error creating %s (%s)", MODULES_CONF,
				strerror(errno));
		return -1;
	}

	info.list   = list;

	modules_modules(&info);
	modules_global(&info);

	fclose(info.output);

	return 0;
}

