/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Henry Margies <h.margies@maxina.de> - Maxina GmbH, Neu-       *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/

/*
 * $Log$
 * Revision 1.27  2006/11/24 09:22:35  roman
 * SIP parameter verification function added
 *
 * Revision 1.26  2006/11/17 11:00:40  henry
 * dtmfmode=info added
 *
 * Revision 1.25  2006/09/12 15:20:56  henry
 * bug 219 fixed
 *
 * Revision 1.24  2006/07/21 15:03:15  henry
 * support for freenet added
 *
 * Revision 1.23  2006/07/21 09:51:48  henry
 * sip proxy support added
 *
 * Revision 1.22  2006/06/27 15:22:20  henry
 * bug fix
 *
 * Revision 1.21  2006/06/20 10:27:30  henry
 * .
 *
 * Revision 1.20  2006/05/09 14:46:18  henry
 * support for multiple pvcs added
 *
 * Revision 1.19  2006/04/25 19:54:06  henry
 * tos=0x18 added
 *
 * Revision 1.18  2006/04/25 18:20:40  henry
 * tos=lowdelay added
 *
 * Revision 1.17  2006/04/18 08:14:40  henry
 * default expiry added
 *
 * Revision 1.16  2006/04/03 14:45:39  henry
 * dont configure sip accounts if device is offline
 *
 * Revision 1.15  2006/03/28 14:59:21  henry
 * new sip configs
 *
 * Revision 1.14  2006/03/20 10:10:08  henry
 * sip fallback disabled
 *
 * Revision 1.13  2006/03/06 14:42:22  henry
 * added new realm
 *
 * Revision 1.12  2006/02/28 12:08:53  henry
 * .
 *
 * Revision 1.11  2006/02/15 15:17:14  henry
 * better localnet handling
 *
 * Revision 1.10  2006/02/10 14:31:32  henry
 * uses dsl board ip now
 *
 * Revision 1.9  2006/02/03 12:04:30  henry
 * added callerid for sip phones
 *
 * Revision 1.8  2006/01/24 14:48:54  henry
 * set nat to yes for phones
 *
 * Revision 1.7  2006/01/23 22:01:15  henry
 * bugs removed... cleanups.. hostip added
 *
 * Revision 1.6  2006/01/19 10:45:48  henry
 * speex codec added
 *
 * Revision 1.5  2006/01/19 10:10:52  henry
 * speex included
 *
 * Revision 1.4  2006/01/09 16:30:25  henry
 * use our ip address as bind address
 *
 * Revision 1.3  2006/01/03 15:43:48  henry
 * dtmfmode changed to info
 *
 * Revision 1.2  2005/12/21 23:26:55  henry
 * stupid bug fixed
 *
 * Revision 1.1  2005/12/20 00:53:50  henry
 * new phone_config written in c
 *
 *
 */

#include <string.h>
#include <errno.h>

#include "phone_cfg_sip.h"

/**********************************************************
 * Function for creation of asterisks extenstion.conf
 **********************************************************/

#define SIP_CONF	"/etc/asterisk/sip.conf"

/** 
 * External IP Support
 */
void stun_service_update(struct info_t* info, const char* id)
{
	char mapped_lbl[1024];
	char server_lbl[1024];
	char list[1024];
	char* stun_server;
	char* stun_list;
	char*  server;

	snprintf(server_lbl, sizeof(server_lbl), "stun_service.%s.query", id);
	stun_server = get_list_value(info->list, server_lbl);

	server = get_indexed_value(info->list, "phone_accounts.stun_server", id);

	/* no update needed? */
	if (stun_server && *stun_server && !strcmp(stun_server, server))
		return; 
	
	/* set new server */
	update_to_nvram(info->nvram, info->list, server_lbl, server);
	snprintf(mapped_lbl, sizeof(mapped_lbl), "stun_service.%s.mapped", id);
	delete_hlst(info->list, mapped_lbl, 0);	

	stun_list = get_list_value(info->list, "stun_service.list");
	update_to_nvram(info->nvram, info->list, mapped_lbl, "");
	nvram_link(info->nvram, mapped_lbl, LINK_MODE, LINK_TARGET);


	/* id is already in list */
	if (stun_list && *stun_list && id_is_in(stun_list, id))
		return;

	/* set new server */
	if (!stun_list || !*stun_list) {
		update_to_nvram(info->nvram, info->list, "stun_service.list", id);
		return;
	}
	
	snprintf(list, sizeof(list), "%s %s", *stun_list, id);
	update_to_nvram(info->nvram, info->list, "stun_service.list", list);
}


char* stun_get_ip(struct info_t* info, const char* id)
{
	char mapped_lbl[1024];
	char run_stunc[1024];
	char *ip, *delay;
	int i;

	/* first check if ip is already there */
	snprintf(mapped_lbl, sizeof(mapped_lbl), "stun_service.%s.mapped", id);
	ip = get_list_value(info->list, mapped_lbl);

	if (ip && *ip)
		return ip;
	
	/* if not, start stunc */
	delay = get_list_value(info->list, "stun_service.delay");
	if (!delay || !*delay) {
		log_msg(LOG_ERR, "stun_service.delay is not set");
		return NULL;
	}
		
	snprintf(run_stunc, sizeof(run_stunc), "stunc stun 10 %s", *delay);
	system("stunc reload");
	system(run_stunc);

	/* check and wait for new ip */
	i=3;
	while(i--){
		update_from_nvram(info->nvram, info->list, mapped_lbl);
		ip = get_list_value(info->list, mapped_lbl);
		if (ip && *ip)
			return ip;

		sleep(1);
	}

	log_msg(LOG_ERR, "Error in getting stun ip (timeout)");
	return NULL;
	
}


char* get_ext_ip(struct info_t* info, const char* id)
{
	char* stun;
	char* pvc;

	stun = get_indexed_value(info->list, "phone_accounts.use_stun", id);

	if (stun && !strcmp(stun, "yes")) {
		/* use stun to get ip */
		stun_service_update(info, id);
		return stun_get_ip(info, id);
	}

	/* use IP from dsl board */

	/* read from dsl board */
	pvc = get_indexed_value(info->list, "phone_accounts.pvc", id);

	if (!pvc)
		return get_list_value(info->list, "wan_dsl.ip_address.1");

	return get_indexed_value(info->list, "wan_dsl.ip_address",pvc);
}



/**
 * General section 
 */
static void sip_general(struct info_t* info)
{
	char* ip;
	char* dsl_ip;
	char* lang;
	
	ip = get_list_value(info->list, "net.ipAddr");
	dsl_ip = get_list_value(info->list, "lan_ip.ipAddress");
	lang = get_list_value(info->list, "global.lang");

	fprintf(info->output,
		"\n[general]\n"
		" realm = HorstBox\n"
		" useragent = HorstBox\n"
		" callerid = HorstBox\n"
		" port = 5060\n"
		" bindaddr = %s\n"
		" hostip = %s\n"
		" defaultexpirey = 3600\n"
		" maxexpirey = 7200\n"
		" context = default\n"
		" qualify = no\n"
		" allowguest = no\n"
		" disable = all\n"
		" allow = alaw\n"
		" allow = ulaw\n"
		" allow = g729\n"
		" allow = gsm\n"
		" allow = speex\n"
		" allow = slinear\n"
		" srvlookup = no\n"
		" canreinvite = no\n"
		" dtmfmode = info\n"
		" tos = 0x18\n"
		" language = de\n", ip, dsl_ip, lang);
	
}

/**
 * Provider 
 */
static void get_localnet(struct info_t* info, char* localnet, int size)
{
	char *lan_ip, *lan_mask;
	int ip[4], mask[4], net[4];

	lan_ip   = get_list_value(info->list, "lan_ip.ipAddress");
	lan_mask = get_list_value(info->list, "lan_ip.subnetMask");

	if (!lan_ip   || !*lan_ip   || 
	    !lan_mask || !*lan_mask ) {
		strncpy(localnet, "", size);
		return;
	}

	sscanf(lan_ip, "%d.%d.%d.%d", &ip[0], &ip[1], &ip[2], &ip[3]);
	sscanf(lan_mask, "%d.%d.%d.%d", &mask[0], &mask[1], &mask[2], &mask[3]);

	net[0] = ip[0] & mask[0];
	net[1] = ip[1] & mask[1];
	net[2] = ip[2] & mask[2];
	net[3] = ip[3] & mask[3];

	snprintf(localnet, size, "%d.%d.%d.%d/%d.%d.%d.%d", 
				net[0], net[1], net[2], net[3],
				mask[0], mask[1], mask[2], mask[3]);
}


static void sip_provider_entry2(struct info_t* info, const char* id)
{
	char *type, *server, *user, *passwd, *ext_ip, *cont, *domain;
	char localnet[1024];

	type   = get_indexed_value(info->list, "phone_accounts.type", id);
	if (!type || strcmp(type, "sip"))
		return;

	/* First check if we are really online. If not skip whole 
	 * sip account section */
	ext_ip = info->p1;

	if (!ext_ip || !strcmp(ext_ip, "0.0.0.0"))
		return;

	if (!sip_parms_ok(info, id))
	        return;

	server = get_indexed_value(info->list, "phone_accounts.server", id);
	user   = get_indexed_value(info->list, "phone_accounts.username", id);
	passwd = get_indexed_value(info->list, "phone_accounts.password", id);

	get_localnet(info, localnet, sizeof(localnet));
	
	/* if username contains @ sign, extract domain from username */
	domain = strchr(user, '@');
	if (domain) 
		domain++;
	else
		domain = server;

	cont = replace_char(strdup(user), '@', '_');
	user = replace_char(strdup(user), '@', '\0');
	
	fprintf(info->output,
/*		"\n[sip_account_%s]\n"*/
		"\n[%s]\n"
		" type     = peer\n"
		" nat      = yes\n"
		" externip = %s\n"
		" localnet = %s\n"
		" insecure = very\n"
		" username = %s\n"
		" fromuser = %s\n"
		" secret   = %s\n"
		" fromdomain  = %s\n"
		" canreinvite = no\n"
		" qualify  = yes\n"
		" language = de\n"
		" host     = %s\n",
		/*id*/cont, ext_ip, localnet, user, user, passwd, domain, domain);

	if (domain != server)
		fprintf(info->output, " outboundproxy = %s\n", server);

	fprintf(info->output,
		"\n[sip_account_in_%s]\n"
		" type     = peer\n"
		" fromdomain  = %s\n"
		" host     = %s\n"
		" context  = sip-in\n"
		" language = de\n",
		id, server, server);

	free(cont);
	free(user);
}

static void sip_provider_entry1(struct info_t* info, const char* id)
{
	char *type, *server, *user, *passwd, *ext_ip, *cont;
	char localnet[1024];

	type   = get_indexed_value(info->list, "phone_accounts.type", id);
	if (!type || strcmp(type, "sip"))
		return;
	
	/* First check if we are really online. If not skip whole 
	 * sip account section */
	ext_ip = info->p1=ext_ip = get_ext_ip(info, id);

	if (!ext_ip || !strcmp(ext_ip, "0.0.0.0")) {
		log_msg(LOG_NOTICE, "Board is not online, so "
				"no sip accounts will be configured.");
		return;
	}

	if(!sip_parms_ok(info, id))
                return;

	server = get_indexed_value(info->list, "phone_accounts.server", id);
	user   = get_indexed_value(info->list, "phone_accounts.username", id);
	passwd = get_indexed_value(info->list, "phone_accounts.password", id);

	get_localnet(info, localnet, sizeof(localnet));

	cont = replace_char(strdup(user), '@', '_');
	
	fprintf(info->output,
		" nat=yes\n"
		" externip = %s\n"
		" localnet = %s\n"
		" register => %s:%s@%s/%s\n",
		ext_ip, localnet, user, passwd, server, cont);

	free(cont);
}

static void sip_provider(struct info_t* info)
{
	for_each(info->list, "phone_accounts.id.",
		(cb_t) sip_provider_entry1,info);
	
	for_each(info->list, "phone_accounts.id.",
		(cb_t) sip_provider_entry2,info);
}


/**
 * Device section
 */
static void sip_device_entry(struct info_t* info, const char* id)
{
	char *type, *number, *passwd, *name;

	type   = get_indexed_value(info->list, "phone_devices.type", id);
	if (!type || strcmp(type, "sip"))
		return;
	
	number = get_indexed_value(info->list, "phone_devices.number", id);
	passwd = get_indexed_value(info->list, "phone_devices.password", id);
	name   = get_indexed_value(info->list, "phone_devices.name", id);
	
	if (!name || !passwd || !*name || !*passwd)
		return;

	fprintf(info->output,
		"\n[%s]\n"
		" type=friend\n"
		" dtmfmode=info\n"
		" username=%s\n"
		" secret=%s\n"
		" callerid=\"%s\" <*%s>\n"
		" host=dynamic\n"
		" nat=yes\n"
		" canreinvite=no\n"
		" disallow=all\n"
		" allow=alaw\n"
		" allow=ulaw\n"
		" allow=gsm\n"
		" allow=speex\n"
		" allow=g729\n"
		" allow=slinear\n"
		" qualify=no\n"
		" callgroup=1\n"
		" context = intern\n",
		number, number, passwd, name, number);

}

static void sip_device(struct info_t* info)
{
	for_each(info->list, "phone_devices.id.",
		(cb_t) sip_device_entry,info);
}



/**
 * Main function
 */

int create_sip_conf(HLST* list, NVCL* nvram)
{
	struct info_t info;

	info.output = fopen(SIP_CONF, "w+");
	if (!info.output) {
		log_msg(LOG_ERR, "Error creating %s (%s)", SIP_CONF,
				strerror(errno));
		return -1;
	}

	info.list   = list;
	info.nvram  = nvram;

	sip_general(&info);
	sip_provider(&info);
	sip_device(&info);

	fclose(info.output);

	return 0;
}

