/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Roman Schmidt - Maxina GmbH, Neu-                             *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/

/*
 * $Log$
 * Revision 1.2  2006/12/05 13:54:10  roman
 * added user chanmon and enabled manager interface per default
 *
 * Revision 1.1  2006/08/07 17:59:10  roman
 * added tapi support
 */

#include <string.h>
#include <errno.h>

#include "phone_cfg_tapi.h"

/**********************************************************
 * Function for creation of asterisks manager.conf
 **********************************************************/

#define MANAGER_CONF	"/etc/asterisk/manager.conf"



/**
 * General section 
 */
static void tapi_general(struct info_t* info)
{

	/* enable manager interface per default */
	fprintf(info->output,
		"\n[general]\n"
		" enabled = yes\n"
		" port = 5038\n"
		" binaddr = 0.0.0.0\n\n");
}


/**
 * Account section 
 */
static void tapi_account(struct info_t* info)
{
	char *tapiAct;
	char *username;
	char *password;
	

	/* set account for channel monitoring */
	fprintf(info->output,
		"\n[chanmon]\n"
		" secret = chanmon\n"
		" permit = 127.0.0.1/255.255.255.255\n"
		" read = call,sys\n");
       
	tapiAct = get_list_value(info->list, "phone_tapi.isAct");
	
	if(tapiAct && !strcmp(tapiAct, "enabled"))
	{
		/* set the tapi account */
		username = get_list_value(info->list, "phone_tapi.username");
		password = get_list_value(info->list, "phone_tapi.passwd");
		
		fprintf(info->output,
			"\n[%s]\n"
			" secret = %s\n"
			" permit = 0.0.0.0/0.0.0.0\n"
			" read = system,call,log,verbose,command,agent,user\n"
			" write = system,call,log,verbose,command,agent,user\n"
			, username, password);
	}	
}


/**
 * Main function
 */

int create_tapi_conf(HLST* list)
{
	struct info_t info;

	info.output = fopen(MANAGER_CONF, "w+");
	if (!info.output) {
		log_msg(LOG_ERR, "Error creating %s (%s)", MANAGER_CONF,
				strerror(errno));
		return -1;
	}

	info.list   = list;

	tapi_general(&info);
	tapi_account(&info);

	fclose(info.output);

	return 0;
}

