/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Henry Margies <h.margies@maxina.de> - Maxina GmbH, Neu-       *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/

/*
 * $Log$
 * Revision 1.6  2006/02/27 15:18:22  henry
 * failover implemented
 *
 * Revision 1.5  2006/02/10 16:19:58  henry
 * added link for wan_dsl.ip_address.1
 *
 * Revision 1.4  2006/01/30 19:15:43  henry
 * caller id blocking added
 *
 * Revision 1.3  2006/01/24 12:56:47  henry
 * changed all to for_each now
 *
 * Revision 1.2  2006/01/02 12:41:18  henry
 * typo fixed
 *
 * Revision 1.1  2005/12/20 00:53:50  henry
 * new phone_config written in c
 *
 *
 */

#include <string.h>
#include <errno.h>

#include "phone_cfg_trigger.h"

static void trigger_accounts(struct info_t* info, const char* id)
{
	char *type, *number;

	type = get_indexed_value(info->list, "phone_accounts.type", id);

	if (type && !strcmp(type, "sip")) {
		nvram_link_indexed(info->nvram, "phone_accounts.server",  id, LINK_MODE, LINK_TARGET);
		nvram_link_indexed(info->nvram, "phone_accounts.username",  id, LINK_MODE, LINK_TARGET);
		nvram_link_indexed(info->nvram, "phone_accounts.password",  id, LINK_MODE, LINK_TARGET);
		nvram_link_indexed(info->nvram, "phone_accounts.use_stun",  id, LINK_MODE, LINK_TARGET);
		nvram_link_indexed(info->nvram, "phone_accounts.stun_server",  id, LINK_MODE, LINK_TARGET);
	}

	nvram_link_indexed(info->nvram, "phone_accounts.number",  id, LINK_MODE, LINK_TARGET);
	nvram_link_indexed(info->nvram, "phone_accounts.block_id",  id, LINK_MODE, LINK_TARGET);
	nvram_link_indexed(info->nvram, "phone_accounts.block_pre",  id, LINK_MODE, LINK_TARGET);
	nvram_link_indexed(info->nvram, "phone_accounts.timeslot_0",  id, LINK_MODE, LINK_TARGET);
	nvram_link_indexed(info->nvram, "phone_accounts.timeslot_10", id, LINK_MODE, LINK_TARGET);
	nvram_link_indexed(info->nvram, "phone_accounts.timeslot_30", id, LINK_MODE, LINK_TARGET);
	nvram_link_indexed(info->nvram, "phone_accounts.timeslot_45", id, LINK_MODE, LINK_TARGET);
	nvram_link_indexed(info->nvram, "phone_accounts.timeslot_60", id, LINK_MODE, LINK_TARGET);

	info->i1++;
}


static void trigger_devices(struct info_t* info, const char* id)
{
	char *type, *number;

	type = get_indexed_value(info->list, "phone_devices.type", id);

	if (type && !strcmp(type, "sip"))
		nvram_link_indexed(info->nvram, "phone_devices.password",  id, LINK_MODE, LINK_TARGET);

	nvram_link_indexed(info->nvram, "phone_devices.name",  id, LINK_MODE, LINK_TARGET);
	nvram_link_indexed(info->nvram, "phone_devices.number",  id, LINK_MODE, LINK_TARGET);

	info->i1++;
}


static void trigger_seq_entry(struct info_t* info, const char* id)
{
	char** list = info->l1;
	char*  name = info->p1;

	while(*list) {
		char var[1024];
		snprintf(var, sizeof(var),  "%s.%s.%s", name, *list, id);
		nvram_link(info->nvram, var, LINK_MODE, LINK_TARGET);

		list++;
	}
	info->i1++;
}

#if 0
static void trigger_seqs(struct info_t* info, const char* name, const char** entries)
{
	info->p1 = (char*)name;
	info->l1 = (char**)entries;
	info->i1 = 0;

	for_seq(info->list, name, (cb_t)trigger_seq_entry, info);
	
	log_msg(LOG_NOTICE, "Triggering %d %s", info->i1, name);
}
#endif 

/*
 * Update trigger 
 */
void  update_trigger(HLST* avp, NVCL* nvram, char* trigger)
{
	char* quickdial[] = {	"qickdial", "number", NULL };
	char* dialplan[] = {	"account", "prefix", "rule", "tohours", 
				"tominutes", "fromminutes", "period",
				"fromhours", "manipulation", 
				"manipulationModifier", "manipulationPrefix", 
				"mon", "tue", "wed", "thu", "fri", "sat", "sun",
				NULL };

	struct info_t info;

	info.list = avp;
	info.nvram= nvram;

	/* trigger quickdials */
	info.i1 = 0;
	info.p1 = "phone_quickdials";
	info.l1 = (char**)quickdial;
	for_each(info.list, "phone_quickdials.id",
		(cb_t)trigger_seq_entry, &info);
	
	log_msg(LOG_NOTICE, "Triggering %d quickdials", info.i1);
	
	/* trigger dialplan */
	info.i1 = 0;
	info.p1 = "phone_dials_config";
	info.l1 = (char**)dialplan;
	for_each(info.list, "phone_quickdials.id",
		(cb_t)trigger_seq_entry, &info);
	
	log_msg(LOG_NOTICE, "Triggering %d dial plans", info.i1);
	
	/* trigger all accounts */
	info.i1 = 0;
	for_each(info.list, "phone_accounts.id",
		(cb_t)trigger_accounts, &info);
	
	log_msg(LOG_NOTICE, "Triggering %d accounts", info.i1);
	
	/* trigger all devices */
	info.i1 = 0;
	for_each(info.list, "phone_devices.id",
		(cb_t)trigger_devices, &info);
	
	log_msg(LOG_NOTICE, "Triggering %d devices", info.i1);

	nvram_link(info.nvram, "phone_accounts.list", LINK_MODE, LINK_TARGET);
	nvram_link(info.nvram, "phone_devices.list", LINK_MODE, LINK_TARGET);
	nvram_link(info.nvram, "phone_accounts.totalElements", LINK_MODE, LINK_TARGET);
	nvram_link(info.nvram, "phone_devices.totalElements", LINK_MODE, LINK_TARGET);
	nvram_link(info.nvram, "phone_dials.standardaccount" , LINK_MODE, LINK_TARGET);
	nvram_link(info.nvram, "phone_dials.sipfailover" , LINK_MODE, LINK_TARGET);
	nvram_link(info.nvram, "phone_dials_config.totalElements" , LINK_MODE, LINK_TARGET);
	nvram_link(info.nvram, "phone_quickdials.totalElements" , LINK_MODE, LINK_TARGET);
	nvram_link(info.nvram, "wan_dsl.ip_address.1" , LINK_MODE, LINK_TARGET);

	nvram_trigger(info.nvram, LINK_TARGET, TRIGGER_MODE, trigger);
}

