/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Henry Margies <h.margies@maxina.de> - Maxina GmbH, Neu-       *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/

/*
 * $Log$
 * Revision 1.6  2006/12/15 15:50:20  henry
 * isdn bug fixed (heading 0)
 *
 * Revision 1.5  2006/07/28 11:08:25  henry
 * prilocaldialplan added, callerid on ISDN should now be the same as on analog
 *
 * Revision 1.4  2006/07/17 04:33:28  henry
 * changed caller id of analog phones
 *
 * Revision 1.3  2006/06/07 18:34:24  henry
 *  bug fix
 *
 * Revision 1.2  2006/06/07 17:33:23  henry
 * makes much more configurable
 *
 * Revision 1.1  2006/02/28 12:09:09  henry
 * .
 *
 *
 */

#include <string.h>
#include <errno.h>

#include "phone_cfg_zapata.h"

/**********************************************************
 * Function for creation of asterisks zapata.conf
 **********************************************************/

#define ZAPATA_CONF	"/etc/asterisk/zapata.conf"

static void zap_header(struct info_t* info)
{
	char* lang = get_list_value(info->list, "global.lang");

	fprintf(info->output, 
		";\n"
		"; Zapata telephony interface\n\n"
		"; Configuration file (automatically created)\n;\n\n"
		"[trunkgroups]\n\n"
		"[channels]\n\n"
		"language=%s\n"
		"\n", lang);
}


static void zap_isdn_nt(struct info_t* info)
{
	char *ec_enable, *ec_training;

	ec_enable   = get_list_value(info->list, "asterisk.ec_enable.nt");
	ec_training = get_list_value(info->list, "asterisk.ec_training.nt");

	fprintf(info->output, 
		"\n; first isdn card (NT)\n"
		"switchtype  = euroisdn\n"
		"signalling  = bri_net_ptmp\n"
		"faxdetect   = both\n"
		"callwaiting = yes\n"
		"usecallerid = yes\n"
		"callerid    = \"Intern\" <*30>\n"
		"prilocaldialplan = unknown\n"
		"pridialplan = unknown\n"
		"nationalprefix = 0\n"
		"internationalprefix = 00\n"
		"usecallingpres=yes\n"
		"threewaycalling=yes\n"
		"priindication=passthrough\n"
		"resetinterval=120\n"
		"transfer    = yes\n"
		"echocancel  = %s\n"
		"echotraining= %s\n"
		"immediate   = no\n"
		"overlapdial = yes\n"
		"group       = 2\n"
		"callgroup   = 1\n"
		"pickupgroup = 1\n"
		"context     = intern\n"
		"channel => %d-%d\n",
		ec_enable, ec_training,
		info->i1, info->i1+1);

	info->i1 += 3; /* use 3 channels */
}
		
static void zap_isdn_te(struct info_t* info)
{
	char *ec_enable, *ec_training;

	ec_enable   = get_list_value(info->list, "asterisk.ec_enable.te");
	ec_training = get_list_value(info->list, "asterisk.ec_training.te");

	fprintf(info->output, 
		"\n; second isdn card (TE)\n"
		"switchtype  = euroisdn\n"
		"signalling  = bri_cpe_ptmp\n"
		"faxdetect   = no\n"
		"callwaiting = yes\n"
		"usecallerid = yes\n"
		"callerid    = asreceived\n"
		"usecallingpres=yes\n"
		"priindication=passthrough\n"
		"resetinterval=never\n"
		"hidecallerid= no\n"
		"pridialplan = unknown\n"
		"prilocaldialplan = unknown\n"
		"nationalprefix = 0\n"
		"internationalprefix = 00\n"
		"echocancel  = %s\n"
		"echotraining= %s\n"
		"immediate   = no\n"
		"group       = 1\n"
		"context     = isdn-in\n"
		"immediate   = no\n"
		"overlapdial = no\n"
		"channel => %d-%d\n",
		ec_enable, ec_training,
		info->i1, info->i1+1);

	info->i1 += 3; /* use 3 channels */
}


static void zap_fxs_1(struct info_t* info)
{
	char *ec_enable, *ec_training, *phone2;

	ec_enable   = get_list_value(info->list, "asterisk.ec_enable.fxs1");
	ec_training = get_list_value(info->list, "asterisk.ec_training.fxs1");
	phone2 = get_list_value(info->list, "phone_devices.name.2");

	fprintf(info->output, 
		"\n; fxs's (telefons) [22]\n"
		"switchtype  = national\n"
		"signalling  = fxo_ls\n"
		"faxdetect   = no\n"
		"callwaiting = yes\n"
		"usecallerid = yes\n"
		"callerid    = \"%s\" <*22>\n"
		"threewaycalling=yes\n"
		"transfer    = yes\n"
		"rxgain      = 0.0\n"
		"txgain      = 0.0\n"
		"echocancel  = %s\n"
		"echotraining= %s\n"
		"immediate   = no\n"
		"group       = 3\n"
		"callgroup   = 1\n"
		"pickupgroup = 1\n"
		"context     = intern\n"
		"channel => %d\n",
		phone2, ec_enable, ec_training,
		info->i1++);
}

static void zap_fxs_2(struct info_t* info)
{
	char *ec_enable, *ec_training, *phone1;

	ec_enable   = get_list_value(info->list, "asterisk.ec_enable.fxs2");
	ec_training = get_list_value(info->list, "asterisk.ec_training.fxs2");
	phone1 = get_list_value(info->list, "phone_devices.name.1");

	fprintf(info->output, 
		"\n; fxs's (telefons) [21]\n"
		"switchtype  = national\n"
		"signalling  = fxo_ls\n"
		"faxdetect   = no\n"
		"callwaiting = yes\n"
		"usecallerid = yes\n"
		"callerid    = \"%s\" <*21>\n"
		"threewaycalling=yes\n"
		"transfer    = yes\n"
		"rxgain      = 0.0\n"
		"txgain      = 0.0\n"
		"echocancel  = %s\n"
		"echotraining= %s\n"
		"immediate   = no\n"
		"group       = 4\n"
		"callgroup   = 1\n"
		"pickupgroup = 1\n"
		"context     = intern\n"
		"channel => %d\n",
		phone1, ec_enable, ec_training,
		info->i1++);
}

static void zap_fxo(struct info_t* info)
{
	char *ec_enable, *ec_training;

	ec_enable   = get_list_value(info->list, "asterisk.ec_enable.fxo");
	ec_training = get_list_value(info->list, "asterisk.ec_training.fxo");

	fprintf(info->output, 
		"\n; fxo's (line)\n"
		"switchtype  = national\n"
		"signalling  = fxs_ls\n"
		"callwaiting = yes\n"
		"usecallerid = yes\n"
		"callerid    = asreceived\n"
		"rxgain      = 0.0\n"
		"txgain      = 0.0\n"
		"echocancel  = %s\n"
		"echotraining= %s\n"
		"immediate   = no\n"
		"group       = 5\n"
		"context     = analog-in\n"
		"channel => %d\n",
		ec_enable, ec_training,
		info->i1++);
}

/**
 * Main function
 */

int create_zapata_conf(HLST* list)
{
	struct info_t info;

	info.output = fopen(ZAPATA_CONF, "w+");
	if (!info.output) {
		log_msg(LOG_ERR, "Error creating %s (%s)", ZAPATA_CONF,
				strerror(errno));
		return -1;
	}

	info.list   = list;
	info.i1     = 1; /* first channel */

	zap_header(&info);
	zap_isdn_nt(&info);

	if (is_true(list, "asterisk.use_te"))
		zap_isdn_te(&info);
	else if (is_true(list, "asterisk.load_te"))
		info.i1+=3;
	
	if (is_true(list, "asterisk.use_fxs"))
	{
		zap_fxs_1(&info);
		zap_fxs_2(&info);
	}
	else if (is_true(list, "asterisk.load_fxs"))
		info.i1+=3;

	if (is_true(list, "asterisk.use_fxo"))
		zap_fxo(&info);
	else if (is_true(list, "asterisk.load_fxo"))
		info.i1+=3;

	fclose(info.output);

	return 0;
}
