/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Henry Margies <h.margies@maxina.de> - Maxina GmbH, Neu-       *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/

/*
 * $Log$
 * Revision 1.1  2006/06/07 17:33:23  henry
 * makes much more configurable
 *
 *
 *
 */

#include <string.h>
#include <errno.h>

#include "phone_cfg_zaptel.h"

/**********************************************************
 * Function for creation of asterisks zaptel.conf
 **********************************************************/

#define ZAPTEL_CONF	"/etc/zaptel.conf"

static void zap_header(struct info_t* info)
{
	fprintf(info->output, 
		"defaultzone=nl\n"
		"loadzone=nl\n\n");
}


static void zap_isdn_nt(struct info_t* info)
{
	fprintf(info->output, 
		"span=1,0,3,ccs,ami\n"
		"bchan=%d-%d\n"
		"dchan=%d\n\n",
		info->i1, 
		info->i1+1,
		info->i1+2);

	info->i1 += 3;
}
		
static void zap_isdn_te(struct info_t* info)
{
	fprintf(info->output, 
		"span=2,1,3,ccs,ami\n"
		"bchan=%d-%d\n"
		"dchan=%d\n\n",
		info->i1, 
		info->i1+1,
		info->i1+2);

	info->i1 += 3;
}


static void zap_fxs(struct info_t* info)
{
	fprintf(info->output, "fxols=%d-%d\n\n",
		info->i1, info->i1+1);
	
	info->i1 += 2;
}

static void zap_fxo(struct info_t* info)
{
	fprintf(info->output, "fxsls=%d\n\n", info->i1++);
}

static void zap_unused(struct info_t* info)
{
	fprintf(info->output, "unused=%d-1023\n\n");
}

/**
 * Main function
 */

int create_zaptel_conf(HLST* list)
{
	struct info_t info;

	info.output = fopen(ZAPTEL_CONF, "w+");
	if (!info.output) {
		log_msg(LOG_ERR, "Error creating %s (%s)", ZAPTEL_CONF,
				strerror(errno));
		return -1;
	}

	info.list   = list;
	info.i1     = 1; /* first channel */

	zap_header(&info);
	zap_isdn_nt(&info);

	if (is_true(list, "asterisk.use_te"))
		zap_isdn_te(&info);
	else if (is_true(list, "asterisk.load_te"))
		info.i1+=3;
	
	if (is_true(list, "asterisk.use_fxs"))
		zap_fxs(&info);
	else if (is_true(list, "asterisk.load_fxs"))
		info.i1+=2;

	if (is_true(list, "asterisk.use_fxo"))
		zap_fxo(&info);
	else if (is_true(list, "asterisk.load_fxo"))
		info.i1+=3;

	fclose(info.output);

	return 0;
}

