/*
 * Copyright (C) 2003-2005 Maxina GmbH - Henry Margies
 *
 * $Id: phone_common.h 3661 2006-11-24 09:22:35Z roman $
 *
 * $Log$
 * Revision 1.10  2006/11/24 09:22:35  roman
 * SIP parameter verification function added
 *
 * Revision 1.9  2006/07/21 15:03:15  henry
 * support for freenet added
 *
 * Revision 1.8  2006/06/07 17:33:23  henry
 * makes much more configurable
 *
 * Revision 1.7  2006/03/28 16:34:54  henry
 * triggers are now set in nvram
 *
 * Revision 1.6  2006/03/20 11:24:55  henry
 * libnvtool added
 *
 * Revision 1.5  2006/02/28 12:08:53  henry
 * .
 *
 * Revision 1.4  2006/01/25 17:45:41  henry
 *  new trigger flags
 *
 * Revision 1.3  2006/01/15 18:45:42  henry
 * for_seq removed
 *
 * Revision 1.2  2005/12/21 21:53:35  henry
 * changed to non delayed trigger
 *
 * Revision 1.1  2005/12/20 00:53:50  henry
 * new phone_config written in c
 *
 *
 */

#ifndef __PHONE_COMMON_H
#define __PHONE_COMMON_H

#include <stdio.h>

#include <nvram/nvram-lib.h>
#include <nvtools/hlst.h>

#define LINK_TARGET	"asterisk.config_trigger"
#define LINK_MODE	"exec_ok,enable_write,delayed_run"
#define TRIGGER_MODE	"link_exec_ok,enable_write"
//#define TRIGGER_MODE	"enable_write,delayed_run"


struct info_t {
	FILE* output;
	HLST* list;
	NVCL* nvram;

	char** l1;

	char* p1;
	char* p2;
	char* p3;
	char* p4;

	int i1;
	int i2;
	int i3;
	int i4;
	int i5;
};

#ifdef NO_SYSLOG
# define log_msg(type, fmt, args...) 	printf(fmt"\n", ##args)
#else
# include <syslog.h>
# define log_msg(type, fmt, args...)	syslog(type, fmt, ##args)
#endif

void nvram_avp_destroy(void *state, void *item, char *key, unsigned klen);
HLST* create_from_nvram(NVCL* nvram);
void update_from_nvram(NVCL* nvram, HLST* avp, const char* varname);
void update_to_nvram(NVCL* nvram, HLST* avp, const char* var, const char* val);

#ifdef SET_LINKS_AND_TRIGGER
void nvram_link_indexed(NVCL* nvram, const char* name, const char* id, const char* mode, const char* target);
void nvram_unlink(NVCL* nvram, const char* name);
void nvram_link(NVCL* nvram, const char* name, const char* mode, const char* target);
void nvram_trigger(NVCL* nvram, const char* name, const char* mode, const char* target);
#else
#define nvram_link_indexed(nvram, name, id, mode, target)
#define nvram_unlink(nvram, name)
#define nvram_link(nvram, name, mode, target)
#define nvram_trigger(nvram, name, mode, target)
#endif

typedef void (*cb_t)(void*, char*);
void for_each(HLST* avp, const char* prefix, cb_t cb, void* data);
//void for_seq(HLST* avp, const char* prefix, cb_t cb, void* data);
char* get_indexed_value(HLST* avp, const char* name, const char* index);
char* get_list_value(HLST* avp, const char* name);
int id_is_in(const char* s, const char* id);
int is_true(HLST* avp, const char* name);
int sip_parms_ok(struct info_t* info, const char* id);


char* replace_char(const char* string, char find, char replace);
#endif

