/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Henry Margies <h.margies@maxina.de> - Maxina GmbH, Neu-       *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/

/*
 * $Log$
 * Revision 1.10  2006/11/07 12:50:13  susi
 * Create an extra trigger program for tapi, because asterisk needs to be restarted for tapi
 * deactivation.
 *
 * Revision 1.9  2006/10/24 15:54:16  henry
 * dsp_gsm codec support added
 *
 * Revision 1.8  2006/08/07 17:59:10  roman
 * added tapi support
 *
 * Revision 1.7  2006/06/07 17:33:23  henry
 * makes much more configurable
 *
 * Revision 1.6  2006/05/30 14:43:50  henry
 * sipproxy support added
 *
 * Revision 1.5  2006/04/18 08:15:33  henry
 * path for asterisk added
 *
 * Revision 1.4  2006/02/28 12:08:53  henry
 * .
 *
 * Revision 1.3  2006/01/15 18:46:13  henry
 * sleep removed
 *
 * Revision 1.2  2005/12/21 21:53:35  henry
 * changed to non delayed trigger
 *
 * Revision 1.1  2005/12/20 00:53:50  henry
 * new phone_config written in c
 *
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "phone_common.h"
#include "phone_cfg_extension.h"
#include "phone_cfg_zapata.h"
#include "phone_cfg_zaptel.h"
#include "phone_cfg_sip.h"
#include "phone_cfg_trigger.h"
#include "phone_cfg_tapi.h"
#include "phone_cfg_modules.h"


int main(int argc, char** argv)
{
	NVCL *nvram;
	HLST* avp;

//	sleep(1);
	
#ifndef NO_SYSLOG
	openlog("phone_config", 0, LOG_DAEMON);
#endif

	/* connect to nvram */
	nvram = nvcl_open(0,0,0,0);
	if (!nvram) {
		/* report error */
#ifndef NO_SYSLOG
		closelog();
#endif
		return -1;
	}

	avp = create_from_nvram(nvram);
	if (!avp) {
		/* report error */
		nvcl_close(nvram);
#ifndef NO_SYSLOG
		closelog();
#endif
		return -2;
	}

	/* create zaptel.conf */
	create_zaptel_conf(avp);

	/* create zapata.conf */
	create_zapata_conf(avp);

	/* create extension.conf */
	create_extension_conf(avp);

	/* create sip.conf */
	create_sip_conf(avp, nvram);

	/* create manager.conf */
	//create_tapi_conf(avp);

	/* create modules.conf */
	create_modules_conf(avp);

	/* recreate host file */
	system("sh /usr/share/board_cfg/phone/phone_sipproxy.sh");

	/* restart/reload asterisk */
	system("/usr/sbin/asterisk -rx reload");

	/* update trigger */
	update_trigger(avp, nvram, *argv);

	

	nvcl_close(nvram);
	destroy_hlst(avp);

#ifndef NO_SYSLOG
	closelog();
#endif

	return 0;
}
