#! /bin/sh
#-------------------------------------------------------------------------#
#   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 #
#                                                                         #
#   This program is free software; you can redistribute it and/or modify  #
#   it under the terms of the GNU General Public License as published by  #
#   the Free Software Foundation; either version 2 of the License, or     #
#   (at your option) any later version.                                   #
#                                                                         #
#   This program is distributed in the hope that it will be useful,       #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#   GNU General Public License for more details.                          #
#                                                                         #
#   You should have received a copy of the GNU General Public License     #
#   along with this program; if not, write to the                         #
#   Free Software Foundation, Inc.,                                       #
#   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
#                                                                         #
#-------------------------------------------------------------------------#

update_sip_phone_status()
{
  d=$1
    
  number="$(nvram phone_devices.number.${d})"
    
  asterisk -rx "sip show peer ${number}" 2>/dev/null | while read name col value; do
    
    case $name in
      "Useragent")
        nvram phone_status.devices.agent.${d} "$value"
        ;;
        
      "Addr->IP")
        nvram phone_status.devices.addr.${d} "$value"
        ;;

    esac
  done
}

update_phone_status()
{
  list="$(nvram phone_devices.list)"
  for d in $list; do
    
    type="$(nvram phone_devices.type.${d})"

    case $type in 
      sip)
        update_sip_phone_status $d
        ;;

      isdn)
        # todo
        ;;

      analog)
        # not possible to check if phone is connected
        ;;

    esac
    
  done
}

update_sip_account_status()
{
  asterisk -rx "sip show registry" 2>/dev/null | while read host user refresh state; do
    
    [ "${host}" = "Host" ] && continue
 
    list="$(nvram phone_accounts.list)"
    for d in $list; do

      type="$(nvram phone_accounts.type.${d})"
      [ "$type" = "sip" ] || continue
      
      username="$(nvram phone_accounts.username.${d})"
      [ "$user" = "$username" ] || continue
    
      # fix port at 5060, change that somehow
      server="$(nvram phone_accounts.server.${d})"
      [ "${host}" = "$server:5060" ] || continue
    
      nvram phone_status.accounts.status.${d} "$state" 

    done
  done
}

update_account_status()
{
  list="$(nvram phone_accounts.list)"
  for d in $list; do
    
    type="$(nvram phone_accounts.type.${d})"

    case $type in 
      isdn)
        # todo
        nvram phone_status.accounts.status.${d} "Configured"
        ;;

      analog)
        nvram phone_status.accounts.status.${d} "`cat /proc/si3050/status`"
        ;;

    esac
    
  done
}

clean_account_status()
{
  for id in $(nvram phone_accounts.list)
  do
    nvram phone_status.accounts.status.${id} "Offline"
  done
}


logger "Updating status of phone devices and accounts"
clean_account_status
update_phone_status
update_account_status
update_sip_account_status 

