#! /bin/sh
#-------------------------------------------------------------------------#
#   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 #
#                                                                         #
#   Author: susi                                                          #
#                                                                         #
#   This program is free software; you can redistribute it and/or modify  #
#   it under the terms of the GNU General Public License as published by  #
#   the Free Software Foundation; either version 2 of the License, or     #
#   (at your option) any later version.                                   #
#                                                                         #
#   This program is distributed in the hope that it will be useful,       #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#   GNU General Public License for more details.                          #
#                                                                         #
#   You should have received a copy of the GNU General Public License     #
#   along with this program; if not, write to the                         #
#   Free Software Foundation, Inc.,                                       #
#   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
#                                                                         #
#-------------------------------------------------------------------------#

selfip=$(nvram net.ipAddr)
nvargs="-S $selfip -P $(nvram net.nvPort)"
prvmac=$(nvram sys_firmware.prvmac)
remcmd="ssh -l root -i /root/.ssh/id_rsa.db -T $(nvram net.gateway)"
prefix=$(dirname $0)/..

# --------------------------------------------------------------------------
# some loggging ...
# --------------------------------------------------------------------------

# set -x
# exec 2>>/tmp/initialise.log
ping -c 1 $(nvram net.dsl_board) >&2 || exit 7

# --------------------------------------------------------------------------
# helpers
# --------------------------------------------------------------------------

trap "echo INTERRUPT; exit 1;" 1 2 3 15

# --------------------------------------------------------------------------
# iptables clean up
# --------------------------------------------------------------------------

nvram_iptable_prepare () {

  cat - <<%
#! /bin/sh
# $(date)

{  iptables -t    nat -N bpack
   iptables -t    nat -N bpack-horst
   iptables -t    nat -N bpack-voice
   iptables -t    nat -N bpack-ssh
   iptables -t    nat -N bpack-admin
   iptables -t    nat -N bpack-smbd
   iptables -t    nat -N bpack-nmbd
   iptables -t    nat -N bpack-printserv
   iptables -t    nat -N bpack-tapi
   iptables -t    nat -N vserver
   iptables -t    nat -N dsl
   iptables -t    nat -N lan
   iptables -t filter -N ofilter
   iptables -t filter -N admin
#    iptables -t mangle -N macacl
   iptables -t mangle -N fw_ext
   iptables -t mangle -N fw_all
} 2>/dev/null

%
}

# --------------------------------------------------------------------------
# chains ...
# --------------------------------------------------------------------------

nvram_iptable_chains () {
  cat - <<%

## MANGLE

iptables -t mangle -F PREROUTING
iptables -t mangle -A PREROUTING -s $selfip -m mac --mac-source $prvmac -j ACCEPT
iptables -t mangle -A PREROUTING -i ppp0                                -j fw_ext
iptables -t mangle -A PREROUTING                                        -j fw_all
# iptables -t mangle -A PREROUTING -i wlan0                               -j macacl

# iptables -t mangle -F INPUT
# iptables -t mangle -F FORWARD
iptables -t mangle -F OUTPUT
# iptables -t mangle -F POSTROUTING


## NAT

iptables -t nat -F PREROUTING
iptables -t nat -A PREROUTING -i br0   -j bpack
iptables -t nat -A PREROUTING -i wlan0 -j bpack

# for voice
for pvc in \$(nvram $nvargs net.voice-pvc)
do
  dev="\$(grep ppp /var/run/ppp-nas\$pvc.pid 2>/dev/null)"
  [ -z \${dev} ] || iptables -t nat -A PREROUTING -i \$dev -j bpack-voice
done

# for ssh 
for pvc in \$(nvram $nvargs net.ssh-pvc)
do
  dev="\$(grep ppp /var/run/ppp-nas\$pvc.pid 2>/dev/null)"
  [ -z \${dev} ] || iptables -t nat -A PREROUTING -i \$dev -j bpack-ssh
done

# for vserver 
for pvc in \$(nvram $nvargs net.vserver-pvc)
do
  dev="\$(grep ppp /var/run/ppp-nas\$pvc.pid 2>/dev/null)"
  [ -z \${dev} ] || iptables -t nat -A PREROUTING -i \$dev -j vserver
done

iptables -t nat -F POSTROUTING
iptables -t nat -A POSTROUTING -o br0   -j lan
iptables -t nat -A POSTROUTING -o wlan0 -j lan
iptables -t nat -A POSTROUTING          -j dsl


## FILTER

iptables -t filter -F INPUT
iptables -t filter -A INPUT -i lo             -j ACCEPT
iptables -t filter -A INPUT -i br0 -s $selfip -j ACCEPT
iptables -t filter -A INPUT                   -j admin

# iptables -t filter -F OUTPUT

iptables -t filter -F FORWARD
iptables -t filter -A FORWARD -p tcp --tcp-flags SYN,RST SYN -j TCPMSS --clamp-mss-to-pmtu
# iptables -t filter -A FORWARD -m state --state ESTABLISHED,RELATED -j ACCEPT

# for ofilter 
for pvc in \$(nvram $nvargs net.ofilter-pvc)
do
  dev="\$(grep ppp /var/run/ppp-nas\$pvc.pid 2>/dev/null)"
  [ -z \${dev} ] || iptables -t filter -A FORWARD -o \$dev -j ofilter
done
%
}

nvram_lock_arp () {
  cat - <<%
#! /bin/sh
# $(date)

ip n a $selfip lladdr $prvmac nud perm dev br0 2>/dev/null
ip n r $selfip lladdr $prvmac nud perm dev br0
%
}

# --------------------------------------------------------------------------
# MAIN
# --------------------------------------------------------------------------

# force some default settings
wpost -x -p fwoff

umask 227

run=/var/run/startup.sh
vop=/var/tmp/voip_fw_url

$remcmd "cat >$run;/bin/sh $run;rm -f $run $vop" <<batch
#! /bin/sh

umask 227

rm -f    /var/tmp/nvram_lock_arp
cat - >> /var/tmp/nvram_lock_arp <<\\EOF
$(nvram_lock_arp)
EOF

rm -f    /var/tmp/nvram_iptable_vserver
cat - >> /var/tmp/nvram_iptable_vserver <<\\EOF
$(/bin/sh $prefix/wan/virtual-config.sh --view)
EOF

rm -f    /var/tmp/nvram_iptable_ofilter
cat - >> /var/tmp/nvram_iptable_ofilter <<\\EOF
$(/bin/sh $prefix/wan/ofilter-config.sh --view)
EOF

rm -f    /var/tmp/nvram_policy_route
cat - >> /var/tmp/nvram_policy_route <<\\EOF
$(/bin/sh $prefix/lan/route-config.sh --view)
EOF

rm -f    /var/tmp/nvram_iptable_admin
cat - >> /var/tmp/nvram_iptable_admin <<\\EOF
$(/bin/sh $prefix/lan/debug-config.sh --view)
EOF

rm -f    /var/tmp/nvram_iptable_bpack
cat - >> /var/tmp/nvram_iptable_bpack <<\\EOF
$(/bin/sh $prefix/lan/neighbour-config.sh --view)
EOF

rm -f    /var/tmp/nvram_iptable_dsl
cat - >> /var/tmp/nvram_iptable_dsl <<\\EOF
$(/bin/sh $prefix/wan/dsl-config.sh --view)
EOF

rm -f    /var/tmp/nvram_iptable_wlanacl
cat - >> /var/tmp/nvram_iptable_wlanacl <<\\EOF
$(/bin/sh $prefix/lan/wlan-acl.sh --view)
EOF

rm -f    /var/tmp/nvram_iptable_fwconfig
cat - >> /var/tmp/nvram_iptable_fwconfig <<\\EOF
$(/bin/sh $prefix/wan/fw-config.sh --view)
EOF

rm -f    /var/tmp/nvram_wanip_changed
cat - >> /var/tmp/nvram_wanip_changed <<\\EOF
$(/bin/sh $prefix/lan/wanip_changed.sh)
EOF

rm -f    /var/tmp/nvram_tr069_vfw_update
cat - >> /var/tmp/nvram_tr069_vfw_update <<\\EOF
$(/bin/sh $prefix/sys/tr069_update.sh)
EOF

rm -f    /var/tmp/nvram_iptable_rule 
cat - >> /var/tmp/nvram_iptable_rule <<\\EOF
$(nvram_iptable_prepare)
$(nvram_iptable_chains)

. /var/tmp/nvram_lock_arp
. /var/tmp/nvram_iptable_vserver
. /var/tmp/nvram_iptable_ofilter
. /var/tmp/nvram_iptable_fwconfig
. /var/tmp/nvram_iptable_dsl
. /var/tmp/nvram_iptable_admin
. /var/tmp/nvram_iptable_bpack
. /var/tmp/nvram_iptable_wlanacl
. /var/tmp/nvram_policy_route

EOF

chmod u+x /var/tmp/nvram_iptable_rule
/var/tmp/nvram_iptable_rule

batch

: END - OK

# --------------------------------------------------------------------------
# End
# --------------------------------------------------------------------------
