#! /bin/sh
#-------------------------------------------------------------------------#
#   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 #
#                                                                         #
#   This program is free software; you can redistribute it and/or modify  #
#   it under the terms of the GNU General Public License as published by  #
#   the Free Software Foundation; either version 2 of the License, or     #
#   (at your option) any later version.                                   #
#                                                                         #
#   This program is distributed in the hope that it will be useful,       #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#   GNU General Public License for more details.                          #
#                                                                         #
#   You should have received a copy of the GNU General Public License     #
#   along with this program; if not, write to the                         #
#   Free Software Foundation, Inc.,                                       #
#   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
#                                                                         #
#-------------------------------------------------------------------------#

test_mtd7()
{
	mount | grep "/dev/mtdblock/7 on /opt type squashfs" > /dev/null && return 0

	return 1
}

test_mtd8()
{
	mount | grep "/dev/mtdblock/8 on /mnt type jffs2" > /dev/null && return 0

	return 1
}

test_usbfs()
{
	mount | grep "usbfs on /proc/bus/usb type usbfs" > /dev/null && return 0

	return 1
}

test_module_loaded()
{
	lsmod | grep $1 > /dev/null && return 0
	return 1
}

test_asterisk_running()
{
	/usr/sbin/asterisk -rx "help" > /dev/null 2>&1 && return 0
	return 1
}

test_mhttpd_running()
{
	mhttp help > /dev/null 2>&1 && return 0
	return 1
}

try_mounting()
{
	dev=$1
	major_id=$2
	minor_id=$3
	
	ret_val=0
	mpoint="/run/mnt/$minor_id"

	mknod $dev b $major_id $minor_id
	mkdir $mpoint

	mount -tauto $dev $mpoint  > /dev/null 2>&1 || ret_val=1 
	umount $mpoint > /dev/null 2>&1 || ret_val=1 
	
	rm -rf $mpoint
	rm -rf $dev
	
	return $ret_val
}

check()
{
	desc=$1
	shift

	if $*; then
		echo "$desc: passed"
	else
		echo "$desc: failed"
	fi
}

is_mounted()
{
	while read subdev rest
	do
		[ "$1" = "$subdev" ] && return 0
	done < /proc/self/mounts

	return 1
}

test_usb_partition()
{
	[ -r /proc/partitions ] || return 1
	
	while read major minor x name; do
	
	  [ -z "$major" ] && continue
	  [ "$major" != "8" ] && continue

	  dev="/dev/usb/$name"
	  if is_mounted $dev || try_mounting $dev $major $minor; then
	  	echo "try_${name}: passed"
		usb_test_successfull=yes
	  else
	  	echo "try_${name}: unable to mount"
	  fi;
		
	  something_tested=yes

	done < /proc/partitions 

	[ -z "$usb_test_successfull" ] &&  echo "USB test failed"
	[ -z "$something_tested" ] || return 0

	echo "No USB partitions found (test failed or no USB device attached)"
}

# basic system test

logger "Self test started"

(
if [ "$(nvram sys_hardware.use_sec_flash)" = "true" ];then 
  check "mtd7_mounted" test_mtd7 
  check "mtd8_mounted" test_mtd8 
fi

check "ixp425_hss_loaded" test_module_loaded 'ixp425[-_]hss'
check "ixp425_spi_loaded" test_module_loaded 'ixp425[-_]spi'
check "zaptel_loaded" test_module_loaded 'zaptel'
check "zaphfc_loaded" test_module_loaded 'zaphfc'
check "si3210_loaded" test_module_loaded 'si3210'
if [ "$(nvram asterisk.load_fxo)" = "true" ];then 
  check "si3050_loaded" test_module_loaded 'si3050'
fi
check "board_reset_loaded" test_module_loaded 'board[-_]reset'

check "asterisk_running" test_asterisk_running
check "mhttpd_running" test_mhttpd_running

check "usbfs_mounted" test_usbfs
test_usb_partition

if [ "$(nvram sys_hardware.has_dsp)" = "true" ];then
check "dsp_working" dsp_test -t
fi

) | nvram -f - sys_test.selftest.result 

nvram sys_test.selftest.result 
