#! /bin/sh
#-------------------------------------------------------------------------#
#   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 #
#                                                                         #
#   This program is free software; you can redistribute it and/or modify  #
#   it under the terms of the GNU General Public License as published by  #
#   the Free Software Foundation; either version 2 of the License, or     #
#   (at your option) any later version.                                   #
#                                                                         #
#   This program is distributed in the hope that it will be useful,       #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#   GNU General Public License for more details.                          #
#                                                                         #
#   You should have received a copy of the GNU General Public License     #
#   along with this program; if not, write to the                         #
#   Free Software Foundation, Inc.,                                       #
#   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
#                                                                         #
#-------------------------------------------------------------------------#

# ---------------------------------------------------------------------------
# Prepare PVC list
# ---------------------------------------------------------------------------

idlist=
wan_dsl_list () {

   [ "$idlist" ] || {
     local item
     for item in $(nvram wan_dsl.list)
     do
	idlist="$idlist $(nvram wan_dsl.id.$item)"
     done
   }
   echo $idlist
}

# ---------------------------------------------------------------------------
# chain ..
# ---------------------------------------------------------------------------

nvram_iptable_dsl () {
  cat <<!
#! /bin/sh
# $(date)

iptables -t nat -F dsl
!

  local nvargs="-S $(nvram net.ipAddr) -P $(nvram net.nvPort)"
  echo connected= assigned=

  local more= id
  for id in $(wan_dsl_list)
  do
     local file="/var/run/ppp-nas$(expr $id - 1).pid"

     cat <<!

odev="\$(grep ppp ${file} 2>/dev/null)"
dev="\$(ip a s dev \$odev 2>/dev/null)"
test "\$dev" && connected=${more:+\$connected}:$id
ip=\$(echo "\$dev"|grep inet|{ read a b c;echo \$b; })
test "\$ip" && assigned=${more:+\$assigned}:$id
nvram $nvargs wan_dsl.ip_address.$id \${ip:-0.0.0.0}
!
     case $(nvram wan_dsl.nat.$id) in ena*)
       echo
       echo "[ -z \$odev ] || iptables -t nat -A dsl -o \$odev -j MASQUERADE"
     esac

     more=ok
   done

   cat <<!

case \$assigned in
:1*) nvram $nvargs wan_dsl.carrier_state "Connected" ;;
*)   case \$connected in
     :*) nvram $nvargs wan_dsl.carrier_state "Negotiating IP Address" ;;
     *)  nvram $nvargs wan_dsl.carrier_state "Not connected"
     esac
esac
!
}

# ---------------------------------------------------------------------------
# MAIN
# ---------------------------------------------------------------------------

case $1 in
-V|--view) nvram_iptable_dsl ; exit 0 ;;
?*)        echo "Usage: $(basename $0) [-V|--view]" >&2 ; exit 2
esac

logger "Configure DSL..."

## set -x

rcmd -sw contest

wpost -x -p dslmodulation
sleep 2

for id in $(wan_dsl_list)
do
   ID=$(expr $id - 1)
   wpost -x -p dsl -i 0x$ID$id
done

rcmd -s run save-now

rcmd -s template dsl-rules "\
cat - >/var/tmp/nvram_iptable_dsl <<EOF"

nvram_iptable_dsl |
while read line
do
   line=`echo "$line"|sed 's/\\\\/\\\\\\\\/g;s/'\''/\\\\'\''/g;s/"/\\\\"/g;s/\\$/\\\\\\\\$/g;'`
   rcmd -s +template dsl-rules "$line"
done

rcmd -s +template dsl-rules "\
EOF
echo OK"

rcmd -s      run dsl-rules
rcmd -s template dsl-rules "" # clean up
rcmd -s run save-now

# ---------------------------------------------------------------------------
# end
# ---------------------------------------------------------------------------
