#! /bin/sh
#-------------------------------------------------------------------------#
#   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 #
#                                                                         #
#   This program is free software; you can redistribute it and/or modify  #
#   it under the terms of the GNU General Public License as published by  #
#   the Free Software Foundation; either version 2 of the License, or     #
#   (at your option) any later version.                                   #
#                                                                         #
#   This program is distributed in the hope that it will be useful,       #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#   GNU General Public License for more details.                          #
#                                                                         #
#   You should have received a copy of the GNU General Public License     #
#   along with this program; if not, write to the                         #
#   Free Software Foundation, Inc.,                                       #
#   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
#                                                                         #
#-------------------------------------------------------------------------#

nv_pfx=wan_firewall.has_

# ---------------------------------------------------------------------------
# chain ..
# ---------------------------------------------------------------------------

has () {

    case $(nvram $nv_pfx$1) in [tTeE]*) return 0 ; esac

    return 1
}

nvram_iptable_fwconfig () {
  cat <<!
#! /bin/sh
# $(date)
iptables -t mangle -F fw_ext
iptables -t mangle -F fw_all
!

  local IPT_EXT="iptables -t mangle -A fw_ext "
  local IPT_ALL="iptables -t mangle -A fw_all "
  local NETCNF=/proc/sys/net/ipv4

  local syncookies=0 redirects=1
  has dosprotection && {
      has synfloodingcheck && syncookies=1
      has icmpredirectioncheck && {
          redirects=0
          echo "$IPT_EXT -p icmp --icmp-type 5 -j DROP"
      }
  }

  echo echo $syncookies "> $NETCNF/tcp_syncookies"
  echo echo 0           "> $NETCNF/conf/all/accept_redirects"
  echo echo $redirects  "> $NETCNF/conf/all/secure_redirects"
  
  has portscanprotection && {
      echo "$IPT_EXT -m psd -j DROP"
      
      has finurgpsh && # FIXME: should rather send back RST
        echo "$IPT_ALL -p tcp --tcp-flags FIN,URG,PSH FIN,URG,PSH -j DROP"

      has xmastreeattack &&
        echo "$IPT_ALL -p tcp --tcp-flags ALL ALL -j DROP"
        
      has nullscanattack &&
        echo "$IPT_ALL -p tcp --tcp-flags ALL NONE -j DROP"

      has synrstattack   &&
        echo "$IPT_ALL -p tcp --tcp-flags SYN,RST SYN,RST -j DROP"

      has synfinattack   &&
        echo "$IPT_ALL -p tcp --tcp-flags SYN,FIN SYN,FIN -j DROP"
  }
  
  has blockping   && {
    echo "$IPT_EXT -p icmp --icmp-type 8 -j DROP"
    echo "$IPT_EXT -p icmp --icmp-type 0 -j DROP"
  }

  has blocktelnet &&
    echo "$IPT_EXT -p tcp --dport 23 -j DROP"
  
  has blockftp    && {
    echo "$IPT_EXT -p tcp -m multiport --dports 20,21 -j DROP"
    echo "$IPT_EXT -p udp "           "--dport  69    -j DROP"
  }

  has blockdns    && {
    echo "$IPT_EXT -p tcp -m multiport --dports 42,53,101,105,202 -j DROP"
    echo "$IPT_EXT -p udp "           "--dport     53             -j DROP"
  }
  
  has blockike    && {
    echo "$IPT_EXT -p tcp --dport 500 -j DROP"
    echo "$IPT_EXT -p udp --dport 500 -j DROP"
  }

  has blockrip    &&
    echo "$IPT_EXT -p tcp -m multiport --dports 2602,2603 -j DROP"

  has blockdhcp   && {
    echo "$IPT_EXT -p tcp -m multiport --dports 67,68 -j DROP"
    echo "$IPT_EXT -p udp -m multiport --dports 67,68 -j DROP"
  }

  has blockicmp   &&
    echo "$IPT_ALL -p icmp -j DROP"
}

# ---------------------------------------------------------------------------
# MAIN
# ---------------------------------------------------------------------------

case $1 in
-V|--view) nvram_iptable_fwconfig ; exit ;;
?*)        echo "Usage: $(basename $0) [-V|--view]" >&2 ; exit 2
esac

logger "Configuring FW rules ..."

rcmd -s template fw-rules "\
cat - >/var/tmp/nvram_iptable_fwconfig <<EOF"

nvram_iptable_fwconfig |
while read line
do
  rcmd -s +template fw-rules "$line"
done

rcmd -s +template fw-rules "\
EOF
echo OK"

rcmd -sw contest

rcmd -s      run fw-rules
rcmd -s template fw-rules "" # clean up
rcmd -s /bin/sh /var/tmp/nvram_iptable_fwconfig

# ---------------------------------------------------------------------------
# End
# ---------------------------------------------------------------------------
