#! /bin/sh
#-------------------------------------------------------------------------#
#   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 #
#                                                                         #
#   Author: Henry Margies <h.margies@maxina.de> - Maxina GmbH, Neu-       #
#   Isenburg, Germany							  #
#                                                                         #
#   This program is free software; you can redistribute it and/or modify  #
#   it under the terms of the GNU General Public License as published by  #
#   the Free Software Foundation; either version 2 of the License, or     #
#   (at your option) any later version.                                   #
#                                                                         #
#   This program is distributed in the hope that it will be useful,       #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#   GNU General Public License for more details.                          #
#                                                                         #
#   You should have received a copy of the GNU General Public License     #
#   along with this program; if not, write to the                         #
#   Free Software Foundation, Inc.,                                       #
#   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
#                                                                         #
#-------------------------------------------------------------------------#

# ---------------------------------------------------------------------------
# chain ..
# ---------------------------------------------------------------------------

nvram_iptable_ofilter () {

  cat <<!
#! /bin/sh
# $(date)
iptables -t filter -F ofilter
!

  local id allow= deny= match=
  for id in $(nvram wan_outbound.list 2>/dev/null)
  do
     local sip=
     case $(nvram wan_outbound.sourceAddress.$id) in specified)
	local a=$(nvram wan_outbound.sourceFrom.$id)
	local b=$(nvram wan_outbound.sourceTo.$id)
	if [ -n "$b" -a "$a" != "$b" ]
        then
	   sip="--src-range $a-$b"
           match="-m iprange"
        else
           sip="-s $a"
        fi
     esac

     local dip=
     case $(nvram wan_outbound.destinationAddress.$id) in specified)
	local a=$(nvram wan_outbound.destinationFrom.$id)
	local b=$(nvram wan_outbound.destinationTo.$id)
	if [ -n "$b" -a "$a" != "$b" ]
        then
	   sip="--dst-range $a-$b"
           match="-m iprange"
        else
           sip="-d $a"
        fi
     esac

     local sp=
     case $(nvram wan_outbound.sourcePort.$id) in specified)
	sp="--sport $(nvram wan_outbound.sourcePortFrom.$id)"
	local     b=$(nvram wan_outbound.sourcePortTo.$id)
	[ -n "$b" -a "$a" != "$b" ] && sp="$sp:$b"
     esac

     local dp=
     case $(nvram wan_outbound.destinationPort.$id) in specified)
	dp="--dport $(nvram wan_outbound.destinationPortFrom.$id)"
	local     b=$(nvram wan_outbound.destinationPortTo.$id)
	[ -n "$b" -a "$a" != "$b" ] && dp="$dp:$b"
     esac

     local action_any action_tcp
     case $(nvram wan_outbound.action.$id) in
     allow)
	allow=ok
	action_any="-j ACCEPT"
	action_tcp="-j ACCEPT"
	;;
     *) deny=ok
        action_any="-j REJECT --reject-with icmp-net-prohibited"
        action_tcp="-j REJECT --reject-with tcp-reset" ;;
     esac

     case $(nvram wan_outbound.protocol.$id) in
     udp)    echo iptables -t filter -A ofilter -p udp $match $sip $dip $sp $dp $action_any ;;
     tcp)    echo iptables -t filter -A ofilter -p tcp $match $sip $dip $sp $dp $action_tcp ;;
     udptcp) echo iptables -t filter -A ofilter -p udp $match $sip $dip $sp $dp $action_any
             echo iptables -t filter -A ofilter -p tcp $match $sip $dip $sp $dp $action_tcp ;;
     *)      echo iptables -t filter -A ofilter        $match $sip $dip         $action_any ;;
     esac
  done

  [ -z "$allow" ] || echo iptables -t filter -A ofilter -j DROP
}

# ---------------------------------------------------------------------------
# MAIN
# ---------------------------------------------------------------------------

case $1 in
-V|--view) nvram_iptable_ofilter ; exit ;;
?*)        echo "Usage: $(basename $0) [-V|--view]" >&2 ; exit 2
esac

logger "Configure Outfilters ..."

rcmd -s template ofilter-rules "\
cat - >/var/tmp/nvram_iptable_ofilter <<EOF"

nvram_iptable_ofilter |
while read line
do
  rcmd -s +template ofilter-rules "$line"
done

rcmd -s +template ofilter-rules "\
EOF
echo OK"

rcmd -sw contest

rcmd -s      run ofilter-rules
rcmd -s template ofilter-rules "" # clean up
rcmd -s /bin/sh /var/tmp/nvram_iptable_ofilter

# ---------------------------------------------------------------------------
# End
# ---------------------------------------------------------------------------
