/***************************************************************************
 *   Copyright (C) 2005 by Henry Margies <h.margies@maxina.de>		   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <linux/module.h>
#include <linux/kernel.h> 
#include <linux/version.h> 
#include <linux/init.h>
#include <linux/proc_fs.h>
#include <linux/sched.h>
#include <linux/timer.h>
#include <linux/wait.h>

#include <asm/arch/irqs.h>

#if KERNEL_VERSION(2,6,0) > LINUX_VERSION_CODE
# include <linux/wrapper.h>
# include <asm/arch/gpio.h>
# define IXP425_RESET_GPIO	IXP425_GPIO_PIN_8
# define IXP425_RESET_IRQ	IRQ_IXP425_GPIO8
# define IRQ_HANDLED
# define irqreturn_t		void
# define SA_TRIGGER_FALLING	0
# define SA_TRIGGER_RISING	0
#else
# include <linux/interrupt.h>
# define IXP425_RESET_GPIO		8
# define MOD_INC_USE_COUNT
# define MOD_DEC_USE_COUNT
# define IXP425_GPIO_IN			IXP4XX_GPIO_IN
# define IXP425_GPIO_TRANSITIONAL	0 /* not implemented? */
# define IXP425_RESET_IRQ		IRQ_IXP4XX_GPIO8
#endif /* KERNEL 2.6 */


#define PROC_FILE_NAME		"board-reset"
#define PROC_FILE_NAME_FAST	"board-reset-immediate"

#define __unused __attribute__((__unused__))

static DECLARE_WAIT_QUEUE_HEAD(wait_q);
static struct timer_list rst_timer;
static int state = 0;

#ifdef PROC_FILE_NAME_FAST
static DECLARE_WAIT_QUEUE_HEAD(wait_q_button);
static int button_state=0;
#endif

static int board_reset_read(char *page, char **start __unused,
			    off_t off __unused, int count __unused,
			    int *eof __unused, void *data __unused)
{
	int ret = 0;
	MOD_INC_USE_COUNT;
	
	if (!state)
		interruptible_sleep_on(&wait_q);
		
	if (state)
		ret = sprintf(page, "RESET");

	MOD_DEC_USE_COUNT;
	return ret;
}

#ifdef PROC_FILE_NAME_FAST
static int board_button_read(char *page, char **start __unused,
			    off_t off __unused, int count __unused,
			    int *eof __unused, void *data __unused)
{
	int ret = 0;
	MOD_INC_USE_COUNT;
	
	if (!button_state)
		interruptible_sleep_on(&wait_q_button);
		
	if (button_state)
		ret = sprintf(page, "PRESSED");

	MOD_DEC_USE_COUNT;
	return ret;
}
#endif

static void board_reset_timer(unsigned long data __unused) 
{
	state = 1;
	wake_up_interruptible(&wait_q);
}

static irqreturn_t board_reset_isr(	int irq __unused,
				void *data __unused,
				struct pt_regs* regs __unused)
{
	int r;
	gpio_line_get(IXP425_RESET_GPIO, &r);

	button_state=!r;

	if (!r) {
		init_timer(&rst_timer);
		rst_timer.data = 0;
		rst_timer.expires = jiffies+1000;
		rst_timer.function = board_reset_timer;
		add_timer(&rst_timer);
		wake_up_interruptible(&wait_q_button);
		return IRQ_HANDLED;
	}

	del_timer(&rst_timer);
	return IRQ_HANDLED;

}

static int __init board_reset_init(void)
{
	int ret;

	gpio_line_config(IXP425_RESET_GPIO, IXP425_GPIO_IN | 
					    IXP425_GPIO_TRANSITIONAL);

	ret = request_irq(IXP425_RESET_IRQ, board_reset_isr,
                                SA_TRIGGER_FALLING | SA_TRIGGER_RISING,
				"board_reset_isr", NULL);
	if (ret) {
		/* error */
	}

	create_proc_read_entry(PROC_FILE_NAME, 0444, NULL, board_reset_read, NULL);
#ifdef PROC_FILE_NAME_FAST
	create_proc_read_entry(PROC_FILE_NAME_FAST, 0444, NULL, board_button_read, NULL);
#endif


	printk("Board reset manager loaded\n");
	
	return 0;
}


static void __exit board_reset_exit(void)
{
	free_irq(IXP425_RESET_IRQ, NULL);
	remove_proc_entry(PROC_FILE_NAME, NULL);
	del_timer(&rst_timer);
	printk("Board reset manager unloaded\n");
}


module_init(board_reset_init);
module_exit(board_reset_exit);

MODULE_AUTHOR("Henry Margies");
MODULE_DESCRIPTION("Driver for reset button");
MODULE_LICENSE("GPL");
