/*
 * BiST: A telefonie hardware testing application
 *
 * Written by  Sven Kolberg <s.kolberg@maxina.de>
 *
 * Copyright (C) 2001, Linux Support Services, Inc.
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <zaptel.h>
#include <pthread.h>
#include <sys/ioctl.h>
#include <ctype.h>
#include <stdint.h>
#include <unistd.h>
#include <sys/fcntl.h>
#include <sys/time.h>
#include <time.h>

#include <tonezone.h>

//#include <zap.h>

#include "Analog_FXO.h"

#ifdef DEBUG
	#define PDEBUG(frmt, ...) \
		{\
		fprintf(stdout, "file: %s function: %s line: %i \n",__FILE__, __PRETTY_FUNCTION__, __LINE__); \
		fprintf(stdout, frmt, ##__VA_ARGS__);\
		fprintf(stdout, "\n");\
		}
#else 
	#define PDEBUG(frmt, ...)
#endif

#ifdef DEBUG
/* "Overload" the open and close commands for debugging reasons (call-monitoring) */
	//#define ioctl(__fd, __request, __p ) myioctl(__fd, __request, __p, __FUNCTION__, __FILE__, __LINE__)
	#define open(__pathname, __flags) myopen(__pathname, __flags, __FUNCTION__, __FILE__, __LINE__)
	//#define open(__pathname, __flags, __mode) myopenmode(__pathname, __flags, __mode, __FUNCTION__, __FILE__, __LINE__)
	#define close(__fd) myclose(__fd, __FUNCTION__, __FILE__, __LINE__)
#endif


#define RING_DISTANCE 10  	/* The maximum time in seconds allowed between to rings withhin a ringing sequence*/

/**
 * \brief Will be called, when the FXO detects an ringing on the line
 * We can answer the call or ignore it
 * \params achan the Data associated with the FXO
 * \returns ?
 */
static void fxo_handle_call(Analog_chan * achan)
{

	struct timeval tv;

	/* Get the Time */
	gettimeofday(&tv, NULL);
	
	PDEBUG("distance %d",(tv.tv_sec - achan->lastring.tv_sec)); 
	
	/* Calculate the time between the last and the current call */
	if(RING_DISTANCE < (tv.tv_sec - achan->lastring.tv_sec)) 
		/* if this is the first ring of the ringtone, zero the number of rings */
		achan->numberofrings = 0;
	else
		achan->numberofrings++;

	/*remeber the time of the last ring */
	achan->lastring = tv;
	
	PDEBUG("Number of rings: %i", achan->numberofrings);

	/*decide if we should go off hook*/
	
	/* Callbackroutine, to notify the switching-logic */
	if(achan->ringing) 
	{
		/* Depending on the return value of the callback-function we have to
		 * do: nothing, accept the call, reject the call (rejecting is not really possible
		 * so take care!)
		 */
		if(achan->ringing(achan,achan->numberofrings) > 0) 
		{
			/* Accept the call */
			fxo_answer_call(achan);
		}
		else
		{
			/* do nothing, take the decision on a further RINGing */
		}
	}
}

/**
 * \brief Handles the Event checking on the FXS
 * \params a pointer to the file handler
 */
static void *fxo_handle_channel(void *data)
{
	/* The File descriptor of the Zaptel device */
	Analog_chan *achan = (Analog_chan *)data;
	int res;

	PDEBUG("File descriptor %d\n", achan->fd);
	while(1) 
	{
		/* check for events */
		res = zaptel_check_event(achan->fd);
		
		
		switch(res) {
			case ZT_EVENT_ONHOOK:  /* hung up */
				PDEBUG("ZAP_EVENT_ONHOOK");
				/* Callbackroutine, to notify the switching-logic*/
				if(achan->call_hangup) achan->call_hangup(achan);
				break;
			case ZT_EVENT_RINGBEGIN:
				PDEBUG("ZT_EVENT_RINGBEGIN, warning may be tringerd more than once!!!");
				break;
			case ZT_EVENT_RINGOFFHOOK:	  /* there is somebody ringing */
				PDEBUG("ZT_EVENT_RINGOFFHOOK");
				/* calculate how long it is ringing and if we shall go off hook*/
				fxo_handle_call(achan);
				break;
			case ZT_EVENT_WINKFLASH:  /* got wink */
			    	PDEBUG("ZAP_EVENT_WINKFLASH");
				break;
			case ZT_EVENT_ALARM:  /* got alarm */
			    	PDEBUG("ZAP_EVENT_ALARM");
				break;
			case ZT_EVENT_NOALARM: /* alarm went away */
				PDEBUG("ZAP_EVENT_NOALARM");
				break;
			case ZT_EVENT_ABORT: /* got HDLC abort */
			    	PDEBUG("ZAP_EVENT_ABORT");
				break;
			case ZT_EVENT_OVERRUN: /* got HDLC overrun */
			    	PDEBUG("ZAP_EVENT_OVERRUN");
				break;
			case ZT_EVENT_BADFCS: /* got HDLC bad FCS */
			    	PDEBUG("ZAP_EVENT_BADFCS");
				break;
			case ZT_EVENT_DIALCOMPLETE: /* dial complete */
			    	PDEBUG("ZAP_EVENT_DIALCOMPLETE");
				/* OK, We are done with dailing and making some noise */
				tone_zone_play_tone(achan->fd, ZT_TONE_RINGTONE);
				/*notify the core-logic, that the dailing is complete */
				if(achan->call_answered) achan->call_answered(achan);
				break;
			case ZT_EVENT_RINGERON: /* ringer on */
			    	PDEBUG("ZAP_EVENT_RINGERON");
				break;
			case ZT_EVENT_RINGEROFF: /* ringer off */
			    	PDEBUG("ZAP_EVENT_RINGEROFF");
				break;
			case ZT_EVENT_HOOKCOMPLETE: /* hook change complete */
			    	PDEBUG("ZAP_EVENT_HOOKCOMPLETE");
				break;
			case ZT_EVENT_NONE:
				break;
			default:  /* otherwise, ignore it */
				//PDEBUG("other")
				fprintf(stdout, "Unknown Event %d\n", res);
				break;
		}
	}
}

/**
 * \brief initialize a FXO channel and start a thread which checks continously for events
 * \params channel the zaptel channel which should be used
 * \returns a pointer to the associated structure
 */
Analog_chan *init_fxo(int channel)
{
	Analog_chan *achan;
	ZT_PARAMS p;
	ZT_BUFFERINFO bi;
	ZT_GAINS gains;
	ZT_SPANINFO spaninfo;
	
	int state;
	int tonezone;
	pthread_t thread;

	/*Get Memory for achan*/
	achan = malloc(sizeof(Analog_chan));
			
	/* at first open the device */
	achan->fd = zaptel_open(channel);
	
	if (achan->fd==-1) {
		PDEBUG("unable to open zapteldevice %i", channel);
		return NULL;
	}


	
	/* get Params */
	if(ioctl(achan->fd, ZT_GET_PARAMS, &p)) {
		PDEBUG("unable to get Parameters");
	}

	/* manipulate Params */
	//Not needed 
	

	/*set Params */
	if(ioctl(achan->fd, ZT_SET_PARAMS, &p)) {
		PDEBUG("unable to Set Parameters");
	}


	/* get Buffinfo */
	if(ioctl(achan->fd, ZT_GET_BUFINFO, &bi)) {
		PDEBUG("unable to Get Buffinfo");
	}

	/* manipulate Buffinfo */
	bi.numbufs = 4;

	/* set Buffinfo */
	if(ioctl(achan->fd, ZT_SET_BUFINFO, &bi)) {
		PDEBUG("unable to Set Buffinfo");
	}

	/* Get Gain */
	gains.chan = channel;
	if(ioctl(achan->fd, ZT_GETGAINS, &gains)) {
		PDEBUG("unabel to Get Gains");
	}
	
	/* Adjust Gain */
	//not needed linear is OK

	/*set Gain*/
	gains.chan = channel;
	if(ioctl(achan->fd, ZT_GETGAINS, &gains)) {
		PDEBUG("unable to set gains");
	}
	

	/*Set Hook*/
	state = ZT_ONHOOK;
	if(ioctl(achan->fd, ZT_HOOK, &state)) {
		PDEBUG("unable to set the Linestatus to ONHOOK");
	}

	
	/* Set the Tonezone */
	tonezone = -1;
	if (ioctl(achan->fd, ZT_SETTONEZONE, &tonezone)) {
		PDEBUG("unable to set tonezone");
	}

	/* Get Span Stats */
	if(ioctl(achan->fd, ZT_SPANSTAT, &spaninfo)) {
		PDEBUG("unable to set spaninfo");
	}

	/* Start Thread with the Start function fxs_handle_channel an pass the File handler onwards */
	if (pthread_create(&thread, NULL, fxo_handle_channel, achan)) 
	{
		PDEBUG("Unable to create Thread for fxs_handle_channel");
		return NULL;

	} 
	else 
	{
		sprintf(achan->description, "FXO(%i)", channel);
		return achan; /* The File descriptor to the caller */
	}
}

/**
 * \brief places a call over the fxo using the provided number
 * \params achan the channel to be used
 * \params number the number to be dialed as a '\0' terminated string
 * \returns 0 on succes, 1 if we are already offhook, -1 on error
 */
int fxo_call_number(Analog_chan * achan, char *number)
{
	/* check if we are already offhook */
	{
		ZT_PARAMS p;
		
		/* get Params */
		if(ioctl(achan->fd, ZT_GET_PARAMS, &p)) 
		{
			PDEBUG("unable to get Parameters");
			return -1;
		}
	
		if (p.rxisoffhook)
		{ 
			PDEBUG ("we are already offhook, so we will go on hook ");
			/*Set Hook*/
			int state = ZT_ONHOOK;
			if(ioctl(achan->fd, ZT_HOOK, &state)) 
			{
				PDEBUG("unable to set the Linestatus to ONHOOK");
				return -1;
			}
			sleep(2);

		}
	}

	/* go offhook */
	{
		/*Set Hook*/
		int state = ZT_OFFHOOK;
		if(ioctl(achan->fd, ZT_HOOK, &state)) 
		{
			PDEBUG("unable to set the Linestatus to OFFHOOK");
			return -1;
		}
	}

	sleep (1);
	/* dail a number */
	{
		ZT_DIAL_OPERATION dop;
		dop.op = ZT_DIAL_OP_REPLACE;
		strcpy(dop.dialstr, number);
		if(ioctl(achan->fd, ZT_DIAL, &dop))
		{
			PDEBUG("unable to Dial number");
			return -1;
		}
	}
#if 0
	/* wait untill dialing comlete */
	{
		int i;
		
		for(;;)
		{
			i = zaptel_wait_event(achan->fd);
			if (i<0) return -1; /* Error detected, abort*/
			if( i == ZT_EVENT_DIALCOMPLETE) break; /*this is what we are waiting for */
		}
	}
	PDEBUG("Dialing complete, playing Tone");

	/* play free tone */
	tone_zone_play_tone(achan->fd, ZT_TONE_RINGTONE);
#endif
	return 0;
}

/**
 * \brief Answers a call (after a ringing for instance)
 * \params achan the channel to be answered
 */
void fxo_answer_call(Analog_chan * achan)
{
	int i;
	/*Go Offhook */
	i = ZT_OFFHOOK;
	ioctl(achan->fd,ZT_HOOK,&i);

	PDEBUG("Go Offhook");

	/*and play a tone*/
	tone_zone_play_tone(achan->fd, ZT_TONE_DIALTONE);
	PDEBUG("Play dialtone");
}

/**
 *  * Go OnHook
 *   * \params achan the channel to hook
 *    */
int fxo_send_onhook(Analog_chan *achan)
{
        /*Set Hook*/
        int state = ZT_OFFHOOK;
        if(ioctl(achan->fd, ZT_HOOK, &state))
        {
                PDEBUG("unable to set the Linestatus to ONHOOK");
                return -1;
	}
        return 0;
}

