/*
 * BiST: A telefonie hardware testing application
 *
 * Written by  Sven Kolberg <s.kolberg@maxina.de>
 *
 * Copyright (C) 2001, Linux Support Services, Inc.
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <zaptel.h>
#include <pthread.h>
#include <sys/ioctl.h>
#include <ctype.h>
#include <stdint.h>
#include <unistd.h>
#include <sys/fcntl.h>

//#include <zap.h>

#include "Analog_Func.h"

#ifdef DEBUG
	#define PDEBUG(frmt, ...) \
		{\
		fprintf(stdout, "file: %s function: %s line: %i \n",__FILE__, __PRETTY_FUNCTION__, __LINE__); \
		fprintf(stdout, frmt, ##__VA_ARGS__);\
		fprintf(stdout, "\n");\
		}
#else 
	#define PDEBUG(frmt, ...)
#endif

#undef open
#undef ioctl
#undef close

void dump_ZT_PARAMS(FILE* fd, char *m,ZT_PARAMS *p)
{
	fprintf(fd, "%s channo: %d\n",m,p->channo);
	fprintf(fd, "%s spanno: %d\n",m, p->spanno);
	fprintf(fd, "%s chanpos: %d\n",m,	p->chanpos);
	fprintf(fd, "%s sigtype: %d\n",m,p->sigtype);
	fprintf(fd, "%s sigcap: %d\n",m, p->sigcap);
	fprintf(fd, "%s rxisoffhook: %d\n",m,p->rxisoffhook);
	fprintf(fd, "%s rxbits: %d\n",m,p->rxbits);
	fprintf(fd, "%s txbits: %d\n",m,p->txbits);
	fprintf(fd, "%s txhooksig: %d\n",m,p->txhooksig);
	fprintf(fd, "%s rxhooksig: %d\n",m,p->rxhooksig);
	fprintf(fd, "%s curlaw: %d\n",m,p->curlaw);
	fprintf(fd, "%s idlebits: %d\n",m,p->idlebits);
	fprintf(fd, "%s name[40]: %s\n",m,p->name);
	fprintf(fd, "%s prewinktime: %d\n",m,p->prewinktime);
	fprintf(fd, "%s preflashtime: %d\n",m,p->preflashtime);
	fprintf(fd, "%s winktime: %d\n",m,p->winktime);
	fprintf(fd, "%s flashtime: %d\n",m,p->flashtime);
	fprintf(fd, "%s starttime: %d\n",m,p->starttime);
	fprintf(fd, "%s rxwinktime: %d\n",m,p->rxwinktime);
	fprintf(fd, "%s rxflashtime: %d\n",m,p->rxflashtime);
	fprintf(fd, "%s debouncetime: %d\n",m,p->debouncetime);
	fprintf(fd, "%s pulsebreaktime: %d\n",m,p->pulsebreaktime);
	fprintf(fd, "%s pulsemaketime: %d\n",m,p->pulsemaketime);
	fprintf(fd, "%s pulseaftertime: %d\n",m,p->pulseaftertime);
}

void dump_ZT_BUFFERINFO(FILE* fd, char * m, ZT_BUFFERINFO *p)
{
 	fprintf(fd, "%s txbufpolicy %d\n", m, p->txbufpolicy);
 	fprintf(fd, "%s rxbufpolicy %d\n", m, p->rxbufpolicy);
 	fprintf(fd, "%s numbufs	%d\n", m, p->numbufs);
 	fprintf(fd, "%s bufsize	%d\n", m, p->bufsize);
 	fprintf(fd, "%s readbufs %d\n", m, p->readbufs);
 	fprintf(fd, "%s writebufs %d\n", m, p->writebufs);
}

void dump_ZT_GAINS(FILE* fd, char *m, ZT_GAINS *p)
{
	int i;
	
	fprintf(fd, "%s chan %d\n", m, p->chan);
	for(i = 0;i<256;i++)
		fprintf(fd, "%s\t rxgain[%i]= %i\t\t txgain[%i]= %i\n", m, i, p->rxgain[i], i, p->txgain[i]);
}

void dump_ZT_SPANINFO(FILE* fd, char *m, ZT_SPANINFO *p)
{
 	fprintf(fd, "%s  spanno: %i\n",m , p->spanno);	
 	fprintf(fd, "%s  name[20]: %s\n",m , p->name);
 	fprintf(fd, "%s  desc[40]: %s\n",m , p->desc);
 	fprintf(fd, "%s  alarms: %i\n",m , p->alarms);
 	fprintf(fd, "%s  txlevel: %i\n",m , p->txlevel);
 	fprintf(fd, "%s  rxlevel: %i\n",m , p->rxlevel);
 	fprintf(fd, "%s  bpvcount: %i\n",m , p->bpvcount);
 	fprintf(fd, "%s  crc4count: %i\n",m , p->crc4count);
 	fprintf(fd, "%s  ebitcount: %i\n",m , p->ebitcount);
 	fprintf(fd, "%s  fascount: %i\n",m , p->fascount);
 	fprintf(fd, "%s  irqmisses: %i\n",m , p->irqmisses);
 	fprintf(fd, "%s  syncsrc: %i\n",m , p->syncsrc);
 	fprintf(fd, "%s  numchans: %i\n",m , p->numchans);
 	fprintf(fd, "%s  totalchans: %i\n",m , p->totalchans);
 	fprintf(fd, "%s  totalspans: %i\n",m , p->totalspans);
}

void dump_ZT_CONFINFO(FILE* fd, char *m, ZT_CONFINFO *p)
{
  	fprintf(fd, "%s chan: %i\n",m ,p->chan);
  	fprintf(fd, "%s confno: %i\n",m , p->confno);
  	fprintf(fd, "%s confmode: %i\n",m , p->confmode);
}

int  myioctl (int __fd, unsigned long int __request, void *p, char *func, char *file, int line)
{
	int rt = ioctl(__fd, __request, p);
	fprintf(stderr, "FILE: %s:%d , Function %s\t ioctl(FD: 0x%08x, request: %lu) return: 0x%08x\n",file, line, func, __fd, __request, rt);

	switch (__request)
	{
		case ZT_SPECIFY: 
			{
				int *channel;
				channel = (int *)p;
				/*p is a point to an int, specifying a channel*/
				fprintf(stderr, "\t ZT_SPECIFY channel %d\n", *channel);
			}
			break;

		case ZT_SET_BLOCKSIZE:
			{
				int *bs = (int *)p;
				/*p is a a pointer to an int, specifying the blocksize */
				fprintf(stderr,"\t ZT_SET_BLOCKSIZE %d\n", *bs);
			}
			break;
		case ZT_GET_PARAMS:
		case ZT_SET_PARAMS:
			{
				if(__request == ZT_GET_PARAMS) {
					fprintf(stderr,"\t ZT_GET_PARAMS ZT_PARAMS\n");
				} else {
					fprintf(stderr,"\t ZT_SET_PARAMS ZT_PARAMS\n");
				}
				ZT_PARAMS *par = (ZT_PARAMS *) p;
				/*p is a pointer to a ZT_PARAMS structure, the ioctl returns the values from the zaptel device */
				dump_ZT_PARAMS(stderr, "\t\t", par);
			}
			break;
				
		case ZT_AUDIOMODE:
			{
				int *mode;
				mode = (int *)p;
				/*p is a point to an int, specifying the audio mode*/
				fprintf(stderr, "\t ZT_AUDIOMODE mode %d\n", *mode);
			}
			break;


		case ZT_GET_BUFINFO:
		case ZT_SET_BUFINFO:
			{
				ZT_BUFFERINFO *bi = (ZT_BUFFERINFO *)p;
				/*p is a pointer to a Bufferinfo structure*/
				if(__request==ZT_GET_BUFINFO) {
					fprintf(stderr, "\tZT_GET_BUFINFO  ZT_BUFFERINFO\n");
				} else {
					fprintf(stderr, "\tZT_SET_BUFINFO  ZT_BUFFERINFO\n");
				}
				dump_ZT_BUFFERINFO(stderr, "\t\t", bi);
			}
			break;

		case ZT_GETGAINS:
		case ZT_SETGAINS:
			{
			//	ZT_GAINS *gain = (ZT_GAINS *)p;
				
				if(__request == ZT_SETGAINS) {
					fprintf(stderr, "\tZT_SETGAINS  ZT_GAINS\n");
				} else {
					fprintf(stderr, "\tZT_GETGAINS  ZT_GAINS\n");
				}

			//	dump_ZT_GAINS(stderr, "\t\t", gain);	
			}
			break;
			
		case ZT_SETTONEZONE:
			{
				int *tonezone = (int *)p;
				fprintf(stderr, "\tZT_SETTONEZONE  tonezone: %i\n", *tonezone);
			}
			break;
		case ZT_SPANSTAT:
			{
				ZT_SPANINFO *info = (ZT_SPANINFO *) p;
				fprintf(stderr, "\tZT_SPANSTAT ZT_SPANINFO\n");
				dump_ZT_SPANINFO(stderr, "\t\t", info);
			}
			break;
		case ZT_HOOK:
			{
				int *stat = (int *)p;
				fprintf(stderr, "\tZT_HOOK state: %i\n", *stat);
			}
			break;
		case ZT_GETEVENT:
			{
				int *event = (int *)p;
				fprintf(stderr, "\tZT_GETEVENT event: %i\n", *event);
			}
			break;
		case ZT_SETLINEAR:
			{
				int *linear = (int *)p;
				fprintf(stderr, "\tZT_SETLINEAR linear: %i\n", *linear);
			}
			break;
		case ZT_TONEDETECT:
			{
				int *tone = (int *)p;
				fprintf(stderr, "\tZT_TONEDETECT tone: %i\n", *tone);
			}
			break;
		case ZT_CONFMUTE:
			{
				int *mute = (int *)p;
				fprintf(stderr, "\tZT_CONFMUTE mute: %i\n", *mute);
			}
			break;
		case ZT_SETLAW:
			{
				int *law = (int *)p;
				fprintf(stderr, "\tZT_SETLAW  law: %i\n", *law);
			}
			break;
		case ZT_SETCONF:
			{
				ZT_CONFINFO *zi = (ZT_CONFINFO *)p;
				fprintf(stderr, "\tZT_SETCONF ZT_CONFINFO\n");
				dump_ZT_CONFINFO(stderr, "\t\t",zi);
			}
			break;
		case ZT_FLUSH:
			{
				int *bit = (int *)p;
				fprintf(stderr,"\t ZT_FLUSH: %s %s %s\n",
						(*bit & ZT_FLUSH_READ)?"ZT_FLUSH_READ":"",
						(*bit & ZT_FLUSH_WRITE)?"ZT_FLUSH_WRITE":"",
						(*bit & ZT_FLUSH_EVENT)?"ZT_FLUSH_EVENT":"");
			}
			break;
		case ZT_DIAL:
			{
				ZT_DIAL_OPERATION *dop = (ZT_DIAL_OPERATION *)p;
				fprintf(stderr,"\t ZT_DIAL_OPERATION\t op: %i, dailstr: %s\n",dop->op, dop->dialstr);
			}
			break;
		case ZT_ECHOCANCEL:
			{
				int *x = (int *)p;
				fprintf(stderr,"\t ZT_ECHOCANCEL\t mode: %i\n", *x);
			}
			break;
		default:
			fprintf(stderr, "\t WARNING: unhandled ioctl\n");
			break;
	}
	return rt;
}

int myopen(const char *pathname, int flags, char *func, char *file, int line)
{
	int rt = open(pathname, flags);
	fprintf(stderr, "FILE: %s:%d , Function %s\t open(pathname: \"%s\", flags: 0x%08x) return: 0x%08x\n", file, line, func, pathname, flags, rt);
	return rt;
}

int myopenmode(const char *pathname, int flags, mode_t mode, char *func, char *file, int line)
{
	int rt = open(pathname, flags, mode);
	fprintf(stderr, "FILE: %s:%d , Function %s\t open(pathname: \"%s\", flags: 0x%08x, mode 0x%08x) return: 0x%08x\n", file, line, func, pathname, flags, mode, rt);
	return rt;
}

int myclose(int fd, char *func, char *file, int line)
{
	int rt = close(fd);
	fprintf(stderr, "FILE: %s:%d , Function %s\t close (fd: 0x%08x) return: 0x%08x\n", file, line, func, fd, rt);
	return rt;
}
/**
 * \brief Check for an Event on a given zap
 * \params fd, the Zaptel device file handler
 * \returns the Event or -1 on failure
 */
int zaptel_check_event(int fd) 
{
	int	j;

	/* get the event info */
	if (ioctl(fd,ZT_GETEVENT,&j) == -1) return(-1);

	return j;
}

/**
 * \brief Wait for an event and return it
 * \params fdm the zaptel device to query
 * \returns the Event or -1 on failure
 */
int zaptel_wait_event(int fd)
{
	int	j;

	for(;;)
	   {
		   /* set bits of interest */
		j = ZT_IOMUX_SIGEVENT;
		    /* wait for some happening */
		if (ioctl(fd,ZT_IOMUX,&j) == -1) return(-1);
		   /* again, if we dont have it */
		if (!(j & ZT_IOMUX_SIGEVENT)) continue;
		  /* get the event info */
		if (ioctl(fd,ZT_GETEVENT,&j) == -1) return(-1);
	
		return j;
	   }
}

/**
 * \brief Opens a zaptel device and does some basic configuring
 * \params filename, the file to be opened
 * \returns the filedescriptor
 */
int zaptel_open(int channel)
{
	int fd;
	
	if((fd= open("/dev/zap/channel", O_RDWR | O_NONBLOCK)))
	{
		//Open succeeded
		int bs = 160;
		
		//Specify channel
		if(ioctl(fd, ZT_SPECIFY, &channel)) {
			PDEBUG("unable to specify channel");
			close(fd);
			return -1;
		}

		//Set Blocksize
		if (ioctl(fd, ZT_SET_BLOCKSIZE, &bs) == -1) {
			PDEBUG("unable to set Blocksize");
			close(fd);
			return -1;
		}

		//We came that far..
		return fd;
	}
	else 
	{
		//Open failed
		return -1;
	}
}

