/*
 * BiST: A telefonie hardware testing application
 *
 * Written by  Sven Kolberg <s.kolberg@maxina.de>
 *
 * Copyright (C) 2001, Linux Support Services, Inc.
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */


#ifndef __ISDN_FUNC__
#define __ISDN_FUNC__

#include <libpri.h>
#include <pri_internal.h>


#define READ_SIZE 160
#define zt_close(fd) if(fd > 0) close(fd);

#define NUM_BCHANNEL 2 //ISDN Primary Rate has only 2 channels

/**
 * The Bchannel definitions
 */
typedef struct isdn_bchannel {
	int 		fds;	//The File descriptor of the Zaptel Device
	ZT_PARAMS	zt_params; //The params of the Zaptel device
	ZT_BUFFERINFO	zt_bufferinfo; //The bufferinfo of the Zapteldevice
	ZT_SPANINFO	zt_spaninfo; //The info of the span
	ZT_GAINS	zt_gain; //The gain of the channel
	int 		inuse; //0 if this channel is in use, !=0  if not
} ISDN_CHANNEL_INFO;

/**
 * The ISDN Info for a Span, there is one for any HFC-Chip
 */
typedef struct isdn_span_info {
	int 		fds; 	//The File descriptor of the Zaptel Device
	struct pri 	*pri;	//The libpri handle of the Dchannel
	int numberofrings;	//The count of rings recieved within a call
#warning The numberofrings variable is a misplaced here, but since we do not have a proper call handling it works, but it's a dirty hack anyway
	pthread_t 	thread;	//The handle of the thread handling the zaptel device
	pthread_mutex_t mutex; 	//Mutex to avoid double access at the same time on the same pri

	char description[256];	//a short description what this Structure is for, e.g. for debugging output
	
	ZT_PARAMS	zt_params; //The params of the Zaptel device
	ZT_BUFFERINFO	zt_bufferinfo; //The bufferinfo of the Zapteldevice
	ZT_SPANINFO	zt_spaninfo; //The info of the span
	/* The B-Channels */
	ISDN_CHANNEL_INFO bchannel[NUM_BCHANNEL];

	/* The Calls */
	q931_call *call;

	/* Callbackroutines */
	int (* call_answered) (struct isdn_span_info *span); //Will be called when a call is answered
	int (* call_hangup) (struct isdn_span_info *span); //Will be called when a call is terminated
	int (* ringing) (struct isdn_span_info *span, int nrings); //Will be called when it's ringing
} ISDN_SPAN_INFO;

	
ISDN_SPAN_INFO *isdn_create_span_info(void);

int mutex_init(pthread_mutex_t *pmutex);
int mutex_destroy(pthread_mutex_t *pmutex);
int isdn_lock_span(ISDN_SPAN_INFO *span);
int isdn_unlock_span(ISDN_SPAN_INFO *span);

void isdn_free_span_info(ISDN_SPAN_INFO *p);
int isdn_get_available_bchannel(ISDN_SPAN_INFO *p);
int isdigit(char c);
int zt_open(char *fn);
int set_actual_gain(int fd, int chan, float rxgain, float txgain, int law);
void isdn_message(char *s);
void isdn_err(char *s);
void dump_pri_event_ring(pri_event_ring *e);
int isdn_open_bchannel(ISDN_CHANNEL_INFO *bchannel, int channelnumber);
ISDN_SPAN_INFO *isdn_open_dchannel(ISDN_SPAN_INFO * dchannel,int channelnum, int span, int node, int switchtype);
#endif	//of #ifndef __ISDN_FUNC__
