/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Henry Margies <h.margies@maxina.de> - Maxina GmbH, Neu-       *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/

#include <stdio.h>
#include <endian.h>

/* I'm include c files, because I#m really lazy. Maybe I will change
 * this later */
#include "ring_test.c"
#include "load_test.c"
#include "regexp_test.c"
#include "align_test.c"

void do_compiler_test()
{
	char*		buffer[1024] = {0};
	char*		p;
	unsigned* 	i;

	printf("Testing for GCC alignment bug: ");

	/* point to somewhere in our memory */
	p = (char*)buffer+101;

	/* make sure it is an odd numbered address */
	if (!((unsigned)p % 2))
		p++;

	/*
	printf("p=%x\n", p);
	printf(	"%02x %02x %02x %02x %02x\n",
		*(p-1), *(p), *(p+1), *(p+2), *(p+3));
	*/
	
	i = (unsigned*)p;
	/* write a 4 byte value to the odd numbered address */
	i[0] = htonl(0x12345678);

	/*
	printf(	"%02x %02x %02x %02x %02x\n",
		*(p-1), *(p), *(p+1), *(p+2), *(p+3));
	*/
	if (*p != 0x12)
		printf("BUG FOUND\n");
	else
		printf("not found\n");
}

/* main test app code, no test code */
struct test_params {
	const char*	name;
	int		autostart;
	void 		(*func)(void);
};

struct test_params params[] = {
	{"compiler_test"	, 1, do_compiler_test},
	{"align_test"		, 0, do_align_test},
	{"ring_test"		, 0, do_ring_test},
	{"load_test"		, 0, do_load_test},
	{"regexp_test"		, 0, do_regexp_test},
};

void print_help()
{
	int i, entries, c;
	
	entries = sizeof(params) / sizeof(struct test_params);
	
	printf(	"\n"
		"Usage: test_app [test1] [test2] ...\n\n"
		"If you don't specify a test, all tests marked "
		"as auto will be startet.\n"
		"Available tests are:\n\n");

	for (i=0; i<entries; i++) {
		printf("\t%s %s\n", params[i].name,
				  (params[i].autostart) ? "[auto]":"");
	}
}


int main(int nargs, const char** pargs)
{
	int i, entries, c, found;
	
	entries = sizeof(params) / sizeof(struct test_params);
	
	if (nargs == 1) /* make standard tests */ {
		for (i=0; i<entries; i++) {
			if (params[i].autostart && params[i].func)
				params[i].func();
		}
		return 0;
	}

	for (c=1; c<nargs; c++) {

		found = 0;
		
		for (i=0; i<entries; i++) {
	
			/* test if names differ */
			if (strcmp(params[i].name, pargs[c]))
				continue;
				
			found = 1;

			/* has entry a valid function */
			if (params[i].func) {
				params[i].func();
				continue;
			}
			
			printf("Test function %s has no test function?!\n", 
					params[i].name);
		}

		if (!found) {
			print_help();
			return 0;
		}
	}
	
	return 0;
}
