#!/bin/sh


test_fs()
{
	case `uname -r` in 

	    2.6*) 
		mount -tsquashfs /dev/mtdblock/3 /mnt -o ro && return 0
		;;
	    2.4*)
		mount -tcramfs /dev/mtdblock/3 /mnt -o ro && return 0
		;;
	esac

	echo "Filesystem is corrupt... "
	echo "Just rebooting..."

	nvram -s
	reboot
	exit 0
}


give_up() 
{
	# test mounting of new filesystem
	test_fs

	echo "Restore old filesystem..."

	nvram UPDATE.ERROR_CODE $1
	nvram -d UPDATE.CONFIG_URL
	nvram -d UPDATE.FINISH_URL
	nvram -d UPDATE.DSL_FS_URL
	nvram -s
	nvram -c load 0x04
	nvram -s

	for a in 1 2 3 4; do
	  echo "Writing nvram configuration..."
	  sh /etc.ro/redconfig.normal || continue
	  break
	done

	echo "Reboot..."
	reboot
	exit 0
}

update_dsl_board()
{
	dsl_ssh="/mnt/usr/bin/ssh -l root -i /mnt/root/.ssh/id_rsa.db  $(nvram net.dsl_board)"
	new_ver="$(nvram UPDATE.DSL_VERSION)"

	wget -O dsl-image "$1" || give_up 101

	mount -tsquashfs /dev/mtdblock/3 /mnt -o ro || give_up 102

	export LD_LIBRARY_PATH=/mnt/lib

	echo "root:x:0:0:root:/mnt/root:/bin/sh" > /etc/passwd

	dsl_ver=$($dsl_ssh  "cat /etc/versions | grep VERSION=")

	if [ "$dsl_ver" != "VERSION=$new_ver" ]; then
 	  $dsl_ssh "cat - > /var/tmp/dsl-image && cd /var/tmp && upgrade -f tmp/dsl-image" < dsl-image 

	  while true
	  do
		 $dsl_ssh "echo dsl-upgrade finished" 2>/dev/null && break
		 echo "Waiting for update to finish..."
		 /mnt/bin/sleep 5
	  done
	else
	  echo "Upgrade of DSL firmware skipped, this version ($dsl_ver) is already installed"
	fi

	unset LD_LIBRARY_PATH
	umount /dev/mtdblock/3

}


ledcfg -b 12345678
cd /tmp

# 2.1 download main-fs
# try 4 times, on error try again
wget -O main-fs $(nvram UPDATE.MAIN_FS_URL) ||
wget -O main-fs $(nvram UPDATE.MAIN_FS_URL) ||
wget -O main-fs $(nvram UPDATE.MAIN_FS_URL) ||
wget -O main-fs $(nvram UPDATE.MAIN_FS_URL) ||
give_up 100

nvram UPDATE.ERROR_CODE 98
nvram -s

# 2.2 verify main-fs
# on error, start old system

# 2.3 write main-fs
echo "Copy main fs to flash..."
flashcp main-fs /dev/mtd/3 || reboot

nvram UPDATE.ERROR_CODE 97
nvram -s

# update dsl firmware
dsl_url="`nvram UPDATE.DSL_FS_URL 2>/dev/null`"
if [ -z $dsl_url ];then 
  echo "Skipping dsl upgrade..."
else
  update_dsl_board "$dsl_url"
fi

# 2.4 write redboot config to boot again main-fs and reboot
wget $(nvram UPDATE.FINISH_URL)
give_up 0
