#!/bin/sh

export PATH=$PATH:/usr/sbin:/usr/bin:/sbin:/bin

# give error to nvram and exit
error() {
	nvram UPDATE.ERROR_CODE $1
	exit $1
}

# give error to nvram and reboot
restart() {
	nvram UPDATE.ERROR_CODE $1
	nvram -s
	reboot
	exit $1
}

# stop services and unload modules
stop_services() {

	/etc/init.d/watchdog stop
	/etc/init.d/dropbear stop
	/etc/init.d/ntp stop
	/etc/init.d/snmp stop
	/etc/init.d/stunc stop
	/etc/init.d/httpd stop
	/etc/init.d/asterisk stop
	
	sleep 1
	killall -9 asterisk
	
	rmmod si3210
	rmmod si3050
	rmmod ixp425-spi
	rmmod zaphfc
	rmmod zaptel
#	rmmod ixp425-hss
	
	# turn on leds
	ledcfg -b 12345678

	
	if mount | grep /mnt >/dev/null; then
		umount /mnt
	fi

	if mount |grep /opt >/dev/null; then
		umount /opt
	fi
}


can_mount() {

	echo "Checking $1 ..."
	case `uname -r` in 

	    2.6*) 
		if mount -t${2} $1 /mnt -o ro; then
			umount /mnt
			return 0
		fi
		echo "Error, could not be mounted!"
		return 1
		;;
	esac

	return 0
}


# go to /tmp, remove possible old files and set
# old error code to 99
cd /tmp
rm -f config.txt zImage update-fs opt-fs
nvram UPDATE.ERROR_CODE 99

# 1.1 download config file
#  on error, cancel and return error
config_url="$(nvram UPDATE.CONFIG_URL)"
if [ "$config_url" != "not_in_use" ]; then
  wget -O config.txt "$config_url" || error 1
else
  echo > config.txt
fi


# 1.2 verify config file
#  on error, cancel and return error

# 1.3 read config data and store it to nvram
#  on error, cancel and return error

grep -v -e "^#" -e "^[ \t]*$" config.txt | while read line
do
	nvram $line
done

[ -z "$(nvram UPDATE.MAIN_FS_URL)" ]	&& error 3
[ -z "$(nvram UPDATE.UPDATE_FS_URL)" ] 	&& error 3
[ -z "$(nvram UPDATE.OPT_FS_URL)" ] 	&& error 3
[ -z "$(nvram UPDATE.ZIMAGE_URL)" ] 	&& error 3


# 1.4 download zImage opt-fs and update-fs
#  on error, cancel and return error
wget -O zImage    $(nvram UPDATE.ZIMAGE_URL) 	|| error 4
wget -O update-fs $(nvram UPDATE.UPDATE_FS_URL)	|| error 4
wget -O opt-fs    $(nvram UPDATE.OPT_FS_URL)	|| error 4

# 1.5 verify zImage and update-fs
#  on error, cancel and return error

# Stop services, after this step we have to reboot our board in 
# case of an error
stop_services;

# 1.6 write opt-fs and update-fs 
#  on error, cancel and return error
flashcp update-fs /dev/mtd/4 || restart 6
can_mount /dev/mtdblock/4 squashfs || restart 10

if cat /proc/mtd | grep mtd7 >/dev/null 2>&1; then
  flashcp opt-fs    /dev/mtd/7 || restart 9
  can_mount /dev/mtdblock/7 squashfs || restart 11
fi


# 1.7 write zImage
#  on error, what to do?
# XXX: We don't know the kind of error, it could be
# 	problem with erasing?
#	zImage is too big?
#	problem with writing to flash? (disabled flash?)
#	verify did not succeed? (also disabled flash?)
# So, what should we do? Maybe try again?
flashcp zImage /dev/mtd/2 || restart 7

# 1.8 change redboot config to boot update-fs
#  see 1.7
sh /etc/redconfig.update || restart 8


# 1.9 reboot
restart 0
