

symlinks:
	-ln -s ../html
	-ln -s ../ledcfg
	-ln -s ../mhttpd
	-ln -s ../nvram
	-ln -s ../small_nvram
	-ln -s ../nvram2env 
	-ln -s ../redconfig
	-ln -s ../run-service
	-ln -s ../si3210
	-ln -s ../stunc
	-ln -s ../board_config
	-ln -s ../gpio_switch
	-ln -s ../board_reset
	-ln -s ../khz_timer
	-ln -s ../zaphfc
	-ln -s ../sound_samples


$(ROOTFS):
	mkdir -p $(ROOTFS)/lib/modules

$(OPTFS):
	mkdir -p $(OPTFS)

$(MNTFS):
	mkdir -p $(MNTFS)

clean_install:
	$(RM) image/zImage image/kernel-image
	$(RM) -r $(ROOTFS)  image/$(MAIN_IMAGE)
	$(RM) -r $(UROOTFS) image/$(UPDATE_IMAGE)
	$(RM) -r $(OPTFS)   image/$(OPT_IMAGE)
	$(RM) -r $(MNTFS)   image/$(MNT_IMAGE)
	$(RM) image/$(MAIN_IMAGE).swapped
	$(RM) image/$(UPDATE_IMAGE).swapped
	$(RM) image/$(OPT_IMAGE).swapped
	$(RM) image/$(MAIN_IMAGE).squashfs
	$(RM) image/$(UPDATE_IMAGE).squashfs
	$(RM) image/$(OPT_IMAGE).squashfs

EMPTY_DIRS = proc sys tmp mnt opt usb

skel_install:
	rm -rf	$(ROOTFS)/etc		\
		$(ROOTFS)/var/lock	\
		$(ROOTFS)/var/run	\
		$(ROOTFS)/var/log	\
		$(ROOTFS)/var/spool	\
		$(ROOTFS)/var/tmp	
	for d in $(EMPTY_DIRS); do			\
		[ -d $(ROOTFS)/$$d ] && continue;	\
		mkdir -p $(ROOTFS)/$$d;			\
	done
	cp -a configs/skel/* $(ROOTFS)/
	find $(ROOTFS) -name CVS -type d | xargs rm -rf
	find $(ROOTFS) -name include -type d | xargs rm -rf
	grep -v "^#" configs/links | while read where to;	\
	do						\
		rm -f $(ROOTFS)/$$to;			\
		ln -s $$where $(ROOTFS)/$$to || exit 1;	\
	done
	chmod 1777 $(ROOTFS)/tmp $(ROOTFS)/run 
	chmod 700 $(ROOTFS)/root $(ROOTFS)/root/.ssh 
	chmod 755 $(ROOTFS)/etc.ro/hotplug.d/*
	chmod 755 $(ROOTFS)/etc.ro/init.d/*
	chmod 755 $(ROOTFS)/sbin/board-resetd
	chmod 755 $(ROOTFS)/sbin/hotplug
	chmod 755 $(ROOTFS)/sbin/update1.sh


strip_all:
	-find $(ROOTFS) -name "lib*.a" | xargs rm
	-find $(ROOTFS) -name "lib*.la" | xargs rm
	-find $(ROOTFS)/lib -name "*.la" | xargs rm
	-find $(ROOTFS)/lib -name "*.a" | xargs rm
	-find $(ROOTFS) -type f -name "*.so.*" | xargs $(CROSS_COMPILE)strip
	-find $(ROOTFS) -type f -perm -100 | xargs $(CROSS_COMPILE)strip

timestamps:
	test -d $(OPTFS) || mkdir $(OPTFS)
	echo "Filesystem was built on `date` at `hostname`" > $(ROOTFS)/etc.ro/build
	echo "Filesystem was built on `date` at `hostname`" > $(OPTFS)/.created
	echo "Filesystem was built on `date` at `hostname`" > $(MNTFS)/.created
	echo "$(TARGET_VERSION)" > $(ROOTFS)/etc.ro/version

createfs: $(ROOTFS) $(OPTFS) $(MNTFS) timestamps
	-mv $(ROOTFS)/opt/* $(OPTFS)
	PATH=/sbin:/usr/sbin:$(PATH) fakeroot sh -c \
	"chown -R root: $(ROOTFS) && sh scripts/mk.dev_nodes $(ROOTFS)/dev/ && $(MKSQUASHFS) $(ROOTFS)/* image/$(MAIN_IMAGE) -be -noappend"
	PATH=/sbin:/usr/sbin:$(PATH) fakeroot sh -c \
	"chown -R root: $(OPTFS) && $(MKSQUASHFS) $(OPTFS)/* image/$(OPT_IMAGE) -be -noappend"
	-mv $(OPTFS)/* $(ROOTFS)/opt/
	PATH=/sbin:/usr/sbin:$(PATH) fakeroot sh -c \
	"chown -R root: $(MNTFS) && $(MKJFFS2) -b -o image/$(MNT_IMAGE) -d $(MNTFS)"
	if [ "`stat --printf '%s' image/$(MAIN_IMAGE)`" -gt 13631488 ]; then	\
	  echo "Main-FS is too big!!!";					\
	  exit 1;							\
	fi
	chmod 644 image/$(MAIN_IMAGE)
	chmod 644 image/$(OPT_IMAGE)

image_distclean: clean_install small_nvram_distclean small_busybox_distclean
	$(RM) horst-image.tar.gz

package:
	echo "HorstBox Firmware package $(TARGET_VERSION)." >  image/version-info
	echo "Package was built on `date` at `hostname`"    >> image/version-info
	tar cfz horst-image-$(TARGET_VERSION).tar.gz 	image/main-fs	\
					image/opt-fs	\
					image/update-fs	\
					image/zImage	\
					image/version-info
	rm -f horst-image.tar.gz
	ln -s horst-image-$(TARGET_VERSION).tar.gz horst-image.tar.gz
	rm image/version-info
