#
# oprofile
#
#

# popt library 

$(DL_DIR)/$(POPT_FILE):
	$(WGET) -P $(DL_DIR) $(POPT_URL)

$(POPT_DIR): $(DL_DIR)/$(POPT_FILE)
	tar xfz $(DL_DIR)/$(POPT_FILE)

$(POPT_SRC)/Makefile:	$(POPT_DIR)
	cd $(POPT_DIR) && ./configure --host=armeb-linux --disable-nls --prefix=$(PWD)/toolchain

popt_build: $(POPT_SRC)/Makefile
	$(MAKE) -C $(POPT_DIR)

popt_install: popt_build
	$(MAKE) -C $(POPT_DIR) install

# libbfd


$(BFD_DIR)/Makefile: $(BINUTILS_DIR)
	cd $(BFD_DIR) && ./configure --host=armeb-linux --disable-nls --prefix=$(PWD)/toolchain

bfd_build: $(BFD_DIR)/Makefile
	$(MAKE) -C $(BFD_DIR)

bfd_install: bfd_build
	$(MAKE) -C $(BFD_DIR) install

# libiberty

$(LIBIBERTY_DIR)/Makefile: $(BINUTILS_DIR)
	cd $(LIBIBERTY_DIR) && ./configure --host=armeb-linux --disable-nls --prefix=$(PWD)/toolchain

libiberty_build: $(LIBIBERTY_DIR)/Makefile
	$(MAKE) -C $(LIBIBERTY_DIR)

# opcodes

$(OPCODES_DIR)/Makefile: $(BINUTILS_DIR)
	cd $(OPCODES_DIR) && ./configure --host=armeb-linux --disable-nls --prefix=$(PWD)/toolchain

opcodes_build: $(OPCODES_DIR)/Makefile
	$(MAKE) -C $(OPCODES_DIR)

# binutils

$(DL_DIR)/$(BINUTILS_FILE):

$(BINUTILS_DIR): $(DL_DIR)/$(BINUTILS_FILE)
	tar xfj $(DL_DIR)/$(BINUTILS_FILE)
	cd $(BINUTILS_DIR) && patch -p1 < $(ARCHIVE)/patch.bfd

$(BINUTILS_SRC)/Makefile: $(BINUTILS_DIR)
	cd $(BINUTILS_SRC) && ./configure --host=armeb-linux --disable-nls --prefix=$(ROOTFS)

binutils_build: $(BINUTILS_SRC)/Makefile bfd_build libiberty_build opcodes_build
	$(MAKE) -C $(BINUTILS_SRC) objdump

binutils_install: binutils_build
	cp $(BINUTILS_SRC)/objdump $(ROOTFS)/bin/

# oprofile 

$(DL_DIR)/$(OPROFILE_FILE):
	$(WGET) -P $(DL_DIR) $(OPROFILE_URL)

$(OPROFILE_DIR): $(DL_DIR)/$(OPROFILE_FILE)
	tar xzf $(DL_DIR)/$(OPROFILE_FILE)

$(OPROFILE_DIR)/Makefile: $(OPROFILE_DIR) popt_install bfd_install
	cd $(OPROFILE_DIR) && ./configure --host=armeb-linux --with-kernel-support --prefix=$(ROOTFS)

oprofile_build: $(OPROFILE_DIR)/Makefile
	$(MAKE) -C $(OPROFILE_DIR)


oprofile_install: binutils_install
	$(MAKE) -C $(OPROFILE_DIR) install-strip
	cp 	toolchain/lib/libpopt.* \
		toolchain/lib/libbfd.* \
		$(ROOTFS)/lib/
	test -d $(ROOTFS)/root/.oprofile || mkdir -p $(ROOTFS)/root/.oprofile
	test -d $(ROOTFS)/var/lib/oprofile || mkdir -p $(ROOTFS)/var/lib/oprofile
	test -d $(ROOTFS)/dev/oprofile || mkdir -p $(ROOTFS)/dev/oprofile
	[ -d $(ROOTFS)/etc.ro/init.d ] || mkdir -p $(ROOTFS)/etc.ro/init.d
	[ -d $(ROOTFS)/etc.ro/rc.d ] || mkdir -p $(ROOTFS)/etc.ro/rc.d
	(echo "#!/bin/sh";				\
	echo "ln -s /proc/mounts /etc/mtab";		\
	echo "mount -tramfs ramfs /root/.oprofile"; 	\
	echo "mount -tramfs ramfs /var/lib/oprofile";	\
	echo "opcontrol --init";			\
	echo "opcontrol --vmlinux=/mnt/vmlinux";	\
	echo) > $(ROOTFS)/etc.ro/init.d/oprofile
	chmod 700 $(ROOTFS)/etc.ro/init.d/oprofile
	rm -f $(ROOTFS)/etc.ro/rc.d/81_oprofile
	ln -s /etc/init.d/oprofile $(ROOTFS)/etc.ro/rc.d/81_oprofile
#	$(MAKE) oprofile_deps


OPROFILE_DEP_FILE=usr/bin/awk bin/bash usr/bin/which usr/bin/tr bin/touch bin/mktemp bin/pidof bin/objdump

oprofile_deps:
	@echo
	@echo "Checking for missing files for oprofile"
	@echo
	@echo
	@error=0;				\
	for i in $(OPROFILE_DEP_FILE); do	\
	  if [ -r $(ROOTFS)/$$i ];then		\
	    printf "%s found\n" "$$i";		\
	  else					\
	    printf "%s is missing\n" "$$i";	\
	    error=1;				\
	  fi					\
	done;					\
	echo;					\
	exit $$error

oprofile_clean:

