/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Roman Schmidt - Maxina GmbH, Neu-                             *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/

#include "chanmon.h"


/* -------------------------------------------------------------------
 * define globals
 * -------------------------------------------------------------------*/
char _username[MAX_LENGTH];
char _secret[MAX_LENGTH];
BOOL _stop_loop;


/* -------------------------------------------------------------------
 * usage method
 * -------------------------------------------------------------------*/
void usage(){
	
	printf	("\nusage: chanmon [OPTION]\n"
		 "Options:\n"
		 "-u <username>    Specifies the username for the manager interface [%s]\n"
		 "-s <secret>      Specifies the password for the manager interface [%s]\n"

		 "-h               This small help\n",
		 DEFAULT_USER, DEFAULT_SECRET);

}

int set_args(int argc, char  **argv){

 	extern char *optarg;
        extern int optind;
	int  option;
	int  ret = 1;

  	/* set defaults */
        strcpy(_username, DEFAULT_USER);
	strcpy(_secret, DEFAULT_SECRET);
	_stop_loop = FALSE;

        do {
		option = getopt(argc, argv, "hu:s:");
	        switch(option){
	        case 'u':       strncpy(_username,optarg,MAX_LENGTH);
	        		break;

                case 's':       strncpy(_secret,optarg,MAX_LENGTH);
                		break;
		
		case -1:	break;
		
		default:	usage();
				ret = 0;
				break;
		}
	}while(option > 0);

	return ret;
}

/* -------------------------------------------------------------------
 * remove leading and trailing spaces
 * -------------------------------------------------------------------*/
char* strip(char *str){

	char *ptr = str;
	char *start = str;
	
	
	/* go over leading spaces */
	while(isspace(*ptr) && *ptr)
		ptr++;
	/* overwrite with free of leading space string */
	while(*str=*ptr){
		str++;
		ptr++;
	}

	/* no characters left */
	if(str == start)
		return start;

	/* remove trailing spaces */
	while(--str != start && isspace(*str));

	/* overwrite the first of the trailing spaces with 0 */
	*(str+1) = '\0';
	
	return start;
}

/* -------------------------------------------------------------------
 * returns the value of a given name out of the message block 
 * ------------------------------------------------------------------*/
char *get_value(MESG *mesg, const char *name){
	int i;	
	char *value = NULL;

	for(i=0; i<mesg->cmd_count; i++){
	
		if(strcasecmp(mesg->command[i].name, name) == 0){
			value = mesg->command[i].value;
			break;
		}
	}	

	return value;
}


BOOL add_elem(ELEM **start, char *id){

	BOOL exists = FALSE;
	ELEM *ptr = *start;
	
	while(ptr){
		if(strcasecmp(ptr->id, id) == 0) {
			exists = TRUE;
			break;
		}

		ptr = ptr->next;
	}
	
	if(!exists){
		ELEM *elem = (ELEM*)malloc(sizeof(ELEM));
		elem->next = *start;
		elem->id = (char*)malloc(strlen(id) + 1);
		strcpy(elem->id, id);
		*start = elem;
	}

	return !exists;
}

BOOL remove_elem(ELEM **start, char *id){

	BOOL exists = FALSE;
	ELEM *ptr = *start;
	
	while(ptr){
		if(strcasecmp(ptr->id, id) == 0) {
			exists = TRUE;
			break;
		}

		ptr = ptr->next;
	}
	
	if(exists){
		ELEM *elem = *start;
		
		if(*start == ptr){
			*start = ptr->next;	
		}
		else{
			while(elem->next != ptr)
				elem = elem->next;
			
			elem->next = ptr->next;
		}
		
		free(ptr->id);
		free(ptr);
	}

	return exists;
}

int queue_size(ELEM *start){
	int count = 0;

	while(start && ++count)
		start = start->next;

	return count;
}

int daemoninit()
{
	pid_t	pid;

#if 0
	if( (pid = fork()) < 0 )
		return -1;
	else if (pid != 0)
		exit(0);
#endif
	setsid();

	chdir("/");

	umask(0);

//	openlog("printserv", LOG_PID, LOG_DAEMON);

	return(0);
}
		
int read_nextline(char *buffer, int length, int socket){

	static char data[MAX_LENGTH];
	int nread;
	int nleft;
	char *ptr;

	if((ptr = strchr(data, '\0')) == NULL)
		ptr = data;

	nleft = MAX_LENGTH - (ptr-data) - 1;
	
	while (strchr(data, '\n') == NULL){
          if ( (nread = read(socket, ptr, nleft)) < 0) {
		if (errno == EINTR)
		        continue;      	/* and call read() again */
	        else
	                return (-1);
	  }else if (nread == 0)		/* EOF */
	  	return 0;     	 

          nleft -= nread;
          ptr += nread;
	  *ptr = '\0'; 
        }

	if(length < sizeof(data))
		return -2;		/* buffer to small */
	
	strncpy(buffer, data, length);

	/* copy all data beginning after the first '\n' */
	strcpy(data, strchr(buffer, '\n') + 1); 

	/* '\0'-terminate the line after '\n' */
	*(strchr(buffer, '\n') + 1) = '\0';

	return strlen(buffer);
}

MESG *read_answer(int socket){

	char buffer[MAX_LENGTH];
	char *ptr1, *ptr2;
	int i=0;
	MESG *mesg = (MESG*)malloc(sizeof(MESG));

	memset(mesg, 0, sizeof(mesg));

	while((mesg->read = read_nextline(buffer, sizeof(buffer), socket)) > 0){
		
		if(!strcmp(buffer,"\r\n") || i >= MAX_CMD)
			break;
	
		if( (ptr1 = strtok(buffer, ":")) == NULL ||
		    (ptr2 = strtok(NULL, ":")) == NULL ){

			syslog(LOG_WARNING, "invalid message was read from manager interface");
			continue;
		}
		
		strip(ptr1);
		strip(ptr2);
		
		mesg->command[i].name 	= (char *) malloc(strlen(ptr1) + 1);	
		mesg->command[i].value 	= (char *) malloc(strlen(ptr2) + 1);	
		
		strcpy(mesg->command[i].name, ptr1);
		strcpy(mesg->command[i].value, ptr2);
		
		i++;
	}

	mesg->cmd_count = i;

	return mesg;
}


int send_mesg(int socket, MESG *mesg){

	char buffer[MAX_LENGTH];
	int i, len, res = -1;

	for (i=0; i<mesg->cmd_count; i++){

		if(mesg->command[i].name && mesg->command[i].value){
			len = snprintf(buffer, MAX_LENGTH, "%s: %s\r\n", 
				mesg->command[i].name, mesg->command[i].value);	
			
			if(len<MAX_LENGTH)	
				len = write(socket, buffer, len);
			else
				res = 1;
		}
	}

	len = snprintf(buffer, MAX_LENGTH, "\r\n");
	
	if(len<MAX_LENGTH)
		write(socket, buffer, len);
	else
		res = 1;
	
	return res;
}

void free_mesg(MESG *mesg){
	int i;
	for(i=0; i<mesg->cmd_count; i++){
		
		if(mesg->command[i].name)
			free(mesg->command[i].name);

		if(mesg->command[i].value)
			free(mesg->command[i].value);
	}
}

BOOL is_resp_success(MESG *mesg){
	BOOL ret = FALSE;
	char *val;
	
	val = get_value(mesg, "response");
	
	if(val){
		if(strcasecmp(val, "success") == 0){
			ret = TRUE;
		}
	}
	
	return ret;
}

int manager_login(int socket){

	char buffer[MAX_LENGTH];
	int res;
	MESG mesg;
	MESG *answer;

	memset(&mesg, 0, sizeof(mesg));
	
	mesg.command[0].name 	= "action";
	mesg.command[0].value 	= "login";
	mesg.command[1].name	= "username";
	mesg.command[1].value	=  _username;
	mesg.command[2].name	= "secret";
	mesg.command[2].value	= _secret;

	mesg.cmd_count = 3;

	send_mesg(socket, &mesg);

	answer = read_answer(socket);

	res = (is_resp_success(answer)) ? 0 : -1;

	free_mesg(answer);
	
	return res;
}

EVENT get_event_type(MESG *mesg){
	char *val;
	EVENT event = OTHER;

	val = get_value(mesg, "event");

	if(val){
		if(strcasecmp(val, "newchannel") == 0){
			event = NEWCHANNEL;
		}
		else if(strcasecmp(val, "hangup") == 0){
			event = HANGUP;
		}
		else if(strcasecmp(val, "shutdown") == 0){
			event = SHUTDOWN;
		}
	}

	return event;
}

void update_nvram(NVCL	*nvram, const char* var, const char* val)
{
        if (val)
		nvcl_putcmd(nvram, "set");
        else {
                nvcl_putcmd(nvram, "del");
                val = NULL;
        }

        nvcl_putavpair(nvram, var, val, 0);

	if (nvcl_rply(nvram, 0) < 1) {
		syslog(LOG_ERR, "Error update to nvram");
	}

}

void trigger_nvram(NVCL *nvram, int new_size){

	static int old_size = 0;
	
	if( old_size == 0 && new_size > 0 )
		update_nvram(nvram, "phone_active_channel.state" , "true");

	if( old_size > 0 && new_size == 0)
		update_nvram(nvram, "phone_active_channel.state" , "false");
	
	old_size = new_size;
}

int do_work(int socket){
	BOOL run = TRUE;
	MESG *mesg;
	EVENT event;
	int counter=0;
	ELEM *queue = NULL;
	NVCL *nvram;

	nvram = nvcl_open(0,0,0,0);
	
	if (!nvram) {
		syslog(LOG_ERR, "can not connect to nvram");
        	return -1;
	}

	sleep(1);

	if(manager_login(socket) != 0){
		syslog(LOG_ERR, "error during manager login");
		return -1;
	}

	while(run){
	
		mesg = read_answer(socket);
		
		if(mesg->read < 1){
			syslog(LOG_WARNING, "socket read error: %d", mesg->read);
			break;
		}

		switch(get_event_type(mesg)){

		case NEWCHANNEL:	add_elem(&queue, 
						get_value(mesg, "uniqueid")); 
					break;

		case HANGUP: 		remove_elem(&queue,
						get_value(mesg, "uniqueid"));
					break;

		case SHUTDOWN:		run = FALSE;	
					break;

		default:		break;	
		}

		
		free_mesg(mesg);
	
		trigger_nvram(nvram, queue_size(queue));
	}

	nvcl_close(nvram);
	
	return 0;
}

int main(int argc, char **argv)
{
	int sockfd, connfd;
	socklen_t len;
	struct sockaddr_in servaddr;
	pid_t	pid;
	char	buffer[MAX_LENGTH];
	int 	connect_tries = 0 ;

	if( !set_args(argc, argv) )
		exit(1);
	

	if( daemoninit() != 0){
		syslog(LOG_ERR, "error while daemoninit(): %m");
		exit(1);
	}
	

 	/* Create the TCP socket */
        if ((sockfd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0) {
		syslog(LOG_ERR,"Failed to create socket: %m");
		exit(1);
	}


	memset(&servaddr, 0, sizeof(servaddr));
	servaddr.sin_family = AF_INET;
	servaddr.sin_addr.s_addr = inet_addr(_serv_addr);
	servaddr.sin_port = htons(_tcp_port);

	while (connect(sockfd,(struct sockaddr*)&servaddr,sizeof(servaddr)) < 0){
		if( ++connect_tries > 3 )
			exit(1);
		syslog(LOG_ERR, "ERROR connecting try: %d failed", connect_tries);
		sleep(1);
	}


	do_work(sockfd);
	
	close(sockfd);
	
	return 0;
}
