/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Henry Margies <h.margies@maxina.de> - Maxina GmbH, Neu-       *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/

#include <linux/config.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/types.h>
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/smp_lock.h>
#include <linux/miscdevice.h>
#include <linux/watchdog.h>
#include <linux/slab.h>
#include <linux/ioport.h>
#include <linux/fcntl.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/system.h>
#include <linux/notifier.h>
#include <linux/reboot.h>
#include <linux/init.h>
#include <linux/delay.h>


#include <asm/arch/irqs.h>
#include <asm/arch/gpio.h>

#define IXP425GPIO_MINOR 0

/* GPIO pins used */
#define IXP425_OV      	    IXP425_GPIO_PIN_0
#define IXP425_BF_nRES      IXP425_GPIO_PIN_1
#define IXP425_SPI_SCLK     IXP425_GPIO_PIN_2
#define IXP425_SPI_SDI      IXP425_GPIO_PIN_3
#define IXP425_SPI_SDO      IXP425_GPIO_PIN_4
#define IXP425_nSPI_CS0     IXP425_GPIO_PIN_5
#define IXP425_nINTA        IXP425_GPIO_PIN_6
#define IXP425_nINTB        IXP425_GPIO_PIN_7
#define IXP425_GPIO8        IXP425_GPIO_PIN_8
#define IXP425_ISDN_PwrOn   IXP425_GPIO_PIN_9
#define IXP425_W_DOG        IXP425_GPIO_PIN_10
#define IXP425_nSLIC_INT    IXP425_GPIO_PIN_11

#define IXP425_nSLIC_RES    IXP425_GPIO_PIN_12
#define IXP425_PCI_CLK      IXP425_GPIO_PIN_13
#define IXP425_EXP_CLK      IXP425_GPIO_PIN_14


/*
 *	Kernel methods.
 */
 
#define GPIO_CMD_ISDN_ON 	1 
#define GPIO_CMD_ISDN_OFF       2
#define GPIO_CMD_BF_RESET_ON    3
#define GPIO_CMD_BF_RESET_OFF   4

//Turn debugging on
#define GPIO_SWITCHER_DEBUG

/* Use macro PDEBUG for printing debug messages */
#undef PDEBUG                           // undef it, just in case
#ifdef GPIO_SWITCHER_DEBUG
        #define PDEBUG(fmt, args...)    \
                printk("[%s:%d] "fmt, __FILE__,__LINE__, ##args)
#else
        #define PDEBUG(fmt, args...)    {}
#endif
 
#define __unused __attribute__((__unused__))

/**
 *	gpio_ioctl:
 *	\param inode: inode of the device
 *	\param file handle to the device
 *	\param watchdog command
 *	\param argument pointer
 *	\returns 0 upon success
 */
static int gpio_ioctl(struct inode *inode, struct file *file, unsigned int cmd,
	unsigned long arg)
{
	switch (cmd)
	{
		case GPIO_CMD_ISDN_ON:
			/* ISDN-Power anschalten, dazu muss ISDN_PwrOn auf Low gehen */
			 gpio_line_set(IXP425_ISDN_PwrOn,0);
			PDEBUG("ISDN-Power On\n");
			break;

		case GPIO_CMD_ISDN_OFF:
			/* ISDN-Power auschalten, dazu muss ISDN_PwrOn auf High gehen */
			 gpio_line_set(IXP425_ISDN_PwrOn,1);
			PDEBUG("ISDN-Power Off\n");
			break;

		case GPIO_CMD_BF_RESET_ON:
			/* Blackfin-Reseten, dazu muss Bf-nRes auf Low gehen */
			 gpio_line_set(IXP425_BF_nRES,0);
			PDEBUG("Blackfin-Reset On\n");
			break;

		case GPIO_CMD_BF_RESET_OFF:
			/* Blackfin-Reset ende, dazu muss Bf-nRes auf High gehen */
			 gpio_line_set(IXP425_BF_nRES,1);
			PDEBUG("Blackfin-Reset Off\n");
			break;

		default:
			PDEBUG("Unkown ioctl()\n");
			return -ENOTTY;

	}

	return 0;
}

/**
 * Main interrupt service routine handling the OV-interrupts.
 * \param irq Interrupt number.
 * \param data Points to private data, in our case to driver config struct.
 * \param regs Not important.
 */
static void gpio_switcher_isr( int irq __unused,
                        void *data,
                        struct pt_regs* regs __unused)
{
	PDEBUG("OverVoltage-Event Occured\n");
}

static struct file_operations gpio_fops = {
	owner:		THIS_MODULE,
	llseek:		NULL,
	read:		NULL,
	write:		NULL,
	ioctl:		gpio_ioctl,
	open:		NULL,
	release:	NULL,
};

static struct miscdevice gpio_miscdev={
	IXP425GPIO_MINOR,
	"ix425gpio",
	&gpio_fops
};


/**
 *	cleanup_module:
 *
 *	Deregister and drop-out
 */
 
static void __exit gpio_exit(void)
{
	misc_deregister(&gpio_miscdev);
}

/**
 * 	gpio_init:
 *
 *	Set up the GPIO-Pins for IO-Operation
 */
 
static int __init gpio_init(void)
{
	int ret;

	//Am System anmelden
	ret = misc_register(&gpio_miscdev);
	if (ret) {
		printk(KERN_ERR "gpio: can't misc_register on minor=%d\n",IXP425GPIO_MINOR);
		return 1;
	}

	//GPIOs reservieren
        gpio_line_config(IXP425_BF_nRES   , IXP425_GPIO_OUT);
        gpio_line_config(IXP425_ISDN_PwrOn, IXP425_GPIO_OUT);
	gpio_line_config(IXP425_OV	  , IXP425_GPIO_IN | 
					    IXP425_GPIO_ACTIVE_HIGH);

	gpio_line_set(IXP425_BF_nRES,0);
	mdelay(10);
	gpio_line_set(IXP425_BF_nRES,1);

	//Interrupt reservieren
//        ret = request_irq(      IRQ_IXP425_GPIO0, gpio_switcher_isr,
//                                0, "gpio-switcher_isr", NULL);
	//und zurck zum System
	return 0;
}

module_init(gpio_init);
module_exit(gpio_exit);

MODULE_AUTHOR("Sven Kolberg");
MODULE_DESCRIPTION("Driver for the GPIOs of the IXP425");
MODULE_LICENSE("GPL");
EXPORT_NO_SYMBOLS;
