#!/usr/bin/perl

use strict;
use warnings;

use Tk;
use Tk::ProgressBar;
use Tk::After;
use Tk::Dialog;
use threads;
use threads::shared;


use horstbox;

# default values
my $host  = '192.168.0.1';
my $user = 'admin';
my $port = 4321;

# text values
my $title_txt  = 'Update HorstBox';
my $update_txt = 'Update';
my $cancel_txt = 'Abbrechen';
my $ipaddr_txt = 'IP Adresse';
my $passwd_txt = 'Passwort';
my $text_txt   = 'Um die Software der HorstBox zu aktualisieren, geben Sie bitte die IP Adresse und das Passwort ein und druecken auf "Update". Waehrend des Updates NIEMALS die Horstbox ausschalten oder neu starten! Das Update kann bis zu 5 Minuten in Anspruch nehmen.';
		
my $error1_txt = 'Die angegebene Adresse kann nicht erreicht werden.';
my $error2_txt = 'Error communicating with HorstBox';
my $error3_txt = 'Update-Daemon konnte nicht gestartet werden';
my $error4_txt = 'Keine korrekte IP Adresse.';
my $error5_txt = 'Update-Datei konnte nicht gefunden werden (1).';
my $error6_txt = 'Update-Datei konnte nicht gefunden werden (2).';
my $error7_txt = 'Update-Datei konnte nicht gefunden werden (3).';
my $error8_txt = 'Update-Datei konnte nicht gefunden werden (4).';
my $error9_txt = 'Update-Datei konnte nicht gefunden werden (5).';

my $success_txt= 'Update abgeschlossen. Die Horstbox wird nun neu gestartet.';

my $image_name = 'dlink-header.gif';
my $logo_name = 'Logo.gif';

my $counter = 0;
my $timer;

# start http daemon
my $httpd_thread = threads->new(sub {&horstbox::init_http_daemon($port);});
	
# create windows
my $window  = MainWindow->new(	-title => $title_txt);
my $image   = $window->Photo( 	-file => $image_name,
				-format => 'gif');
my $header  = $window->Label( 	-image => $image);
my $text_l  = $window->Label( 	-background => '#ffffff', 
				-wraplength => 580, 
				-justify => 'left', 
				-text => $text_txt);
my $addr_l  = $window->Label( 	-background => '#ffffff', 
				-text => $ipaddr_txt);
my $addr_e  = $window->Entry( 	-background => '#ffffff');
my $pass_l  = $window->Label( 	-background => '#ffffff', 
				-text => $passwd_txt);
my $pass_e  = $window->Entry( 	-background => '#ffffff', 
				-show => '*');
my $update  = $window->Button( 	-text => $update_txt,
				-width=> 10, 
				-command => \&init_update);
my $cancel  = $window->Button(	-text => $cancel_txt,
				-width=> 10, 
				-command => sub { $window->destroy(); });
my $progr   = $window->ProgressBar( 
				-width => 20, 
				-length => 580, 
				-from => 0, 
				-to => 210, 
				-blocks => 30, 
				-background => '#ffffff', 
				-colors => [0, '#0069a2'],
				-troughcolor => '#ffffff',
				-borderwidth => 2,
				-highlightthickness => 1,
				-anchor => 'w');
my $icon   = $window->Photo(	-file => $logo_name,
				-format => 'gif');


$addr_e->insert( 0, $host);

# organise widgets
$header->pack();

$text_l->place( -x => 100, -y => 180);
$addr_l->place(	-x => 250, -y => 250);
$addr_e->place(	-x => 350, -y => 250);
$pass_l->place(	-x => 250, -y => 300);
$pass_e->place(	-x => 350, -y => 300);
$update->place(	-x => 280, -y => 350);
$cancel->place(	-x => 380, -y => 350);
$progr->place(  -x => 100, -y => 400);

# endless loop until our window lives
$window->resizable(0,0);
$window->Icon(-image => $icon);

$timer = $window->repeat(1000, \&check_daemon);
$window->waitWindow();
exit 0;
	

sub error_msg
{
	$window->Dialog(
		-title          => "Error",
		-text           => $_[0],
		-bitmap         => 'error',
		-default_button => "OK",
		-buttons	=> ['OK'],
	)->Show();
}


sub success_msg
{
	$window->Dialog(
		-title          => "Message",
		-text           => $_[0],
		-bitmap         => 'info',
		-default_button => "OK",
		-buttons	=> ['OK'],
	)->Show();
}


sub disable_controls
{
	$addr_e->configure(-state=>'disabled');
	$pass_e->configure(-state=>'disabled');
	$cancel->configure(-state=>'disabled');
	$update->configure(-state=>'disabled');
}


sub init_update
{
	my $host = $addr_e->get;

	if ($host !~ m/^\d*\.\d*\.\d*\.\d*$/) {
		&error_msg($error4_txt);
		return;
	}
	
	my $ip   = &horstbox::get_ip_address( $host);

	if ($ip eq "") {
		&error_msg($error1_txt);
		return;
	}
	
	my $res = &start_update();
	if ($res) {
		&error_msg($error2_txt);
		return;
	}

	&disable_controls();
}


sub start_update
{
	my $pass = $pass_e->get;
	my $res  = 0;
	my $host = $addr_e->get;

	# start update process
	$res = &horstbox::trigger_update2( $host, $user, $pass, $port);
	if ($res) {
		print("First contact failed\n");
		my $ip   = &horstbox::get_ip_address( $host);
		print "Using local ip $ip and port $port\n";

		$res = &horstbox::trigger_update( 	$host, 
							$user, 
							$pass, 
							$ip, $port);
		if ($res) {
			return 2;
		}
	}

	return 0;
}

sub check_daemon
{
	# Status:
	# 0 - not running 
	# 1 - just started
	# 2 - download in progress
	# 3 - last file downloaded
	# 
	my $stat = &horstbox::get_status();

	if (!$stat) {
		$timer->cancel();
		&error_msg($error3_txt);
		$window->destroy();
		return;
	}

	if ($stat == 9) {
		$timer->cancel();
		&error_msg($error5_txt);
		$window->destroy();
		return;
	}

	if ($stat == 2) {
		if ($counter == 0) {
			&disable_controls();
		}
		&update_update();
	}

	if ($stat == 3) {
		$counter=20;
		$progr->value($counter);
	
	}

	if ($stat == 4) {
		$timer->cancel();
		&success_msg($success_txt);
		$window->destroy();
	}
}

sub update_update
{
	my $host = $addr_e->get;
	my $pass = $pass_e->get;
	
	$counter++;
	$progr->value($counter);

}

