#!/usr/bin/perl

use strict;
use warnings;

use threads;
use threads::shared;


use Getopt::Long;
use horstbox;

# autoflush 
$| = 1;

# default values
my $host = '192.168.0.1';
my $user = 'admin';
my $pass = 'admin';
my $port = 4321;

my $help       = 0;
my $wo_trigger = 0;
my $sh_progress= 0;

GetOptions(	"user=s"=>\$user,
		"pass=s"=>\$pass,
		"port=i"=>\$port,
		"host=s"=>\$host,
		"help"=>\$help,
		"without-trigger"=>\$wo_trigger,
		"show-progress"=>\$sh_progress);

if ($help) {
  print("update_cli - written by Henry Margies\n");
  print("Usage:\n");
  print("        update_cli [--user username]   \\\n");
  print("                   [--pass password]   \\\n");
  print("                   [--host hostname]   \\\n");
  print("                   [--port portnumber] \\\n");
  print("                   [--without-trigger] \\\n");
  print("                   [--show-progress] 	\\\n");
  print("                   [--help]\n");
  print("\n");
  exit 0;
}

# start http daemon
my $httpd_thread = threads->new(sub {&horstbox::init_http_daemon($port);});
	
my $res  = 0;
my $stat = 0;

# start update process
if (!$wo_trigger) {
  $res = &horstbox::trigger_update2( $host, $user, $pass, $port);
  if ($res) {
    my $ip   = &horstbox::get_ip_address( $host);
    
    print("First contact failed\n");
    print "Using local ip $ip and port $port\n";

    $res = &horstbox::trigger_update( $host, $user, $pass, $ip, $port);
    if ($res) {
      print("Error while triggering the update...\n");
      exit 2;
    }
  }
} else {
  sleep(2);
}

print("Ready...\n");

my $i	     = 1;
my $z	     = 0;
my @progress = ( "|", "/", "-", "\\" );
my $progr1   = "======================================================================";
my $progr2   = "                                                                      ";
my $l	     = length($progr1);

while($i)
{

	# Status:
	# 0 - not running 
	# 1 - just started
	# 2 - download in progress
	# 3 - last file downloaded
	# 
	$stat = &horstbox::get_status();

	if (!$stat) {
		print("HTTP Daemon not running...\n");
		exit 1;
	}

	if ($stat == 9) {
		print("Firmware files where not found...\n");
		exit 1;
	}

	if ($stat == 2) {
		if ($sh_progress) {

			if (($i % 2) == 0) {
				$z++;
				if ($z > $l) {
					$z=0;
				}
			}

			print("  [", 	substr($progr1,$l-$z), 
					substr($progr2,$z), "]  [", 
					$progress[$i % 4], "]\r");
		} else {
			print("\r $progress[$i % 4]\r");
		}
		$i++;
	}

	if ($stat == 3) {
	}

	if ($stat == 4) {
		print("Update finished...\n");
		exit 0;
	}

	sleep(1);
}
