#!/bin/sh

fill()
{
	size=$(printf %d $1)
	source=$2
	dest=$3

	if [ "$size" = 0 ];then
		echo "Wrong size for $2..."
		return 1
	fi

	if [ ! -r "$source" ]; then
		echo "Can not read $source..."
		return 1
	fi

	fs=$(stat -c %s $source)
	if [ "$source" != "/dev/null" -a "$fs" = 0 ];then
		echo "Wrong file size for $2..."
		return 1
	fi
	is=$(expr $size - $fs)

	dd if=/dev/zero bs=$is count=1 2>/dev/null | cat $source - > $dest || return 1

	fs=$(stat -c %s $dest)

	if [ "$fs" != $(printf %d $size) ]; then
		echo "Wrong file size of $dest..."
		return 1;
	fi

	return 0
}


destdir=images
targetdir=flash
build_env=build_env
flash_dumps=flash-dumps

version=$(cat ${build_env}/image/rootfs/etc.ro/version 2>/dev/null|| echo unkown)

[ -d ${targetdir} ] || mkdir ${targetdir}

fill 0x80000   ${flash_dumps}/redboot.bin      	${destdir}/part1-bootloader	|| exit 1
fill 0x40000   /dev/null                       	${destdir}/part2-nvram		|| exit 1
fill 0x100000  ${build_env}/image/zImage 	${destdir}/part3-zImage		|| exit 1
fill 0xD00000  ${build_env}/image/main-fs	${destdir}/part4-main-fs	|| exit 1
fill 0x100000  ${build_env}/image/update-fs 	${destdir}/part5-update-fs	|| exit 1
fill 0x20000   ${flash_dumps}/redconfig         ${destdir}/part6-redconfig	|| exit 1
fill 0x20000   ${flash_dumps}/fisconfig         ${destdir}/part7-fisconfig	|| exit 1
fill 0x800000  ${build_env}/image/opt-fs	${destdir}/part8-opt-fs		|| exit 1
fill 0x800000  ${build_env}/image/mnt-fs	${destdir}/part9-mnt-fs		|| exit 1

cd ${destdir}
cat 	part1-bootloader\
	part2-nvram 	\
	part3-zImage	\
	part4-main-fs 	\
	part5-update-fs \
	part6-redconfig \
	part7-fisconfig > ../${targetdir}/flash-image-$(date +"%Y%m%d")

cat 	part8-opt-fs\
	part9-mnt-fs > ../${targetdir}/opt-flash-image-$(date +"%Y%m%d")

cd -

tar cfz horst-flash-${version}.tar.gz flash/*

ls -alh ${targetdir} *.tar.gz
