#ifndef __LINUX_LEDMAN_H__
#define __LINUX_LEDMAN_H__

#include <linux/ioctl.h>

#define CONFIG_LEDMAN

/*
 * ioctl cmds
 */
#define LEDMAN_IOC_MAGIC 'l'

/* turn on briefly to show activity */
#define LEDMAN_CMD_SET		_IO(LEDMAN_IOC_MAGIC, 0x01)
/* turn LED on permanently */
#define LEDMAN_CMD_ON		_IO(LEDMAN_IOC_MAGIC, 0x02)
/* turn LED off permanently */
#define LEDMAN_CMD_OFF		_IO(LEDMAN_IOC_MAGIC, 0x03)
/* flash this LED */
#define LEDMAN_CMD_FLASH	_IO(LEDMAN_IOC_MAGIC, 0x04)
/* reset LED to default behaviour */
#define LEDMAN_CMD_RESET	_IO(LEDMAN_IOC_MAGIC, 0x05)

/* LED is being used for non-std reasons */
#define LEDMAN_CMD_ALT_ON	_IO(LEDMAN_IOC_MAGIC, 0x06)
/* LED is being used for std reasons */
#define LEDMAN_CMD_ALT_OFF	_IO(LEDMAN_IOC_MAGIC, 0x07)
/* enable and init the ledman driver */
#define LEDMAN_CMD_STARTTIMER	_IO(LEDMAN_IOC_MAGIC, 0x08)
/* disable the ledman driver */
#define LEDMAN_CMD_KILLTIMER	_IO(LEDMAN_IOC_MAGIC, 0x09)


/* Blackfin related funktions */
/* make the GPIO 0 go to zero */
#define LEDMAN_CMD_BLACKFIN_RST_ON	_IO(LEDMAN_IOC_MAGIC, 0x0a)
/* make the GPIO 0 go to one */
#define LEDMAN_CMD_BLACKFIN_RST_OFF	_IO(LEDMAN_IOC_MAGIC, 0x0b)

#ifdef __KERNEL__

extern int  ledman_cmd(int cmd, unsigned long led);

#else

#include	<fcntl.h>

#define ledman_cmd(cmd, led) ({ \
	int fd; \
	if ((fd = open("/dev/ledman", O_RDWR)) != -1) { \
		ioctl(fd, cmd, led); \
		close(fd); \
	} \
})

#endif


#endif /* __LINUX_LEDMAN_H__ */
