/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Henry Margies <h.margies@maxina.de> - Maxina GmbH, Neu-       *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/

#include <linux/config.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/init.h>


#include "khz-timer.h"


/* list definition */
static LIST_HEAD(khz_timer_list);

struct khz_timer {
	
	timer_func		func;
	void*			data;
	struct list_head 	list;
};

/* prototypes */
static int  khz_timer_init(void);
static void khz_timer_exit(void);

/* register a 1khz timer */
void register_1khz_callback(timer_func func, void* data)
{
	struct khz_timer* entry;

	if (!func)
		return;

	entry = kmalloc(sizeof(struct khz_timer), GFP_KERNEL);
	entry->func = func;
	entry->data = data;

	list_add(&entry->list, &khz_timer_list);
	MOD_INC_USE_COUNT;

	printk("1KHz Timer registered\n");
}

/* unregister a 1khz timer */
void unregister_1khz_callback(timer_func func)
{
	struct list_head* tmp;
	struct khz_timer* p;
	
	list_for_each(tmp, &khz_timer_list) {
		p = list_entry(tmp, struct khz_timer, list); 

		if (p->func != func)
			continue;

		list_del(&p->list);
		kfree(p);
		MOD_DEC_USE_COUNT;
		printk("1KHz Timer unregistered\n");
		return;
	}
}


void khz_timer_1khz_ping(void)
{
	struct list_head* tmp;
	struct khz_timer* p;
	
	list_for_each(tmp, &khz_timer_list) {
		p = list_entry(tmp, struct khz_timer, list); 

		if (p->func)
			p->func(p->data);
	}
}


static int khz_timer_init(void)
{
	return 0;
}

static void khz_timer_exit(void)
{
}

MODULE_AUTHOR("Henry Margies");
MODULE_DESCRIPTION("Driver that provides 1 KHz");
MODULE_LICENSE("GPL");

module_init(khz_timer_init);
module_exit(khz_timer_exit);

