/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Henry Margies <h.margies@maxina.de> - Maxina GmbH, Neu        *
*   Isenburg, Germany                                                     *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/

#include <linux/ledman.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>

extern char *optarg;

static void set_leds(int cmd, char* leds)
{
  while(*leds) {

    if (*leds < '0' || *leds > '8') {
      fprintf(stderr, "Invalid LED %c\n", *leds);
      exit(-1);
    }

    ledman_cmd(LEDMAN_CMD_ALT_OFF, (*leds)-'0');
    ledman_cmd(cmd, (*leds)-'0');
    //		printf("cmd %d led %d\n", cmd, (*leds)-'0');
    leds++;
  }
}


static void special(void)
{
  int i = 0;

  for (;;) {
    /* all leds off */
    ledman_cmd(LEDMAN_CMD_ALT_OFF, 0);
    ledman_cmd(LEDMAN_CMD_OFF, 0);

    for (i=2; i<9; i++) {
      ledman_cmd(LEDMAN_CMD_ON, i);
      ledman_cmd(LEDMAN_CMD_OFF, i-1);
      usleep(70000);
    }

    usleep(100000);
			
    for (i=7; i>0; i--) {
      ledman_cmd(LEDMAN_CMD_OFF, i+1);
      ledman_cmd(LEDMAN_CMD_ON, i);
      usleep(70000);
    }
  }
}


static void help(void)
{
  printf(	"ledcfg - Application to set ledman leds\n"
		"GPL-Licensed - Written by Henry Margies "
		"<h.margies@maxina.de>\n\n"
		"Usage: ledcfg -n <LEDS> -o <LEDS> -b <LEDS>\n"
		"        -n Turns leds on\n"
		"        -o Turns leds off\n"
		"        -b Turn leds in blinking mode\n"
		"	 -s Special feature with leds :)\n"
		"	 -a Active Timer for leds and watchdog\n"
		"	 -r Stop Timer (reboots device if watchdog is active)\n"
		"	 -x Blackfin reset on\n"
		"	 -y Blackfin reset off\n"
		"        -h This help screen\n\n"
		" <LEDS> = [12345678]\n"
		" Examples: ledcfg -n 1357 -o 2468\n"
		"           ledcfg -n 12 -o 34 -b 56\n\n");
	
}


static void check_ledman_dev(void)
{
  int ret;

  ret = open("/dev/ledman", O_WRONLY);
  if (ret != -1) {
    close(ret);
    return;
  }

  /* device is not there, creare it */

  ret = mknod("/dev/ledman", S_IFCHR, 0x7e00);
  chmod("/dev/ledman", 0666);

  if (ret != -1) 
    return;

  fprintf(stderr, "Could not open or create /dev/ledman\n");
}

int main(int nargs, char** argv) {

  char 	params[] = "shb:o:n:arxy";
  int 	i;

  /* test if l/dev/ledman is there */
  check_ledman_dev();

  while(1) {
    i = getopt(nargs, argv, params);
    if (i == -1) 
      break;

    switch(i) {
			
    case 'n':
      set_leds(LEDMAN_CMD_ON, optarg);
      break;				

    case 'o':
      set_leds(LEDMAN_CMD_OFF, optarg);
      break;
    case 'b':
      set_leds(LEDMAN_CMD_FLASH, optarg);
      break;

    case 'h':
      help();
      break;
    case 's':
      special();
      special();
      break;
    case 'a':
      ledman_cmd(LEDMAN_CMD_STARTTIMER,0);
      break;
    case 'r':
      ledman_cmd(LEDMAN_CMD_KILLTIMER,0);
      break;
    case 'x':
      ledman_cmd(LEDMAN_CMD_BLACKFIN_RST_ON,0);
      break;
    case 'y':
      ledman_cmd(LEDMAN_CMD_BLACKFIN_RST_OFF,0);
      break;
    }
  }
  return 0;
}
