@
@ KNAIZUK Test of RAM
@
@ Written by Henry Margies
@ (inspired by head.S from arm linux kernel)
@
@ GPL
@

	@
	@ Macros for serial output
	@
		.macro	loadsp, rb
		mov	\rb, #0xc8000000
		.endm

		.macro	writeb, rb
		str	\rb, [r3, #0]
		.endm
		
		.macro	m_putc,val
		mov	r0, \val
		bl	putc
		.endm

		.macro	m_puts, addr
		adr	r0, \addr
		bl 	puts
		.endm
		
		.macro	m_phex,val,len
		mov	r0, \val
		mov	r1, #\len
		bl	phex
		.endm


	@ 
	@ Start testing :)
	@
		.align
		bl	prepare
		b	start

msg_welcome:	.ascii "\n\n\nKNAIZUK Test - written by Henry Margies\0"
msg_pc:		.ascii "\nProgramm is located at 0x\0"
msg_addr_start:	.ascii "\nStart address: 0x\0"
msg_addr_end:	.ascii "\nEnd address  : 0x\0"
		
		.align
		.type	start,#function

start:
		m_puts	msg_welcome
		m_puts	msg_pc
		adr	r7, start
		m_phex	r7, 8
		m_putc	#'\n'

		
9:	
		mov	r7, #0x1000
		mov	r8, #0x2000000

		m_puts	msg_addr_start
		m_phex	r7, 8
		m_puts	msg_addr_end
		m_phex	r8, 8
		m_putc	#'\n'

		mov	r9, #0xffffffff
		bl	knaizuk_fill
		bl	knaizuk_test

		mov	r9, #2
		bl 	knaizuk_fill2
		bl 	knaizuk_test2
		
		mov	r9, #1
		bl 	knaizuk_fill2
		bl 	knaizuk_test2
		
		mov	r9, #0
		bl 	knaizuk_fill2
		bl 	knaizuk_test2
		
		mov	r9, #0x0
		bl	knaizuk_fill
		bl	knaizuk_test

		mov	r9, #2
		bl 	knaizuk_fill3
		bl 	knaizuk_test3
		
		mov	r9, #1
		bl 	knaizuk_fill3
		bl 	knaizuk_test3

		mov	r9, #0
		bl 	knaizuk_fill3
		bl 	knaizuk_test3

		m_puts	msg_done

 		b	9b

msg_done:	.ascii "\nTest done\n\n\0"

	@ Code for printing last values on error
	@ params: address (r6), value (r5), expected (r4)
	@
msg_error:	.ascii "\n\n !!! Error occured !!! \n\n\0"
msg_addr:	.ascii "\nAddress        : 0x\0"
msg_read:	.ascii "\nValue read     : 0x\0"
msg_expt:	.ascii "\nExpected value : 0x\0"
		.align
error:
		m_puts	msg_error
		m_puts	msg_addr
		m_phex	r6, 8
		m_puts	msg_read
		m_phex	r5, 8
		m_puts	msg_expt
		m_phex	r4, 8
1:		b	1b


		.align
		.type	phexbuf,#object
phexbuf:	.space	12
		.size	phexbuf, . - phexbuf

phex:		adr	r3, phexbuf
		mov	r2, #0
		strb	r2, [r3, r1]
1:		subs	r1, r1, #1
		movmi	r0, r3
		bmi	puts
		and	r2, r0, #15
		mov	r0, r0, lsr #4
		cmp	r2, #10
		addge	r2, r2, #7
		add	r2, r2, #'0'
		strb	r2, [r3, r1]
		b	1b

puts:		loadsp	r3
1:		ldrb	r2, [r0], #1
		teq	r2, #0
		moveq	pc, lr
2:		writeb	r2
		mov	r1, #0x00020000
3:		subs	r1, r1, #1
		bne	3b
		teq	r2, #'\n'
		moveq	r2, #'\r'
		beq	2b
		teq	r0, #0
		bne	1b
		mov	pc, lr
putc:
		mov	r2, r0
		mov	r0, #0
		loadsp	r3
		b	2b

	
	@ Sets memory (in 32 byte steps)
	@ r4, r5, r6 will be changed
	@ r7 - start address (should be aligned)
	@ r8 - end address
	@ r9 - 32 bit pattern for writing
msg_fill1:	.ascii "\nFilling memory with   0x\0"

		.align
knaizuk_fill:
		mov	r10, lr
		mov	r6, r7
		mov	r4, #0
		
		m_puts	msg_fill1
		m_phex	r9, 8
		m_putc	#' '
		
1:		
		@ store double word (pattern) to address in r6
		str 	r9, [r6]

		@ print progress
		add	r4, r4, #1
		cmp	r4, #0x10000
		moveq	r0, #'.'
		moveq	r4, #0x0
		bleq	putc
		
		@ increase r6 by 4
		add	r6, r6, #4

		cmp	r6, r8
		blt	1b

		mov	pc, r10
		
	@ Checks memory
	@ r5, r6 will changed 
	@ start   - start address (should be aligned)
	@ end     - end address
	@ pattern - 32 bit pattern for comparing
msg_test1:	.ascii "\nTesting memory for    0x\0"
		
		.align
knaizuk_test:
		mov	r10, lr

		mov	r4, #0
		mov	r6, r7
		
		m_puts	msg_test1
		m_phex	r9, 8
		m_putc	#' '
1:
		@ get value and compare
		ldr	r5, [r6]
		cmp	r5, r9
		beq	2f

		@ error, address is r6, value read is r5, 
		@ wanted value r9->r4
		mov	r9, r4
		b	error

2:
		@ print progress
		add	r4, r4, #1
		cmp	r4, #0x10000
		moveq	r0, #'.'
		moveq	r4, #0x0
		bleq	putc
		
		@ increase address
		add	r6, r6, #4
		cmp	r6, r8
		blt	1b

		mov	pc, r10


	@ Sets each 3rd byte in memory to zero, rest to 0xff
	@ r4, r5, r6 will be changed
	@ r7 - start address (should be aligned)
	@ r8 - end address
	@ r9 - start of first 0x00 pattern (0-2)
msg_fill2:	.ascii "\nWriting pattern for knaizuk-1 #\0"

		.align
knaizuk_fill2:
		mov	r10, lr

		m_puts	msg_fill2
		m_phex	r9, 1
		m_putc	#' '

		mov	r6, r7
		mov	r4, #0

		mov	r5, r9
		cmp	r5, #1
		blt	2f
		beq	3f
		
1:		
		@ store 16 bit
		mov 	r5, #0x00ff
		orr	r5, r5, #0xff00
		strh	r5, [r6]
		add	r6, r6, #2
2:
		mov	r5, #0x00ff
		strh	r5, [r6]
		add	r6, r6, #2
3:
		mov	r5, #0xff00
		strh	r5, [r6]
		add	r6, r6, #2

		@ print progress
		add	r4, r4, #1
		cmp	r4, #0x10000
		moveq	r0, #'.'
		moveq	r4, #0x0
		bleq	putc
		
		cmp	r6, r8
		blt	1b

		mov	pc, r10

	@ Checks memory for pattern generated in fill2
	@ r3, r4, r5, r6 will be changed
	@ r7 - start address (should be aligned)
	@ r8 - end address
	@ r9 - start of pattern (0-2)
msg_test2:	.ascii "\nTesting knaizuk pattern-1     #\0"

		.align
knaizuk_test2:
		mov	r10, lr

		m_puts	msg_test2
		m_phex	r9, 1
		m_putc	#' '

		mov	r6, r7
		mov	r4, #0

		mov	r5, r9
		cmp	r5, #1
		blt	2f
		beq	3f
		
1:		
		@ load 16 bit
		mov 	r3, #0x00ff
		orr	r3, r3, #0xff00
		ldrh	r5, [r6]
		cmp	r5, r3
		movne 	r4, r3
		bne	4f
		add	r6, r6, #2
2:
		ldrh	r5, [r6]
		cmp	r5, #0x00ff
		movne	r4, #0x00ff
		bne	4f
		add	r6, r6, #2
3:		
		ldrh	r5, [r6]
		cmp	r5, #0xff00
		movne	r4, #0xff00
		bne	4f
		add	r6, r6, #2

		b	6f

4:
		@ error, address is r6, value read is r5, 
		@ wanted value r4
		b	error

6:
		@ print progress
		add	r4, r4, #1
		cmp	r4, #0x10000
		moveq	r0, #'.'
		moveq	r4, #0x0
		bleq	putc
		
		cmp	r6, r8
		blt	1b
		
		mov	pc, r10

	@ Sets each 3rd byte in memory to 0xff, rest to 0x00
	@ r4, r5, r6 will be changed
	@ r7 - start address (should be aligned)
	@ r8 - end address
	@ r9 - start of first 0x00 pattern (0-2)
msg_fill3:	.ascii "\nWriting pattern for knaizuk-2 #\0"

		.align
knaizuk_fill3:
		mov	r10, lr

		m_puts	msg_fill3
		m_phex	r9, 1
		m_putc	#' '

		mov	r6, r7
		mov	r4, #0

		mov	r5, r9
		cmp	r5, #1
		blt	2f
		beq	3f
		
1:		
		@ store 16 bit
		mov 	r5, #0x0000
		strh	r5, [r6]
		add	r6, r6, #2
2:
		mov	r5, #0xff00
		strh	r5, [r6]
		add	r6, r6, #2
3:
		mov	r5, #0x00ff
		strh	r5, [r6]
		add	r6, r6, #2

		@ print progress
		add	r4, r4, #1
		cmp	r4, #0x10000
		moveq	r0, #'.'
		moveq	r4, #0x0
		bleq	putc
		
		cmp	r6, r8
		blt	1b

		mov	pc, r10

	@ Checks memory for pattern generated in fill3
	@ r3, r4, r5, r6 will be changed
	@ r7 - start address (should be aligned)
	@ r8 - end address
	@ r9 - start of pattern (0-2)
msg_test3:	.ascii "\nTesting knaizuk pattern-2     #\0"

		.align
knaizuk_test3:
		mov	r10, lr

		m_puts	msg_test3
		m_phex	r9, 1
		m_putc	#' '

		mov	r6, r7
		mov	r4, #0

		mov	r5, r9
		cmp	r5, #1
		blt	2f
		beq	3f
		
1:		
		@ load 16 bit
		mov 	r3, #0x0000
		ldrh	r5, [r6]
		cmp	r5, r3
		movne 	r4, r3
		bne	4f
		add	r6, r6, #2
2:
		ldrh	r5, [r6]
		cmp	r5, #0xff00
		movne	r4, #0xff00
		bne	4f
		add	r6, r6, #2
3:		
		ldrh	r5, [r6]
		cmp	r5, #0x00ff
		movne	r4, #0x00ff
		bne	4f
		add	r6, r6, #2

		b	6f

4:
		@ error, address is r6, value read is r5, 
		@ wanted value r4
		b	error

6:
		@ print progress
		add	r4, r4, #1
		cmp	r4, #0x10000
		moveq	r0, #'.'
		moveq	r4, #0x0
		bleq	putc
		
		cmp	r6, r8
		blt	1b
		
		mov	pc, r10


	@
	@
	@
msg_copy_from:	.ascii "\nCopy code from address 0x\0"
msg_copy_till:	.ascii "\nCopy code till address 0x\0"
msg_copy_to:	.ascii "\nCopy code to address   0x\0"
		.align

prepare:
		mov	r10, lr
		mov	r9,  pc
		mov	r8,  #0

		m_puts	msg_copy_from
		m_phex	r10, 8
		m_puts	msg_copy_till
		m_phex	r9, 8
		m_puts	msg_copy_to
		m_phex	r8, 8

1:
		ldr	r7, [r10]
		str	r7, [r8]
		add	r10, r10, #4
		add	r8, r8,   #4
		cmp	r10, r9
		blt	1b

		mov	pc, #0

		mov	pc, r10
