/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

#include "http-plugin.h"
#include "nvalloc.h"
#include "util.h"
#include "logger.h"

typedef
struct _state {
    NVCL        *nvram ;
    const char *htpath ;
} STATE ;


/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

void *htinit (const char *doc_path, NVCL *nv) {
	STATE *s = ZMALLOC (STATE) ;

    s->nvram  =       0  ;
    s->htpath = doc_path ;

    return s ;
}

void htcleanup (void *desc) {
	// STATE *s = desc ;
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
    
    struct stat sbuf ;
    int fd ;
	IOBUF iobuf ;
	const char *dir ;
	STATE  *s = (STATE*)desc ;
    
    if (argc < 2) {
        logger (LOG_ERR, __FILE__ ": file name missing");
        return ;
    }

	XZERO (iobuf);
	dir = make_path (s->htpath, &iobuf, argv[1]);
    
    if (stat (dir, &sbuf) < 0) {
        logger (LOG_ERR, __FILE__ ": cannot stat %s: %s",
                dir, strerror (errno));

		kill_space (&iobuf);
        return ;
    }

    if (S_ISREG (sbuf.st_mode) == 0) {
        logger (LOG_ERR, __FILE__ ": not a regular file: %s", dir);

		kill_space (&iobuf);
        return ;
    }

    if ((fd = open (dir, O_RDONLY)) < 0) {
        logger (LOG_ERR, __FILE__ ": cannot open %s for reading: %s",
                dir, strerror (errno));

		kill_space (&iobuf);
        return ;
    }

	kill_space (&iobuf);

    body_app_file (r, fd, sbuf.st_size) ;
    close (fd);
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
