/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "hlst.h"
#include "util.h"
#include "nvalloc.h"
#include "http-plugin.h"
#include "parser.h"
#include "settings.h"
#include "logger.h"

/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */

#ifdef WITH_DMALLOC
typedef struct {
    const char *file ;
    int         line ;
} __avdump_rec ;
static
int __avdump_print (void *state, void *val, char *key, unsigned keylen) {
    __avdump_rec *a = state ;
    dmalloc_verify (val);
    logger (LOG_DEBUG, "AVPRINT at %s(%d): %s=%s.\n",
            a->file, a->line, key, val ? val : "(null)");
    return 0;
}
static
void __avdump (HLST *h, const char *file, int line) {
    __avdump_rec a = {file, line};
    dmalloc_verify (h);
    for_hlst_do (h, __avdump_print, &a); 
    dmalloc_log_stats ();                          \
}
#define avdump(c) __avdump (c,__FILE__, __LINE__)
#define DMALLOC_VERIFY(g) {\
                 dmalloc_verify (g);                            \
                 dmalloc_log_stats ();                          \
        }

#define DMALLOC_MESSAGE(g,fmt, ...) {\
                 dmalloc_message (fmt, ##__VA_ARGS__) ; \
                 DMALLOC_VERIFY(g) ;                    \
        }
#else
#define DMALLOC_VERIFY(g)         /* n/a */
#define DMALLOC_MESSAGE(fmt, ...) /* n/a */
#define avdump(c)                 /* n/a */
#endif

typedef
struct _state {
    NVCL        *nvram ;
    const char *htpath ;
} STATE ;


/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

void *htinit (const char *doc_path, NVCL *nv) {
    STATE *s = ZMALLOC (STATE) ;

    s->nvram  =       nv ;
    s->htpath = doc_path ;
    return s ;
}

void htcleanup (void *desc) {
    // STATE *s = desc ;
    free (desc);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
    STATE  *s = desc ;
    PARSER   *parser ;
    HLST        *avp ;
    const char *data ;

    if (argc < 2)
        return ;

    if (http_fallback (r)) {
        const char *s = rplycode (r, 404);

        body_printf (r, ("<html>"
                         "<head><title>%s</title></head>"
                         "<body><h1>%s</h1>URL not available: %s</body>"
                         "</html>"), s, s, http_request (r));
        return ;
    }
    
    avp = http_args (r);

    handle_global_settings (avp, s->nvram);

    parser = open_parser (avp, s->nvram, s->htpath) ;

    if (file_parser (parser, argv [1]) == 0)
        return ;

    /* parser */
    run_parser (parser) ;

    // avdump (avp); /* debugging */

    /* print result */
    if ((data = data_collect (parser)) != 0)
        body_append (r, data, 0);

    /* clean up */
    file_parser_end (parser);
    close_parser (parser) ;
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
