/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdarg.h>

#include "util.h"
#include "list.h"
#include "mime-types.h"
#include "nvalloc.h"
#include "logger.h"

#define  __HTTP_DEFAULTS_PRIVATE
#include "http-defaults.h"

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

void http_load_defaults (LIST *lst, int overwrite) {
    const char *p = htdefs ;

    while (*p) {
        const char *uri, *ena, *so, **argv, *s ;
        int argc ;
        MIMETYPE mm ;
        PAGE_ACT pa ;

        uri = p ;
        p += strlen (p) + 1;

        pa  = * p ++ ;

        ena =  p [0] ? p : 0 ;
        p += strlen (p) + 1;
        
        so = modname_decode [(unsigned char) * p ++];
        mm = mime_decode    [(unsigned char) * p ++];

        argc = 1 ;
        argv = (const char**) xmalloc (sizeof (char*));
        argv [0] = uri ;
        while (* p) {
            argv = (const char**)
                zrealloc (argv, (argc + 1) * sizeof (char *));
            argv [argc ++] = p ;
            p += strlen (p) + 1;
        }
        p ++ ;

        if ((s = list_get_so (lst, uri)) != 0) {
	    if ((overwrite > 1 &&
		 (verbosity & HTDEFAULTS_OVWRITE_LOG)) ||
		overwrite == 1)
                logger (LOG_WARNING,
	                "%s: overwriting default by %s -> %s", so, uri, s);
	    if (overwrite < 1) {
		if (verbosity & HTDEFAULTS_OVWRITE_LOG)
                    logger (LOG_WARNING, "%s: not overwriting default "
			    "by %s -> %s", so, uri, s);
	        continue ;
	    }
	}

        list_set_argv (lst, pa, ena, so, mm, argc, argv);
    }
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
