/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifndef __HTTP_DEFAULTS_H
#define __HTTP_DEFAULTS_H

#include "list.h"

extern void http_load_defaults (LIST*,int) ;

#ifdef __HTTP_DEFAULTS_PRIVATE

#include "settings.h"
#include "mimedump.h"
#include "modlistdump.h"

static const MIMETYPE mime_decode [] = {
    0
#   define              __mapper(mmt,name,code) , mmt
    ENCODED_MIME_TYPES (__mapper)
#   undef               __mapper
} ;

static const char *modname_decode [] = {
    0
#   define               __mapper(file,name,code) , file
    ENCODED_MODULE_LIST (__mapper)
#   undef                __mapper
} ;

#define _display()       _PRESENT
#define _nvEnable(a)     _HIDEBYDEFAULT    a
#define _nvNotEnable(a)  _HIDENOTBYDEFAULT a
#define _nvDisable(a)    _SHOWBYDEFAULT    a
#define _nvNotDisable(a) _SHOWNOTBYDEFAULT a

#define _htdef(uri, pa, mod, mmt, data) \
               uri "\0" pa "\0" mod mmt data "\0\0"

#define _htmpl(uri, pa, tmpl) \
        _htdef(uri, pa, _HTMPL_SO, _TEXT_HTML, tmpl)

/*
 * Example:
 *
 *     _htmpl ("/", _display (), "status.html")
 *
 * makes sure that the ressource https://horst.box/ is always served.
 *
 *     _htmpl ("/", _nvEnable ("otto"), "status.html")
 *
 * makes sure, that the resource https://horst.box/ is served only
 * if the nvram variable "http.access.otto" (or whatever the macro
 * LIST_ACCESS_BASE is set to) is "true" (or "yes", or "enabled" or 1).
 * Alternatively,
 *
 *     _htmpl ("/", _nvNotEnable ("otto"), "status.html")
 *
 * makes sure, that the resource https://horst.box/ is served if the
 * the nvram variable "http.access.otto" is not "true " (or "yes", or
 * "yes" etc.) So _nvNotEnable() is complementary to _nvEnable().
 *
 *     _htmpl ("/", _nvDisable ("otto"), "status.html")
 *
 * makes sure, that the resource https://horst.box/ is served unless
 * the nvram variable "http.access.otto" is "false" (or "no" or
 * "enabled" or 0).
 *
 *     _htmpl ("/", _nvNotDisable ("otto"), "status.html")
 *
 * makes sure, that the resource https://horst.box/ is served only if
 * the nvram variable "http.access.otto" is not "false" (or "no" etc.)
 * So _nvNotDisable() is complementary to _nvDisable().
 *
 * Note:
 *
 *  + An unserved resource is treated as if it were not existent.
 *  + Looking at the nvram variables, only the first character is
 *    checked (for true, false etc).
 */

static const char htdefs [] = {

    /* default/fall back page */
    _htmpl (LIST_DEFAULT_URI, _display (), "test/testpage.html")
    
#  include  "test-settings.h"
#  include  "info-settings.h"
#  include   "wan-settings.h"
#  include   "lan-settings.h"
#  include "phone-settings.h"
#  include   "sys-settings.h"
#  include   "wiz-settings.h"
#  include "help-settings.h"
#  include "misc-settings.h"

   "\0"
} ;

#endif /* __HTTP_DEFAULTS_PRIVATE */
#endif /* __HTTP_DEFAULTS_H */
