
/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany                                                     *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/

#ifndef __MHTTP_HTTP_H
#define __MHTTP_HTTP_H

#include "hlst.h"
#include "buffer.h"
#include "list.h"
#include "do.h"
#include "util.h"
#include "http-dl.h"

#ifdef HAVE_OPENSSL
#include "ssl.h"
#endif

typedef
struct _rply_rec {
    struct _hchan  *c ;
    MIMETYPE       mt ;
    IOBUF        path ;
    IOBUF      header ;
    IOBUF        body ;
    HSRCH       *args ;
    const char *error ;
} RPLY_REC ;

#ifdef __HTTP_PRIVATE
typedef
enum {
    none = 0,

    http_get_header,
    http_get_body,
    http_get_ok,

    http_post_header,
    http_post_body,
    http_post_ok,

    http_sent_ok
} HTPARSE ;
#endif


typedef
struct _hchan {
#   ifdef __HTTP_PRIVATE
    int           fd ;
    time_t   visited ;
    ip4addr_t client ;
    uint16_t  clport ;
#   ifdef HAVE_OPENSSL
    XXL         *ssl ;
#   endif
    struct {
        int access_count ;
        IOBUF        buf ;
        struct {
            HTPARSE      state ;
            const char  *htreq ;
            int      acc_ena:1 ;
            int       acc_ok:1 ;
            int   conn_close:1 ;
            int should_close:1 ;
            int     url_args:1 ;
            int   basic_auth:1 ;
            int     fallback:1 ;
            time_t  keep_alive ;
	    int    multipart:1 ; /* set to 1 if form submitted as multipart/form-data (file upload) */
	    char     *boundary ; /* multipart boundary */
            unsigned      seen ;
            unsigned      body ;
            size_t        hlen ;
            size_t        clen ;
        } http ;
        struct {
            const char     *so ;
            MIMETYPE        mt ;
            int           argc ;
            const char  **argv ;
            HLST       *htargs ;
        } args ;
    } req ;
    struct _http *base ;
#   else
    char          any ;
#   endif
} HCHAN ;

typedef
struct _htstats {
    /* independent part - will not be touched after initialisation */
    unsigned         sslok ;

    struct {
        unsigned available ;
        unsigned    active ;
    } channel ;

    struct {
        unsigned     depth ;
    } dll ;

    /* volatile part - can be flushed/reset */
    time_t               uptime ;

    struct {
        struct {
            size_t   bytes_sent ;
            unsigned      total ;
            unsigned     failed ;
            unsigned    code200 ;
            unsigned    code400 ;
            unsigned    code401 ;
            unsigned    code404 ;
            unsigned    code503 ;
        } reply ;

        struct {
            size_t   bytes_read ;
            unsigned   accepted ;
            unsigned    overrun ;
        } request ;

        struct {
            unsigned used_slots ;
            const char   *cache ;
        } dll ;
#       ifdef HAVE_OPENSSL
        SESS_INFO           ssl ;
#       endif
    } ht ;
} HTSTATS ;


typedef
struct _http {
#   ifdef __HTTP_PRIVATE
    int             listen_fd ;
    DO                *action ;
    LIST              *ht_lst ;
    NVCL               *nvram ;
#   ifdef HAVE_OPENSSL
    CTY                  *cty ;
#   endif
    time_t            started ;
    HTSTATS             stats ; /* statistics */
    struct sockaddr_in listen ;
    const char       *htadmin ; /* basic .. */
    const char      *htpasswd ; /* .. authentication */
    const char       *htrealm ; /* .. authentication */
    const char       *dll_dir ;
    const char       *doc_dir ;
    unsigned        dll_slots ;
    DLL_CACHE      *dll_cache ;
    unsigned     channel_idle ;
    unsigned         channels ;
    HCHAN         channel [1] ;
#   else
    char                  any ;
#   endif
} HTTP;


/* public interface */
extern HTTP    *http_init      (DO*,LIST*,unsigned,unsigned,
                                ip4addr_t,uint16_t,
                                const char*,const char*,const char*,
                                const char*,const char*,NVCL*
#                               ifdef HAVE_OPENSSL
                                ,CTY*
#                               endif
    );
extern unsigned http_reopen    (HTTP*);
extern void     http_status    (HTTP*, HTSTATS*) ;
extern char    *http_codestr   (HTTP*, int) ;
extern void     http_close     (HTTP*) ;
extern void     http_idlecheck (HTTP*) ;
extern void     http_statflush (HTTP*) ;
#endif /* __MHTTP_HTTP_H */
