/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "hlst.h"
#include "http-plugin.h"
#include "parser.h"
#include "nvalloc.h"
#include "plugin-util.h"
#include "logger.h"

/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */

#define INVALID_SRC_MAC_ADDRESS		CUSTOM_START_ERROR + 10
#define INVALID_SRC_MAC_EXISTS	    	CUSTOM_START_ERROR + 11

#define INVALID_DEST_MAC_ADDRESS	CUSTOM_START_ERROR + 12
#define INVALID_DEST_MAC_EXISTS	    	CUSTOM_START_ERROR + 13

#define INVALID_SRC_PORT		CUSTOM_START_ERROR + 20
#define INVALID_DEST_PORT		CUSTOM_START_ERROR + 21

#define INVALID_TOTAL_ELEMENTS		CUSTOM_START_ERROR + 60

#define TOTAL_ELEMENTS_LABEL		"lan_bridge_add.totalElements"
#define LIST_LABEL			"lan_bridge_add.list"
#define GROUP_LABEL			"lan_bridge_add"

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

int setId(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	char *nextIndex;
	char **val;
	char **ind;

	ind		= (char**)find_hlst(avp, "index", 0);
	//val 		= (char**)find_hlst(avp, varName, 0);
	nextIndex	= getNextIndex(nvram_avp, LIST_LABEL, TOTAL_ELEMENTS_LABEL);
   
	if(!ind || **ind == '\0')
	{
		if(nextIndex)
		{
			forceAddToList(avp,varName,nextIndex);
		}
		else
		{
			error = INVALID_TOTAL_ELEMENTS;
		}
	}
	else
	{
		addToList(avp,varName,*ind);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return NO_ERROR;
}

int checkSrcMacAddress(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int res, error = NO_ERROR;
	char *existVar, **val; 

	if(val = (char**)find_hlst(avp,varName,0))
	{
		res = validMacAddr(val);
		if(!res)
			error = INVALID_SRC_MAC_ADDRESS;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkDestMacAddress(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int res, error = NO_ERROR;
	char *existVar, **val; 

	if(val = (char**)find_hlst(avp,varName,0))
	{
		res = validMacAddr(val);
		if(!res)
			error = INVALID_DEST_MAC_ADDRESS;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkSrcPort(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
   
	if(getVarAsPort(avp, varName)==-1)
	{
		error = INVALID_SRC_PORT; 
        }

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkDestPort(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
   
	if(getVarAsPort(avp, varName)==-1)
	{
		error = INVALID_DEST_PORT; 
        }

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

HTMLOBJECT *init_htmlVars(){

	int max_size = 8;
	int max_index = max_size - 1;
	int i=0;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);

	p[i].revise 	= setId;
	p[i++].name 	= "lan_bridge_add.id";

	p[i].revise 	= checkSrcMacAddress;
	p[i++].name 	= "lan_bridge_add.srcMac";

	p[i].revise 	= checkDestMacAddress;
	p[i++].name 	= "lan_bridge_add.destMac";

	p[i].revise 	= noError;
	p[i++].name 	= "lan_bridge_add.srcPort";

	p[i].revise 	= noError;
	p[i++].name 	= "lan_bridge_add.destPort";

	p[i].revise 	= noError;
	p[i++].name 	= "lan_bridge_add.protocol";

	p[i].revise 	= noError;
	p[i++].name 	= "lan_bridge_add.mode";

       
	//sentinel
	p[i>(max_index) ? max_index : i].revise = NULL;
	p[i>(max_index) ? max_index : i].name 	= NULL;


	return p;
}

void free_htmlVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}

void handle_varError(HLST *avp, int ecode){

 
	switch(ecode){
	
	case INVALID_SRC_MAC_ADDRESS:	      	addToList(avp, "lan_bridge_add.srcMac.error", "1");
						break;
	case INVALID_DEST_MAC_ADDRESS:		addToList(avp, "lan_bridge_add.destMac.error", "1");
						break;
	case INVALID_SRC_PORT:			addToList(avp, "lan_bridge_add.srcPort.error", "1");
         					break;
	case INVALID_DEST_PORT:			addToList(avp, "lan_bridge_add.destPort.error", "1");
						break;
           

	default	:		    		//UNKNOWN error code, nothing to do
						logger (LOG_WARNING, "UNKNOWN error code: %d: ", ecode);
						return;
	}


	addToList(avp, "error", "1");
    
	return;
}


void *htinit (const char *doc_path, NVCL *nv) {
	STATE *s = ZMALLOC (STATE) ;

	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= "lan/lan_bridge.html";
	s->htmlvars.total_elements_label 	= TOTAL_ELEMENTS_LABEL;
	s->htmlvars.group_label 		= GROUP_LABEL;

	s->nvram  	= nv ;
	s->htpath 	= doc_path ;
	return s ;
}

void htcleanup (void *desc) {
	STATE *s = desc ;

	free_htmlVars(s->htmlvars.objects);
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
	STATE  *s = desc ;
	PARSER   *parser ;
	HLST        *avp ;
	const char *data ;
	char **retVal;
	char *html_file = (char*)argv[1];

	if (argc < 2)
		return ;
	
	avp = http_args (r);

	if((char**)find_hlst(avp,"save", 0))
	{
		//store
		if(save_indexHttpVars(s,avp))
			html_file = (char*)s->htmlvars.load_on_success;
	}
	else if((char**)find_hlst(avp,"delete", 0))
	{
		 //delete
		if(delete_indexHttpVars(s,avp))
			html_file = (char*)s->htmlvars.load_on_success;
	}
	else if((char**)find_hlst(avp,"add", 0))
	{
		 //do nothing for add
	}
	else
	{
		  load_indexHttpVars(s, avp);
	}
	
	handle_global_settings(avp, s->nvram);
        
	parser = open_parser (avp, s->nvram, s->htpath) ;

	if (file_parser (parser, htdocs_p (r, html_file)) == 0)
		return ;

	/* parser */
	run_parser (parser) ;

	avdump (avp); /* debugging */

	/* print result */
	if ((data = data_collect (parser)) != 0)
		body_append (r, data, 0);

	/* clean up */
	file_parser_end (parser);
	close_parser (parser) ;
}



/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

