/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: n/n (D-Link)			                          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <netinet/in.h>

#include "hlst.h"
#include "util.h"
#include "http-plugin.h"
#include "plugin-util.h"
#include "parser.h"
#include "nvalloc.h"
#include "logger.h"


/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */

#define INVALID_SSID_EXISTS		CUSTOM_START_ERROR + 10
#define INVALID_SSID_EXISTS2		CUSTOM_START_ERROR + 11
#define INVALID_SSID_EMPTY		CUSTOM_START_ERROR + 12
	
#define TOTAL_ELEMENTS_LABEL		"lan_mgmt_multiple.totalElements"
#define LIST_LABEL			"lan_mgmt_multiple.list"
#define GROUP_LABEL			"lan_mgmt_multiple"
#define LOAD_ON_SUCCESS_LABEL		"lan/lan_management_multiplessid.html"


/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

int checkSsid(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **ssid;

	ssid = (char**)find_hlst(avp, varName, 0);

	if(ssid && **ssid!='\0')
	{
		char *existVar;
		char **val;

		if(existVar = getExistNvramVar(nvram_avp, varName, *ssid))	
		{
			/* check if we found the variable that we have to edit */
			if(!isEditVariable(avp,existVar,varName))
				error = INVALID_SSID_EXISTS;
		}
		else if(val = (char**)find_hlst(nvram_avp, "lan_wlan.ssid", 0))
		{
			if(!strcmp(*val,*ssid))
				error = INVALID_SSID_EXISTS2;
		}
	}
	else
	{
		error = INVALID_SSID_EMPTY;
	}


	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}


int setId(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	char *nextIndex;
	char **val;
	char **ind;

	ind			= (char**)find_hlst(avp, "index", 0);
	//val 			= (char**)find_hlst(avp, varName, 0);
	nextIndex		= getNextIndex(nvram_avp, LIST_LABEL, TOTAL_ELEMENTS_LABEL);
   
	if(!ind || **ind == '\0')
	{
		if(nextIndex)
		{
			forceAddToList(avp,varName,nextIndex);
		}
	}
	else
	{
		addToList(avp,varName,*ind);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return NO_ERROR;
}


HTMLOBJECT *init_htmlVars(){

	int max_size = 4;
	int i=0;
	int max_index = max_size - 1;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);

	p[i].revise 	= setId;
	p[i++].name 	= "lan_mgmt_multiple.id";

	p[i].revise 	= checkSsid;
	p[i++].name	= "lan_mgmt_multiple.ssid";


	//sentinel
	p[i>max_index ? max_index : i].revise 	= NULL;
	p[i>max_index ? max_index : i].name    	= NULL;

	return p;
}

void free_htmlVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}



void handle_varError(HLST *avp, int ecode){

 
	switch(ecode){

	case INVALID_SSID_EXISTS:     	addToList(avp, "lan_mgmt_multiple.ssidExists.error", "1"); 
					break;

	case INVALID_SSID_EXISTS2:     	addToList(avp, "lan_mgmt_multiple.primarySsidExists.error", "1"); 
					break;

	case INVALID_SSID_EMPTY:     	addToList(avp, "lan_mgmt_multiple.ssidEmpty.error", "1"); 
					break;

	default	:			//UNKNOWN error code, nothing to do
					logger (LOG_WARNING, "UNKNOWN error code: %d: ", ecode);
					return;
	}


	addToList(avp, "error", "1");
    
	return;

}




void *htinit (const char *doc_path, NVCL *nv) {
    STATE *s = ZMALLOC (STATE) ;

	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= LOAD_ON_SUCCESS_LABEL ;
	s->htmlvars.group_label			= GROUP_LABEL;
	s->htmlvars.total_elements_label	= TOTAL_ELEMENTS_LABEL;

	s->nvram  	= nv ;
	s->htpath 	= doc_path ;
	return s;
}

void htcleanup (void *desc) {
	STATE *s = desc ;

	free_htmlVars(s->htmlvars.objects);
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
    
	STATE  *s = desc ;
	PARSER   *parser ;
        HLST        *avp ;
	const char *data ;
	char *html_file = (char*)argv[1];

	if (argc < 2)
		return ;

	avp = http_args (r);

	if((char**)find_hlst(avp,"save", 0))
	{
		 //store
		if(save_indexHttpVars(s,avp))
			html_file = (char*)s->htmlvars.load_on_success;

	}
	else if((char**)find_hlst(avp,"delete", 0))
	{
		 //delete
		if(delete_indexHttpVars(s,avp))
			html_file = (char*)s->htmlvars.load_on_success;
	}
	else if((char**)find_hlst(avp,"add", 0))
	{
		 //do nothing for add
	}
	else
	{
		  load_indexHttpVars(s, avp);
	}
        
	handle_global_settings(avp, s->nvram);

	parser = open_parser (avp, s->nvram, s->htpath) ;

	if (file_parser (parser,  htdocs_p (r, html_file)) == 0)
		return ;

	/* parser */
	run_parser (parser) ;

	avdump (avp); /* debugging */

	/* print result */
	if ((data = data_collect (parser)) != 0)
		body_append (r, data, 0);
	
	/* clean up */
	file_parser_end (parser);
	close_parser (parser) ;
}



/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

