/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: n/n (D-Link)			                          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <netinet/in.h>

#include "hlst.h"
#include "util.h"
#include "http-plugin.h"
#include "plugin-util.h"
#include "parser.h"
#include "nvalloc.h"
#include "logger.h"


/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */

#define INVALID_BEACON_INTERVAL	 		CUSTOM_START_ERROR + 10
#define INVALID_DTIM		              	CUSTOM_START_ERROR + 20
#define INVALID_RTS				CUSTOM_START_ERROR + 30
#define INVALID_FRAG				CUSTOM_START_ERROR + 40
    


/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

int isHiddenEnabled(HLST *avp)
{
	char **val = (char**)find_hlst(avp, "lan_performance.has_hiddenSsid", 0);

	if(val && !strcmp(*val, "true"))
	{
		return 1;
	}

	return 0;
}


int checkBeacon(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	int beacon;


	beacon = getVarAsLong(avp, varName);

	if(beacon < 1 || beacon > 65535) 
	{
		error = INVALID_BEACON_INTERVAL;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkDtim(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	int dtim;


	dtim = getVarAsLong(avp, varName);

	if(dtim < 1 || dtim > 255)
	{
		error = INVALID_DTIM;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}


int checkRts(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	int rts;


	if(isHiddenEnabled(avp))
	{
		rts = getVarAsLong(avp, varName);
		if( rts < 0 || rts > 4096 )
		{
			error = INVALID_RTS;
		}
	}
	else
	{
		delete_hlst(avp, varName, 0);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}


int checkFrag(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	int frag;


	if(isHiddenEnabled(avp))
	{
		frag = getVarAsLong(avp, varName);
		if( frag < 256 || frag > 4096)
		{
			error = INVALID_FRAG;
		}
	}
	else
	{
		delete_hlst(avp, varName, 0);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}


HTMLOBJECT *init_htmlVars(){

	int max_size = 8;
	int i=0;
	int max_index = max_size - 1;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);

	p[i].revise 	= checkBeacon;
	p[i++].name    	= "lan_performance.beaconInterval";

	p[i].revise 	= checkDtim;
	p[i++].name	= "lan_performance.dtim";

	p[i].revise 	= ensureCheckState;
	p[i++].name	= "lan_performance.has_hiddenSsid";

	p[i].revise 	= noError;
	p[i++].name	= "lan_performance.antennaPower";

	p[i].revise 	= checkRts;
	p[i++].name	= "lan_performance.rtsThreshold";

	p[i].revise 	= checkFrag;
	p[i++].name	= "lan_performance.fragThreshold";

	p[i].revise 	= noError;
	p[i++].name	= "lan_performance.bgMode";

	//sentinel
	p[i>max_index ? max_index : i].revise 	= NULL;
	p[i>max_index ? max_index : i].name	= NULL;

	return p;
}

void free_htmlVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}



void handle_varError(HLST *avp, int ecode){

 
	switch(ecode){

	case INVALID_BEACON_INTERVAL:	  	addToList(avp, "lan_performance.beaconInterval.error", "1"); 
						break;

	case INVALID_DTIM:	   	      	addToList(avp, "lan_performance.dtim.error", "1"); 
						break;

	case INVALID_RTS:	   		addToList(avp, "lan_performance.rtsThreshold.error", "1"); 
						break;

	case INVALID_FRAG:	   		addToList(avp, "lan_performance.fragThreshold.error", "1"); 
						break;

	default	:				//UNKNOWN error code, nothing to do
						logger (LOG_WARNING, "UNKNOWN error code: %d: ", ecode);
						return;
	}

	addToList(avp, "error", "1");
    
	return;
}




void *htinit (const char *doc_path, NVCL *nv) {
	STATE *s = ZMALLOC (STATE) ;

	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= NULL;
	s->htmlvars.group_label			 	= NULL;
	s->htmlvars.total_elements_label	= NULL;

	s->nvram  	= nv ;
	s->htpath 	= doc_path ;
	return s ;
}

void htcleanup (void *desc) {
	STATE *s = desc ;

	free_htmlVars(s->htmlvars.objects);
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
    
	STATE  *s = desc ;
	PARSER   *parser ;
	HLST        *avp ;
	const char *data ;
	char *html_file = (char*)argv[1];

	if (argc < 2)
		return ;

	avp = http_args (r);

	CALL_HTTPVARS_ACTION(s,avp);
        
	handle_global_settings(avp, s->nvram);

	parser = open_parser (avp, s->nvram, s->htpath) ;

	if (file_parser (parser,  htdocs_p (r, html_file)) == 0)
		return ;

	/* parser */
	run_parser (parser) ;

	avdump (avp); /* debugging */

	/* print result */
	if ((data = data_collect (parser)) != 0)
		body_append (r, data, 0);

	/* clean up */
	file_parser_end (parser);
	close_parser (parser) ;
}



/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

