/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: n/n (D-Link)			                          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <netinet/in.h>

#include "hlst.h"
#include "util.h"
#include "http-plugin.h"
#include "plugin-util.h"
#include "parser.h"
#include "nvalloc.h"
#include "logger.h"


/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */

#define INVALID_DESTINATION_IP		CUSTOM_START_ERROR + 10
#define INVALID_SUBNETMASK		CUSTOM_START_ERROR + 20
#define INVALID_GATEWAY_IP		CUSTOM_START_ERROR + 30
#define INVALID_NETWORK_EXISTS		CUSTOM_START_ERROR + 40
#define INVALID_DESTINATION_EXISTS	CUSTOM_START_ERROR + 50
#define INVALID_DESTINATION_MASK	CUSTOM_START_ERROR + 60
	
#define TOTAL_ELEMENTS_LABEL		"lan_routing.totalElements"
#define LIST_LABEL			"lan_routing.list"
#define GROUP_LABEL			"lan_routing"

typedef enum{
	ROUTE_TYPE_NOT_SET,
	ROUTE_TYPE_GATEWAY,
	ROUTE_TYPE_PVC
}ROUTE_TYPE;


/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */
ROUTE_TYPE getRouteType(HLST *avp)
{
	char **val = (char**)find_hlst(avp, "lan_routing.routing", 0);

	if(val)
	{
		if(!strcmp("gateway",*val))
			return ROUTE_TYPE_GATEWAY;
		if(!strcmp("pvc",*val))
			return ROUTE_TYPE_PVC;
	}

	return ROUTE_TYPE_NOT_SET;
}

#if 0
int createNetworkAddress(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **dest = (char**)find_hlst(avp, "lan_routing.destination", 0);
	char **mask = (char**)find_hlst(avp, "lan_routing.netmask", 0);
		
	if(dest && validIpAddr(dest) &&
	   mask && validSubnetMask(mask))
	{
		char *tmp;
		char *existVar;
		struct in_addr addr;
		struct in_addr netmask;
		struct in_addr new_dest;
	
		inet_aton(*dest,&addr);
		inet_aton(*mask,&netmask);
	
		new_dest.s_addr = addr.s_addr & netmask.s_addr;
				 
		tmp = (char*)inet_ntoa(new_dest);

		addToList(avp,varName,tmp);

		if(existVar = getExistNvramVar(nvram_avp, varName, tmp))	
		{
			/* check if we found the variable that we have to edit */
			if(!isEditVariable(avp,existVar,varName))
				error = INVALID_NETWORK_EXISTS;
		}
	}


	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}
#endif

int checkDestIp(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **mask = (char**)find_hlst(avp, "lan_routing.netmask", 0);
	char **dest = (char**)find_hlst(avp, varName, 0);

	
	if(!dest || !validNetworkAddr(dest))
	{
		 error = INVALID_DESTINATION_IP;
	}
	else
	{
		if(mask && validSubnetMask(mask))
		{
			char *tmp;
			char *existVar;
			struct in_addr new_dest;
			struct in_addr netmask;
			struct in_addr addr;

			inet_aton(*dest,&addr);
			inet_aton(*mask,&netmask);
		
			new_dest.s_addr = addr.s_addr & netmask.s_addr;
					 
			tmp = (char*)inet_ntoa(new_dest);

			if(new_dest.s_addr != addr.s_addr)
			{
			      error = INVALID_DESTINATION_MASK;
			}
			else if(existVar = getExistNvramVar(nvram_avp, varName, tmp))	
			{
				/* check if we found the variable that we have to edit */
				if(!isEditVariable(avp,existVar,varName))
					error = INVALID_DESTINATION_EXISTS;
			}
		}

	}


	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkGatewayIp(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;


	if(getRouteType(avp) == ROUTE_TYPE_GATEWAY)
	{
		char **ip = (char**)find_hlst(avp, varName, 0);
		
		if(!ip || !validIpAddr(ip))
		{
			error = INVALID_GATEWAY_IP;
		}
	}
	else
	{
		forceAddToList(avp,varName,"");
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}


int checkWanPvc(HLST *avp, HLST *nvram_avp, const char *varName)
{

	if(getRouteType(avp) != ROUTE_TYPE_PVC)
	{
	    forceAddToList(avp,varName,"");
	}


	return NO_ERROR;
}


int checkSubnetMask(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **mask;

	mask = (char**)find_hlst(avp, varName, 0);

	if(!mask || !validSubnetMask(mask))
	{
		error = INVALID_SUBNETMASK;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}


int setId(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	char *nextIndex;
	char **val;
	char **ind;

	ind			= (char**)find_hlst(avp, "index", 0);
	//val 			= (char**)find_hlst(avp, varName, 0);
	nextIndex		= getNextIndex(nvram_avp, LIST_LABEL, TOTAL_ELEMENTS_LABEL);
   
	if(!ind || **ind == '\0')
	{
		if(nextIndex)
		{
			forceAddToList(avp,varName,nextIndex);
		}
	}
	else
	{
		addToList(avp,varName,*ind);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return NO_ERROR;
}

HTMLOBJECT *init_htmlVars(){

	int max_size = 8;
	int i=0;
	int max_index = max_size - 1;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);

	p[i].revise 	= setId;
	p[i++].name 	= "lan_routing.id";

	/* not useful at the moment */
	//p[i].revise 	= createNetworkAddress;
	//p[i++].name	= "lan_routing.networkAddress";

	p[i].revise 	= checkDestIp;
	p[i++].name	= "lan_routing.destination";

	p[i].revise 	= checkSubnetMask;
	p[i++].name	= "lan_routing.netmask";

	p[i].revise 	= noError;
	p[i++].name	= "lan_routing.routing";

	p[i].revise 	= checkGatewayIp;
	p[i++].name	= "lan_routing.gateway";

	p[i].revise 	= checkWanPvc;
	p[i++].name	= "lan_routing.wanPvc";


	//sentinel
	p[i>max_index ? max_index : i].revise 	= NULL;
	p[i>max_index ? max_index : i].name    	= NULL;

	return p;
}

void free_htmlVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}



void handle_varError(HLST *avp, int ecode){

 
	switch(ecode){

	case INVALID_DESTINATION_IP:   		addToList(avp, "lan_routing.destination.error", "1"); 
						break;

	case INVALID_GATEWAY_IP:   		addToList(avp, "lan_routing.gateway.error", "1"); 
						break;

					
	case INVALID_SUBNETMASK:		addToList(avp, "lan_routing.netmask.error", "1"); 
						break;

	case INVALID_NETWORK_EXISTS:		addToList(avp, "lan_routing.networkExists.error", "1"); 
						break;

	case INVALID_DESTINATION_EXISTS:	addToList(avp, "lan_routing.destinationExists.error", "1"); 
						break;

	case INVALID_DESTINATION_MASK:		addToList(avp, "lan_routing.destinationMask.error", "1"); 
						break;

	default	:				//UNKNOWN error code, nothing to do
						logger (LOG_WARNING, "UNKNOWN error code: %d: ", ecode);
						return;
	}


	addToList(avp, "error", "1");
    
	return;

}




void *htinit (const char *doc_path, NVCL *nv) {
	STATE *s = ZMALLOC (STATE) ;

	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= "lan/lan_routing.html";
	s->htmlvars.group_label			= GROUP_LABEL;
	s->htmlvars.total_elements_label	= TOTAL_ELEMENTS_LABEL;

	s->nvram  	= nv ;
	s->htpath 	= doc_path ;
	return s;
}

void htcleanup (void *desc) {
	STATE *s = desc ;

	free_htmlVars(s->htmlvars.objects);
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
    
	STATE  *s = desc ;
	PARSER   *parser ;
        HLST        *avp ;
	const char *data ;
	char *html_file = (char*)argv[1];

	if (argc < 2)
		return ;

	avp = http_args (r);

	if((char**)find_hlst(avp,"save", 0))
	{
		 //store
		if(save_indexHttpVars(s,avp))
			html_file = (char*)s->htmlvars.load_on_success;

	}
	else if((char**)find_hlst(avp,"delete", 0))
	{
		 //delete
		if( delete_indexHttpVars(s,avp))
			html_file = (char*)s->htmlvars.load_on_success;
	}
	else if((char**)find_hlst(avp,"add", 0))
	{
		 //do nothing for add
	}
	else
	{
		  load_indexHttpVars(s, avp);
	}
        
	handle_global_settings(avp, s->nvram);

	parser = open_parser (avp, s->nvram, s->htpath) ;

	if (file_parser (parser,  htdocs_p (r, html_file)) == 0)
		return ;

	/* parser */
	run_parser (parser) ;

	avdump (avp); /* debugging */

	/* print result */
	if ((data = data_collect (parser)) != 0)
		body_append (r, data, 0);
	
	/* clean up */
	file_parser_end (parser);
	close_parser (parser) ;
}



/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

