/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Susanne Goldammer <s.goldammer@maxina.de> - Maxina GmbH,      *
*   Neu-Isenburg, Germany						  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "hlst.h"
#include "http-plugin.h"
#include "parser.h"
#include "nvalloc.h"
#include "plugin-util.h"
#include "logger.h"

/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */


#define INVALID_WORKGROUP			CUSTOM_START_ERROR + 10


/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

/* check if the user wants to use a specific username and passwort for dsl access */
int isSambaEnabled(HLST *avp)
{
	char **retVal;
	retVal = (char**)find_hlst(avp, "lan_samba_conf.hasShare", 0);

	if(retVal && !strcmp(*retVal,"true"))
		return 1;
	else
		return 0;
}


int checkWorkgroup(HLST *avp, HLST *nvram_avp, const char *varName)
{
	char **val;
	int error = NO_ERROR;
	
	if(!isSambaEnabled(avp))
	{
		delete_hlst(avp,varName,0);
		return NO_ERROR;
	}

	val = (char**)find_hlst(avp, varName, 0);	
	if(val && **val!='\0')
	{
		removeBlanks(*val);	
		forceAddToList(avp,varName,*val);		
		if(**val == '\0')
			error = INVALID_WORKGROUP;
	}
	else 
	{
		error = INVALID_WORKGROUP;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);
		

	return error;
}


HTMLOBJECT *init_htmlVars(){

	int max_size = 3;
	int max_index = max_size - 1;
	int i=0;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);

	p[i].revise 	= checkWorkgroup;
	p[i++].name 	= "lan_samba_conf.workgroup";

	p[i].revise 	= ensureCheckState;
	p[i++].name 	= "lan_samba_conf.hasShare";


	//sentinel
	p[i>(max_index) ? max_index : i].revise = NULL;
	p[i>(max_index) ? max_index : i].name 	= NULL;


	return p;
}

void handle_varError(HLST *avp, int ecode){

 
	switch(ecode){
	
	case INVALID_WORKGROUP:		addToList(avp, "lan_samba_conf.workgroup.error", "1");
								break;
	
	default	:					//UNKNOWN error code, nothing to do
								logger (LOG_WARNING, "UNKNOWN error code: %d: ", ecode);
								return;
	}


	addToList(avp, "error", "1");
    
	return;

}

void free_htmlVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}

void *htinit (const char *doc_path, NVCL *nv) {
    STATE *s = ZMALLOC (STATE) ;

	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= "lan/lan_samba.html";
	//s->htmlvars.total_elements_label 	= TOTAL_ELEMENTS_LABEL;
	//s->htmlvars.group_label 		= "lan_samba_conf.shares";

    s->nvram  =       nv ;
    s->htpath = doc_path ;
    return s ;
}


void htcleanup (void *desc) {
    // STATE *s = desc ;
    free (desc);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
    STATE  *s = desc ;
    PARSER   *parser ;
    HLST        *avp ;
	const char *data ;
	char **retVal;
	char *html_file = (char*)argv[1];

	if (argc < 2)
		return ;

	avp = http_args (r);   

    if (http_fallback (r)) {
        const char *s = rplycode (r, 404);

        body_printf (r, ("<html>"
                         "<head><title>%s</title></head>"
                         "<body><h1>%s</h1>URL not available: %s</body>"
                         "</html>"), s, s, http_request (r));
        return ;
    }

   
    CALL_HTTPVARS_ACTION(s,avp);
    

    handle_global_settings (avp, s->nvram);

    parser = open_parser (avp, s->nvram, s->htpath) ;

    if (file_parser (parser, argv [1]) == 0)
        return ;

    /* parser */
    run_parser (parser) ;

    // avdump (avp); /* debugging */

    /* print result */
    if ((data = data_collect (parser)) != 0)
        body_append (r, data, 0);

    /* clean up */
    file_parser_end (parser);
    close_parser (parser) ;
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
