/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: n/n (D-Link)			                          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "hlst.h"
#include "http-plugin.h"
#include "parser.h"
#include "nvalloc.h"
#include "plugin-util.h"
#include "logger.h"

/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */

#define INVALID_NAME				CUSTOM_START_ERROR + 10
#define INVALID_NAME_EXISTS			CUSTOM_START_ERROR + 11
#define MNTPOINT_NOT_EXISTS			CUSTOM_START_ERROR + 20

#define INVALID_TOTAL_ELEMENTS		CUSTOM_START_ERROR + 40

#define USB_MNTPOINT_NOT_FOUND		CUSTOM_START_ERROR + 50	
#define MNTPOINT_NOT_FOUND			CUSTOM_START_ERROR + 60
#define MINOR_NOT_FOUND				CUSTOM_START_ERROR + 70
#define INVALID_MINOR				CUSTOM_START_ERROR + 80

#define INVALID_USER				CUSTOM_START_ERROR + 90
#define FORBIDDEN_USER				CUSTOM_START_ERROR + 100

#define MNTPOINT_IN_USE				CUSTOM_START_ERROR + 110
#define INVALID_MNTPOINT_EMPTY		CUSTOM_START_ERROR + 120

#define TOTAL_ELEMENTS_LABEL		"lan_samba_conf.shares.totalElements"
#define LIST_LABEL					"lan_samba_conf.shares.list"
#define USB_MNTPOINT_LABEL			"lan_usb_storage.partition.mntpoint"
#define SMB_MNTPOINT_LABEL			"lan_samba_conf.shares.mntpoint"
#define SMB_MINOR_LABEL				"lan_samba_conf.shares.minor"
#define DENIED_USER_LIST			"lan_samba_user.denied_user_list"



/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

int checkName(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char *existVar, **val;

	val = (char**)find_hlst(avp, varName, 0);	
	if(val && **val!='\0')
	{
		//removeBlanks(*val);
		if(existVar = getExistNvramVar(nvram_avp, varName, *val))	
		{
			/* check if we found the variable that we have to edit */
			if(!isEditVariable(avp,existVar,varName))
				error = INVALID_NAME_EXISTS;
		}
	}
	else 
	{
		error = INVALID_NAME;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);
		

	return error;
}


int setId(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	char *nextIndex;
	char **val;
	char **ind;

	ind			= (char**)find_hlst(avp, "index", 0);
	//val 			= (char**)find_hlst(avp, varName, 0);
	nextIndex		= getNextIndex(nvram_avp, LIST_LABEL, TOTAL_ELEMENTS_LABEL); 
	
	if(!ind || **ind == '\0')
	{
		if(nextIndex)
		{
			forceAddToList(avp,varName,nextIndex);
		}
		else
		{
			error = INVALID_TOTAL_ELEMENTS;
		}
	}
	else
	{
		addToList(avp,varName,*ind);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return NO_ERROR;
}

int isExistingMntPoint(HLST *nvram_avp, char *mntPoint )
{
	HSRCH 		*lookup;
	char 		*label 		= USB_MNTPOINT_LABEL;
	const char	**Val		= NULL;
	char		*var 		= NULL;
	
	if(!mntPoint)
		return 0;

	/* check if there is a lan_usb_storage.partition.mntpoint variable in the nvram which has he same mountPoint */
	lookup = open_hlst_search (nvram_avp);
	while ((Val = (const char**) next_hlst_search (lookup)) != 0) 
	{
		//printf("Val: %s\n", *Val);    
		if(strcasecmp(*Val,mntPoint)!=0)
		   continue;

		//printf("Val = mntPoint\n");    
		/* value was equal to the mntPoint of this smb_conf, so get the associated variable */
		var = query_key_hlst((void **)Val) ;

		if(strncmp(label, var, strlen(label)) == 0)
		{
			/* we found a variable beginning with label and having the same value as mntPoint */			
			return 1;
		}
	
		/* that was not what we're looking for */
		var = NULL;
	}
	close_hlst_search (lookup) ;

	return 0;
}


int checkMntPoint(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	/* check if there is a lan_usb_storage.partition variable in the nvram which has he same mountPoint */
	
	int			res			= 0;
	int 		error		= NO_ERROR;
	char 		*existVar	= NULL;
	char 		**ind 		= (char**)find_hlst(avp, "index", 0);
	char 		**mntPoint 	= (char**)find_hlst(avp, varName, 0);
    
	if(mntPoint && removeBlanks(*mntPoint))
	{		
		/* check if there is a device with the same mntPoint */
		res = isExistingMntPoint(nvram_avp, *mntPoint);
	
		/* if not, create an error */	
    	if(!res)
		{
			handle_varError(avp, MNTPOINT_NOT_EXISTS);
        	return MNTPOINT_NOT_EXISTS;
		}
	
		/* check if the mntPoint is already shared through another samba share config */
		if(existVar = getExistNvramVar(nvram_avp, varName, *mntPoint))	
		{
			/* check if we found the variable that we have to edit */
			if(!isEditVariable(avp,existVar,varName))
				error = MNTPOINT_IN_USE;
		}
	}
	else
	{
		error = INVALID_MNTPOINT_EMPTY;
	}
	
	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}



int isDeniedUsername(HLST *avp, HLST *nvram_avp, const char *varName)
{
	char 	**indexList, **Val;
	char	*_dup, *p;
	_dup 		= NULL;

	Val		= (char**)find_hlst(avp, varName, 0);
	indexList	= (char**)find_hlst(nvram_avp, DENIED_USER_LIST, 0);

	if(indexList && (_dup = zstrdup(*indexList)))
	{
		int stop;
		p = strtok(_dup, " ");

		do
		{	
			if(Val && *Val && strcmp(p,*Val) == 0)
				return 1;

		}while(p = strtok(NULL, " "));

		free(_dup);
	}

	
	return 0;
}

int checkUser(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char *existVar, **val, **ind;
	char **denied;

	/* make sure that the new samba user is not already existing for one of our system users (asterisk, web etc.) */	

	val = (char**)find_hlst(avp, varName, 0);	
	if(val && removeBlanks(*val))
	{
		/* replace the name with stripped lanks */
		forceAddToList(avp,varName,removeBlanks(*val));
		
		//check if user name is already existing as a system user
		if(isDeniedUsername(avp, nvram_avp, varName))
			error = FORBIDDEN_USER;
	}
	else 
	{
		error = INVALID_USER;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);
		

	return error;
}

int isActive(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	char **val	= (char**)find_hlst(avp, varName, 0);	

	/* check whether the value is true, else set false as default value, so that the user cannot save wrong data */
	if(val && **val!='\0')
	{
		if(strcmp(*val, "true"))
		{
			/* value is not "true" */ 
			forceAddToList(avp,varName,"false");
		}
	}
	else
	{
		/* if there is no value, save "false" per default */
		addToList(avp,varName,"false");
	}
	return NO_ERROR;	
}

int isPhyActive(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int 	res			= 0;
	char	**mntPoint	= (char**)find_hlst(avp, SMB_MNTPOINT_LABEL , 0);;

	/* get the mntPoint for the samba conf to be saved */
	res 		= isExistingMntPoint(nvram_avp, *mntPoint);

	/* if not, create an error */	
    if(!res)
	{
		forceAddToList(avp,varName,"false");
	}
	else
	{
		forceAddToList(avp,varName,"true");
	}
	
	return NO_ERROR;
}

#if 0

int checkMinor(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error 	= NO_ERROR;
	char		**minor, **usb_mntPoint, **mntPoint;
	char		*dest = NULL;
	char		*label = USB_MNTPOINT_LABEL;

	minor		= (char**)find_hlst(avp, SMB_MINOR_LABEL , 0);

	if(minor && *minor)
	{
		/* test if lan_usb_storage.mntpoint.$minor == $mntPoint */	

		dest = (char *)xmalloc( strlen(label) + strlen(*minor) + 2 ); /* 1 for . and 1 for \n */
		snprintf(dest,strlen(label)+1,"%s",label);			
		dest = strcat(dest, ".");
		dest = strncat (dest, *minor, strlen(*minor));
		
		/* get the mntPoint for the usb device which matchs the minor id */
		usb_mntPoint	= (char**)find_hlst(nvram_avp, dest, 0);

		/* get the mntPoint for the samba conf to be saved */
		mntPoint	= (char**)find_hlst(avp, "lan_samba_conf.shares.mntpoint" , 0);

		if(!usb_mntPoint || !*usb_mntPoint) 	
			error = USB_MNTPOINT_NOT_FOUND; 
		
		else if(!mntPoint || !*mntPoint) 
			error = MNTPOINT_NOT_FOUND; 

		else if(!strcmp(*mntPoint, *usb_mntPoint))
		{
			//printf("usbMntPoint is: %s  MntPoint is: %s\n", *usb_mntPoint, *mntPoint);
			error = NO_ERROR;
		}
		else
			error = INVALID_MINOR;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkModel(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	return NO_ERROR;
}

int checkVendor(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	return NO_ERROR;
}
#endif

HTMLOBJECT *init_htmlVars(){

	int max_size = 9;
	int max_index = max_size - 1;
	int i=0;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);

	p[i].revise 	= setId;
	p[i++].name 	= "lan_samba_conf.shares.id";

	p[i].revise 	= checkName;
	p[i++].name 	= "lan_samba_conf.shares.name";

	p[i].revise 	= checkUser;
	p[i++].name 	= "lan_samba_conf.shares.user";

	p[i].revise 	= checkMntPoint;			/* first check if mountpoint still exists */
	p[i++].name 	= "lan_samba_conf.shares.mntpoint";

	p[i].revise 	= isActive;
	p[i++].name 	= "lan_samba_conf.shares.isactive";

	p[i].revise 	= isPhyActive;
	p[i++].name 	= "lan_samba_conf.shares.isphyactive";
	
	p[i].revise 	= noError;				
	p[i++].name 	= "lan_samba_conf.shares.userlabel";

	p[i].revise 	= noError;				
	p[i++].name 	= "lan_samba_conf.shares.isreadonly";	

#if 0
	p[i].revise 	= checkModel;				/* check if model matches for the given mntPoint */
	p[i++].name 	= "lan_samba_conf.shares.model";	

	p[i].revise 	= checkVendor;				/* check if vendor matches for the given mntPoint */
	p[i++].name 	= "lan_samba_conf.shares.vendor";
	
	p[i].revise 	= checkMinor;				/* check if vendor matches for the given mntPoint */
	p[i++].name 	= "lan_samba_conf.shares.minor";
#endif


	//sentinel
	p[i>(max_index) ? max_index : i].revise = NULL;
	p[i>(max_index) ? max_index : i].name 	= NULL;


	return p;
}


void free_htmlVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}

void handle_varError(HLST *avp, int ecode){

 
	switch(ecode){
	
	case INVALID_NAME:				addToList(avp, "lan_samba_conf.name.error", "1");
									break;
	case INVALID_NAME_EXISTS:		addToList(avp, "lan_samba_conf.nameExists.error", "1");
									break;
	case MNTPOINT_NOT_EXISTS:		addToList(avp, "lan_samba_conf.mntpoint.error", "1");
									break;
	case INVALID_USER:				addToList(avp, "lan_samba_conf.user.error", "1");
									break;
	case FORBIDDEN_USER:			addToList(avp, "lan_samba_conf.userForbidden.error", "1");
									break;
	case MNTPOINT_IN_USE:			addToList(avp, "lan_samba_conf.mntpoint.alreadyInUse.error", "1");
									break;
	case INVALID_MNTPOINT_EMPTY:	addToList(avp, "lan_samba_conf.mntpoint.empty.error", "1");
									break;


	/* errors which should never happen, unless the user sends corrupted get data */
	case USB_MNTPOINT_NOT_FOUND:			addToList(avp, "lan_samba_conf.mntpoint.usbNotFound.error", "1");
							break;
	case INVALID_MINOR:				addToList(avp, "lan_samba_conf.minor.invalid.error", "1");
							break;	
	/*case MNTPOINT_NOT_FOUND:			addToList(avp, "lan_samba_conf.Internal.error", "1");
							break;
	case MINOR_NOT_FOUND:				addToList(avp, "lan_samba_conf.Internal.error", "1");
							break;	*/
	default	:					//UNKNOWN error code, nothing to do
							logger (LOG_WARNING, "UNKNOWN error code: %d: ", ecode);
							return;
	}


	addToList(avp, "error", "1");
    
	return;

}


void *htinit (const char *doc_path, NVCL *nv) {
	STATE *s = ZMALLOC (STATE) ;

	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= "lan/lan_samba.html";
	s->htmlvars.total_elements_label 	= TOTAL_ELEMENTS_LABEL;
	s->htmlvars.group_label 		= "lan_samba_conf.shares";

	s->nvram  	= nv ;
	s->htpath 	= doc_path ;
	return s ;
}

void htcleanup (void *desc) {
	STATE *s = desc ;

	free_htmlVars(s->htmlvars.objects);
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
	STATE  *s = desc ;
	PARSER   *parser ;
	HLST        *avp ;
	const char *data ;
	char **retVal;
	char *html_file = (char*)argv[1];

	if (argc < 2)
		return ;

	avp = http_args (r);

	if((char**)find_hlst(avp,"save", 0))
	{
		 //store
		if(save_indexHttpVars(s,avp)) {
			html_file = (char*)s->htmlvars.load_on_success;
    			count_types(s, avp, "lan_samba_conf.shares");
		}

	}
	else if((char**)find_hlst(avp,"delete", 0))
	{
		//delete
		if(delete_indexHttpVars(s,avp)) {
			html_file = (char*)s->htmlvars.load_on_success;
    			count_types(s, avp, "lan_samba_conf.shares");
		}		
	}
	else if((char**)find_hlst(avp,"add", 0))
	{
		 //do nothing for add
	}
	else if((char**)find_hlst(avp,"choose", 0))
	{
		//load a html file to choose a partition
		html_file = "lan/lan_samba_choose.html";
	}
	else if((char**)find_hlst(avp,"accept", 0))
	{
		//load a html file to choose a partition
		html_file = "lan/lan_samba_config.html";
	}
	else if((char**)find_hlst(avp,"cancel", 0))
	{
		//load a html file to choose a partition
		html_file = "lan/lan_samba.html";
	}
	else
	{
		  load_indexHttpVars(s, avp);
	}
        
        
	handle_global_settings(avp, s->nvram);
	
	parser = open_parser (avp, s->nvram, s->htpath) ;

	if (file_parser (parser, htdocs_p (r, html_file)) == 0)
		return ;

	/* parser */
	run_parser (parser) ;

	avdump (avp); /* debugging */

	/* print result */
	if ((data = data_collect (parser)) != 0)
		body_append (r, data, 0);

	/* clean up */
	file_parser_end (parser);
	close_parser (parser) ;
}



/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

