/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: n/n (D-Link)			                          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "hlst.h"
#include "http-plugin.h"
#include "parser.h"
#include "nvalloc.h"
#include "plugin-util.h"
#include "logger.h"

/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */

#define INVALID_NAME			CUSTOM_START_ERROR + 10
#define INVALID_NAME_EXISTS		CUSTOM_START_ERROR + 11
#define FORBIDDEN_NAME			CUSTOM_START_ERROR + 30
#define INVALID_TOTAL_ELEMENTS	CUSTOM_START_ERROR + 40
#define SHARE_USER_DEPENDENCY	CUSTOM_START_ERROR + 50

#define TOTAL_ELEMENTS_LABEL	"lan_samba_user.totalElements"
#define LIST_LABEL				"lan_samba_user.list"

#define DENIED_USER_LIST		"lan_samba_user.denied_user_list"
#define USER_NAME				"lan_samba_user.name."
#define SHARE_USER_LABEL		"lan_samba_conf.shares.user"


/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */
	
/*
 * Is user used for a share?
 */
int hasShareDependency(STATE  *s,  HLST *avp)
{  
	char 	**user, **index, *label;
	HLST 	*nvram_avp;
	int	error = NO_ERROR;

	index = (char**)find_hlst(avp,"index", 0);
	
	/* create nvram list */
	nvram_avp = create_hlst(300, nvram_avp_destroy, NULL);   
	load_from_nvram(s->nvram , nvram_avp, NULL);

	/* create label for username and index */
	label = strcat (strcpy ((char*) xmalloc (strlen (USER_NAME) + strlen(*index) + 1), USER_NAME), *index);
	user = (char**)find_hlst(nvram_avp,label, 0);
	
	if(user && **user!='\0')
	{		 
		//check if user is configured fo a lan_samba_conf.shares.user variable
		if(getExistNvramVar(nvram_avp, SHARE_USER_LABEL, *user))
		{
			error = SHARE_USER_DEPENDENCY;
			handle_varError(avp, error);
		}		
	}
	destroy_hlst(nvram_avp);
	
	return error;
}



int isDeniedUsername(HLST *avp, HLST *nvram_avp, const char *varName)
{
	char 	**indexList, **Val;
	char	*_dup, *p;
	_dup 		= NULL;

	Val		= (char**)find_hlst(avp, varName, 0);
	indexList	= (char**)find_hlst(nvram_avp, DENIED_USER_LIST, 0);

	if(indexList && (_dup = zstrdup(*indexList)))
	{
		int stop;
		p = strtok(_dup, " ");

		do
		{	
			if(Val && *Val && strcmp(p,*Val) == 0)
				return 1;

		}while(p = strtok(NULL, " "));

		free(_dup);
	}

	if(!strcmp(*Val,"nobody"))
		return 1;
	
	return 0;
}


int checkName(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char *existVar, **val, **ind;
	char **denied;

	/* username is only accepted when a new samba account is created. A user cannot change a user name */
	ind = (char**)find_hlst(avp,"index", 0);
	
	/* check if we are in edit mode */
    	if(ind && (**ind != '\0'))
	{
		//remove user name from list, so the changes will not be saved
		delete_hlst(avp,varName,0);
		return NO_ERROR;
	}

	/* so we are in add mode and we need to make sure that the new samba user is not already existing for one
	   of our system users (asterisk, web etc.) */	

	val = (char**)find_hlst(avp, varName, 0);	
	if(val && **val!='\0')
	{
		/* replace the name with stripped lanks */
		forceAddToList(avp,varName,removeBlanks(*val));
		
		//check if user name is already existing as an other samba user
		if(existVar = getExistNvramVar(nvram_avp, varName, *val))	
		{
			/* check if we found the variable that we have to edit */
			if(!isEditVariable(avp,existVar,varName))
				error = INVALID_NAME_EXISTS;
		}
		//check if user name is already existing as a system user
		if(isDeniedUsername(avp, nvram_avp, varName))
			error = FORBIDDEN_NAME;
	}
	else 
	{
		error = INVALID_NAME;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);
		

	return error;
}


int setId(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	char *nextIndex;
	char **val;
	char **ind;

	ind			= (char**)find_hlst(avp, "index", 0);
	//val 			= (char**)find_hlst(avp, varName, 0);
	nextIndex		= getNextIndex(nvram_avp, LIST_LABEL, TOTAL_ELEMENTS_LABEL);
   
	if(!ind || **ind == '\0')
	{
		if(nextIndex)
		{
			forceAddToList(avp,varName,nextIndex);
		}
		else
		{
			error = INVALID_TOTAL_ELEMENTS;
		}
	}
	else
	{
		addToList(avp,varName,*ind);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return NO_ERROR;
}



HTMLOBJECT *init_htmlVars(){

	int max_size = 4;
	int max_index = max_size - 1;
	int i=0;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);

	p[i].revise 	= setId;
	p[i++].name 	= "lan_samba_user.id";

	p[i].revise 	= checkName;
	p[i++].name 	= "lan_samba_user.name";

	p[i].revise 	= noError;
	p[i++].name 	= "lan_samba_user.password";	


	//sentinel
	p[i>(max_index) ? max_index : i].revise = NULL;
	p[i>(max_index) ? max_index : i].name 	= NULL;


	return p;
}


void free_htmlVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}

void handle_varError(HLST *avp, int ecode){

 
	switch(ecode){
	
	case INVALID_NAME:				addToList(avp, "lan_samba_user.name.error", "1");
									break;
	case INVALID_NAME_EXISTS:		addToList(avp, "lan_samba_user.nameExists.error", "1");
									break;
	case FORBIDDEN_NAME:			addToList(avp, "lan_samba_user.nameForbidden.error", "1");
									break;
	case SHARE_USER_DEPENDENCY:		addToList(avp, "lan_samba_user.shareDependency.error", "1");
									break;
	
	default	:					//UNKNOWN error code, nothing to do
							logger (LOG_WARNING, "UNKNOWN error code: %d: ", ecode);
							return;
	}


	addToList(avp, "error", "1");
    
	return;

}


void *htinit (const char *doc_path, NVCL *nv) {
	STATE *s = ZMALLOC (STATE) ;

	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= "lan/lan_samba_user.html";
	s->htmlvars.total_elements_label 	= TOTAL_ELEMENTS_LABEL;
	s->htmlvars.group_label 			= "lan_samba_user";

	s->nvram  	= nv ;
	s->htpath 	= doc_path ;
	return s ;
}

void htcleanup (void *desc) {
	STATE *s = desc ;

	free_htmlVars(s->htmlvars.objects);
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
	STATE  *s = desc ;
	PARSER   *parser ;
	HLST        *avp ;
	const char *data ;
	char **retVal;
	char *html_file = (char*)argv[1];

	if (argc < 2)
		return ;

	avp = http_args (r);

	if((char**)find_hlst(avp,"save", 0))
	{
		 //store
		if(save_indexHttpVars(s,avp)) {
			html_file = (char*)s->htmlvars.load_on_success;
    			count_types(s, avp, "lan_samba_user");
		}

	}
	else if((char**)find_hlst(avp,"delete", 0))
	{
		//delete
		if(!hasShareDependency(s,avp) && delete_indexHttpVars(s,avp)) {
			html_file = (char*)s->htmlvars.load_on_success;
    			count_types(s, avp, "lan_samba_user");
		}
	}
	else if((char**)find_hlst(avp,"add", 0))
	{
		 //do nothing for add
	}	
	else if((char**)find_hlst(avp,"cancel", 0))
	{
		//load a html file to choose a partition
		html_file = "lan/lan_samba_user.html";
	}
	else
	{
		  load_indexHttpVars(s, avp);
	}
        
        
	handle_global_settings(avp, s->nvram);
	
	parser = open_parser (avp, s->nvram, s->htpath) ;

	if (file_parser (parser, htdocs_p (r, html_file)) == 0)
		return ;

	/* parser */
	run_parser (parser) ;

	avdump (avp); /* debugging */

	/* print result */
	if ((data = data_collect (parser)) != 0)
		body_append (r, data, 0);

	/* clean up */
	file_parser_end (parser);
	close_parser (parser) ;
}



/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

