/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Susanne Goldammer <s.goldammer@maxina.de> - Maxina GmbH,      *
*   Neu-Isenburg, Germany						  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "hlst.h"
#include "http-plugin.h"
#include "parser.h"
#include "nvalloc.h"
#include "plugin-util.h"
#include "logger.h"

/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */


#define NO_PRINTER_FOUND_ERROR		CUSTOM_START_ERROR + 10
#define EMPTY_USERLABEL				CUSTOM_START_ERROR + 20
#define USERLABEL_NOT_EXISTS		CUSTOM_START_ERROR + 30

#define PRINTER_CONNECTED_LABEL		"lan_usb_printer.userEnabled"
#define PRINTER_USER_LABEL			"lan_usb_printer.userlabel"


/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

#if 0
int checkPrinterDev(HLST *avp, HLST *nvram_avp, const char *varName)
{
	char **val;
	int error = NO_ERROR;	

	/* First ensure a correct value for varName */
	ensureCheckState(avp, nvram_avp, varName);
	

	/* If value was "true", the user wants to share the printer. Check whether there is actually a printer connected. */
	val = (char**)find_hlst(avp, varName, 0);	
	if(val && *val && strcmp(*val,"true")==0 )
	{
		printf("**********hasPrinter was true\n");
		val = (char**)find_hlst(nvram_avp, PRINTER_CONNECTED_LABEL, 0);	
		if(!val || !(*val) || !strcmp(*val,"false"))
		{
			printf("**********connected is %s\n", *val);
			error = NO_PRINTER_FOUND_ERROR;		
			forceAddToList(avp,varName,"false");		
		}
	}
	printf("**********error is %d\n", error);

	if(error != NO_ERROR)
		handle_varError(avp, error);		

	return error;

}
#endif

int isExistingUserlabel(HLST *nvram_avp, char *userlabel )
{
	HSRCH 		*lookup;
	char 		*label 		= PRINTER_USER_LABEL;
	const char	**Val		= NULL;
	char		*var 		= NULL;
	
	/* check if there is a lan_usb_storage.partition.mntpoint variable in the nvram which has he same mountPoint */
	lookup = open_hlst_search (nvram_avp);
	while ((Val = (const char**) next_hlst_search (lookup)) != 0) 
	{
		//printf("Val: %s\n", *Val);    
		if(strcasecmp(*Val,userlabel)!=0)
		   continue;

		//printf("Val = mntPoint\n");    
		/* value was equal to the userlabel of the printer_conf, so get the associated variable */
		var = query_key_hlst((void **)Val) ;

		if(strncmp(label, var, strlen(label)) == 0)
		{
			/* we found a variable beginning with label and having the same value as userlabel */			
			return 1;
		}
	
		/* that was not what we're looking for */
		var = NULL;
	}
	close_hlst_search (lookup) ;

	return 0;
}

int checkUserlabel(HLST *avp, HLST *nvram_avp, const char *varName)
{
	/* check whether there is an existing lan_usb_printer.userlabel with the same value */

	int			res				= 0;
	char 		**userlabel 	= (char**)find_hlst(avp, varName, 0);
    	
	if(!userlabel || **userlabel=='\0')
	{
		handle_varError(avp, EMPTY_USERLABEL);
        return EMPTY_USERLABEL;
	}	

	if(!strcmp(*userlabel,"false"))	
		return NO_ERROR;

	res = isExistingUserlabel(nvram_avp, *userlabel);

	/* if not, create an error */	
    if(!res)
	{
		handle_varError(avp, USERLABEL_NOT_EXISTS);
        return USERLABEL_NOT_EXISTS;
	}
	
	return NO_ERROR;
}

HTMLOBJECT *init_htmlVars(){

	int max_size = 2;
	int max_index = max_size - 1;
	int i=0;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);

	p[i].revise 	= checkUserlabel;
	p[i++].name 	= "lan_printer_conf.userlabel";

	//sentinel
	p[i>(max_index) ? max_index : i].revise = NULL;
	p[i>(max_index) ? max_index : i].name 	= NULL;


	return p;
}

void handle_varError(HLST *avp, int ecode){

 
	switch(ecode){
	
	case NO_PRINTER_FOUND_ERROR:	addToList(avp, "lan_usb_printer.noPrinterConnected.error", "1");
									break;
	case USERLABEL_NOT_EXISTS:		addToList(avp, "lan_usb_printer.userlabelNotFound.error", "1");
									break;
	case EMPTY_USERLABEL:			addToList(avp, "lan_usb_printer.userlabelEmpty.error", "1");
									break;
	
	default	:						//UNKNOWN error code, nothing to do
									logger (LOG_WARNING, "UNKNOWN error code: %d: ", ecode);
									return;
	}


	addToList(avp, "error", "1");
    
	return;

}

void free_htmlVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}

void *htinit (const char *doc_path, NVCL *nv) {
    STATE *s = ZMALLOC (STATE) ;

	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= "lan/lan_usb_printer.html";
	//s->htmlvars.total_elements_label 	= TOTAL_ELEMENTS_LABEL;
	//s->htmlvars.group_label 		= "lan_samba_conf.shares";

    s->nvram  =       nv ;
    s->htpath = doc_path ;
    return s ;
}


void htcleanup (void *desc) {
    // STATE *s = desc ;
    free (desc);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
    STATE  *s = desc ;
    PARSER   *parser ;
    HLST        *avp ;
	const char *data ;
	char **retVal;
	char *html_file = (char*)argv[1];

	if (argc < 2)
		return ;

	avp = http_args (r);   

    if (http_fallback (r)) {
        const char *s = rplycode (r, 404);

        body_printf (r, ("<html>"
                         "<head><title>%s</title></head>"
                         "<body><h1>%s</h1>URL not available: %s</body>"
                         "</html>"), s, s, http_request (r));
        return ;
    }

   
    CALL_HTTPVARS_ACTION(s,avp);
    

    handle_global_settings (avp, s->nvram);

    parser = open_parser (avp, s->nvram, s->htpath) ;

    if (file_parser (parser, argv [1]) == 0)
        return ;

    /* parser */
    run_parser (parser) ;

    // avdump (avp); /* debugging */

    /* print result */
    if ((data = data_collect (parser)) != 0)
        body_append (r, data, 0);

    /* clean up */
    file_parser_end (parser);
    close_parser (parser) ;
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
