/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: n/n (D-Link)			                          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <netinet/in.h>

#include "hlst.h"
#include "util.h"
#include "http-plugin.h"
#include "plugin-util.h"
#include "parser.h"
#include "nvalloc.h"
#include "logger.h"


/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */

#define SSID_EMPTY				CUSTOM_START_ERROR + 10
#define INVALID_CHANNEL				CUSTOM_START_ERROR + 20
#define INVALID_CIPHER_SELECTION		CUSTOM_START_ERROR + 30
#define INVALID_CIPHER_A			CUSTOM_START_ERROR + 31
#define INVALID_CIPHER_B			CUSTOM_START_ERROR + 32
#define INVALID_CIPHER_C			CUSTOM_START_ERROR + 33
#define INVALID_CIPHER_D			CUSTOM_START_ERROR + 34
#define INVALID_KEYINTERVALL			CUSTOM_START_ERROR + 40
#define INVALID_SERVERIP			CUSTOM_START_ERROR + 50
#define INVALID_PORT				CUSTOM_START_ERROR + 60
#define INVALID_SECRET				CUSTOM_START_ERROR + 70
#define INVALID_HEX				CUSTOM_START_ERROR + 80
#define INVALID_STRING				CUSTOM_START_ERROR + 90





typedef enum{
	SEC_SETTINGS_NOT_SET,
	SEC_SETTINGS_NONE,
	SEC_SETTINGS_WEP,
	SEC_SETTINGS_WPA
}SEC_SETTINGS;


typedef enum{
	AUTH_TYPE_NOT_SET,
	AUTH_TYPE_OPEN,
	AUTH_TYPE_SHARED,
	AUTH_TYPE_BOTH
}AUTH_TYPE;

typedef enum{
	ENC_TYPE_NOT_SET,
	ENC_TYPE_A,
	ENC_TYPE_B,
	ENC_TYPE_C,
	ENC_TYPE_D
}ENC_TYPE;

typedef enum{
	WPA_TYPE_NOT_SET,
	WPA_TYPE_8021X,
	WPA_TYPE_HEX,
	WPA_TYPE_STRING
}WPA_TYPE;

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

SEC_SETTINGS getSecSetting(HLST *avp)
{
	char **val = (char**)find_hlst(avp, "lan_wlan.security", 0);

	if(val)
	{
		if(!strcmp("none",*val))
			return SEC_SETTINGS_NONE;
		if(!strcmp("wep",*val))
			return SEC_SETTINGS_WEP;
		if(!strcmp("wpa",*val))
			return SEC_SETTINGS_WPA;
	}

	return SEC_SETTINGS_NOT_SET;
}

AUTH_TYPE getAuthType(HLST *avp)
{
	char **val = (char**)find_hlst(avp, "lan_wlan.authtype", 0);

	if(val)
	{
		if(!strcmp("open",*val))
			return AUTH_TYPE_OPEN;
		if(!strcmp("shared",*val))
			return AUTH_TYPE_SHARED;
		if(!strcmp("both",*val))
			return AUTH_TYPE_BOTH;
	}

	return AUTH_TYPE_NOT_SET;
}

ENC_TYPE getEncType(HLST *avp)
{
	char **val = (char**)find_hlst(avp, "lan_wlan.encryptionselection", 0);

	if(val)
	{
		if(!strcmp("A",*val))
			return ENC_TYPE_A;
		if(!strcmp("B",*val))
			return ENC_TYPE_B;
		if(!strcmp("C",*val))
			return ENC_TYPE_C;
		if(!strcmp("D",*val))
			return ENC_TYPE_D;
	}

	return ENC_TYPE_NOT_SET;
}

WPA_TYPE getWpaType(HLST *avp)
{
	char **val = (char**)find_hlst(avp, "lan_wlan.wpatype", 0);

	if(val)
	{
		if(!strcmp("8021x",*val))
			return WPA_TYPE_8021X;
		if(!strcmp("pskhex",*val))
			return WPA_TYPE_HEX;
		if(!strcmp("pskstring",*val))
			return WPA_TYPE_STRING;
	}

	return WPA_TYPE_NOT_SET;
}


int isApEnabled(HLST *avp)
{
	char **val = (char**)find_hlst(avp, "lan_wlan.isAccesspoint", 0);

	if(val && !strcmp(*val, "true"))
	{
		return 1;
	}

	return 0;
}

/*! \brief Set lan_wlan.accesspointEnabled to value of lan_wlan.isAccesspoint

    \param Pointer to list of variables
    \param Pointer to NVRAM list of variables (unused)
    \param Variable name (unused)
*/
int adjustApEnabled(HLST *avp, HLST *nvram_avp, const char *varName)
{
	char **val = (char**)find_hlst(avp, "lan_wlan.isAccesspoint", 0);

	add_to_list(avp, "lan_wlan.accesspointEnabled", *val);

	return 0;
}


int checkSsid(HLST *avp, HLST *nvram_avp, const char *varName){

	int	error = NO_ERROR;

	if(isApEnabled(avp))
	{
		char **val = (char**)find_hlst(avp, varName, 0);

		if(!val || **val=='\0')
			error = SSID_EMPTY;

	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkChannel(HLST *avp, HLST *nvram_avp, const char *varName){

	long 	i;
	char 	*s, *existVar;
	int 	error = NO_ERROR;

	if(!isApEnabled(avp))
		return NO_ERROR;

	i = getVarAsLong(avp, varName);

	if(i<1 || i>13)
	{
			error = INVALID_CHANNEL;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkEnc(HLST *avp, HLST *nvram_avp, const char *varName){

	char **enc, **cipher;
	int error = NO_ERROR;
	int res = 0;

	if((!isApEnabled(avp)) ||
		(getSecSetting(avp) != SEC_SETTINGS_WEP)) 
	   
		return NO_ERROR;


	/* validate key A */
	enc 	= (char**)find_hlst(avp, "lan_wlan.encryption_A", 0);
	cipher 	= (char**)find_hlst(avp, "lan_wlan.cipher_A", 0);

	if(enc && cipher)
	{
		
		if((**enc == '\0') && (getEncType(avp) != ENC_TYPE_A))
			res = 1;
		else if(!strcmp("64",*cipher))
			res = validEnc64(*enc);

		else if(!strcmp("128",*cipher))
			res = validEnc128(*enc);

		else if(!strcmp("256",*cipher))
			res = validEnc256(*enc);
		else
			res = 0;


		if(!res)
		{
			error = INVALID_CIPHER_A;
			handle_varError(avp, error);
		}
			
	}


	/* validate key B */
	enc 	= (char**)find_hlst(avp, "lan_wlan.encryption_B", 0);
	cipher 	= (char**)find_hlst(avp, "lan_wlan.cipher_B", 0);

	if(enc && cipher)
	{
		if((**enc == '\0') && (getEncType(avp) != ENC_TYPE_B))
			res = 1;
		else if(!strcmp("64",*cipher))
			res = validEnc64(*enc);

		else if(!strcmp("128",*cipher))
			res = validEnc128(*enc);

		else if(!strcmp("256",*cipher))
			res = validEnc256(*enc);
		else
			res = 0;


		if(!res)
		{
			error = INVALID_CIPHER_B;
			handle_varError(avp, error);
		}
	}


	/* validate the key C */
	enc 	= (char**)find_hlst(avp, "lan_wlan.encryption_C", 0);
	cipher 	= (char**)find_hlst(avp, "lan_wlan.cipher_C", 0);

	if(enc && cipher)
	{
		if((**enc == '\0') && (getEncType(avp) != ENC_TYPE_C))
			res = 1;
		else if(!strcmp("64",*cipher))
			res = validEnc64(*enc);

		else if(!strcmp("128",*cipher))
			res = validEnc128(*enc);

		else if(!strcmp("256",*cipher))
			res = validEnc256(*enc);
		else
			res = 0;


		if(!res)
		{
			error = INVALID_CIPHER_C;
			handle_varError(avp, error);
		}
	} 

	/* validate the key D */
	enc 	= (char**)find_hlst(avp, "lan_wlan.encryption_D", 0);
	cipher 	= (char**)find_hlst(avp, "lan_wlan.cipher_D", 0);

	if(enc && cipher)
	{
		if((**enc == '\0') && (getEncType(avp) != ENC_TYPE_D))
			res = 1;
		else if(!strcmp("64",*cipher))
			res = validEnc64(*enc);

		else if(!strcmp("128",*cipher))
			res = validEnc128(*enc);

		else if(!strcmp("256",*cipher))
			res = validEnc256(*enc);
		else
			res = 0;


		if(!res)
		{
			error = INVALID_CIPHER_D;
			handle_varError(avp, error);
		}
	} 

	return error;
}


int checkGroupKeyInt(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	char **val;

	if((!isApEnabled(avp)) ||
	   (getSecSetting(avp) != SEC_SETTINGS_WPA))
		return NO_ERROR;

	val = (char**)find_hlst(avp, "lan_wlan.groupkeyinterval", 0);

	if(!val || **val=='\0')
	{
		error = INVALID_KEYINTERVALL;
	}


	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkServerIp(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	char **val;

	if((!isApEnabled(avp)) ||
	   (getSecSetting(avp) != SEC_SETTINGS_WPA) || 
	   (getWpaType(avp) != WPA_TYPE_8021X))
		return NO_ERROR;

	val = (char**)find_hlst(avp, "lan_wlan.serverip", 0);

	if(!val || !validIpAddr(val))
	{
		error = INVALID_SERVERIP;
	}


	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkPort(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	char **val;

	if((!isApEnabled(avp)) ||
	   (getSecSetting(avp) != SEC_SETTINGS_WPA) || 
	   (getWpaType(avp) != WPA_TYPE_8021X))
		return NO_ERROR;

	val = (char**)find_hlst(avp, "lan_wlan.port", 0);

	if(!val || (getVarAsLong(avp, varName) < 1) || (getVarAsLong(avp, varName) > 65535))
	{
		error = INVALID_PORT;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkSecret(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	char **val;

	if((!isApEnabled(avp)) ||
	   (getSecSetting(avp) != SEC_SETTINGS_WPA) || 
	   (getWpaType(avp) != WPA_TYPE_8021X))
		return NO_ERROR;

	val = (char**)find_hlst(avp, "lan_wlan.secret", 0);

	if(!val || **val=='\0')
	{
		error = INVALID_SECRET;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkHex(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	char **val;

	if((!isApEnabled(avp)) ||
	   (getSecSetting(avp) != SEC_SETTINGS_WPA) || 
	   (getWpaType(avp) != WPA_TYPE_HEX))
		return NO_ERROR;

	val = (char**)find_hlst(avp, "lan_wlan.hex", 0);

	if(!val || !validHexOnly(*val))
	{
		error = INVALID_HEX;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkString(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	int len;
	char **val;

	if((!isApEnabled(avp)) ||
	   (getSecSetting(avp) != SEC_SETTINGS_WPA) || 
	   (getWpaType(avp) != WPA_TYPE_STRING))
		return NO_ERROR;

	val = (char**)find_hlst(avp, "lan_wlan.string", 0);

	if(!val || **val=='\0')
	{
		error = INVALID_STRING;
	} else {
		len = strlen(*val);
		if (len<8 || len > 63)
			error = INVALID_STRING;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}



HTMLOBJECT *init_htmlVars(){

	int max_size = 26;
	int i=0;
	int max_index = max_size - 1;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);

	p[i].revise 	= ensureCheckState;
	p[i++].name	= "lan_wlan.isAccesspoint";

	p[i].revise 	= adjustApEnabled;
	p[i++].name	= "lan_wlan.accesspointEnabled";

	p[i].revise 	= checkSsid;
	p[i++].name	= "lan_wlan.ssid";

	p[i].revise 	= checkChannel;
	p[i++].name	= "lan_wlan.channel";

	p[i].revise 	= noError;
	p[i++].name	= "lan_wlan.security";

	//LAN-Security = wep
	p[i].revise 	= noError;
	p[i++].name	= "lan_wlan.authtype";

	p[i].revise 	= checkEnc;
	p[i++].name	= "lan_wlan.encryptionselection";
    
	p[i].revise 	= noError;
	p[i++].name	= "lan_wlan.encryption_A";

	p[i].revise 	= noError;
	p[i++].name	= "lan_wlan.cipher_A";

	p[i].revise 	= noError;
	p[i++].name	= "lan_wlan.encryption_B";

	p[i].revise 	= noError;
	p[i++].name	= "lan_wlan.cipher_B";

	p[i].revise 	= noError;
	p[i++].name	= "lan_wlan.encryption_C";

	p[i].revise 	= noError;
	p[i++].name	= "lan_wlan.cipher_C";

	p[i].revise 	= noError;
	p[i++].name	= "lan_wlan.encryption_D";

	p[i].revise 	= noError;
	p[i++].name	= "lan_wlan.cipher_D";

	//LAN-Security = wpa
	p[i].revise 	= checkGroupKeyInt;
	p[i++].name	= "lan_wlan.groupkeyinterval";

	p[i].revise 	= noError;
	p[i++].name	= "lan_wlan.wpatype";

	p[i].revise 	= checkServerIp;
	p[i++].name	= "lan_wlan.serverip";

	p[i].revise 	= checkPort;
	p[i++].name	= "lan_wlan.port";

	p[i].revise 	= checkSecret;
	p[i++].name	= "lan_wlan.secret";

	p[i].revise 	= noError;
	p[i++].name	= "lan_wlan.wpatype";

	p[i].revise 	= checkHex;
	p[i++].name	= "lan_wlan.hex";

	p[i].revise 	= checkString;
	p[i++].name	= "lan_wlan.string";

	//sentinel
	p[i>max_index ? max_index : i].revise 	= NULL;
	p[i>max_index ? max_index : i].name    	= NULL;

	return p;
}

void free_htmlVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}



void handle_varError(HLST *avp, int ecode){

 
	switch(ecode){

	case SSID_EMPTY:		       	addToList(avp, "lan_wlan.ssid.error", "1"); 
						break;
								
	case INVALID_CHANNEL:			addToList(avp, "lan_wlan.channel.error", "1"); 
						break;

	case INVALID_CIPHER_SELECTION:		addToList(avp, "lan_wlan.encryptionSelection.error", "1"); 
						break;

	case INVALID_CIPHER_A:			addToList(avp, "lan_wlan.encryption_A.error", "1"); 
						break;

	case INVALID_CIPHER_B:			addToList(avp, "lan_wlan.encryption_B.error", "1"); 
						break;

	case INVALID_CIPHER_C:			addToList(avp, "lan_wlan.encryption_C.error", "1"); 
						break;

	case INVALID_CIPHER_D:			addToList(avp, "lan_wlan.encryption_D.error", "1"); 
						break;

	case INVALID_KEYINTERVALL:		addToList(avp, "lan_wlan.groupkeyinterval.error", "1"); 
						break;

	case INVALID_SERVERIP:		  	addToList(avp, "lan_wlan.serverip.error", "1"); 
						break;

	case INVALID_PORT:		  	addToList(avp, "lan_wlan.port.error", "1"); 
						break;

	case INVALID_SECRET:		  	addToList(avp, "lan_wlan.secret.error", "1"); 
						break;

	case INVALID_HEX:  		 	addToList(avp, "lan_wlan.hex.error", "1"); 
						break;

	case INVALID_STRING:		  	addToList(avp, "lan_wlan.string.error", "1"); 
						break;

	default	:				//UNKNOWN error code, nothing to do
						logger (LOG_WARNING, "UNKNOWN error code: %d: ", ecode);
						return;
	}


	addToList(avp, "error", "1");
    
	return;

}




void *htinit (const char *doc_path, NVCL *nv) {
	STATE *s = ZMALLOC (STATE) ;

	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= NULL;
	s->htmlvars.group_label			= "wan_dsl";
	s->htmlvars.total_elements_label	= NULL;

	s->nvram  	= nv ;
	s->htpath 	= doc_path ;
	return s ;
}

void htcleanup (void *desc) {
	STATE *s = desc ;

	free_htmlVars(s->htmlvars.objects);
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
    
	STATE  *s = desc ;
	PARSER   *parser ;
	HLST        *avp ;
	const char *data ;
	char *html_file = (char*)argv[1];

	if (argc < 2)
		return ;

	avp = http_args (r);


	CALL_HTTPVARS_ACTION(s,avp);
        
	handle_global_settings(avp, s->nvram);

	parser = open_parser (avp, s->nvram, s->htpath) ;

	if (file_parser (parser,  htdocs_p (r, html_file)) == 0)
		return ;

	/* parser */
	run_parser (parser) ;

	avdump (avp); /* debugging */

	/* print result */
	if ((data = data_collect (parser)) != 0)
		body_append (r, data, 0);

	/* clean up */
	file_parser_end (parser);
	close_parser (parser) ;
}



/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

