/*
 * Copyright (C) 2006 Maxina GmbH - Jordan Hrycaj
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 * Author: Jordan Hrycaj <jordan@mjh.teddy.net.com>
 *
 * $Id: tvcalc.h 3108 2006-08-15 13:52:57Z jordan $
 */

#ifndef __SHAT_TVCALC_H
#define __SHAT_TVCALC_H

#ifdef TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# ifdef HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

extern void     tval_set_ms (struct timeval *sum, int ms);
extern void     tval_add_ms (struct timeval *sum, int ms);
extern void     tval_nxt_ms (struct timeval *now, int ms);
extern int      tval_cmp_ms (struct timeval *a,   int ms);

extern void     tval_sub_tv (struct timeval *res,  struct timeval *sub);
extern void     tval_add_tv (struct timeval *a,    struct timeval *b);
extern void     tval_nxt_tv (struct timeval *now,  struct timeval *offs);
extern void     tval_tmo_tv (struct timeval *offs, struct timeval *when);
extern int      tval_cmp_tv (struct timeval *a,    struct timeval *b);
extern int      tval_min_tv (struct timeval *a,    struct timeval *b);
extern unsigned tval_eq_tv  (struct timeval *a,    struct timeval *b);

extern int        ms_tmo_tv (struct timeval *when);

#define tval_le_ms(a,b) (tval_cmp_ms (a, b) <= 0)
#define tval_lt_ms(a,b) (tval_cmp_ms (a, b) <  0)
#define tval_ge_ms(a,b) (tval_cmp_ms (a, b) >= 0)
#define tval_gt_ms(a,b) (tval_cmp_ms (a, b) >  0)
#define tval_ne_ms(a,b) (tval_cmp_ms (a, b) != 0)
#define tval_eq_ms(a,b) (tval_cmp_ms (a, b) == 0)

#define tval_le_tv(a,b) (tval_cmp_tv (a, b) <= 0)
#define tval_lt_tv(a,b) (tval_cmp_tv (a, b) <  0)
#define tval_ge_tv(a,b) (tval_cmp_tv (a, b) >= 0)
#define tval_gt_tv(a,b) (tval_cmp_tv (a, b) >  0)
#define tval_ne_tv(a,b) (tval_eq_tv  (a, b) == 0)

#define tval_used(when) ((when)->tv_sec || (when)->tv_usec)  
#define tval_wait(when) (tval_used (when) == 0 || ms_tmo_tv (when) >  0)
#define tval_due(when)  (tval_used (when)      && ms_tmo_tv (when) <= 0)

const char *tval_print (struct timeval*) ;
const char *time_print (time_t) ;

#endif /* __SHAT_TVCALC_H */
