/*
 * Copyright (C) 2006 Maxina GmbH - Jordan Hrycaj
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 * Author: Jordan Hrycaj <jordan@mjh.teddy.net.com>
 *
 * $Id: ctrl.h 3128 2006-09-01 07:17:15Z jordan $
 */

#ifndef __SHAT_CTRL_H
#define __SHAT_CTRL_H
#ifndef DISCARD_CMD_SOCKET

#include <pwd.h>

# define CTRL_REQUEST_LOG           0x004 /* log selector flag */
# define CTRL_REPLY_LOG             0x008 /* bits 0, 1 in nvramlib */

#ifdef __CTRL_PRIVATE
# include <netinet/in.h>
# include "buffer.h"
# define CTRL_MAX_REQUEST_SIZE       2048
# define CTRL_DEFAULT_REPLY_SIZE     6000
# define CTRL_DEFAULT_REQUEST_PATH  "/var/run/ctrl/server"
#endif

typedef struct _ctrl
#ifdef __CTRL_PRIVATE
{
    int                cmd_fd;

    struct sockaddr_un     un;  /* the socket path */
    unsigned        length_un;  /* length */

    size_t         iobuf_size;
    char               *iobuf;  /* response data buffer */
    unsigned            ioinx;  /* append to io puffer */
    char       iobuf_reply_ok;  /* flag that a reply is wanted */
    char        *reply_socket;  /* reply socket path */

    struct {
        IOBUF     request_log;  /* do not print unless approved */
        unsigned    set_noisy;  /* delayed verbosity set up */
        unsigned        noisy;
    } on_leave ;
    
    uid_t                 uid;  /* user id of the sending process */
    gid_t                 gid;  /* group id of the sending process */
    pid_t                 pid;

    struct {
        struct {
            const char *usage;  /* customised message */
            const char *noisy;  /* customised message */
        } msg ;
        unsigned   *noisebits;  /* customised verbosity */
    } custom ;

    /* call back functions */
    unsigned              dim;  /* number of handlers */
    struct _service {
        const char  name [19];  /* service by name */
        char         use_ctrl;  /* ignore fn_state and use CTRL* */
        void        *fn_state;
        void (*fn) (void*,int,const char**);  /* call back hook */
    } service [1] ;
}
#endif
CTRL ;

extern CTRL *ctrl_init          (gid_t,size_t,const char*,unsigned*);
extern CTRL *ctrl_add           (CTRL*,const char*,
                                 void(*)(void*,int,const char**),void*);
extern void  ctrl_handler       (CTRL*);
extern void  ctrl_usage         (void*,int,const char**);
extern void  ctrl_usage_msg     (CTRL*,const char*);
extern void  ctrl_verbosity     (void*,int,const char**);
extern void  ctrl_verbosity_msg (CTRL*,const char*);
extern void  ctrl_resource      (void*,int,const char**);

extern int   ctrl_reply         (CTRL*, const char *format, ...);
extern void  ctrl_shutdown      (CTRL*);
extern void  ctrl_destroy       (CTRL*);
extern int   ctrl_fd            (CTRL*);

#ifndef DONT_USE_RECVMSG
extern uid_t ctrl_uid           (CTRL*);
extern gid_t ctrl_gid           (CTRL*);
extern pid_t ctrl_pid           (CTRL*);
#endif

#endif /* DISCARD_CMD_SOCKET */
#endif /* __SHAT_CTRL_H */
