/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifndef __MHTTPD_LIST_H
#define __MHTTPD_LIST_H

#include "mime-types.h"
#include "nvram-lib.h"

#define PAGE_ACT_SYMBOLS(x) \
    x (Present,          "page is always present") \
    x (ShowByDefault,    "can be switched off via nvram") \
    x (ShowNotByDefault, "can be switched off via nvram") \
    x (HideByDefault,    "can be switched on via nvram")  \
    x (HideNotByDefault, "can be switched on via nvram")

typedef enum _page_act {
    paAbsent = 0            /* page does not exist */
#   define            __pa(x,y) , pa ## x
    PAGE_ACT_SYMBOLS (__pa)
#   undef             __pa
} PAGE_ACT ;

#ifdef __MHTTPD_LIST_PRIVATE
#include "hlst.h"
typedef struct _argv {
    const char      *so ;
    MIMETYPE         mt ;
    PAGE_ACT         pa ;
    const char   *nvena ;
    unsigned       argc ;
    const char *argv [1];
} ARGV ;
#endif

typedef struct _list {
#   ifdef __MHTTPD_LIST_PRIVATE
    HLST *table ;
    NVCL *nvram ;
#   else
    char    any ;
#   endif
} LIST;


extern LIST      *list_open     (NVCL*);
extern void       list_close    (LIST*);
extern void       list_flush    (LIST*);
extern unsigned   list_del_argv (LIST*,const char*);
extern const char*list_get_so   (LIST*,const char*);
extern unsigned   list_get_argv (LIST*,const char*,PAGE_ACT*,const char**,
                                 const char**,MIMETYPE*,int*,const char***);
extern void       list_set_argv (LIST*,PAGE_ACT,const char*,
                                 const char*,MIMETYPE,int,const char**);
extern unsigned   list_used     (LIST*);
extern void       list_print    (LIST*,void(*)(void*,const char*,...),void*);

#endif /* __MHTTPD_LIST_H */
