/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <signal.h>
#include <setjmp.h>
#include <pwd.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#ifdef HAVE_ERRNO_H
# include <errno.h>
#endif
#ifdef HAVE_SYS_SOCKET_H
# include <sys/socket.h>
#endif
#ifdef HAVE_SYS_TIME_H
# include <sys/time.h>
#endif
#ifdef HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif
#ifdef HAVE_SYS_STAT_H
# include <sys/stat.h>
#endif
#ifdef HAVE_FCNTL_H
# include <fcntl.h>
#endif
#ifdef HAVE_CTYPE_H
# include <ctype.h>
#endif

#define _GNU_SOURCE
#include <getopt.h>

#include "do.h"
#include "list.h"
#include "comhandler.h"
#include "settings.h"
#include "varstr.h"
#include "http.h"
#include "http-defaults.h"
#include "plugin-util.h"
#include "nvram-lib.h"
#include "nvalloc.h"
#include "logger.h"

#ifdef HAVE_OPENSSL
#include "ssl.h"
#endif

/* ----------------------------------------------------------------------- *
 * variables and definitions
 * ----------------------------------------------------------------------- */

#ifdef _DEBUG
#define _MAIN_DEBUG
#endif

typedef
struct _desc {
    DO    *loop ;
    COMM  *comm ;
    LIST   *lst ;
    NVCL *nvram ;
    HTTP  *http ;
#   ifdef HAVE_OPENSSL
    CTY    *cty ;
#   endif
} SESSION ;


extern char *optarg;
extern int optind, opterr, optopt;

static jmp_buf go_here;           /* jump buffer for long jumps */

/* ----------------------------------------------------------------------- *
 * Fiddling around with options ...
 * ----------------------------------------------------------------------- */

#define __arg1of4(a,b,c,d)    a
#define __arg2of4(a,b,c,d)    b
#define __arg3of4(a,b,c,d)    c
#define __arg4of4(a,b,c,d)    d
#define __str(x)              # x
#define str(x)                __str (x)

/* I am fed up with option name and option key management using strings
   and letters at several places - so keep them together -- jordan */

#define opt_name(x)         x (__arg1of4)
#define opt_key(x)          x (__arg2of4)
#define opt_argtype(x)      x (__arg3of4)
#define opt_info(x)         x (__arg4of4)

#define OPT_VERSION(x)      x ("version",       'V', 0,           "print version and exit")
#define OPT_VERBOSE(x)      x ("verbose",       'v', 0,           "print extra diagnostic messages")
#define OPT_SYSLOG(x)       x ("syslog",        's', "FACILITY",  "use syslog with the given facility")
#define OPT_NOTIMEPFX(x)    x ("no-timestamp",  'n', 0,           "do not prefix log data by a time stamp")
#define OPT_DETACH(x)       x ("detach",        'd', 0,           "daemon mode, put yourself in the background")
#define OPT_PID_FILE(x)     x ("pid-file",      'p', "FILE",      "save the process pid")
#define OPT_CHROOT(x)       x ("chroot",        'r', "DIR",       "chroot to DIR after initialisation")
#define OPT_GROUPID(x)      x ("groupid",       'g', "GID",       "allow this group to connect and send admin requests")
#define OPT_NO_DEFAULTS(x)  x ("no-defaults",   'D', 0,           "do not load the HTTP default table upon start up")

#ifdef HAVE_OPENSSL
#define OPT_SSL_CACHE(x)    x ("ssl-cache",     'Q', "NUM",       "number of entries in the ssl/tls session cache (default: " str (XXL_SESSION_CACHE) ")")
#endif /* HAVE_OPENSSL */

#ifndef DISCARD_INFO
#define OPT_HELP(x)         x ("help",          'h', 0,           "instead of a manual page")
#endif /* DISCARD_INFO */

#ifdef ENABLE_EXTRA_OPTIONS

# if HTTP_DEFAULT_IP_ADDRESS == 0
#  define _DEF_IP_ADDRESS "all local addresses"
# else
#  if HTTP_DEFAULT_IP_ADDRESS == 0x7f000001
#   define _DEF_IP_ADDRESS "127.0.0.1"
#  else
#   define _DEF_IP_ADDRESS str (HTTP_DEFAULT_IP_ADDRESS)
#  endif
# endif

#ifdef HAVE_OPENSSL
#define OPT_HTTP_NSSL(x)    x ("http-no-ssl",   'Z', 0,           "do not use SSL transport layer encryption")
#endif /* HAVE_OPENSSL */

#define OPT_HTTP_BIND(x)    x ("http-bind",     'B', "ADDRESS",   "address to listen on http requests (default: " _DEF_IP_ADDRESS ")")
#define OPT_HTTP_PORT(x)    x ("http-port",     'P', "PORT",      "port to listen for remote admin access (default: " str (HTTP_DEFAULT_SERVER_PORT) ")")
#define OPT_HTTP_ADM(x)     x ("http-admin",    'L', "VARNAME",   "NVRAM variable that holds the login name for the HTTP basic authentication (default: " str (HTTP_ADMIN_LOGNAME) ")")
#define OPT_HTTP_PWD(x)     x ("http-passwd",   'W', "VARNAME",   "NVRAM variable that holds the password for the HTTP basic authentication (default: " str (HTTP_ADMIN_PASSWD) ")")
#define OPT_HTTP_RLM(x)     x ("http-realm",    'M', "VARNAME",   "NVRAM variable that holds the realm for the HTTP basic authentication (default: " str (HTTP_ADMIN_REALM) ")")

#endif /* ENABLE_EXTRA_OPTIONS */

#define OPT_HTTP_SLOTS(x)   x ("http-slots",    'N', "NUM",       "number of sessions concurrently served (default: " str (HTTP_DEFAULT_CHANNELS) ")")
#define OPT_HTTP_NBA(x)     x ("http-no-basic", 'X', 0,           "do not use HTTP basic authentication")
#define OPT_SO_POOL(x)      x ("so-pool",       'S', "PREFIX",    "Path name prefix for shared objects (default: " str (HTTP_DLPOOL_PFX) ")")
#define OPT_SO_CACHE(x)     x ("so-cache-depth",'C', "NUM",       "Number of slots in the shared objects cache (default: " str (HTTP_DEFAULT_CACHE_DIM) ")")
#define OPT_HT_DOCS(x)      x ("ht-docs",       'H', "PREFIX",    "Path name prefix for http docs (default: "      str (HTTP_HTDOCS_PFX) ")")
#define OPT_NVRAM_HOST(x)   x ("nvram-host",    'A', "ADDRESS",   "address of nvramd")
#define OPT_NVRAM_PORT(x)   x ("nvram-port",    'Y', "PORT",      "port of nvramd")
#define OPT_NULL(x)         x (0,               0 , 0,            0)

#define LONG_OPTION_RECORD(x,y)  {opt_name (x), y, 0, opt_key (x)}
static const
struct option long_options [] = {
    LONG_OPTION_RECORD (OPT_VERSION,      0),
    LONG_OPTION_RECORD (OPT_VERBOSE,      0),
    LONG_OPTION_RECORD (OPT_NOTIMEPFX,    0),
    LONG_OPTION_RECORD (OPT_GROUPID,      1),
    LONG_OPTION_RECORD (OPT_DETACH,       0),
    LONG_OPTION_RECORD (OPT_CHROOT,       1),
    LONG_OPTION_RECORD (OPT_PID_FILE,     1),
    LONG_OPTION_RECORD (OPT_SYSLOG,       1),
    LONG_OPTION_RECORD (OPT_NO_DEFAULTS,  0),
    LONG_OPTION_RECORD (OPT_HTTP_SLOTS,   1),
#   ifdef               OPT_HELP
    LONG_OPTION_RECORD (OPT_HELP,         0),
#   endif
#   ifdef               OPT_HTTP_PORT
    LONG_OPTION_RECORD (OPT_HTTP_PORT,    1),
#   endif
#   ifdef               OPT_HTTP_BIND
    LONG_OPTION_RECORD (OPT_HTTP_BIND,    1),
#   endif
#   ifdef               OPT_HTTP_PWD
    LONG_OPTION_RECORD (OPT_HTTP_PWD,     1),
#   endif
#   ifdef               OPT_HTTP_ADM
    LONG_OPTION_RECORD (OPT_HTTP_ADM,     1),
#   endif
#   ifdef               OPT_HTTP_RLM
    LONG_OPTION_RECORD (OPT_HTTP_RLM,     1),
#   endif
#   ifdef               OPT_HTTP_NSSL
    LONG_OPTION_RECORD (OPT_HTTP_NSSL,    0),
#   endif
#   ifdef               OPT_SSL_CACHE
    LONG_OPTION_RECORD (OPT_SSL_CACHE,    1),
#   endif
    LONG_OPTION_RECORD (OPT_HTTP_NBA,     0),
    LONG_OPTION_RECORD (OPT_SO_POOL,      1),
    LONG_OPTION_RECORD (OPT_SO_CACHE,     1),
    LONG_OPTION_RECORD (OPT_HT_DOCS,      1),
    LONG_OPTION_RECORD (OPT_NVRAM_HOST,   1),
    LONG_OPTION_RECORD (OPT_NVRAM_PORT,   1),
    LONG_OPTION_RECORD (OPT_NULL,         0)
};
#undef LONG_OPTION_RECORD


static const
char short_options [] = {
    opt_key (OPT_VERSION),
    opt_key (OPT_VERBOSE),
    opt_key (OPT_NOTIMEPFX),
    opt_key (OPT_GROUPID),      ':',
    opt_key (OPT_DETACH),
    opt_key (OPT_CHROOT),       ':',
    opt_key (OPT_PID_FILE),     ':',
    opt_key (OPT_SYSLOG),       ':',
    opt_key (OPT_NO_DEFAULTS),
#   ifdef    OPT_HELP
    opt_key (OPT_HELP),
#   endif
#   ifdef    OPT_HTTP_PORT
    opt_key (OPT_HTTP_PORT),    ':',
#   endif
#   ifdef    OPT_HTTP_BIND
    opt_key (OPT_HTTP_BIND),    ':',
#   endif
#   ifdef    OPT_HTTP_ADM
    opt_key (OPT_HTTP_ADM),     ':',
#   endif
#   ifdef    OPT_HTTP_PWD
    opt_key (OPT_HTTP_PWD),     ':',
#   endif
#   ifdef    OPT_HTTP_RLM
    opt_key (OPT_HTTP_RLM),     ':',
#   endif
#   ifdef    OPT_HTTP_NSSL
    opt_key (OPT_HTTP_NSSL),
#   endif
#   ifdef    OPT_SSL_CACHE
    opt_key (OPT_SSL_CACHE),    ':',
#   endif
    opt_key (OPT_HTTP_NBA),
    opt_key (OPT_HTTP_SLOTS),   ':',
    opt_key (OPT_SO_POOL),      ':',
    opt_key (OPT_SO_CACHE),     ':',
    opt_key (OPT_HT_DOCS),      ':',
    opt_key (OPT_NVRAM_HOST),   ':',
    opt_key (OPT_NVRAM_PORT),   ':',
    opt_key (OPT_NULL)
} ;

/* ----------------------------------------------------------------------- *
 * Timeout signal handler
 * ----------------------------------------------------------------------- */
static void s_go_here (int n) { longjmp (go_here, 1); }

/* ----------------------------------------------------------------------- *
 * call back functions
 * ----------------------------------------------------------------------- */

static
void idle (void *dsc) {
    SESSION *s = dsc ;

    if (verbosity & MAIN_LOG_IDLE_LOOPING)
        logger (LOG_INFO, "mhttpd: Idle handler.");

    /* purge outdated channels */
    http_idlecheck (s->http);
}


#if 0
static
void resched (void *dsc, int timeout) {  /* reschedule idle timer */
    SESSION *s = dsc ;

    if (verbosity & MAIN_LOG_IDLE_LOOPING)
        logger (LOG_INFO, "Rescheduling the idle timer: %d.", timeout);

    /* the scheduler wants millisecs while the arp handler knows seconds */
    do_resched (s->loop, timeout * 1000);
}
#endif /* 0 */

/* ----------------------------------------------------------------------- *
 * helpers
 * ----------------------------------------------------------------------- */

static
void cpry (
#   ifdef HAVE_OPENSSL
           int no_ssl
#   else
           void
#   endif
           ) {
    logger (LOG_INFO, "MHTTPD - micro http server");
    logger (LOG_INFO, "Copyright (c) 2006 - D-Link (Europe) Ltd., London, UK");
    logger (LOG_INFO, "Copyright (c) 2006 - Maxina GmbH, Neu-Isenburg, DE");
    logger (LOG_INFO, "Version " VERSION " -- Jordan Hrycaj <jordan@maxina.de>");
#   ifdef HAVE_OPENSSL
    if (no_ssl == 0)
        logger (LOG_INFO, "SSL/TLS support: %s", ssl_version ());
#   endif
}

#ifndef DISCARD_INFO
static
void usage_line (const char indent,
                 const  char *name,
                 char          key,
                 const char   *arg,
                 const char  *info) {
    char longopt [100], format [30] ;
    strcpy (longopt, name) ;
    if (arg) { strcat (longopt, "="); strcat (longopt, arg); }
    sprintf (format, "%%%us-%%c, --%%-18s %%s\n", indent);
    fprintf (stderr, format, "", key, longopt, info);
}
#endif /* DISCARD_INFO */


static
void usage (const char *info, const char *opt) {

#   ifndef DISCARD_INFO
    if (info != 0) {
        fprintf (stderr, "\n%s: ERROR: %s", progname, info);
        if (opt) fprintf (stderr, " (option arg was %s)", opt);
        fprintf (stderr, "\n");
    }

#   define SHOW_OPTION(indent,opt) usage_line (indent,            \
                                               opt_name    (opt), \
                                               opt_key     (opt), \
                                               opt_argtype (opt), \
                                               opt_info    (opt)  )

    fprintf (stderr, "\nUsage: %s [options]\n", progname);

    fputs ("\nGeneral options:", stderr);
    SHOW_OPTION ( 2, OPT_NO_DEFAULTS);
#   ifdef            OPT_HELP
    SHOW_OPTION (18, OPT_HELP);
#   endif

    fputs ("\nLogging options:", stderr);
    SHOW_OPTION ( 2, OPT_VERBOSE);
    SHOW_OPTION (18, OPT_SYSLOG);
    SHOW_OPTION (18, OPT_NOTIMEPFX);

    fputs ("\nProcess control:", stderr);
    SHOW_OPTION ( 2, OPT_DETACH);
    SHOW_OPTION (18, OPT_CHROOT);
    SHOW_OPTION (18, OPT_PID_FILE);
    SHOW_OPTION (18, OPT_GROUPID);

    fputs ("\nHttp server:", stderr);
    SHOW_OPTION ( 6, OPT_HTTP_SLOTS);
#   ifdef            OPT_HTTP_PORT
    SHOW_OPTION (18, OPT_HTTP_PORT);
#   endif
#   ifdef            OPT_HTTP_BIND
    SHOW_OPTION (18, OPT_HTTP_BIND);
#   endif
#   ifdef            OPT_HTTP_ADM
    SHOW_OPTION (18, OPT_HTTP_ADM);
#   endif
#   ifdef            OPT_HTTP_PWD
    SHOW_OPTION (18, OPT_HTTP_PWD);
#   endif
#   ifdef            OPT_HTTP_RLM
    SHOW_OPTION (18, OPT_HTTP_RLM);
#   endif
    SHOW_OPTION (18, OPT_HTTP_NBA);
#   ifdef            OPT_HTTP_NSSL
    SHOW_OPTION (18, OPT_HTTP_NSSL);
#   endif
#   ifdef            OPT_SSL_CACHE
    SHOW_OPTION (18, OPT_SSL_CACHE);
#   endif
    SHOW_OPTION (18, OPT_SO_CACHE);
    SHOW_OPTION (18, OPT_SO_POOL);
    SHOW_OPTION (18, OPT_HT_DOCS);
    
    fputs ("\nNvram server:", stderr);
    SHOW_OPTION ( 5, OPT_NVRAM_HOST);
    SHOW_OPTION (18, OPT_NVRAM_PORT);

#   undef SHOW_OPTION
#   endif /* DISCARD_INFO */

    exit (1);
}


#ifndef DISCARD_INFO
static
void help (void) {
    fputs ("\n", stderr);
    syslog_init (0, 0); /* no timestamps with the introduction */
#   ifdef HAVE_OPENSSL
    cpry (0);
#   else
    cpry ();
#   endif
    fputs ("\n"
           "\n ...\n"
           "",  stderr);
}
#endif /* DISCARD_INFO */



static
ip4addr_t hostip (const char *s) {
    
    extern int h_errno;
    int       e ;
    ip4addr_t ip ;

    struct hostent* h = gethostbyname (s) ;

    if (h != 0) {
        struct in_addr sin_addr;
        sin_addr.s_addr = *(in_addr_t*)h->h_addr;
        return ntohl (sin_addr.s_addr);
    }

    e = h_errno ;
    if ((ip = ntohl (inet_addr (s))) != 0 && ip != -1)
        return ip;
    errno = e ;
    return -1;
}


static
unsigned tcp_port (const char *s) {

    unsigned port ;
    struct servent* h = getservbyname (s, "tcp") ;

    if (h != 0)
        return ntohs (h->s_port) ;

    if (parse_unsigned (& port, s, 0) != 0 && (port & ~0xffff) == 0)
        return port ;
    
    return -1;
}

/* ----------------------------------------------------------------------- *
 * MAIN
 * ----------------------------------------------------------------------- */

#ifndef _MAIN_DEBUG
#undef main /* in case some debugging redirection applied */
#endif

int main (int argc, char *argv[]) {
    const char *t ;
    int n, c, option_inx ;

    char       *new_root =  0 ;
    char      *facility  =  0 ;
    int          timepfx =  1 ;
    int           pid_fd = -1 ;
    int           detach =  0 ;
    int          verbose =  0 ;
    int     dont_defload =  0 ;
    int  dont_basic_auth =  0 ;
#   ifdef HAVE_OPENSSL
    int         dont_ssl = -1 ;
    XXL_INIT         ssl ;
#   endif
    int     idle_timeout =  MAIN_IDLE_TIMEOUT ;
    unsigned     set_gid =  0 ;
    CTRL           *ctrl =  0 ;
    const char   *dlpool = HTTP_DLPOOL_PFX ;
    const char   *htdocs = HTTP_HTDOCS_PFX ;
    const char  *htadmin =  0 ;
    const char *htpasswd =  0 ;
    const char  *htrealm =  0 ;
    ip4addr_t     ipaddr = -1 ;
    unsigned        port = -1 ;
    int listener_default =  0 ;
    unsigned  chan_slots = -1 ; /* use default in http_init */
    unsigned cache_slots = -1 ; /* use default in http_init */
    ip4addr_t   nvserver =  0 ;
    unsigned      nvport =  0 ;
    unsigned       nvtmo = MAIN_NVRPLY_TIMEOUT ;

    SESSION  s ;
    XZERO   (s);
#   ifdef HAVE_OPENSSL
    XZERO (ssl);
#   endif

#   ifdef WITH_DMALLOC
#   define  DMALLOC_LOG "/var/tmp/dmalloc.log"
    unlink (DMALLOC_LOG "~");
    rename (DMALLOC_LOG, DMALLOC_LOG "~");
    dmalloc_debug_setup ("debug=0x4f47d03,log=" DMALLOC_LOG);
    dmalloc_message ("*** %s(%d): dmalloc starts.\n", __FILE__, __LINE__);
#   undef   DMALLOC_LOG
#   endif

    /* progname ::= only the last path segment */
    if ((progname = strrchr (argv [0], '/')) == 0)
        progname = argv [0] ;
    else
        ++ progname ;
    
    while ((c = getopt_long (argc, argv, short_options, 
                             long_options, &option_inx)) != -1)
        switch (c) {
            struct passwd *pwd;
        case opt_key (OPT_HTTP_SLOTS):
            if (parse_unsigned (& chan_slots, optarg, 0) == 0 ||
                (chan_slots & ~0xffff)                   != 0 ||
                (chan_slots)                             == 0)
                usage ("illegal number of channels", optarg);
            continue ;
#       ifdef         OPT_HTTP_PORT
        case opt_key (OPT_HTTP_PORT):
            if (parse_unsigned (& port, optarg, 0) == 0 ||
		(port & ~0xffff) != 0)
                usage ("illegal port number", optarg);
            continue ;
#       endif
#       ifdef         OPT_HTTP_BIND
        case opt_key (OPT_HTTP_BIND):
            if ((ipaddr = hostip (optarg)) == -1)
                usage ("illegal ip addess", optarg);
            continue ;
#       endif
#       ifdef         OPT_HTTP_ADM
        case opt_key (OPT_HTTP_ADM):
            htadmin = optarg ;
            continue ;
#       endif
#       ifdef         OPT_HTTP_PWD
        case opt_key (OPT_HTTP_PWD):
            htpasswd = optarg ;
            continue ;
#       endif
#       ifdef         OPT_HTTP_RLM
        case opt_key (OPT_HTTP_RLM):
            htrealm = optarg ;
            continue ;
#       endif
        case opt_key (OPT_HTTP_NBA):
            dont_basic_auth = 1 ;
            continue ;
#       ifdef         OPT_HTTP_NSSL
        case opt_key (OPT_HTTP_NSSL):
            dont_ssl = 1 ;
            continue ;
#       endif
#       ifdef         OPT_SSL_CACHE
        case opt_key (OPT_SSL_CACHE):
            if (parse_unsigned (& ssl.session_cache, optarg, 0) == 0 ||
                ssl.session_cache > XXL_SESSION_CACHE_MAX)
                usage ("illegal number of sessions, max="
                       str (XXL_SESSION_CACHE_MAX), optarg);
            continue ;
#       endif
        case opt_key (OPT_NVRAM_PORT):
            if (parse_unsigned (& nvport, optarg, 0) == 0 ||
		(nvport & ~0xffff) != 0)
                usage ("illegal port number", optarg);
            continue ;
        case opt_key (OPT_NVRAM_HOST):
            if ((nvserver = hostip (optarg)) == -1)
                usage ("illegal ip addess", optarg);
            continue ;
        case opt_key (OPT_SO_POOL):
            dlpool = optarg ;
            continue ;
        case opt_key (OPT_SO_CACHE):
            if (parse_unsigned (& cache_slots, optarg, 0) == 0 ||
                (cache_slots & ~0xffff)                   != 0 ||
                (cache_slots)                             == 0)
                usage ("illegal number of cache size", optarg);
            continue ;
        case opt_key (OPT_HT_DOCS):
            htdocs = optarg ;
            continue ;
        case opt_key (OPT_NO_DEFAULTS):
            dont_defload = 1 ;
            continue ;
        case opt_key (OPT_GROUPID):
            if ((parse_unsigned (& set_gid, optarg, 0) == 0  ||
                 (pwd = getpwuid (set_gid))            == 0) && 
                (pwd = getpwnam (optarg))              == 0)
                usage ("group name dooes not exist", optarg);
            set_gid = pwd->pw_gid;
            continue ;
        case opt_key (OPT_CHROOT):
            new_root = optarg;
            continue ;
        case opt_key (OPT_NOTIMEPFX):
            timepfx = 0 ;
            continue ;
        case opt_key (OPT_DETACH):
            detach = 1 ;
            continue ;
        case opt_key (OPT_PID_FILE):
            unlink (optarg); /* don't be tricked by a symlink */
            if ((pid_fd = open (optarg, O_CREAT|O_EXCL|O_WRONLY, 0644)) < 0)
                usage ("cannot open or create pid file", optarg);
            continue ;
        case opt_key (OPT_SYSLOG):
            facility = optarg ;
            continue ;
        case opt_key (OPT_VERSION):
#           ifdef HAVE_OPENSSL
            cpry (0);
#           else
            cpry ();
#           endif
            exit (0);
        case opt_key (OPT_VERBOSE):
            verbose = -1 ;
            continue ;
#       ifndef DISCARD_INFO
        case '?':
            usage (0, "?") ; /* no return */
        case opt_key (OPT_HELP):
            help (); /* and usage */
#       endif /* DISCARD_INFO */
        default:
            usage (0,0) ; /* no return */
        }

    if (dont_basic_auth) {
        if (htadmin || htpasswd || htrealm)
            usage ("disabling basic authentication contradicts "
                   "logname/password setting", 0);
    }
    else {
        if (htadmin == 0)
            htadmin = HTTP_ADMIN_LOGNAME ;
        if (htpasswd == 0)
            htpasswd = HTTP_ADMIN_PASSWD ;
        if (htrealm == 0)
            htrealm = HTTP_ADMIN_REALM ;
    }
    
    if (optind < argc) {
        fprintf
            (stderr, "%s: Unexpected non-optional arguments: ", progname);
        while (optind < argc)
            fprintf (stderr, "%s ", argv [optind++]);
        fprintf (stderr, "\n");
        exit (2);
    }

    /* we must use syslog when running in the background */
    if (detach && facility == 0) facility = "daemon" ;

    /* initialise logging subsystem */
    if (facility) {
        if (syslog_init (facility, timepfx) == 0)
            usage ("cannot open the syslog facility", facility);
        /* detach, run in the backgrond */
        if (detach) {
            if (fork()) exit (0);
            setsid();
        }
        freopen ("/dev/null", "r", stdin);
        freopen ("/dev/null", "w", stdout);
        freopen ("/dev/null", "w", stderr);
    }
    else
        syslog_init (0, timepfx);

    /* say hello */
#   ifdef HAVE_OPENSSL
    cpry (dont_ssl == 1);
#   else
    cpry ();
#   endif

    /* record own process id */
    if (pid_fd >= 0) {
        char buf [20];
        sprintf (buf, "%d\n", getpid ());
        write (pid_fd, buf, strlen (buf));
        close (pid_fd);
    }

    /* initialise the server control socket before chroot */
    if ((ctrl = ctrl_init (set_gid, MAIN_CTRL_IOBUF_SIZE,
                           MAIN_CTRL_REQ_PATH, &verbosity)) == 0) {
        logger (LOG_ERR, "Cannot start the communication control handler ");
        exit (2);
    }

    /* change root environment */
    if (new_root && (chroot (new_root) || chdir ("/"))) {
        logger (LOG_ERR, "chroot failed: %s", strerror (errno));
        exit (2);
    }

    /* initialise the action loop */
    s.loop = do_init (idle, &s);

    /* set verbosity */
    if (verbose)
        verbosity = ~ (0);
    else
        verbosity = MAIN_DEFAULT_VERBOSITY ;

    /* initalise the nvram server */
    if ((s.nvram = nvcl_open (0, nvserver, nvport, nvtmo)) == 0) {
        logger (LOG_ERR, "Cannot initialise NVRAM commumication system") ;
        exit (2);
    }

    /* open variables list */
    if ((s.lst = list_open (s.nvram)) == 0) {
        logger (LOG_ERR, "Cannot initialise page registry");
        exit (2);
    }

#   ifdef OPT_HTTP_BIND
    if (ipaddr == -1)
#   endif
        if ((t = get_from_nvram (s.nvram, HTTP_BIND_IP)) ==  0 ||
            (ipaddr = hostip (t))                        == -1) {
            logger (LOG_WARNING, "HTTP listener address is not properly "
                    "configured or not accessable (%s)", HTTP_BIND_IP);
            /* fall back to default */
            listener_default = 1 ;
        }

#   ifdef OPT_HTTP_PORT
    if (port == -1)
#   endif
        if ((t = get_from_nvram (s.nvram, HTTP_BIND_PORT)) ==  0 ||
            (port = tcp_port (t))                          == -1) {
            logger (LOG_WARNING,  "HTTP listener port not properly "
                    "configured or not accessable (%s)", HTTP_BIND_PORT);
            /* fall back to default */
            listener_default = 1 ;
        }

#   ifdef HAVE_OPENSSL

#   ifndef HTTP_DEFAULT_USE_SSL
#   define HTTP_DEFAULT_USE_SSL 0
#   endif

#   ifdef OPT_HTTP_NSSL
    if (dont_ssl == -1) {
#   endif
        if ((t = get_from_nvram (s.nvram, HTTP_BIND_SSLOK)) == 0) {
            logger (LOG_WARNING, "SSL mode for the HTTP server is not "
                    "properly configured or not accessable (%s)",
                    HTTP_BIND_SSLOK);
            /* fall back to default */
            listener_default = 1 ;
        }
        else switch (t [0]) {
#       if HTTP_DEFAULT_USE_SSL
        case  'F':
        case  'f':
        case  '0':
            dont_ssl = 1 ;
            break ;
#       else
        case  'T':
        case  't':
        case  '1':
            dont_ssl = 0 ;
            break ;
#       endif /* HTTP_DEFAULT_USE_SSL */
        default:
            dont_ssl = (HTTP_DEFAULT_USE_SSL == 0);
        }
#   ifdef OPT_HTTP_NSSL
    }
#   endif
#   endif

    if (listener_default) {
        logger (LOG_WARNING, "Using default listener parameters");
        ipaddr   = -1 ;
        port     = -1 ;
#       ifdef HAVE_OPENSSL
        dont_ssl = (HTTP_DEFAULT_USE_SSL == 0);
#       endif
    }
    
#   ifdef HAVE_OPENSSL
    if (dont_ssl == 0 && (s.cty = ssl_create (s.nvram, &ssl)) == 0) {
        logger (LOG_ERR, "Cannot initialise SSL/TLS");
        exit (2);
    }
#   endif

    /* initalise the http server */
    if ((s.http = http_init
         (s.loop, s.lst, chan_slots, cache_slots, ipaddr, port, dlpool,
          htdocs, htadmin, htpasswd, htrealm, s.nvram
#         ifdef HAVE_OPENSSL
          , s.cty
#         endif
             )) == 0) {
        logger (LOG_ERR, "Cannot initialise http commumication system");
        sleep (5); /* in case we are looping */
        exit (2);
    }

    /* add services ... */
    logger (LOG_INFO, "Adding control socket services ...");
    if ((s.comm = comm_init (s.lst, s.http)) == 0) {
        logger (LOG_ERR, "Cannot initialise local commumication system");
        exit (2);
    }
    ctrl = ctrl_add (ctrl, "set",     comm_set,     s.comm);
    ctrl = ctrl_add (ctrl, "del",     comm_del,     s.comm);
    ctrl = ctrl_add (ctrl, "list",    comm_list,    s.comm);
    ctrl = ctrl_add (ctrl, "info",    comm_info,    s.comm);
    ctrl = ctrl_add (ctrl, "restart", comm_restart, s.comm);
    ctrl = ctrl_add (ctrl, "rusage",  ctrl_resource,  ctrl);
    ctrl = ctrl_add (ctrl, "noisy",   ctrl_verbosity, ctrl);
    ctrl = ctrl_add (ctrl, "help",    ctrl_usage,     ctrl);
    s.comm = comm_update (s.comm, ctrl);

    /* initialise http tesource/plugin table */
    if (dont_defload == 0)
        http_load_defaults (s.lst, 0) ;

    if (setjmp (go_here) == 0) {
        signal (SIGINT,  s_go_here);
        signal (SIGTERM, s_go_here);
        signal (SIGPIPE, s_go_here);

        /* accept commands */
        do_action
            (s.loop, ctrl_fd (ctrl), (void(*)(void*))ctrl_handler, ctrl);

        for (;;) {
            n = do_loop (s.loop, idle_timeout);
            if (verbose) 
                logger (LOG_INFO, "s%: do_loop -> %d\n", progname, n);
        }
        /*NOTREACHED*/
    }

#   ifdef _MAIN_DEBUG
    /* release dispatcher */
    do_destroy (s.loop);
    
    /* release the server control socket */
    ctrl_destroy (ctrl) ;

    return 0;
#   else
    exit (0);
#   endif /* _MAIN_DEBUG */
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
