/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <malloc.h>
#include <errno.h>
#include <ctype.h>

#include "list.h"

#define MAKEFILE "Makefile"
#define SO_EXT   ".so"
#define LINKTYPE "-module"
#define TRAILOR  "_la_LDFLAGS"

#define __str(x)              # x
#define str(x)                __str (x)

char *progname;

char *upcase (char *s) {
    static char *buf;
    int n, len = strlen (s);
    if (buf)
        free (buf) ;
    if ((buf = malloc (len + 1)) == 0) {
        fprintf (stderr, "%s: malloc (%u) returned NULL (%s)!\n",
                 progname, n, strerror (errno));
        exit (2) ;
    }
    for (n = 0; n < len; n++)
        buf [n] = toupper (s [n]);
    buf [n] = '\0' ;
    return buf ;
}

const char **append (const char **p, const char *s) {
    char *t ;
    int inx, n;
    
    if (p == 0) {
        n = sizeof (char *) ;
        if ((p = malloc (n)) == 0) goto malloc_out_o_mem;
        p [0] = 0 ;
    }
    if (s == 0) return p ;

    /* find last entry */
    inx = 0; while (p [inx]) inx ++ ;

    /* extend by 1 */
    n = (inx + 2) * sizeof (char *) ;
    if ((p = realloc (p, n)) == 0) {
        fprintf (stderr, "%s: realloc (%u) returned NULL (%s)!\n",
                 progname, n, strerror (errno));
        exit (2) ;
    }

    /* duplicate arg text */
    n = strlen (s) + 1 ;
    if ((t = malloc (n)) == 0) goto malloc_out_o_mem ;
    
    p [inx ++] = memcpy (t, s, n);
    p [inx]    = 0 ;
    return p;

malloc_out_o_mem:
    fprintf (stderr, "%s: malloc (%u) returned NULL (%s)!\n",
             progname, n, strerror (errno));
    exit (2) ;
}

static int code_count ;
char *code_str (void) {
    static char buf [10] ;
    sprintf (buf, "\\x%02x", ++ code_count) ;
    return buf;
}

const char **load_module_list (const char *s) {
    const char **list      = append (0, 0);
    const char linktype [] = LINKTYPE ;
    const char trailor  [] = TRAILOR ;
            
    char buf [1000];
    int inx ;
    FILE *fp ;

    if ((fp = fopen (s, "r")) == 0) {
        fprintf (stderr, "%s: Cannot open %s for reading (%s)!\n",
                 progname, s, strerror (errno));
        exit (2);
    }


    for (;;) {
        int c, n;

        /* expect a string: <file-name>_la_LDFLAGS = -module ... */
        switch (c = fgetc (fp)) {
        case EOF: goto done ;
        default: /* file name does not start with '_' */
            if (isalnum (c)) break ;
            goto new_line ;
        }
        buf [inx = 0] = c ;

        /* parse the rest of the base name */
        for (;;) {
            switch (c = fgetc (fp)) {
            case EOF: goto done ;
            case '_': break ;
            default:
                if (isalnum (c)) break ;
                goto end_basename ;
            }
            if (++ inx >= sizeof (buf) - 2) {
                buf [sizeof (buf) - 2] == '\0' ;
                fprintf (stderr, "%s: module name too long in %s: %s\n",
                         progname, s, buf);
                exit (2) ;
            }
            buf [inx] = c ;
        }
    end_basename:
        buf [++ inx] = '\0' ;

        /* need the extension _la_LDFLAGS */
        if (inx <= sizeof (trailor)) goto new_line ;
        inx -= (sizeof (trailor) - 1);
        if (strcmp (buf + inx, trailor) != 0) goto new_line ;
        buf [inx] = '\0' ;
            
        /* skip spaces */
        do if ((c = fgetc (fp)) == EOF) goto done; while (isspace (c));
        
        /* the character '=' is wanted */
        if (c != '=') goto new_line ;

        /* skip spaces */
        for (;;) {
            if ((c = fgetc (fp)) == EOF) goto done;
            if (isspace (c)) continue ;
            ungetc (c, fp);
            break ;
        }

        /* the string -module wanted */
        for (n = 0; linktype [n] != 0; n ++) {
            switch (c = fgetc (fp)) {
            case EOF: goto done ;
            default:
                if (c != linktype [n]) goto new_line;
            }
        }
        list = append (list, buf);
        // printf ("got it: %s\n", buf);
            
    new_line:
        do
            if ((c = fgetc (fp)) == EOF) goto done ;
        while (c != '\n');
    }
 
done:
    fclose (fp);
    return list;
}


int main (int argc, char **argv) {
    const char **l, **p ;
    
    if ((progname = strrchr (argv [0], '/')) == 0)
        progname = argv [0] ;
    else
        ++ progname ;

    l = load_module_list (MAKEFILE) ;


    p = l ;
    code_count = 0;
    while (*p) {
        printf ("#define _%s_SO \"%s\" /* %s%s */\n",
                upcase ((char*)*p), code_str (), *p, SO_EXT);
        p ++ ;
    }
    printf ("\n");

    
    printf ("/* grep '.*%s *= *%s' %s */\n", TRAILOR, LINKTYPE, MAKEFILE);
    printf ("#define ENCODED_MODULE_LIST(x) \\\n");
    p = l ;
    code_count = 0;
    while (*p) {
        printf ("    x (\"%s%s\", _%s_SO, %d)  \\\n",
                *p, SO_EXT, upcase ((char*)*p), ++ code_count);
        p ++ ;
    }
    printf ("\n\n");


    printf ("/* typedef enum _page_act */\n");
#   define __pa(x,y) printf ("#define _%s \"\\%02x\" /* %s */\n", \
                             upcase (str (x)), (int)pa ## x, y);
    PAGE_ACT_SYMBOLS (__pa)
#   undef             __pa
    printf ("\n");
    
    exit (0);
}
