/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#ifdef HAVE_UNISTD_H
# include <unistd.h>
# include <sys/types.h>
# else
# ifdef HAVE_SYS_TYPES_H
#  include <sys/types.h>
# endif
#endif

#ifdef HAVE_SYS_STAT_H
# include <sys/stat.h>
#endif
#ifdef HAVE_ERRNO_H
# include <errno.h>
#endif
#ifdef HAVE_FCNTL_H
# include <fcntl.h>
#endif

#include "buffer.h"
#include "util.h"
#include "settings.h"
#include "nvalloc.h"
#include "ssl.h"
#include "nvram-lib.h"

#ifndef NVRAM_EXE
#define NVRAM_EXE "nvram"
#endif

#define NVCL_MAX_VALUE_SIZE (MAIN_CTRL_IOBUF_SIZE - 100)

char *pname ;
NVCL *nvcl;


char *malloc_file (size_t *Len, const char *file) {

    char *s ;
    int fd, n, not_stdin = strcmp (file, "-") ;

    if (not_stdin) {
        struct stat st ;

        if ((fd = open (file, O_RDONLY)) < 0) {
            fprintf (stderr, "%s: cannot open file %s: %s",
                    pname, file, strerror (errno));
            exit (2);
        }

        if (fstat (fd, &st) < 0) {
            fprintf (stderr, "%s: cannot stat file %s: %s",
                    pname, file, strerror (errno));
            exit (2);
        }

        if (st.st_size > NVCL_MAX_VALUE_SIZE) {
            fprintf (stderr, "%s: File %s is too large (max %u bytes)",
                    pname, file, NVCL_MAX_VALUE_SIZE);
            exit (2);
        }

        /* add \0 terminator */
        s = (char*)xmalloc (st.st_size + 1);
        
        if ((n = read (fd, s, st.st_size)) < st.st_size) {
            fprintf (stderr, "%s: error reading file %s: %s",
                    pname, file, n < 0 ? strerror (errno) : "short read");
            exit (2);
        }

        close (fd);
    }
    else {
        int left = NVCL_MAX_VALUE_SIZE + 1;
        
        if ((fd = fileno (stdin)) < 0) {
            fprintf (stderr, "%s: cannot read from stdin - "
                     "no file number: %s",
                     pname, strerror (errno));
            exit (2);
        }

        s = (char*)xmalloc (left);

        n = 0 ;
        do {
            int m = read (fd, s + n, left) ;
            if (m <= 0) {
                if (m == 0)
                    break ;
                fprintf (stderr, "%s: error reading from stdin: %s",
                        pname, strerror (errno));
                exit (2);
            }
            n    += m ;
            left -= m ;
        }
        while (left > 0);

        if (n > NVCL_MAX_VALUE_SIZE) {
            fprintf (stderr, "%s: Too much data from stdin (max %u bytes)",
                    pname, NVCL_MAX_VALUE_SIZE);
            exit (2);
        }
    }
    
    s [n] = 0;

    if (Len != 0)
        *Len = n ;

    return s;
}


void nvupload_text (const char  *var,
                    const char  *val,
                    size_t       len,
                    const char *info) {

    nvcl_putcmd    (nvcl, "set");
    nvcl_putavpair (nvcl, var, val, len);

    if (nvcl_rply (nvcl, 0) <= 0)
        fprintf (stderr, "*** %s: NVRAM update error (%s).\n",
                 pname, info);
    else
        fprintf (stderr, "%s: set NVRAM variable %s (%s).\n",
                 pname, var, info);
}


void nvupload_file (const char *var, const char *file) {

    struct stat sb;
    char *val ;
    size_t len ;

    if (stat (file, &sb)) {
        IOBUF b;
        XZERO (b);
		data_printf (&b, "make %s", file);
		system (b.data);
		if (stat (file, &sb)) {
			fprintf (stderr,
                     "*** %s: File %s is missing and I "
                     "cannot generate it.\n",
                     pname, file);
		     exit (2);
		}
		kill_space (&b);
    }

    if ((val = malloc_file (&len, file)) != 0) {
       nvupload_text (var, val, len, file);
       free (val);
    }
}


int main (int argc, char **argv) {
    char      *root  = "" ;
    ip4addr_t server =  0 ;
    uint16_t  port   =  0 ;
    unsigned  tmo_ms = 30000; /* 30s */

    if ((pname = strrchr (*argv, '/')) != 0)
        pname ++ ;
    else
        pname = *argv ;

    if ((nvcl = nvcl_open (root, server, port, tmo_ms)) == 0)
        exit (2);

    nvupload_file (XXL_CERT_NAME,    "mhttpd-cert.der");
    nvupload_file (XXL_KEY_NAME,     "mhttpd-key.der");
    nvupload_file (XXL_DHPARAM_NAME, "mhttpd-dhparam.der");

    nvupload_text (XXL_CTX0SID_NAME,XXL_SID0_CTX, 0,"SSL session [0] context");
    nvupload_text (XXL_CTX1SID_NAME,XXL_SID1_CTX, 0,"SSL session [1] context");

    exit (0);
}

