/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifndef __PARSER_H
#define __PARSER_H

#include "nvram-lib.h"
#include "hlst.h"

#ifdef __PARSER_PRIVATE
#include <stdio.h>
#include "util.h"
#include "http.h"


typedef
struct _xio_file {
    FILE             *fp ;
    const char *filename ;
    int           lineno ;
} XIOFILE ;

typedef
struct _xio_buf {
    IOBUF           *buf ;
    size_t           pos ;
} XIOBUF ;


typedef
struct _p_state {

    int nesting_level ;
    int dont_exec_lvl ;

    int strict_varsyntax:1 ;
    int  delete_on_empty:1 ;
    int      dont_expand:1 ;

    struct {
        int  if_true_pending:1 ;
        int if_false_pending:1 ;
        int     else_pending:1 ;
        int   sort_block_level ;
        struct {
            const char *var ;
            unsigned  count ;
        } loop ;
    } block ;
        
    struct {
        int   stop ;

        enum {
            no_input = 0,
            instream,
            instream_clone,
            instring,
            instring_clone,
        } input_type ;

        union {
            XIOFILE *file ;
            XIOBUF *iobuf ;
        } reader ;
    } in ;

    struct {
        int suspend_output:1 ;

        enum {
            no_output = 0,
            outstream,
            outstream_clone,
            outstring,
            outstring_clone,
            nullout,
        } output_type ;

        union {
            XIOFILE *file ;
            XIOBUF *iobuf ;
        } writer ;
    } out ;

    struct _p_state *next;
} P_STATE ;
#endif


typedef
struct _parser {
#   ifdef __PARSER_PRIVATE
    NVCL      *nvram ;
    P_STATE   *state ;
    HLST        *avp ;
    const char *docf ;
#   else
    char         any ;
#   endif
} PARSER ;


extern PARSER     *open_parser     (HLST*,NVCL*,const char*);
extern void       close_parser     (PARSER*);
extern void         run_parser     (PARSER*);
extern unsigned    file_parser     (PARSER*,const char*);
extern void        file_parser_end (PARSER*);
extern const char *data_collect    (PARSER*);

#ifndef DISCARD_EXTENSIVE_LOGGING
extern int       nvload_verbosity  (NVCL*);
extern int       nvretr_verbosity  (NVCL*);
extern int       nvpacl_verbosity  (NVCL*);
extern int      nvstore_verbosity  (NVCL*);
extern int    nvrestore_verbosity  (NVCL*,int);
#endif

#endif /* __PARSER_H */
