/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: n/n (D-Link)			                          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "hlst.h"
#include "http-plugin.h"
#include "parser.h"
#include "nvalloc.h"
#include "plugin-util.h"
#include "logger.h"

/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */

#define INVALID_NAME			CUSTOM_START_ERROR + 10
#define INVALID_NAME_EXISTS		CUSTOM_START_ERROR + 11
#define INVALID_NUMBER			CUSTOM_START_ERROR + 20
#define INVALID_NUMBER_EXISTS		CUSTOM_START_ERROR + 21
#define MAX_ELEMENTS_EXCEEDED		CUSTOM_START_ERROR + 30

#define INVALID_TOTAL_ELEMENTS		CUSTOM_START_ERROR + 40
#define STANDARD_ACCOUNT_DEPENDENCY	CUSTOM_START_ERROR + 50
#define FAILOVER_ACCOUNT_DEPENDENCY	CUSTOM_START_ERROR + 55
#define PHONE_DIALS_DEPENDENCY		CUSTOM_START_ERROR + 60
#define INVALID_CALLERID_BLOCKING_PRFX	CUSTOM_START_ERROR + 70
#define PHONE_DEVICES_DEPENDENCY       	CUSTOM_START_ERROR + 80

#define ACCOUNT_TYPE			"analog"
#define TOTAL_ELEMENTS_LABEL		"phone_accounts.totalElements"
#define LIST_LABEL			"phone_accounts.list"
#define DEVICE_LIST			"phone_devices.list"

#define STANDARD_ACCOUNT_LABEL		"phone_dials.standardaccount"
#define FAILOVER_ACCOUNT_LABEL		"phone_dials.sipfailover"
#define PHONE_DIALS_PREFIX  		"phone_dials_config.account"
#define PHONE_DEVICES_STDACCOUNT	"phone_devices.stdaccount"


/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

int assertMaxAnalogElements(HLST *avp, HLST *nvram_avp, const char *total_elements_label)
{
	HSRCH 		*lookup;
	int 		total	= 0;
	char		*var 	= NULL;
	char 		*label 	= "phone_accounts.type";
	const char	**Val	= NULL;
	char 		**ind 	= (char**)find_hlst(avp, "index", 0);
    
    
    /* check if we are in edit mode */
    if(ind && (**ind != '\0'))
		return NO_ERROR;

	lookup = open_hlst_search (nvram_avp);
	while ((Val = (const char**) next_hlst_search (lookup)) != 0) 
	{
		if(strcasecmp(*Val,ACCOUNT_TYPE)!=0)
		   continue;

		/* value was equal to ACCOUNT_TYPE, so get the associated variable */
		var = query_key_hlst((void **)Val) ;

		if(strncmp(label, var, strlen(label)) == 0)
		{
			/* we found a variable beginning with label and having the same value as ACCOUNT_TYPE */
			total++;
		}
	
		/* that was not what we're looking for */
		var = NULL;
	}
	close_hlst_search (lookup) ;

       
    if((total == 0))
    {
        return NO_ERROR;
    }
    else
    {
        handle_varError(avp, MAX_ELEMENTS_EXCEEDED);
        return MAX_ELEMENTS_EXCEEDED;
    }


}

int checkName(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char *existVar, **val;

	error = assertMaxAnalogElements(avp,nvram_avp, TOTAL_ELEMENTS_LABEL);
	if(error != NO_ERROR)
		return error;


	val = (char**)find_hlst(avp, varName, 0);
	if(val && **val!='\0')
	{
		//removeBlanks(*val);

		if(existVar = getExistNvramVar(nvram_avp, varName, *val))	
		{
			/* check if we found the variable that we have to edit */
			if(!isEditVariable(avp,existVar,varName))
				error = INVALID_NAME_EXISTS;
		}
	}
	else 
	{
		error = INVALID_NAME;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);
		

	return error;
}

int checkNumber(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char *existVar, **val;
	long total;

	error = assertMaxAnalogElements(avp,nvram_avp, TOTAL_ELEMENTS_LABEL);
	if(error != NO_ERROR)
		return error;

	val = (char**)find_hlst(avp, varName, 0);
	if(!val || **val=='\0' || !validPhoneNumber(val))
	{
		error = INVALID_NUMBER;
	}
	else if(existVar = getExistNvramVar(nvram_avp, varName, *val))	
	{
		/* check if we found the variable that we have to edit */
		if(!isEditVariable(avp,existVar,varName))
			error = INVALID_NUMBER_EXISTS;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int setId(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	char *nextIndex;
	char **val;
	char **ind;

	ind			= (char**)find_hlst(avp, "index", 0);
	//val 			= (char**)find_hlst(avp, varName, 0);
	nextIndex		= getNextIndex(nvram_avp, LIST_LABEL, TOTAL_ELEMENTS_LABEL);
   
	if(!ind || **ind == '\0')
	{
		if(nextIndex)
		{
			forceAddToList(avp,varName,nextIndex);
		}
		else
		{
			error = INVALID_TOTAL_ELEMENTS;
		}
	}
	else
	{
		addToList(avp,varName,*ind);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return NO_ERROR;
}


int initTimeslots(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **val;
	char **device_list;

	//first look if we can find the variable index, if yes, do nothing here
	//if no, then initialize the timeslots with all current devices (phone_devices.list)
	char **ind = (char**)find_hlst(avp, "index", 0);

	/* check if we are in edit mode */
	if(ind && (**ind != '\0'))
	{
		return NO_ERROR;					//found index
	}
	else
	{
		/* TO-DO: copy current device list phone_devices.list" */

		device_list = (char**)find_hlst(nvram_avp, DEVICE_LIST, 0);
		if(device_list && **device_list != '\0')
			forceAddToList(avp,varName,*device_list);	//copy current device_list
		else
			forceAddToList(avp,varName," ");			//save an empty device list for timeslot
	}
	return error;
}

int setType(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	forceAddToList(avp,varName,ACCOUNT_TYPE);
        
	return NO_ERROR;
}

int checkIdBlockingPrefix(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	char **val = (char**)find_hlst(avp, "phone_accounts.block_id", 0);
	
	if(!val || !*val || !**val)
		return NO_ERROR;
	
	val = (char**)find_hlst(avp, varName, 0);
	
	if(!val || !*val || !**val) {
		handle_varError(avp, INVALID_CALLERID_BLOCKING_PRFX);
		return INVALID_CALLERID_BLOCKING_PRFX;
	}

	return NO_ERROR;
}

HTMLOBJECT *init_htmlVars(){

	int max_size = 12;
	int max_index = max_size - 1;
	int i=0;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);

	p[i].revise 	= setId;
	p[i++].name 	= "phone_accounts.id";

	p[i].revise 	= checkName;
	p[i++].name 	= "phone_accounts.name";

	p[i].revise 	= checkNumber;
	p[i++].name 	= "phone_accounts.number";

	p[i].revise 	= setType;
	p[i++].name 	= "phone_accounts.type";

	p[i].revise 	= initTimeslots;
	p[i++].name 	= "phone_accounts.timeslot_0";

	p[i].revise 	= initTimeslots;
	p[i++].name 	= "phone_accounts.timeslot_10";

	p[i].revise 	= initTimeslots;
	p[i++].name 	= "phone_accounts.timeslot_30";

	p[i].revise 	= initTimeslots;
	p[i++].name 	= "phone_accounts.timeslot_45";

	p[i].revise 	= initTimeslots;
	p[i++].name 	= "phone_accounts.timeslot_60";

	p[i].revise 	= checkIdBlockingPrefix;
	p[i++].name 	= "phone_accounts.block_pre";
  
	p[i].revise 	= checkIdBlocking;
	p[i++].name 	= "phone_accounts.block_id";

	//sentinel
	p[i>(max_index) ? max_index : i].revise = NULL;
	p[i>(max_index) ? max_index : i].name 	= NULL;


	return p;
}

int hasAccDependency(STATE  *s,  HLST *avp)
{
  
	char 	**index, **dependency, **failover;
	HLST 	*nvram_avp;
	int	error = NO_ERROR;

	if(!(index = (char**)find_hlst(avp,"index", 0)) || (**index=='\0') )
	{
		return NO_ERROR;
	}

  
	nvram_avp = create_hlst(300, nvram_avp_destroy, NULL);
   
	load_from_nvram(s->nvram , nvram_avp, NULL);

	//check the first dependency with standard account
	dependency = (char**)find_hlst(nvram_avp, STANDARD_ACCOUNT_LABEL, 0);
	if(dependency && *dependency && **dependency)
	{
		if (!strcmp(*dependency, *index)) {
			error = STANDARD_ACCOUNT_DEPENDENCY;
			handle_varError(avp, error);
		}
	}
	
	//check for failover account
	failover = (char**)find_hlst(nvram_avp, FAILOVER_ACCOUNT_LABEL, 0);
	if(failover && *failover && **failover)
	{
		if (!strcmp(*failover, *index)) {
			error = FAILOVER_ACCOUNT_DEPENDENCY;
			handle_varError(avp, error);
		}
	}
	

	//check the seconde dependency with phone dials list
	if(getExistNvramVar(nvram_avp, PHONE_DIALS_PREFIX, *index))
	{
		error = PHONE_DIALS_DEPENDENCY;
		handle_varError(avp, error);
	}

	//check the third dependency with phone devices list
	if(getExistNvramVar(nvram_avp, PHONE_DEVICES_STDACCOUNT, *index))
	{
		error = PHONE_DEVICES_DEPENDENCY;
		handle_varError(avp, error);
	}
	
	
	destroy_hlst(nvram_avp);


	return error;
}


void free_htmlVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}



void handle_varError(HLST *avp, int ecode){

 
	switch(ecode){
	
	case INVALID_NAME:				addToList(avp, "phone_accounts_analog.name.error", "1");
							break;
	case INVALID_NAME_EXISTS:			addToList(avp, "phone_accounts_analog.nameExists.error", "1");
							break;
	case INVALID_NUMBER:				addToList(avp, "phone_accounts_analog.number.error", "1");
							break;
	case INVALID_NUMBER_EXISTS:			addToList(avp, "phone_accounts_analog.numberExists.error", "1");
							break;
	case MAX_ELEMENTS_EXCEEDED:			addToList(avp, "phone_accounts_analog.maxExceeded.error", "1");
							break;
	case STANDARD_ACCOUNT_DEPENDENCY:		addToList(avp, "standard_accout_dependency.error", "1");
							break;
	case FAILOVER_ACCOUNT_DEPENDENCY:		addToList(avp, "standard_failover_dependency.error", "1");
							break;
	case PHONE_DIALS_DEPENDENCY:			addToList(avp, "phone_dials_dependency.error", "1");
							break;
	case INVALID_CALLERID_BLOCKING_PRFX:		addToList(avp, "phone_accounts_analog.block_pre.error", "1");
							break;
	case PHONE_DEVICES_DEPENDENCY:	     		addToList(avp, "phone_devices_dependency.error", "1");
							break;

	default	:					//UNKNOWN error code, nothing to do
							logger (LOG_WARNING, "UNKNOWN error code: %d: ", ecode);
							return;
	}


	addToList(avp, "error", "1");
    
	return;

}


void *htinit (const char *doc_path, NVCL *nv) {
	STATE *s = ZMALLOC (STATE) ;

	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= "phone/phone_accounts.html";
	s->htmlvars.total_elements_label 	= TOTAL_ELEMENTS_LABEL;
	s->htmlvars.group_label 		= "phone_accounts";

	s->nvram  	= nv ;
	s->htpath 	= doc_path ;
	return s ;
}

void htcleanup (void *desc) {
	STATE *s = desc ;

	free_htmlVars(s->htmlvars.objects);
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
	STATE  *s = desc ;
	PARSER   *parser ;
	HLST        *avp ;
	const char *data ;
	char **retVal;
	char *html_file = (char*)argv[1];

	if (argc < 2)
		return ;

	avp = http_args (r);

	if((char**)find_hlst(avp,"save", 0))
	{
		 //store
		if(save_indexHttpVars(s,avp)) {
			html_file = (char*)s->htmlvars.load_on_success;
    			count_types(s, avp, "phone_accounts");
		}

	}
	else if((char**)find_hlst(avp,"delete", 0))
	{
		 //delete
		if(!hasAccDependency(s,avp) && delete_indexHttpVars(s,avp)) {
			html_file = (char*)s->htmlvars.load_on_success;
    			count_types(s, avp, "phone_accounts");
		}
	}
	else if((char**)find_hlst(avp,"add", 0))
	{
		 //do nothing for add
	}
	else
	{
		  load_indexHttpVars(s, avp);
	}
        
        
	handle_global_settings(avp, s->nvram);
	
	parser = open_parser (avp, s->nvram, s->htpath) ;

	if (file_parser (parser, htdocs_p (r, html_file)) == 0)
		return ;

	/* parser */
	run_parser (parser) ;

	avdump (avp); /* debugging */

	/* print result */
	if ((data = data_collect (parser)) != 0)
		body_append (r, data, 0);

	/* clean up */
	file_parser_end (parser);
	close_parser (parser) ;
}



/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

