/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: n/n (D-Link)			                          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "hlst.h"
#include "http-plugin.h"
#include "parser.h"
#include "nvalloc.h"
#include "plugin-util.h"
#include "logger.h"

/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */


#define USERNAME_EMPTY 			CUSTOM_START_ERROR + 10
#define PASSWORD_EMPTY			CUSTOM_START_ERROR + 20
#define PASSWORD_NONEQUAL		CUSTOM_START_ERROR + 21
#define INVALID_NAME			CUSTOM_START_ERROR + 30
#define INVALID_NAME_EXISTS		CUSTOM_START_ERROR + 31
#define INVALID_NUMBER			CUSTOM_START_ERROR + 40
#define INVALID_NUMBER_EXISTS		CUSTOM_START_ERROR + 41
#define INVALID_SERVER			CUSTOM_START_ERROR + 50
#define INVALID_STUN_SERVER		CUSTOM_START_ERROR + 60


#define INVALID_TOTAL_ELEMENTS		CUSTOM_START_ERROR + 70
#define STANDARD_ACCOUNT_DEPENDENCY	CUSTOM_START_ERROR + 80
#define PHONE_DIALS_DEPENDENCY		CUSTOM_START_ERROR + 90
#define PHONE_DEVICES_DEPENDENCY       	CUSTOM_START_ERROR + 100

#define ACCOUNT_TYPE			"sip"
#define TOTAL_ELEMENTS_LABEL		"phone_accounts.totalElements"
#define LIST_LABEL			"phone_accounts.list"
#define DEVICE_LIST			"phone_devices.list"
#define STANDARD_ACCOUNT_LABEL		"phone_dials.standardaccount"
#define PHONE_DIALS_PREFIX  		"phone_dials_config.account"
#define PHONE_DEVICES_STDACCOUNT	"phone_devices.stdaccount"

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */


int checkName(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char *existVar, **val = (char**)find_hlst(avp, varName, 0);

	if(val && **val!='\0')
	{
		//removeBlanks(*val);

		if(existVar = getExistNvramVar(nvram_avp, varName, *val))	
		{
			/* check if we found the variable that we have to edit */
			if(!isEditVariable(avp,existVar,varName))
				error = INVALID_NAME_EXISTS;
		}
	}
	else
	{
		error = INVALID_NAME;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}


int checkUsername(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **hasAccess;
	char **val = (char**)find_hlst(avp, varName, 0);
	
	if(!val || (**val=='\0'))
		error = USERNAME_EMPTY;


	if(error != NO_ERROR)
		handle_varError(avp, error);


	return error;
}


int checkPassword(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **passwd1;
	char **passwd2;
	int len1 = 0;
	int len2 = 0;

	passwd1 = (char**)find_hlst(avp, varName, 0);
   
	if((len1=strlen(*passwd1))==0)
		error = PASSWORD_EMPTY;

	passwd2 = (char**)find_hlst(avp, "phone_accounts.password2", 0);
	
	if(*passwd2 && (len2=strlen(*passwd2))==0)
		error = PASSWORD_EMPTY;

	if(len1 != len2 || (strcmp(*passwd1,*passwd2)))
		error = PASSWORD_NONEQUAL;

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}


int checkNumber(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char *existVar, **val = (char**)find_hlst(avp, varName, 0);

	if(!val || **val=='\0' || !validPhoneNumber(val))
	{
		error = INVALID_NUMBER;
	}
	else if(existVar = getExistNvramVar(nvram_avp, varName, *val))	
	{
		/* check if we found the variable that we have to edit */
		if(!isEditVariable(avp,existVar,varName))
			error = INVALID_NUMBER_EXISTS;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}


int checkServer(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **val = (char**)find_hlst(avp, varName, 0);

	if(!val || **val=='\0')
	{
		error = INVALID_SERVER;
	}
   
	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkStun(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **val = (char**)find_hlst(avp, varName, 0);

	if(!val || strcmp("yes", *val)){
		/* we dont need to consider this variable */
		delete_hlst(avp, "phone_accounts.stun_server",0);
		add_to_list(avp, varName, "no");
		return NO_ERROR;
	}

	val = (char**)find_hlst(avp, "phone_accounts.stun_server", 0);

	if(!val || **val=='\0')	{
		error = INVALID_STUN_SERVER;
		handle_varError(avp, error);
	}

	return error;
}


int setId(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	char *nextIndex;
	char **val;
	char **ind;

	ind			= (char**)find_hlst(avp, "index", 0);
	//val 			= (char**)find_hlst(avp, varName, 0);
	nextIndex		= getNextIndex(nvram_avp, LIST_LABEL, TOTAL_ELEMENTS_LABEL);
   
	if(!ind || **ind == '\0')
	{
		if(nextIndex)
		{
                        forceAddToList(avp,varName,nextIndex);
		}
		else
		{
			error = INVALID_TOTAL_ELEMENTS;
		}
	}
	else
	{
		addToList(avp,varName,*ind);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return NO_ERROR;
}

int initTimeslots(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **val;
	char **device_list;

	//first look if we can find the variable index, if yes, do nothing here
	//if no, then initialize the timeslots with all current devices (phone_devices.list)
	char **ind = (char**)find_hlst(avp, "index", 0);

	/* check if we are in edit mode */
	if(ind && (**ind != '\0'))
	{
		return NO_ERROR;					//found index
	}
	else
	{
		/* TO-DO: copy current device list phone_devices.list" */

		device_list = (char**)find_hlst(nvram_avp, DEVICE_LIST, 0);
		if(device_list && **device_list != '\0')
			forceAddToList(avp,varName,*device_list);	//copy current device_list
		else
			forceAddToList(avp,varName," ");			//save an empty device list for timeslot
	}
	return error;
}


int setType(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	forceAddToList(avp,varName,ACCOUNT_TYPE);
	
	return NO_ERROR;
}



HTMLOBJECT *init_htmlVars(){

	int max_size = 15;
	int max_index = max_size - 1;
	int i=0;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);

	p[i].revise 	= setId;
	p[i++].name 	= "phone_accounts.id";

	p[i].revise 	= checkName;
	p[i++].name 	= "phone_accounts.name";

	p[i].revise 	= checkNumber;
	p[i++].name 	= "phone_accounts.number";

	p[i].revise 	= setType;
	p[i++].name 	= "phone_accounts.type";
  
	p[i].revise 	= checkServer;
	p[i++].name 	= "phone_accounts.server";
  
	p[i].revise 	= checkUsername;
	p[i++].name 	= "phone_accounts.username";
  
	p[i].revise 	= checkPassword;
	p[i++].name	= "phone_accounts.password";

	p[i].revise 	= initTimeslots;
	p[i++].name 	= "phone_accounts.timeslot_0";

	p[i].revise 	= initTimeslots;
	p[i++].name 	= "phone_accounts.timeslot_10";

	p[i].revise 	= initTimeslots;
	p[i++].name 	= "phone_accounts.timeslot_30";

	p[i].revise 	= initTimeslots;
	p[i++].name 	= "phone_accounts.timeslot_45";

	p[i].revise 	= initTimeslots;
	p[i++].name 	= "phone_accounts.timeslot_60";

	p[i].revise 	= checkStun;
	p[i++].name 	= "phone_accounts.use_stun";

	p[i].revise 	= noError;
	p[i++].name 	= "phone_accounts.stun_server";


	//sentinel
	p[i>(max_index) ? max_index : i].revise	= NULL;
	p[i>(max_index) ? max_index : i].name 	= NULL;


	return p;
}

void free_htmlVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}

int hasAccDependency(STATE  *s,  HLST *avp)
{
  
	char 	**index, **dependency;
	HLST 	*nvram_avp;
	int		error = NO_ERROR;

	if(!(index = (char**)find_hlst(avp,"index", 0)) || (**index=='\0') )
	{
		return error;
	}

  
	nvram_avp = create_hlst(300, nvram_avp_destroy, NULL);
   
	load_from_nvram(s->nvram , nvram_avp, NULL);

	//check the first dependency with standard account
	dependency = (char**)find_hlst(nvram_avp, STANDARD_ACCOUNT_LABEL, 0);
	if(dependency && *dependency && **dependency)
	{
		if (!strcmp(*dependency, *index)) {
			error = STANDARD_ACCOUNT_DEPENDENCY;
			handle_varError(avp, error);
		}
	}
	
	//check the seconde dependency with phone dials list
	if(getExistNvramVar(nvram_avp, PHONE_DIALS_PREFIX, *index))
	{
		error = PHONE_DIALS_DEPENDENCY;
		handle_varError(avp, error);
	}

	//check the third dependency with phone devices list
	if(getExistNvramVar(nvram_avp, PHONE_DEVICES_STDACCOUNT, *index))
	{
		error = PHONE_DEVICES_DEPENDENCY;
		handle_varError(avp, error);
	}

	
	
	destroy_hlst(nvram_avp);


	return error;
}



void handle_varError(HLST *avp, int ecode){

 
	switch(ecode){
	case INVALID_NAME:			addToList(avp, "phone_accounts_sip.name.error", "1");
						break;

	case INVALID_NAME_EXISTS:		addToList(avp, "phone_accounts_sip.nameExists.error", "1");
						break;

	case INVALID_NUMBER:			addToList(avp, "phone_accounts_sip.number.error", "1");
						break;

	case INVALID_NUMBER_EXISTS:		addToList(avp, "phone_accounts_sip.numberExists.error", "1");
						break;

	case INVALID_SERVER:			addToList(avp, "phone_accounts_sip.server.error", "1");
						break;

	case INVALID_STUN_SERVER:		addToList(avp, "phone_accounts_sip.stun_server.error", "1");
						break;

	case USERNAME_EMPTY:			addToList(avp, "phone_accounts_sip.username.error", "1");
						break;

	case PASSWORD_EMPTY:			addToList(avp, "phone_accounts_sip.password.error", "1");
						break;

	case PASSWORD_NONEQUAL:			addToList(avp, "phone_accounts_sip.password.error", "1");
						break;

	case STANDARD_ACCOUNT_DEPENDENCY:	addToList(avp, "standard_accout_dependency.error", "1");
						break;

	case PHONE_DIALS_DEPENDENCY:		addToList(avp, "phone_dials_dependency.error", "1");
						break;

	case PHONE_DEVICES_DEPENDENCY:	       	addToList(avp, "phone_devices_dependency.error", "1");
						break;
									

	default	:				//UNKNOWN error code, nothing to do
						logger (LOG_WARNING, "UNKNOWN error code: %d: ", ecode);
						return;
	}


	addToList(avp, "error", "1");
    
	return;

}



void *htinit (const char *doc_path, NVCL *nv) {
	STATE *s = ZMALLOC (STATE) ;

	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= "phone/phone_accounts.html";
	s->htmlvars.total_elements_label 	= TOTAL_ELEMENTS_LABEL;
	s->htmlvars.group_label 		= "phone_accounts";

	s->nvram  	= nv ;
	s->htpath 	= doc_path ;

	return s ;
}

void htcleanup (void *desc) {
	STATE *s = desc ;

	free_htmlVars(s->htmlvars.objects);
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
	STATE  *s = desc ;
	PARSER   *parser ;
	HLST        *avp ;
	const char *data ;
	char **retVal;
	char *html_file;

	if (argc < 2)
		return ;

	html_file = (char*)argv[1];

	avp = http_args (r);

	if((char**)find_hlst(avp,"save", 0))
	{
		 //store
		if(save_indexHttpVars(s,avp)) {
			html_file = (char*)s->htmlvars.load_on_success;
    			count_types(s, avp, "phone_accounts");
		}

	}
	else if((char**)find_hlst(avp,"delete", 0))
	{
		 //delete
		if(!hasAccDependency(s,avp) && delete_indexHttpVars(s,avp)) {
			html_file = (char*)s->htmlvars.load_on_success;
    			count_types(s, avp, "phone_accounts");
		}
	}
	else if((char**)find_hlst(avp,"add", 0))
	{
		 //do nothing for add, just do not call load_indexHttpVars
	}
	else
	{
		  load_indexHttpVars(s, avp);
	}
	
	
        
	handle_global_settings(avp, s->nvram);
	
	parser = open_parser (avp, s->nvram, s->htpath) ;

	if (file_parser (parser, htdocs_p (r, html_file)) == 0)
		return ;

	/* parser */
	run_parser (parser) ;

	avdump (avp); /* debugging */

	/* print result */
	if ((data = data_collect (parser)) != 0)
		body_append (r, data, 0);

	/* clean up */
	file_parser_end (parser);
	close_parser (parser) ;
}


/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

