/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: n/n (D-Link)			                          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "hlst.h"
#include "http-plugin.h"
#include "parser.h"
#include "nvalloc.h"
#include "plugin-util.h"
#include "logger.h"

/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */

#define INVALID_RULENAME		 	CUSTOM_START_ERROR + 10

#define INVALID_START_PORT			CUSTOM_START_ERROR + 30
#define INVALID_END_PORT			CUSTOM_START_ERROR + 31
#define INVALID_PORT_RANGE			CUSTOM_START_ERROR + 32

#define INVALID_PORT				CUSTOM_START_ERROR + 40
#define INVALID_PORTMAP				CUSTOM_START_ERROR + 41
#define INVALID_PORTMAP_TOO_BIG			CUSTOM_START_ERROR + 42


/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

char* getDeviceList4Slot(HLST *avp, const char *timeslot, char* buf, int buf_size)
{
	const char 	**Val;
	char		*var = NULL;
	int 		ret	= 0;
	int 		len = 0;
	char 		/**buf = NULL,*/ *newBuf=NULL;
	HSRCH 		*lookup = open_hlst_search (avp);
    

	while ((Val = (const char**) next_hlst_search (lookup)) != 0) 
	{
        var = query_key_hlst((void **)Val) ;	//get variable-name

		if(strncmp(var, timeslot, strlen(timeslot)) == 0)
		{
			/* we found a variable with the beginning of the given timeslot */
			
			newBuf = strchr(var,'.')+1;
			len = strlen(buf) + strlen(newBuf)+ 1;
			if(len >= buf_size)
				break;

			//buf = (char*)realloc(buf,len);
            buf = strcat(buf,newBuf);
            buf = strcat(buf," ");

			/* finally delete variable from avp, because we don't need it anymore */
			delete_hlst(avp,var,0);
 		}

		/* that was not what we're looking for */
		var = NULL;
	}

	close_hlst_search (lookup) ;

	return NULL;
}

int getDevices4Slot(HLST *avp, const char *timeslot)
{
	const char 	**Val;
	char		val[100],tmp[100];
	char		*var = NULL;
	char 		*buf, *newVar;
	HSRCH 		*lookup = open_hlst_search (avp);
   
	while ((Val = (const char**) next_hlst_search (lookup)) != 0) 
	{
        var = query_key_hlst((void **)Val) ;	//get variable-name
        
		if(strncmp(var, timeslot, strlen(timeslot)) == 0)
		{
			/* we found a variable with the beginning of the given timeslot */
			/* now look for the ids of the devices which should ring for this timeslot */

			strncpy(val,*Val,100);			//copy value for strtok
			val[99] = '\0';				//to make sure.
			buf = strtok(val," ");			//device ids are saved in the format: 1 2 4 6 3 

			do
			{
				if(buf && *buf != '\0')
				{
					//in html file phone_calls_edit.html we have a checkbox-matrix for each timeslot and 
					//phone device
					//these variables are in the format e.g. timeslot_0.4 where the 4 is the id of the device
					//now here we wanna create these variables timeslot_x.id for each existing phone device id in the 
					//timeslot variable of the account: means: phone_accounts.timeslot_0 = 1 3 5 let us create
					//the variables timeslot_0.1 , timeslot_0.3 and timeslot_0.5.
					//Only checkboxes which have a name like one of the defined variables will be checked 

					tmp[0] = '\0';						//reset buffer
					strcpy(tmp,strchr(timeslot,'.')+1);			//add prefix 
					strcat(tmp,".");					//add '.'
					strcat(tmp,buf);					//copy device variable / id
					newVar = (char*)zmalloc(strlen(tmp)+1);
					strcpy(newVar,tmp);
					addToList(avp,newVar,"true");	//value is true to check the checkbox
				}

			}while(buf = strtok(NULL," "));

			//finished with this timeslot
			break;
			
 		}

		/* that was not what we're looking for */
		var = NULL;
	}

	delete_hlst(avp,timeslot,0);

	close_hlst_search (lookup) ;

	return NO_ERROR;
}


int setTimeslots(HLST *avp, HLST *nvram_avp, const char *varName)
{
	const int size = 100;
	char buf[500];
	buf[0] = '\0';

	//now search avp for any entry beginning with "timeslot_x"
	getDeviceList4Slot(avp,strchr(varName,'.')+1,buf,500);
	
	//save buf for varName
	if(*buf != '\0' )
		addToList(avp,varName,buf);
	else
		addToList(avp,varName," ");

	return NO_ERROR;
}

int getTimeslots(HLST *avp)
{
	//now search avp for any entry beginning with "timeslot_x"
	getDevices4Slot(avp,"phone_accounts.timeslot_0");
	getDevices4Slot(avp,"phone_accounts.timeslot_10");
	getDevices4Slot(avp,"phone_accounts.timeslot_30");
	getDevices4Slot(avp,"phone_accounts.timeslot_45");
	getDevices4Slot(avp,"phone_accounts.timeslot_60");
	
	return NO_ERROR;
}

HTMLOBJECT *init_htmlVars(){
	int max_size = 7;
	int i=0;
	int max_index = max_size - 1;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);

	p[i].revise 	= noError;					//dont delete phone_accounts_analog, phone_accounts_isdn, phone_accounts
	p[i++].name	= "phone_accounts.id";

	p[i].revise 	= setTimeslots;
	p[i++].name	= "phone_accounts.timeslot_0";

	p[i].revise 	= setTimeslots;
	p[i++].name	= "phone_accounts.timeslot_10";

	p[i].revise 	= setTimeslots;
	p[i++].name	= "phone_accounts.timeslot_30";

	p[i].revise 	= setTimeslots;
	p[i++].name	= "phone_accounts.timeslot_45";

	p[i].revise 	= setTimeslots;
	p[i++].name	= "phone_accounts.timeslot_60";

	
	//sentinel
	p[i>max_index ? max_index : i].revise 	= NULL;
	p[i>max_index ? max_index : i].name    	= NULL;

	return p;
}



void free_htmlVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}

void handle_varError(HLST *avp, int ecode){

 
	switch(ecode){
	default	:				//UNKNOWN error code, nothing to do
						logger (LOG_WARNING, "UNKNOWN error code: %d: ", ecode);
						return;
	}

	addToList(avp, "error", "1");
    
	return;

}

void *htinit (const char *doc_path, NVCL *nv) {
	STATE *s = ZMALLOC (STATE) ;

	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= "phone/phone_calls_choose.html";
	s->htmlvars.total_elements_label 	= NULL; //"phone_accounts.totalElements";
	s->htmlvars.group_label 		= "phone_calls";


	s->nvram  	= nv ;
	s->htpath 	= doc_path ;

	return s ;
}

void htcleanup (void *desc) {
	STATE *s = desc ;

	free_htmlVars(s->htmlvars.objects);
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
	STATE  *s = desc ;
	PARSER   *parser ;
	HLST        *avp ;
	const char *data ;
	char **retVal;
	char *html_file;

	if (argc < 2)
		return ;

	html_file = (char*)argv[1];

	avp = http_args (r);

	if((char**)find_hlst(avp,"save", 0))
	{
		 //store
		if(save_indexHttpVars(s,avp))
			html_file = (char*)s->htmlvars.load_on_success ;

	}
	else if((char**)find_hlst(avp,"delete", 0))
	{
		//reorder, delete
		if(delete_indexHttpVars(s,avp))
			html_file = (char*)s->htmlvars.load_on_success ;
	}
	else if((char**)find_hlst(avp,"add", 0))
	{
		 //do nothing for add, just do not call load_indexHttpVars
	}
	else
	{
		  load_indexHttpVars(s, avp);
		  getTimeslots(avp);
	}
	
	
        
	handle_global_settings(avp, s->nvram);
	
	parser = open_parser (avp, s->nvram, s->htpath) ;

	if (file_parser (parser, htdocs_p (r, html_file)) == 0)
		return ;

	/* parser */
	run_parser (parser) ;

	avdump (avp); /* debugging */

	/* print result */
	if ((data = data_collect (parser)) != 0)
		body_append (r, data, 0);

	/* clean up */
	file_parser_end (parser);
	close_parser (parser) ;
}

