/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Henry Margies <h.margies@maxina.de> - Maxina GmbH, Neu-       *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "hlst.h"
#include "util.h"
#include "http-plugin.h"
#include "parser.h"
#include "nvalloc.h"
#include "plugin-util.h"



static void add_cdr_value(HLST* avp, int id, const char* name, const char* val)
{
	char buffer[1024];

	snprintf(buffer, sizeof(buffer), "%s.%d",name,id);
	add_to_list(avp, buffer, val);
}


static int read_cdr_entry(HLST* avp, FILE* cdr, int *id)
{
	char buf[1024];
	char *start, *end;
	int  i;
	int ret;
	int calltime; 
	int minutes; 
	int seconds;

	start = end = fgets(buf, sizeof(buf), cdr);
	if (!start)
		return 0;
	
	for (i=0; i<16; i++) {

		for(;;) {
			end = strchr(end+1, ',');
			if (!end)
				return 1;

			if (*start != '"')
				break;

			if (*(end-1) == '"')
				break;
		}

		*end = '\0';

		/* remove " */
		if (*start == '"')
			start++;

		if (*(end-1) == '"')
			*(end-1) = '\0';
		
		switch (i) {
			case 1:
				add_cdr_value(avp, *id, "source", start);
				break;
			case 2:
				add_cdr_value(avp, *id, "dest", start);
				break;

			case 3:
				if ( 	!strcmp(start, "analog-in") ||
					!strcmp(start, "isdn-in")   ||
					!strcmp(start, "sip-in")) {
					add_cdr_value(avp, *id, "direction", "in");
					add_cdr_value(avp, *id, "status", "ANSWERED");
				} else if (!strncmp(start, "account-", 8)) {
					add_cdr_value(avp, *id, "direction", "out");
					add_cdr_value(avp, *id, "status", "ANSWERED");
				} else
						return 1;
			case 4:
				add_cdr_value(avp, *id, "clid", start);
				break;
			case 9:
				add_cdr_value(avp, *id, "time_from", start);
				break;
			case 11:
				add_cdr_value(avp, *id, "time_to", start);
				break;
			case 13: 
				calltime = atoi(start); //umrechnen von sekunden in min:sec
				seconds = calltime % 60;
				calltime = calltime - seconds;
				minutes = calltime / 60;
				sprintf(start, "%02d:%02d", minutes, seconds);
				add_cdr_value(avp, *id, "cti", start);
				(*id)++;
				break;
			/*
			case 14:
				add_cdr_value(avp, *id, "status", start);
				(*id)++;
				break;
			*/
		}

		*end = ',';
		start = end+1;
	}
	return 1;
}

static void fill_call_details(HLST* avp)
{
	FILE* cdr;
	int   counter = 1;


	cdr = fopen("/var/log/asterisk/cdr-csv/Master.csv", "r");
	if (!cdr)
		cdr = fopen("Master.csv", "r");
	
	if (!cdr)
		return;

	while( read_cdr_entry(avp, cdr, &counter) >0 );

	close(cdr);
}
		
static void reset_call_details(NVCL* nvram)
{
	add_to_nvram(nvram, "phone_cdr.reset", "true");
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) 
{
	STATE* 		s = desc;
	HLST*		avp;

	if (argc < 2)
		return ;

	/* get POST variables */
	avp = http_args(r);

	handle_global_settings(avp, s->nvram);

	if (get_from_list(avp, "Reset")) {
		/* reset list */
		reset_call_details(s->nvram);
	} else {
		/* list call detail records */
		fill_call_details(avp);
	}
	
	/* create next html page */
	send_html_page(avp, s, r, argv[1]);

}

void *htinit (const char *doc_path, NVCL *nv) 
{
	STATE *s = ZMALLOC(STATE);

	s->nvram  	= nv;
	s->htpath 	= doc_path;

	return s ;
}


void htcleanup (void *desc) 
{
	STATE* s = desc;

	free(s);
}



/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

