/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: n/n (D-Link)			                          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "hlst.h"
#include "http-plugin.h"
#include "parser.h"
#include "nvalloc.h"
#include "plugin-util.h"
#include "phone_common.h"
#include "logger.h"

/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */


#define INVALID_DEVNAME			CUSTOM_START_ERROR + 10
#define INVALID_DEVNAME_EXISTS		CUSTOM_START_ERROR + 11
#define INVALID_NUMBER			CUSTOM_START_ERROR + 20
#define INVALID_NUMBER_EXISTS		CUSTOM_START_ERROR + 21

#define INVALID_TOTAL_ELEMENTS		CUSTOM_START_ERROR + 40

#define TOTAL_ELEMENTS_LABEL		"phone_devices.totalElements"
#define LIST_LABEL			"phone_devices.list"
#define DEVICE_TYPE			"external"

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */


int checkDeviceName(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char *existVar, **val = (char**)find_hlst(avp, varName, 0);

	if(!val || **val=='\0')
	{
		error = INVALID_DEVNAME;
	}
	else if(existVar = getExistNvramVar(nvram_avp, varName, *val))	
	{
		/* check if we found the variable we have to edit */
		if(!isEditVariable(avp,existVar,varName))
			error = INVALID_DEVNAME_EXISTS;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}


int checkExternalNumber(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char *existVar, **val = (char**)find_hlst(avp, varName, 0);

	if(!val || **val=='\0' || !validPhoneNumber(val))
	{
		error = INVALID_NUMBER;
	}
	else if(existVar = getExistNvramVar(nvram_avp, varName, *val))	
	{
		/* check if we found the variable we have to edit */
		if(!isEditVariable(avp,existVar,varName))
			error = INVALID_NUMBER_EXISTS;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int setId(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	char *nextIndex;
	char **val;
	char **ind;
	char buf[10];


	ind		       	= (char**)find_hlst(avp, "index", 0);
	//val 			= (char**)find_hlst(avp, varName, 0);
	nextIndex		= getNextIndex(nvram_avp, LIST_LABEL, TOTAL_ELEMENTS_LABEL);
   
	if(!ind || **ind == '\0')
	{
		if(nextIndex)
		{ 
			//method checks whether there is already a value and reallocates the memory then
			forceAddToList(avp,varName,nextIndex);
		}
		else
		{
			error = INVALID_TOTAL_ELEMENTS;
		}
	}
	else
	{
		addToList(avp,varName,*ind);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return NO_ERROR;
}


int setType(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	forceAddToList(avp,varName,DEVICE_TYPE);
        
	return NO_ERROR;
}


HTMLOBJECT *init_htmlVars(){

	int max_size = 6;
	int max_index = max_size - 1;
	int i=0;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);

	p[i].revise 	= setId;
	p[i++].name 	= "phone_devices.id";

	p[i].revise 	= setType;
	p[i++].name 	= "phone_devices.type";

	p[i].revise 	= checkDeviceName;
	p[i++].name 	= "phone_devices.name";

	p[i].revise 	= checkExternalNumber;
	p[i++].name 	= "phone_devices.number";
      
	p[i].revise 	= noError;
	p[i++].name 	= "phone_devices.icon";
  
	//sentinel
	p[i>(max_index) ? max_index : i].revise	= NULL;
	p[i>(max_index) ? max_index : i].name 	= NULL;


	return p;
}

void free_htmlVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}


void handle_varError(HLST *avp, int ecode){

 
	switch(ecode){
	case INVALID_DEVNAME:		addToList(avp, "phone_devices_external.name.error", "1");
					break;
	case INVALID_DEVNAME_EXISTS:	addToList(avp, "phone_devices_external.name.exists.error", "1");
					break;
	case INVALID_NUMBER:		addToList(avp, "phone_devices_external.number.error", "1");
					break;
	case INVALID_NUMBER_EXISTS:	addToList(avp, "phone_devices_external.number.exists.error", "1");
					break;

	default	:			//UNKNOWN error code, nothing to do
					logger (LOG_WARNING, "UNKNOWN error code: %d: ", ecode);
					return;
	}

	addToList(avp, "error", "1");
    
	return;
}


void *htinit (const char *doc_path, NVCL *nv) {
	STATE *s = ZMALLOC (STATE) ;

	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= "phone/phone_devices.html";
	s->htmlvars.total_elements_label 	= TOTAL_ELEMENTS_LABEL;
	s->htmlvars.group_label 		= "phone_devices";


	s->nvram  	= nv ;
	s->htpath 	= doc_path ;

	return s ;
}

void htcleanup (void *desc) {
	STATE *s = desc ;

	free_htmlVars(s->htmlvars.objects);
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
	STATE  *s = desc ;
	PARSER   *parser ;
	HLST        *avp ;
	const char *data ;
	char **retVal;
	char *html_file;

	if (argc < 2)
		return ;

	html_file = (char*)argv[1];

	avp = http_args (r);

	
	if((char**)find_hlst(avp,"save", 0))
	{
		//store
		if(save_indexHttpVars(s,avp))
		{
			/* add device id to all account-timeslots, so the devices rings per default */
			/* do not do this for external devices!!!! [HM] */
    			/* publishDevId(s,avp,ADD_DEV_INDEX); */
			/* set file to load */
			html_file = (char*)s->htmlvars.load_on_success;
    			count_types(s, avp, "phone_devices");
		}
	}
	else if((char**)find_hlst(avp,"delete", 0))
	{
		 //reorder, delete
		if(delete_indexHttpVars(s,avp))
    		{
    			/* remove device id to all account-timeslots */
			publishDevId(s,avp, REMOVE_DEV_INDEX, handle_varError);
			/* set file to load */
			html_file = (char*)s->htmlvars.load_on_success;
    			count_types(s, avp, "phone_devices");
		}
	}
	else if((char**)find_hlst(avp,"add", 0))
	{
		 //do nothing for add, just do not call load_indexHttpVars
	}
	else
	{
		  load_indexHttpVars(s, avp);
	}
	
	handle_global_settings(avp, s->nvram);
       
        
	parser = open_parser (avp, s->nvram, s->htpath) ;
	
	if (file_parser (parser, htdocs_p (r, html_file)) == 0)
		return ;
	
	/* parser */
	run_parser (parser) ;
	
	avdump (avp); /* debugging */
	
	/* print result */
	if ((data = data_collect (parser)) != 0)
		body_append (r, data, 0);
	
	/* clean up */
	file_parser_end (parser);
	close_parser (parser) ;
}




/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

