/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: n/n (D-Link)			                          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "hlst.h"
#include "http-plugin.h"
#include "parser.h"
#include "nvalloc.h"
#include "plugin-util.h"
#include "phone_common.h"
#include "logger.h"

/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */


#define INVALID_DEVNAME			CUSTOM_START_ERROR + 10
#define INVALID_DEVNAME_EXISTS		CUSTOM_START_ERROR + 11

#define INVALID_PASSWORD		CUSTOM_START_ERROR + 20

#define INVALID_NUMBER			CUSTOM_START_ERROR + 30
#define INVALID_NUMBER_EXISTS		CUSTOM_START_ERROR + 31

#define MAX_ELEMENTS_EXCEEDED		CUSTOM_START_ERROR + 40

#define INVALID_TOTAL_ELEMENTS		CUSTOM_START_ERROR + 50

#define TOTAL_ELEMENTS_LABEL		"phone_devices.totalElements"
#define LIST_LABEL			"phone_devices.list"
#define DEVICE_TYPE			"sip"

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

int assertMaxSipElements(HLST *avp, HLST *nvram_avp, const char *total_elements_label)
{

		HSRCH 		*lookup;
	int 		total	= 0;
	char		*var 	= NULL;
	char 		*label 	= "phone_devices.type";
	const char	**Val	= NULL;
	char 		**ind 	= (char**)find_hlst(avp, "index", 0);


	/* check if we are in edit mode */
	if(ind && (**ind != '\0'))
		return NO_ERROR;

	lookup = open_hlst_search (nvram_avp);
	while ((Val = (const char**) next_hlst_search (lookup)) != 0) 
	{
		if(strcasecmp(*Val,DEVICE_TYPE)!=0)
		   continue;

		/* value was equal to DEVICE_TYPE, so get the associated variable */
		var = query_key_hlst((void **)Val) ;

		if(strncmp(label, var, strlen(label)) == 0)
		{
			/* we found a variable beginning with label and having the same value as DEVICE_TYPE */
			total++;
		}

		/* that was not what we're looking for */
		var = NULL;
	}
	close_hlst_search (lookup) ;

    if((total!= -1) && (total <= 7))
    {
        return NO_ERROR;
    }
    else
    {
        handle_varError(avp, MAX_ELEMENTS_EXCEEDED);
        return MAX_ELEMENTS_EXCEEDED;
    }
}


int checkDeviceName(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char *existVar, **val = (char**)find_hlst(avp, varName, 0);

//	do not check [hm]
//	error = assertMaxSipElements(avp, nvram_avp, TOTAL_ELEMENTS_LABEL);
	if(error != NO_ERROR)
		return error;

	if(!val || **val=='\0')
	{
		error = INVALID_DEVNAME;
	}
	else if(existVar = getExistNvramVar(nvram_avp, varName, *val))	
	{
		/* check if we found the variable we have to edit */
		if(!isEditVariable(avp,existVar,varName))
			error = INVALID_DEVNAME_EXISTS;
	}
      
	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkInternalNumber(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char *existVar, **val = (char**)find_hlst(avp, varName, 0);

	if(!val || **val=='\0')
	{
		error = INVALID_NUMBER;
	}
	else if(existVar = getExistNvramVar(nvram_avp, varName, *val))	
	{
		/* check if we found the variable we have to edit */
		if(!isEditVariable(avp,existVar,varName))
			error = INVALID_NUMBER_EXISTS;
	}
	
	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkPassword(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **val = (char**)find_hlst(avp, varName, 0);

	if(val)
	{
		if(**val == '\0')
			error = INVALID_PASSWORD;
    }

	if(error != NO_ERROR)
			handle_varError(avp, error);

	return error;
}

int setId(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
    char *nextIndex;
	char **val;
	char **ind;

	ind		       	= (char**)find_hlst(avp, "index", 0);
	//val 			= (char**)find_hlst(avp, varName, 0);
	nextIndex		= getNextIndex(nvram_avp, LIST_LABEL, TOTAL_ELEMENTS_LABEL);
   
	if(!ind || **ind == '\0')
	{
		if(nextIndex)
		{
			forceAddToList(avp,varName,nextIndex);
		}
		else
		{
			error = INVALID_TOTAL_ELEMENTS;
		}
	}
	else
	{
		addToList(avp,varName,*ind);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return NO_ERROR;
}


int setType(HLST *avp, HLST *nvram_avp, const char *varName) 
{
       forceAddToList(avp,varName,DEVICE_TYPE);
                
	return NO_ERROR;
}


HTMLOBJECT *init_htmlVars(){

	int max_size = 8;
	int max_index = max_size - 1;
	int i=0;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);

	p[i].revise 	= setId;
	p[i++].name 	= "phone_devices.id";

	p[i].revise 	= setType;
	p[i++].name 	= "phone_devices.type";

	p[i].revise 	= checkDeviceName;
	p[i++].name 	= "phone_devices.name";

	p[i].revise 	= checkInternalNumber;
	p[i++].name 	= "phone_devices.number";

	p[i].revise 	= checkPassword;
	p[i++].name 	= "phone_devices.password";

	p[i].revise 	= noError;
	p[i++].name 	= "phone_devices.stdaccount";

	p[i].revise 	= noError;
	p[i++].name 	= "phone_devices.icon";
  
	//sentinel
	p[i>(max_index) ? max_index : i].revise = NULL;
	p[i>(max_index) ? max_index : i].name 	= NULL;


	return p;
}

void free_htmlVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}



void handle_varError(HLST *avp, int ecode){

 
	switch(ecode){
	case INVALID_DEVNAME:		addToList(avp, "phone_devices_sip.name.error", "1");
					break;
	case INVALID_DEVNAME_EXISTS:	addToList(avp, "phone_devices_sip.name.exists.error", "1");
					break;
	 case INVALID_NUMBER:		addToList(avp, "phone_devices_sip.number.error", "1");
					break;
	case INVALID_NUMBER_EXISTS:	addToList(avp, "phone_devices_sip.number.exists.error", "1");
					break;
	case INVALID_PASSWORD:	   	addToList(avp, "phone_devices_sip.password.error", "1");
					break; 
	case MAX_ELEMENTS_EXCEEDED:	addToList(avp, "phone_devices_sip.totalElements.exceeded.error", "1");
					break;
   
	default	:			//UNKNOWN error code, nothing to do
					logger (LOG_WARNING, "UNKNOWN error code: %d: ", ecode);
					return;
	}

	addToList(avp, "error", "1");
    
	return;
}


void *htinit (const char *doc_path, NVCL *nv) {
    STATE *s = ZMALLOC (STATE) ;

	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= "phone/phone_devices.html";
	s->htmlvars.total_elements_label 	= TOTAL_ELEMENTS_LABEL;
	s->htmlvars.group_label 		= "phone_devices";

	s->nvram  	= nv ;
	s->htpath 	= doc_path ;
	return s ;
}

void htcleanup (void *desc) {
	STATE *s = desc ;

	free_htmlVars(s->htmlvars.objects);
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
	STATE  *s = desc ;
	PARSER   *parser ;
	HLST        *avp ;
	const char *data ;
	char **retVal;
	char *html_file;

	if (argc < 2)
        return ;

	html_file = (char*)argv[1];

	avp = http_args (r);

	if((char**)find_hlst(avp,"save", 0))
	{
		 //store
		if(save_indexHttpVars(s,avp))
		{
			/* add device id to all account-timeslots, so the devices rings per default */
    			publishDevId(s,avp,ADD_DEV_INDEX, handle_varError);
			/* set file to load */
			html_file = (char*)s->htmlvars.load_on_success;
    			count_types(s, avp, "phone_devices");
		}
	}
	else if((char**)find_hlst(avp,"delete", 0))
	{
		 //reorder, delete
		if(delete_indexHttpVars(s,avp))
    		{
    			/* remove device id to all account-timeslots */
			publishDevId(s,avp, REMOVE_DEV_INDEX, handle_varError);
			/* set file to load */
			html_file = (char*)s->htmlvars.load_on_success;
    			count_types(s, avp, "phone_devices");
		}
	}
	else if((char**)find_hlst(avp,"add", 0))
	{
		//do nothing for add, just do not call load_indexHttpVars
	}
	else
	{
		load_indexHttpVars(s, avp);
	}
	
	
        
	handle_global_settings(avp, s->nvram);
	
	parser = open_parser (avp, s->nvram, s->htpath) ;
	
	if (file_parser (parser, htdocs_p (r, html_file)) == 0)
		return ;
	
	/* parser */
	run_parser (parser) ;
	
	avdump (avp); /* debugging */
	
	/* print result */
	if ((data = data_collect (parser)) != 0)
		body_append (r, data, 0);
	
	/* clean up */
	file_parser_end (parser);
	close_parser (parser) ;
}




/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

