/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: n/n (D-Link)			                          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "hlst.h"
#include "http-plugin.h"
#include "parser.h"
#include "nvalloc.h"
#include "plugin-util.h"
#include "logger.h"

/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */


#define INVALID_NUMBER			CUSTOM_START_ERROR + 10

#define INVALID_QUICKDIAL		CUSTOM_START_ERROR + 20
#define INVALID_QUICKDIAL_EXISTS	CUSTOM_START_ERROR + 21

#define INVALID_VANITY			CUSTOM_START_ERROR + 30
#define INVALID_VANITY_EXISTS		CUSTOM_START_ERROR + 31

#define INVALID_TOTAL_ELEMENTS	       	CUSTOM_START_ERROR + 50

#define TOTAL_ELEMENTS_LABEL		"phone_quickdials.totalElements"
#define LIST_LABEL			"phone_quickdials.list"

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

int setId(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	char *nextIndex;
	char **val;
	char **ind;

	ind		= (char**)find_hlst(avp, "index", 0);
	//val 		= (char**)find_hlst(avp, varName, 0);
	nextIndex	= getNextIndex(nvram_avp, LIST_LABEL, TOTAL_ELEMENTS_LABEL);
   
	if(!ind || **ind == '\0')
	{
		if(nextIndex)
		{
			forceAddToList(avp,varName,nextIndex);
		}
		else
		{
			error = INVALID_TOTAL_ELEMENTS;
		}
	}
	else
	{
		addToList(avp,varName,*ind);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return NO_ERROR;
}


int checkQuickdial(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char *existVar;
	char **val = (char**)find_hlst(avp, varName, 0);
	char **selection = (char**)find_hlst(avp, "phone_quickdials.selection", 0);


	if(!val || !**val) {

		error = INVALID_QUICKDIAL;

	} else if (selection && !strcmp(*selection,"quickdial")) {
		/* check if value of varName contains numbers only */

		if (getVarAsLong(avp,varName) == -1) {
			error = INVALID_QUICKDIAL;
		} else if (existVar = getExistNvramVar(nvram_avp, varName, *val)) {
			/* check if we found the variable we have to edit */
			if(!isEditVariable(avp,existVar,varName))
				error = INVALID_QUICKDIAL_EXISTS;
		}

		forceAddToList(avp,"phone_quickdials.vanity","0");

	} else /* if (selection && !strcmp(*selection,"vanity")) */ {
		char buf[10];
		int ret;

		ret = getNumber4Vanity(avp,varName,buf,10);
		if(ret != -1) {
			if(existVar = getExistNvramVar(nvram_avp, varName, buf)) {
				/* check if we found the variable we have to edit */
				if(!isEditVariable(avp,existVar,varName)) {
					error = INVALID_VANITY_EXISTS;
				}
			}

			forceAddToList(avp,"phone_quickdials.vanity",*val);
			forceAddToList(avp,varName,buf);
		} else {
			error = INVALID_VANITY;
		}
	}
	

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkNumber(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char *existVar, **val = (char**)find_hlst(avp, varName, 0);

	if(!val || **val=='\0')
	{
		error = INVALID_NUMBER;
	} else if (!validateRegExp(*val, "^([0-9*#]*)$")) {
		error = INVALID_NUMBER;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}


int getNumber4Vanity(HLST *avp,const char *varName,char *buf, int bufsize)
{
	int i 	= 0;
	int ret = 0;
	char **val = (char**)find_hlst(avp, varName, 0);

	if(val)
	{
		for(i=0; i<strlen(*val) && i<bufsize; i++)
		{
			switch((*val)[i])
			{
			case 'a': case 'b': case 'c':
			case 'A': case 'B': case 'C':			buf[i] = '2';
									break;
			case 'd': case 'e': case 'f':
			case 'D': case 'E': case 'F':			buf[i] = '3';
									break;
			case 'g': case 'h': case 'i':
			case 'G': case 'H': case 'I':			buf[i] = '4';
									break;
			case 'j': case 'k': case 'l':
			case 'J': case 'K': case 'L':			buf[i] = '5';
									break;
			case 'm': case 'n': case 'o':
			case 'M': case 'N': case 'O':			buf[i] = '6';
									break;
			case 'p': case 'q': case 'r': case 's':
			case 'P': case 'Q': case 'R': case 'S':		buf[i] = '7';
									break;
			case 't': case 'u': case 'v':
			case 'T': case 'U': case 'V':			buf[i] = '8';
									break;
			case 'w': case 'x': case 'y': case 'z':	
			case 'W': case 'X': case 'Y': case 'Z':		buf[i] = '9';
									break;

			default:					buf[0] = '\0';
									ret = -1;
			}
		}
		buf[i] = '\0';
	}
	return ret;
}

HTMLOBJECT *init_htmlVars(){

	int max_size = 6;
	int max_index = max_size - 1;
	int i=0;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);

	p[i].revise 	= setId;   
	p[i++].name	= "phone_quickdials.id";

	p[i].revise 	= noError;
	p[i++].name 	= "phone_quickdials.selection";

	p[i].revise 	= checkQuickdial;
	p[i++].name 	= "phone_quickdials.quickdial";

	p[i].revise 	= noError;
	p[i++].name 	= "phone_quickdials.vanity";

	p[i].revise 	= checkNumber;
	p[i++].name 	= "phone_quickdials.number";

	//sentinel
	p[i>(max_index) ? max_index : i].revise = NULL;
	p[i>(max_index) ? max_index : i].name 	= NULL;


	return p;
}

void free_htmlVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}


void handle_varError(HLST *avp, int ecode){

 
	switch(ecode){
	
	case INVALID_QUICKDIAL:			addToList(avp, "phone_quickdials.quickdial.error", "1");
						break;
	case INVALID_QUICKDIAL_EXISTS:		addToList(avp, "phone_quickdials.quickdial.exists.error", "1");
						break;
	case INVALID_NUMBER:			addToList(avp, "phone_quickdials.number.error", "1");
						break;
	case INVALID_VANITY:			addToList(avp, "phone_quickdials.vanity.error", "1");
						break;
	case INVALID_VANITY_EXISTS:		addToList(avp, "phone_quickdials.vanity.exists.error", "1");
						break;
   
	default	:				//UNKNOWN error code, nothing to do
						logger (LOG_WARNING, "UNKNOWN error code: %d: ", ecode);
						return;
	}

	addToList(avp, "error", "1");
    
	return;

}



void *htinit (const char *doc_path, NVCL *nv) {
	STATE *s = ZMALLOC (STATE) ;

	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= "phone/phone_quickdials.html";
	s->htmlvars.total_elements_label 	= "phone_quickdials.totalElements";
	s->htmlvars.group_label 		= "phone_quickdials";
   
	s->nvram  	= nv ;
	s->htpath 	= doc_path ;

	return s ;
}

void htcleanup (void *desc) {
	STATE *s = desc ;

	free_htmlVars(s->htmlvars.objects);
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
	STATE  *s = desc ;
	PARSER   *parser ;
	HLST        *avp ;
	const char *data ;
	char **retVal;
	char *html_file;

	if (argc < 2)
		return ;

	html_file = (char*)argv[1];

	avp = http_args (r);

	
	if((char**)find_hlst(avp,"save", 0))
	{
		 //store
		if(save_indexHttpVars(s,avp))
			html_file = (char*)s->htmlvars.load_on_success;

	}
	else if((char**)find_hlst(avp,"delete", 0))
	{
		 //reorder, delete
		if(delete_indexHttpVars(s,avp))
			html_file = (char*)s->htmlvars.load_on_success;
	}
	else if((char**)find_hlst(avp,"add", 0))
	{
		 //do nothing for add, just do not call load_indexHttpVars
	}
	else
	{
		  load_indexHttpVars(s, avp);
	}
        
        
	handle_global_settings(avp, s->nvram);

	parser = open_parser (avp, s->nvram, s->htpath) ;

	if (file_parser (parser, htdocs_p (r, html_file)) == 0)
		return ;

	/* parser */
	run_parser (parser) ;

	avdump (avp); /* debugging */

	/* print result */
	if ((data = data_collect (parser)) != 0)
		body_append (r, data, 0);

	/* clean up */
	file_parser_end (parser);
	close_parser (parser) ;
}



/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

