/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: n/n (D-Link)			                          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#include "hlst.h"
#include "util.h"
#include "http-plugin.h"
#include "parser.h"
#include "nvalloc.h"
#include "plugin-util.h"
#include "logger.h"

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

void *htinit (const char *doc_path, NVCL *nv) 
{
	STATE *s = ZMALLOC (STATE);

	s->nvram  = nv;
	s->htpath = doc_path;

	return s;
}


void htcleanup (void *desc) 
{
	STATE *s = desc ;
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) 
{
	STATE	*s = desc;
	PARSER	*parser;
	HLST	*avp;
	FILE*	pipe;
	const char *data;

	if (argc < 2)
		return;

	avp = http_args(r);

	handle_global_settings(avp, s->nvram);
    
	parser = open_parser(avp, s->nvram, s->htpath);

	if (file_parser(parser, argv [1]) == 0)
		return;

	pipe = popen("/bin/sh /usr/share/board_cfg/phone/phone_status.sh", "r");
	if (pipe)
		pclose(pipe);
	else
		logger(LOG_ERR, "Could not execute phone_status.sh (%s)",
			strerror(errno));

	count_types(s, avp, "phone_accounts");
	count_types(s, avp, "phone_devices");

	/* parser */
	run_parser(parser);

	/* print result */
	if ((data = data_collect(parser)) != 0)
		body_append(r, data, 0);

	/* clean up */
	file_parser_end(parser);
	close_parser(parser);
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
