/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Susanne Goldammer <s.goldammer@maxina.de> - Maxina GmbH,      *
*   Neu-Isenburg, Germany						  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifndef __MHTTPD_PLUGIN_UTIL_H
#define __MHTTPD_PLUGIN_UTIL_H

#include "hlst.h"
#include "nvram-lib.h"
#include "http.h"


#define NO_ERROR					0
#define NVRAM_ERROR					1
#define INVALID_RADIO_OPTION		2

#define CUSTOM_START_ERROR			10

/*
#define CALL_HTTPVARS_ACTION_OLD(s,avp)    	{ \
										(query_hlst_size(avp) == 0) ? 	\
											load_httpvars(s, avp):		\
											save_httpvars(s, avp);		\
										}
*/

//should be used instead of CALL_HTTPVARS_ACTION
#define CALL_HTTPVARS_ACTION(s,avp)    	{ \
										((char**)find_hlst(avp,"save",0)) ? 	\
											save_httpvars(s, avp):		\
											load_httpvars(s, avp);		\
	         								}

typedef
struct _object {
	const char *name;
	int (*revise)(HLST *, HLST *, const char *);
	
}HTMLOBJECT;



typedef
struct _htmlvar {
	const char 	*group_label;
	const char	*total_elements_label;
	const char 	*load_on_success;
	HTMLOBJECT	*objects;
}HTMLVAR;


typedef
struct _state {
    NVCL        *nvram ;
    const char 	*htpath ;
	HTMLVAR 	htmlvars;
} STATE ;


#ifdef WITH_DMALLOC
typedef struct {
    const char *file ;
    int         line ;
} __avdump_rec ;
static
int __avdump_print (void *state, void *val, char *key, unsigned keylen) {
    __avdump_rec *a = state ;
    dmalloc_verify (val);
    logger (LOG_DEBUG, "AVPRINT at %s(%d): %s=%s.\n",
            a->file, a->line, key, val ? val : "(null)");
    return 0;
} 
static
void __avdump (HLST *h, const char *file, int line) {
    __avdump_rec a = {file, line};
    dmalloc_verify (h);
    for_hlst_do (h, __avdump_print, &a); 
    dmalloc_log_stats ();                          \
}
#define avdump(c) __avdump (c,__FILE__, __LINE__)
#define DMALLOC_VERIFY(g) {\
                 dmalloc_verify (g);                            \
                 dmalloc_log_stats ();                          \
        }

#define DMALLOC_MESSAGE(g,fmt, ...) {\
                 dmalloc_message (fmt, ##__VA_ARGS__) ; \
                 DMALLOC_VERIFY(g) ;                    \
        }
#else
#define DMALLOC_VERIFY(g)         /* n/a */
#define DMALLOC_MESSAGE(fmt, ...) /* n/a */
#define avdump(c)                 /* n/a */
#endif





extern int validIpAddr(char **val);
extern int validMacAddr(char **val);
extern void addToList(HLST *avp, const char *varName, const char *varValue);
extern void forceAddToList(HLST *avp, const char *varName, const char *varValue);
extern unsigned load_from_nvram(NVCL *nvram ,  HLST *avp, const char *pfx );
extern void nvram_avp_destroy (void *state, void *item, char *key, unsigned klen);
extern void save_httpvars(STATE  *s,  HLST *avp);
extern void load_httpvars(STATE  *s,  HLST *avp);
extern void handle_varError(HLST *avp, int ecode);
extern int ensureRadioChecked(HLST *avp, HLST *nvram_avp, const char *varName);
extern char *getExistNvramVar(HLST *nvram_avp, const char *varName, const char *newVal);
extern int isEditVariable(HLST *avp, const char *existVar, const char *varName);
extern void load_indexHttpVars(STATE  *s,  HLST *avp);
extern int save_indexHttpVars(STATE  *s,  HLST *avp);
extern int delete_indexHttpVars(STATE  *s,  HLST *avp);
extern long getVarAsLong(HLST *nvram_avp, const char *var);
extern int ensureCheckState(HLST *avp, HLST *nvram_avp, const char *varName);
extern char *addNewIndex(HLST *nvram_avp, char *listLabel, const char *index);
extern char *removeIndex(HLST *nvram_avp, char *listLabel, const char *index);
extern long getVarAsPort(HLST *avp, const char *varName);
extern int validSubnetMask(char **val);
extern int noError(HLST *avp, HLST *nvram_avp, const char *varName);

extern void add_to_list(HLST* avp, const char* name, const char* value);
extern char* get_from_list(HLST* avp, const char* name);
extern void add_to_nvram(NVCL* nvram, const char* name, const char* value);
extern char* get_from_nvram(NVCL* nvram, const char* name);
extern void send_html_page(HLST* avp, STATE* s, RPLY_REC* r, const char* templ_name);
extern int checkIdBlocking(HLST *avp, HLST *nvram_avp, const char *varName);
extern char *removeBlanks(char *val);
extern int is_enabled(NVCL* nvram, const char* name);

/*! \brief Extract a substring from a string pointer

    \param pointer to input string
    \param beginning of extracted string
    \param length length of substring to be extracted

    \return pointer to substring
*/
char *substr(char *string, int start, size_t count);

/*! \brief strip leading and trailing whitespaces from a string

    \param string pointer

    \return string pointer to stripped string
*/
char *stripstr(char *string);

/* \!brief securely read a single line from a file

   \param output buffer

   \param file handle
*/
#include <stdio.h>
int fggets(char *ln, FILE *f);

#endif /* __MHTTPD_PLUGIN_UTIL_H */
