/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifndef __MHTTP_SETTINGS_H
#define __MHTTP_SETTINGS_H

/* mhttpd.c (main) */
#define MAIN_LOG_IDLE_LOOPING  0x0000100 /* log selector flag */
#define MAIN_IDLE_TIMEOUT          30000 /* ms timeout for idle loop */
#define MAIN_NVRPLY_TIMEOUT        10000 /* ms */
#define MAIN_CTRL_IOBUF_SIZE        2000

#define MAIN_DEFAULT_LOGMODE   (~ (PARSER_GETVAR_LOG  |\
                       	           PARSER_SETVAR_LOG  |\
                       	           PARSER_LISTVAR_LOG ))

#define MAIN_DEFAULT_VERBOSITY (~ (DO_LOG_ACTION_NEXTIDLE |\
				   PARSER_GETVAR_LOG      |\
                                   PARSER_SETVAR_LOG      |\
                                   PARSER_LISTVAR_LOG     |\
                                   MAIN_LOG_IDLE_LOOPING  |\
                                   XXL_TRAFFIC_LOG        |\
                                   XXL_STATS_LOG          |\
                                   XXL_VERIFY_LOG         |\
                                   XXL_SETUP_LOG          |\
                                   HTTP_INTRO_LOG         |\
                                   HTTP_LOG_DEFLOAD       |\
                                   HTTP_LOG_CLOSE         |\
                                   HTTP_REPLY_LOG         |\
                                   HTTP_PURGE_LOG         |\
                                   HTTP_REQUEST_LOG       |\
				   DO_LOG_ACTION_REGISTER ))

#ifdef USE_LOCAL_REQUEST_FILE
# define MAIN_CTRL_REQ_PATH    USE_LOCAL_REQUEST_FILE
#else
# define MAIN_CTRL_REQ_PATH    "/var/run/mhttpd/server"
#endif

/* comhandler.c */
#define COMM_LOG_DEFLOAD        0x00000200
#define COMM_LIST_FILE_DEFAULT  "/tmp/mhttpd.out"

/* parser.c */
#define PARSER_GETVAR_LOG       0x00000400
#define PARSER_GETVAR_NVOFF     (CTRL_REQUEST_LOG |\
                                 CTRL_REPLY_LOG   )
#define PARSER_SETVAR_LOG       0x00000800
#define PARSER_SETVAR_NVOFF     (CTRL_REQUEST_LOG |\
                                 CTRL_REPLY_LOG   )
#define PARSER_LISTVAR_LOG      0x00001000
#define PARSER_LISTVAR_NVOFF    (CTRL_REQUEST_LOG |\
                                 CTRL_REPLY_LOG   )
#define PARSER_PAGE_ACL_LOG     0x00002000
#define PARSER_PAGE_ACL_NVOFF   (CTRL_REQUEST_LOG |\
                                 CTRL_REPLY_LOG   )

/* http.c */
#define HTTP_NVRPLY_TIMEOUT        1000 /* ms */

/* http-defaults.c */
#define HTDEFAULTS_OVWRITE_LOG  0x00004000

/* http-server.c */
#define HTTP_KEEP_ALIVE        (265 * 3600) /* default: on year seconds */
#define HTTP_MIN_REQBUF               2000
#define HTTP_MAX_REQBUF               5000
#define HTTP_DLPOOL_PFX         "/usr/lib/htpool"
#define HTTP_HTDOCS_PFX         "/usr/lib/htdocs"
#define HTTP_ADMIN_LOGNAME      "sys_admin.login"
#define HTTP_ADMIN_PASSWD       "sys_admin.passwd"
#define HTTP_ADMIN_REALM        "http.admin.realm"
#define HTTP_BIND_IP            "sys_admin.ip"
#define HTTP_BIND_PORT          "sys_admin.port"
#define HTTP_BIND_SSLOK         "sys_admin.sslok"
#define HTTP_DEFAULT_CHANNEL_IDLE       90 /* otherwise purge */
#define HTTP_DEFAULT_SERVER_PORT       800 /* http configuration port */
#define HTTP_DEFAULT_USE_SSL             1 /* 0 or 1 or comment out */
#define HTTP_DEFAULT_IP_ADDRESS 0x7f000001 /* 127.0.0.1 */
#define HTTP_DEFAULT_CHANNELS           15 /* listener backlog */
#define HTTP_DEFAULT_CACHE_DIM           8
#define HTTP_LOG_DEFLOAD        0x00010000
#define HTTP_LOG_CLOSE          0x00020000
#define HTTP_REQUEST_LOG        0x00040000
#define HTTP_REPLY_LOG          0x00080000
#define HTTP_AUTH_LOG           0x00100000
#define HTTP_PURGE_LOG          0x00200000
#define HTTP_INTRO_LOG          0x00400000

/* list.c */
#define LIST_DEFAULT_URI               "*"
#define LIST_ACCESS_BASE    "http.access."
#define LIST_NVRAM_TIMEOUT	     1000 /* ms */

/* ssl.h */
#define XXL_SESSION_CACHE               30 /* twice the num of http slots */
#define XXL_SESSION_CACHE_MAX         1000 /* openssl default is 20000 */
#define XXL_SETUP_LOG           0x01000000 /* log selector flag */
#define XXL_VERIFY_LOG          0x02000000 /* log selector flag */
#define XXL_STATS_LOG           0x04000000 /* log selector flag */
#define XXL_TRAFFIC_LOG         0x08000000 /* log selector flag */

#define XXL_VERIFY_USE_ONLY_MY      0x4000
#define XXL_SID0_CTX            "mhttpd SID0"
#define XXL_SID1_CTX            "mhttpd SID1"
#define XXL_REPEAT_ACCEPT               10 /* non-blocking mode needs that */
#define XXL_WAIT_REWRITE                10 /* ms */
#define XXL_WAIT_REREAD                 10 /* ms */

#define XXL_CERT_NAME           "http.ssl.cert"
#define XXL_KEY_NAME            "http.ssl.key"
#define XXL_DHPARAM_NAME        "http.ssl.dhparam"
#define XXL_CTX0SID_NAME        "http.ssl.ctx_sid.1"
#define XXL_CTX1SID_NAME        "http.ssl.ctx_sid.2"
// #define XXL_CHAIN_NAME       "http.ssl.chain"
#define XXL_CIPHER_LIST         "http.ssl.cipherlist"
// #define XXL_ADMIN_NAME       "http.admin.subject"
// #define XXL_ADMIN_CHAIN      "http.admin.chain"

/* test_plugin.c */
#define TPLUG_NVRAM_IP	         0xc0000202
#define TPLUG_NVRAM_PORT	 888
#define TPLUG_NVRAM_TIMEOUT	 1000	/* ms */
#define TPLUG_LOG_SIZE	         (16 * 1024)
#define TPLUG_DSL_BOARD_IP	"192.0.2.240"
#define TPLUG_BACKPACK_IP	"192.0.2.241"

#endif /* __MHTTP_SETTINGS_H */
