/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifndef __MHTTPD_SSL_H
#define __MHTTPD_SSL_H
#ifdef HAVE_OPENSSL

#include "nvram-lib.h"

typedef struct _xxl_init {
    const char      *dh_name ;
    const char    *cert_name ;
    const char     *key_name ;
    const char *sid_name [2] ;
    const char  *cipher_name ;
    const char      *ca_file ;
    const char       *ca_dir ;
    const char  *sid_ctx [2] ; /* defaults for the value of sid_name */
    int         verify_depth ;
    unsigned     verify_mode ;
    unsigned   session_cache ;
    unsigned session_timeout ;
} XXL_INIT ;

typedef struct _sess_info {
    int accept_loop ;
    int number ;
    int connect ;
    int connect_good ;
    int connect_renegotiate ;
    int accept ;
    int accept_good ;
    int accept_renegotiate ;
    int hits ;
    int depth ;
    int misses ;
    int timeouts ;
} SESS_INFO ;

typedef struct _cty {
#   ifdef __MHTTPD_SSL_PRIVATE
    SESS_INFO      info ;
    SSL_CTX        *ctx ;
    unsigned    sid_inx ;
    const char *sid_ctx ;
    size_t      sid_len ;
    XXL_INIT        xxl ;
    NVCL            *nv ;
#   else
    char            any ;
#   endif
} CTY ;

typedef struct _xxl {
#   ifdef __MHTTPD_SSL_PRIVATE
    SSL        *ssl ; /* SSL Connection */
    int          fd ; /* io socket */
    CTY          *y ; /* parent */
    const char  *dn ; /* the current client id */
    char   *delayed ; /* to be send later on */
    unsigned  d_len ;
    int ssl_pending ; /* reset when the initial handshake is done */
#   else
    char        any ;
#   endif
} XXL ;

extern const char *ssl_version     (void);
extern CTY        *ssl_create      (NVCL*,XXL_INIT*);
extern void        ssl_destroy     (CTY*);
extern unsigned    ssl_reopen      (CTY*);
// extern void     ssl_check_cache (CTY*);
extern XXL        *ssl_accept      (CTY*,int,ip4addr_t,uint16_t);
extern SESS_INFO  *ssl_info        (CTY*);
extern void        ssl_finish      (XXL*);
extern const char *ssl_client_dn   (XXL*);
extern const char *ssl_strerror    (XXL*);
extern int         ssl_recv        (XXL*,void*,int);
extern int         ssl_send        (XXL*,void*,int);
extern int         ssl_pending     (XXL*);
extern void        ssl_flush       (XXL*);

#endif /* HAVE_OPENSSL */
#endif /* __MHTTPD_SSL_H */
