/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: n/n (D-Link)			                          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <netinet/in.h>

#include "hlst.h"
#include "util.h"
#include "http-plugin.h"
#include "plugin-util.h"
#include "parser.h"
#include "nvalloc.h"
#include "logger.h"


/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */

#define INVALID_HTTP_IP      	     	CUSTOM_START_ERROR + 10
#define INVALID_HTTP_NETMASK		CUSTOM_START_ERROR + 11

#define INVALID_TELNET_IP      	     	CUSTOM_START_ERROR + 20
#define INVALID_TELNET_NETMASK 	     	CUSTOM_START_ERROR + 21

#define INVALID_PORT		       	CUSTOM_START_ERROR + 30

#define INVALID_PASSWORD_EMPTY	       	CUSTOM_START_ERROR + 40
#define INVALID_PASSWORD_NONEQUAL      	CUSTOM_START_ERROR + 41
#define INVALID_PASSWORD_INVALID      	CUSTOM_START_ERROR + 42

#define GROUP_LABEL			"sys_admin"

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

int isHttpMgmtEnabled(HLST *avp)
{
	char **val = (char**)find_hlst(avp, "sys_admin.hasRemoteHttpMgmt", 0);

	if(val && !strcmp(*val, "true"))
	{
		return 1;
	}

	return 0;
}

int isTelnetMgmtEnabled(HLST *avp)
{
	char **val = (char**)find_hlst(avp, "sys_admin.hasRemoteTelnetMgmt", 0);

	if(val && !strcmp(*val, "true"))
	{
		return 1;
	}

	return 0;
}

int checkPort(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;

	if(getVarAsPort(avp, varName) < 1)
	{
		error = INVALID_PORT;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}



int checkHttpIp(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;

	if(isHttpMgmtEnabled(avp))
	{
		char **val = (char**)find_hlst(avp,varName,0); 
		if(!val || !validIpAddr(val))
		{
			error = INVALID_HTTP_IP;
		}
	}

	if(error != NO_ERROR)
		handle_varError(avp,error );

	return error;
}


int checkHttpNetmask(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;

	if(isHttpMgmtEnabled(avp))
	{
		char **val = (char**)find_hlst(avp,varName,0);
		if(!val || !validSubnetMask(val))
		{
			error = INVALID_HTTP_NETMASK;
		}
	}

	if(error != NO_ERROR)
		handle_varError(avp,error );


        return error;
}

int checkTelnetIp(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;

	if(isTelnetMgmtEnabled(avp))
	{
		char **val = (char**)find_hlst(avp,varName,0);
		if(!val || !validIpAddr(val))
		{
			error = INVALID_TELNET_IP;
		}
	}

	if(error != NO_ERROR)
		handle_varError(avp,error );


        return error;
}

int checkTelnetNetmask(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;

	if(isTelnetMgmtEnabled(avp))
	{
		char **val = (char**)find_hlst(avp,varName,0);
		if(!val || !validSubnetMask(val))
		{
			error = INVALID_TELNET_NETMASK;
		}
	}

	if(error != NO_ERROR)
		handle_varError(avp,error );


        return error;
}


int confirmPasswd(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **passwd1;
	char **passwd2;
	int len1 = 0;
	int len2 = 0;

	passwd1 = (char**)find_hlst(avp, varName, 0);
   
	if(!(len1=strlen(*passwd1)))
		error = INVALID_PASSWORD_EMPTY;

	passwd2 = (char**)find_hlst(avp, "sys_admin.passwd2", 0);
	
	if(*passwd2 && !(len2=strlen(*passwd2)))
		error = INVALID_PASSWORD_EMPTY;

	if(len1 != len2 || strcmp(*passwd1,*passwd2))
		error = INVALID_PASSWORD_NONEQUAL;

	if (validateRegExp(*passwd1, "[\"<>= ]"))
		error = INVALID_PASSWORD_INVALID;

	if(error != NO_ERROR)
		handle_varError(avp, error);

	if (error)
		**passwd2 = **passwd1 = '\0';

	return error;
}


int loadLogin(HLST *avp, HLST *nvram_avp, const char *varName)
{
	add_to_list(avp, varName, get_from_list(nvram_avp, varName));

	return NO_ERROR;
}



HTMLOBJECT *init_htmlVars(){

	int max_size = 10;
	int i=0;
	int max_index = max_size - 1;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);

	p[i].revise 	= loadLogin;
	p[i++].name	= "sys_admin.login";

	p[i].revise 	= confirmPasswd;
	p[i++].name	= "sys_admin.passwd";

	p[i].revise 	= checkPort;
	p[i++].name	= "sys_admin.port";

	p[i].revise 	= ensureCheckState;
	p[i++].name	= "sys_admin.hasRemoteHttpMgmt";

	p[i].revise 	= checkHttpIp;
	p[i++].name	= "sys_admin.httpIp";

	p[i].revise 	= checkHttpNetmask;
	p[i++].name	= "sys_admin.httpNetmask";

/* [@HM] not yet supported 
	
   	p[i].revise 	= ensureCheckState;
	p[i++].name	= "sys_admin.hasRemoteTelnetMgmt";

	p[i].revise 	= checkTelnetIp;
	p[i++].name	= "sys_admin.telnetIp";

	p[i].revise 	= checkTelnetNetmask;
	p[i++].name	= "sys_admin.telnetNetmask";

*/
	//sentinel
	p[i>max_index ? max_index : i].revise 	= NULL;
	p[i>max_index ? max_index : i].name	= NULL;

	return p;
}

void free_htmlVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}



void handle_varError(HLST *avp, int ecode){

 
	switch(ecode){

        case INVALID_HTTP_IP:		addToList(avp, "sys_admin.httpIp.error", "1"); 
					break;
	case INVALID_HTTP_NETMASK:	addToList(avp, "sys_admin.httpNetmask.error", "1"); 
					break;
								
	case INVALID_TELNET_IP:		addToList(avp, "sys_admin.telnetIp.error", "1"); 
					break;
	case INVALID_TELNET_NETMASK:	addToList(avp, "sys_admin.telnetNetmask.error", "1"); 
					break;

	case INVALID_PORT:		addToList(avp, "sys_admin.port.error", "1"); 
					break;

	case INVALID_PASSWORD_EMPTY:	addToList(avp, "sys_admin.passwd.empty.error", "1"); 
					break;
	case INVALID_PASSWORD_NONEQUAL:	addToList(avp, "sys_admin.passwd.nonequal.error", "1"); 
					break;
	case INVALID_PASSWORD_INVALID:	addToList(avp, "sys_admin.passwd.invalid.error", "1"); 
					break;

	default	:      			//UNKNOWN error code, nothing to do
					logger (LOG_WARNING, "UNKNOWN error code: %d: ", ecode);
					return;
	}


	addToList(avp, "error", "1");
    
	return;

}




void *htinit (const char *doc_path, NVCL *nv) {
	STATE *s = ZMALLOC (STATE) ;

	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= NULL;
	s->htmlvars.group_label			= GROUP_LABEL;
	s->htmlvars.total_elements_label	= NULL;

	s->nvram  	= nv ;
	s->htpath 	= doc_path ;

	return s ;
}

void htcleanup (void *desc) {
	STATE *s = desc ;

	free_htmlVars(s->htmlvars.objects);
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
    
	STATE  *s = desc ;
	PARSER   *parser ;
	HLST        *avp ;
	const char *data ;
	char *html_file = (char*)argv[1];

	if (argc < 2)
		return ;

	avp = http_args (r);

	CALL_HTTPVARS_ACTION(s,avp);
        
	handle_global_settings(avp, s->nvram);

	parser = open_parser (avp, s->nvram, s->htpath) ;

	if (file_parser (parser,  htdocs_p (r, html_file)) == 0)
		return ;

	/* parser */
	run_parser (parser) ;

	avdump (avp); /* debugging */

	/* print result */
	if ((data = data_collect (parser)) != 0)
		body_append (r, data, 0);

	/* clean up */
	file_parser_end (parser);
	close_parser (parser) ;
}



/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

