/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Simon Oellrich <s.oellrich@gmx.de>                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "hlst.h"
#include "util.h"
#include "http-plugin.h"
#include "parser.h"
#include "nvalloc.h"
#include "plugin-util.h"

static void strip_end(char *s)
{
	char* p;
	for (p = s+strlen(s)-1; p>=s; p--) {
		switch(*p) {
			case '\n':
			case '\r':
			case ' ':
			case '\t':
				*p = '\0';
				break;
			default:
				return;
		}
	}
}

static void set_cur_firmware_ver(HLST* avp)
{
	char 	version[1024];
	FILE* 	f;
	
	f = fopen("/etc/version", "r");

	if (!f) {
		add_to_list(avp, "sys_firmware.cur_version", "error");
		return;
	}

	fgets(version, sizeof(version), f);
	fclose(f);

	strip_end(version);

	add_to_list(avp, "sys_firmware.cur_version", (const char*)version);
}


static void set_new_firmware_ver(HLST* avp, NVCL* nvram)
{
	int i;
	time_t t = time(NULL);
	
	/* do not check? */
	if (!get_from_list(avp, "check")) {
		add_to_list(avp, "sys_firmware.new_version",
			get_from_nvram(nvram, "sys_firmware.new_version"));
		return;
	}

	/* trigger check for new version */
	add_to_nvram(nvram, "sys_firmware.new_version", NULL);
	add_to_nvram(nvram, "sys_firmware.check", ctime(&t));
	add_to_nvram(nvram, "UPDATE.ERROR_CODE", NULL);

	for(i=0;i<15;i++) {
		char* e;
		char* v;
		
		v = get_from_nvram(nvram, "sys_firmware.new_version");
		if (v && *v) {
			add_to_list(avp, "sys_firmware.new_version", v);
			add_to_list(avp, "sys_firmware.success", "1");
			return;
		}

		e = get_from_nvram(nvram, "UPDATE.ERROR_CODE");
		if (e && *e)
			return;

		sleep(1);
	}
	add_to_nvram(nvram, "UPDATE.ERROR_CODE", "2");

}

static void start_update(NVCL* nvram)
{
	time_t t = time(NULL);
	
	/* trigger update */
	add_to_nvram(nvram, "sys_firmware.update", ctime(&t));
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) 
{
	STATE* 		s = desc;
	HLST*		avp;

	if (argc < 2)
		return ;

	/* get POST variables */
	avp = http_args(r);

	if (!is_enabled(s->nvram, "sys_firmware.configurable"))
		return;

	handle_global_settings(avp, s->nvram);

	if (get_from_list(avp, "update")) {

		/* start update */
		start_update(s->nvram);
		send_html_page(avp, s, r, argv[1]);
		return;
	}

	/* get current firmware version */
	set_cur_firmware_ver(avp);
	
	/* check for new firmware version */
	set_new_firmware_ver(avp, s->nvram);
	
	/* set error */
	add_to_list(avp, "sys_firmware.last_error", 
		get_from_nvram(s->nvram, "UPDATE.ERROR_CODE"));

	/* create next html page */
	send_html_page(avp, s, r, argv[1]);

}

void *htinit (const char *doc_path, NVCL *nv) 
{
	STATE *s = ZMALLOC(STATE);

	s->nvram  	= nv;
	s->htpath 	= doc_path;

	return s ;
}


void htcleanup (void *desc) 
{
	STATE* s = desc;

	free(s);
}



/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

