/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Henry Margies <h.margies@maxina.de> - Maxina GmbH, Neu-       *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#include "hlst.h"
#include "util.h"
#include "http-plugin.h"
#include "parser.h"
#include "nvalloc.h"
#include "plugin-util.h"
#include "logger.h"

#define SYSLOG_SIZE	16*1024

static void read_log(HLST* avp)
{
	FILE* fd;
	unsigned char* data;
	int ret;
	/* open pipe */
	fd = popen("logread", "r");
	if (NULL == fd) {
		logger(LOG_ERR, "Could not open logread");
		return;
	}

	/* get buffer for log file */
	data = (unsigned char*) zmalloc(SYSLOG_SIZE);
	if (!data) {
		pclose(fd);
		logger(LOG_ERR, "Could not get enough memory");
		return;
	}

	/* XXX This is just a hack, has to be reviewed!! */
	
	/* read logread output */
	ret = fread(data, 1, SYSLOG_SIZE, fd);

	/* save data to avp */
	add_to_list(avp, "logdata", data);

	free(data);
	pclose(fd);
}


/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

void *htinit (const char *doc_path, NVCL *nv) 
{
	STATE *s = ZMALLOC (STATE);

	s->nvram  = nv;
	s->htpath = doc_path;

	return s;
}


void htcleanup (void *desc) 
{
	STATE *s = desc ;
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) 
{
	STATE	*s = desc;
	PARSER	*parser;
	HLST	*avp;
	FILE*	pipe;
	const char *data;
	const char* html = argv[1];
	char **deleteSyslog;

	if (argc < 2)
		return;

	avp = http_args(r);

	if (!is_enabled(s->nvram, "syslog.configurable"))
		return;

	handle_global_settings(avp, s->nvram);

	deleteSyslog = (char **)find_hlst(avp, "syslog.delete", 0);

	if (deleteSyslog) {
	  logger(LOG_INFO, "deleteSyslog: %s", deleteSyslog);
	  system("/etc/init.d/syslogd stop");
	  system("/etc/init.d/syslogd start");
	}

	parser = open_parser(avp, s->nvram, s->htpath);

	if (file_parser(parser, html) == 0)
		return;

	read_log(avp);
	
	/* parser */
	run_parser(parser);

	/* print result */
	if ((data = data_collect(parser)) != 0)
		body_append(r, data, 0);

	/* clean up */
	file_parser_end(parser);
	close_parser(parser);
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
