/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: n/n (D-Link)			                          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <netinet/in.h>

#include "hlst.h"
#include "util.h"
#include "http-plugin.h"
#include "plugin-util.h"
#include "parser.h"
#include "nvalloc.h"
#include "logger.h"

/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */

#define STORE_FAILURE      	     	CUSTOM_START_ERROR + 10
#define REBOOT_FAILURE      	     	CUSTOM_START_ERROR + 11
#define RESTORE_FAILURE      	     	CUSTOM_START_ERROR + 12
#define LOAD_FAILURE                    CUSTOM_START_ERROR + 13
#define LOAD_WRONG_VERSION              CUSTOM_START_ERROR + 14

#define GROUP_LABEL			"sys_settings"

#define CONFIG_SIZE			100*1024

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

int printSettings(HLST* avp, STATE *s)
{
      const char *var, *val;
      int slot, used, total ;
      unsigned char *data;

      /* get buffer for log file */
      data = (unsigned char*) zmalloc(CONFIG_SIZE);
      if (!data) {
          logger(LOG_ERR, "Could not get enough memory");
	  handle_varError(avp, STORE_FAILURE);
          return 0;
      }

      strcpy(data, "# settings for HorstBox ");
      strcat(data, get_from_nvram(s->nvram, "sys_firmware.cur_version"));
      strcat(data, "\n");

      nvcl_putcmd (s->nvram, "slots");
      if (nvcl_rply (s->nvram, 0) < 1)
          return 1;

      if (nvcl_getavpair (s->nvram, 0, &val, 0, 0) == 0)
          return 1;

      used = atoi (val);

      if (nvcl_getavpair (s->nvram, 0, &val, 0, 0) == 0)
          return 1;

      total = atoi (val);

      for (slot = 1; slot <= total && used >= 0; slot ++) {
          char t [20];

          sprintf (t, "%u", slot);
          nvcl_putcmd    (s->nvram, "get");
          nvcl_putavpair (s->nvram, t, 0, 0);

          if (nvcl_rply (s->nvram, 0) < 1)
              continue ;

          while (nvcl_getavpair (s->nvram, &var, &val, 0, 0) != 0) {
	    if (strcmp(var, "wan_dsl.ip_address.1") != 0 && strcmp(var, "wan_dsl.ip_address.2") != 0 && 
		strcmp(var, "wan_dsl.ip_address.3") != 0 && strcmp(var, "wan_dsl.ip_address.4") != 0 &&
		strcmp(var, "wan_dsl.ip_address.5") != 0 && strcmp(var, "wan_dsl.ip_address.6") != 0 &&
		strcmp(var, "wan_dsl.ip_address.7") != 0 && strcmp(var, "wan_dsl.ip_address.8") != 0 &&
		strcmp(var, "wan_dsl.new-ip") != 0 && strcmp(var, "sys_firmware.nvramd") != 0 &&
		strcmp(var, "sys_firmware.cur_version") != 0 && strcmp(var, "sys_firmware.cur_buildv") != 0 &&
		strcmp(var, "sys_firmware.cur_serialnr") != 0 && strcmp(var, "ssh.authorized_keys") != 0 &&
		strcmp(var, "ssh.rsa_host_key") != 0 && strcmp(var, "http.ssl.cert") != 0 &&
		strcmp(var, "ssh.dss_host_key") != 0 && strcmp(var, "http.ssl.key") != 0 && 
		strcmp(var, "http.ssl.dhparam") != 0) {
              strcat(data, var);
              strcat(data, " \'");
              strcat(data, val);
              strcat(data, "\'\n");
	    }
	    used -- ;
          }
          
      }

      add_to_list(avp, "configuration", data);
      // hh TODO
      // add_to_list(avp, "Content-Disposition", "attachment; filename=HorstBox.cfg;");
      free(data);

      return 0;
}

int loadSettingsFromFile(HLST *avp, STATE *s)
{

  FILE *uploadfile, *settingsfile;
  char *buffer, *header, *match, *value;
  HLST 	*tmp_avp = NULL;

  logger(LOG_INFO, "Importing configuration from file");

  buffer = zmalloc(1024*sizeof(char *));
  if (!buffer) {
    logger(LOG_ERR, "Cannot allocate file buffer memory");
    handle_varError(avp, LOAD_FAILURE);
    return LOAD_FAILURE;
  }

  /* prepare version check */
  header = (char *)zmalloc(strlen("# settings for HorstBox ") + strlen(get_from_nvram(s->nvram, "sys_firmware.cur_version"))*sizeof(char *));
  strcpy(header, "# settings for HorstBox ");
  strcat(header, get_from_nvram(s->nvram, "sys_firmware.cur_version"));

  uploadfile = fopen("/var/tmp/mhttp-upload", "r");
  if (uploadfile == NULL) {
    free(buffer);
    logger(LOG_ERR, "Cannot open uploaded file");
    handle_varError(avp, LOAD_FAILURE);
    return LOAD_FAILURE;
  }

  /* read header */
  if (!feof(uploadfile))
    fgets(buffer, 1024, uploadfile);

  /* check validity */
  if (strcmp(stripstr(buffer), header) != 0) {
    free(buffer);
    fclose(uploadfile);
    logger(LOG_ERR, "Configuration file does not match firmware version");
    handle_varError(avp, LOAD_WRONG_VERSION);
    return LOAD_WRONG_VERSION;
  }

  /* write configuration file to temporary NVRAM storage */
  settingsfile = fopen("/mnt/HorstBox.cfg", "w");
  if (uploadfile == NULL) {
    free(buffer);
    fclose(uploadfile);
    logger(LOG_ERR, "Cannot open configuration file");
    handle_varError(avp, LOAD_FAILURE);
    return LOAD_FAILURE;
  }

  while (!feof(uploadfile)) {
    if ((fputc(fgetc(uploadfile), settingsfile)) == EOF) {
      free(buffer);
      fclose(uploadfile);
      fclose(settingsfile);
      logger(LOG_ERR, "Cannot write to configuration file");
      handle_varError(avp, LOAD_FAILURE);
      return LOAD_FAILURE;
    }
  }

  /*close files and delete temporary upload file */
  free(buffer);
  fclose(uploadfile);
  fclose(settingsfile);
  remove("/var/tmp/mhttp-upload");

  addToList(avp, "success", "1");
  addToList(avp, "load", "1");

  /* trigger factory reset and reboot */
  tmp_avp = create_hlst(1, nvram_avp_destroy, NULL);
  addToList(tmp_avp,"sys_settings.restore","true");
  if(store_into_nvram(s->nvram,tmp_avp) < 0 )
    // handle_varError(avp, LOAD_FAILURE);
  destroy_hlst(tmp_avp);

  return 0;

}
int saveSettings(STATE  *s)
{
       int error = NO_ERROR;

       //first tell nvram to store all its entries into flash
       /* update nvram */
        nvcl_putcmd(s->nvram, "store");

        if (nvcl_rply (s->nvram, 0) <= 0) {
            //errlog (s, "Cannot store nvram content: %s\n", nvcl_rply_error (s->nvram));
		//handle_varError(avp,STORE_FAILURE);
        }


       return error;
}


void initiateReboot(HLST *avp, STATE  *s)
{
	HLST 	*tmp_avp 	= NULL;
	tmp_avp 		= create_hlst(1, nvram_avp_destroy, NULL);

	//load_from_nvram(s->nvram , tmp_avp, NULL);
	addToList(tmp_avp,"sys_settings.save_and_reboot","true");

	if(store_into_nvram(s->nvram,tmp_avp) < 0 )
		//handle_varError(tmp_avp,REBOOT_FAILURE);

	destroy_hlst(tmp_avp);

	addToList(avp, "success", "1");
}

void initiateRestore(HLST *avp, STATE  *s)
{
	HLST 	*tmp_avp 	= NULL;
	tmp_avp 		= create_hlst(1, nvram_avp_destroy, NULL);

	//load_from_nvram(s->nvram , tmp_avp, NULL);
	addToList(tmp_avp,"sys_settings.restore","true");

	if(store_into_nvram(s->nvram,tmp_avp) < 0 )
		//handle_varError(tmp_avp,RESTORE_FAILURE);

	destroy_hlst(tmp_avp);

	addToList(avp, "success", "1");
}



HTMLOBJECT *init_htmlVars(){

	int max_size = 3;
	int i=0;
	int max_index = max_size - 1;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);

	p[i].revise 	= noError;
	p[i++].name	= "filename";

	p[i].revise 	= noError;
	p[i++].name	= "import";

        //sentinel
	p[i>max_index ? max_index : i].revise 	= NULL;
	p[i>max_index ? max_index : i].name	= NULL;

	return p;
}

void free_htmlVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}

void handle_varError(HLST *avp, int ecode){

 
	switch(ecode){

        case STORE_FAILURE:		addToList(avp, "sys_settings.save.error", "1"); 
					break;
        case LOAD_FAILURE:		addToList(avp, "sys_settings.load.error", "1"); 
					break;
        case LOAD_WRONG_VERSION:	addToList(avp, "sys_settings.version.error", "1"); 
					break;

	default	:      			//UNKNOWN error code, nothing to do
					logger (LOG_WARNING, "UNKNOWN error code: %d: ", ecode);
					return;
	}


	addToList(avp, "error", "1");
    
	return;

}

void *htinit (const char *doc_path, NVCL *nv) {
	STATE *s = ZMALLOC (STATE) ;

	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= NULL;
	s->htmlvars.group_label			= GROUP_LABEL;
	s->htmlvars.total_elements_label	= NULL;

	s->nvram  	= nv ;
	s->htpath 	= doc_path ;

	return s ;
}

void htcleanup (void *desc) {
	STATE *s = desc ;

	free_htmlVars(s->htmlvars.objects);
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
    
	STATE  *s = desc ;
	PARSER   *parser ;
	HLST        *avp ;
	const char *data ;
	char *html_file = (char*)argv[1];

	int error = NO_ERROR;

	if (argc < 2)
		return ;

	avp = http_args (r);

	if(find_hlst(avp,"save",0))
	{
		if((error = saveSettings(s)) != NO_ERROR)
			handle_varError(avp, error);
	}
	else if(find_hlst(avp,"save_and_reboot",0))
	{
		if((error = saveSettings(s)) != NO_ERROR)
			handle_varError(avp, error);
		//now set the variable to trigger the reboot of the box
		initiateReboot(avp,s);
	}
	else if(find_hlst(avp,"restore",0))
	{
		initiateRestore(avp,s);
	}
	else if(find_hlst(avp,"export",0))
	{
	        printSettings(avp,s);
	}
	else if(find_hlst(avp,"import",0))
	{
	        loadSettingsFromFile(avp, s);
	}

	handle_global_settings(avp, s->nvram);

	parser = open_parser (avp, s->nvram, s->htpath) ;

	if (file_parser (parser,  htdocs_p (r, html_file)) == 0)
		return ;

	/* parser */
	run_parser (parser) ;

	avdump (avp); /* debugging */

	/* print result */
	if ((data = data_collect (parser)) != 0)
		body_append (r, data, 0);

	/* clean up */
	file_parser_end (parser);
	close_parser (parser) ;
}



/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

