/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: n/n (D-Link)			                          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <time.h>

#include "hlst.h"
#include "util.h"
#include "http-plugin.h"
#include "plugin-util.h"
#include "parser.h"
#include "nvalloc.h"
#include "logger.h"


/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */


#define INVALID_IP_ADDRESS		CUSTOM_START_ERROR + 10
#define INVALID_YEAR			CUSTOM_START_ERROR + 20
#define INVALID_MONTH			CUSTOM_START_ERROR + 30
#define INVALID_DAY			CUSTOM_START_ERROR + 40
#define INVALID_HOUR			CUSTOM_START_ERROR + 50
#define INVALID_MINUTE			CUSTOM_START_ERROR + 60
#define INVALID_SECOND			CUSTOM_START_ERROR + 70
#define INVALID_TIMESTAMP		CUSTOM_START_ERROR + 80
#define INVALID_NTP_SERVER		CUSTOM_START_ERROR + 90

typedef enum{
	TIME_MODE_NOT_SET,
	TIME_MODE_AUTOMATIC,
	TIME_MODE_COMPUTER,
	TIME_MODE_MANUAL
}TIME_MODE;


/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

TIME_MODE getTimeMode(HLST *avp)
{
	char **val = (char**)find_hlst(avp, "sys_time.time", 0);

	if(val)
	{
		if(!strcmp("automatic",*val))
			return TIME_MODE_AUTOMATIC;
		if(!strcmp("computer",*val))
			return TIME_MODE_COMPUTER;
		if(!strcmp("manual",*val))
			return TIME_MODE_MANUAL;
	}


	return TIME_MODE_NOT_SET;
}

int isDaylightSavingEnabled(HLST *avp)
{
	char **val = (char**)find_hlst(avp, "sys_time.daylightsaving", 0);

	if(val && !strcmp(*val, "true"))
	{
		return 1;
	}

	return 0;
}


int checkNtpServer(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;

	/* a selected TIME_MODE_AUTOMATIC radio option is required for a new NTP server value */
	if(getTimeMode(avp)==TIME_MODE_AUTOMATIC)
	{
		char **val = (char**)find_hlst(avp, varName, 0);

		if(!val || !strlen(removeBlanks(*val)))
		{
			error = INVALID_NTP_SERVER;
		}
	}
	else
	{
		delete_hlst(avp,"sys_time.ntpServer",0);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}



int checkDaylightSaving(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	if(getTimeMode(avp)==TIME_MODE_AUTOMATIC)
	{
		ensureCheckState(avp, nvram_avp, varName);
	}

	return NO_ERROR;
}

int createTimestamp(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	TIME_MODE mode = getTimeMode(avp);

        if(mode==TIME_MODE_MANUAL || mode==TIME_MODE_COMPUTER)
	{
		struct tm newTime;
		//struct tm saveTime;
		time_t timestamp;
	       
		memset(&newTime, 0, sizeof(struct tm));

		if(mode==TIME_MODE_MANUAL)
		{
			newTime.tm_year = getVarAsLong(avp, "sys_time.year")-1900;
			newTime.tm_mon 	= getVarAsLong(avp, "sys_time.month")-1;
			newTime.tm_mday = getVarAsLong(avp, "sys_time.day");
			newTime.tm_hour = getVarAsLong(avp, "sys_time.hour");
			newTime.tm_min 	= getVarAsLong(avp, "sys_time.minute");
			newTime.tm_sec 	= getVarAsLong(avp, "sys_time.second");
		}
		else
		{
			newTime.tm_year = getVarAsLong(avp, "sys_time.jsYear")-1900;
			newTime.tm_mon 	= getVarAsLong(avp, "sys_time.jsMonth")-1;
			newTime.tm_mday = getVarAsLong(avp, "sys_time.jsDay");
			newTime.tm_hour = getVarAsLong(avp, "sys_time.jsHour");
			newTime.tm_min 	= getVarAsLong(avp, "sys_time.jsMinute");
			newTime.tm_sec 	= getVarAsLong(avp, "sys_time.jsSecond");
		}

		//memcpy(&saveTime, &newTime, sizeof(struct tm));

		/* mktime sets the tm struct to valid date (e.g. 29.02.2006 -> 01.03.2006) */
		timestamp = mktime(&newTime);

		if(timestamp == -1 )
		{
			error = INVALID_TIMESTAMP;
		}
		else
		{
			char timeBuffer[20];
			sprintf(timeBuffer,"%d",timestamp);
			forceAddToList(avp, varName, timeBuffer);
		}

	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}


int createDate(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	TIME_MODE mode = getTimeMode(avp);

	if(mode==TIME_MODE_MANUAL || mode==TIME_MODE_COMPUTER)
	{
		if(getVarAsLong(avp, "sys_time.timestamp") > 0)
		{
			char buf[16];
			long year;
			long month;
			long day;
			long hour;
			long minute;
			long second;

			if(mode==TIME_MODE_MANUAL)
			{
				year   = getVarAsLong(avp, "sys_time.year");
				month  = getVarAsLong(avp, "sys_time.month");
				day    = getVarAsLong(avp, "sys_time.day");
				hour   = getVarAsLong(avp, "sys_time.hour");
				minute = getVarAsLong(avp, "sys_time.minute");
				second = getVarAsLong(avp, "sys_time.second");
			}
			else
			{
				year   = getVarAsLong(avp, "sys_time.jsYear");
				month  = getVarAsLong(avp, "sys_time.jsMonth");
				day    = getVarAsLong(avp, "sys_time.jsDay");
				hour   = getVarAsLong(avp, "sys_time.jsHour");
				minute = getVarAsLong(avp, "sys_time.jsMinute");
				second = getVarAsLong(avp, "sys_time.jsSecond");
			}
	
			sprintf(buf, "%02d%02d%02d%02d%04d.%02d", month, day, hour, minute, year, second);
	
			forceAddToList(avp, varName, buf);
		}
		else
		{
			error = INVALID_TIMESTAMP;
		}
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}


int checkYear(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	if(getTimeMode(avp)==TIME_MODE_MANUAL)
	{
		long year = getVarAsLong(avp, varName)-1900;
	   
		if(year < 70 || year > 138)
		{
			error = INVALID_YEAR;
		}
	}
	else
	{
		delete_hlst(avp, varName, 0);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkMonth(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	if(getTimeMode(avp)==TIME_MODE_MANUAL)
	{
		long month = getVarAsLong(avp, varName);
	   
		if(month < 1 || month > 12)
		{
			error = INVALID_MONTH;
		}
	}
	else
	{
		delete_hlst(avp, varName, 0);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int is_leapyear(const unsigned year){

	if ((year % 400) == 0)
		return 1;

	if ((year % 100) == 0)
		return 0;
	
	if ((year % 4)   == 0)
		return 1;

	return 0;
}

short getDaysOfMonth(const int month, const int year)
{	//		     Jan,Feb,Mar,Apr,May,Jun,Jul,Aug,Sep,Oct,Nov,Dec
	short days[13] = { 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

	if (month == 2 && is_leapyear(year)) 
		return 29;

	if (month > 0 && month < 13)
		return days[month];

	return 0;
}

int checkDay(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	if(getTimeMode(avp)==TIME_MODE_MANUAL)
	{
		int day = getVarAsLong(avp, varName);
	  	int month = getVarAsLong(avp, "sys_time.month");
		int year = getVarAsLong(avp, "sys_time.year");

		if(day < 1 || day > getDaysOfMonth(month, year))
		{
			error = INVALID_DAY;
		}
	}
	else
	{
		delete_hlst(avp, varName, 0);
	}


	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkHour(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	if(getTimeMode(avp)==TIME_MODE_MANUAL)
	{

		long hour = getVarAsLong(avp, varName);
	   
		if(hour < 0 || hour > 23)
		{
			error = INVALID_HOUR;
		}
	}
	else
	{
		delete_hlst(avp, varName, 0);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkMinute(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	if(getTimeMode(avp)==TIME_MODE_MANUAL)
	{

		long minute = getVarAsLong(avp, varName);
	   
		if(minute < 0 || minute > 59)
		{
			error = INVALID_MINUTE;
		}
	}
	else
	{
		delete_hlst(avp, varName, 0);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkSecond(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	if(getTimeMode(avp)==TIME_MODE_MANUAL)
	{

		long second = getVarAsLong(avp, varName);
	   
		if(second < 0 || second > 59)
		{
			error = INVALID_SECOND;
		}
	}
	else
	{
		delete_hlst(avp, varName, 0);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

HTMLOBJECT *init_htmlVars(){

	int max_size = 13;
	int i=0;
	int max_index = max_size - 1;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);

	p[i].revise 	= ensureRadioChecked;
	p[i++].name	= "sys_time.time";

	p[i].revise 	= noError;
	p[i++].name	= "sys_time.timezone";

	p[i].revise 	= checkDaylightSaving;
	p[i++].name	= "sys_time.daylightsaving";

	p[i].revise 	= checkNtpServer;
	p[i++].name	= "sys_time.ntpServer";

	p[i].revise 	= checkYear;
	p[i++].name	= "sys_time.year";

	p[i].revise 	= checkMonth;
	p[i++].name	= "sys_time.month";

	p[i].revise 	= checkDay;
	p[i++].name	= "sys_time.day";

	p[i].revise 	= checkHour;
	p[i++].name	= "sys_time.hour";

	p[i].revise 	= checkMinute;
	p[i++].name	= "sys_time.minute";

	p[i].revise 	= checkSecond;
	p[i++].name	= "sys_time.second";

	p[i].revise 	= createTimestamp;
	p[i++].name	= "sys_time.timestamp";

	p[i].revise 	= createDate;
	p[i++].name	= "sys_time.date";

	//sentinel
	p[i>max_index ? max_index : i].revise 	= NULL;
	p[i>max_index ? max_index : i].name	= NULL;

	return p;
}

void free_htmlVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}



void handle_varError(HLST *avp, int ecode){

 
	switch(ecode){
	case INVALID_IP_ADDRESS:   		addToList(avp, "sys_time.ntpServer.error", "1"); 
						break;

	case INVALID_NTP_SERVER:   		addToList(avp, "sys_time.ntpServer.error", "1"); 
						break;

	case INVALID_YEAR:   			addToList(avp, "sys_time.year.error", "1"); 
						break;

	case INVALID_MONTH:   			addToList(avp, "sys_time.month.error", "1"); 
						break;

	case INVALID_DAY:   			addToList(avp, "sys_time.day.error", "1"); 
						break;

	case INVALID_HOUR:   			addToList(avp, "sys_time.hour.error", "1"); 
						break;

	case INVALID_MINUTE:   			addToList(avp, "sys_time.minute.error", "1"); 
						break;

	case INVALID_SECOND:   			addToList(avp, "sys_time.second.error", "1"); 
						break;

	case INVALID_TIMESTAMP: 		addToList(avp, "sys_time.timestamp.error", "1"); 
						break;

	default	:				//UNKNOWN error code, nothing to do
						logger (LOG_WARNING, "UNKNOWN error code: %d: ", ecode);
						return;
	}


	addToList(avp, "error", "1");
    
	return;

}


void *htinit (const char *doc_path, NVCL *nv) {
	STATE *s = ZMALLOC (STATE) ;

	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= NULL;
	s->htmlvars.group_label			= NULL;
	s->htmlvars.total_elements_label	= NULL;

	s->nvram  	= nv ;
	s->htpath 	= doc_path ;
	return s ;
}

void htcleanup (void *desc) {
	STATE *s = desc ;

	free_htmlVars(s->htmlvars.objects);
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
    
	STATE  *s = desc ;
	PARSER   *parser ;
	HLST        *avp ;
	const char *data ;
	char *html_file = (char*)argv[1];

	if (argc < 2)
		return ;

	avp = http_args (r);

	CALL_HTTPVARS_ACTION(s,avp);
        
	handle_global_settings(avp, s->nvram);

	parser = open_parser (avp, s->nvram, s->htpath) ;

	if (file_parser (parser,  htdocs_p (r, html_file)) == 0)
		return ;

	/* parser */
	run_parser (parser) ;

	avdump (avp); /* debugging */

	/* print result */
	if ((data = data_collect (parser)) != 0)
		body_append (r, data, 0);

	/* clean up */
	file_parser_end (parser);
	close_parser (parser) ;
}



/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

