/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: n/n (D-Link)			                          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "hlst.h"
#include "util.h"
#include "http-plugin.h"
#include "parser.h"
#include "nvalloc.h"
#include "plugin-util.h"

#ifdef HAVE_STRING_H
# include <string.h>
#endif

/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */


/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

HTMLOBJECT *init_htmlVars(){

	int max_size = 1;
	int i=0;
	int max_index = max_size - 1;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);


	//sentinel
	p[i>max_index ? max_index : i].revise 	= NULL;
	p[i>max_index ? max_index : i].name	= NULL;

	return p;
}


void free_htmlVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}


void handle_varError(HLST *avp, int ecode){

 
	switch(ecode){
	
	default	:				//UNKNOWN error code, nothing to do
						return;
	}

	addToList(avp, "error", "1");
    
	return;

}

void *htinit (const char *doc_path, NVCL *nv) {
    STATE *s = ZMALLOC (STATE) ;

    s->htmlvars.objects = init_htmlVars();
    s->htmlvars.load_on_success 	 = NULL; // phone/phone_calls_choose.html
    s->htmlvars.total_elements_label = NULL; // phone_accounts.totalElements
    s->htmlvars.group_label 		 = NULL; // phone_calls

    s->nvram  	= nv ;
    s->htpath 	= doc_path ;
    return s ;
}

void htcleanup (void *desc) {
    STATE *s = desc ;

	free_htmlVars(s->htmlvars.objects);
}


void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
    STATE  *s = desc ;
    PARSER   *parser ;
    HLST        *avp ;
    const char *data ;

    if (argc < 2)
        return ;

    avp = http_args (r);
	
    handle_global_settings(avp, s->nvram);

	//process_httpargs(avp, s->httpvars);
        
    parser = open_parser (avp, s->nvram, s->htpath) ;

    if (file_parser (parser, htdocs_p (r, argv [1])) == 0)
        return ;

    /* parser */
    run_parser (parser) ;

    avdump (avp); /* debugging */

    /* print result */
    if ((data = data_collect (parser)) != 0)
        body_append (r, data, 0);

    /* clean up */
    file_parser_end (parser);
    close_parser (parser) ;
}



/*

void *htinit (const char *doc_path) {
    STATE *s = XMALLOC (STATE) ;

    s->avp    = create_hlst (300, destroy, 0) ;
    s->parser = open_parser (s->avp, doc_path) ;


	s->httpvar = create_httpVars();

    return s ;
}

void htcleanup (void *desc) {
    STATE *s = desc ;

	free_httpVars(s->httpvar);
    close_parser (s->parser) ;
    destroy_hlst (s->avp) ;

}




void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {



	STATE *s = desc ;
	const char *data ;
    char **retVal ;

	int c, error;

   

    if (argc < 2)
        return ;


	avp = http_args (r);

	c = -1;

	while(s->httpvar[++c].name)
	{

		retVal = (char**)find_hlst(hlst,s->httpvar[c].name,0);


		if(retVal)
		{
			
			error = s->httpvar[c].evaluate(retVal);
			if(error != NO_ERROR)
			{
				handle_varError(s->avp, error);
			}
			else
			{
				//success
				//we have to store it to nvram
			}

			addToList(s->avp, s->httpvar[c].name, *retVal);
		}
		else
		{
			//read variable from nvram
            

			addToList(s->avp, s->httpvar[c].name, "briged-vcmux");
		}

	}

  
    if (file_parser (s->parser, htdocs_p (r, argv [1])) == 0)
        return ;
    


	run_parser (s->parser) ;

   
    if ((data = data_collect (s->parser)) != 0)
        body_append (r, data, 0);

    file_parser_end (s->parser);
    flush_hlst (s->avp, 0, 0);
}

*/
/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

