/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Henry Margies <h.margies@maxina.de> - Maxina GmbH, Neu-       *
*   Isenburg, Germany                                                     *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "hlst.h"
#include "util.h"
#include "http-plugin.h"
#include "nvalloc.h"
#include "parser.h"
#include "plugin-util.h"

static void set_url(NVCL* nvram, char* name, char* ip, char* port, char* file)
{
	char url[1024];
	snprintf(url, sizeof(url), "http://%s:%s/%s", ip, port, file);
	add_to_nvram(nvram, name, url);
}

static void start_update(HLST* avp, NVCL* nvram, char* ip)
{
	char *port, *dsl_version;
	time_t t = time(NULL);

	port  = get_from_list(avp, "port");

	if (!port)
		return;

	add_to_nvram(nvram, "UPDATE.CONFIG_URL",	"not_in_use");
	
	set_url(nvram, "UPDATE.ZIMAGE_URL",	ip, port, "zImage");
	set_url(nvram, "UPDATE.MAIN_FS_URL", 	ip, port, "main-fs");
	set_url(nvram, "UPDATE.UPDATE_FS_URL", 	ip, port, "update-fs");
	set_url(nvram, "UPDATE.OPT_FS_URL", 	ip, port, "opt-fs");
	set_url(nvram, "UPDATE.FINISH_URL", 	ip, port, "finish");

	dsl_version   = get_from_list(avp, "dsl_version");
	
	if (dsl_version) {
		set_url(nvram, "UPDATE.DSL_FS_URL", ip, port, "dsl-image");
		add_to_nvram(nvram, "UPDATE.DSL_VERSION", dsl_version);
	}

	add_to_nvram(nvram, "sys_firmware.update", ctime(&t));
	add_to_list(avp, "OK", "OK");
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) 
{
	STATE* 		s = desc;
	HLST*		avp;

	if (argc < 2)
		return ;

	/* get POST variables */
	avp = http_args(r);

	handle_global_settings(avp, s->nvram);

	if (!is_enabled(s->nvram, "sys_firmware.local_update.configurable"))
		return;

	/* start update */
	start_update(avp, s->nvram, ip4addr_pp(http_client(r)));
	
	/* create next html page */
	send_html_page(avp, s, r, argv[1]);

}

void *htinit (const char *doc_path, NVCL *nv) 
{
	STATE *s = ZMALLOC(STATE);

	s->nvram  	= nv;
	s->htpath 	= doc_path;

	return s ;
}


void htcleanup (void *desc) 
{
	STATE* s = desc;

	free(s);
}



/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

