/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Susanne Goldammer <s.goldammer@maxina.de> - Maxina GmbH,      *
*   Neu-Isenburg, Germany						  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "hlst.h"
#include "util.h"
#include "http-plugin.h"
#include "nvalloc.h"
#include "parser.h"
#include "plugin-util.h"
#include "logger.h"


/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */
#define USERNAME_EMPTY 				CUSTOM_START_ERROR + 10
#define PASSWORD_EMPTY				CUSTOM_START_ERROR + 11
#define PASSWORD_NONEQUAL			CUSTOM_START_ERROR + 12
#define TIME_NEGATIVE				CUSTOM_START_ERROR + 20

#define INVALID_VPI			CUSTOM_START_ERROR + 30
#define INVALID_VCI			CUSTOM_START_ERROR + 40
#define INVALID_MTU			CUSTOM_START_ERROR + 50
#define INVALID_MRU			CUSTOM_START_ERROR + 60

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

/*
typedef enum{
	WAN_SETTINGS_NOT_SET,
	WAN_SETTINGS_STATIC = 11,
	WAN_SETTINGS_PPP,
	WAN_SETTINGS_DYNAMIC,
	WAN_SETTINGS_BRIDGE
}WAN_SETTINGS;


WAN_SETTINGS getWanSetting(HLST *avp)
{
	char **val = (char**)find_hlst(avp, "wan_dsl.wanSettings", 0);

	if(val)
	{
		if(!strcmp("static",*val))
			return WAN_SETTINGS_STATIC;
		if(!strcmp("ppp",*val))
			return WAN_SETTINGS_PPP;
		if(!strcmp("dynamic",*val))
			return WAN_SETTINGS_DYNAMIC;
		if(!strcmp("bridge",*val))
			return WAN_SETTINGS_BRIDGE;

	}

	return WAN_SETTINGS_NOT_SET;
}
*/

/* check if the user wants to use a specific username and passwort for dsl access */
int isAccessChecked(HLST *avp)
{
	char **retVal;
	retVal = (char**)find_hlst(avp, "wan_dsl.virtualCircuit.1", 0);

	if(retVal && !strcmp(*retVal,"enabled"))
		return 1;
	else
		return 0;
}


/* check username of the dsl access data */
int checkUsername(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **hasAccess;
	char **val = (char**)find_hlst(avp, varName, 0);

	//just check the username if the user wants to use specific dsl access data */
	if(isAccessChecked(avp) && (strlen(*val)==0 || strlen(*val) > 64) )
		error = USERNAME_EMPTY;

	if(error != NO_ERROR)
		handle_varError(avp, error);


	return error;
}

int checkPassword(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **passwd1;
	char **passwd2;
	int len1 = 0;
	int len2 = 0;
	

	/* just check the password if the user wants to use specific dsl access data */
	if(!isAccessChecked(avp))
		return NO_ERROR;

	passwd1 = (char**)find_hlst(avp, varName, 0);

	//if(!passwd1 || (len1=strlen(*passwd1)) == 0 || len1 > 64 )
	if(!passwd1 || (len1=strlen(*passwd1)) > 64 )
		error = PASSWORD_EMPTY;			//first password was empty

	passwd2 = (char**)find_hlst(avp, "wan_dsl.password2.1", 0);
	
	//if(!passwd2 || (len2=strlen(*passwd2)) == 0 || len2 > 64 )	
	if(!passwd2 || (len2=strlen(*passwd2)) > 64 )	
		error = PASSWORD_EMPTY;			//second password was empty

	/* if one of the passwords was empty, set an error message and do not check if both passwords are equal or not */
	if(error != NO_ERROR)
	{
		handle_varError(avp, error);
		return error;
	}


	if(len1 != len2 || strcmp(*passwd1,*passwd2))
		error = PASSWORD_NONEQUAL;		//password and confirmPassword were nonequal


	if(error != NO_ERROR)
		handle_varError(avp, error);
	else if(len1 == 0)
                forceAddToList(avp,varName,"\"\"");

	return error;
}


/* check whether the user wants to keep his/her dsl connection alive or if he/she wants to disconnect after a specific time (in minutes)*/
int checkForDisconnect(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **val = (char**)find_hlst(avp, varName, 0);

	/* disconnect connection after a certain time? */
	if(strncmp(*val,"disconnect",strlen("disconnect"))==0)
	{
		char **disconnectTime;
		long i;
		char *s;

		/* get time in seconds after we should disconnect */
		disconnectTime = (char**)find_hlst(avp,"wan_dsl.disconnectTime.1",0);

		if(disconnectTime)
		{
			i = strtol(*disconnectTime,&s,10);
			if(*s != '\0' || i <= 0 || i > 1092 )
				error = TIME_NEGATIVE;
		}

	}
	else if(strncmp(*val,"keepalive",strlen("keepalive"))==0)
	{
		//nothing important to check here
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkAccess(HLST *avp, HLST *nvram_avp, const char *varName)
{
	if(!isAccessChecked(avp))
		addToList(avp, varName, "disabled"); 

	return NO_ERROR;
}


int checkVpi(HLST *avp, HLST *nvram_avp, const char *varName){

	long 	i;
	char 	*s, *existVar;
	int 	error = NO_ERROR;
	char	**val;

	if(!isAccessChecked(avp))
		return NO_ERROR;

        
	if(val = (char**)find_hlst(avp, varName, 0))
	{
	
		i = strtol(*val, &s, 10);
	
		if(**val == '\0' || *s != '\0' || i < 0 || i > 255 )
		{
			error = INVALID_VPI;
		}
	}
	else
		error = INVALID_VPI;
        
	if(error != NO_ERROR)
		handle_varError(avp, error);
   

	return error;
}

int checkVci(HLST *avp, HLST *nvram_avp, const char *varName){

	long i;
	char *s, *existVar;
	int error = NO_ERROR;
	char **val;

	if(!isAccessChecked(avp))
		return NO_ERROR;
       

	if( val = (char**)find_hlst(avp, varName, 0))
	{
		i = strtol(*val, &s, 10);
	
		if(**val == '\0' || *s != '\0' || i < 32 || i > 65535 )
		{
			error = INVALID_VCI;
		} 
		/*else if(!find_hlst(avp,"wan_dsl.vpi.error",0))
		{
			
			error = checkVpc(avp, nvram_avp, varName);
	
		}*/
	}
	else
		error = INVALID_VCI;


	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkMtu(HLST *avp, HLST *nvram_avp, const char *varName)
{
	long i;
	char *s;
	int error = NO_ERROR;
	char **val;

	/*if((getWanSetting(avp) == WAN_SETTINGS_NOT_SET) ||
	   (getWanSetting(avp) == WAN_SETTINGS_BRIDGE))
		return NO_ERROR;*/

	 if(!isAccessChecked(avp))
		return NO_ERROR;


	if(val = (char**)find_hlst(avp, varName, 0))
	{

		i = strtol(*val, &s, 10);
	
		if(**val == '\0' || *s != '\0' || i < 128 || i > 65535 )
		{
			error = INVALID_MTU;
		}
	}
	else
		error = INVALID_MTU;


	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkMru(HLST *avp, HLST *nvram_avp, const char *varName)
{
	long i;
	char *s;
	int error = NO_ERROR;
	char **val;

       /* if((getWanSetting(avp) != WAN_SETTINGS_PPP))
		return NO_ERROR;
		*/
	if(!isAccessChecked(avp))
		return NO_ERROR;


	if(val = (char**)find_hlst(avp, varName, 0))
	{

		i = strtol(*val, &s, 10);
	
		if(**val == '\0' || *s != '\0' || i < 128 || i > 1500 )
		{
			error = INVALID_MRU;
		}
	}
	else
		error = INVALID_MRU;


	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkForForceNewConnection(HLST *avp, HLST *nvram_avp, const char *varName)
{
	if (!get_from_list(avp, varName))
		add_to_list(avp, varName, "false");

	return NO_ERROR;
}



HTMLOBJECT *init_htmlVars(){

	int max_size = 13;
	int i=0;
	int max_index = max_size - 1;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);


	p[i].revise 	= checkAccess;
	p[i++].name	= "wan_dsl.virtualCircuit.1";

	p[i].revise 	= checkUsername;
	p[i++].name	= "wan_dsl.username.1";

	p[i].revise 	= checkPassword;
	p[i++].name	= "wan_dsl.password.1";

	p[i].revise 	= checkForDisconnect;
	p[i++].name	= "wan_dsl.connection.1";

	p[i].revise 	= noError;
	p[i++].name	= "wan_dsl.disconnectTime.1";

	p[i].revise 	= checkForForceNewConnection;
	p[i++].name	= "wan_dsl.forceNewConnection.1";

	p[i].revise 	= noError;
	p[i++].name	= "wan_dsl.forceNewConnectionTime.1";

	p[i].revise 	= checkVpi;
	p[i++].name	= "wan_dsl.vpi.1";

	p[i].revise 	= checkVci;
	p[i++].name	= "wan_dsl.vci.1";

	p[i].revise 	= checkMtu;
	p[i++].name	= "wan_dsl.mtu.1";

	p[i].revise 	= checkMru;
	p[i++].name	= "wan_dsl.mru.1";

	p[i].revise     = noError;
	p[i++].name     = "wan_dsl.modulation";
	
	//sentinel
	p[i>max_index ? max_index : i].revise 	= NULL;
	p[i>max_index ? max_index : i].name	= NULL;

	return p;
}

void free_htmlVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}



void handle_varError(HLST *avp, int ecode){

 
	switch(ecode){

	case TIME_NEGATIVE:			addToList(avp, "wan_access.disconnectTime.error", "1");
						break;	

	case USERNAME_EMPTY:			addToList(avp, "wan_access.username.error", "1");
						break;

	case PASSWORD_EMPTY:			addToList(avp, "wan_access.password.empty.error", "1");
						break;

	case PASSWORD_NONEQUAL:			addToList(avp, "wan_access.password.nonequal.error", "1");
						break;

	case INVALID_VPI:			addToList(avp, "wan_dsl.vpi.error", "1"); 
						break;

	case INVALID_VCI:	 	  	addToList(avp, "wan_dsl.vci.error", "1"); 
						break;

	case INVALID_MTU:			addToList(avp, "wan_dsl.mtu.error", "1"); 
						break;

	case INVALID_MRU:			addToList(avp, "wan_dsl.mru.error", "1"); 
						break;


	default	:				//UNKNOWN error code, nothing to do
						logger (LOG_WARNING, "UNKNOWN error code: %d: ", ecode);
						return;
	}


	addToList(avp, "error", "1");
    
	return;

}

void *htinit (const char *doc_path, NVCL *nv) {
	STATE *s = ZMALLOC (STATE) ;


	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= NULL;
	s->htmlvars.total_elements_label 	= NULL;
	s->htmlvars.group_label 		= NULL;

	s->nvram  	= nv ;
	s->htpath 	= doc_path ;

	return s ;
}

void htcleanup (void *desc) {
	STATE *s = desc ;

	free_htmlVars(s->htmlvars.objects);
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
	STATE  *s = desc ;
	PARSER   *parser ;
	HLST        *avp ;
	const char *data ;

	if (argc < 2)
		return ;

	avp = http_args (r);

	if (!is_enabled(s->nvram, "wan_dsl.configurable"))
		return;

	CALL_HTTPVARS_ACTION(s,avp);
        
	handle_global_settings(avp, s->nvram);

	parser = open_parser (avp, s->nvram, s->htpath) ;

	if (file_parser (parser, argv [1]) == 0)
		return ;

	/* parser */
	run_parser (parser) ;

	avdump (avp); /* debugging */

	/* print result */
	if ((data = data_collect (parser)) != 0)
		body_append (r, data, 0);

	/* clean up */
	file_parser_end (parser);
	close_parser (parser) ;
}


/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

