/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: n/n (D-Link)			                          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "hlst.h"
#include "http-plugin.h"
#include "parser.h"
#include "nvalloc.h"
#include "plugin-util.h"
#include "logger.h"


/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */
#define INVALID_IPADDRESS   		CUSTOM_START_ERROR + 10


/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

int isDemilZoneChecked(HLST *avp)
{
	char **retVal;
	retVal = (char**)find_hlst(avp, "wan_dmz.has_demilitarizedZone", 0);

	if(retVal && !strcmp(*retVal,"true"))
		return 1;
	else
		return 0;
}

int checkIpAddress(HLST *avp, HLST *nvram_avp, const char *varName)
{
	char **retVal;
	char **ip;
	int error = NO_ERROR;

    
	if(isDemilZoneChecked(avp))
	{
		ip = (char**)find_hlst(avp, varName, 0);
		if(!ip || !validIpAddr(ip))
		{
			error = INVALID_IPADDRESS;
		}
	}
	else
	{
		delete_hlst(avp, "wan_dmz.ipAddress", 0);
	}
    
	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}


HTMLOBJECT *init_htmlVars(){

	int max_size = 3;
	int i=0;
	int max_index = max_size - 1;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);

	p[i].revise 	= ensureCheckState;
	p[i++].name 	= "wan_dmz.has_demilitarizedZone";

	p[i].revise 	= checkIpAddress;
	p[i++].name 	= "wan_dmz.ipAddress";

  
	//sentinel
	p[i>max_index ? max_index : i].revise 	= NULL;
	p[i>max_index ? max_index : i].name		= NULL;

	return p;
}

void free_htmlVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}


void handle_varError(HLST *avp, int ecode){

 
	switch(ecode){
	case INVALID_IPADDRESS:		addToList(avp, "wan_dmz.ipAddress.error", "1"); 
				       	break;

	default	:		       	//UNKNOWN error code, nothing to do
					logger (LOG_WARNING, "UNKNOWN error code: %d: ", ecode);
					return;
	}
	addToList(avp, "error", "1");
    
	return;

}


void *htinit (const char *doc_path, NVCL *nv) {
	STATE *s = ZMALLOC (STATE) ;

	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= NULL;
	s->htmlvars.total_elements_label 	= NULL;
	 s->htmlvars.group_label 		= NULL;

	 s->nvram  	= nv ;
	 s->htpath 	= doc_path ;

	 return s ;
}

void htcleanup (void *desc) {
	STATE *s = desc ;

	free_htmlVars(s->htmlvars.objects);
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
	STATE  *s = desc ;
        PARSER   *parser ;
	HLST        *avp ;
	const char *data ;

	if (argc < 2)
		return ;

	avp = http_args (r);

	CALL_HTTPVARS_ACTION(s,avp);
        
	handle_global_settings(avp, s->nvram);

	parser = open_parser (avp, s->nvram, s->htpath) ;

	if (file_parser (parser, htdocs_p (r, argv [1])) == 0)
		return ;

	/* parser */
	run_parser (parser) ;

	avdump (avp); /* debugging */

	/* print result */
	if ((data = data_collect (parser)) != 0)
		body_append (r, data, 0);

	/* clean up */
	file_parser_end (parser);
	close_parser (parser) ;
}



/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

