/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: n/n (D-Link)			                          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <netinet/in.h>

#include "hlst.h"
#include "util.h"
#include "http-plugin.h"
#include "plugin-util.h"
#include "parser.h"
#include "nvalloc.h"
#include "logger.h"


/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */

#define INVALID_VPI			CUSTOM_START_ERROR + 10
#define INVALID_VCI			CUSTOM_START_ERROR + 20
#define INVALID_HOSTNAME		CUSTOM_START_ERROR + 30
#define INVALID_IP			CUSTOM_START_ERROR + 40
#define INVALID_SUBNETMASK		CUSTOM_START_ERROR + 50
#define INVALID_GATEWAY			CUSTOM_START_ERROR + 60
#define INVALID_PRIMARYDNS		CUSTOM_START_ERROR + 70
#define INVALID_MTU			CUSTOM_START_ERROR + 80
#define INVALID_MRU			CUSTOM_START_ERROR + 90
#define INVALID_PCR			CUSTOM_START_ERROR + 100
#define INVALID_SCR			CUSTOM_START_ERROR + 110
#define INVALID_STATICIP		CUSTOM_START_ERROR + 120
#define INVALID_MACADDRESS		CUSTOM_START_ERROR + 130
#define INVALID_WANNAME			CUSTOM_START_ERROR + 140
#define INVALID_WANNAME_EXISTS		CUSTOM_START_ERROR + 141
#define INVALID_PVC_EXISTS		CUSTOM_START_ERROR + 150
#define LAN_ROUTING_PVC_DEPENDENCY	CUSTOM_START_ERROR + 160
#define SYS_UPNP_PVC_DEPENDENCY		CUSTOM_START_ERROR + 170


#define TOTAL_ELEMENTS_LABEL		"wan_dsl.totalElements"
#define LIST_LABEL			"wan_dsl.list"
#define GROUP_LABEL			"wan_dsl"


typedef enum{
	WAN_SETTINGS_NOT_SET,
	WAN_SETTINGS_STATIC = 11,
	WAN_SETTINGS_PPP,
	WAN_SETTINGS_DYNAMIC,
	WAN_SETTINGS_BRIDGE
}WAN_SETTINGS;


typedef enum{
	SERVICE_CAT_NOT_SET,
	SERVICE_CAT_UBR = 11,
	SERVICE_CAT_CBR,
	SERVICE_CAT_VBR
}SERVICE_CAT;

typedef enum{
	IP_CONTROL_NOT_SET,
	IP_CONTROL_DYNAMIC = 11,
	IP_CONTROL_STATIC
}IP_CONTROL;


/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

WAN_SETTINGS getWanSetting(HLST *avp)
{
	char **val = (char**)find_hlst(avp, "wan_dsl.wanSettings", 0);

	if(val)
	{
		if(!strcmp("static",*val))
			return WAN_SETTINGS_STATIC;
		if(!strcmp("ppp",*val))
			return WAN_SETTINGS_PPP;
		if(!strcmp("dynamic",*val))
			return WAN_SETTINGS_DYNAMIC;
		if(!strcmp("bridge",*val))
			return WAN_SETTINGS_BRIDGE;

	}

	return WAN_SETTINGS_NOT_SET;
}

SERVICE_CAT getServiceCat(HLST *avp)
{
	char **val = (char**)find_hlst(avp, "wan_dsl.serviceCat", 0);

	if(val)
	{
		if(!strcmp("ubr",*val))
			return SERVICE_CAT_UBR;
		if(!strcmp("cbr",*val))
			return SERVICE_CAT_CBR;
		if(!strcmp("vbr",*val))
			return SERVICE_CAT_VBR;
	}

	return SERVICE_CAT_NOT_SET;
}

IP_CONTROL getIpControl(HLST *avp)
{
	char **val = (char**)find_hlst(avp, "wan_dsl.ipControl", 0);

	if(val)
	{
		if(!strcmp("dynamic",*val))
			return IP_CONTROL_DYNAMIC;
		if(!strcmp("static",*val))
			return IP_CONTROL_STATIC;
	}

	return IP_CONTROL_NOT_SET;
}


int checkVpc(HLST *avp, HLST *nvram_avp, const char *varName){

	long new_vpi, new_vci;
	int error = NO_ERROR;
	char **idlist, **index;
	
	

	new_vpi = getVarAsLong(avp,"wan_dsl.vpi");
	new_vci = getVarAsLong(avp,"wan_dsl.vci");
	index	= (char**)find_hlst(avp, "index", 0);
	idlist 	= (char**)find_hlst(nvram_avp, "wan_dsl.list", 0);

	if( new_vpi > -1 && new_vci > -1 && idlist)
	{
		long vpi, vci;
		char *_dup,*p;
		char *_tmp;
		

		_dup = zstrdup(*idlist);
	
		p = strtok(_dup, " ");
	
		while(p && (error==NO_ERROR))
		{
			if(index && !strcmp(*index, p))
			{
				p = strtok(NULL, " ");
				continue;
			}

			_tmp = zmalloc(strlen("wan_dsl.vpi")+strlen(p)+2);

			strcpy(_tmp, "wan_dsl.vpi");
			strcat(_tmp, ".");
			strcat(_tmp, p);

			vpi = getVarAsLong(nvram_avp, _tmp);

			strcpy(_tmp, "wan_dsl.vci");
			strcat(_tmp, ".");
			strcat(_tmp, p);

			vci = getVarAsLong(nvram_avp, _tmp);
			
			if(new_vpi == vpi && new_vci == vci)
			{
				error = INVALID_PVC_EXISTS;
			}


			free(_tmp);
			p = strtok(NULL, " ");
		}

		free(_dup);
	}


	if(error != NO_ERROR)
		handle_varError(avp, error);
   

	return error;

}


int checkVpi(HLST *avp, HLST *nvram_avp, const char *varName){

	long 	i;
	char 	*s, *existVar;
	int 	error = NO_ERROR;

	char **val = (char**)find_hlst(avp, varName, 0);



	i = strtol(*val, &s, 10);

	if(**val == '\0' || *s != '\0' || i < 0 || i > 255 )
	{
		error = INVALID_VPI;
	}
        
	if(error != NO_ERROR)
		handle_varError(avp, error);
   

	return error;
}

int checkVci(HLST *avp, HLST *nvram_avp, const char *varName){

	long i;
	char *s, *existVar;
	int error = NO_ERROR;
	char **val = (char**)find_hlst(avp, varName, 0);

	i = strtol(*val, &s, 10);

	if(**val == '\0' || *s != '\0' || i < 32 || i > 65535 )
	{
		error = INVALID_VCI;
	} 
	else if(!find_hlst(avp,"wan_dsl.vpi.error",0))
	{
		
		error = checkVpc(avp, nvram_avp, varName);

	}
                
	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}


int checkIp(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **val;

	if(getWanSetting(avp) != WAN_SETTINGS_STATIC)
		return NO_ERROR;

	val = (char**)find_hlst(avp, varName, 0);

	if(!val || !validIpAddr(val))
	{
		error = INVALID_IP;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}


int checkSubnetMask(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **val;

	if(getWanSetting(avp) != WAN_SETTINGS_STATIC)
		return NO_ERROR;

	val = (char**)find_hlst(avp, varName, 0);

	if(!val || !validSubnetMask(val))
	{
		error = INVALID_SUBNETMASK;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkGateway(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **val;

	if(getWanSetting(avp) != WAN_SETTINGS_STATIC)
		return NO_ERROR;

	val = (char**)find_hlst(avp, varName, 0);

	if(!val || !validIpAddr(val))
	{
		error = INVALID_GATEWAY;
	}

	if(error != NO_ERROR)

		handle_varError(avp, error);

	return error;
}


int checkPrimaryDns(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **val;

	if(getWanSetting(avp) != WAN_SETTINGS_STATIC)
		return NO_ERROR;

	val = (char**)find_hlst(avp, varName, 0);

	if(!val || !validIpAddr(val))
	{
		error = INVALID_PRIMARYDNS;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;	
}


int checkStaticIp(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **val;

	if((getIpControl(avp) != IP_CONTROL_STATIC) ||
	   (getWanSetting(avp) != WAN_SETTINGS_PPP))
		return NO_ERROR;

	val = (char**)find_hlst(avp, varName, 0);

	if(!val || !validIpAddr(val))
	{
		error = INVALID_STATICIP;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;	
}


int checkMtu(HLST *avp, HLST *nvram_avp, const char *varName)
{
	long i;
	char *s;
	int error = NO_ERROR;
	char **val;

	if((getWanSetting(avp) == WAN_SETTINGS_NOT_SET) ||
	   (getWanSetting(avp) == WAN_SETTINGS_BRIDGE))
		return NO_ERROR;

	if(val = (char**)find_hlst(avp, varName, 0))
	{

		i = strtol(*val, &s, 10);
	
		if(**val == '\0' || *s != '\0' || i < 128 || i > 65535 )
		{
			error = INVALID_MTU;
		}
	}
	else
		error = INVALID_MTU;


	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkMru(HLST *avp, HLST *nvram_avp, const char *varName)
{
	long i;
	char *s;
	int error = NO_ERROR;
	char **val;

	if((getWanSetting(avp) != WAN_SETTINGS_PPP))
		return NO_ERROR;

	if(val = (char**)find_hlst(avp, varName, 0))
	{

		i = strtol(*val, &s, 10);
	
		if(**val == '\0' || *s != '\0' || i < 128 || i > 1500 )
		{
			error = INVALID_MRU;
		}
	}
	else
		error = INVALID_MRU;


	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}


int checkPcr(HLST *avp, HLST *nvram_avp, const char *varName)
{
	char *s, **val;
	int error = NO_ERROR;

	if((getServiceCat(avp) == SERVICE_CAT_NOT_SET) ||
	   (getServiceCat(avp) == SERVICE_CAT_UBR))
		return NO_ERROR;

	if(val = (char**)find_hlst(avp, varName, 0))
	{
		long i = strtol(*val, &s, 10);
	
		if(**val == '\0' || *s != '\0' || i < 0 || i > 65534 )
		{
			error = INVALID_PCR;
		}
	}
	else
		error = INVALID_PCR;


	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkScr(HLST *avp, HLST *nvram_avp, const char *varName)
{
	char *s, **val;
	int error = NO_ERROR;

	if(getServiceCat(avp) != SERVICE_CAT_VBR)
		return NO_ERROR;

	if(val = (char**)find_hlst(avp, varName, 0))
	{

		long i = strtol(*val, &s, 10);
	
		if(**val == '\0' || *s != '\0' || i < 0 || i > 65534 )
		{
			error = INVALID_SCR;
		}
	}
	else
		error = INVALID_SCR;


	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkClonedMac(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **val;

	if(getWanSetting(avp) != WAN_SETTINGS_DYNAMIC)
		return NO_ERROR;

	val = (char**)find_hlst(avp, varName, 0);

	if(!val || !validMacAddr(val))
	{
		error = INVALID_MACADDRESS;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;	
}


int checkWanName(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char *existVar, **val = (char**)find_hlst(avp, varName, 0);

	if(!val || **val=='\0')
	{
		error = INVALID_WANNAME;
	}
	else if(existVar = getExistNvramVar(nvram_avp, varName, *val))	
	{
		/* check if we found the variable that we have to edit */
		if(!isEditVariable(avp,existVar,varName))
			error = INVALID_WANNAME_EXISTS;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int setId(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	char *nextIndex;
	char **val;
	char **ind;

	ind			= (char**)find_hlst(avp, "index", 0);
	//val 			= (char**)find_hlst(avp, varName, 0);
	nextIndex		= getNextIndex(nvram_avp, LIST_LABEL, TOTAL_ELEMENTS_LABEL);
   
	if(!ind || **ind == '\0')
	{
		if(nextIndex)
		{
			forceAddToList(avp,varName,nextIndex);
		}
	}
	else
	{
		addToList(avp,varName,*ind);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return NO_ERROR;
}


HTMLOBJECT *init_htmlVars(){

	int max_size = 30;
	int i=0;
	int max_index = max_size - 1;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);

	p[i].revise 	= setId;
	p[i++].name 	= "wan_dsl.id";

	p[i].revise 	= checkWanName;
	p[i++].name	= "wan_dsl.wanName";

	p[i].revise 	= checkVpi;
	p[i++].name	= "wan_dsl.vpi";

	p[i].revise 	= checkVci;
	p[i++].name	= "wan_dsl.vci";

	p[i].revise 	= ensureRadioChecked;
	p[i++].name	= "wan_dsl.virtualCircuit";

	p[i].revise 	= noError;
	p[i++].name	= "wan_dsl.wanSettings";

	p[i].revise 	= noError;
	p[i++].name	= "wan_dsl.serviceCat";
	
	p[i].revise 	= checkPcr;
	p[i++].name	= "wan_dsl.pcr";

	p[i].revise 	= checkScr;
	p[i++].name	= "wan_dsl.scr";


	//common for Wan-Settings

	
	p[i].revise 	= checkMtu;
	p[i++].name	= "wan_dsl.mtu";

	p[i].revise 	= ensureRadioChecked;
	p[i++].name	= "wan_dsl.nat";

	p[i].revise 	= ensureRadioChecked;
	p[i++].name	= "wan_dsl.firewall";


	//Wan-Settings = static

	p[i].revise 	= noError;
	p[i++].name	= "wan_dsl.connectionType.static";
	
	p[i].revise 	= checkIp;
	p[i++].name	= "wan_dsl.ipAddress";

	p[i].revise 	= checkSubnetMask;
	p[i++].name	= "wan_dsl.subnetMask";

	p[i].revise 	= checkGateway;
	p[i++].name	= "wan_dsl.gatewayAddress";

	p[i].revise 	= checkPrimaryDns;
	p[i++].name 	= "wan_dsl.primaryDnsAddress";

  


	//Wan-Settings = ppp 

	p[i].revise 	= noError;
	p[i++].name	= "wan_dsl.connectionType.ppp";

	p[i].revise 	= noError;
	p[i++].name 	= "wan_dsl.username";

	p[i].revise 	= noError;
	p[i++].name	= "wan_dsl.password";

	p[i].revise 	= checkMru;
	p[i++].name	= "wan_dsl.mru";

	p[i].revise 	= ensureRadioChecked;
	p[i++].name	= "wan_dsl.defaultRoute";

	p[i].revise 	= ensureRadioChecked;
	p[i++].name	= "wan_dsl.ipControl";

	p[i].revise 	= checkStaticIp;
	p[i++].name	= "wan_dsl.staticIpAddress";



	//Wan-Settings = dynamic

	p[i].revise 	= noError;
	p[i++].name	= "wan_dsl.connectionType.dynamic";

	p[i].revise 	= checkClonedMac;
	p[i++].name 	= "wan_dsl.clonedMacAddress";


	//Wan-Settings = bridge
	
	p[i].revise 	= noError;
	p[i++].name	= "wan_dsl.connectionType.bridge";



	//sentinel
	p[i>max_index ? max_index : i].revise 	= NULL;
	p[i>max_index ? max_index : i].name	= NULL;

	return p;
}

void free_htmlVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}



void handle_varError(HLST *avp, int ecode){

 
	switch(ecode){

	case INVALID_VPI:			addToList(avp, "wan_dsl.vpi.error", "1"); 
						break;

	case INVALID_VCI:	 	  	addToList(avp, "wan_dsl.vci.error", "1"); 
						break;

	case INVALID_IP:	 	  	addToList(avp, "wan_dsl.ipAddress.error", "1"); 
						break;

	case INVALID_SUBNETMASK:  		addToList(avp, "wan_dsl.subnetMask.error", "1"); 
						break;

	case INVALID_GATEWAY:			addToList(avp, "wan_dsl.gatewayAddress.error", "1"); 
						break;
									
	case INVALID_PRIMARYDNS:		addToList(avp, "wan_dsl.primaryDnsAddress.error", "1"); 
						break;

	case INVALID_MTU:			addToList(avp, "wan_dsl.mtu.error", "1"); 
						break;

	case INVALID_MRU:			addToList(avp, "wan_dsl.mru.error", "1"); 
						break;

	case INVALID_PCR:			addToList(avp, "wan_dsl.pcr.error", "1"); 
						break;

	case INVALID_SCR:			addToList(avp, "wan_dsl.scr.error", "1"); 
						break;

	case INVALID_STATICIP:			addToList(avp, "wan_dsl.staticIpAddress.error", "1"); 
						break;

	case INVALID_MACADDRESS:		addToList(avp, "wan_dsl.clonedMacAddress.error", "1"); 
						break;

	case INVALID_RADIO_OPTION:		addToList(avp, "wan_dsl.radioOption.error", "1"); 
						break;
								
	case INVALID_WANNAME:			addToList(avp, "wan_dsl.wanName.error", "1"); 
						break;

	case INVALID_PVC_EXISTS:		addToList(avp, "wan_dsl.pvcExists.error", "1");
						break;

	case INVALID_WANNAME_EXISTS:		addToList(avp, "wan_dsl.wanNameExists.error", "1");
						break;

	case LAN_ROUTING_PVC_DEPENDENCY:	addToList(avp, "wan_dsl.lanRoutingDependency.error", "1");
						break;

	case SYS_UPNP_PVC_DEPENDENCY:		addToList(avp, "sys_upnp.sysUpnpDependency.error", "1");
						break;


	default	:			       	//UNKNOWN error code, nothing to do
						logger (LOG_WARNING, "UNKNOWN error code: %d: ", ecode);
						return;
	}

	addToList(avp, "error", "1");
    
	return;

}

int hasDependency(STATE  *s,  HLST *avp)
{
  
	char 	**index, **dependency;
	HLST 	*nvram_avp;
	int	error = NO_ERROR;

	if(!(index = (char**)find_hlst(avp,"index", 0)) || (**index=='\0') )
	{
		return error;
	}
  
	nvram_avp = create_hlst(300, nvram_avp_destroy, NULL);
   
	load_from_nvram(s->nvram , nvram_avp, NULL);
	

	//check if there is a dependency with lan_routing.wanPvc
	if(getExistNvramVar(nvram_avp, "lan_routing.wanPvc", *index))
	{
		error = LAN_ROUTING_PVC_DEPENDENCY;
		handle_varError(avp, error);
	}


	//check if there is a dependency with sys_upnp.pvc
	if(getExistNvramVar(nvram_avp, "sys_upnp.pvc", *index))
	{
		error = SYS_UPNP_PVC_DEPENDENCY;
		handle_varError(avp, error);
	}
	
	
	destroy_hlst(nvram_avp);


	return error;
}


void *htinit (const char *doc_path, NVCL *nv) {
	STATE *s = ZMALLOC (STATE) ;

	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= "wan/wan_dsl.html";
	s->htmlvars.group_label			= GROUP_LABEL;
	s->htmlvars.total_elements_label	= TOTAL_ELEMENTS_LABEL;

	s->nvram  	= nv ;
	s->htpath 	= doc_path ;
	return s ;
}

void htcleanup (void *desc) {
	STATE *s = desc ;

	free_htmlVars(s->htmlvars.objects);
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {

	STATE  *s = desc ;
	PARSER   *parser ;
	HLST        *avp ;
	const char *data ;

	char *html_file = (char*)argv[1];

	if (argc < 2)
		return ;

	avp = http_args (r);

	/* list version 2, lists unsorted with fixed index */
	if((char**)find_hlst(avp,"save", 0))
	{
		 //store
		if(save_indexHttpVars(s,avp))
			html_file = (char*)s->htmlvars.load_on_success;

	}
	else if((char**)find_hlst(avp,"delete", 0))
	{
		/* before delete check if the pvcid is used by the lan_routing plugin (hasAccDependency)*/
		if(!hasDependency(s,avp) && delete_indexHttpVars(s,avp))
			html_file = (char*)s->htmlvars.load_on_success;
	}
	else if((char**)find_hlst(avp,"add", 0))
	{
		 //do nothing for add
	}
	else
	{
		  load_indexHttpVars(s, avp);
	}
        
	handle_global_settings(avp, s->nvram);
  
	parser = open_parser (avp, s->nvram, s->htpath) ;

	if (file_parser (parser,  htdocs_p (r, html_file)) == 0)
		return ;

	/* parser */
	run_parser (parser) ;

	avdump (avp); /* debugging */

	/* print result */
	if ((data = data_collect (parser)) != 0)
		body_append (r, data, 0);

	/* clean up */
	file_parser_end (parser);
	close_parser (parser) ;
}



/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

