/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: n/n (D-Link)			                          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "hlst.h"
#include "util.h"
#include "http-plugin.h"
#include "parser.h"
#include "nvalloc.h"
#include "plugin-util.h"
#include "logger.h"


/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */
#define INVALID_USERNAME			CUSTOM_START_ERROR + 10
#define INVALID_HOSTNAME			CUSTOM_START_ERROR + 30
#define PASSWORD_EMPTY				CUSTOM_START_ERROR + 40
#define PASSWORD_NONEQUAL			CUSTOM_START_ERROR + 50

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

/*
int isPresent(HLST *avp, HLST *nvram_avp, const char *varName)
{
	char **retVal ;

	char **val = (char**)find_hlst(avp, varName, 0);

	retVal = (char**)find_hlst(avp, "wan_dynDns.dnsrelay", 0);

	if(retVal)
	{
		if(strcmp(*retVal, "disabled"))
		{
			if(**val == '\0')
			{
				return 0;
			}
		}

	}
	return 1;
}
*/

int isDynDnsEnabled(HLST *avp)
{
	char **retVal ;

	retVal = (char**)find_hlst(avp, "wan_dynDns.hasDynDns", 0);

	if(retVal)
	{
		if(strcmp(*retVal, "true") == 0)
			return 1;

	}
	return 0;
}


int checkName(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **val = (char**)find_hlst(avp, varName, 0);

	if(val)
	{
		if(isDynDnsEnabled(avp))	//check Name
		{
			if(**val == '\0' || strlen(*val) > 32)
				error = INVALID_USERNAME;
		}
		else		// do not save an eventually empty value
		{
			delete_hlst(avp,varName,0);
		}
	}

	 if(error != NO_ERROR)
			handle_varError(avp, error);

	return error;
}

/*
int checkPassword(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **val = (char**)find_hlst(avp, varName, 0);

	if(val)
	{
		if(!isDynDnsDisabled(avp, nvram_avp, varName))	//check Name
		{
			if(**val == '\0' || strlen(*val) > 32)
				error = INVALID_PASSWORD;
		}
		else		// do not save an eventually empty value
		{
			delete_hlst(avp,varName,0);
		}
	}

	if(error != NO_ERROR)
			handle_varError(avp, error);

	return error;
}
*/

int checkPassword(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **passwd1;
	char **passwd2;
	int len1 = 0;
	int len2 = 0;

	
	if(!isDynDnsEnabled(avp))
	{
		delete_hlst(avp,varName,0);
		return NO_ERROR;
	}
	

	passwd1 = (char**)find_hlst(avp, varName, 0);

	if(!passwd1 || (len1=strlen(*passwd1)) == 0 || len1 > 64 )
		error = PASSWORD_EMPTY;			//first password was empty

	passwd2 = (char**)find_hlst(avp, "wan_dynDns.password2", 0);
	
	if(!passwd2 || (len2=strlen(*passwd2)) == 0 || len2 > 64 )	
		error = PASSWORD_EMPTY;			//second password was empty

	/* if one of the passwords was empty, set an error message and do not check if both passwords are equal or not */
	if(error != NO_ERROR)
	{
		handle_varError(avp, error);
		return error;
	}


	if(len1 != len2 || strcmp(*passwd1,*passwd2))
		error = PASSWORD_NONEQUAL;		//password and confirmPassword were nonequal


	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkHostname(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **val = (char**)find_hlst(avp, varName, 0);

	if(val)
	{
		if(isDynDnsEnabled(avp))
		{
			if(**val == '\0' || strlen(*val) > 32 )
				error = INVALID_HOSTNAME;
		}
		else		// do not save an eventually empty value
		{
			delete_hlst(avp,varName,0);
		}
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}


HTMLOBJECT *init_htmlVars()
{
	int max_size = 6;
	int i=0;
	int max_index = max_size - 1;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);

	
	p[i].revise 	= ensureCheckState;
	p[i++].name 	= "wan_dynDns.hasDynDns";

	p[i].revise 	= noError;
	p[i++].name 	= "wan_dynDns.dnsRelay";

	p[i].revise 	= checkName;
	p[i++].name	= "wan_dynDns.username";

	p[i].revise 	= checkPassword;
	p[i++].name 	= "wan_dynDns.password";

	p[i].revise 	= checkHostname;
	p[i++].name	= "wan_dynDns.hostname";
  
	//sentinel
	p[i>max_index ? max_index : i].revise 	= NULL;
	p[i>max_index ? max_index : i].name	= NULL;

	return p;
}

void free_htmlVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}



void handle_varError(HLST *avp, int ecode){

 
	switch(ecode){

	case INVALID_USERNAME:		addToList(avp, "wan_dynDns.username.error", "1"); 
					break;

	case INVALID_HOSTNAME:		addToList(avp, "wan_dynDns.hostname.error", "1"); 
					break;

	case PASSWORD_EMPTY:		addToList(avp, "wan_dynDns.password.empty.error", "1");
					break;

	case PASSWORD_NONEQUAL:		addToList(avp, "wan_dynDns.password.nonequal.error", "1");
					break;

	default	:		     	//UNKNOWN error code, nothing to do
					logger (LOG_WARNING, "UNKNOWN error code: %d: ", ecode);
					return;
	}


	addToList(avp, "error", "1");
    
	return;

}

void *htinit (const char *doc_path, NVCL *nv) {
	STATE *s = ZMALLOC (STATE) ;

	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= NULL;
	s->htmlvars.total_elements_label 	= NULL;
	s->htmlvars.group_label 		= NULL;

	s->nvram  	= nv ;
	s->htpath 	= doc_path ;

	return s ;
}

void htcleanup (void *desc) {
	STATE *s = desc ;

	free_htmlVars(s->htmlvars.objects);
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
	STATE  *s = desc ;
	PARSER   *parser ;
	HLST        *avp ;
	const char *data ;

	if (argc < 2)
		return ;

	avp = http_args (r);

	CALL_HTTPVARS_ACTION(s,avp);
        
	handle_global_settings(avp, s->nvram);

	parser = open_parser (avp, s->nvram, s->htpath) ;

	if (file_parser (parser, htdocs_p (r, argv [1])) == 0)
		return ;

	/* parser */
	run_parser (parser) ;

	avdump (avp); /* debugging */

	/* print result */
	if ((data = data_collect (parser)) != 0)
		body_append (r, data, 0);

	/* clean up */
	file_parser_end (parser);
	close_parser (parser) ;
}



/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

