/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Susanne Goldammer <s.goldammer@maxina.de> - Maxina GmbH,      *
*   Neu-Isenburg, Germany						  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <netinet/in.h>

#include "hlst.h"
#include "util.h"
#include "http-plugin.h"
#include "nvalloc.h"
#include "parser.h"
#include "plugin-util.h"
#include "logger.h"


/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */

#define INVALID_SOURCE_START_IP			CUSTOM_START_ERROR + 10
#define INVALID_SOURCE_END_IP			CUSTOM_START_ERROR + 11
#define INVALID_SOURCE_ADDRESS_RANGE		CUSTOM_START_ERROR + 12

#define INVALID_DESTINATION_START_IP		CUSTOM_START_ERROR + 20
#define INVALID_DESTINATION_END_IP		CUSTOM_START_ERROR + 21
#define INVALID_DESTINATION_ADDRESS_RANGE	CUSTOM_START_ERROR + 22

#define INVALID_SOURCE_START_PORT		CUSTOM_START_ERROR + 30
#define INVALID_SOURCE_END_PORT			CUSTOM_START_ERROR + 31
#define INVALID_SOURCE_PORT_RANGE		CUSTOM_START_ERROR + 32

#define INVALID_DESTINATION_START_PORT		CUSTOM_START_ERROR + 40
#define INVALID_DESTINATION_END_PORT		CUSTOM_START_ERROR + 41
#define INVALID_DESTINATION_PORT_RANGE		CUSTOM_START_ERROR + 42


/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

HTMLOBJECT *init_htmlVars(){
	int max_size = 1;
	int i=0;
	int max_index = max_size - 1;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);

	//sentinel
	p[i>max_index ? max_index : i].revise 	= NULL;
	p[i>max_index ? max_index : i].name		= NULL;

	return p;
}

void free_htmlVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}

void handle_varError(HLST *avp, int ecode){
 
	switch(ecode){

	default	:	   			//UNKNOWN error code, nothing to do
						logger (LOG_WARNING, "UNKNOWN error code: %d: ", ecode);
						return;
	}


	addToList(avp, "error", "1");
    
	return;
}


void *htinit (const char *doc_path, NVCL *nv) {
	STATE *s = ZMALLOC (STATE) ;

	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= NULL;
	s->htmlvars.total_elements_label 	= NULL;
	s->htmlvars.group_label 		= NULL;

	s->nvram  	= nv ;
	s->htpath 	= doc_path ;
	return s ;
}

void htcleanup (void *desc) {
	STATE *s = desc ;

	free_htmlVars(s->htmlvars.objects);
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
	STATE  *s = desc ;
	PARSER   *parser ;
	HLST        *avp ;
	const char *data ;
	
	if (argc < 2)
		return ;
	
	avp = http_args (r);

	handle_global_settings(avp, s->nvram);
		
	parser = open_parser (avp, s->nvram, s->htpath) ;
	
	if (file_parser (parser, htdocs_p (r, argv [1])) == 0)
		return ;
	
	/* parser */
	run_parser (parser) ;
	
	avdump (avp); /* debugging */
	
	/* print result */
	if ((data = data_collect (parser)) != 0)
		body_append (r, data, 0);
	
	/* clean up */
	file_parser_end (parser);
	close_parser (parser) ;
}


/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

