/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: n/n (D-Link)			                          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "hlst.h"
#include "http-plugin.h"
#include "parser.h"
#include "nvalloc.h"
#include "plugin-util.h"
#include "logger.h"

/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */

#define INVALID_SOURCE_START_IP			CUSTOM_START_ERROR + 10
#define INVALID_SOURCE_END_IP			CUSTOM_START_ERROR + 11
#define INVALID_SOURCE_ADDRESS_RANGE		CUSTOM_START_ERROR + 12

#define INVALID_DESTINATION_START_IP		CUSTOM_START_ERROR + 20
#define INVALID_DESTINATION_END_IP		CUSTOM_START_ERROR + 21
#define INVALID_DESTINATION_ADDRESS_RANGE	CUSTOM_START_ERROR + 22

#define INVALID_SOURCE_START_PORT		CUSTOM_START_ERROR + 30
#define INVALID_SOURCE_END_PORT			CUSTOM_START_ERROR + 31
#define INVALID_SOURCE_PORT_RANGE		CUSTOM_START_ERROR + 32

#define INVALID_DESTINATION_START_PORT		CUSTOM_START_ERROR + 40
#define INVALID_DESTINATION_END_PORT		CUSTOM_START_ERROR + 41
#define INVALID_DESTINATION_PORT_RANGE		CUSTOM_START_ERROR + 42

#define INVALID_TOTAL_ELEMENTS			CUSTOM_START_ERROR + 50

#define TOTAL_ELEMENTS_LABEL			"wan_outbound.totalElements"
#define LIST_LABEL				"wan_outbound.list"


/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

int setId(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	char *nextIndex;
	char **val;
	char **ind;

	ind		= (char**)find_hlst(avp, "index", 0);
	//val 		= (char**)find_hlst(avp, varName, 0);
	nextIndex	= getNextIndex(nvram_avp, LIST_LABEL, TOTAL_ELEMENTS_LABEL);
   
	if(!ind || **ind == '\0')
	{
		if(nextIndex)
		{
			forceAddToList(avp,varName,nextIndex);
		}
		else
		{
			error = INVALID_TOTAL_ELEMENTS;
		}
	}
	else
	{
		addToList(avp,varName,*ind);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return NO_ERROR;
}

int isSourcePortEnabled(HLST *avp)
{
	char **val = (char**)find_hlst(avp, "wan_outbound.sourcePort", 0);

	if(val && !strcmp(*val, "specified"))
	{
		return 1;
	}

	return 0;
}

int isDestPortEnabled(HLST *avp)
{
	char **val = (char**)find_hlst(avp, "wan_outbound.destinationPort", 0);

	if(val && !strcmp(*val, "specified"))
	{
		return 1;
	}

	return 0;
}

int isSourceIpAny(HLST *avp)
{
	char **val = (char**)find_hlst(avp, "wan_outbound.sourceAddress", 0);

	if(val && !strcmp(*val, "any"))
	{
		return 1;
	}
	return 0;
}

int isDestIpAny(HLST *avp)
{
	char **val = (char**)find_hlst(avp, "wan_outbound.destinationAddress", 0);

	if(val && !strcmp(*val, "any"))
	{
		return 1;
	}
	return 0;
}

int checkSourceFrom(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	char **val;

	if(!isSourceIpAny(avp) && 
	   (val = (char**)find_hlst(avp,varName,0)) )
	{
		if(!validIpAddr(val))
		{
			error = INVALID_SOURCE_START_IP;
		}
	}
	else
	{
		delete_hlst(avp,varName,0);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkSourceTo(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	char **val;

	if(!isSourceIpAny(avp) && 
	   (val = (char**)find_hlst(avp,varName,0)) )
	{
		if(!validIpAddr(val))
		{
			error = INVALID_SOURCE_END_IP;
		}
	}
	else
	{
		delete_hlst(avp,varName,0);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}


int checkDestFrom(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	char **val;

	if(!isDestIpAny(avp) && 
	   (val = (char**)find_hlst(avp,varName,0)) )
	{
		if(!validIpAddr(val))
		{
			error = INVALID_DESTINATION_START_IP;
		}
	}
	else
	{
		delete_hlst(avp,varName,0);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkDestTo(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	char **val;

	if(!isDestIpAny(avp) && 
	   (val = (char**)find_hlst(avp,varName,0)) )
	{
		if(!validIpAddr(val))
		{
			error = INVALID_DESTINATION_END_IP;
		}
	}
	else
	{
		delete_hlst(avp,varName,0);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkSourceIpRange(HLST *avp, HLST *nvram_avp, const char *varName)
{
	/* test whether filter is for any Source ip or not*/

	unsigned long ret = 0;
	struct in_addr start, end;
	int error = NO_ERROR;
	char **val = (char**)find_hlst(avp, varName, 0);

	if(!isSourceIpAny(avp))
	{
		char **sourceFrom = (char**)find_hlst(avp,"wan_outbound.sourceFrom",0);
		char **sourceTo = (char**)find_hlst(avp,"wan_outbound.sourceTo",0);;

		/* check if the start ip is smaller than the end ip */
		if(sourceFrom && validIpAddr(sourceFrom) && 
		   sourceTo   && validIpAddr(sourceTo))
		{
			 inet_aton(*sourceFrom, &start );
			 inet_aton(*sourceTo, &end);
			 if(ntohl(start.s_addr) > ntohl(end.s_addr))
				 error = INVALID_SOURCE_ADDRESS_RANGE;
		}
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}


int checkDestIpRange(HLST *avp, HLST *nvram_avp, const char *varName)
{
	/* test whether filter is for any Source ip or not*/

	unsigned long ret = 0;
	struct in_addr start, end;
	int error = NO_ERROR;
	char **val = (char**)find_hlst(avp, varName, 0);

	if(!isDestIpAny(avp))
	{
		char **destFrom = (char**)find_hlst(avp,"wan_outbound.destinationFrom",0);
		char **destTo = (char**)find_hlst(avp,"wan_outbound.destinationTo",0);;

		/* check if the start ip is smaller than the end ip */
		if(destFrom && validIpAddr(destFrom) && 
		   destTo   && validIpAddr(destTo))
		{
			 inet_aton(*destFrom, &start );
			 inet_aton(*destTo, &end);
			 if(ntohl(start.s_addr) > ntohl(end.s_addr))
				 error = INVALID_DESTINATION_ADDRESS_RANGE;
		}
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}


int checkSourcePort(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
   
	if(isSourcePortEnabled(avp))
	{
		long start, end;					//numeric values for start and end port
		//get the startPort
		start = getVarAsPort(avp,"wan_outbound.sourcePortFrom");

		//get the endPort
		end = getVarAsPort(avp,"wan_outbound.sourcePortTo");

		if(start>0 && end>0)
		{
			if(start > end)
				error = INVALID_SOURCE_PORT_RANGE;
		}	
	}
	else
	{
		delete_hlst(avp,"wan_outbound.sourcePortFrom",0);
		delete_hlst(avp,"wan_outbound.sourcePortTo",0);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkDestPort(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
   
	if(isDestPortEnabled(avp))
	{
		long start, end;					//numeric values for start and end port
		//get the startPort
		start = getVarAsPort(avp,"wan_outbound.destinationPortFrom");

		//get the endPort
		end = getVarAsPort(avp,"wan_outbound.destinationPortTo");

		if(start>0 && end>0)
		{
			if(start > end)
				error = INVALID_DESTINATION_PORT_RANGE;
		}	
	}
	else
	{
		delete_hlst(avp,"wan_outbound.destinationPortFrom",0);
		delete_hlst(avp,"wan_outbound.destinationPortTo",0);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkSourcePortFrom(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
   
	if(isSourcePortEnabled(avp))
	{
		if(getVarAsPort(avp, varName)==-1)
		{
			error = INVALID_SOURCE_START_PORT; 
		}
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}


int checkSourcePortTo(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
   
	if(isSourcePortEnabled(avp))
	{
	    if(getVarAsPort(avp, varName)==-1)
		{
			error = INVALID_SOURCE_END_PORT; 
		}
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkDestPortFrom(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
   
	if(isDestPortEnabled(avp))
	{
		if(getVarAsPort(avp, varName)==-1)
		{
			error = INVALID_DESTINATION_START_PORT; 
		}
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}



int checkDestPortTo(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
   
	if(isDestPortEnabled(avp))
	{
	    if(getVarAsPort(avp, varName)==-1)
		{
			error = INVALID_DESTINATION_END_PORT; 
		}
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

HTMLOBJECT *init_htmlVars(){
	int max_size = 16;
	int i=0;
	int max_index = max_size - 1;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);

	p[i].revise 	= setId;
	p[i++].name 	= "wan_outbound.id";

	p[i].revise 	= checkSourceIpRange;
	p[i++].name	= "wan_outbound.sourceAddress";

	p[i].revise 	= checkSourceFrom;
	p[i++].name	= "wan_outbound.sourceFrom";

	p[i].revise 	= checkSourceTo;
	p[i++].name	= "wan_outbound.sourceTo";

	p[i].revise 	= checkDestIpRange;
	p[i++].name	= "wan_outbound.destinationAddress";

	p[i].revise 	= checkDestFrom;
	p[i++].name	= "wan_outbound.destinationFrom";

	p[i].revise 	= checkDestTo;
	p[i++].name	= "wan_outbound.destinationTo";

	p[i].revise 	= checkSourcePort;
	p[i++].name 	= "wan_outbound.sourcePort";

	p[i].revise 	= checkSourcePortFrom;
	p[i++].name 	= "wan_outbound.sourcePortFrom";

	p[i].revise 	= checkSourcePortTo;
	p[i++].name 	= "wan_outbound.sourcePortTo";

	p[i].revise 	= checkDestPort;
	p[i++].name	= "wan_outbound.destinationPort";

	p[i].revise 	= checkDestPortFrom;
	p[i++].name	= "wan_outbound.destinationPortFrom";

	p[i].revise 	= checkDestPortTo;
	p[i++].name	= "wan_outbound.destinationPortTo";

	p[i].revise 	= noError;
	p[i++].name	= "wan_outbound.protocol";

	p[i].revise 	= noError;
	p[i++].name	= "wan_outbound.action";

	
	//sentinel
	p[i>max_index ? max_index : i].revise 	= NULL;
	p[i>max_index ? max_index : i].name		= NULL;

	return p;
}



void free_httpVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}


void handle_varError(HLST *avp, int ecode){

 
	switch(ecode){

	case INVALID_SOURCE_START_IP:			addToList(avp, "wan_outbound.sourceFrom.error", "1");
							break;	
	case INVALID_SOURCE_END_IP:			addToList(avp, "wan_outbound.sourceTo.error", "1");
							break;
	case INVALID_SOURCE_ADDRESS_RANGE: 		addToList(avp, "wan_outbound.sourceAddressRange.error", "1");
							break;

	case INVALID_DESTINATION_START_IP:		addToList(avp, "wan_outbound.destinationFrom.error", "1");
							break;
	case INVALID_DESTINATION_END_IP:		addToList(avp, "wan_outbound.destinationTo.error", "1");
							break;
	case INVALID_DESTINATION_ADDRESS_RANGE: 	addToList(avp, "wan_outbound.destinationAddressRange.error", "1");
							break;
											
	case INVALID_SOURCE_START_PORT:			addToList(avp, "wan_outbound.sourcePortFrom.error", "1");
							break;
	case INVALID_SOURCE_END_PORT:			addToList(avp, "wan_outbound.sourcePortTo.error", "1");
							break;
	case INVALID_SOURCE_PORT_RANGE: 		addToList(avp, "wan_outbound.sourcePortRange.error", "1");
							break;

	case INVALID_DESTINATION_START_PORT:		addToList(avp, "wan_outbound.destinationPortFrom.error", "1");
							break;
	case INVALID_DESTINATION_END_PORT:		addToList(avp, "wan_outbound.destinationPortTo.error", "1");
							break;
	case INVALID_DESTINATION_PORT_RANGE: 		addToList(avp, "wan_outbound.destinationPortRange.error", "1");
							break;

	default	:					//UNKNOWN error code, nothing to do
							logger (LOG_WARNING, "UNKNOWN error code: %d: ", ecode);
							return;
	}

	addToList(avp, "error", "1");
    
	return;

}


void *htinit (const char *doc_path, NVCL *nv) {
	STATE *s = ZMALLOC (STATE) ;

	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= "wan/wan_outbound.html";
	s->htmlvars.total_elements_label 	= TOTAL_ELEMENTS_LABEL;
	s->htmlvars.group_label 		= "wan_outbound";

	s->nvram  	= nv ;
	s->htpath 	= doc_path ;

	return s ;
}

void htcleanup (void *desc) {
	STATE *s = desc ;

	free_httpVars(s->htmlvars.objects);
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
	STATE  *s = desc ;
	PARSER   *parser ;
	HLST        *avp ;
	const char *data ;
	char **retVal;
	char *html_file;

	if (argc < 2)
		return ;

	html_file = (char*)argv[1];

	avp = http_args (r);

	
	if((char**)find_hlst(avp,"save", 0))
	{
		//store
		if(save_indexHttpVars(s,avp))
			html_file = (char*)s->htmlvars.load_on_success;

	}
	else if((char**)find_hlst(avp,"delete", 0))
	{
		//reorder, delete
		if(delete_indexHttpVars(s,avp))
			html_file = (char*)s->htmlvars.load_on_success;
	}
	else if((char**)find_hlst(avp,"add", 0))
	{
		 //do nothing for add, just do not call load_indexHttpVars
	}
	else
	{
		  load_indexHttpVars(s, avp);
	}
	
        
	handle_global_settings(avp, s->nvram);

	parser = open_parser (avp, s->nvram, s->htpath) ;

	if (file_parser (parser, htdocs_p (r, html_file)) == 0)
		return ;

	/* parser */
	run_parser (parser) ;

	avdump (avp); /* debugging */

	/* print result */
	if ((data = data_collect (parser)) != 0)
		body_append (r, data, 0);

	/* clean up */
	file_parser_end (parser);
	close_parser (parser) ;
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
