/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: n/n (D-Link)			                          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "hlst.h"
#include "http-plugin.h"
#include "parser.h"
#include "nvalloc.h"
#include "plugin-util.h"
#include "logger.h"

/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */

#define INVALID_RULENAME			CUSTOM_START_ERROR + 10

#define INVALID_START_PORT			CUSTOM_START_ERROR + 30
#define INVALID_END_PORT			CUSTOM_START_ERROR + 31
#define INVALID_PORT_RANGE			CUSTOM_START_ERROR + 32

#define INVALID_PORT				CUSTOM_START_ERROR + 40
#define INVALID_PORTMAP				CUSTOM_START_ERROR + 41
#define INVALID_PORTMAP_TOO_BIG			CUSTOM_START_ERROR + 42
	
#define INVALID_TOTAL_ELEMENTS			CUSTOM_START_ERROR + 50

#define TOTAL_ELEMENTS_LABEL			"wan_virtual_rule.totalElements"
#define LIST_LABEL				"wan_virtual_rule.list"

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

int setId(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	char *nextIndex;
	char **val;
	char **ind;

	ind		= (char**)find_hlst(avp, "index", 0);
	//val 		= (char**)find_hlst(avp, varName, 0);
	nextIndex	= getNextIndex(nvram_avp, LIST_LABEL, TOTAL_ELEMENTS_LABEL);
   
	if(!ind || **ind == '\0')
	{
		if(nextIndex)
		{
			forceAddToList(avp,varName,nextIndex);
		}
		else
		{
			error = INVALID_TOTAL_ELEMENTS;
		}
	}
	else
	{
		addToList(avp,varName,*ind);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return NO_ERROR;
}


int checkStartPort(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;

	long start,end;					//numeric values for start and end port

	//check for startPort

	if( (start = getVarAsPort(avp,varName)) > 0 )
	{
		if((end = getVarAsPort(avp,"wan_virtual_rule.endport")) > 0 )
		{
			if(start > end)	//startPort should be smaller or equal to endPort
				error = INVALID_PORT_RANGE;
		}
	}
	else
	{
		error = INVALID_START_PORT;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}



int checkEndPort(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;

	int port = getVarAsPort(avp, varName);

	if(port < 1)
	{
		error = INVALID_END_PORT;
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkRulename(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	char **val = (char**)find_hlst(avp, varName, 0);
	
	if(strlen(*val)==0)
		error = INVALID_RULENAME;

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}

int checkPortmap(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int error = NO_ERROR;
	long pm;					//numeric values for start and end port
	long startPort, endPort;
	char **portmap;
	char *s;
	char buf[10];

	startPort 	= getVarAsPort(avp,"wan_virtual_rule.startport");
	endPort 	= getVarAsPort(avp,"wan_virtual_rule.endport");

 
	//check for startPort
	portmap = (char**)find_hlst(avp, varName ,0);

	if(portmap && (**portmap!='\0') )
	{
		pm = strtol(*portmap,&s,10);

		if((*s != '\0') || (pm < 0) )
		{
			error = INVALID_PORTMAP;
		}
		else if((65535 - (endPort - startPort)) < pm)
		{
			error = INVALID_PORTMAP_TOO_BIG;
		}
	}
	else
	{
		if(startPort>0)
		{
			sprintf(buf,"%d",startPort);
			forceAddToList(avp,varName,buf);
		}
	}

	 if(error != NO_ERROR)
		handle_varError(avp, error);

	return error;
}


HTMLOBJECT *init_htmlVars(){
	int max_size = 8;
	int i=0;
	int max_index = max_size - 1;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);

	p[i].revise 	= setId;
	p[i++].name 	= "wan_virtual_rule.id";

	p[i].revise 	= checkRulename;
	p[i++].name	= "wan_virtual_rule.rulename";

	p[i].revise 	= noError;
	p[i++].name	= "wan_virtual_rule.protocol";

	p[i].revise 	= checkStartPort;
	p[i++].name	= "wan_virtual_rule.startport";

	p[i].revise 	= checkEndPort;
	p[i++].name	= "wan_virtual_rule.endport";

	p[i].revise 	= checkPortmap;
	p[i++].name	= "wan_virtual_rule.portmap";

	p[i].revise 	= noError;
	p[i++].name	= "wan_virtual_rule.ip";

	
	//sentinel
	p[i>max_index ? max_index : i].revise 	= NULL;
	p[i>max_index ? max_index : i].name		= NULL;

	return p;
}



void free_htmlVars(HTMLOBJECT *p){
	free(p);
	p = NULL;
}



void handle_varError(HLST *avp, int ecode){

 
	switch(ecode){

	case INVALID_START_PORT:		addToList(avp, "wan_virtual_rule.startport.error", "1");
						break;
	case INVALID_END_PORT:			addToList(avp, "wan_virtual_rule.endport.error", "1");
						break;
	case INVALID_PORT_RANGE: 		addToList(avp, "wan_virtual_rule.portRange.error", "1");
						break;
	case INVALID_RULENAME:			addToList(avp, "wan_virtual_rule.rulename.error", "1");
						break;
	case INVALID_PORTMAP_TOO_BIG:		addToList(avp, "wan_virtual_rule.portmap_tooBig.error", "1");
						break;
	case INVALID_PORTMAP:			addToList(avp, "wan_virtual_rule.portmap.error", "1");
						break;

	default	:				//UNKNOWN error code, nothing to do
						logger (LOG_WARNING, "UNKNOWN error code: %d: ", ecode);
						return;
	}


	addToList(avp, "error", "1");
    
	return;

}



void *htinit (const char *doc_path, NVCL *nv) {
    STATE *s = ZMALLOC (STATE) ;

	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= "wan/wan_virtual.html";
	s->htmlvars.total_elements_label 	= TOTAL_ELEMENTS_LABEL;
	s->htmlvars.group_label 		= "wan_virtual_rule";

	s->nvram  	= nv ;
	s->htpath 	= doc_path ;

	return s ;
}

void htcleanup (void *desc) {
	STATE *s = desc ;

	free_htmlVars(s->htmlvars.objects);
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) {
	STATE  *s = desc ;
	PARSER   *parser ;
	HLST        *avp ;
	const char *data ;
	char **retVal;
	char *html_file;

	if (argc < 2)
		return ;

	html_file = (char*)argv[1];

	avp = http_args (r);


	if((char**)find_hlst(avp,"save", 0))
	{
		 //store
		if(save_indexHttpVars(s,avp))
			html_file = (char*)s->htmlvars.load_on_success ;

	}
	else if((char**)find_hlst(avp,"delete", 0))
	{
		//reorder, delete
		if(delete_indexHttpVars(s,avp))
			html_file = (char*)s->htmlvars.load_on_success ;
	}
	else if((char**)find_hlst(avp,"add", 0))
	{
		 //do nothing for add, just do not call load_indexHttpVars
	}
	else
	{
		  load_indexHttpVars(s, avp);
	}

        
	handle_global_settings(avp, s->nvram);

	parser = open_parser (avp, s->nvram, s->htpath) ;

	if (file_parser (parser, htdocs_p (r, html_file)) == 0)
		return ;

	/* parser */
	run_parser (parser) ;

	avdump (avp); /* debugging */

	/* print result */
	if ((data = data_collect (parser)) != 0)
		body_append (r, data, 0);

	/* clean up */
	file_parser_end (parser);
	close_parser (parser) ;
}



/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
