/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: n/n (D-Link)			                          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "hlst.h"
#include "http-plugin.h"
#include "parser.h"
#include "nvalloc.h"
#include "plugin-util.h"
#include "logger.h"

/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */

#define INVALID_IP			CUSTOM_START_ERROR + 10
#define INVALID_TOTAL_ELEMENTS		CUSTOM_START_ERROR + 20

#define TOTAL_ELEMENTS_LABEL		"wan_virtual_apply.totalElements"
#define LIST_LABEL			"wan_virtual_apply.list"

#define NO_DIRTY_HACKS /* not hacking wan_virtual_rule.reconfigure */

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

int setId(HLST *avp, HLST *nvram_avp, const char *varName) 
{
	int error = NO_ERROR;
	char *nextIndex;
	char **val;
	char **ind;

	ind		= (char**)find_hlst(avp, "index", 0);
	//val 		= (char**)find_hlst(avp, varName, 0);
	nextIndex	= getNextIndex(nvram_avp, LIST_LABEL, TOTAL_ELEMENTS_LABEL);
   
	if(!ind || **ind == '\0')
	{
		if(nextIndex)
		{
			forceAddToList(avp,varName,nextIndex);
		}
		else
		{
			error = INVALID_TOTAL_ELEMENTS;
		}
	}
	else
	{
		addToList(avp,varName,*ind);
	}

	if(error != NO_ERROR)
		handle_varError(avp, error);

	return NO_ERROR;
}


int checkIp(HLST *avp, HLST *nvram_avp, const char *varName)
{
	char **val;
	
	/* get IP address value */
	val = (char**)find_hlst(avp, varName, 0);
	if(!val) /* no entry */
		return INVALID_IP;
	
	/* check if IP address is valid */
	if (validIpAddr(val))
		return NO_ERROR;

	/* IP address is not valid */
	handle_varError(avp, INVALID_IP);
	return INVALID_IP;
}


int checkRules(HLST *avp, HLST *nvram_avp, const char *varName)
{
	int elements, i;
	char **val, buf[128];

	char **ip = (char**)find_hlst(avp, "wan_virtual_apply.ip", 0);

	if (!ip || !*ip || !validIpAddr(ip)) {
		/* no ip address or invalid ip address */
		handle_varError(avp, INVALID_IP);
		return INVALID_IP;
	}

	val = (char**)find_hlst(nvram_avp, "wan_virtual_rule.totalElements", 0);
	if (!val || !*val) {
		/* strange, this value should exist */
		return NO_ERROR;
	}

	elements = atoi(*val);
	if (elements<0 || elements>65536) {
		logger(LOG_ERR, 
			"Strange wan_virtual_rule.totalElements value\n");
		return NO_ERROR;
	}
		
	for (i=1; i<=elements; i++) {

		sprintf(buf,"wan_virtual_rule.ip.%d",i);

		val = (char**)find_hlst(avp, buf, 0);
		if (val) {
			/* rule was activated, set value to our ip */
			*val = ZREALLOC(*val, char, strlen(*ip)+1);
			strcpy(*val, *ip);
			continue;
		}

		/* rule is not active, maybe rule was deleted? */
		val = (char**)find_hlst(nvram_avp, buf, 0);
		if (val) {
			/* check if current ip of rule is our ip */
			if (!strcmp(*ip, *val)) {
				/* remove this rule */
				addToList(avp,buf,"0");
			}
		}
	}
		
	return NO_ERROR;
}


int custom_delete(STATE  *s, HLST *avp)
{
	HLST *temp_avp, *nvram_avp;
	int elements, i, ret;
	char **val, buf[128];

	char **ip = (char**)find_hlst(avp, "wan_virtual_apply.ip", 0);

	if (!ip || !*ip ) {
		return 0;
	}

	temp_avp   = create_hlst( 20, nvram_avp_destroy, NULL);
	nvram_avp = create_hlst(300, nvram_avp_destroy, NULL);
	load_from_nvram(s->nvram , nvram_avp, NULL);

	val = (char**)find_hlst(nvram_avp, "wan_virtual_rule.totalElements", 0);
	if (!val || !*val) {
		return 0;
	}

	elements = atoi(*val);
	if (elements<0 || elements>65536) {
		logger(LOG_ERR, 
			"Strange wan_virtual_rule.totalElements value\n");
		return 0;
	}
		
	for (i=1; i<=elements; i++) {

		sprintf(buf,"wan_virtual_rule.ip.%d",i);

		/* get rule ip from nvram */
		val = (char**)find_hlst(nvram_avp, buf, 0);
		if (val) {
			/* check if current ip of rule is our ip */
			if (!strcmp(*ip, *val)) {
				/* remove this rule */
				addToList(temp_avp,buf,"");
			}
		}
	}
		
	ret = store_into_nvram(s->nvram,  temp_avp);

	destroy_hlst(nvram_avp);
	destroy_hlst(temp_avp);

	return ret;
}


HTMLOBJECT *init_htmlVars(){
	int max_size = 4;

	int i=0;
	int max_index = max_size - 1;

	HTMLOBJECT *p = (HTMLOBJECT *)xmalloc(sizeof(HTMLOBJECT) * max_size);

	p[i].revise 	= setId;
	p[i++].name 	= "wan_virtual_apply.id";

	p[i].revise 	= noError;
	p[i++].name	= "wan_virtual_apply.ip";

	p[i].revise 	= checkRules;
	p[i++].name	= "wan_virtual_rule.ip";
	
	//sentinel
	p[i>max_index ? max_index : i].revise 	= NULL;
	p[i>max_index ? max_index : i].name	= NULL;

	return p;
}



void free_htmlVars(HTMLOBJECT *p)
{
	free(p);
	p = NULL;
}



void handle_varError(HLST *avp, int ecode)
{
	switch(ecode){

	case INVALID_IP:                addToList(avp, "wan_virtual_apply.ip.error", "1");
					break;	
    
	default:                        /* UNKNOWN error code, nothing to do */
					logger (LOG_ERR, "UNKNOWN error code: %d: ", ecode);
					return;
	}

	addToList(avp, "error", "1");
}


void *htinit (const char *doc_path, NVCL *nv) 
{

	STATE *s = ZMALLOC (STATE) ;

	s->htmlvars.objects = init_htmlVars();
	s->htmlvars.load_on_success 		= "wan/wan_virtual.html";
	s->htmlvars.total_elements_label	= TOTAL_ELEMENTS_LABEL;
	s->htmlvars.group_label 		= "wan_virtual_apply";

	s->nvram  = nv;
	s->htpath = doc_path;
	
	return s ;
}

void htcleanup (void *desc) 
{
	STATE *s = desc ;
	free_htmlVars(s->htmlvars.objects);
	free(s);
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) 
{
	STATE	*s = desc;
	PARSER	*parser;
	HLST	*avp;
	
	const char *data;
	char **retVal;
	char *html_file;

	if (argc < 2)
        	return;

	html_file = (char*)argv[1];

	avp = http_args(r);

	if((char**)find_hlst(avp,"save", 0)) {
		
		 /* store */
		if(save_indexHttpVars(s,avp)) {
			html_file = (char*)s->htmlvars.load_on_success;

#ifndef NO_DIRTY_HACKS
			{
			 time_t t = time(NULL);
			 add_to_nvram(s->nvram, "wan_virtual_rule.reconfigure", ctime(&t));
			}
#endif
		}

		
	} else if((char**)find_hlst(avp,"delete", 0)) {
		
		 /* reorder, delete */
		custom_delete(s,avp);
		if(delete_indexHttpVars(s,avp)) {
			html_file = (char*)s->htmlvars.load_on_success;
#ifndef NO_DIRTY_HACKS
			{
			 time_t t = time(NULL);
			 add_to_nvram(s->nvram, "wan_virtual_rule.reconfigure", ctime(&t));
			}
#endif
		}
		
	} else if((char**)find_hlst(avp,"add", 0)) {
		
		 /* do nothing for add, just do not call load_indexHttpVars */

	} else {
		  load_indexHttpVars(s, avp);
	}
	
	handle_global_settings(avp, s->nvram);

	parser = open_parser (avp, s->nvram, s->htpath);

	if (file_parser (parser, htdocs_p (r, html_file)) == 0)
	        return ;

	/* parser */
	run_parser(parser);
	
	avdump(avp); /* debugging */
	
	/* print result */
	if ((data = data_collect (parser)) != 0)
		body_append (r, data, 0);
	
	/* clean up */
	file_parser_end (parser);
	close_parser (parser) ;
}



/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

