/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: n/n (D-Link)			                          *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "hlst.h"
#include "util.h"
#include "http-plugin.h"
#include "plugin-util.h"
#include "parser.h"
#include "nvalloc.h"

static void init_temp_vars(HLST* avp, NVCL* nvram)
{
	/* initialize variablen from nvram*/

	add_to_list(avp, "wiz_1wan.username",
			get_from_nvram(nvram, "wan_dsl.username.1"));

	add_to_list(avp, "wiz_1wan.password", 
			get_from_nvram(nvram, "wan_dsl.password.1"));
	
	add_to_list(avp, "wiz_1wan.connection", 
			get_from_nvram(nvram, "wan_dsl.connection.1"));
			
	add_to_list(avp, "wiz_1wan.disconnectTime", 
			get_from_nvram(nvram, "wan_dsl.disconnectTime.1"));
}


static void store_temp_vars(HLST* avp, NVCL* nvram)
{
	add_to_nvram(nvram, "wan_dsl.connection.1",
			get_from_list(avp, "wiz_1wan.connection"));

	add_to_nvram(nvram, "wan_dsl.username.1",
			get_from_list(avp, "wiz_1wan.username"));
	
	add_to_nvram(nvram, "wan_dsl.password.1",
			get_from_list(avp, "wiz_1wan.password"));
	
	add_to_nvram(nvram, "wan_dsl.virtualCircuit.1", "enabled");
}


static int check_username(HLST* avp)
{
	int errors = 0;
	char *username, *password;

	username = get_from_list(avp, "wiz_1wan.username");
	if (!username || !*username) {
		errors++;
		add_to_list(avp, "wiz_1wan.username.empty.error", "1");
	}

	password = get_from_list(avp, "wiz_1wan.password");
	if (!password || !*password) {
		errors++;
		add_to_list(avp, "wiz_1wan.password.empty.error", "1");
	}

	return errors;
}

static void check_dslAvailable(HLST* avp, NVCL* nvram)
{
	char* status = get_from_nvram(nvram, "wan_dsl.carrier_state");

	if (!strncmp("Con", status, 3))
		add_to_list(avp, "wiz_1wan.status", "connected");
	else if (!strncmp("\"Con", status, 4))
		add_to_list(avp, "wiz_1wan.status", "connected");
	else if (!strncmp("Not", status, 3))
		add_to_list(avp, "wiz_1wan.status", "not_connected");
	else if (!strncmp("\"Not", status, 4))
		add_to_list(avp, "wiz_1wan.status", "not_connected");
	else 
		add_to_list(avp, "wiz_1wan.status", "waiting");
		
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) 
{
	STATE* 		s = desc;
	HLST*		avp;

	char		tmpl_name[1024];
	const char	*html_name = argv[1];
	char		*step;
	char		*back, *jumpback;
	int		error = 0;


	if (argc < 2)
		return ;

	/* get POST variables */
	avp = http_args(r);
    
	/* handle global variables such as language */
	handle_global_settings(avp, s->nvram);

	/* check for correct values */
	step = get_from_list(avp, "wiz_1wan.step");
	back = get_from_list(avp, "back");
	jumpback = get_from_list(avp, "jumpback");

	if (jumpback)
	{
		/* set temp variables */
		init_temp_vars(avp, s->nvram);
		
	 	/* check if the dsl account works */
	 	check_dslAvailable(avp, s->nvram);
		
		add_to_list(avp, "wiz_1wan.step", "5");
		
		/* create next html page */
		snprintf(tmpl_name, sizeof(tmpl_name), html_name, 5);
		send_html_page(avp, s, r, tmpl_name);
		return;
	}	
	if (step == NULL)
	{
		/* set temp variables */
		init_temp_vars(avp, s->nvram);

		add_to_list(avp, "wiz_1wan.step", "0");
		step = get_from_list(avp, "wiz_1wan.step");
	}	
	
	if (back) {
		/* go one page back, do not check for errors */
		(*step)--;
		if (*step <= '0')
		{
			/* cannot be reached normally */
			html_name = "wiz/wiz_0start.html";
			send_html_page(avp, s, r, html_name);
		}
		else
		{
			add_to_list(avp, "wiz_1wan.step", step);
		}

		/* create previous html page */
		snprintf(tmpl_name, sizeof(tmpl_name), html_name, atoi(step));
		/* use html page */
		send_html_page(avp, s, r, tmpl_name);

		return;
	}


	switch(*step) {
		case '0':
		case '1':
		case '2':
			/* nothing to check here */
			break;
		case '3':
			/* username and password */
			error += check_username(avp);
			break;
		case '4':
			/* connection type */
			/* save values to nvram */
			store_temp_vars(avp, s->nvram);
			
			/* check if the dsl account works */
			check_dslAvailable(avp, s->nvram);
			break;
		case '5':
			/* summary */
			html_name = "wiz/wiz_2lan1.html";
			add_to_list(avp, "wiz_2lan.step", "1");
			break;
	}

	if (error) {
		add_to_list(avp, "error", "1");
		
	} else {
		/* no error, load next page */
		(*step)++;
		add_to_list(avp, "wiz_1wan.step", step);
	}

	
	/* create next html page */
	snprintf(tmpl_name, sizeof(tmpl_name), html_name, atoi(step));
	send_html_page(avp, s, r, tmpl_name);

}

void *htinit (const char *doc_path, NVCL *nv) 
{
	STATE *s = ZMALLOC(STATE);

	s->nvram  	= nv;
	s->htpath 	= doc_path;

	return s ;
}


void htcleanup (void *desc) 
{
	STATE* s = desc;

	free(s);
}



/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

