/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Henry Margies <h.margies@maxina.de> - Maxina GmbH, Neu-       *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "hlst.h"
#include "util.h"
#include "http-plugin.h"
#include "plugin-util.h"
#include "parser.h"
#include "nvalloc.h"

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */


static void init_temp_vars(HLST* avp, NVCL* nvram)
{
	/* initialize variablen from nvram*/

	add_to_list(avp, "wiz_2lan.ssid",
			get_from_nvram(nvram, "lan_wlan.ssid"));

	add_to_list(avp, "wiz_2lan.security",
			get_from_nvram(nvram, "lan_wlan.security"));

	add_to_list(avp, "wiz_2lan.wpastring",
			get_from_nvram(nvram, "lan_wlan.string"));

	add_to_list(avp, "wiz_2lan.wepencryption",
			get_from_nvram(nvram, "lan_wlan.encryption_A"));
}


static void store_temp_vars(HLST* avp, NVCL* nvram)
{
	add_to_nvram(nvram, "lan_wlan.isAccesspoint", "true");

	add_to_nvram(nvram, "lan_wlan.ssid",
			get_from_list(avp, "wiz_2lan.ssid"));

	add_to_nvram(nvram, "lan_wlan.security",
			get_from_list(avp, "wiz_2lan.security"));

	if(!strcmp("wpa", get_from_list(avp, "wiz_2lan.security")))
	{
		add_to_nvram(nvram, "lan_wlan.string",
				get_from_list(avp, "wiz_2lan.wpastring"));
		add_to_nvram(nvram, "lan_wlan.wpatype", "pskstring");
		add_to_nvram(nvram, "lan_wlan.groupkeyinterval", "50");
	}
	else if(!strcmp("wep", get_from_list(avp, "wiz_2lan.security")))
	{
		char* enc = get_from_list(avp, "wiz_2lan.wepencryption");

		add_to_nvram(nvram, "lan_wlan.encryption_A", enc);

		add_to_nvram(nvram, "lan_wlan.encryptionselection", "A");
		if (validEnc64 (enc)) 
			add_to_nvram(nvram, "lan_wlan.chiper_A", "64");
		else if (validEnc128(enc)) 
			add_to_nvram(nvram, "lan_wlan.chiper_A", "128");
		else
			add_to_nvram(nvram, "lan_wlan.chiper_A", "256");
	}
}

static int check_ssid(HLST *avp)
{
	int errors = 0;
	char *ssidname;

	ssidname = get_from_list(avp, "wiz_2lan.ssid");
	
	if (!ssidname || strlen(ssidname)==0)
	{
		errors++;
		add_to_list(avp, "wiz_2lan.ssid.empty.error", "1");
	}

	return errors;
}

static int check_encryption(HLST *avp)
{
	int errors = 0;
	char *radio, *wpastring, *wepencryption;

	radio = get_from_list(avp, "wiz_2lan.security");

	if(!strcmp("wpa", radio))
	{
		wpastring = get_from_list(avp, "wiz_2lan.wpastring");
		if (!wpastring || !*wpastring)
		{
			errors++;
			add_to_list(avp, "wiz_2lan.wpa.string.empty.error", "1");
		}
	}
	if(!strcmp("wep", radio))
	{
		wepencryption = get_from_list(avp, "wiz_2lan.wepencryption");
		if (	!validEnc64 (wepencryption) &&
			!validEnc128(wepencryption) &&
			!validEnc256(wepencryption))
		{
			errors++;
			add_to_list(avp, "wiz_2lan.wepencryption.error", "1");
		}
	}

	return errors;
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) 
{
	STATE* 		s = desc;
	HLST*		avp;

	char		tmpl_name[1024];
	const char	*html_name = argv[1];
	char		*step;
	char		*back, *jumpback;
	int		error = 0;


	if (argc < 2)
		return ;

	/* get POST variables */
	avp = http_args(r);
    
	/* handle global variables such as language */
	handle_global_settings(avp, s->nvram);

	/* check for correct values */
	step = get_from_list(avp, "wiz_2lan.step");
	back = get_from_list(avp, "back");
	jumpback = get_from_list(avp, "jumpback");
  
	if (jumpback)
	{
		/* set temp variables */
		init_temp_vars(avp, s->nvram);
		
		add_to_list(avp, "wiz_2lan.step", "5");
		
		/* create next html page */
		snprintf(tmpl_name, sizeof(tmpl_name), html_name, 5);
		send_html_page(avp, s, r, tmpl_name);
		return;
	}	
	if (step == NULL)
	{
		/* set temp variables */
		init_temp_vars(avp, s->nvram);

		add_to_list(avp, "wiz_2lan.step", "0");
		step = get_from_list(avp, "wiz_2lan.step");
	}	
	
	if (back) {
		/* go one page back, do not check for errors */
		(*step)--;
		if (*step <= '0')
		{
			/* cannot be reached normally */
			html_name = "wiz/wiz_0start.html";
			send_html_page(avp, s, r, html_name);
		}
		else
		{
			add_to_list(avp, "wiz_2lan.step", step);
			/* create next html page */
			snprintf(tmpl_name, sizeof(tmpl_name), html_name, atoi(step));
			/* use next html page */
			send_html_page(avp, s, r, tmpl_name);
		}
	
		return;
	}


	switch(*step) {
		case '0':
		case '1':
		case '2':
			/* nothing to check here */
			break;
		case '3':
			/* ssid */
			error += check_ssid(avp);
			break;
		case '4':
			/* encryption settings */
			error += check_encryption(avp);
			
			/* save values to nvram */
			if (!error) store_temp_vars(avp, s->nvram);
			break;
		case '5':
			/* summary */
			html_name = "wiz/wiz_3phone.html";
			add_to_list(avp, "wiz_3phone.step", "1");
			break;
	}

	if (error) {
		add_to_list(avp, "error", "1");
		
	} else {
		/* no error, load next page */
		(*step)++;
		add_to_list(avp, "wiz_2lan.step", step);
	}

	
	/* create next html page */
	snprintf(tmpl_name, sizeof(tmpl_name), html_name, atoi(step));
	send_html_page(avp, s, r, tmpl_name);

}

void *htinit (const char *doc_path, NVCL *nv) 
{
	STATE *s = ZMALLOC(STATE);

	s->nvram  	= nv;
	s->htpath 	= doc_path;

	return s ;
}


void htcleanup (void *desc) 
{
	STATE* s = desc;

	free(s);
}



/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

