/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Simon Oellrich <s.oellrich@gmx.de>                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "hlst.h"
#include "util.h"
#include "http-plugin.h"
#include "plugin-util.h"
#include "parser.h"
#include "nvalloc.h"

#define MAX_VAL_PHONE_DEVICES 300
#define MAX_VAL_PHONE_ACCOUNTS 21

//#define STD_NAME_ANALOG_1 	"analoges Telefon an Port A"
//#define STD_NAME_ANALOG_2 	"analoges Telefon an Port B"
//#define STD_NAME_ISDN	 	"erstes ISDN- Telefon"
//#define STD_ACCOUNT_ANALOG	"analoger Telefonanschluss"
//#define STD_ACCOUNT_ISDN	"(weiterer) ISDN-Anschluss %i"

#define STD_ICON		"phone-white"
#define STD_UNKNOWN		""

#define REWRITE_LABEL_ANALOG_1	"wiz_3phone.Analog1.rewrite"
#define REWRITE_LABEL_ANALOG_2	"wiz_3phone.Analog2.rewrite"
#define REWRITE_LABEL_ISDN	"wiz_3phone.ISDN.rewrite"
#define REWRITE_LABEL_ACCOUNTS	"wiz_3phone.ac.rewrite"
#define REWRITE_LABEL_SIP	"wiz_3phone.ac.sip.rewrite"

typedef enum{
	CURRENT_ANALOG_1,
	CURRENT_ANALOG_2,
	CURRENT_ISDN,
}CURRENT_PHONE;

typedef enum{
	HIDDENVAR_TO_ICONVAR,
	ICONVAR_TO_HIDDENVAR,
}ICON_SHIFT_DIRECTION;


typedef enum{
	REWRITE_TRUE,
	REWRITE_FALSE,
}REWRITE_MODUS;

typedef enum{
	ANALOG_CONNECTED,
	ANALOG_NOT_CONNECTED,
}ANALOG_CONECTION_STATUS;

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

ANALOG_CONECTION_STATUS checkAnalogConnected()
{
	return ANALOG_CONNECTED;
	//return ANALOG_NOT_CONNECTED;
}


static void init_temp_vars(HLST* avp, NVCL* nvram)
{
	int		totalAnalog 	= 0;
	int		analog1 	= 0;
	int		analog2 	= 0;
	int		totalIsdn 	= 0;
	int		totalSip	= 0;
	char		*val 		= NULL;
	char 		*tmp		= NULL;
	int 		i;
	char 		str[40];
	char 		label[40];

	/*******************/
	/* DEVICE HANDLING */
	/*******************/
	for(i = 1; (i <= MAX_VAL_PHONE_DEVICES) && ((totalAnalog < 2) || (totalIsdn < 1)); i++){
		snprintf(str, sizeof(str), "phone_devices.type.%i", i);
		tmp = get_from_nvram(nvram, str);
		
		if(tmp == NULL)
			continue;
	
		/* analog device found AND there are not more than two analog phones */
		if(totalAnalog < 2 && !strcmp("analog", tmp)){
			totalAnalog++;
			snprintf(str, sizeof(str), "phone_devices.number.%i", i);
			/* check if it is the first analog phone */
			if(!strcmp("21", get_from_nvram(nvram, str)))
			{
				analog1++;
				/* saving the name of the first anlaog phone */
				snprintf(str, sizeof(str), "phone_devices.name.%i", i);
				add_to_list(avp, "wiz_3phone.name.Analog1",get_from_nvram(nvram, str));
				/* saving the icon of the first anlaog phone */
				snprintf(str, sizeof(str), "phone_devices.icon.%i", i);
				add_to_list(avp, "wiz_3phone.icon.Analog1",get_from_nvram(nvram, str));
				/* well, the id of the first anlaog phone is just i */
				snprintf(str, sizeof(str), "%i", i);
				add_to_list(avp, "wiz_3phone.id.Analog1", str);
					/* add an indicator that these values are already custom */
					//add_to_list(avp, "wiz_3phone.Analog1.isnew", "false");
			}
			/* or the second one */
			else if(!strcmp("22", get_from_nvram(nvram, str)))
			{
				analog2++;
				/* saving the name of the second anlaog phone */
				snprintf(str, sizeof(str), "phone_devices.name.%i", i);
				add_to_list(avp, "wiz_3phone.name.Analog2",get_from_nvram(nvram, str));
				/* saving the icon of the second anlaog phone */
				snprintf(str, sizeof(str), "phone_devices.icon.%i", i);
				add_to_list(avp, "wiz_3phone.icon.Analog2",get_from_nvram(nvram, str));
				/* well, the id of the second anlaog phone is just i */
				snprintf(str, sizeof(str), "%i", i);
				add_to_list(avp, "wiz_3phone.id.Analog2", str);
					/* add an indicator that these values are already custom */
					//add_to_list(avp, "wiz_3phone.Analog2.isnew", "false");
			}
		}
		
		/* isdn device found AND it is the first ISDN phone*/
		if((totalIsdn < 1) && (!strcmp("isdn", tmp))){
			totalIsdn++;
			/* saving the name of the ISDN phone */
			snprintf(str, sizeof(str), "phone_devices.name.%i", i);
			add_to_list(avp, "wiz_3phone.name.ISDN",get_from_nvram(nvram, str));
			/* saving the icon of the ISDN phone */
			snprintf(str, sizeof(str), "phone_devices.icon.%i", i);
			add_to_list(avp, "wiz_3phone.icon.ISDN",get_from_nvram(nvram, str));
			/* well, the id of the ISDN phone is just i */
			snprintf(str, sizeof(str), "%i", i);
			add_to_list(avp, "wiz_3phone.id.ISDN", str);
			/* saving the number of the ISDN phone */
			snprintf(str, sizeof(str), "phone_devices.number.%i", i);
			add_to_list(avp, "wiz_3phone.number.ISDN",get_from_nvram(nvram, str));
				/* add an indicator that these values are already custom */
				//add_to_list(avp, "wiz_3phone.ISDN.isnew", "false");
		}
	}
	
	if (!analog1)
	{
		add_to_list(avp, "wiz_3phone.name.Analog1", ""); // now in html: STD_NAME_ANALOG_1);
		add_to_list(avp, "wiz_3phone.icon.Analog1", STD_ICON);
			/* add an indicator that these values are std.vals. */
			//add_to_list(avp, "wiz_3phone.Analog1.isnew", "true");
	}
	if (!analog2)
	{
		add_to_list(avp, "wiz_3phone.name.Analog2", ""); // now in html:  STD_NAME_ANALOG_2);
		add_to_list(avp, "wiz_3phone.icon.Analog2", STD_ICON);
			/* add an indicator that these values are std.vals. */
			//add_to_list(avp, "wiz_3phone.Analog2.isnew", "true");
	}

	if(!totalIsdn)
	{
		add_to_list(avp, "wiz_3phone.name.ISDN", ""); // now in html: STD_NAME_ISDN);
		add_to_list(avp, "wiz_3phone.number.ISDN", "300"); 
		add_to_list(avp, "wiz_3phone.icon.ISDN", STD_ICON);
			/* add an indicator that these values are std.vals. */
			//add_to_list(avp, "wiz_3phone.ISDN.isnew", "true");
	}

	/* add an indicator if the phones should be rewritten to nvram */
	add_to_list(avp, REWRITE_LABEL_ANALOG_1, "false");
	add_to_list(avp, REWRITE_LABEL_ANALOG_2, "false");
	add_to_list(avp, REWRITE_LABEL_ISDN, "false");
	



	/********************/
	/* ACCOUNT HANDLING */
	/********************/

	totalAnalog 	= 0;
	totalIsdn 	= 0;	
	totalSip 	= 0;
	
	for(i = 1; ((i <= MAX_VAL_PHONE_DEVICES) && ((totalAnalog < 1) || (totalIsdn < 6))); i++){
		snprintf(str, sizeof(str), "phone_accounts.type.%i", i);
		tmp = get_from_nvram(nvram, str);
		
		if(tmp == NULL)
			continue;
	
		/* analog account found AND there is no analog account */
		if(totalAnalog < 1 && !strcmp("analog", tmp)){
			totalAnalog++;
				/* saving the name of the account 
				snprintf(str, sizeof(str), "phone_account.name.%i", i);
				add_to_list(avp, "wiz_3phone.ac.analog.name",get_from_nvram(nvram, str));*/
				/* saving the number of the account */
			snprintf(str, sizeof(str), "phone_accounts.number.%i", i);
			add_to_list(avp, "wiz_3phone.ac.analog.number",get_from_nvram(nvram, str));
			/* well, the id is just i , a given id indicates, that this is not a new account */
			snprintf(str, sizeof(str), "%i", i);
			add_to_list(avp, "wiz_3phone.ac.analog.id", str);
			
				/* add an indicator that the account already exists */
				//add_to_list(avp, "wiz_3phone.ac.analog.isnew", "false");
		}
		
		/* isdn account found AND less than 6 ISDN accounts*/
		if((totalIsdn < 6) && (!strcmp("isdn", tmp))){
			totalIsdn++;
				/* saving the name of the account 
				snprintf(str, sizeof(str), "phone_accounts.name.%i", i);
				snprintf(label, sizeof(label), "wiz_3phone.ac.isdn.name.%i", totalIsdn);
				add_to_list(avp, label ,get_from_nvram(nvram, str));*/
			/* saving the number of the account */
			snprintf(str, sizeof(str), "phone_accounts.number.%i", i);
			snprintf(label, sizeof(label), "wiz_3phone.ac.isdn.number.%i", totalIsdn);
			add_to_list(avp, label, get_from_nvram(nvram, str));
			/* well, the id is just i , a given id indicates, that this is not a new account */
			snprintf(str, sizeof(str), "%i", i);
			snprintf(label, sizeof(label), "wiz_3phone.ac.isdn.id.%i", totalIsdn);
			add_to_list(avp, label, str);
				/* add an indicator that the account already exists */
				//snprintf(label, sizeof(label), "wiz_3phone.ac.isdn.isnew.%i", totalIsdn);
				//add_to_list(avp, label, "false");
		}

		/* sip account found */
		if((totalSip < 1) && (!strcmp("sip", tmp))){
			totalSip++;
			/* saving the number of the account */
			snprintf(str, sizeof(str), "phone_accounts.number.%i", i);
			add_to_list(avp, "wiz_3phone.ac.sip.number", get_from_nvram(nvram, str));
			/* well, the id is just i , a given id indicates, that this is not a new account */
			snprintf(str, sizeof(str), "%i", i);
			add_to_list(avp, "wiz_3phone.ac.sip.id", str);
			/* saving the server of the account */
			snprintf(str, sizeof(str), "phone_accounts.server.%i", i);
			add_to_list(avp, "wiz_3phone.ac.sip.server", get_from_nvram(nvram, str));
			/* username of the account */
			snprintf(str, sizeof(str), "phone_accounts.username.%i", i);
			add_to_list(avp, "wiz_3phone.ac.sip.username", get_from_nvram(nvram, str));
			/* password of the account */
			snprintf(str, sizeof(str), "phone_accounts.password.%i", i);
			add_to_list(avp, "wiz_3phone.ac.sip.passwd", get_from_nvram(nvram, str));
		}
	}
	
	if (!totalAnalog)
	{
		add_to_list(avp, "wiz_3phone.ac.analog.number", STD_UNKNOWN);
			/* add an indicator that these values are std.vals. */
			//add_to_list(avp, "wiz_3phone.ac.analog1.isnew", "true");
	}

	if (!totalSip)
	{
		add_to_list(avp, "wiz_3phone.ac.sip.number", STD_UNKNOWN);
	}

	for (i = totalIsdn+1; i < 6; i++)
	{
		snprintf(label, sizeof(label), "wiz_3phone.ac.isdn.number.%i", i);
		add_to_list(avp, label, "");
			/* add an indicator that the account already exists */
			//snprintf(label, sizeof(label), "wiz_3phone.ac.isdn.isnew.%i", i);
			//add_to_list(avp, label, "true");
	}

	/* indicator for the main account: analog or isdn */
	if (totalIsdn)
		add_to_list(avp, "wiz_3phone.ac.main", "isdn");
	else if (checkAnalogConnected(nvram, avp) == ANALOG_NOT_CONNECTED)
		add_to_list(avp, "wiz_3phone.ac.main", "isdn");
	else
		add_to_list(avp, "wiz_3phone.ac.main", "analog");

	/* add an indicator if the acounts should be rewritten to nvram */
	add_to_list(avp, REWRITE_LABEL_ACCOUNTS, "false");
	add_to_list(avp, REWRITE_LABEL_SIP, "false");
}

char *addElem(HLST *avp, const char *list, const char *totalElem)
{
	char *val;
	char *id;

	/* get the next available phone id, based on the list and the max elements */
	id = getNextIndex(avp, list, totalElem);

	/* add the id to the given list */
	val = addNewIndex(avp, (char*)list, id);
	/* add the list var */
	add_to_list(avp, list, val);

	/* get the current total elements */
	val = get_from_list(avp, totalElem);
	/* increase it */
	(*val)++;
	/* and write it back */
	add_to_list(avp, totalElem, val);

	/* increase tot */
	//val = increasedTotalElem(avp, totalElem);
	//add_to_list(avp, totalElem, val);

	return id;
}

void initTimeslots2 (HLST *avp, NVCL* nvram, const char *id)
{
	char *phonelist;
	char label[100];
	char *test;
				add_to_list(avp, "DEBUGQ", "dd");
					
	phonelist = get_from_nvram(nvram, "phone_devices.list");
				add_to_list(avp, "DEBUGE", phonelist);	
	if (!phonelist || !*phonelist)
	{
		phonelist = " ";
	}
				add_to_list(avp, "DEBUGR", phonelist);		
	
	snprintf(label, sizeof(label), "phone_accounts.timeslot_0.%i", atoi(id));
			add_to_list(avp, "DEBUG0A", phonelist);	
	add_to_nvram(nvram, label, phonelist);
			add_to_list(avp, "DEBUG01", label);	
			add_to_list(avp, "DEBUG02", phonelist);	
	snprintf(label, sizeof(label), "phone_accounts.timeslot_10.%i", atoi(id));
	add_to_nvram(nvram, label, phonelist);
			add_to_list(avp, "DEBUG03", label);	
			add_to_list(avp, "DEBUG04", phonelist);	
	snprintf(label, sizeof(label), "phone_accounts.timeslot_30.%i", atoi(id));
	add_to_nvram(nvram, label, phonelist);
			add_to_list(avp, "DEBUG05", label);	
			add_to_list(avp, "DEBUG06", phonelist);	
	snprintf(label, sizeof(label), "phone_accounts.timeslot_45.%i", atoi(id));
	add_to_nvram(nvram, label, phonelist);
			add_to_list(avp, "DEBUG07", label);	
			add_to_list(avp, "DEBUG08", phonelist);	
	snprintf(label, sizeof(label), "phone_accounts.timeslot_60.%i", atoi(id));
	add_to_nvram(nvram, label, phonelist);
			add_to_list(avp, "DEBUG09", label);	
			add_to_list(avp, "DEBUG00", phonelist);	
			
	phonelist = " a bc ";
	test = "phone_XX_test1";
	add_to_nvram(nvram, test, phonelist);	
	test = "phone_XX_test2";
	add_to_nvram(nvram, test, phonelist);	
	test = "phone_XX_test3";
	add_to_nvram(nvram, test, phonelist);	
	test = "phone_XX_test4";
	add_to_nvram(nvram, test, phonelist);	
	test = "phone_XX_test5";
	add_to_nvram(nvram, test, phonelist);	
	test = "phone_XX_test6";
	add_to_nvram(nvram, test, phonelist);	
	test = "phone_XX_test7";
	add_to_nvram(nvram, test, phonelist);	
	
	
/*	phonelist = "abc";
	add_to_nvram(nvram, "phone_XX_test1", phonelist);	
	add_to_nvram(nvram, "phone_XX_test2", phonelist);	
	add_to_nvram(nvram, "phone_XX_test3", phonelist);	
	add_to_nvram(nvram, "phone_XX_test4", phonelist);	
	add_to_nvram(nvram, "phone_XX_test5", phonelist);	
	add_to_nvram(nvram, "phone_XX_test6", phonelist);	
	add_to_nvram(nvram, "phone_XX_test7", phonelist);	
	phonelist = "a b c";
	add_to_nvram(nvram, "phone_XX_test_1", phonelist);	
	add_to_nvram(nvram, "phone_XX_test_2", phonelist);	
	add_to_nvram(nvram, "phone_XX_test_3", phonelist);	
	add_to_nvram(nvram, "phone_XX_test_4", phonelist);	
	add_to_nvram(nvram, "phone_XX_test_5", phonelist);	
	add_to_nvram(nvram, "phone_XX_test_6", phonelist);	
	add_to_nvram(nvram, "phone_XX_test_7", phonelist);	*/
			
}

void initTimeslots (HLST *avp, NVCL* nvram, const char *id)
{
	char label[70];
	
	snprintf(label, sizeof(label), "phone_accounts.timeslot_0.%i", atoi(id));
	add_to_nvram(nvram, label, get_from_nvram(nvram, "phone_devices.list"));
	snprintf(label, sizeof(label), "phone_accounts.timeslot_10.%i", atoi(id));
	add_to_nvram(nvram, label, get_from_nvram(nvram, "phone_devices.list"));
	snprintf(label, sizeof(label), "phone_accounts.timeslot_30.%i", atoi(id));
	add_to_nvram(nvram, label, get_from_nvram(nvram, "phone_devices.list"));
	snprintf(label, sizeof(label), "phone_accounts.timeslot_45.%i", atoi(id));
	add_to_nvram(nvram, label, get_from_nvram(nvram, "phone_devices.list"));
	snprintf(label, sizeof(label), "phone_accounts.timeslot_60.%i", atoi(id));
	add_to_nvram(nvram, label, get_from_nvram(nvram, "phone_devices.list"));

}

void pubDevId(NVCL* nvram, const char *id)
{
	HSRCH 		*lookup;
	char 		*timeslot_label		= "phone_accounts.timeslot";
	HLST 		*nvram_avp 		= NULL;
	HLST 		*tmp_avp 		= NULL;		/* to have a list which contains only the values we wanna store */ 
	const char	**Val			= NULL;
	char		*var 			= NULL;
	char 		*_newTimeslot		= NULL;
     
	/* load nvram variables for search */
	tmp_avp = create_hlst(100, nvram_avp_destroy, NULL);
	nvram_avp = create_hlst(300, nvram_avp_destroy, NULL);
	load_from_nvram(nvram , nvram_avp, NULL);

	/* we are adding a new device, so add the id to the current timeslot-list of the phone_accounts */
	lookup = open_hlst_search (nvram_avp);
	while ((Val = (const char**) next_hlst_search (lookup)) != 0) 
	{
		/* get the associated variable */
		var = query_key_hlst((void **)Val) ;
		
		if(strncmp(timeslot_label, var, strlen(timeslot_label)) == 0)
		{
			/* we found a timeslot variable, lets add the device id */

			/* if the index is already in the list, remove it first and then add it again */
			_newTimeslot = removeIndex(nvram_avp, var, id);

			/* add changed timeslot list to nvram_avp */
			if(_newTimeslot)
				forceAddToList(nvram_avp,var,_newTimeslot);
			/* now add new Index to timeslot */
			_newTimeslot = addNewIndex(nvram_avp,var,id);
                        
			if(_newTimeslot)
			{
				if(*_newTimeslot == '\0')
					forceAddToList(tmp_avp,var," ");
				else
					forceAddToList(tmp_avp,var,_newTimeslot);

				free(_newTimeslot);
				_newTimeslot = NULL;
			}
		}
		
        
		/* that was not what we're looking for */
		var = NULL;
	}
	close_hlst_search (lookup) ;


	store_into_nvram(nvram,tmp_avp);
	destroy_hlst(nvram_avp);
	destroy_hlst(tmp_avp);
}

static void store_temp_vars(HLST* avp, NVCL* nvram)
{
	char tmp[70];
	char label[70];
	char *id;
	char *list;
	char *elem;
	int i;

	/* *** ACCOUNTS *** */

	add_to_list(avp, "wiz_3phone.totalElements", get_from_nvram(nvram, "phone_accounts.totalElements"));	
	add_to_list(avp, "wiz_3phone.list", get_from_nvram(nvram, "phone_accounts.list"));	

	/* check wether a account should be rewritten to nvram */
	if(!strcmp("true", get_from_list(avp, REWRITE_LABEL_ACCOUNTS)))
	{
		if(!strcmp("analog", get_from_list(avp, "wiz_3phone.ac.main")))
		{
			id = get_from_list(avp, "wiz_3phone.ac.analog.id");
			if(!id || *id=='\0')
			{
				/* new account, so new id, new type+number, new list, new totalElements, new timeslots */
				id = addElem(avp, "wiz_3phone.list", "wiz_3phone.totalElements");
				
				snprintf(tmp, sizeof(tmp), "phone_accounts.name.%s", id);
				add_to_nvram(nvram, tmp, get_from_list(avp, "STD_ACCOUNT_ANALOG"));
				snprintf(tmp, sizeof(tmp), "phone_accounts.type.%s", id);
				add_to_nvram(nvram, tmp, "analog");
				snprintf(tmp, sizeof(tmp), "phone_accounts.id.%s", id);
				add_to_nvram(nvram, tmp, id);
				initTimeslots (avp, nvram, id);
				
				/* make id public to hidden vars, important for back-button */
				add_to_list(avp, "wiz_3phone.ac.analog.id", id);
			}
					//else
					//{
						/* the account just exists, but some values might have changed */
					//	id = get_from_list(avp, "wiz_3phone.ac.analog.id");
					//}

			/* these values must be set anyway, independent of if its a new account or not */
			snprintf(tmp, sizeof(tmp), "phone_accounts.number.%i", atoi(id));
			add_to_nvram(nvram, tmp, get_from_list(avp, "wiz_3phone.ac.analog.number"));

			add_to_nvram(nvram, "phone_dials.standardaccount", id);
			add_to_nvram(nvram, "phone_dials.sipfailover", id);
		}
		else
		{
			for (i = 1; i < 6; i++)
			{
				snprintf(tmp, sizeof(tmp), "wiz_3phone.ac.isdn.id.%i", i);
				id = get_from_list(avp, tmp);
				if(!id || *id=='\0')
				{
					/* new account, so new id, new type+number, new list, new totalElements, new timeslots */
					snprintf(label, sizeof(label), "wiz_3phone.ac.isdn.number.%i", i);
					elem = get_from_list(avp, label);
					if(elem && *elem)
					{
						id = addElem(avp, "wiz_3phone.list", "wiz_3phone.totalElements");

						snprintf(tmp, sizeof(tmp), "phone_accounts.number.%i", atoi(id));
						add_to_nvram(nvram, tmp, elem);
						snprintf(tmp, sizeof(tmp), "phone_accounts.name.%i", atoi(id));
						snprintf(label, sizeof(label), "%s %i",get_from_list(avp, "STD_ACCOUNT_ISDN"), i);
						add_to_nvram(nvram, tmp, label);
						snprintf(tmp, sizeof(tmp), "phone_accounts.type.%i", atoi(id));
						add_to_nvram(nvram, tmp, "isdn");
						snprintf(tmp, sizeof(tmp), "phone_accounts.id.%i", atoi(id));
						add_to_nvram(nvram, tmp, id);

						initTimeslots (avp, nvram, id);
						
						/* make id public to hidden vars, important for back-button */
						snprintf(tmp, sizeof(tmp), "wiz_3phone.ac.isdn.id.%i", i);
						add_to_list(avp, tmp, id);
						
					}
				}
				else
				{
					/* the account just exists, but some values might have changed */
					snprintf(tmp, sizeof(tmp), "phone_accounts.number.%s", id);
					snprintf(label, sizeof(label), "wiz_3phone.ac.isdn.number.%i", i);
					add_to_nvram(nvram, tmp, get_from_list(avp, label));
				}
				if (i==1) {
					add_to_nvram(nvram, "phone_dials.standardaccount", id); 
					add_to_nvram(nvram, "phone_dials.sipfailover", id);
				}
			}
		}
	}
	
	if(!strcmp("true", get_from_list(avp, REWRITE_LABEL_SIP))){
		id = get_from_list(avp, "wiz_3phone.ac.sip.id");
		if(!id || *id=='\0')
		{
			id = addElem(avp, "wiz_3phone.list", "wiz_3phone.totalElements");
				
			snprintf(tmp, sizeof(tmp), "phone_accounts.id.%s", id);
			add_to_nvram(nvram, tmp, id);
			snprintf(tmp, sizeof(tmp), "phone_accounts.name.%s", id);
			add_to_nvram(nvram, tmp, "voip-konto");
			snprintf(tmp, sizeof(tmp), "phone_accounts.type.%s", id);
			add_to_nvram(nvram, tmp, "sip");
			
			initTimeslots (avp, nvram, id);
			
			/* make id public to hidden vars, important for back-button */
			add_to_list(avp, "wiz_3phone.ac.sip.id", id);			
		
			
		}	
		
		snprintf(tmp, sizeof(tmp), "phone_accounts.server.%s", id);
		add_to_nvram(nvram, tmp, get_from_list(avp, "wiz_3phone.ac.sip.server"));
		snprintf(tmp, sizeof(tmp), "phone_accounts.username.%s", id);
		add_to_nvram(nvram, tmp, get_from_list(avp, "wiz_3phone.ac.sip.username"));
		snprintf(tmp, sizeof(tmp), "phone_accounts.password.%s", id);
		add_to_nvram(nvram, tmp, get_from_list(avp, "wiz_3phone.ac.sip.passwd"));
		snprintf(tmp, sizeof(tmp), "phone_accounts.number.%s", id);
		add_to_nvram(nvram, tmp, get_from_list(avp, "wiz_3phone.ac.sip.number"));
		
	}

	add_to_nvram(nvram, "phone_accounts.totalElements", get_from_list(avp, "wiz_3phone.totalElements"));	
	add_to_nvram(nvram, "phone_accounts.list", get_from_list(avp, "wiz_3phone.list"));

	/* *** DEVICES *** */

	add_to_list(avp, "wiz_3phone.totalElements", get_from_nvram(nvram, "phone_devices.totalElements"));	
	add_to_list(avp, "wiz_3phone.list", get_from_nvram(nvram, "phone_devices.list"));	
	
	/* check wether this phone should be rewritten to nvram */
	if(!strcmp("true", get_from_list(avp, REWRITE_LABEL_ANALOG_1)))
	{
		id = get_from_list(avp, "wiz_3phone.id.Analog1");
		if(!id || !*id)
		{
			/* new phone, so new id, new type+number, new list, new totalElements */
			id = addElem(avp, "wiz_3phone.list", "wiz_3phone.totalElements");
			
			snprintf(tmp, sizeof(tmp), "phone_devices.number.%s", id);
			add_to_nvram(nvram, tmp, "21");
			snprintf(tmp, sizeof(tmp), "phone_devices.type.%s", id);
			add_to_nvram(nvram, tmp, "analog");
			snprintf(tmp, sizeof(tmp), "phone_devices.id.%s", id);
			add_to_nvram(nvram, tmp, id);
			
			/* add device id to all account-timeslots, so the devices rings per default */
    			pubDevId(nvram, id);
    			
			/* make id public to hidden vars, important for back-button */
			add_to_list(avp, "wiz_3phone.id.Analog1", id);	    			
		}
				//else
				//{
					/* the phone just exists, but some values might have changed */
					//id = get_from_list(avp, "wiz_3phone.id.Analog1");
					//addPhonesToAccounts(nvram, avp, id, "onlyNewOnes");
				//}

		/* these values must be set anyway, independent of if its a new phone or not */
		snprintf(tmp, sizeof(tmp), "phone_devices.name.%s", id);
		add_to_nvram(nvram, tmp, get_from_list(avp, "wiz_3phone.name.Analog1"));
		snprintf(tmp, sizeof(tmp), "phone_devices.icon.%s", id);
		add_to_nvram(nvram, tmp, get_from_list(avp, "wiz_3phone.icon.Analog1"));

	}
	if(!strcmp("true", get_from_list(avp, REWRITE_LABEL_ANALOG_2)))
	{
		id = get_from_list(avp, "wiz_3phone.id.Analog2");
		if(!id || !*id)
		{
			/* new phone, so new id, new type+number, new list, new totalElements */
			id = addElem(avp, "wiz_3phone.list", "wiz_3phone.totalElements");
			
			snprintf(tmp, sizeof(tmp), "phone_devices.number.%s", id);
			add_to_nvram(nvram, tmp, "22");
			snprintf(tmp, sizeof(tmp), "phone_devices.type.%s", id);
			add_to_nvram(nvram, tmp, "analog");
			snprintf(tmp, sizeof(tmp), "phone_devices.id.%s", id);
			add_to_nvram(nvram, tmp, id);
			
			/* add device id to all account-timeslots, so the devices rings per default */
    			pubDevId(nvram, id);
    			
			/* make id public to hidden vars, important for back-button */
			add_to_list(avp, "wiz_3phone.id.Analog2", id);	    			
		}
				//else
				//{
					/* the phone just exists, but some values might have change */
					//id = get_from_list(avp, "wiz_3phone.id.Analog2");
					//addPhonesToAccounts(nvram, avp, id, "onlyNewOnes");
				//}

		/* these values must be set anyway, independent of if its a new phone or not */
		snprintf(tmp, sizeof(tmp), "phone_devices.name.%i", atoi(id));
		add_to_nvram(nvram, tmp, get_from_list(avp, "wiz_3phone.name.Analog2"));
		snprintf(tmp, sizeof(tmp), "phone_devices.icon.%i", atoi(id));
		add_to_nvram(nvram, tmp, get_from_list(avp, "wiz_3phone.icon.Analog2"));

	}
	if(!strcmp("true", get_from_list(avp, REWRITE_LABEL_ISDN)))
	{
		id = get_from_list(avp, "wiz_3phone.id.ISDN");
		if(!id || !*id)
		{
			/* new phone, so new id, new type+number, new list, new totalElements */
			id = addElem(avp, "wiz_3phone.list", "wiz_3phone.totalElements");
			
			snprintf(tmp, sizeof(tmp), "phone_devices.type.%s", id);
			add_to_nvram(nvram, tmp, "isdn");
			snprintf(tmp, sizeof(tmp), "phone_devices.id.%s", id);
			add_to_nvram(nvram, tmp, id);
			snprintf(tmp, sizeof(tmp), "phone_devices.number.%s", id);
			add_to_nvram(nvram, tmp, "300");
    			
			/* add device id to all account-timeslots, so the devices rings per default */
    			pubDevId(nvram, id);
    			
			/* make id public to hidden vars, important for back-button */
			add_to_list(avp, "wiz_3phone.id.ISDN", id);	    			
		}
				//else
				//{
					/* the phone just exists, but some values might have change */
					//id = get_from_list(avp, "wiz_3phone.id.ISDN");
					//addPhonesToAccounts(nvram, avp, id, "onlyNewOnes");
				//}

		/* these values must be set anyway, independent of if its a new phone or not */
		snprintf(tmp, sizeof(tmp), "phone_devices.name.%s", id);
		add_to_nvram(nvram, tmp, get_from_list(avp, "wiz_3phone.name.ISDN"));
		snprintf(tmp, sizeof(tmp), "phone_devices.icon.%s", id);
		add_to_nvram(nvram, tmp, get_from_list(avp, "wiz_3phone.icon.ISDN"));

	}

	add_to_nvram(nvram, "phone_devices.totalElements", get_from_list(avp, "wiz_3phone.totalElements"));	
	add_to_nvram(nvram, "phone_devices.list", get_from_list(avp, "wiz_3phone.list"));
}

int checkUsername(HLST *avp)
{
	char *username = get_from_list(avp, "wiz_3phone.ac.sip.username");
	
	if(!username || strlen(username)==0 ){
		add_to_list(avp, "wiz_3phone.ac.sip.username.empty.error", "1");
		return 1;
	}

	return 0;
}

int checkServer(HLST *avp)
{
	char *server = get_from_list(avp, "wiz_3phone.ac.sip.server");
	
	if(!server || strlen(server)==0 ){
		add_to_list(avp, "wiz_3phone.ac.sip.server.empty.error", "1");
		return 1;
	}

	return 0;
}

static int checkPassword(HLST *avp){
	char *passwd = get_from_list(avp, "wiz_3phone.ac.sip.passwd");
	
	if(!passwd || strlen(passwd)==0){
		add_to_list(avp, "wiz_3phone.ac.sip.passwd.empty.error", "1");
		return 1;
	}

	return 0;
}

int checkCallnumber(HLST *avp, NVCL *nvram)
{
	int error = 0;
	int i = 0;
	HLST *nvram_avp;
	char label[40];	
	char *val;
	char *callnumber = get_from_list(avp, "wiz_3phone.ac.sip.number");
	char **var = (char**)find_hlst(avp, "wiz_3phone.ac.sip.number", 0);

	if(!callnumber || strlen(callnumber)==0 || !validPhoneNumber(var)){
	//if(!callnumber || strlen(callnumber)==0 ){
		add_to_list(avp, "wiz_3phone.ac.sip.number.empty.error", "1");
		return 1;
	}
	
	/* check if its an known phone and the name has not changed yet
	   (the problem is, that then its name is on the nvram und would 
	    provide a false positive) */
	val = get_from_list(avp, "wiz_3phone.ac.sip.id");
	if (val && *val)
	{
		snprintf(label, sizeof(label), "phone_accounts.number.%s", val);
		val = get_from_nvram(nvram, label);
		/* if the names are identical, the current name is ok, of cource */
		if (strcmp(val, callnumber) == 0)
		{
			return 0;
		}
	}
	
	/* check if the device name is already used on nvram */
	nvram_avp = create_hlst(300, nvram_avp_destroy, NULL);
	load_from_nvram(nvram, nvram_avp, NULL);	
	
	if(getExistNvramVar(nvram_avp, "phone_accounts.number", callnumber))	
	{
		error++;
		add_to_list(avp, "wiz_3phone.ac.sip.number.exists.error", "1");
	}

	if(!strcmp(get_from_list(avp, "wiz_3phone.ac.rewrite"), "true")){
		if(!strcmp(get_from_list(avp, "wiz_3phone.ac.main"), "analog") && !strcmp(get_from_list(avp, "wiz_3phone.ac.analog.number"),get_from_list(avp, "wiz_3phone.ac.sip.number"))){
			error++;
			add_to_list(avp, "wiz_3phone.ac.sip.analog.number.exists.error", "1");
		}
		if(!strcmp(get_from_list(avp, "wiz_3phone.ac.main"), "isdn")){
			for( i = 1; i < 6; i++){
				snprintf(label, sizeof(label), "wiz_3phone.ac.isdn.number.%i", i);
				if(!strcmp(get_from_list(avp, label), callnumber)){
					error++;
					add_to_list(avp, "wiz_3phone.ac.sip.isdn.number.exists.error", "1");
				}
			}
		}
	}
	
	destroy_hlst(nvram_avp);
   
	return error;
}

int checkAccounts(HLST *avp, NVCL *nvram)
{
	int error = 0;
	int i,j;
	char *accountType = get_from_list(avp, "wiz_3phone.ac.main");
	char *analog;
	char *isdn[5];
	char *ids[5];
	char *id;
	char label[60];
	char **numberEntity = NULL;
	char *nvram_number;
	HLST *nvram_avp;
	int flag_checkNv = 1;

	accountType = get_from_list(avp, "wiz_3phone.ac.main");
	if (!strcmp("analog", accountType))
	{
		analog = get_from_list(avp, "wiz_3phone.ac.analog.number");
		numberEntity = (char**)find_hlst(avp, "wiz_3phone.ac.analog.number", 0);

		/* if analog account already exists it must have a valid number */
		id = get_from_list(avp, "wiz_3phone.ac.analog.id");
		if(!analog || !*analog)
		{
			add_to_list(avp, "wiz_3phone.ac.analog.empty.error", "1");
			return 1;
		}
		/*else if(id && (!analog || !*analog))
		{
			add_to_list(avp, "wiz_3phone.ac.analog.exists.empty.error", "1");
			return 1;
		}*/
		else if(!validPhoneNumber(numberEntity))
		{
			add_to_list(avp, "wiz_3phone.ac.analog.invalid.error", "1");
			return 1;
		}
		else 
		{
			/* check if the device name is already used on nvram */
			/* create temp list */
			nvram_avp = create_hlst(300, nvram_avp_destroy, NULL);
			load_from_nvram(nvram, nvram_avp, NULL);

			/* check if its an known phone and the name has not changed yet
			   (the problem is, that then its name is on the nvram und would 
			    provide a false positive) */
			if (id && *id)
			{
				snprintf(label, sizeof(label), "phone_accounts.number.%s", id);
				nvram_number = get_from_nvram(nvram, label);
				/* if the names are identical, the current name is ok, of cource */
				if (strcmp(nvram_number, analog) == 0)
				{
					flag_checkNv = 0;
				}
			}
			
			/* otherwise check, if the number is used by any other account */
			if(flag_checkNv && getExistNvramVar(nvram_avp, "phone_accounts.number", analog))
			{
				snprintf(label, sizeof(label), "wiz_3phone.ac.analog.exits.double", (i+1));
				add_to_list(avp, label, "1");
				error = 1;
			}			
			else if(!strcmp(analog, get_from_list(avp, "wiz_3phone.ac.sip.number")))
			{
				add_to_list(avp, "wiz_3phone.ac.analog.sip.number.exists.error", "1");
				error = 1;
			}			
			
			destroy_hlst(nvram_avp);
		}
	}
	else if (!strcmp("isdn", accountType))
	{
		/* fill isdn, empty number of existing account is not allowed */
		for (i = 0; i < 5; i++)
		{
			/* get number */
			snprintf(label, sizeof(label), "wiz_3phone.ac.isdn.number.%i", (i+1));
			isdn[i] = get_from_list(avp, label);
			numberEntity = (char**)find_hlst(avp, label, 0);
			/*get id */
			snprintf(label, sizeof(label), "wiz_3phone.ac.isdn.id.%i", (i+1));
			ids[i] = get_from_list(avp, label);
			
			/* the first number may not be empty */
			if (i == 0 && (!isdn[i] || !*isdn[i]))
			{
				error++;
				add_to_list(avp, "wiz_3phone.ac.isdn.stdnum.empty.error", "1");
				isdn[i] = NULL;			
			}
			/* if account exists (id is set) the number may not me deleted */
			else if(ids[i] && *ids[i] && (!isdn[i] || !*isdn[i]))
			{
				error++;
				snprintf(label, sizeof(label), "wiz_3phone.ac.isdn.%i.exists.empty.error", (i+1));
				add_to_list(avp, label, "1");
				isdn[i] = NULL;
			}
			/* check for valid number */
			else if(!validPhoneNumber(numberEntity))
			{
				error++;
				snprintf(label, sizeof(label), "wiz_3phone.ac.isdn.%i.invalid.error", (i+1));
				add_to_list(avp, label, "1");
				isdn[i] = NULL;
					
			}			
		}
		
		/* if a number is given, it may not de the same as any of the other given */
		for (i = 0; i < 5; i++){
			if (isdn[i] && *isdn[i]){
				for (j = 0; j < 5; j++){
					if (i!=j && (isdn[j] && *isdn[j]) && strcmp(isdn[i], isdn[j]) == 0)
					{
						error++;
						add_to_list(avp, "wiz_3phone.ac.isdn.hidden.double.error", "1");
						j = 5;
						i = 5;
					}
				}
			}
		}
		
		if (error)
		{
			return error;
		}
		
		/* check if the device name is already used on nvram */
		/* create temp list */
		nvram_avp = create_hlst(300, nvram_avp_destroy, NULL);
		load_from_nvram(nvram, nvram_avp, NULL);
		
		for (i = 0; i < 5; i++)
		{
			if (isdn[i] && *isdn[i])
			{
				/* check if its an known account and the number has not changed yet
				   (the problem is, that then its number is on the nvram und would 
				    provide a false positive) */				
				if (ids[i] && *ids[i])
				{
					snprintf(label, sizeof(label), "phone_accounts.number.%s", ids[i]);
					nvram_number = get_from_nvram(nvram, label);

					/* if the numbers are identical, the current number is ok, 
					   because it is already in nvram */
					if (strcmp(nvram_number, isdn[i]) == 0)
					{
						continue;
					}
				}
				
				/* otherwise check, if the number is used by any other account */
				if(getExistNvramVar(nvram_avp, "phone_accounts.number", isdn[i]))	
				{
					snprintf(label, sizeof(label), "wiz_3phone.ac.isdn.%i.exits.double", (i+1));
					add_to_list(avp, label, "1");
					error++;
				}					
		
				/* check if number is used for sip, but only if the number is NOT used in nvram.
				   Otherwise, if sip number entered and on nvram is the same, the input field provides 2 errors */
				else if(!strcmp(isdn[i],get_from_list(avp, "wiz_3phone.ac.sip.number")) && !strcmp(get_from_list(avp, REWRITE_LABEL_SIP),"true")){
					error++;
					snprintf(label, sizeof(label), "wiz_3phone.ac.isdn.sip.number.exists.error.%i", (i+1));
					add_to_list(avp, label, "1");
				}	
			}
		}				

		destroy_hlst(nvram_avp);

	}
	else
	{
		return 1;
	}
		
	return error;	
}	

int checkDeviceName(HLST *avp, NVCL *nvram, CURRENT_PHONE currentPhone)
{
	HLST *nvram_avp;
	int error = 0;
	char label[40];	
	char *val;

	char *dev_value = NULL;
	char *dev_label = NULL;
	char *dev_id_label = NULL;
	char *dev_label_ana1 = "wiz_3phone.name.Analog1";
	char *dev_label_ana2 = "wiz_3phone.name.Analog2";
	char *dev_label_isdn = "wiz_3phone.name.ISDN";
	char *dev_label_id_ana1 = "wiz_3phone.id.Analog1";
	char *dev_label_id_ana2 = "wiz_3phone.id.Analog2";
	char *dev_label_id_isdn = "wiz_3phone.id.ISDN";
	char *dev_value_ana1 = get_from_list(avp, dev_label_ana1);
	char *dev_value_ana2 = get_from_list(avp, dev_label_ana2);
	char *dev_value_isdn = get_from_list(avp, dev_label_isdn);

	/* check if any of the names provided in the wizzard are doubled */
	if (strcmp(dev_value_ana1, dev_value_ana2) == 0 ||
	    strcmp(dev_value_ana1, dev_value_isdn) == 0 ||
	    strcmp(dev_value_ana2, dev_value_isdn) == 0 )
	{
		add_to_list(avp, "wiz_3phone.devicename.exists.error", "1");
		return 1;
	}
	if(currentPhone == CURRENT_ANALOG_1)
	{
		dev_id_label = dev_label_id_ana1;
		dev_label = dev_label_ana1;
		dev_value = dev_value_ana1;
	}
	else if(currentPhone == CURRENT_ANALOG_2)
	{
		dev_id_label = dev_label_id_ana2;
		dev_label = dev_label_ana2;
		dev_value = dev_value_ana2;
	}
	else if(currentPhone == CURRENT_ISDN)
	{
		dev_id_label = dev_label_id_isdn;
		dev_label = dev_label_isdn;
		dev_value = dev_value_isdn;
	}
	else
	{
			//add_to_list(avp, "DEBUG1", "cur");
		return 1;
	}

		
	if (!dev_value || !*dev_value)
	{
		add_to_list(avp, "wiz_3phone.devicename.empty.error", "1");
		return 1;
	}
	
	/* check if its an known phone and the name has not changed yet
	   (the problem is, that then its name is on the nvram und would 
	    provide a false positive) */
	val = get_from_list(avp, dev_id_label);
	if (val && *val)
	{
		snprintf(label, sizeof(label), "phone_devices.name.%s", val);
		val = get_from_nvram(nvram, label);
		/* if the names are identical, the current name is ok, of course */
		if (strcmp(val, dev_value) == 0)
		{
			return 0;
		}
	}
	
	

	/* check if the device name is already used on nvram */
	nvram_avp = create_hlst(300, nvram_avp_destroy, NULL);
	load_from_nvram(nvram, nvram_avp, NULL);	
	
	if(getExistNvramVar(nvram_avp, "phone_devices.name", dev_value))	
	{
		error++;
		add_to_list(avp, "wiz_3phone.devicename.exists.error", "1");
	}
	
	destroy_hlst(nvram_avp);

	return error;
}

shiftIcon(HLST *avp, CURRENT_PHONE currentPhone, ICON_SHIFT_DIRECTION direction)
{
	int error = 0;
	char *icon = NULL;
	char *dev_value = NULL;
	char *dev_label = NULL;
	
	char *dev_label_ana1 = "wiz_3phone.icon.Analog1";
	char *dev_label_ana2 = "wiz_3phone.icon.Analog2";
	char *dev_label_isdn = "wiz_3phone.icon.ISDN";
	
	if(currentPhone == CURRENT_ANALOG_1)
		dev_label = dev_label_ana1;
	else if(currentPhone == CURRENT_ANALOG_2)
		dev_label = dev_label_ana2;
	else if(currentPhone == CURRENT_ISDN)
		dev_label = dev_label_isdn;
	else return;
	
	if(direction == ICONVAR_TO_HIDDENVAR)
	{
		icon = get_from_list(avp, "phone_devices.icon");
		if (!icon || !*icon)
		{
			add_to_list(avp, dev_label, STD_ICON);
			add_to_list(avp, "phone_devices.icon", STD_ICON);
		}
		else
			add_to_list(avp, dev_label, icon);
	}
	else if(direction == HIDDENVAR_TO_ICONVAR)
	{
		icon = get_from_list(avp, dev_label);
		if (!icon || !*icon)
		{
			add_to_list(avp, dev_label, STD_ICON);
			add_to_list(avp, "phone_devices.icon", STD_ICON);
		}
		else
			add_to_list(avp, "phone_devices.icon", icon);
	}
}

void letItRing(NVCL* nvram, const char* number)
{
	if (!number)
		return;

	/* let it ring */
	add_to_nvram(nvram, "phone_devices.test", number);
	
}

void checkRing(HLST *avp, NVCL *nvram, CURRENT_PHONE currentPhone, int *step) 
{


	char *val;
	char *number = NULL;
	char label[40];

	if (!get_from_list(avp, "test"))
		return;
			
	if(currentPhone == CURRENT_ANALOG_1)
		number = "21";
	else if(currentPhone == CURRENT_ANALOG_2)
		number = "22";
	else if(currentPhone == CURRENT_ISDN)
	{
		val = get_from_list(avp, "wiz_3phone.id.ISDN");
		if (!val || !*val)
			number = "300";
		else
		{
			snprintf(label, sizeof(label), "phone_devices.number.%s", val);
			val = get_from_nvram(nvram, label);
			number = val;
		}
	}
	else return;
	
	(*step)--;

	letItRing(nvram, number);
}

void changeRewrite(HLST *avp, const char* label, REWRITE_MODUS modus)
{
	if (modus == REWRITE_TRUE)
		add_to_list(avp, label, "true");
	else if (modus == REWRITE_FALSE)
		add_to_list(avp, label, "false");
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) 
{
	STATE* 		s = desc;
	HLST*		avp;

	char		tmpl_name[1024];
	const char	*html_name = argv[1];
	char		*step;
	int 		stepInt = 0;
	char		*back, *jumpback, *skip;
	int		error = 0;
	char 		myStep[3];
	
	char 		*id;
	char		str[50];
	char		label[50];
	int 		i;
	char 		*ac_main;

	int newStep = 0;
	char *resetLabel = NULL;
	
	if (argc < 2)
		return ;

	/* get POST variables */
	avp = http_args(r);
    
	/* handle global variables such as language */
	handle_global_settings(avp, s->nvram);

	/* check for correct values */
	step = get_from_list(avp, "wiz_3phone.step");
	back = get_from_list(avp, "back");
	skip = get_from_list(avp, "skip");
	jumpback = get_from_list(avp, "jumpback");



	if (step == NULL)
	{
		/* set temp variables */
		init_temp_vars(avp, s->nvram);
		add_to_list(avp, "wiz_3phone.step", "0");
		step = get_from_list(avp, "wiz_3phone.step");
	}	
	
	stepInt = atoi(step);
	
	if (skip)
	{
		switch(stepInt)
		{
			case 2:
			case 3:
			case 4:
				newStep = 5;
				resetLabel = REWRITE_LABEL_ANALOG_1;	
				/* overwrite hidden vars with nvram */
				id = get_from_list(avp, "wiz_3phone.id.Analog1");
				if (id && *id)
				{
					/* saving the name of the first anlaog phone */
					snprintf(str, sizeof(str), "phone_devices.name.%s", id);
					add_to_list(avp, "wiz_3phone.name.Analog1",get_from_nvram(s->nvram, str));
					/* saving the icon of the first anlaog phone */
					snprintf(str, sizeof(str), "phone_devices.icon.%s", id);
					add_to_list(avp, "wiz_3phone.icon.Analog1",get_from_nvram(s->nvram, str));
				}
								
				break;
			case 5:
			case 6:
			case 7:
				newStep = 8;
				resetLabel = REWRITE_LABEL_ANALOG_2;	
				/* overwrite hidden vars with nvram */
				id = get_from_list(avp, "wiz_3phone.id.Analog2");
				if (id && *id)
				{
					/* saving the name of the first anlaog phone */
					snprintf(str, sizeof(str), "phone_devices.name.%s", id);
					add_to_list(avp, "wiz_3phone.name.Analog2",get_from_nvram(s->nvram, str));
					/* saving the icon of the first anlaog phone */
					snprintf(str, sizeof(str), "phone_devices.icon.%s", id);
					add_to_list(avp, "wiz_3phone.icon.Analog2",get_from_nvram(s->nvram, str));
				}
				break;
			case 8:
			case 9:
			case 10:
				newStep = 11;
				resetLabel = REWRITE_LABEL_ISDN;	
				id = get_from_list(avp, "wiz_3phone.id.ISDN");
				if (id && *id)
				{
					/* saving the name of the first anlaog phone */
					snprintf(str, sizeof(str), "phone_devices.name.%s", id);
					add_to_list(avp, "wiz_3phone.name.ISDN",get_from_nvram(s->nvram, str));
					/* saving the icon of the first anlaog phone */
					snprintf(str, sizeof(str), "phone_devices.icon.%s", id);
					add_to_list(avp, "wiz_3phone.icon.ISDN",get_from_nvram(s->nvram, str));
				}
				break;
			case 11:
			case 12:
				newStep = 13;
				resetLabel = REWRITE_LABEL_ACCOUNTS;
				
				ac_main = get_from_list(avp, "wiz_3phone.ac.main");
				if (!strcmp("isdn", ac_main))
				{
					for (i = 1; i <= 5; i++)
					{
						snprintf(str, sizeof(str), "wiz_3phone.ac.isdn.id.%i", i);
						id = get_from_list(avp, str);
						if (id && *id)
						{
							/* saving the name of the first anlaog phone */
							snprintf(str, sizeof(str), "wiz_3phone.ac.isdn.number.%i", i);
							snprintf(label, sizeof(label), "phone_accounts.number.%s", id);
							add_to_list(avp, str, get_from_nvram(s->nvram, label));
						}				
					}
				}
				else if (!strcmp("analog", ac_main))
				{
					for (i = 1; i <= 5; i++)
					{
						id = get_from_list(avp, "wiz_3phone.ac.analog.id");
						if (id && *id)
						{
							/* saving the name of the first anlaog phone */
							snprintf(label, sizeof(label), "phone_accounts.number.%s", id);
							add_to_list(avp, "wiz_3phone.ac.analog.number", get_from_nvram(s->nvram, label));
						}				
					}
				} 

				
				break;
			case 13:
			case 14:
				newStep = 15;
				resetLabel = REWRITE_LABEL_SIP;
				/* must be set now, because writing to nvram */
				changeRewrite(avp, resetLabel, REWRITE_FALSE);


				id = get_from_list(avp, "wiz_3phone.ac.sip.id");
				if (id && *id)
				{
					/* saving the number of the account */
					snprintf(str, sizeof(str), "phone_accounts.number.%s", id);
					add_to_list(avp, "wiz_3phone.ac.sip.number", get_from_nvram(s->nvram, str));				
					/* saving the server of the account */
					snprintf(str, sizeof(str), "phone_accounts.server.%s", id);
					add_to_list(avp, "wiz_3phone.ac.sip.server", get_from_nvram(s->nvram, str));
					/* username of the account */
					snprintf(str, sizeof(str), "phone_accounts.username.%s", id);
					add_to_list(avp, "wiz_3phone.ac.sip.username", get_from_nvram(s->nvram, str));
					/* password of the account */
					snprintf(str, sizeof(str), "phone_accounts.password.%s", id);
					add_to_list(avp, "wiz_3phone.ac.sip.passwd", get_from_nvram(s->nvram, str));
				}
				
				store_temp_vars(avp, s->nvram);
				
				break;
			
			default:
				/* cannot be reached normally */
				html_name = "wiz/wiz_0start.html";
				send_html_page(avp, s, r, html_name);			
		}
		
		
		/* the skipped section should not be saved later */
		changeRewrite(avp, resetLabel, REWRITE_FALSE);
		/* set the new step */
		snprintf(myStep, sizeof(myStep), "%d", newStep);
		add_to_list(avp, "wiz_3phone.step", myStep);
		/* create next html page */
		snprintf(tmpl_name, sizeof(tmpl_name), html_name, newStep);
		send_html_page(avp, s, r, tmpl_name);
		return;
	}
	if (jumpback)
	{
		/* set temp variables */
		init_temp_vars(avp, s->nvram);
		add_to_list(avp, "wiz_3phone.step", "15");
		/* create next html page */
		snprintf(tmpl_name, sizeof(tmpl_name), html_name, 15);
		send_html_page(avp, s, r, tmpl_name);
		return;
	}
	if (back) {
		/* go one page back, do not check for errors */
		stepInt--;
		if (stepInt <= 0)
		{
			/* cannot be reached normally */
			html_name = "wiz/wiz_0start.html";
			send_html_page(avp, s, r, html_name);
		}
		else
		{
			switch(stepInt)
			{
				case 1:
				case 2:
					break;
				case 3:
					shiftIcon(avp, CURRENT_ANALOG_1, HIDDENVAR_TO_ICONVAR);
					break;
				case 4:
					resetLabel = REWRITE_LABEL_ANALOG_1;	
					break;
				case 5:
					break;
				case 6:
					shiftIcon(avp, CURRENT_ANALOG_2, HIDDENVAR_TO_ICONVAR);
					break;
				case 7:
					resetLabel = REWRITE_LABEL_ANALOG_2;	
					break;
				case 8:
					break;
				case 9:
					shiftIcon(avp, CURRENT_ISDN, HIDDENVAR_TO_ICONVAR);
					break;
				case 10:
					resetLabel = REWRITE_LABEL_ISDN;	
					break;
				case 11:
					break;
				case 12:
					resetLabel = REWRITE_LABEL_ACCOUNTS;	
					break;
				case 13:
					break;
				case 14:
					resetLabel = REWRITE_LABEL_SIP;	
					break;
				
				default:
					/* cannot be reached normally */
					html_name = "wiz/wiz_0start.html";
					send_html_page(avp, s, r, html_name);			
			}
			
			
			/* the skipped section should not be saved later */
			changeRewrite(avp, resetLabel, REWRITE_TRUE);
			
			snprintf(myStep, sizeof(myStep), "%d", stepInt);
			add_to_list(avp, "wiz_3phone.step", myStep);
			
			/* create next html page */
			snprintf(tmpl_name, sizeof(tmpl_name), html_name, stepInt);
			/* use next html page */
			send_html_page(avp, s, r, tmpl_name);
		}
	
		return;
	}

	switch(stepInt) {
		case 0:
		case 1:
		case 11:
		case 13:
			/* nothing to check here */
			break;
		case 2:
			shiftIcon(avp, CURRENT_ANALOG_1, HIDDENVAR_TO_ICONVAR);
			break;
		case 3:
			/* check name for analog phone 1 */     
			error += checkDeviceName(avp, s->nvram, CURRENT_ANALOG_1);
			shiftIcon(avp, CURRENT_ANALOG_1, ICONVAR_TO_HIDDENVAR);
			break;
		case 4:
			/* test function of phones */
			/* if test, show same site again */
			checkRing(avp, s->nvram, CURRENT_ANALOG_1, &stepInt);
			changeRewrite(avp, REWRITE_LABEL_ANALOG_1, REWRITE_TRUE);
			break;
		case 5:
			shiftIcon(avp, CURRENT_ANALOG_2, HIDDENVAR_TO_ICONVAR);
			break;
		case 6:
			/* check name for analog phone 2 */
			error += checkDeviceName(avp, s->nvram, CURRENT_ANALOG_2);
			shiftIcon(avp, CURRENT_ANALOG_2, ICONVAR_TO_HIDDENVAR);
			break;
		case 7:
			/* test function of phones */
			/* if test, show same site again */
			checkRing(avp, s->nvram, CURRENT_ANALOG_2, &stepInt);
			changeRewrite(avp, REWRITE_LABEL_ANALOG_2, REWRITE_TRUE);
			break;
		case 8:
			shiftIcon(avp, CURRENT_ISDN, HIDDENVAR_TO_ICONVAR);
			break;
		case 9:
			/* check name for isdn phone*/
			error += checkDeviceName(avp, s->nvram, CURRENT_ISDN);
			shiftIcon(avp, CURRENT_ISDN, ICONVAR_TO_HIDDENVAR);
			break;
		case 10:
			/* test function of phones */
			/* if test, show same site again */
			checkRing(avp, s->nvram, CURRENT_ISDN, &stepInt);
			changeRewrite(avp, REWRITE_LABEL_ISDN, REWRITE_TRUE);
			break;
		case 12:
			/* check phone numbers*/
			error += checkAccounts(avp, s->nvram);
			changeRewrite(avp, REWRITE_LABEL_ACCOUNTS, REWRITE_TRUE);
			break;
		case 14:
			/* check voip settings (server, callnumber, username, password) */
			error += checkServer(avp);
			error += checkCallnumber(avp, s->nvram);
			error += checkUsername(avp);
			error += checkPassword(avp);

			changeRewrite(avp, REWRITE_LABEL_SIP, REWRITE_TRUE);
			
			/* save values to nvram */
			if (!error) store_temp_vars(avp, s->nvram);
			break;
		case 15:
			/* summary */
			html_name = "wiz/wiz_4sys.html";
			add_to_list(avp, "wiz_4sys.step", "1");
			break;
	}

	if (error) {
		add_to_list(avp, "error", "1");
		
	} else {
		/* no error, load next page */
		stepInt++;
		snprintf(myStep, sizeof(myStep), "%d", stepInt);
		add_to_list(avp, "wiz_3phone.step", myStep);
	}
	
	/* create next html page */
	snprintf(tmpl_name, sizeof(tmpl_name), html_name, stepInt);
	send_html_page(avp, s, r, tmpl_name);

}

void *htinit (const char *doc_path, NVCL *nv) 
{
	STATE *s = ZMALLOC(STATE);

	s->nvram  	= nv;
	s->htpath 	= doc_path;

	return s ;
}


void htcleanup (void *desc) 
{
	STATE* s = desc;

	free(s);
}




/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

