/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Henry Margies <h.margies@maxina.de> - Maxina GmbH, Neu-       *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <time.h>

#include "hlst.h"
#include "util.h"
#include "http-plugin.h"
#include "parser.h"
#include "nvalloc.h"
#include "plugin-util.h"


static void init_temp_vars(HLST* avp, NVCL* nvram)
{
	/* initialize variablen from nvram*/

	add_to_list(avp, "wiz_4sys.time", get_from_nvram(nvram, "sys_time.time"));
	add_to_list(avp, "wiz_4sys.passwd", get_from_nvram(nvram, "sys_admin.passwd"));
}

static void store_temp_vars(HLST* avp, NVCL* nvram)
{
	char *time = get_from_list(avp, "wiz_4sys.time");
	/* write variables to nvram*/

	if(!time)
		return;

	add_to_nvram(nvram, "sys_time.time", time);
	add_to_nvram(nvram, "sys_admin.passwd", get_from_list(avp, "wiz_4sys.passwd"));
	
        if(strcmp(time ,"automatic"))
	{
		
		struct tm myTime;
		time_t timestamp;
		char timeBuffer[128];
		
		memset(&myTime, 0, sizeof(struct tm));

		myTime.tm_year 	= getVarAsLong(avp, "wiz_4sys.year")-1900;
		myTime.tm_mon 	= getVarAsLong(avp, "wiz_4sys.month");
		myTime.tm_mday 	= getVarAsLong(avp, "wiz_4sys.day");
		myTime.tm_hour 	= getVarAsLong(avp, "wiz_4sys.hour");
		myTime.tm_min 	= getVarAsLong(avp, "wiz_4sys.minute");
		myTime.tm_sec 	= getVarAsLong(avp, "wiz_4sys.second");

		timestamp = mktime(&myTime);
	
		if(timestamp == -1)
			return;
		
		sprintf(timeBuffer,"%d",timestamp);
		add_to_nvram(nvram, "sys_time.timestamp", timeBuffer);
		
		sprintf(timeBuffer,"%02d%02d%02d%02d%04d.%02d",
				myTime.tm_mon,
				myTime.tm_mday,
				myTime.tm_hour,
				myTime.tm_min,
				myTime.tm_year+1900,
				myTime.tm_sec);
		add_to_nvram(nvram, "sys_time.date", timeBuffer);
		
	}

	return;
}

static int checkTime(HLST *avp)
{
	char *val = get_from_list(avp, "wiz_4sys.time");

	if(!val || !*val)
		return 1;

	if (!strcmp(val, "automatic"))
		return 0;
	
	val = get_from_list(avp, "wiz_4sys.hour");
	if(!val || !*val){
		add_to_list(avp, "wiz_4sys.error.nojs", "1");
		return 1;
	}

	return 0;
}

static int checkPassword(HLST *avp){
	int errors = 0;
	char *passwd = get_from_list(avp, "wiz_4sys.passwd");
	
	if(!passwd || strlen(passwd)==0 || strchr(passwd, ' ')){
		add_to_list(avp, "wiz_4sys.error.passwd", "1");
		return 1;
	}

	return 0;
}

void htrun (void *desc, RPLY_REC *r, int argc, const char **argv) 
{
	STATE* 		s = desc;
	HLST*		avp;

	char		tmpl_name[1024];
	const char	*html_name = argv[1];
	char		*step;
	char		*back, *jumpback;
	int		error = 0;


	if (argc < 2)
		return ;

	/* get POST variables */
	avp = http_args(r);
    
	/* handle global variables such as language */
	handle_global_settings(avp, s->nvram);

	/* check for correct values */
	step = get_from_list(avp, "wiz_4sys.step");
	back = get_from_list(avp, "back");
	jumpback = get_from_list(avp, "jumpback");

	if (jumpback)
	{
		/* set temp variables */
		init_temp_vars(avp, s->nvram);
		
	 	/* check if the dsl account works */
	 	//check_dslAvailable(avp);
		
		add_to_list(avp, "wiz_4sys.step", "4");
		
		/* create next html page */
		snprintf(tmpl_name, sizeof(tmpl_name), html_name, 4);
		send_html_page(avp, s, r, tmpl_name);
		return;
	}	
	if (step == NULL)
	{
		/* set temp variables */
		init_temp_vars(avp, s->nvram);

		add_to_list(avp, "wiz_4sys.step", "0");
		step = get_from_list(avp, "wiz_4sys.step");
	}	
	
	if (back) {
		/* go one page back, do not check for errors */
		(*step)--;
		if (*step <= '0')
		{
			/* cannot be reached normally */
			html_name = "wiz/wiz_3phone.html";
			send_html_page(avp, s, r, html_name);
		}
		else
		{
			add_to_list(avp, "wiz_4sys.step", step);
		}

		/* create previous html page */
		snprintf(tmpl_name, sizeof(tmpl_name), html_name, atoi(step));
		/* use html page */
		send_html_page(avp, s, r, tmpl_name);

		return;
	}


	switch(*step) {
		case '0':
		case '1':break;
		case '2':
			/* check time and date */
			error += checkTime(avp);
			break;
		case '3':
			/* check password */
			error += checkPassword(avp);
			
			if(!error)
			{
				store_temp_vars(avp, s->nvram);
			}
			break;
		case '4':
			/* summary */
			html_name = "wiz/wiz_5end.html";
			add_to_list(avp, "wiz_5end.step", "1");
			break;
	}

	if (error) {
		add_to_list(avp, "error", "1");
		
	} else {
		/* no error, load next page */
		(*step)++;
		add_to_list(avp, "wiz_4sys.step", step);
	}

	
	/* create next html page */
	snprintf(tmpl_name, sizeof(tmpl_name), html_name, atoi(step));
	send_html_page(avp, s, r, tmpl_name);

}

void *htinit (const char *doc_path, NVCL *nv) 
{
	STATE *s = ZMALLOC(STATE);

	s->nvram  	= nv;
	s->htpath 	= doc_path;

	return s ;
}


void htcleanup (void *desc) 
{
	STATE* s = desc;

	free(s);
}



/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */

