#
# nvramd - active data base server and flash memory cache
#
# Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA
#
# Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-Isenburg, Germany
#
# $Id: acinclude.m4 2690 2006-06-27 06:23:33Z jordan $
#

m4_include([libnvtools/acinclude.m4])
m4_include([libnvram/acinclude.m4])


AC_DEFUN([AC_ENABLE_FLASH],[
  AC_ARG_ENABLE([flash], [AC_HELP_STRING([--enable-flash=TYPE],
    [include flash io driver by the file alternatives/flash-TYPE.h])],
    [case "$enableval" in
      no) enable_flash=file ;;
     yes) enable_flash=horst ;;
       *) enable_flash=$enableval
     esac],
    [enable_flash=file])
])


AC_DEFUN([AC_CHECK_FLASH],[
  AC_MSG_CHECKING([which flash driver type to use])
  if test -f alternatives/flash-$enable_flash.c; then
    AC_MSG_RESULT([$enable_flash])
  else
    AC_MSG_RESULT([$enable_flash is missing])
    enable_flash=missing
  fi
  flash_c_in=alternatives/flash-$enable_flash.c
])


AC_DEFUN([AC_MSG_FLASH],[
  case "$enable_flash" in
  missing) AC_MSG_NOTICE([Assuming flash memory - but there is no driver available]) ;;
    file) AC_MSG_NOTICE([Using file emulation instead of a flash driver]) ;;
       *) AC_MSG_NOTICE([Flash memory configured - using driver: $enable_flash])
  esac
])


AC_DEFUN([AC_ENABLE_PRELOAD_DATA],[
  AC_ARG_ENABLE([preload-data], [AC_HELP_STRING([--enable-preload-data=TYPE],
    [include preload data by the file alternatives/preload-TYPE.h])],
    [case "$enableval" in
      no) enable_preload=empty ;;
     yes) enable_preload=default ;;
       *) enable_preload=$enableval
     esac],
    [if test -f alternatives/preload-horst.h; then
       enable_preload=horst
     else
       enable_preload=default
     fi])
])

AC_DEFUN([AC_CHECK_PRELOAD_DATA],[
  AC_MSG_CHECKING([which preload data type to compile in])
  if test -f alternatives/preload-$enable_preload.h; then
    AC_MSG_RESULT([$enable_preload])
  else
    AC_MSG_RESULT([$enable_preload is missing])
    enable_preload=missing
  fi
  preload_data_h_in=alternatives/preload-$enable_preload.h
])

AC_DEFUN([AC_MSG_PRELOAD_DATA],[
  case "$enable_preload" in
  missing) AC_MSG_NOTICE([Please use --enable-preload-data=TYPE - compilation will fail]) ;;
  default) AC_MSG_NOTICE([Using default preload data - consider to provide your own]);;
        *) AC_MSG_NOTICE([Using preload data type: $enable_preload])
  esac
])
