/*
 * nvramd - active data base server and flash memory cache
 *
 * Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA
 *
 * Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-Isenburg, Germany
 *
 * $Id: preload-default.h 3600 2006-11-14 07:46:22Z jordan $
 */

 /*****************************
  *  nvram preload example
  *****************************/

#ifndef __NVRAMD_PRELOAD_DATA_H
#define __NVRAMD_PRELOAD_DATA_H

/* NOTE:
 *
 * the following characters MUST be escaped with a backslash
 * when used as first character of the value string:
 *
 *  ! ? \\ (see ACTION_TYPE and ACTION_CONSTANTS in list.h)
 *
 *  needs to be coded as:
 *
 *  \\! \\? \\\\
 */

static const char flash_defaults [] =

#define VBASE(x)  "firewall." x /* use macro to prevent typos */
#define VSCRIPT   VBASE ("setup")

     /* execute "fwconfig.sh" when these entries get changed */
     _wrlink (VBASE ("block_ping"),   VSCRIPT, AV_TRUE)
     _wrlink (VBASE ("block_telnet"), VSCRIPT, AV_TRUE)
     _wrlink (VBASE ("block_ftp"),    VSCRIPT, AV_TRUE)
     _wrlink (VBASE ("block_dns"),    VSCRIPT, AV_TRUE)
     _wrlink (VBASE ("block_ike"),    VSCRIPT, AV_TRUE)
     _wrlink (VBASE ("block_rip"),    VSCRIPT, AV_TRUE)
     _wrlink (VBASE ("block_dhcp"),   VSCRIPT, AV_TRUE)
     _wrlink (VBASE ("block_icmp"),   VSCRIPT, AV_TRUE)

     _vtrig  (VSCRIPT, "fwconfig.sh", AV_FALSE)

#undef VBASE
#undef VSCRIPT

#define VBASE(x)  "lan." x /* use macro to prevent typos */
#define VSCRIPT   VBASE ("setup")

     /* execute "ipconfig.sh" when these entries get changed */
    _wrlink (VBASE ("address"), VSCRIPT, "192.168.0.1")
    _wrlink (VBASE ("netmask"), VSCRIPT, "255.255.255.0")

    _vtrig  (VSCRIPT, "ipconfig.sh", AV_FALSE)

#undef VBASE
#undef VSCRIPT

#define VBASE(x)  "map." x 
#define VSCRIPT   VBASE ("setup")

    /* the next three entries are simple assignment-value pairs */
    _avpair (VBASE ("total"),      "7")
    _avpair (VBASE ("totalFixed"), "7")
    _avpair (VBASE ("list"), "1 2 3 4 5 6 7")

    /* too lazy to repeat tedious data - using a macro for that */
#   define FIXRULESET(id,name,proto,port) \
        _avpair (VBASE ("id."        str (id)), str    (id)) \
        _avpair (VBASE ("protocol."  str (id)), str (proto)) \
        _avpair (VBASE ("startport." str (id)), str  (port)) \
        _avpair (VBASE ("endport."   str (id)), str  (port)) \
        _avpair (VBASE ("portmap."   str (id)), str  (port)) \
        _avpair (VBASE ("type."      str (id)), "fix")

    /* these entries are simple assignment-value pairs */
    FIXRULESET (1,    FTP, tcp,  21)
    FIXRULESET (2,   HTTP, tcp,  80)
    FIXRULESET (3,  HTTPS, tcp, 443)
    FIXRULESET (4,    SSH, tcp,  22)
    FIXRULESET (5,   SMTP, tcp,  25)
    FIXRULESET (6,   POP3, tcp, 110)
    FIXRULESET (7, Telnet, tcp,  23)
#   undef FIXRULESET

    /* activate "mapconfig.sh" whenever an ip addriess gets assigned as in
       "nvram map.ip.7 192.168.0.22" for telnet redirect */
    _wrltmpl(VBASE ("ip."), VSCRIPT)

    _vtrig  (VSCRIPT, "mapconfig.sh", AV_FALSE)

#undef VBASE
#undef VSCRIPT

     "\0" ;

/* have a random mac address created from the private range */
#define MAC_TEMPLATE mac_template

static const char MAC_TEMPLATE [] =

    _avpair ("private.macs", "ac:de:48:23:42:07")

     "\0" ;

#endif /* __NVRAMD_PRELOAD_DATA_H */
