/*
 * nvramd - active data base server and flash memory cache
 *
 * Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA
 *
 * Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-Isenburg, Germany
 *
 * $Id: comhandler.h 3128 2006-09-01 07:17:15Z jordan $
 */

#ifndef __NVRAMD_COMMHANDLER_H
#define __NVRAMD_COMMHANDLER_H

#include "util.h"
#include "ctrl.h"
#include "list.h"
#include "http-server.h"

/* communication through the control socket */
typedef struct _comm
#ifdef __COMHANDLER_PRIVATE
{
    time_t    started ;
    unsigned  timeout ;
    gid_t       admin ; /* group with admin rights */
    CTRL        *ctrl ;
    LIST        *list ;
}
#endif
COMM;


extern COMM *comm_init       (LIST*,gid_t);
extern COMM *comm_update     (COMM*,CTRL*,int);

/* unix domain socket commands */
extern void  comm_close      (COMM*);
extern void  comm_store      (void*,int,const char**);
extern void  comm_load       (void*,int,const char**);
extern void  comm_slots      (void*,int,const char**);
extern void  comm_set        (void*,int,const char**);
extern void  comm_get        (void*,int,const char**);
extern void  comm_del        (void*,int,const char**);
extern void  comm_symlink    (void*,int,const char**);
extern void  comm_rm_symlink (void*,int,const char**);
extern void  comm_trigger    (void*,int,const char**);
extern void  comm_rm_trigger (void*,int,const char**);
extern void  comm_access     (void*,int,const char**);
extern void  comm_rm_access  (void*,int,const char**);

#ifndef DISCARD_HTTP         /* http commands */
extern void store_cmd        (void*,RPLY_REC*);
extern void  load_cmd        (void*,RPLY_REC*);
extern void  list_cmd        (void*,RPLY_REC*);
extern void   get_cmd        (void*,RPLY_REC*);
extern void   set_cmd        (void*,RPLY_REC*);
extern void   del_cmd        (void*,RPLY_REC*);
extern void slots_cmd        (void*,RPLY_REC*);
extern void error_cmd        (void*,RPLY_REC*);
#endif /* DISCARD_HTTP */

#endif /* __NVRAMD_COMMHANDLER_H */
