#! /bin/sh
#
# nvramd - active data base server and flash memory cache
#
# Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA
#
# Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-Isenburg, Germany
#
# $Id: dsa-example.sh 2968 2006-07-29 19:58:03Z jordan $
#

nvsign=nvram # eg. on system 1
nvrify=nvram # eg. on system 2

# ----------------------------------------------------------------------
# Verify system 1 (nvsign) on system 2 (nvrify)
# ----------------------------------------------------------------------

# get a challenge
challenge=`$nvrify crypt.dsa0.vfy`

# sign that challenge
signhandle=`$nvsign crypt.dsa0.sign`
$nvsign $signhandle $challenge
signature=`$nvsign $signhandle`

# have the signature verified
$nvrify $challenge $signature

# verify the signature: gets "ok" or "fail" (or nothing upon error)
echo Signature verification `$nvrify $challenge`

# ----------------------------------------------------------------------
# sign arbitrary text
# ----------------------------------------------------------------------

# get a function handle
keyhandle=`nvram crypt.dsa1.key`

# load the private key, use ./dsa1key for convenience or have a look at
# the macro DSA1PRIVKEY in the header file crypt/dsa1key.h
nvram $keyhandle `./dsa1key --private-key`

# get the signature for the text "text1" 
signhandle=`nvram $keyhandle`
nvram $signhandle "text1"
signature=`nvram $signhandle`
echo Signature for \"text1\" is $signature

# get the signature for the text "text2" 
signhandle=`nvram $keyhandle`
nvram $signhandle "text2"
signature=`nvram $signhandle`
echo Signature for \"text2\" is $signature

# ----------------------------------------------------------------------
# End
# ----------------------------------------------------------------------
