/*
 * nvramd - active data base server and flash memory cache
 *
 * Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA
 *
 * Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-Isenburg, Germany
 *
 * $Id: dsafunctions.h 3128 2006-09-01 07:17:15Z jordan $
 */

#ifndef __NVRAMD_DSA_FUNCTIONS_H
#define __NVRAMD_DSA_FUNCTIONS_H

#ifdef HAVE_OPENSSL
#include <openssl/dsa.h>
#endif

#include "list.h"
#include "settings.h"

#ifdef __NVRAMD_DSA_FUNCTIONS_PRIVATE
typedef
struct _key {
    unsigned char inst ;
    unsigned char type ;
    /* key material */
    union {
        /* type == 1: encoded private key */
        struct {
            const char *data ;
            unsigned     len ;
        } b64 ;
        /*  type == 2: public key */
        DSA          *public ;
    } u ;
} KEY ;
#endif

typedef
struct _dsaop
#ifdef __NVRAMD_DSA_FUNCTIONS_PRIVATE
{
    unsigned        dim ;
    struct _sigdata *sd ;
    KEY             key ;
    LIST             *l ;
}
#endif
DSAOP ;

extern void        dsafree  (void*);
extern const char *dsasign  (LIST*,void*,const char*,int,const char*);
extern const char *dsavfy   (LIST*,void*,const char*,int,const char*);

extern void mkdsafunctions (LIST*) ;

#endif /* __NVRAMD_DSA_FUNCTIONS_H */
