/*
 * nvramd - active data base server and flash memory cache
 *
 * Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA
 *
 * Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-Isenburg, Germany
 *
 * $Id: functions.c 3175 2006-09-23 19:17:11Z jordan $
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef STDC_HEADERS
# include <stdlib.h>
# include <stddef.h>
#else
# ifdef HAVE_STDLIB_H
#  include <stdlib.h>
# endif
#endif

#include "functions.h"
#include "tvcalc.h"
#include "varstr.h"
#include "nvalloc.h"

/* ----------------------------------------------------------------------- *
 * Private: constants, private data, macros
 * ----------------------------------------------------------------------- */

/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

const char *timedate (LIST         *l,
                      void      *desc,
                      const char *var,
                      int       flags,
                      const char *val) {
    TIMEDATE *d = (TIMEDATE *) desc ;
    struct timeval now ;
        
    if (val) {
        unsigned u ;
        int sign ;

        if (val [0] == '-') {
            sign = -1 ;
            val ++ ;
        }
        else
            sign = 1 ;

        if (parse_unsigned (&u, val, 0) == 0)
            return (const char *) -1 ;

        tval_set_ms (&now, sign * u);

        return tval_eq_tv (&now, &d->offset) ? 0 : "" ;
    }

    tval_nxt_tv (&now, &d->offset);
    return tval_print (&now);
}

void mkfunctions (LIST *l) {
    list_fregister (l, "timedate", ZMALLOC (TIMEDATE), timedate, 0,0,0);
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
