/*
 * nvramd - active data base server and flash memory cache
 *
 * Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA
 *
 * Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-Isenburg, Germany
 *
 * $Id: dsa0key.c 3447 2006-10-27 09:24:45Z henry $
 *
 * compile as:
 *  cc -DHAVE_CONFIG_H -I. helper/dsa0key.c -o dsa0key -lcrypt -lssl -lcrypto
 */

/*
 * Don't use any config.h or auto-generated config files, because they are
 * most probably just for the target environment.
 */
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/time.h>
#include <stdlib.h>
#include <stddef.h>
#include <stdint.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <libgen.h>
#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <limits.h>
#include <stdio.h>

#define SYSLOG_NAMES
#include <syslog.h>


#include <openssl/dsa.h>
#include <openssl/bn.h>
#include <openssl/err.h>
#include <openssl/x509.h>
#include <openssl/blowfish.h>

/* ----------------------------------------------------------------------- *
 * not using -lnvram
 * ----------------------------------------------------------------------- */

#undef HAVE_CONFIG_H
#include "../libnvram/varstr.c"
#include "../libnvram/util.c"
#include "../libnvram/logger.c"
#include "../libnvram/base64.c"
#include "../libnvram/nvalloc.c"
#include "../libnvram/tvcalc.c"
#include "../libnvram/buffer.c"

/* ----------------------------------------------------------------------- *
 * need precompiled dsa params
 * ----------------------------------------------------------------------- */

#include "../helper/dsakey-functions.h"

#define __DSA_PRIVATE_KEY
#define __DSA_PUBLIC_KEY
#include "../crypt/dsaparam.h"
#include "../crypt/dsa0key.h"

#ifndef get_dsa_priv_key0
#error  get_dsa_priv_key0 not available - maybe dsa0key.h is wrong
#endif

#define get_dsa_priv_key get_dsa_priv_key0

/* ----------------------------------------------------------------------- *
 * Variables and definitions
 * ----------------------------------------------------------------------- */

const char _key_id [] = str (get_dsa_priv_key);
const char *key_id    = _key_id + 5 ;

/* ----------------------------------------------------------------------- *
 * Helpers
 * ----------------------------------------------------------------------- */

void usage (void) {
    fprintf (stderr, "Usage: %s [instance-num]\n", progname);
    exit (2) ;
}

/* ----------------------------------------------------------------------- *
 * Main
 * ----------------------------------------------------------------------- */

int main (int argc, const char **argv) {
    DSA *dsa ;
    char *inst = "" ;

    if ((progname = strrchr (argv [0], '/')) == 0)
        progname = (char*) argv [0] ;
    else
        ++ progname ;

    switch (argc) {
        IOBUF b ;
        unsigned u;
    case 2:
        if (parse_unsigned (&u, argv [1], 0) == 0) {
            fprintf (stderr, "%s: <instance-num> must be a number between "
                     "0 and %u\n", progname, 0x7fffffff);
            exit (2);
        }
        XZERO (b);
        data_printf (&b, "%u", u);
        inst = b.data ;
    case 1:
        break ;
    default:
        usage () ;
    }

    if ((dsa = get_dsa_priv_key (0)) == 0) {
        fprintf
            (stderr, "%s: Error retrieving the DSA private key\n", progname);
        exit (2);
    }

    print_cpry ();
    print_dsa (dsa, inst, key_id, str (get_dsa));

    exit (0);
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
