/*
 * nvramd - active data base server and flash memory cache
 *
 * Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA
 *
 * Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-Isenburg, Germany
 *
 * $Id: dsakey-functions.h 2968 2006-07-29 19:58:03Z jordan $
 */

/* ----------------------------------------------------------------------- *
 * Variables and definitions
 * ----------------------------------------------------------------------- */

#define __str(x)              # x
#define str(x)                __str (x)

/* ----------------------------------------------------------------------- *
 * Helpers
 * ----------------------------------------------------------------------- */

static
unsigned char *encrypt (const unsigned char   *p,
                        unsigned               n,
                        const unsigned char *key,
                        unsigned            klen,
                        const unsigned char  *iv) {
    unsigned char i [8], *q = (unsigned char*) zmalloc (n);
    BF_KEY b;
    BF_set_key (&b, klen, key);
    BF_cbc_encrypt (p, q, n, &b, memcpy (i, iv, 8), BF_ENCRYPT);
    return q;
}


static
unsigned derpub (const DSA             *d,
                 int                    x,
                 const char           *id,
                 const unsigned char *key,
                 unsigned            klen,
                 const unsigned char  *iv) {
    
    unsigned n = i2d_DSA_PUBKEY ((DSA*)d, 0) ;
    
    if (n > 1) {
        int logn1, m, N ;
        unsigned char *q, *p ;
        char r [61] ;

        /* get prefix length */	
        for (logn1 = 0; (n >> (7 * logn1)) != 0; logn1 ++)
            ;

        N = (n + logn1 + 7) & ~7 ;
        p = (unsigned char *) zmalloc (N);
        
        /* store length prefix */
        m = 0; q = p;
        while (m < logn1 - 1) * q ++ = 0x80 | ((n >> (7 * m ++)) & 0x7f);
        * q ++ = ((n >> (7 * m ++ )) & 0x7f) ;

        /* append key */
        i2d_DSA_PUBKEY ((DSA*)d, &q);

        /* encrypt data */
        q = encrypt (p, N, key, klen, iv);
        free ((char*)p);

        p = (unsigned char*)b64alloc ((char*)q, N);
        printf ("#ifdef __DSA_IMPORT_KEY\n");
        printf ("#define DSA_%u%s ( /* %d bytes payload */\\\n", x, id, n);
        r [sizeof (r) - 1] = '\0' ;
        m = 0 ;
        for (;;) {
            strncpy (r, (char*)p + m, sizeof (r) - 1);
            printf ("\t\"%s\"", strncpy (r, (char*)p + m, sizeof (r) - 1));
            if (p [m += strlen (r)] == 0) break ;
            printf (" \\\n");
        }
        printf (")\n");
        printf ("#endif /* __DSA_IMPORT_KEY */\n\n");
        free ((char*)p);

        printf ("#define dsa_%u%s { /* %d bytes payload */\\\n", x, id, n);
        printf ("\t0x%02x, ", q [m = 0]);
        while (++ m < N - 1) {
            printf ("0x%02x, ", q [m]);
            if ((m % 12) == 11)
                fputs ("\\\n\t", stdout);
        }
        printf ("0x%02x}\n\n", q [m]);
        free ((char*)q);
    }

    return n;
}


static
unsigned derprv (const DSA             *d,
                 int                    x,
                 const char           *id,
                 const unsigned char *key,
                 unsigned            klen,
                 const unsigned char  *iv) {
    
    unsigned n = i2d_DSAPrivateKey ((DSA*)d, 0) ;
    
    if (n > 1) {
        int logn1, m, N ;
        unsigned char *q, *p;
        char r [61] ;

        /* get prefix length */	
        for (logn1 = 0; (n >> (7 * logn1)) != 0; logn1 ++)
            ;

        N = (n + logn1 + 7) & ~7 ;
        p = (unsigned char *) zmalloc (N);

        /* store length prefix */
        m = 0; q = p;
        while (m < logn1 - 1) * q ++ = 0x80 | ((n >> (7 * m ++)) & 0x7f);
        * q ++ = ((n >> (7 * m ++ )) & 0x7f) ;

        /* append key */
        i2d_DSAPrivateKey ((DSA*)d, &q);

        /* encrypt data */
        q = encrypt (p, N, key, klen, iv);
        free ((char*)p);

        p = (unsigned char*)b64alloc ((char*)q, N);
        printf ("#ifdef __DSA_IMPORT_KEY\n");
        printf ("#define DSA_%u%s ( /* %d bytes payload */\\\n", x, id, n);
        r [sizeof (r) - 1] = '\0' ;
        m = 0 ;
        for (;;) {
            strncpy (r, (char*)p + m, sizeof (r) - 1);
            printf ("\t\"%s\"", strncpy (r, (char*)p + m, sizeof (r) - 1));
            if (p [m += strlen (r)] == 0) break ;
            printf (" \\\n");
        }
        printf (")\n");
        printf ("#endif /* __DSA_IMPORT_KEY */\n\n");
        free ((char*)p);

        printf ("#define dsa_%u%s { /* %d bytes payload */\\\n", x, id, n);
        printf ("\t0x%02x, ", q [m = 0]);
        while (++ m < N - 1) {
            printf ("0x%02x, ", q [m]);
            if ((m % 12) == 11)
                fputs ("\\\n\t", stdout);
        }
        printf ("0x%02x}\n\n", q [m]);
        free ((char*)q);
    }
    
    return n;
}


static
void print_cpry (void) {
    printf ("/*\n");
    printf (" * nvramd - active data base server and flash memory cache\n");
    printf (" *\n");
    printf (" * Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK\n");
    printf (" *\n");
    printf (" *  This library is free software; you can redistribute it and/or\n");
    printf (" *  modify it under the terms of the GNU Lesser General Public\n");
    printf (" *  License as published by the Free Software Foundation; either\n");
    printf (" *  version 2.1 of the License, or (at your option) any later version.\n");
    printf (" *\n");
    printf (" *  This library is distributed in the hope that it will be useful,\n");
    printf (" *  but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
    printf (" *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU\n");
    printf (" *  Lesser General Public License for more details.\n");
    printf (" *\n");
    printf (" *  You should have received a copy of the GNU Lesser General Public\n");
    printf (" *  License along with this library; if not, write to the Free Software\n");
    printf (" *  Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA\n");
    printf (" *\n");
    printf (" * Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-Isenburg, Germany\n");
    printf (" *\n");
    printf (" * generated: %s\n", time_print (time (0)));
    printf (" * $Id: dsakey-functions.h 2968 2006-07-29 19:58:03Z jordan $\n");
    printf (" */\n\n");
}


static
void print_dsa (DSA             *dsa,
                const char        *s,
                const char       *id,
                const char *param_id) {
    const char  *p;
    unsigned char bf [12], iv [8] ;
    int n8, n9, i ;
    
    rndata ((char*)bf, sizeof (bf));
    rndata ((char*)iv, sizeof (iv));
    

    printf ("#ifndef __DSAKEY%s_H\n", id);
    printf ("#define __DSAKEY%s_H\n\n", id);

    printf ("#if defined (__DSA_PUBLIC_KEY) || ");
    printf     ("defined (__DSA_PRIVATE_KEY) || ");
    printf     ("defined (__DSA_IMPORT_KEY)\n");
    printf ("#include <openssl/blowfish.h>\n");
    printf ("#include <openssl/dsa.h>\n");
    printf ("#include <openssl/x509.h>\n\n");

    n8 = derpub (dsa, 8, id, bf, sizeof (bf), iv);
    n9 = derprv (dsa, 9, id, bf, sizeof (bf), iv);

    printf ("static unsigned char *dsa_a%s (", id);
    printf ("const unsigned char *p, unsigned m) {\n");
    printf ("\tunsigned char iv [8] = {0x%02x", iv [i=0]&0xff);
    while (++ i < sizeof (iv)) printf (", 0x%02x", iv [i] & 0xff);
    printf ("};\n");
    printf ("\tunsigned char k [%d] = {0x%02x", sizeof (bf), bf[i=0]&0xff);
    while (++ i < sizeof (bf)) printf (", 0x%02x", bf [i] & 0xff);
    printf ("};\n");
    printf ("#\tifdef xmalloc\n");   
    printf ("\tunsigned char *q = (unsigned char*) xmalloc (m);\n");
    printf ("#\telse\n");
    printf ("\tunsigned char *q = (unsigned char*) malloc (m);\n");
    printf ("\tif (q) {\n");
    printf ("#\tendif /* xmalloc */\n");   
    printf ("\t\tBF_KEY b;\n");
    printf ("\t\tBF_set_key (&b, %d, k);\n", sizeof (bf));
    printf ("\t\tBF_cbc_encrypt (p, q, m, &b, iv, %d);\n", BF_DECRYPT);
    printf ("#\tifndef xmalloc\n");   
    printf ("\t}\n");
    printf ("#\tendif /* xmalloc */\n");   
    printf ("\treturn q;\n");
    printf ("}\n\n");

    if ((p = strchr (param_id, '_')) == 0)
        p = param_id ;
    else
        p += 2 ;

    printf ("#ifdef __DSA_IMPORT_KEY\n");
    printf ("#define dsa%sparam_id    \"%s\"\n", s, p) ;
    printf ("#define dsa%skey_id      \"%s\"\n\n", s, id) ;
    printf ("#define dsa%spubkey      dsa_8%s\n", s, id) ;
    printf ("#define DSA%sPUBKEY      DSA_8%s\n", s, id) ;
    printf ("#define dsa%sprivkey     dsa_9%s\n", s, id) ;
    printf ("#define DSA%sPRIVKEY     DSA_9%s\n\n", s, id) ;
    printf ("#define dsa%sdecode(p,m) dsa_a%s (p, m)\n", s, id) ;
    printf ("#endif /* __DSA_IMPORT_KEY */\n\n");
    printf ("#endif /* PUBLIC || PRIVATE || IMPORT */\n\n\n");
    
    printf ("#ifdef __DSA_PUBLIC_KEY\n");
    printf ("\nstatic DSA *dsa_b%s (void) {\n", id);
    printf ("\tconst unsigned char s [] = dsa_8%s ;\n", id);
    printf ("\tconst unsigned char *p = dsa_a%s (s, %d);\n", id, (n8+7)&~7);
    printf ("\tif (p) {\n");

    /* get prefix length */	
    for (i = 0; (n8 >> (7 * i)) != 0; i ++)
        ;
    printf ("\t\tconst unsigned char *q = p + %d;\n", i);
    printf ("\t\tDSA *d = d2i_DSA_PUBKEY (0, &q, %d);\n", n8);

    printf ("\t\tfree ((char*)p);\n");
    printf ("\t\treturn d;\n");
    printf ("\t}\n");
    printf ("\treturn 0;\n");
    printf ("}\n\n");
    printf ("#define dsa_get_pub_key%s() dsa_b%s ()\n", s, id);
    printf ("#else\n");
    printf ("#define dsa_get_pub_key%s() 0\n", s);
    printf ("#endif /* __DSA_PUBLIC_KEY */\n\n\n");

    printf ("#ifdef __DSA_PRIVATE_KEY\n");
    printf ("\nstatic DSA *dsa_c%s (void) {\n", id);
    printf ("\tconst unsigned char s [] = dsa_9%s ;\n", id);
    printf ("\tconst unsigned char *p = dsa_a%s (s, %d);\n", id, (n9+7)&~7);
    printf ("\tif (p) {\n");

    /* get prefix length */	
    for (i = 0; (n8 >> (7 * i)) != 0; i ++)
        ;
    
    printf ("\t\tconst unsigned char *q = p + %d;\n", i);
    printf ("\t\tDSA *d = d2i_DSAPrivateKey (0, &q, %d);\n", n9);

    printf ("\t\tfree ((char*)p);\n");
    printf ("\t\treturn d;\n");
    printf ("\t}\n");
    printf ("\treturn 0;\n");
    printf ("}\n\n");
    printf ("#define dsa_get_priv_key%s() dsa_c%s ()\n", s, id);
    printf ("#else\n");
    printf ("#define dsa_get_priv_key%s() 0\n", s);
    printf ("#endif /* __DSA_PRIVATE_KEY */\n\n");

    printf ("#endif /* __DSAKEY%s_H */\n", id);
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
