/*
 * nvramd - active data base server and flash memory cache
 *
 * Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA
 *
 * Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-Isenburg, Germany
 *
 * $Id: dsaparam.c 3447 2006-10-27 09:24:45Z henry $
 */

/*
 * Don't use any config.h or auto-generated config files, because they are
 * most probably just for the target environment.
 */
#define SYSLOG_NAMES

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <sys/time.h>
#include <time.h>
#include <unistd.h>
#include <syslog.h>
#include <libgen.h>
#include <ctype.h>
#include <errno.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdint.h>
#include <limits.h>

#include <openssl/dsa.h>
#include <openssl/bn.h>
#include <openssl/err.h>

/* ----------------------------------------------------------------------- *
 * not using -lnvram
 * ----------------------------------------------------------------------- */

#undef HAVE_CONFIG_H
#include "../libnvram/varstr.c"
#include "../libnvram/util.c"
#include "../libnvram/logger.c"
#include "../libnvram/base64.c"
#include "../libnvram/nvalloc.c"
#include "../libnvram/tvcalc.c"
#include "../libnvram/buffer.c"

/* ----------------------------------------------------------------------- *
 * Variables and definitions
 * ----------------------------------------------------------------------- */

#define __str(x)              # x
#define str(x)                __str (x)

typedef
struct _genk {
    int prime, count ;
} GENK ;

/* ----------------------------------------------------------------------- *
 * Helpers
 * ----------------------------------------------------------------------- */

static
void genkcb (int state, int m, void *desc) {
    GENK *genk = desc ;

    switch (state) {
    case 0:
        switch (genk->prime) {
        case '\0':
            fprintf (stderr, "%c: ", genk->prime = 'q');
            return ;
        case 'q':
        case 'p':
            genk->count ++ ;
            return ;
        }
        fputc ('?', stderr) ;
        return ;
    case 1:
        if (m == -1) {
            fprintf (stderr, "%u ", genk->count);
            genk->count = 0 ;
        }
        else
            fputc ('.', stderr) ;
        return ;
    case 2:
        switch (genk->prime) {
        case 'q':
            genk->prime = 'p';
            break ;
        case 'p':
            genk->prime = 'g';
            break ;
        default:
            fputc ('&', stderr) ;
            return ;
        }
        fprintf (stderr, "\n%c:", genk->prime);
        return ;
    case 3:
        switch (genk->prime) {
        case 'p':
            fputc (' ', stderr) ;
            return;
        case 'g':
            fputs (" ok\n", stderr);
        }
    }
}

static
void print_bn (const BIGNUM *a, int x, const char *id) {

    int            n = BN_num_bytes (a) ;
    unsigned char *p = (unsigned char *) xmalloc (n);
    int            m = 0;

    BN_bn2bin (a, p) ;
    printf ("static unsigned char dsa_%u%s [] = {\n\t0x%02x", x, id, p [0]);
    
    if (n > 1) {
        fputs (", ", stdout);
        while (++ m < n - 1) {
            printf ("0x%02x, ", p [m]);
            if ((m % 12) == 11)
                fputs ("\n\t", stdout);
        }
        printf ("0x%02x", p [m]);
    }

    fputs ("\n};\n", stdout);
    free (p);
}

static
void print_cpry (void) {
    printf ("/*\n");
    printf (" * nvramd - active data base server and flash memory cache\n");
    printf (" *\n");
    printf (" * Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK\n");
    printf (" *\n");
    printf (" *  This library is free software; you can redistribute it and/or\n");
    printf (" *  modify it under the terms of the GNU Lesser General Public\n");
    printf (" *  License as published by the Free Software Foundation; either\n");
    printf (" *  version 2.1 of the License, or (at your option) any later version.\n");
    printf (" *\n");
    printf (" *  This library is distributed in the hope that it will be useful,\n");
    printf (" *  but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
    printf (" *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU\n");
    printf (" *  Lesser General Public License for more details.\n");
    printf (" *\n");
    printf (" *  You should have received a copy of the GNU Lesser General Public\n");
    printf (" *  License along with this library; if not, write to the Free Software\n");
    printf (" *  Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA\n");
    printf (" *\n");
    printf (" * Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-Isenburg, Germany\n");
    printf (" *\n");
    printf (" * generated: %s\n", time_print (time (0)));
    printf (" * $Id: dsaparam.c 3447 2006-10-27 09:24:45Z henry $\n");
    printf (" */\n\n");
}

static
void print_dsa (DSA *dsa) {
    const char *id = rndalloc (10);

    fputs ("#ifndef __DSAPARAM_H\n", stdout);
    fputs ("#define __DSAPARAM_H\n\n", stdout);

    fputs ("#include <openssl/dsa.h>\n", stdout);
    fputs ("#include <openssl/bn.h>\n", stdout);

    putchar ('\n');
    print_bn (dsa->p, 0, id);
    putchar ('\n');
    print_bn (dsa->q, 1, id);
    putchar ('\n');
    print_bn (dsa->g, 2, id);
    putchar ('\n');

#   define Q(sym,n) "dsa->" str (sym) " = " \
                    "BN_bin2bn (dsa_" str (n) "%s, " \
                    "sizeof (dsa_" str (n) "%s), 0)"

    printf ("static DSA *dsa_3%s (void) {\n", id);
    fputs  ("\tDSA *dsa;\n",                              stdout);
    fputs  ("\tif ((dsa = DSA_new ()) == 0) return 0;\n", stdout);
    printf ("\tif ((" Q (p, 0) ") == 0) goto error;\n", id, id);
    printf ("\tif ((" Q (q, 1) ") == 0) goto error;\n", id, id);
    printf ("\tif ((" Q (g, 2) ") == 0) goto error;\n", id, id);
    fputs  ("\treturn dsa;\n",                            stdout);
    fputs  ("error:\n",                                   stdout);
    fputs  ("\tDSA_free (dsa);\n",                        stdout);
    fputs  ("\treturn 0;\n",                              stdout);
    fputs  ("}\n\n",                                      stdout);

#   undef Q

    printf ("#define get_dsa dsa_3%s\n\n", id);

    fputs  ("#endif /* __DSAPARAM_H */\n", stdout);

    free ((char*)id);
}

void usage (void) {
    fprintf (stderr, "Usage: %s <num-bits>\n",
             progname);
    exit (2) ;
}

/* ----------------------------------------------------------------------- *
 * Main
 * ----------------------------------------------------------------------- */

int main (int argc, const char **argv) {
    DSA *dsa ;
    int c_ret ;
    unsigned long h_ret;
    unsigned bits ;
    GENK genk ;

    if ((progname = strrchr (argv [0], '/')) == 0)
        progname = (char*) argv [0] ;
    else
        ++ progname ;

    if (argc != 2)
        usage () ;
  
    if (parse_unsigned (&bits, argv [1], 0) == 0 ||
        bits < 512 || 4096 < bits) {
        fprintf (stderr, "%s: <num-bits> must be a number between "
                 "512 and 4096\n", progname);
        exit (2);
    }
    
    XZERO (genk);
    dsa = DSA_generate_parameters (bits, 0,0, &c_ret, &h_ret, genkcb, &genk);

    if (dsa == 0) {
        long e = ERR_get_error ();
        fprintf (stderr, "%s: Error generating DSA parameters: %s (%lu)\n",
                 progname, ERR_error_string (e, 0), e);
        exit (2);
    }

    print_cpry () ;
    print_dsa (dsa);

    exit (0);
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
