/*
 * nvramd - active data base server and flash memory cache
 *
 * Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA
 *
 * Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-Isenburg, Germany
 *
 * $Id: symdump.c 3447 2006-10-27 09:24:45Z henry $
 */

/*
 * Don't use any config.h or auto-generated config files, because they are
 * most probably just for the target environment.
 */
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/time.h>
#include <stdlib.h>
#include <stddef.h>
#include <stdint.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <libgen.h>
#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <limits.h>
#include <stdio.h>
#include <stdarg.h>

#undef HAVE_CONFIG_H
#include "../buffer.h"
#include "../hlst.h"
#include "../prefix.h"

#define __NVRAM_LIST_PRIVATE
#include "../list.h"

#define __str(x)              # x
#define str(x)                __str (x)

#undef malloc

char *progname;

char *char2str (int c) {
   static char buf [10];
   switch (c) {
   case '\0': return  "\\0" ;
   case '\n': return  "\\n" ;
   case '\r': return  "\\r" ;
   case '\t': return  "\\t" ;
   case '\\': return "\\\\" ;
   }
   if (31 < c && c < 128)
     sprintf (buf, "%c", c);
   else
     sprintf (buf, "\\x%x", c & 0xff);
   return buf ;
}

char *upcase (char *s) {
    static char *buf;
   int n, len = strlen (s);
   if (buf)
       free (buf) ;
   if ((buf = malloc (len + 1)) == 0) {
       fprintf (stderr, "%s: malloc (%u) returned NULL (%s)!\n",
                progname, len + 1, strerror (errno));
       exit (2) ;
   }
   for (n = 0; n < len; n++)
       buf [n] = toupper (s [n]);
   buf [n] = '\0' ;
   return buf ;
}
             
int main (int argc, char **argv) {
    if ((progname = strrchr (argv [0], '/')) == 0)
        progname = argv [0] ;
    else
        ++ progname ;

    printf ("\n");

    printf ("/* typedef enum _action_bits */\n");
    printf ("#define ACTION_NO_BIT%-20s 0\n", "");
#   define        __flags(a) printf ("#define ACTION_BIT_%-22s%2u\n", \
                                     upcase (str (a)), action_bit_ ## a);
    ACTION_FLAGS (__flags)
#   undef         __flags
    printf ("\n");


    printf ("/* typedef enum _action_flags */\n");
#   define        __flags(a) printf ("#define ACTION_%-23s0x%03x\n", \
                                     upcase (str (a)), action_ ## a);
    ACTION_FLAGS (__flags)
#   undef         __flags
    printf ("\n");


    printf ("/* typedef enum _import_bits */\n");
#   define        __flags(a) printf ("#define IMPORT_BIT_%-22s%2u\n", \
                                     upcase (str (a)), import_bit_ ## a);
    IMPORT_FLAGS (__flags)
#   undef         __flags
    printf ("\n");


    printf ("/* typedef enum _import_flags */\n");
#   define        __flags(a) printf ("#define IMPORT_%-23s0x%03x\n", \
                                     upcase (str (a)), import_ ## a);
    IMPORT_FLAGS (__flags)
#   undef         __flags
    printf ("\n");


    printf ("/* typedef enum _action_type */\n");
#   define __token(a) printf ("#define %-31s\"%s\"\n", \
                              upcase (str (a)), char2str (a));
    __token (no_trigger)
    __token (access_trigger)
    __token (execute_trigger)
    __token (symlink_trigger)
    __token (dont_touch_trg)
#   undef __token
    printf ("\n");


    printf ("/* typedef enum _action_constants */\n");
#   define __token(a) printf ("#define %-31s\"%s\"\n", \
                              upcase (str (a)), char2str (a));
    __token (action_field_delim)
    __token (action_flag_delim)
    __token (escape_character)
#   undef __token
    printf ("\n");


    printf ("/* derived ... */\n");
#   define __token "#define %-28s 0x%03x /* %s */\n"

    printf (__token, "ACTION_VOLATILE_FUNCTION",
            (action_function_ref |
             action_volatile_data), ("ACTION_FUNCTION_REF" " | "
                                     "ACTION_VOLATILE_DATA"  ) );

    printf (__token, "ACTION_ENABLE_EXEC_ON_WRITE",
            (action_exec_ok      |
             action_link_exec_ok |
             action_enable_write ), ("ACTION_EXEC_OK"      " | "
                                     "ACTION_LINK_EXEC_OK" " | "
                                     "ACTION_ENABLE_WRITE"   ) );

    printf (__token, "ACTION_DELAY_EXEC_ON_WRITE",
            (action_exec_ok      |
             action_link_exec_ok |
             action_enable_write |
             action_delayed_run  ), ("ACTION_EXEC_OK"      " | "
                                     "ACTION_LINK_EXEC_OK" " | "
                                     "ACTION_ENABLE_WRITE" " | "
                                     "ACTION_DELAYED_RUN"    ) );

    printf (__token, "ACTION_ENABLE_EXEC_ON_WRITE_VOLATILE",
            (action_exec_ok      |
             action_link_exec_ok |
             action_enable_write |
             action_volatile_data), ("ACTION_EXEC_OK"      " | "
                                     "ACTION_LINK_EXEC_OK" " | "
                                     "ACTION_ENABLE_WRITE" " | "
                                     "ACTION_VOLATILE_DATA"  ) );

    printf (__token, "ACTION_DELAY_EXEC_ON_WRITE_VOLATILE",
            (action_exec_ok      |
             action_link_exec_ok |
             action_enable_write |
             action_delayed_run  |
             action_volatile_data), ("ACTION_EXEC_OK"      " | "
                                     "ACTION_LINK_EXEC_OK" " | "
                                     "ACTION_ENABLE_WRITE" " | "
                                     "ACTION_DELAYED_RUN"  " | "
                                     "ACTION_VOLATILE_DATA"  ) );

    printf (__token, "ACTION_ENABLE_XLINK_ON_WRITE",
            (action_exec_ok      |
             action_enable_write ), ("ACTION_EXEC_OK"      " | "
                                     "ACTION_ENABLE_WRITE"   ) );

    printf (__token, "ACTION_ENABLE_XLINK_ON_WRITE_VOLATILE",
            (action_exec_ok      |
             action_enable_write |
             action_volatile_data), ("ACTION_EXEC_OK"      " | "
                                     "ACTION_ENABLE_WRITE" " | "
                                     "ACTION_VOLATILE_DATA"  ) );

    printf (__token, "ACTION_DELAY_XLINK_ON_WRITE",
            (action_exec_ok      |
             action_enable_write |
             action_delayed_run  ), ("ACTION_EXEC_OK"      " | "
                                     "ACTION_ENABLE_WRITE" " | "
                                     "ACTION_DELAYED_RUN"    ) );

    printf (__token, "ACTION_DELAY_XLINK_ON_WRITE_VOLATILE",
            (action_exec_ok      |
             action_enable_write |
             action_delayed_run  |
             action_volatile_data), ("ACTION_EXEC_OK"      " | "
                                     "ACTION_ENABLE_WRITE" " | "
                                     "ACTION_DELAYED_RUN"  " | "
                                     "ACTION_VOLATILE_DATA"  ) );

    printf (__token, "ACTION_DELAY_XLINK_ON_WRITE_TMPL",
            (action_exec_ok         |
             action_enable_write    |
             action_delayed_run     |
             action_template_prefix ), ("ACTION_EXEC_OK"      " | "
                                        "ACTION_ENABLE_WRITE" " | "
                                        "ACTION_DELAYED_RUN"  " | "
                                        "ACTION_TEMPLATE_PREFIX") );
#   undef __token
    printf ("\n");

    exit (0);
}
