/*
 * nvramd - active data base server and flash memory cache
 *
 * Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA
 *
 * Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-Isenburg, Germany
 *
 * $Id: http-server.h 3128 2006-09-01 07:17:15Z jordan $
 */

#ifndef __NVRAMD_HTTP_SERVER_H
#define __NVRAMD_HTTP_SERVER_H

#include "util.h"
#include "list.h"
#include "do.h"
#include "buffer.h"

typedef
struct _rply_rec {
    struct _hchan *c ;
    IOBUF response ;
    IOBUF body ;
    const char *error ;
} RPLY_REC ;

#ifdef __HTTP_SERVER_PRIVATE
typedef
enum {
    none = 0,

    http_get_header,
    http_get_body,
    http_get_ok,

    http_post_header,
    http_post_body,
    http_post_ok,

    http_sent_ok
} HTPARSE ;

typedef
struct _cmd_map {
    const char *name ;
    size_t       len ;
    void (*cmd) (void*, RPLY_REC*) ;
    int    args_ok:2 ;
} CMD_MAP ;
#endif


typedef
struct _hchan
#ifdef __HTTP_SERVER_PRIVATE
{
    int           fd ;
    ip4addr_t client ;
    struct {
        IOBUF    buf ;
        struct {
            HTPARSE      state ;
            char cmd_args_ok:2 ;
            char     acc_ena:1 ;
            char    acc_text:1 ;
            char  conn_close:1 ;
            char    url_args:1 ;
            time_t  keep_alive ;
            unsigned      seen ;
            unsigned   cmd_req ;
            unsigned      args ;
            unsigned      body ;
            /* command rquest */
            size_t        clen ;
            void (*cmd) (void*, RPLY_REC*);
        } http ;
    } req ;
    struct _http *base ;
}
#endif
HCHAN ;

typedef
struct _http
#ifdef __HTTP_SERVER_PRIVATE
{
    int     listen_fd ;
    DO        *action ;
    LIST         *lst ;
    struct _handler {
        unsigned num_cmds ;
        CMD_MAP     *cmds ;
        CMD_MAP   def_cmd ;
    } handler ;
    unsigned channels ;
    HCHAN channel [1] ;
}
#endif
HTTP;


/* http response strings */
extern char e400 [] ;
extern char e503 [] ;
extern char e404 [] ;

/* public interface */
extern HTTP       *http_init      (DO*,LIST*,unsigned,ip4addr_t,uint16_t);
extern  int        http_fd        (HTTP*) ;
extern void        http_close     (HTTP*) ;
extern void        http_accept    (void*) ;
extern HTTP       *http_cmdadd    (HTTP*,const char*,int,
                                   void(*)(void*,RPLY_REC*), void*);

/* method call back handler example */
extern void        http_verbosity (void*, RPLY_REC*);

/* call back method handler support */
extern const char *http_shiftargs (RPLY_REC*) ;
extern IOBUF      *http_rplybody  (RPLY_REC*) ;
extern LIST       *http_rplylist  (RPLY_REC*) ;
extern void        http_rplyerror (RPLY_REC*,const char*);
extern void        http_rplydata  (RPLY_REC*,char*);
extern unsigned    http_haveargs  (RPLY_REC*);
extern const char *http_urlmethod (RPLY_REC*);
extern const char *http_cmdname   (RPLY_REC*);

#endif /* __NVRAMD_HTTP_SERVER_H */
