/*
 * Copyright (C) 2006 Maxina GmbH - Jordan Hrycaj
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 * Author: Jordan Hrycaj <jordan@mjh.teddy.net.com>
 *
 * $Id: logger.c 3108 2006-08-15 13:52:57Z jordan $
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>

#ifdef STDC_HEADERS
# include <stdlib.h>
# include <stddef.h>
#else
# ifdef HAVE_STDLIB_H
#  include <stdlib.h>
# endif
#endif

#ifdef HAVE_STRING_H
# if !defined (STDC_HEADERS) && defined (HAVE_MEMORY_H)
#  include <memory.h>
# endif
# include <string.h>
#endif

#ifdef TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#ifdef HAVE_UNISTD_H
# include <unistd.h>
# include <sys/types.h>
# else
# ifdef HAVE_SYS_TYPES_H
#  include <sys/types.h>
# endif
#endif

#ifdef HAVE_ERRNO_H
# include <errno.h>
#endif

#ifdef HAVE_SYSLOG_H
# define SYSLOG_NAMES
# include <syslog.h>
#endif

#include "buffer.h"
#include "util.h"
#include "logger.h"

/* ----------------------------------------------------------------------- *
 * Global variables
 * ----------------------------------------------------------------------- */

int logger_errno ;

/* ----------------------------------------------------------------------- *
 * Local variables
 * ----------------------------------------------------------------------- */

static int log_syslog = 0;
static int want_timestamp = 1 ;

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

void vlogger (int level, const char *format, va_list ap) {
    int n = errno ;

    IOBUF buf ;
    memset (&buf, 0, sizeof (buf));

    if (want_timestamp) {
        data_append (&buf, stamp_pp (0), 0);
        char_append (&buf, ' ');
    }
    data_vprintf (&buf, format, ap);

    while (buf.top && buf.data [buf.top - 1] == '\n')
        buf.data [-- buf.top] = '\0' ;

    if (log_syslog)
        syslog (level, "%s", buf.data);
    else {
        fprintf (stderr, "%s\n", buf.data);
        fflush (stderr);
    }

    kill_space (&buf);
    logger_errno = errno ;
    errno = n;
}


void logger (int level, const char *format, ...) {
    va_list ap;
    va_start (ap, format);
    vlogger (level, format, ap);
    va_end (ap);
}



unsigned syslog_init (char *fac, int timestamp) {

    int n = errno ;
    want_timestamp = timestamp ;

    if (fac) {
        int i = -1;
        while (facilitynames [++ i].c_name) {
            char *prog = progname ;
            if (prog == 0) prog = "[no progname]" ;
            if (strcasecmp (facilitynames [i].c_name, fac) == 0) {
                openlog (prog, LOG_PID, facilitynames [i].c_val);
                return log_syslog = 1 ;
            }
        }
    }

    if (log_syslog)
        closelog ();

    logger_errno = errno ;
    errno = n;
    return log_syslog = 0 ; 
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
