/*
 * nvram lib - flash data base client library
 *
 * Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA
 *
 * Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-Isenburg, Germany
 *
 * $Id: util.h 2640 2006-06-22 10:01:06Z jordan $
 */

#ifndef __NVRAMLIB_UTIL_H
#define __NVRAMLIB_UTIL_H

#include <time.h>
#include <string.h>
#include <stdarg.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/if_ether.h>

#ifdef HAVE_INTTYPES_H
# include <inttypes.h>
#else
# if HAVE_STDINT_H
# include <stdint.h>
# endif
#endif

#ifdef HAVE_SYS_UN_H
# include <sys/un.h>
# ifndef UNIX_PATH_MAX
# define UNIX_PATH_MAX	sizeof (((struct sockaddr_un*)0)->sun_path)
# endif
#else
# ifdef HAVE_LINUX_UN_H
# include <linux/un.h>
# endif
#endif

/* nvram-lib.c */
#ifdef __NVRAMLIB_UTIL_PRIVATE
# ifndef  NVCL_REQUEST_SOCKET   /* local debugging, only */
#  define NVCL_REQUEST_SOCKET   "/var/run/nvram/server"
# endif
#else
# ifdef  NVCL_REQUEST_SOCKET   /* local debugging, only */
#  warning Using a local socket will not work reliably after installation !!!
# endif
# define NVCL_REQUEST_SOCKET	nvcl_request_socket
#endif /* __NVRAMLIB_UTIL_PRIVATE */

#ifndef  NVCL_HTTP_CONFIG_PORT /* local debugging, only */
# define NVCL_HTTP_CONFIG_PORT 888 /* http configuration port */
#endif

#define NVCL_REPLY_PFX         "/tmp/nvram-"
#define NVCL_DEFAULT_CHROOT    ""
#define NVCL_DEFAULT_REPLY_TMO 5000 /* ms */
#define NVCL_BASE64_INTRO      '}'
#define NVCL_OPEN_LOG          0x000001 /* log selector flag */
#define NVCL_TIMEOUT_LOG       0x000002 /* log selector flag */
#define NVCL_IOBUF_SIZE        4000


/* util.c */
#define DEFAULT_RANDOM_DEV     "/dev/urandom" 
extern void          rndata  (char*, size_t) ;

/* global variables */
extern char nvcl_request_socket [] ; /* server socket */
extern char     *progname ;          /* program name */
extern unsigned verbosity ;          /* verbosity bits */

/* ipv4 address in host/register byte order */
typedef uint32_t ip4addr_t ;

/* pretty print an ipv4 address, keeps the last 4 return values 
   allocated on static memory */
extern const char  *ipaddr_pp (const struct in_addr *); /* arg may be NULL */
extern const char *ip4addr_pp (ip4addr_t) ;

/* pretty print the given time stamp */
extern char   *stamp_pp (const struct timeval *); /* ptr arg maybe NULL */
extern char    *time_pp (long t) ;
#endif /* __NVRAMLIB_UTIL_H */
