/*
 * nvramd - active data base server and flash memory cache
 *
 * Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA
 *
 * Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-Isenburg, Germany
 *
 * $Id: list.c 3877 2007-04-12 18:54:58Z hholtschneider $
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>

#ifdef STDC_HEADERS
# include <stdlib.h>
# include <stddef.h>
#else
# ifdef HAVE_STDLIB_H
#  include <stdlib.h>
# endif
#endif

#ifdef HAVE_STRING_H
# if !defined (STDC_HEADERS) && defined (HAVE_MEMORY_H)
#  include <memory.h>
# endif
# include <string.h>
#endif

#ifdef TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# ifdef HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#ifdef HAVE_UNISTD_H
# include <unistd.h>
# include <sys/types.h>
# else
# ifdef HAVE_SYS_TYPES_H
#  include <sys/types.h>
# endif
#endif

#ifdef HAVE_ERRNO_H
# include <errno.h>
#endif
#ifdef HAVE_CTYPE_H
# include <ctype.h>
#endif
#ifdef HAVE_SYS_SELECT_H
# include <sys/select.h>
#endif
#ifdef HAVE_SYS_STAT_H
# include <sys/stat.h>
#endif
#ifdef HAVE_SYS_WAIT_H
# include <sys/wait.h>
#endif
#ifdef HAVE_FCNTL_H
# include <fcntl.h>
#endif
#ifdef HAVE_SIGNAL_H
# include <signal.h>
#endif

#include "buffer.h"
#include "varstr.h"
#include "flash.h"
#include "settings.h"
#include "tvcalc.h"
#include "hlst.h"
#include "nvalloc.h"
#include "prefix.h"
#include "logger.h"

#define __NVRAM_LIST_PRIVATE
#include "list.h"

#if defined (_DEBUG) && !defined (_DEBUG_IDLE_SCHED)
#define _DEBUG_IDLE_SCHED 1
#endif

#undef TRACE_VARIABLE1
#undef TRACE_VARIABLE2
#undef TRACE_VARIABLE3
#undef TRACE_VARIABLE4
#undef TRACE_VARIABLE5
#undef TRACE_VARIABLE6

// #define TRACE_VARIABLE1 "wan_virtual_rule.endport.6"

/* ----------------------------------------------------------------------- *
 * private: memory debugging stuff
 * ----------------------------------------------------------------------- */

#if defined (TRACE_VARIABLE1) || defined (TRACE_VARIABLE2) ||\
    defined (TRACE_VARIABLE3) || defined (TRACE_VARIABLE4) ||\
    defined (TRACE_VARIABLE5) || defined (TRACE_VARIABLE6)
#define TRACE_VARIABLE
#endif

#ifdef TRACE_VARIABLE
static
unsigned trace_variable (const char *var) {
    return 0
#       ifdef TRACE_VARIABLE1
        || strcmp (var, TRACE_VARIABLE1) == 0
#       endif
#       ifdef TRACE_VARIABLE2
        || strcmp (var, TRACE_VARIABLE2) == 0
#       endif
#       ifdef TRACE_VARIABLE3
        || strcmp (var, TRACE_VARIABLE3) == 0
#       endif
#       ifdef TRACE_VARIABLE4
        || strcmp (var, TRACE_VARIABLE4) == 0
#       endif
#       ifdef TRACE_VARIABLE5
        || strcmp (var, TRACE_VARIABLE5) == 0
#       endif
#       ifdef TRACE_VARIABLE6
        || strcmp (var, TRACE_VARIABLE6) == 0
#       endif
        ;
}
#endif /* TRACE_VARIABLE */

#ifdef WITH_DMALLOC
#define __DMALLOC_SAY(s) {\
     dmalloc_message ("*** %s(%d): dmalloc " s ".\n",__FILE__,__LINE__);\
  }
#define DMALLOC_VERIFY(l) {\
     __DMALLOC_SAY ("LIST check point"); \
     dmalloc_verify (l->data.data);      \
     dmalloc_verify (l->var.data);       \
     dmalloc_verify (l->list.slot);      \
     dmalloc_verify (l->lookup);         \
     dmalloc_log_stats ();               \
  }
#define DMALLOC_VERIFZ(l) {\
     __DMALLOC_SAY ("check point");      \
     dmalloc_verify (l);                 \
     dmalloc_log_stats ();               \
  }
#define DMALLOC_CHK_EXEC_ITEM(q) {\
     if ((q)->a.trigger_type == execute_trigger && (q)->t.execute.argv) { \
        int m ;                                                           \
        dmalloc_verify ((q)->t.execute.argv);                             \
        for (m = 0 ;  (q)->t.execute.argv [m] ; m ++) {                   \
            dmalloc_verify ((q)->t.execute.argv [m]);                     \
        }                                                                 \
     }                                                                    \
  }
#define DMALLOC_CHK_EXEC(l) {\
     int n ; AVPAIR **Q, *q ;                                           \
     __DMALLOC_SAY ("exec data checker");                               \
     if (l) {                                                           \
       dmalloc_verify (l);                                              \
       for (n = 0; n < (l)->list.entries; n ++)                         \
          if ((Q = (l)->list.slot [n]) != 0 && (q = *Q) != 0) {         \
            DMALLOC_CHK_EXEC_ITEM (q) ;                                 \
          }                                                             \
     }                                                                  \
     dmalloc_log_stats ();                                              \
  }

#else
#define DMALLOC_VERIFY(l)        /* n/a */
#define DMALLOC_VERIFZ(s)        /* n/a */
#define DMALLOC_CHK_EXEC_ITEM(q) /* n/a */
#define DMALLOC_CHK_EXEC(l)      /* n/a */
#endif

/* ----------------------------------------------------------------------- *
 * private: constants, private data, macros
 * ----------------------------------------------------------------------- */

static const char action_field_delim_str [2] = {action_field_delim, '\0'};

static unsigned list_add_slot      (LIST*);

#define DIM(x)  (sizeof (x) / sizeof (x [0]))

#define __str(x)              # x
#define str(x)                __str (x)


static const struct {

    int  msfac ; /* wait factor: usec -> msec */
    int signal ;

} try [] = { {1000, SIGTERM},
             {2000, SIGTERM},
             {4000, SIGTERM},
             {2000, SIGKILL},
             {2000, SIGKILL} } ;


static const struct {

    const char *name ;
    size_t       len ;

} action2flag [] = {

#   define        __flags(a) {str (a), sizeof (str (a)) - 1},
    ACTION_FLAGS (__flags)
#   undef         __flags

} ;

#define __rwx_flags               (action_private_ok      |\
                                   action_public_read     |\
                                   action_public_write    |\
                                   action_public_modify   )

#define __fun_flags               (action_function_ref    |\
                                   action_volatile_data   )

#define __actv_flags              (action_enable_write    |\
                                   action_enable_read     )

#define __when_flags              (action_force_immediate |\
                                   action_delayed_run     )

#define acceptable_function_flags (__rwx_flags  | __fun_flags)

#define acceptable_access_flags   (__rwx_flags  | __fun_flags  |\
                                   action_template_prefix      )

#define acceptable_exec_flags     (__rwx_flags  | __fun_flags  |\
                                   __actv_flags | __when_flags |\
                                   action_exec_ok              |\
                                   action_link_exec_ok         |\
                                   action_template_prefix      )

#define acceptable_link_flags     (__rwx_flags  | __fun_flags  |\
                                   __actv_flags | __when_flags |\
                                   action_exec_ok              |\
                                   action_link_exec_ok         |\
                                   action_template_prefix      )
 
/* ----------------------------------------------------------------------- *
 * private: bits'n flags
 * ----------------------------------------------------------------------- */

static
int str2flags (int *Invert, const char *s) {

    int flags ;
    const char *t = s;

    if (s == 0)
        return 0 ;

    switch (s [0]) {
    case '\0':
        return 0;
    case  '!':
        if (s [1] == '\0') {
            if (verbosity & LIST_LOG_FLAG_PARSE_ERROR)
                logger (LOG_ERR,
                        "Error parsing flags \"%s\": flags missing", t);
            return -1 ;
        }
        if (Invert == 0) {
            if (verbosity & LIST_LOG_FLAG_PARSE_ERROR)
                logger (LOG_ERR, "Error parsing flags \"%s\": negotiation "
                        "operator not allowed, here", t);
            return -1 ;
        }
        s ++ ;
        *Invert = 1 ;
        break ;
    default:
        if (Invert != 0)
            *Invert = 0 ;
    }

    flags = 0 ;
   
    if (strcmp (s, str (NO_FLAGS_SYMBOL)) != 0)

        for (;;) {
            const char *end_token, *next_token ;
            int bit_num ;
            size_t tlen ;

            /* find the end of the input field */
            for (end_token = s;
                 isalnum (*end_token) || *end_token == '_';
                 end_token ++)
                ;

            /* find start of the next token (if any) */
            for (next_token = end_token ;
                 *next_token == action_flag_delim || *next_token == ' ';
                 next_token ++)
                ;

            /* check validity: token strings must properly terminate */
            if (*next_token           != '\0' &&
                isalnum (*next_token) ==   0  ) {
                /* *next_token != '_') { */
                if (verbosity & LIST_LOG_FLAG_PARSE_ERROR)
                    logger (LOG_ERR, "Error parsing flags \"%s\": "
                            "not a token at \"... %s\"", t, next_token);
                return -1 ;
            }

            /* are we through? */
            if (end_token == s)
                break ;
            
            /* check whether this string is in the table */
            bit_num = 0 ;
            tlen = end_token - s ;
            for (;;) {
                if (action2flag [bit_num].len                     == tlen &&
                    strncmp (s, action2flag [bit_num].name, tlen) == 0) {
                    flags |= (1 << bit_num) ;
                    s = next_token ;
                    break ;
                }
                if (++ bit_num >= DIM (action2flag)) {
                    if (verbosity & LIST_LOG_FLAG_PARSE_ERROR) {
                        int save_c = end_token [0] ;
                        ((char*)end_token) [0] = '\0' ;
                        logger (LOG_ERR, "Error parsing flags \"%s\": "
                                "no such token \"%s\"", t, s);
                        ((char*)end_token) [0] = save_c ;
                    }
                    return -1 ;
                }
            } /* for (string is in the table check) */
        } /* for (loop over the token list) */
    
    if (flags & (action_public_read   |
                 action_public_write  |
                 action_public_modify ))
        flags &= ~action_private_ok ;  /* clear unneeded flags */

    return flags;
}


static
int trigger_map (int trigger_flags) {

    if ((trigger_flags & action_exec_ok) == 0)
        return 0;

    if (trigger_flags & (action_force_immediate|action_delayed_run)) {
        if (trigger_flags & action_force_immediate)
            trigger_flags &= ~action_delayed_run ;
    }
    else
        trigger_flags |= action_force_immediate ;

    return trigger_flags ;
}


static
int link_trigger_map (int link_flags, int trigger_flags) {

    if ((trigger_flags & action_link_exec_ok) == 0)
        return 0;

    if ((link_flags & action_exec_ok) == 0)
        return 0;   

    if (link_flags & (action_force_immediate|action_delayed_run)) {
        if (link_flags & action_force_immediate)
            link_flags &= ~action_delayed_run ;
    }
    else
        link_flags |= action_force_immediate ;

    return
        link_flags&(trigger_flags|action_force_immediate|action_delayed_run);
}


static
AVPAIR **link_follow_avp (LIST *l, AVPAIR *q, int *Flags) {
    
    if (q != 0 && q->a.trigger_type == symlink_trigger) {
        AVPAIR **R = (AVPAIR**) find_hlst (l->lookup, q->t.symlink.ref,0);

        if (R != 0 && *R != 0 && (*R)->a.trigger_type == execute_trigger) {
            int x_flags = link_trigger_map (q->a.flags, (*R)->a.flags);

            if (x_flags != 0) {
                if (Flags)
                    *Flags = x_flags ;
                return R ;
            }
        }
    }

    return 0 ;
}


static
void complain_on_missing_fn (LIST *l, const char *var, const char *val) {
    if (val != 0 && val [0] != '\0') {
        VALFUN **F = (VALFUN **) find_hlst (l->function, val, 0);
        if (F == 0 || *F == 0)
            logger (LOG_WARNING, "There is no function named \"%s\" "
                    "for varaible \"%s\"", var, val);
    }
    else {
        logger (LOG_WARNING,
                "NULL-function assigned to variable \"%s\"", var);
    }
}

/* ----------------------------------------------------------------------- *
 * private: data table output
 * ----------------------------------------------------------------------- */

static /* escape colons in the field text */
const char *action_field (LIST *l, const char *s) {

    const char *q = strchr (s, action_field_delim) ;
    
    if (q == 0)
        return s ;

    kill_space  (&l->var);

    do {
        data_append (&l->var, s, q - s);
        char_append (&l->var, escape_character);
        char_append (&l->var, * q);
    }
    while ((s = q + 1, q = strchr (s, action_field_delim)) != 0);

    return l->var.data ;
}


static
void dump_execute_fields (LIST *l, AVPAIR *a, const char *var) {
    const char **Av, *arg ;
    
    if (a->t.execute.argv == 0 || a->t.execute.argv [0] == '\0') {
        logger (LOG_ERR, "Not writing trigger without command ars: %s", var);
        return ;
    }

    /* append to data store */
    char_append (&l->data, execute_trigger);

    if (a->a.flags)
        data_printf (&l->data, "%#x", a->a.flags);

    char_append (&l->data, action_field_delim);

    if (a->t.execute.uid >= 0)
        data_printf (&l->data, "%d", a->t.execute.uid);

    char_append (&l->data, action_field_delim);

    if (a->t.execute.gid >= 0)
        data_printf (&l->data, "%d", a->t.execute.gid);

    char_append (&l->data, action_field_delim);

    if (a->t.execute.chroot [0])
        data_append (&l->data, action_field (l, a->t.execute.chroot), 0);

    if ((Av = a->t.execute.argv) == 0)
        return ;

    while ((arg = * Av ++) != 0)
        data_printf
            (&l->data, "%c%s", action_field_delim, action_field (l, arg));

    char_append (&l->data, action_field_delim);
}


static
void dump_symlink_fields (LIST *l, AVPAIR *a, const char *var) {

    if (a->t.symlink.ref == 0 || a->t.symlink.ref [0] == '\0') {
        logger (LOG_ERR, "Not writing symlink without target: %s", var);
        return ;
    }

    /* append to data store */
    char_append (&l->data, symlink_trigger);

    if (a->a.flags)
        data_printf (&l->data, "%#x", a->a.flags);

    data_printf (&l->data, "%c%s%c",
                 action_field_delim, a->t.symlink.ref, action_field_delim);
}


static
void dump_access_fields (LIST *l, AVPAIR *a) {
    
    /* append to data store */
    char_append (&l->data, access_trigger);

    if (a->a.flags)
        data_printf (&l->data, "%#x", a->a.flags);

    char_append (&l->data, action_field_delim);
}


static
const char **argv_execute_fields (LIST *l, AVPAIR *a) {
    int n, m, flags ;

    kill_space (&l->var);
    make_space (&l->var, (a->t.execute.argc + 6) * sizeof (char*));
    l->var.top         = (a->t.execute.argc + 6) * sizeof (char*) ;

    ((char **)l->var.data) [n = 0] = (char*)l->var.top ;
    char_append (&l->var, execute_trigger);
    char_append (&l->var, '\0');

    ((char **)l->var.data) [++ n] = (char*)l->var.top ;
    if ((flags = a->a.flags) != 0) {
        int bit_num = 0, seen = 0 ;
        do
            if ((flags & 1) != 0) {
                if (seen ++)
                    char_append (&l->var, action_flag_delim);
                data_append (&l->var, action2flag [bit_num].name, 0);
            }
        while ((flags >>= 1) && ++ bit_num < DIM (action2flag)) ;
    }
    char_append (&l->var, '\0');
    
    ((char **)l->var.data) [++ n] = (char*)l->var.top ;
    data_printf (&l->var, "%d", a->t.execute.uid);
    char_append (&l->var, '\0');
                
    ((char **)l->var.data) [++ n] = (char*)l->var.top ;
    data_printf (&l->var, "%d", a->t.execute.gid);
    char_append (&l->var, '\0');

    ((char **)l->var.data) [++ n] = (char*)l->var.top ;
    if (a->t.execute.chroot [0])
        data_append (&l->var, a->t.execute.chroot, 0);
    char_append (&l->var, '\0');

    /* update pointers from offsets */
    for (m = 0; m <= n; m ++)
        ((char **)l->var.data) [m] =
            l->var.data + (int) ((char **)l->var.data) [m] ;

    /* assign argv [] */
    for (m = 0; m < a->t.execute.argc; m ++)
        ((const char **)l->var.data) [++ n] = a->t.execute.argv [m] ;

    ((char **)l->var.data) [++ n] = 0 ;

    return (const char **) l->var.data ;
}


static
const char **argv_symlink_fields (LIST *l, AVPAIR *a) {
    int n, m, flags ;
    
    kill_space (&l->var);
    make_space (&l->var, 4 * sizeof (char*));
    l->var.top         = 4 * sizeof (char*) ;

    ((char **)l->var.data) [n = 0] = (char*)l->var.top ;
    char_append (&l->var, symlink_trigger);
    char_append (&l->var, '\0');

    ((char **)l->var.data) [++ n] = (char*)l->var.top ;
    if ((flags = a->a.flags) != 0) {
        int bit_num = 0, seen = 0 ;
        do
            if ((flags & 1) != 0) {
                if (seen ++)
                    char_append (&l->var, action_flag_delim);
                data_append (&l->var, action2flag [bit_num].name, 0);
            }
        while ((flags >>= 1) && ++ bit_num < DIM (action2flag)) ;
    }
    char_append (&l->var, '\0');

    ((char **)l->var.data) [++ n] = (char*)l->var.top ;
    data_append (&l->var, a->t.symlink.ref, 0);

    ((char **)l->var.data) [++ n] = 0 ;

    /* update pointers from offsets */
    for (m = 0; m < n; m ++)
         ((char **)l->var.data) [m] =
             l->var.data + (int) ((char **)l->var.data) [m] ;
        
    return (const char **) l->var.data ;
}


static
const char **argv_access_fields (LIST *l, AVPAIR *a) {
    int n, m,flags ;
    
    kill_space (&l->var);
    make_space (&l->var, 4 * sizeof (char*));
    l->var.top         = 4 * sizeof (char*) ;

    ((char **)l->var.data) [n = 0] = (char*)l->var.top ;
    char_append (&l->var, access_trigger);
    char_append (&l->var, '\0');

    ((char **)l->var.data) [++ n] = (char*)l->var.top ;
    if ((flags = a->a.flags) != 0) {
        int bit_num = 0, seen = 0 ;
        do
            if ((flags & 1) != 0) {
                if (seen ++)
                    char_append (&l->var, action_flag_delim);
                data_append (&l->var, action2flag [bit_num].name, 0);
            }
        while ((flags >>= 1) && ++ bit_num < DIM (action2flag)) ;
    }
    char_append (&l->var, '\0');

    ((char **)l->var.data) [++ n] = 0 ;

    /* update pointers from offsets */
    for (m = 0; m < n; m ++)
         ((char **)l->var.data) [m] =
             l->var.data + (int) ((char **)l->var.data) [m] ;
        
    return (const char **) l->var.data ;
}


/* ----------------------------------------------------------------------- *
 * private: input data parser
 * ----------------------------------------------------------------------- */

static
const char **split_fields (const char *s) {

    const char **P = 0;
    unsigned     n = 0;

    goto start_here;

    do {
        s ++ ; /* skip delimiter */
        if (s [-2] == escape_character)
            continue ;

    start_here:
        P = (const char **) zrealloc (P, (n + 2) * sizeof (char*));
        P [n ++] = s ;
    }
    while ((s = strchr (s, action_field_delim)) != 0) ;

    P [n] = 0 ;
    return P;
}


static
const char *alloc_value (const char *old, const char *s) {
    if (s) {
        size_t len ;
        if (s [0] == escape_character)
            s ++ ;
        len = strlen (s) + 1;
        /* old == 0 is ok for zrealloc() */
        return memcpy (zrealloc ((char*)old, len), s, len) ;
    }
    if (old)
        free ((char*)old);
    return 0;  
}


static
void free_avpair_argv (AVPAIR *q) {

    if (q) {
        DMALLOC_CHK_EXEC_ITEM (q) ;

        /* free the argv vector */
        if (q->a.trigger_type == execute_trigger) {
            const char **p = q->t.execute.argv ;

            if (p) {
                /* clean up an argument list */
                while (* p) {
                    DMALLOC_VERIFZ (*p);
                    free ((char*) * p ++);
                }
                DMALLOC_VERIFZ (q->t.execute.argv);
                free ((char*) q->t.execute.argv);
                q->t.execute.argv = 0 ;
            }
        }
    }
}


static
void free_avpair (AVPAIR *q) {
    alloc_value (q->a.val, 0);
    free_avpair_argv (q);
    free (q);
}


static
AVPAIR *realloc_avpair (AVPAIR *q, ACTION_TYPE t, size_t extra) {
    size_t len ;

    if (q && t == dont_touch_trg)
        t = q->a.trigger_type ;

    switch (t) {
    case execute_trigger:
        len = extra + (sizeof (struct _avcommon) + sizeof (struct _execute));
        break;
    case symlink_trigger:
        free_avpair_argv (q);
        len = extra + (sizeof (struct _avcommon) + sizeof (struct _symlink));
        break;
    default:
        if (t != access_trigger) {
            t = no_trigger ;
            if (q)
                q->a.flags = 0 ;
        }
        free_avpair_argv (q);
        len = sizeof (struct _avcommon) ;
    }

    q = (AVPAIR*) zrealloc (q, len) ;

    switch (q->a.trigger_type = t) {
    case execute_trigger:
        *(char*)(q->t.execute.chroot + extra) = '\0' ;
        break;
    case symlink_trigger:
        *(char*)(q->t.symlink.ref    + extra) = '\0' ;
        break;
    default:
        ;
    }

    DMALLOC_VERIFZ (q) ;
    return q;
}


static
AVPAIR **populate_trigger_slotx (LIST             *l,
                                 AVPAIR          **Q,
                                 const char     *var,
                                 ACTION_TYPE new_trg, /* use when creating */
                                 ACTION_TYPE set_trg, /* use when exists */
                                 ACTION_TYPE vlt_trg, /* use when volatile */
                                 size_t          len) {

    if (Q == 0 && (Q = (AVPAIR **) find_hlst (l->lookup, var, 0)) == 0) {

        Q = (AVPAIR **) make_hlst (l->lookup, var, 0) ;
        *Q = realloc_avpair (0, new_trg, len);
        (*Q)->a.just_created = 1 ;

        (*Q)->a.inx = list_add_slot (l) ;
        l->list.slot [(*Q)->a.inx] = Q ;
    }
    else {

        (*Q)->a.just_created = 0 ;
        if ((*Q)->a.flags & action_volatile_data)
            set_trg = vlt_trg ;

        switch (set_trg) {
        case execute_trigger:
            *Q = realloc_avpair (*Q, execute_trigger, len);
            memset (&(*Q)->t.execute, 0, sizeof ((*Q)->t.execute) + len - 1);
            break ;
            
        case symlink_trigger:
            *Q = realloc_avpair (*Q, symlink_trigger, len);
            memset (&(*Q)->t.symlink, 0, sizeof ((*Q)->t.symlink) + len - 1);
            break ;

        case dont_touch_trg:
            break ;
 
        default: /* set_trg: no_trigger or access_trigger */
            if ((*Q)->a.trigger_type == execute_trigger)
                free_avpair_argv (*Q);
            *Q = realloc_avpair (*Q, set_trg, 0);
        }
    }

    DMALLOC_VERIFZ (*Q) ;
    return Q ;
}


static
AVPAIR **populate_trigger_slot (LIST         *l,
                                AVPAIR      **Q,
                                const char *var,
                                ACTION_TYPE   t,
                                size_t      len) {
    return populate_trigger_slotx (l, Q, var, t, t, t, len);
}


static
AVPAIR *parse_execute (const char *s, const char *err) {

    const char **P = split_fields (s);
    AVPAIR *a ;
    
    unsigned u ;
    size_t vlen ;
    int n = 0, m ;

    if (P [0] == 0) { a = 0; goto ignore; }
    a = realloc_avpair (0, execute_trigger, 0);
    
    /* flags */
    if (P [n] + 1 < P [n + 1]) {
        if (isdigit (P [n] [0]) && parse_unsigned
            (&u, P [n], action_field_delim_str))
            a->a.flags = u ;
        else
            goto flags;
    }
    if (P [++ n] == 0) goto ignore ;

    /* user id */
    a->t.execute.uid = (isdigit (P [n] [0]) && parse_unsigned
                        (&u, P [n], action_field_delim_str)) ? u : -1 ;
    if (P [++ n] == 0) goto ignore ;

    /* group id */
    a->t.execute.gid =  (isdigit (P [n] [0]) && parse_unsigned
                         (&u, P [n], action_field_delim_str)) ? u : -1 ;
    if (P [++ n] == 0) goto ignore ;

    /* chroot */
    if ((vlen = P [n + 1] ? P [n + 1] - P [n] - 1 : strlen (P [n])) > 1) {
        a = realloc_avpair (a, execute_trigger, vlen); /* adds \0 marker */
        memcpy ((char*)a->t.execute.chroot, P [n], vlen);
    }
    if (P [++ n] == 0) goto ignore ;
        
    /* command string and arguments */
    m = n ;
    while (P [n + 1] != 0) {
        vlen = P [n + 1] - P [n] ;
        a->t.execute.argv = (const char **)
            zrealloc (a->t.execute.argv, (n - m + 2) * sizeof (char*));
        a->t.execute.argv [n - m] = memcpy (xmalloc (vlen), P [n], vlen - 1);
        ((char*)(a->t.execute.argv [n - m])) [vlen - 1] = '\0' ;
        DMALLOC_VERIFZ (a->t.execute.argv [n - m]) ;
        n ++ ;
    }
    a->t.execute.argv [a->t.execute.argc = n - m] = 0 ;

    goto all_done ;
    /* NOTREACHED */

 flags:
    logger (LOG_ERR, "Illegal exec/command definition flags: %s", err);
    goto reset_avpair;

 ignore: /* no command to execute - ignore these settings */
    logger (LOG_ERR, "Command def has not enough fields: %s", err);
    /* goto reset_avpair ; */
    
 reset_avpair:
    if (a) {
        free (a) ; /* free_avpair() not needed, yet */
        a = 0 ;
    }

 all_done:
    if (a)
        a->a.val = alloc_value (0, P [n]) ;
    free ((char*)P);
    return a ;
}


static
AVPAIR *parse_symlink (const char *s, const char *err) {

    const char **P = split_fields (s);
    size_t vlen ;
    AVPAIR *a = 0;

    if (P [0] == 0 || P [1] == 0) {
        /* no link target - ignore these settings */
        logger (LOG_ERR, "Link definition has not enough fields: %s", err);
        goto leave_with_a ;
    }

    if ((vlen = P [2] ? P [2] - P [1] - 1 : strlen (P [1])) == 0) {
        logger (LOG_ERR, "Link definition has no link target: %s", err);
        goto leave_with_a ;
    }
        
    a = realloc_avpair (0, symlink_trigger, vlen); /* adds \0 marker */

    /* flags */
    if (P [0] + 1 < P [1]) {
        unsigned u;
        if (isdigit (P [0] [0])                                == 0 ||
            parse_unsigned (&u, P [0], action_field_delim_str) == 0) {
            logger (LOG_ERR, "Illegal link definition flags: %s", err);
            free (a) ; /* free_avpair() not needed, yet */
            a = 0 ;
            goto leave_with_a ;
        }
        a->a.flags = u ;
    }

    /* link target */
    memcpy ((char*)a->t.symlink.ref, P [1], vlen);
    a->a.val = alloc_value (0, P [2]) ;

 leave_with_a:
    free ((char*)P);
    return a ;
}


static
AVPAIR *parse_access (const char *s, const char *err) {

    const char **P = split_fields (s);
    int n = 0 ;
    unsigned u;
    AVPAIR *a ;

    if (P [0] == 0) { a = 0; goto ignore; }
    a = realloc_avpair (0, access_trigger, 0);

    /* flags */
    if (P [n] + 1 < P [n +1]) {
        if (isdigit (P [n] [0]) &&  parse_unsigned
            (&u, P [n], action_field_delim_str))
            a->a.flags = u ;
        else
            goto flags;
    }
    if (P [++ n] == 0) goto ignore;
    
    goto all_done ;
    /* NOTRACHED */

flags:
    logger (LOG_ERR, "Invalid variable access definition flags: %s", err);
    goto reset_avpair;
    
ignore: /* no link target - ignore these settings */
    logger (LOG_ERR, "Access definition has not enough fields: %s", err);
    /* goto reset_avpair ; */
    
reset_avpair:
    if (a) {
        free (a) ; /* free_avpair() not needed, yet */
        a = 0 ;
    }

all_done:
    if (a)
        a->a.val = alloc_value (0, P [n]) ;
    free ((char*)P);
    return a ;
}

/* ----------------------------------------------------------------------- *
 * private: exec helpers
 * ----------------------------------------------------------------------- */

static
void commit_scheduler (LIST *l) {
    int n ;
    if (tval_used (&l->time2check)) {
        if ((n = ms_tmo_tv (&l->time2check)) <= 0)
            n = 1 ;
    }
    else
        n = 0 ;
    do_resched (l->sched, n);
}


static
void update_scheduler (LIST *l, struct timeval *due, unsigned delta) {

#   ifdef _DEBUG_IDLE_SCHED
    struct timeval now ;
    char a0 [50], a1 [50], a2 [50], b1 [50], b2 [50], *op;
    tval_nxt_ms (&now, 0);
    strcpy (a0, tval_print (&now));
    strcpy (a1, tval_print (&l->time2check));
    strcpy (a2, tval_print (due));
    op = (due == &l->time2commit) ? "NVRAM commit" : "trigger" ;
#   endif

    tval_nxt_ms (due, delta);
    tval_min_tv (&l->time2check, due) ;

#   ifdef _DEBUG_IDLE_SCHED
    strcpy (b1, tval_print (&l->time2check));
    strcpy (b2, tval_print (due));
    logger (LOG_DEBUG,
            "Rescheduling at %s(%dms): time2check %s => %s, %s "
            "%s => %s", a0, delta, a1, b1, op, a2, b2);
#   endif
}


static
void register_child (LIST *l, unsigned slot, pid_t pid, int lifetime) {
    CHILD *c = l->children ;

    while (c)
        if (c->slot_inx == slot)
            goto update_child ;
        else
            c = c->next ;

    c = (CHILD *) ZMALLOC (CHILD) ;
    c->slot_inx = slot ;
    c->next     = l->children ;
    l->children = c ;

update_child: {
        struct _execute *e = &(*l->list.slot [slot])->t.execute ;
        if (lifetime > 0) {
            e->lifetime = (lifetime + 500) / 1000; /* ms -> s */
            e->time2die = time (0) + e->lifetime ;
        }
        else
            e->lifetime =  e->time2die = 0 ;               
        e->child    = pid ; /* same for the exec record and .. */
        c->child    = pid ; /* .. the pid registry */
    }
}


static
void update_children (LIST *l, int kill_all) {
    int n ;
    
    for (n = 0 ; n < DIM (try) ; n ++) {
        CHILD **C = &l->children ;
        int signal_sent = 0;
        struct timeval nap ;
        time_t now = time (0);

        /* clean up zombies */
        while (*C) {
            AVPAIR **Q = 0 ;
            CHILD *c = *C ;
            int inx ;

            pid_t qid = waitpid (c->child, 0, WNOHANG) ;

            /* does such an exec record still exist ? */
            if ((inx = c->slot_inx) < l->list.entries   &&
                (Q = l->list.slot [inx]) != 0           &&
                (*Q)                     != 0           &&
                (*Q)->a.trigger_type == execute_trigger &&
                (*Q)->t.execute.child    != 0           ) {

                time_t span = (*Q)->t.execute.lifetime ;
                time_t ttd  = (*Q)->t.execute.time2die ;

                /* check whether this process reached its life time */
                if (kill_all == 0 &&
                    (qid == 0 || (qid < 0 && errno != ECHILD))) {

                    if (now < ttd) {
                        /* check whether somebody set the time backwards */
                        if (now + span < ttd) {
                            if (verbosity & LIST_LOG_TIME2DIE)
                                logger (LOG_DEBUG, "Time leap: adjusting "
                                        "the life span for \"%s\" with "
                                        "process id %d",
                                        query_key_hlst ((void**)Q),
                                        c->child);
                            (*Q)->t.execute.time2die = now + span ;
                        }
                        C = &c->next ;
                        continue ;
                    }

                    /* kill it later unless the current time is reliable */
                    if (span == 0 || do_time_cont (l->sched, span) == 0) {
                        if (verbosity & LIST_LOG_TIME2DIE)
                            logger (LOG_DEBUG, "Time leap: killing "
                                    "\"%s\" with process id %d postponed",
                                    query_key_hlst ((void**)Q), c->child);
                        C = &c->next ;
                        continue ;
                    }
                }
            }
            else
                Q = 0 ;

            if (verbosity & LIST_LOG_TIME2DIE)
                logger (LOG_INFO, "Time to die for \"%s\": sending signal "
                        "%d to process %d",
                        ((inx >= 0 && Q) ? query_key_hlst ((void**)Q) : "*"),
                        try [n].signal, c->child);

            /* terminate sub process */
            if (kill (-c->child, try [n].signal) >= 0 || errno != ESRCH) {
                signal_sent ++ ;
                C = &c->next ;
                continue ;
            }

            if (Q) {
                /* reset process statistics */
                (*Q)->t.execute.child    = 0 ;
                (*Q)->t.execute.time2die = 0 ;
            }
                     
            /* unlink process reference */
            (*C) = c->next ;
            free (c);

            /* next entry */
            c = *C ;
        }

        /* nothing to do, anymore ? */
        if (signal_sent == 0) return ;
            
        /* wait some time for the next round */
        nap.tv_sec   =                       0 ;
        nap.tv_usec  = l->lazy.wait_after_kill ;
        nap.tv_usec *=           try [n].msfac ;
        select              (0, 0, 0, 0, &nap) ;
    }
}

static
const char *printav (int argc, const char **argv, char delim) {
    int i ;
    IOBUF ioenv;
    XZERO (ioenv);

    if (delim == 0)
        delim = ' ';

    data_append (&ioenv, argc ? argv [0] : "", 0);

    for (i = 1; i < argc && argv [i]; i++)
        data_printf (&ioenv, "%c%s", delim, argv [i]);
    
    return ioenv.data ; /* must be free()-d */
}

static
const char *xprintf (const char *format, ...) {
    IOBUF ioenv;
    va_list ap;
    va_start (ap, format);

    XZERO (ioenv);
    data_vprintf (&ioenv, format, ap);

    va_end (ap);
    return ioenv.data ; /* must be free()-d */
}


static
char* const* xenv (char* const* av, int *N, const char *str) {
    char **xv ;
    xv = (char**) zrealloc ((void*)av, (*N + 1) * sizeof (char*)) ;
    xv [*N] = (char*) str;
    if (str) (*N) ++ ;
    return (char* const*) xv ;
}


static
const char *envstr (const char *var) {
    const char *val = getenv (var) ;
    return val ? xprintf ("%s=%s", var, val) : 0 ;
}


static
unsigned run_exec_job (LIST         *l,
                       AVPAIR       *q,
                       const char *key,
                       int       event) {
    char* const* envp ;
    pid_t pid ;
    int n ;
    AVPAIR **R ;

    const char *triggerstatus = list_get_val(l, -1, "sys_nvramd.triggers_disabled", -1);
    if ((triggerstatus != NULL) && (strcmp(triggerstatus, "true") ==  0)) {
      logger(LOG_INFO, "trigger execution temporarily disabled.");
      return 0;
    }
    
    if (q->a.trigger_type != execute_trigger)
        return 0 ;

    if (q->t.execute.child) {
        time_t now = time (0) ;

#       ifdef _DEBUG_IDLE_SCHED
        char a1 [50], a2 [50], b1 [50] ;
        strcpy (a1, tval_print (&l->time2check));
        strcpy (a2, tval_print (&q->t.execute.time2run));
#       endif

        if (q->t.execute.time2die <= now) {
            /* next check is due: now */
            tval_nxt_ms (&l->time2check, 0) ;
        }
        else {
            /* set the time when this process expires */
            tval_nxt_ms (&q->t.execute.time2run,
                         (now - q->t.execute.time2die) * 1000);

            /* find out when the next job is due */
            tval_min_tv (&l->time2check, &q->t.execute.time2run);
        }
                    
#       ifdef _DEBUG_IDLE_SCHED
        strcpy (b1, tval_print (&l->time2check));
        logger (LOG_DEBUG, "Old job still running: next job "
                "at %s: time2check %s => %s", a2, a1, b1);
#       endif

        /* check whether we need to kill the running process */
        logger (LOG_ERR, "Exec failed: Process \"%s\" still running: "
                "time to kill not reached, yet", key);
        return 0 ;
    }

    if (q->t.execute.link_inx == -1                     ||
        q->t.execute.link_inx >= l->list.entries        ||
        (R = l->list.slot [q->t.execute.link_inx]) == 0 || *R == 0)
        R = 0 ;

    if ((pid = fork ()) > 0) {
        if (verbosity & LIST_LOG_EXEC) {
            const char *s = printav (q->t.execute.argc,q->t.execute.argv,0);
            logger (LOG_INFO, "Exec: \"%s\" [%s] -> %d", key, s, pid);
            free ((char*)s);
        }
        register_child (l, q->a.inx, pid, LIST_TRIGGER_LIFE_TIME);
        return 1 ; /* parent */
    }

    if (pid == -1) {
        const char *s = printav (q->t.execute.argc, q->t.execute.argv, 0);
        logger (LOG_ERR, "Exec failed: cannot fork process \"%s\" [%s]: %s",
                key, s, strerror (errno));
        free ((char*)s);
        return 0 ;
    }

    /* child - execute program */
    if (q->t.execute.chroot [0] && chroot (q->t.execute.chroot) < 0) {
        logger (LOG_ERR, "Exec failed: \"%s\" cannot chroot (%s): %s",
                key, q->t.execute.chroot, strerror (errno));
        exit (2);
    }
    if (q->t.execute.uid != -1 && setuid (q->t.execute.uid) < 0) {
        logger (LOG_ERR, "Exec failed: \"%s\" [%s] cannot setuid (%d): %s",
                key, q->t.execute.uid, strerror (errno));
        exit (2);
    }
    if (q->t.execute.gid != -1 && setgid (q->t.execute.gid) < 0) {
        logger (LOG_ERR, "Exec failed: \"%s\" cannot setgid (%d): %s",
                key, q->t.execute.gid, strerror (errno));
        exit (2);
    }

    envp = 0 ;
    n    = 0 ;
        
    envp = xenv (envp, &n, envstr ("PATH"));
    envp = xenv (envp, &n, envstr ("HOME"));
    envp = xenv (envp, &n, envstr ("SHELL"));
    envp = xenv (envp, &n, envstr ("PWD"));
    envp = xenv (envp, &n, envstr ("TMPDIR"));
    envp = xenv (envp, &n, envstr ("LD_LIBRARY_PATH"));
    envp = xenv (envp, &n, envstr ("LD_PRELOAD")) ;

    envp = xenv (envp, &n, xprintf ("NVRAM_VARIABLE=%s", key));
    envp = xenv (envp, &n, xprintf ("NVRAM_EVENT=%d",    event));
    envp = xenv (envp, &n, xprintf ("NVRAM_FLAGS=%#x",   q->a.flags));

    if (R) {
        envp = xenv (envp, &n, xprintf
                     ("NVRAM_LNKVARIABLE=%s",  query_key_hlst ((void**)R)));
        envp = xenv (envp, &n, xprintf
                     ("NVRAM_LNKFLAGS=%#x", (*R)->a.flags));
    }
    
    envp = xenv (envp, &n, 0);
      
    setsid ();

    execve (q->t.execute.argv [0], (char* const*) q->t.execute.argv, envp);

    logger (LOG_ERR, "Exec failed: \"%s\" cannot start [%s] env=(%s): %s",
            key, printav (q->t.execute.argc, q->t.execute.argv, 0),
            printav (n, (const char **)envp, ';'), strerror (errno));

    exit (2);
}

/* ----------------------------------------------------------------------- *
 * private: template list management
 * ----------------------------------------------------------------------- */

static
void update_template_list (LIST *l, const char *var, AVPAIR *q) {

    if (q->a.flags & action_template_prefix) {
        unsigned *t = (unsigned *) update_plst (l->template, var, 0);
        *t = q->a.inx ;
    }
    else
        remove_plst (l->template, var, 0);
}


static /* applies templates to the variable */
AVPAIR **apply_template2var (LIST         *l,
                             AVPAIR      **Q,
                             const char *var,
                             size_t      len) {
    int *T ;

    /* is not applicable to triggers and links */
    if (Q)
        switch ((*Q)->a.trigger_type) {
        case access_trigger:
            if (((*Q)->a.flags & action_template_prefix) == 0)
                /* not applicable to another template */
                break ;
        case execute_trigger:
        case symlink_trigger:
            return Q ;
        }

#   ifdef TRACE_VARIABLE
    if (trace_variable (var))
        logger (LOG_DEBUG, "apply_template2var: got %s", var);
#   endif
    
    if ((T = (int*) find_plst (l->template, var, len)) != 0) {
        /* get the template pointer */
        AVPAIR *t = *(AVPAIR **)l->list.slot [*T];
           
        /* get the extra size of the template record */
        int xl ;
        switch (t->a.trigger_type) {
        case execute_trigger: xl = strlen (t->t.execute.chroot) ; break ;
        case symlink_trigger: xl = strlen (t->t.symlink.ref)    ; break ;
        default:              xl = 0;
        }

        /* type cast the given variable/trigger or create a new one */
        Q = populate_trigger_slot (l, Q, var, t->a.trigger_type, xl);
        
        /* set up trigger entries as defined in the template */
        switch (t->a.trigger_type) {
        case execute_trigger:
            memcpy (&(*Q)->t.execute,
                    &t->t.execute, sizeof (t->t.execute) + xl - 1);
            /* copy arguments */
            if (t->t.execute.argv != 0) {
                int n = -1 ;
                (*Q)->t.execute.argv = (const char **) xmalloc
                    ((t->t.execute.argc + 1) * sizeof (const char*)) ;
                while (++ n < t->t.execute.argc)
                    (*Q)->t.execute.argv [n] = t->t.execute.argv [n]
                        ? zstrdup (t->t.execute.argv [n])
                        : 0
                        ;
                (*Q)->t.execute.argv [n] = 0 ;
            }
            break ;
        case symlink_trigger:
            memcpy (&(*Q)->t.symlink,
                    &t->t.symlink, sizeof (t->t.symlink) + xl - 1);
            break ;
        default:
            ;
        }

        (*Q)->a.flags = t->a.flags & ~action_template_prefix ;
        
        if ((*Q)->a.just_created)
            (*Q)->a.val = zstrdup (t->a.val);

        return Q ;
    }

    return 0;
}


/* ----------------------------------------------------------------------- *
 * private: hlst call back functions
 * ----------------------------------------------------------------------- */

static /* destructor call back for the hlst */
void av_free (void *state, void* item, char* key, unsigned keylen) {
    if (item)
        free_avpair ((AVPAIR*)item) ;
}


static /* destructor call back for the hlst */
void av_destroy (void *state, void* item, char* key, unsigned keylen) {
    LIST   *l = state ;
    AVPAIR *q = item ;

    if (q) {
        if (q->a.inx < l->list.entries) {
            if (q->a.inx + 1 == l->list.entries) {
                /* find the greatest active entry */
                while (-- l->list.entries &&
                       l->list.slot [l->list.entries - 1] == 0)
                    ;
                /* shrink slots list */
                l->list.slot = (AVPAIR***)
                    zrealloc (l->list.slot,
                              l->list.entries * sizeof (AVPAIR**));
            }
            else
                l->list.slot [q->a.inx] = 0 ;
        }
        av_free (0, item, 0, 0);
    }
}


static /* destructor call back for the hlst */
void fn_destroy (void *state, void* item, char* key, unsigned keylen) {
    VALFUN *q = item ;

    if (q) {
        if (q->fstate) {
            if (q->free)
                (*q->free) (q->fstate) ;
            else
                free (q->fstate) ;
        }
        free (q);
    }
}


static /* appends "var=value" string */
int av_collect (void *state, void *item, char *key, unsigned keylen) {
    AVPAIR *q = item ;

    if (q && (q->a.flags & action_volatile_data) == 0) {
        LIST *l = state ;

#       ifdef TRACE_VARIABLE
        if (trace_variable (key))
            logger (LOG_DEBUG, "av_collect: got %s", key);
#       endif

        if (key [keylen - 1] == 0) keylen -- ;
        data_append (&l->data, key, keylen);

        if (q->a.val) {
            char_append (&l->data, '=');
        
            switch (q->a.trigger_type) {
            case execute_trigger:
                dump_execute_fields (l, q, key);
                break;
            case symlink_trigger:
                dump_symlink_fields (l, q, key);
                break;
            case  access_trigger:
                dump_access_fields  (l, q);
            }

            if (q->a.val) {
                switch (q->a.val [0]) {
                case  execute_trigger:
                case  symlink_trigger:
                case   access_trigger:
                case escape_character:
                    char_append (&l->data, escape_character);
                }
                data_append (&l->data, q->a.val, 0);
            }
        }

        char_append (&l->data, '\0');
    }

    return 0;
}


static
int av_delayed_jobs (void *state, void *item, char *key, unsigned keylen) {
    LIST   *l = state ;
    AVPAIR *q = item ;

    if (q && q->a.trigger_type == execute_trigger) {

        if (tval_due (&q->t.execute.time2run)) {
            if (run_exec_job (l, q, key, action_delayed_run))
                // if (q->t.execute.child == 0)
                XZERO (q->t.execute.time2run) ;
        }
        else if (tval_used (&q->t.execute.time2run)) {
#           ifdef _DEBUG_IDLE_SCHED
            char a1 [50], a2 [50], b1 [50] ;
            strcpy (a1, tval_print (&l->time2check));
            strcpy (a2, tval_print (&q->t.execute.time2run));
#           endif

            /* find out when the next job is due */
            tval_min_tv (&l->time2check, &q->t.execute.time2run);

#           ifdef _DEBUG_IDLE_SCHED
            strcpy (b1, tval_print (&l->time2check));
            logger (LOG_DEBUG, "Next delayed job at %s: time2check %s => %s",
                    a2, a1, b1);
#           endif
        }
    }

    return 0;
}


static /* applies templates to the variable */
int av_template (void *state, void *item, char *key, unsigned len) {

    if (item) {
        LIST    *l = (LIST*) state ;
        AVPAIR  *q = (AVPAIR*) item ;
        apply_template2var (l, (AVPAIR **)l->list.slot [q->a.inx], key, len);
    }

    return 0;
}

/* ----------------------------------------------------------------------- *
 * private
 * ----------------------------------------------------------------------- */

static
unsigned store2nvram (LIST *l) {

    int n ;
    kill_space (&l->data);
    for_hlst_do (l->lookup, av_collect, l);
    char_append (&l->data, '\0');

    n = flash_write (l->data.data, l->data.top) ;
    kill_space (&l->data);

    if (n)
        XZERO (l->time2commit) ;
 
    return n;
}


static
void list_auto_commit (LIST *l, int noisy) {
    
    if (tval_used (&l->time2commit)) {
        int n = ms_tmo_tv (&l->time2commit);

#       ifdef _DEBUG_IDLE_SCHED
        struct timeval tmp ;
        char b1 [100];
        tval_nxt_ms (&tmp, 0);
        strcpy (b1, tval_print (&tmp));
        logger (LOG_DEBUG, "Autosave at %s: Save action at %s",
                b1, tval_print (&l->time2commit));
#       endif

        if (n > 0) {
            /* Consecutive rescheduling requests will result in an
               immediate call to this function. So we have to make
               sure that it is already time to save the NVRAM data. */
            update_scheduler (l, &l->time2commit, n) ;
            if (noisy && (verbosity & LIST_LOG_AUTOSTORE))
                logger (LOG_INFO,
                        "Autosave: Time not ready, %dms too early", n);
        }
        else if (store2nvram (l)) {
            if (verbosity & LIST_LOG_AUTOSTORE)
                logger (LOG_INFO,
                        "Automagically saved the NVRAM cache to flash!");
        }
        else {
            logger (LOG_ERR,
                    "Autosave: Error writing the list cache to flash!");
            update_scheduler (l, &l->time2commit, 0);
        }
    }
}


static /* return the left hand side from "varname= ..." */
const char *varname (LIST          *l,
                     const char  *var,
                     const char **Val) {

    const char *q = strchr ((char*)var, '=');

    if (q != 0) {
        kill_space  (&l->var);
        data_append (&l->var, var, q - var);
        var = l->var.data ;
    }
    if (Val)
        *Val = q ? q + 1 : "";

    return var ;
}


static
unsigned list_add_slot (LIST *l) {
    unsigned size, inx ;

    /* find a free slot */
    if ((size = query_hlst_size (l->lookup)) < l->list.entries) {
        unsigned inx = size ;

        /* do the upper part fist */
        do
            if (l->list.slot [inx] == 0)
                return inx;
        while (++ inx < l->list.entries);

        /* check the lower part */
        for (inx = 0; inx < size; inx ++)
            if (l->list.slot [inx] == 0)
                return inx;
    }

    /* create new slot */
    inx                = l->list.entries ++ ;
    l->list.slot       = (AVPAIR***) zrealloc
            (l->list.slot, l->list.entries * sizeof (AVPAIR**));
    l->list.slot [inx] = 0 ;

    DMALLOC_VERIFY (l);
    return inx;
}



static
void insert_or_update (LIST *l, const char *s, int mode) {

    const char *val, *var = varname (l, s, &val);

    DMALLOC_CHK_EXEC (l) ;

#   ifdef TRACE_VARIABLE
    if (trace_variable (var))
        logger (LOG_DEBUG, "insert_or_update: got %s", var);
#   endif

    if (val) {
        AVPAIR **Q, *q ;
        ACTION_TYPE f0, f1, f2 ;

        /* parse complex data */
        switch (f0 = val [0]) {
        case execute_trigger:
            if ((q = parse_execute (val + 1, s)) == 0)
                return ;
            break ;
        case symlink_trigger:
            if ((q = parse_symlink (val + 1, s)) == 0)
                return ;
            break ;
        case access_trigger:
            if ((q = parse_access (val + 1, s)) == 0)
                return ;
            break ;
        default:
            q = 0 ;
            f0 = no_trigger ;
        }

        /* set mode flags for populating the variable/trigger slot */
        if (mode & import_trigger_ok) {
            f1 = f0 ;
            f2 = (mode & import_volatile_ok) ? f0 : dont_touch_trg ;
        }
        else
            f1 = f2 = dont_touch_trg ;
            
        /* get slot or make new slot */
        Q = populate_trigger_slotx (l, 0, var, f0, f1, f2, 0);

        /* get flags from the slot */
        if ((*Q)->a.just_created) {
            /* noting at present - overwrite ok */
            mode = (import_trigger_ok |
                    import_flags_ok   |
                    import_value_ok   );
        }
        else if (((*Q)->a.flags & action_volatile_data) != 0 &&
                 (mode          & import_volatile_ok)   == 0)
            /* nothing to do with volatile entries */
            goto ready_to_go ;
        
        if (q) {
            if ((mode & import_trigger_ok) != 0)
                /* we have something different from no_trigger */
                switch (f0) {
                    size_t len ;
                case execute_trigger:
                    if ((len = strlen (q->t.execute.chroot)) != 0)
                        *Q = realloc_avpair (*Q, execute_trigger, len);
                    memcpy (&(*Q)->t.execute, &q->t.execute,
                            sizeof (q->t.execute) + len);
                    q->t.execute.argv = 0 ;
                    break ;

                case symlink_trigger:
                    if ((len = strlen (q->t.symlink.ref)) != 0)
                        *Q = realloc_avpair (*Q, symlink_trigger, len);
                    memcpy
                        ((char*)(*Q)->t.symlink.ref, q->t.symlink.ref, len);
                default:
                    ;
                }

            if ((mode & import_flags_ok) != 0) {
                (*Q)->a.flags = q->a.flags ;
                if (q->a.flags & action_template_prefix)
                    update_template_list (l, var, *Q) ;
            }
            val =  q->a.val ;
        } /* q != 0 */

        if ((mode & import_value_ok) != 0)
            (*Q)->a.val = alloc_value ((*Q)->a.val, val);
           
    ready_to_go:
        if ((*Q)->a.flags & action_function_ref)
            complain_on_missing_fn (l, var, (*Q)->a.val);
                    
        if (q)
            free_avpair (q);
        return ;
    }

    DMALLOC_VERIFY (l);
}


static
void flush_list_and_lookup_data (LIST *l) {
    update_children (l, 1);
    free  (l->list.slot);
    XZERO (l->list) ;
    flush_hlst (l->lookup, av_free, 0);
    flush_plst (l->template);
    kill_space (&l->data);
    kill_space (&l->var);
}


static
unsigned set_bits (int *var, int val, int msk) {

    if ((*var & msk) == val)
        return 0 ;

    *var &= ~ msk        ;
    *var |=  (msk & val) ;
    return 1 ;
}


static
unsigned vfy_access (AVPAIR *q, int msk) {

    if (msk == 0)
        return 1;

    if (q->a.trigger_type == no_trigger)
        return 0 ;
    
    return (q->a.flags & msk) == msk ;
}

/* ----------------------------------------------------------------------- *
 * private: exec
 * ----------------------------------------------------------------------- */

static
void list_exec (LIST *l, AVPAIR **Q, pid_t not_this_pid) {
    const char *var ;
    int x_flags ;

    if (Q == 0 || (*Q) == 0)
        return ;

    var     = query_key_hlst ((void**)Q) ;
    x_flags = trigger_map ((*Q)->a.flags) ;

    switch ((*Q)->a.trigger_type) {
        AVPAIR **R ;

    case symlink_trigger:
        if (x_flags == 0) {
            if (verbosity & LIST_LOG_SYMLINK)
                logger (LOG_INFO, "Link exec failed: Permission flags %#x "
                        "deny running \"%s\"", (*Q)->a.flags, var);
            return ;
        }

        if ((R = link_follow_avp (l, *Q, &x_flags)) == 0)
            return ;

        if (not_this_pid != -1 && (*R)->t.execute.child == not_this_pid) {
            if (verbosity & LIST_LOG_SYMLINK)
                logger (LOG_INFO, "Link exec %s->%s not invoked by client "
                        "with pid=%d", var, (*Q)->t.symlink.ref,
                        not_this_pid);
            return ;
        }

        (*R)->t.execute.link_inx = (*Q)->a.inx ;

        if (x_flags & action_force_immediate) {
            Q = R ;
            goto exec_now ;
        }

        if (verbosity & LIST_LOG_SYMLINK)
            logger (LOG_INFO,
                    "Link exec %s->%s delayed", var, (*Q)->t.symlink.ref);
        Q = R ;
        goto exec_later ;

    case execute_trigger:
        if (x_flags == 0) {
            if (verbosity & LIST_LOG_TRIGGER)
                logger (LOG_INFO, "Trigger exec failed: Permission flags "
                        "%#x deny running \"%s\"", (*Q)->a.flags, var);
            return ;
        }

        if (not_this_pid != -1 && (*Q)->t.execute.child == not_this_pid) {
            if (verbosity & LIST_LOG_TRIGGER)
                logger (LOG_INFO, "Trigger exec %s not invoked by pid=%d",
                        var, not_this_pid);
            return ;
        }

        (*Q)->t.execute.link_inx = -1 ;

        if (x_flags & action_force_immediate)
            goto exec_now ;

        if (verbosity & LIST_LOG_TRIGGER)
            logger (LOG_INFO, "Trigger exec delayed: \"%s\"", var);
        goto exec_later ;

    case access_trigger:
    case     no_trigger:
        return;
        
    default:
        logger (LOG_ERR, "Exec failed: No such type %#x with variable %s",
                (*Q)->a.trigger_type, query_key_hlst ((void**)Q));
    }

    return ;
    /*@NOTREACHED@*/

exec_now:
    update_children (l, 0) ;
    if (run_exec_job (l, *Q, var, action_force_immediate))
        return ;

exec_later:
    update_scheduler (l, &(*Q)->t.execute.time2run, l->lazy.trigger_wake_up);
}

/* ----------------------------------------------------------------------- *
 * Public: simple in-memory list manager
 * ----------------------------------------------------------------------- */

LIST *list_open (DO              *sched,
                 unsigned     size_hint,
                 unsigned  nvram_commit,
                 unsigned trigger_delay) {
    LIST *l = ZMALLOC (LIST);

    if ((l->sched = sched) == 0) {
        logger (LOG_ERR, "Uninitialised \"DO\" action scheduler");
        return 0;
    }

    l->lookup    = create_hlst (size_hint, av_destroy, l);
    l->function  = create_hlst         (0, fn_destroy, l);
    l->template  = create_plst (0);
    l->list.slot = (AVPAIR***) xmalloc (0) ;

    if ((l->lazy.nvram_commit = nvram_commit) == 0)
        l->lazy.nvram_commit = LIST_LAZY_NVRAM_COMMIT ;

    if ((l->lazy.trigger_wake_up = trigger_delay) == 0)
        l->lazy.trigger_wake_up = LIST_LAZY_TRIGGER_WAKE_UP ;

    return l;
}


unsigned list_fregister (LIST           *l,
                         const char *fname,
                         void      *fstate,
                         const char*(*s)(LIST*,void*,
                                         const char*,int,
                                         const char*),
                         void  (*r)(void*),
                         const char   *var,
                         unsigned    flags) {

    VALFUN *f, **F;

    /* check for clean up */
    if (s == 0) {
        if (var)
            delete_hlst (l->lookup, var, 0);
        return delete_hlst (l->function, fname, 0) != 0 ;
    }

    if ((F = (VALFUN **) find_hlst (l->function, fname, 0)) == 0) {
        F = (VALFUN **) make_hlst (l->function, fname, 0);
        f = *F = ZMALLOC (VALFUN);
    }
    else {
        fn_destroy (l, f = *F, 0, 0);
        XZERO (*f);
    }

    f->fstate = fstate ;
    f->set    =      s ;
    f->free   =      r ;

    if (var) {
        AVPAIR **Q = populate_trigger_slotx (l, 0, var,
                                             access_trigger, /* creating */
                                             dont_touch_trg, /* if exists */
                                             dont_touch_trg, /* if v'tile */
                                             0);
        flags         |= (action_function_ref | action_volatile_data);
        flags         &= acceptable_function_flags ;
        (*Q)->a.flags |= flags ;
        (*Q)->a.val = alloc_value ((*Q)->a.val, fname) ;
    }

    return 1;
}


unsigned list_load (LIST *l, const char *s, int mode) {

    if (mode & import_clear_ok)
        list_flush (l);

    if (s)
        while (*s != 0) {
            insert_or_update (l, s, mode);
            s += strlen (s) + 1;
            DMALLOC_CHK_EXEC (l) ;
        }

    if (mode & import_template_ok)
        for_hlst_do (l->lookup, av_template, l);

    return l->list.entries ;
}


unsigned list_store (LIST *l) {

    int n = store2nvram (l);

    if (n && (verbosity & LIST_LOG_AUTOSTORE))
        logger (LOG_INFO, "Wrote the NVRAM cache to flash!");    

    return n;
}


void list_flush (LIST *l) {
    flush_list_and_lookup_data (l);
    l->list.slot = (AVPAIR***) xmalloc (0) ;
    XZERO (l->time2commit) ;
}


void list_close (LIST *l) {
    flush_list_and_lookup_data (l);
    destroy_hlst (l->lookup);
    destroy_hlst (l->function);
    destroy_plst (l->template);
    free (l);
}


unsigned list_get_slot (LIST *l, const char *s) {
 
    if (s) {
        const char *var = varname (l, s, 0);
        AVPAIR **Q = (AVPAIR**) find_hlst (l->lookup, var, 0) ;
        if (Q && *Q)
            return (*Q)->a.inx + 1 ;
    }
    
    return 0 ;
}


// const char *list_var_query (LIST *l,  const char *var) {
//    AVPAIR **Q = (AVPAIR**) find_hlst (l->lookup, var, 0) ;
//    return Q ? query_key_hlst ((void**)Q) : 0;
// }


const char *list_get_var (LIST            *l,
                          int     privileged,
                          const char *filter,
                          unsigned       inx) {

    if (0 < inx && inx <= l->list.entries) {

        AVPAIR **Q = (AVPAIR **)l->list.slot [inx - 1] ;       
        DMALLOC_VERIFY (l);

        if (Q) {
            int flags, invert ;
            
            if ((flags = str2flags (&invert, filter)) == -1)
                return 0 ;

            if (privileged == 0 &&
                (*Q)       != 0 &&
                ((*Q)->a.flags & action_public_read) == 0) {
                if (verbosity & LIST_LOG_UNAUTH_ACCESS)
                    logger (LOG_INFO, "Unauthorised name read request for "
                            "the variable \"%s\" on slot %d",
                            query_key_hlst ((void**)Q), inx);
                return 0;
            }
            
            if (filter != 0) {
                if (invert == 0) {
                    if (((*Q)->a.flags & flags) == 0 ||
                        (*Q)->a.flags != 0           ||
                        flags         != 0)
                        return (char*)-1 ;
                }
                else /* invert != 0 */
                    if (((*Q)->a.flags & flags) != 0)
                        return (char*)-1 ;
            }
            return query_key_hlst ((void**)Q);
        }
    }

    DMALLOC_VERIFY (l);
    return 0 ;
}


const char *list_get_val (LIST        *l,
                          int privileged,
                          const char  *s,
                          pid_t      pid) {

    if (s) {
        const char *var = varname (l, s, 0);
        AVPAIR      **Q = (AVPAIR **) find_hlst (l->lookup, var, 0) ;
        
        if (Q == 0 || *Q == 0)
            return 0;
 
        if (privileged == 0 && ((*Q)->a.flags & action_public_read) == 0) {
            if (verbosity & LIST_LOG_UNAUTH_ACCESS)
                logger (LOG_INFO, "Unauthorised value read request "
                        "for the variable \"%s\"", s);
            return 0;
        }
        if (Q && ((*Q)->a.flags & action_enable_read) != 0)
            list_exec (l, Q, pid) ;

        if ((*Q)->a.flags & action_function_ref) {
            VALFUN *f, **F ;
            int flags ;
            if ((F = (VALFUN **) find_hlst
                 (l->function, (*Q)->a.val, 0)) == 0 || (f = *F) == 0)
                return 0 ;
            flags = (*Q)->a.flags & acceptable_function_flags ;
            return (*f->set) (l, f->fstate, var, flags, 0);
        }

        return (*Q)->a.val ;
    }
    
    DMALLOC_VERIFY (l);
    return 0 ;
}


unsigned list_avp_replace (LIST         *l,
                           int  privileged,
                           const char   *s, /* var name */
                           const char *val, /* its value */
                           pid_t       pid,
                           int create_only) {

    unsigned inx, volatile_data = 0, changes = 0 ;
    const char *var, *u ;
    VALFUN *f, **F ;
    AVPAIR **Q ;

    var = varname (l, s, 0);
    Q   = (AVPAIR **) find_hlst (l->lookup, var, 0) ;

    if (create_only && Q != 0)
        return 0;

    if (Q == 0 && val) {
        if (privileged == 0)
            goto access_denied ;
        /* check for a template set up if that is a new variable */
        Q = apply_template2var (l, 0, var, 0);
    }

    if (Q) {

        if (*Q) {

            if (privileged == 0 && vfy_access (*Q, action_public_write) == 0)
                goto access_denied ;

            inx = (*Q)->a.inx + 1 ;
            
            if (val) {
                if ((*Q)->a.val == 0) {
                    (*Q)->a.val = alloc_value ((*Q)->a.val, val);
                    changes ++ ;
                }
                else if ((*Q)->a.flags & action_function_ref) {
                    int flags ;
                    if ((F = (VALFUN **) find_hlst
                         (l->function, (*Q)->a.val, 0)) == 0 ||
                        (f = *F) == 0)
                        return 0 ;
                    flags = (*Q)->a.flags & acceptable_function_flags ;
                    if ((u = (*f->set)
                         (l, f->fstate, var, flags, val)) != 0) {
                        if (u == (const char *)-1)
                            return 0;
                        changes ++ ;
                    }
                }
                else if (strcmp (val, (*Q)->a.val) != 0) {
                    (*Q)->a.val = alloc_value ((*Q)->a.val, val);
                    changes ++ ;
                }
            }

            /* set it that early - no flag anymore when deleted */
            if (((*Q)->a.flags & action_volatile_data) != 0)
                volatile_data = 1 ;
        }
        else {
            if (privileged == 0)
                goto access_denied ;
            inx = 0 ;
        }

        if (val == 0) {
            /* if Q is a link, then we probably need to trigger, a
               real trigger would not work as we will loose the
               scheduler information when the trigger gets removed */
            if (Q && ((*Q)->a.flags & action_enable_write) != 0 &&
                (*Q)->a.trigger_type == symlink_trigger)
                list_exec (l, Q, pid) ;
            if (delete_hlst (l->lookup,
                             query_key_hlst    ((void**)Q),
                             query_keylen_hlst ((void**)Q)) == 0) 
                Q = 0, changes ++ ;
        }
    }
    else if (val) { /* here: Q == 0 */

        if (privileged == 0)
            goto access_denied ;

        Q = populate_trigger_slot (l, Q, var, no_trigger, 0);
        (*Q)->a.val = alloc_value (0, val) ;
        inx = (*Q)->a.inx + 1 ;
        changes ++ ;
    }
    else
        return 0 ;

    if (changes) {
        if (Q && ((*Q)->a.flags & action_enable_write) != 0)
            list_exec (l, Q, pid) ;
        if (volatile_data == 0) {
            /* don't save that variable and data necessarily */
            update_scheduler (l, &l->time2commit, l->lazy.nvram_commit);
            commit_scheduler (l) ;
        }
    }


    return inx;

 access_denied:
    if (verbosity & LIST_LOG_UNAUTH_ACCESS)
        logger (LOG_INFO, "Unauthorised variable modification "
                "request for the variable \"%s\"", s);

    return 0;
}


unsigned list_exe_replace (LIST            *l,
                           int     privileged,
                           const char      *s,
                           const char      *f,
                           uid_t          uid,
                           gid_t          gid,
                           const char   *chrt,
                           unsigned      argc,
                           const char  **argv,
                           unsigned     flags) {

    unsigned changes = 0, f2, inx ;
    const char *var ;
    AVPAIR **Q ;

    if ((f2 = str2flags (0, f)) == -1)
        return 0 ;

    flags |= f2 ;
    flags &= acceptable_exec_flags ;
    var    = varname (l, s, 0);
    Q      = (AVPAIR **) find_hlst (l->lookup, var, 0) ;

    if (argv == 0) {

        if (Q == 0 || ((*Q) && (*Q)->a.trigger_type != execute_trigger))
            return 0;
        
        if (privileged == 0 && vfy_access (*Q, action_public_modify) == 0)
            goto access_denied ;

        inx = (*Q) ? (*Q)->a.inx + 1 : 0 ;

        if ((*Q) == 0 || (*Q)->a.val == 0) { /* delete variable */
            if (delete_hlst (l->lookup,
                             query_key_hlst    ((void**)Q),
                             query_keylen_hlst ((void**)Q)) == 0)
                /* Q = 0, */ changes ++ ;
        }
        else {
            *Q = realloc_avpair (*Q, no_trigger, 0);
            changes ++ ;
        }
    }
    else {
        unsigned n ;
        size_t clen;
        
        if (privileged == 0 &&
            (Q == 0 || vfy_access (*Q, action_public_modify) == 0))
            goto access_denied ;

        if (Q)
            free_avpair_argv (*Q);

        clen = chrt ? strlen (chrt) : 0 ;

        Q = populate_trigger_slot (l, Q, var, execute_trigger, clen);
        
        inx = (*Q)->a.inx + 1 ; 

        (*Q)->t.execute.uid  =                                uid ;
        (*Q)->t.execute.gid  =                                gid ;
        (*Q)->a.flags        =                              flags ;
        (*Q)->t.execute.argc =                               argc ;
        (*Q)->t.execute.argv =
            (const char **) xmalloc ((argc + 1) * sizeof (char*)) ;

        if (clen) /* populate_trigger_slot () => chroot [strlen] = '\0' */
            memcpy ((char*)(*Q)->t.execute.chroot, chrt, clen);

        for (n = 0; n < argc && argv [n]; n ++)
            (*Q)->t.execute.argv [n] = zstrdup (argv [n]);
        (*Q)->t.execute.argv [n] = 0 ;

        if (flags & action_function_ref)
            complain_on_missing_fn (l, var, (*Q)->a.val);
        
        changes ++ ;
    }

    if (changes) {
        update_scheduler (l, &l->time2commit, l->lazy.nvram_commit);
        commit_scheduler (l) ;
    }

    return inx ;
    
 access_denied:
    if (verbosity & LIST_LOG_UNAUTH_ACCESS)
        logger (LOG_INFO, "Unauthorised trigger modification "
                "request for the variable \"%s\"", s);

    return 0;
#   undef accptable_flags
}


unsigned list_lnk_replace (LIST         *l,
                           int  privileged,
                           const char   *s,
                           const char   *f,
                           const char *lnk,
                           unsigned  flags) {

    unsigned changes = 0, f2, inx ;
    const char *var ;
    AVPAIR **Q ;
    
    if ((f2 = str2flags (0, f)) == -1)
        return 0 ;

    flags |= f2 ;
    flags &= acceptable_link_flags;
    var    = varname (l, s, 0);
    Q      = (AVPAIR **) find_hlst (l->lookup, var, 0) ;
 
    if (lnk == 0) {

        if (Q == 0 || ((*Q) && (*Q)->a.trigger_type != symlink_trigger))
            return 0;

        if (privileged == 0 && vfy_access (*Q, action_public_modify) == 0)
            goto access_denied ;

        inx = (*Q) ? (*Q)->a.inx + 1 : 0 ;

        if ((*Q) == 0 || (*Q)->a.val == 0) {  /* delete variable */
            if (delete_hlst (l->lookup,
                             query_key_hlst    ((void**)Q),
                             query_keylen_hlst ((void**)Q)) == 0)
                /* Q = 0, */ changes ++ ;
        }
        else {
            *Q = realloc_avpair (*Q, no_trigger, 0);
            changes ++ ;
        }
    }
    else {

        size_t len;

        if (privileged == 0 &&
            (Q == 0 || vfy_access (*Q, action_public_modify) == 0))
            goto access_denied ;

        len = strlen (lnk) ;
        
        Q = populate_trigger_slot (l, Q, var, symlink_trigger, len);
        
        inx = (*Q)->a.inx + 1 ;
        
        (*Q)->a.flags = flags ;

        if (flags & action_function_ref)
            complain_on_missing_fn (l, var, (*Q)->a.val);

        /* populate_trigger_slot () => ref [strlen] = '\0' */
        memcpy ((char*)(*Q)->t.symlink.ref, lnk, len);

        changes ++ ;
    }
    
    if (changes) {
        update_scheduler (l, &l->time2commit, l->lazy.nvram_commit);
        commit_scheduler (l) ;
    }

    return inx;
 
 access_denied:
    if (verbosity & LIST_LOG_UNAUTH_ACCESS)
        logger (LOG_INFO, "Unauthorised link modification "
                "request for the variable \"%s\"", s);

    return 0;
}



unsigned list_access_replace (LIST         *l,
                              int  privileged,
                              const char   *s,
                              const char   *f,
                              unsigned  flags) {

    unsigned changes = 0, inx ;
    const char *var ;
    AVPAIR **Q ;

    if (f) {
        unsigned f2 ;
        if ((f2 = str2flags (0, f)) == -1)
            return 0 ;
        flags |= f2 ;
        flags &= acceptable_access_flags ;
    }

    var = varname (l, s, 0);
    Q   = (AVPAIR **) find_hlst (l->lookup, var, 0) ;

    if (flags && Q == 0) {
        if (privileged == 0)
            goto access_denied ;
        /* check for a template set up if that is a new variable */
        Q = apply_template2var (l, 0, var, 0);
    }

    if (flags == 0) {

        /* delete access restrictions */
        if (Q == 0 || *Q == 0)
            return 0;

        if (privileged == 0 && vfy_access (*Q, action_public_modify) == 0)
            goto access_denied ;

        inx = (*Q)->a.inx + 1 ;

        if ((*Q)->a.trigger_type == access_trigger) {
            /* remove plain access restrictions record */
            *Q = realloc_avpair (*Q, no_trigger, 0);
            changes ++ ;
        }
    }
    else {
        if (privileged == 0 &&
            (Q == 0 || vfy_access (*Q, action_public_modify) == 0))
            goto access_denied ;

        Q = populate_trigger_slot (l, Q, var, access_trigger,0);

        inx = (*Q)->a.inx + 1 ;

        if (flags & action_function_ref)
            complain_on_missing_fn (l, var, (*Q)->a.val);

        changes ++ ;
    }

    if (changes + set_bits (&(*Q)->a.flags, flags,acceptable_access_flags)) {
        update_scheduler (l, &l->time2commit, l->lazy.nvram_commit);
        commit_scheduler (l) ;
    }

    return inx;

 access_denied:
    if (verbosity & LIST_LOG_UNAUTH_ACCESS)
        logger (LOG_INFO, "Unauthorised acccess control modification "
                "request for the variable \"%s\"", s);

    return 0;
}



const char **list_get_trigger (LIST *l, const char *s, ACTION_TYPE match) {

    if (s) {
        const char *var = varname (l, s, 0);
        AVPAIR      **Q = (AVPAIR **) find_hlst (l->lookup, var, 0) ;

        if (Q && *Q && match == (*Q)->a.trigger_type)
            switch ((*Q)->a.trigger_type) {
            case execute_trigger:
                return argv_execute_fields (l, *Q);
            case symlink_trigger:
                return argv_symlink_fields (l, *Q);
            case access_trigger:
                return argv_access_fields  (l, *Q);
            }
    }
    
    DMALLOC_VERIFY (l);
    return 0 ;
}


unsigned list_used_slots (LIST *l) {
    return query_hlst_size (l->lookup) ;
}


unsigned list_num_slots (LIST *l) {
    return l->list.entries ;
}


void list_do_events (LIST *l) {

    /* reset for the next check */
    XZERO (l->time2check);

    /* save data */
    list_auto_commit (l, 1 /* noisy */);
    
    /* process table clean up */
    update_children (l, 0);

    /* run unfinished jobs and prepare for the next */
    for_hlst_do (l->lookup, av_delayed_jobs, l);

    /* save data */
    list_auto_commit (l, 0 /* silent */);

    /* finish up */
    commit_scheduler (l);
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
