/*
 * nvramd - active data base server and flash memory cache
 *
 * Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA
 *
 * Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-Isenburg, Germany
 *
 * $Id: nvramd.c 3671 2006-11-28 19:49:27Z jordan $
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>

#ifdef STDC_HEADERS
# include <stdlib.h>
# include <stddef.h>
#else
# ifdef HAVE_STDLIB_H
#  include <stdlib.h>
# endif
#endif

#ifdef HAVE_STRING_H
# if !defined (STDC_HEADERS) && defined (HAVE_MEMORY_H)
#  include <memory.h>
# endif
# include <string.h>
#endif

#ifdef HAVE_UNISTD_H
# include <unistd.h>
# include <sys/types.h>
# else
# ifdef HAVE_SYS_TYPES_H
#  include <sys/types.h>
# endif
#endif

#ifdef TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# ifdef HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#ifdef HAVE_ERRNO_H
# include <errno.h>
#endif
#ifdef HAVE_CTYPE_H
# include <ctype.h>
#endif
#ifdef HAVE_SIGNAL_H
# include <signal.h>
#endif
#ifdef HAVE_FCNTL_H
# include <fcntl.h>
#endif
#ifdef HAVE_SYS_STAT_H
# include <sys/stat.h>
#endif
#ifdef HAVE_SETJMP_H
# include <setjmp.h>
#endif
#ifdef HAVE_SYS_SOCKET_H
# include <sys/socket.h>
#endif
#ifdef HAVE_NETINET_IN_H
# include <netinet/in.h>
#endif
#ifdef HAVE_GRP_H
# include <grp.h>
#endif
#ifdef HAVE_ARPA_INET_H
# include <arpa/inet.h>
#endif
#ifdef HAVE_NETDB_H
# include <netdb.h>
#endif

#ifdef HAVE_OPENSSL
#include <openssl/ssl.h>
#endif

#define _GNU_SOURCE
#include <getopt.h>

#include "nvalloc.h"
#include "varstr.h"
#include "comhandler.h"
#include "settings.h"
#include "logger.h"
#include "ctrl.h"

#ifdef DISCARD_HTTP
# include "do.h"
# include "util.h"
#else
# include "http-server.h"
#endif

/* ----------------------------------------------------------------------- *
 * variables and definitions
 * ----------------------------------------------------------------------- */

#ifdef _DEBUG
#define _MAIN_DEBUG
#endif

typedef
struct _desc {
    DO      *loop ;
    COMM    *comm ;
    LIST    *lst ;
    // LOOKUP *lookup ;
} SESSION ;


extern char *optarg;
extern int optind, opterr, optopt;

static jmp_buf go_here, go_there;     /* jump buffer for long jumps */

/* ----------------------------------------------------------------------- *
 * Fiddling around with options ...
 * ----------------------------------------------------------------------- */

#define __arg1of4(a,b,c,d)    a
#define __arg2of4(a,b,c,d)    b
#define __arg3of4(a,b,c,d)    c
#define __arg4of4(a,b,c,d)    d
#define __str(x)              # x
#define str(x)                __str (x)

/* I am fed up with option name and option key management using strings
   and letters at several places - so keep them together -- jordan */

#define opt_name(x)         x (__arg1of4)
#define opt_key(x)          x (__arg2of4)
#define opt_argtype(x)      x (__arg3of4)
#define opt_info(x)         x (__arg4of4)

#define OPT_VERSION(x)      x ("version",          'V', 0,           "print version and exit")
#define OPT_VERBOSE(x)      x ("verbose",          'v', 0,           "print extra diagnostic messages")
#define OPT_HELP(x)         x ("help",             'h', 0,           "instead of a manual page")
#define OPT_SYSLOG(x)       x ("syslog",           's', "FACILITY",  "use syslog with the given facility")
#define OPT_NOTIMEPFX(x)    x ("no-timestamp",     'n', 0,           "do not prefix log data by a time stamp")
#define OPT_DETACH(x)       x ("detach",           'd', 0,           "daemon mode, put yourself in the background")
#define OPT_PID_FILE(x)     x ("pid-file",         'p', "FILE",      "save the process pid")
#define OPT_CHROOT(x)       x ("chroot",           'r', "DIR",       "chroot to DIR after initialisation")
#define OPT_GROUPID(x)      x ("groupid",          'g', "GID",       "allow this group to connect and send admin requests")
#define OPT_NO_DEFAULTS(x)  x ("no-defaults",      'D', 0,           "do not load NVRAM defaults upon start up")
#define OPT_CACHE_FLUSH(x)  x ("cache-write-back", 'W', "TIMEOUT",   "update flash TIMEOUT ms after the last change (default: " str (LAZY_NVRAM_COMMIT) "ms)")
#define OPT_TRIG_DELAY(x)   x ("delayed-trigger",  'X', "TIMEOUT",   "run delayed triggers TIMEOUT ms later (default: " str (LAZY_TRIGGER_WAKE_UP) "ms)")
#ifndef DISCARD_HTTP
#define OPT_HTTP_PORT(x)    x ("http-port",        'H', "PORT",      "port to listen for remote admin access (default: " str (HTTP_CONFIG_PORT) " )")
#define OPT_HTTP_BIND(x)    x ("http-bind",        'B', "ADDRESS",   "addres to listen on remote admin access requests (default: all local addresses)")
#define OPT_HTTP_SLOTS(x)   x ("http-slots",       'N', "NUM",       "number of sessions concurrently served (default: " str (HTTP_CONFIG_SLOTS) ")")
#endif /* DISCARD_HTTP */
#define OPT_NULL(x)         x (0,               0 , 0,            0)


#define LONG_OPTION_RECORD(x,y)  {opt_name (x), y, 0, opt_key (x)}
static const
struct option long_options [] = {
    LONG_OPTION_RECORD (OPT_VERSION,      0),
    LONG_OPTION_RECORD (OPT_VERBOSE,      0),
    LONG_OPTION_RECORD (OPT_NOTIMEPFX,    0),
    LONG_OPTION_RECORD (OPT_CACHE_FLUSH,  0),
    LONG_OPTION_RECORD (OPT_TRIG_DELAY,   0),
#   ifndef DISCARD_INFO
    LONG_OPTION_RECORD (OPT_HELP,         0),
#   endif /* DISCARD_INFO */
    LONG_OPTION_RECORD (OPT_GROUPID,      1),
    LONG_OPTION_RECORD (OPT_DETACH,       0),
    LONG_OPTION_RECORD (OPT_CHROOT,       1),
    LONG_OPTION_RECORD (OPT_PID_FILE,     1),
    LONG_OPTION_RECORD (OPT_SYSLOG,       1),
    LONG_OPTION_RECORD (OPT_NO_DEFAULTS,  0),
#   ifndef DISCARD_HTTP
    LONG_OPTION_RECORD (OPT_HTTP_PORT,    1),
    LONG_OPTION_RECORD (OPT_HTTP_BIND,    1),
    LONG_OPTION_RECORD (OPT_HTTP_SLOTS,   1),
#   endif /* DISCARD_HTTP */
    LONG_OPTION_RECORD (OPT_NULL,         0)
};
#undef LONG_OPTION_RECORD


static const
char short_options [] = {
    '+',
    opt_key (OPT_VERSION),
    opt_key (OPT_VERBOSE),
    opt_key (OPT_NOTIMEPFX),
#   ifndef DISCARD_INFO
    opt_key (OPT_HELP),
#   endif /* DISCARD_INFO */
    opt_key (OPT_CACHE_FLUSH),  ':',
    opt_key (OPT_TRIG_DELAY),   ':',
    opt_key (OPT_GROUPID),      ':',
    opt_key (OPT_DETACH),
    opt_key (OPT_CHROOT),       ':',
    opt_key (OPT_PID_FILE),     ':',
    opt_key (OPT_SYSLOG),       ':',
    opt_key (OPT_NO_DEFAULTS),
#   ifndef DISCARD_HTTP
    opt_key (OPT_HTTP_PORT),    ':',
    opt_key (OPT_HTTP_BIND),    ':',
    opt_key (OPT_HTTP_SLOTS),   ':',
#   endif /* DISCARD_HTTP */
    opt_key (OPT_NULL)
} ;

/* ----------------------------------------------------------------------- *
 * Timeout signal handler
 * ----------------------------------------------------------------------- */
static
void di () {
    signal (SIGHUP,  SIG_IGN);
    signal (SIGINT,  SIG_IGN);
    signal (SIGQUIT, SIG_IGN);
    signal (SIGILL,  SIG_IGN);
    signal (SIGUSR1, SIG_IGN);
    signal (SIGUSR2, SIG_IGN);
    signal (SIGABRT, SIG_IGN);
    signal (SIGTERM, SIG_IGN);
}

static void s_go_here  (int n) {
    if (verbosity & MAIN_LOG_INTERRUPT)
        logger (LOG_INFO, "%s: Got signal %d: continuing ...", progname, n);
    longjmp (go_here, 1);
}
static void s_go_interrupt (int n) {
    di ();
    if (verbosity & MAIN_LOG_INTERRUPT)
        logger (LOG_INFO, "%s: INTERRUPT(%d): terminating ...", progname, n);
    longjmp (go_there, 1);
}
static void s_go_endup (int n) {
    di ();
    if (verbosity & MAIN_LOG_INTERRUPT)
        logger (LOG_INFO, "%s: Got signal %d: terminating ...", progname, n);
    longjmp (go_there, 1);
}

/* ----------------------------------------------------------------------- *
 * call back functions
 * ----------------------------------------------------------------------- */

static
void idle (void *dsc) {
    SESSION *s = dsc ;

    if (verbosity & MAIN_LOG_IDLE_LOOPING)
        logger (LOG_INFO, "%s: Idle/cleanup/exec handler.", progname);

    list_do_events (s->lst);
}

/* ----------------------------------------------------------------------- *
 * helpers
 * ----------------------------------------------------------------------- */

static
void cpry (void) {
    logger (LOG_INFO,  "NVRAMD - nvram configuration management server");
    logger (LOG_INFO,  "Copyright (c) 2006 D-Link (Europe) Ltd., London, UK"
		       " -- LGPL licensed");
    logger (LOG_INFO, ("Version " VERSION
                       " -- Jordan Hrycaj <jordan@maxina.de>"));
#   ifdef HAVE_OPENSSL
    logger (LOG_INFO,  "Crypto support: %s", SSLeay_version (SSLEAY_VERSION));
#   endif
}

#ifndef DISCARD_INFO
static
void usage_line (const char indent,
                 const  char *name,
                 char          key,
                 const char   *arg,
                 const char  *info) {
    char longopt [100], format [30] ;
    strcpy (longopt, name) ;
    if (arg) { strcat (longopt, "="); strcat (longopt, arg); }
    sprintf (format, "%%%us-%%c, --%%-17s %%s\n", indent);
    fprintf (stderr, format, "", key, longopt, info);
}
#endif /* DISCARD_INFO */


static
#ifdef DISCARD_INFO
void _usage (const char *opt)
#define usage(a,b) _usage (b)
#else
void usage (const char *info, const char *opt)
#endif
{
#   ifdef DISCARD_INFO
    if (opt) fprintf (stderr, "Option error: %s\n", opt);

#   else
    if (info != 0) {
        fprintf (stderr, "\n%s: ERROR: %s", progname, info);
        if (opt) fprintf (stderr, " (option arg was %s)", opt);
        fprintf (stderr, "\n");
    }
#   endif

    fprintf (stderr, "\nUsage: %s [options]\n", progname);

#   ifndef DISCARD_INFO
#   define SHOW_OPTION(indent,opt) usage_line (indent,            \
                                               opt_name    (opt), \
                                               opt_key     (opt), \
                                               opt_argtype (opt), \
                                               opt_info    (opt)  )
    fputs ("\nGeneral options:", stderr);
    SHOW_OPTION ( 2, OPT_NO_DEFAULTS);
    SHOW_OPTION (18, OPT_CACHE_FLUSH);
    SHOW_OPTION (18, OPT_TRIG_DELAY);
    SHOW_OPTION (18, OPT_HELP);

    fputs ("\nLogging options:", stderr);
    SHOW_OPTION ( 2, OPT_VERBOSE);
    SHOW_OPTION (18, OPT_SYSLOG);
    SHOW_OPTION (18, OPT_NOTIMEPFX);

    fputs ("\nProcess control:", stderr);
    SHOW_OPTION ( 2, OPT_DETACH);
    SHOW_OPTION (18, OPT_CHROOT);
    SHOW_OPTION (18, OPT_PID_FILE);
    SHOW_OPTION (18, OPT_GROUPID);

#   ifndef DISCARD_HTTP
    SHOW_OPTION (18, OPT_HTTP_PORT);
    SHOW_OPTION (18, OPT_HTTP_BIND);
    SHOW_OPTION (18, OPT_HTTP_SLOTS);
#   endif /* DISCARD_HTTP */

#   undef SHOW_OPTION
#   endif /* DISCARD_INFO */

    exit (1);
}


#ifndef DISCARD_INFO
static
void help (void) {
    fputs ("\n", stderr);
    syslog_init (0, 0); /* no timestamps with the introduction */
    cpry ();
    fputs ("\n"
           "\n ...\n"
           "",  stderr);
}
#endif /* DISCARD_INFO */



#ifndef DISCARD_HTTP
static
ip4addr_t hostip (const char *s) {
    
    extern int h_errno;
    int       e ;
    ip4addr_t ip ;

    struct hostent* h = gethostbyname (s) ;

    if (h != 0) {
        struct in_addr sin_addr;
        sin_addr.s_addr = *(in_addr_t*)h->h_addr;
        return ntohl (sin_addr.s_addr);
    }

    e = h_errno ;
    if ((ip = ntohl (inet_addr (s))) != 0 && ip != -1)
        return ip;
    errno = e ;
    return 0;
}
#endif /* DISCARD_HTTP */

/* ----------------------------------------------------------------------- *
 * MAIN
 * ----------------------------------------------------------------------- */

#ifndef _MAIN_DEBUG
#undef main /* in case some debugging redirection applied */
#endif

int main (int argc, char *argv[]) {

    int n, c, option_inx ;

    char         *new_root =  0 ;
    char        *facility  =  0 ;
    int            timepfx =  1 ;
    int             pid_fd = -1 ;
    int             detach =  0 ;
    int            verbose =  0 ;
    int       dont_defload =  0 ;
    int       idle_timeout =  MAIN_IDLE_TIMEOUT ;
    unsigned       set_gid =  0 ;
    CTRL             *ctrl =  0 ;
    unsigned  nvram_commit =  0 ;
    unsigned trigger_delay =  0 ;
    
#   ifndef DISCARD_HTTP
    HTTP             *http =  0 ;
    unsigned         slots =  HTTP_CONFIG_SLOTS ;
    unsigned       no_http =  0 ;
    ip4addr_t       ipaddr =  0 ;
    unsigned          port = HTTP_CONFIG_PORT ;
#   endif /* DISCARD_HTTP */
    

    SESSION s ;
    XZERO  (s);

#   ifdef WITH_DMALLOC
    dmalloc_debug_setup ("debug=0x4f47d03,log=/tmp/dmalloc.log");
#   endif

    /* progname ::= only the last path segment */
    if ((progname = strrchr (argv [0], '/')) == 0)
        progname = argv [0] ;
    else
        ++ progname ;
    
    while ((c = getopt_long (argc, argv, short_options, 
                             long_options, &option_inx)) != -1)
        switch (c) {
            struct group *grp;
#       ifndef DISCARD_HTTP
        case opt_key (OPT_HTTP_SLOTS):
            if (parse_unsigned (& slots, optarg, 0) == 0 ||
                (slots & ~0xffff) != 0)
                usage ("illegal number of slots", optarg);
            if (slots == 0)
                no_http = 1 ;
            continue ;
        case opt_key (OPT_HTTP_PORT):
            if (parse_unsigned (& port, optarg, 0) == 0 ||
                (port & ~0xffff) != 0)
                usage ("illegal port number", optarg);
            no_http = 0 ;
            continue ;
        case opt_key (OPT_HTTP_BIND):
            if ((ipaddr = hostip (optarg)) == 0)
                usage ("illegal ip addess", optarg);
            no_http = 0 ;
            continue ;
#       endif /* DISCARD_HTTP */
        case opt_key (OPT_CACHE_FLUSH):
            if (parse_unsigned (& nvram_commit, optarg, 0) == 0)
                usage ("illegal cache flush timeout seconds", optarg);
            continue ;
        case opt_key (OPT_TRIG_DELAY):
            if (parse_unsigned (& trigger_delay, optarg, 0) == 0)
                usage ("illegal trigger delay seconds", optarg);
            continue ;
        case opt_key (OPT_NO_DEFAULTS):
            dont_defload = 1 ;
            continue ;
        case opt_key (OPT_GROUPID):
            if ((parse_unsigned (& set_gid, optarg, 0) == 0 ||
                 (grp = getgrgid (set_gid)) == 0) && 
                (grp = getgrnam (optarg)) == 0)
                usage ("group name does not exist", optarg);
            set_gid = grp->gr_gid;
            continue ;
        case opt_key (OPT_CHROOT):
            new_root = optarg;
            continue ;
        case opt_key (OPT_NOTIMEPFX):
            timepfx = 0 ;
            continue ;
        case opt_key (OPT_DETACH):
            detach = 1 ;
            continue ;
        case opt_key (OPT_PID_FILE):
            unlink (optarg); /* don't be tricked by a symlink */
            if ((pid_fd = open (optarg, O_CREAT|O_EXCL|O_WRONLY, 0644)) < 0)
                usage ("cannot open or create pid file", optarg);
            continue ;
        case opt_key (OPT_SYSLOG):
            facility = optarg ;
            continue ;
        case opt_key (OPT_VERSION):
            cpry ();
            exit (0);
        case opt_key (OPT_VERBOSE):
            verbose = -1 ;
            continue ;
#       ifndef DISCARD_INFO
        case '?':
            usage (0, "?") ; /* no return */
        case opt_key (OPT_HELP):
            help (); /* and usage */
#       endif /* DISCARD_INFO */
        default:
            usage (0,0) ; /* no return */
        }

    if (optind < argc) {
#       ifdef DISCARD_INFO
        usage (0,0) ; /* no return */
#       else
        fprintf
            (stderr, "%s: Unexpected non-optional arguments: ", progname);
        while (optind < argc)
            fprintf (stderr, "%s ", argv [optind++]);
        fprintf (stderr, "\n");
        exit (2);
#       endif
    }

    /* we must use syslog when running in the background */
    if (detach && facility == 0) facility = "daemon" ;

    /* initialise logging subsystem */
    if (facility) {
        int devnull ;
        /* ready for closing stdin/stdout/stderr */
        if ((devnull = open ("/dev/null", O_RDWR)) < 0) {
            usage ("Cannot open /dev/null: %s", strerror (errno));
            exit (2);
        }
        if (syslog_init (facility, timepfx) == 0)
            usage ("cannot open the syslog facility", facility);
        /* detach, run in the backgrond */
        if (detach) {
            if (fork ()) exit (0);
            setsid ();
        }
        for (n = 0; n < 3; n ++) {
            close (n) ;
            if (dup2 (devnull, n) == n)
                continue ;
            logger (LOG_ERR, "Cannot dup /dev/null to %d: %s",
                    n, strerror (errno));
            exit (2) ;
        }
    }
    else {
        syslog_init (0, timepfx);
        close (1) ;
        if (dup2 (2, 1) != 1) {
            logger (LOG_ERR, "Cannot dup stdout to stderr: %s",
                    strerror (errno));
            exit (2);
        }
    }

    /* say hello */
    cpry ();

    /* record own process id */
    if (pid_fd >= 0) {
        char buf [20];
        sprintf (buf, "%d\n", getpid ());
        write (pid_fd, buf, strlen (buf));
        close (pid_fd);
    }

    /* initialise the server control socket before chroot */
    ctrl = ctrl_init (set_gid,
                      NVCL_IOBUF_SIZE, NVCL_REQUEST_SOCKET,
                      &verbosity);

    /* change root environment */
    if (new_root && (chroot (new_root) || chdir ("/"))) {
        logger (LOG_ERR, "chroot failed: %s", strerror (errno));
        exit (2);
    }

    /* initialise the action loop */
    s.loop = do_init (idle, &s);

    /* set verbosity */
    verbosity = verbose ? (~ 0) : MAIN_LOG_DEFAULTS ;
    
    /* open variables list */
    if ((s.lst = list_open (s.loop, 0, nvram_commit, trigger_delay)) == 0) {
        logger (LOG_ERR, "Cannot initialise nvram cache");
        exit (2);
    }

    /* add services ... */
    logger (LOG_INFO, "Adding control socket services ...");
    if ((s.comm = comm_init (s.lst, set_gid)) == 0) {
        logger (LOG_ERR, "Cannot initialise local commumication system");
        exit (2);
    }
    ctrl = ctrl_add (ctrl, "set",       comm_set,        s.comm);
    ctrl = ctrl_add (ctrl, "del",       comm_del,        s.comm);
    ctrl = ctrl_add (ctrl, "slots",     comm_slots,      s.comm);
    ctrl = ctrl_add (ctrl, "get",       comm_get,        s.comm);
    ctrl = ctrl_add (ctrl, "link",      comm_symlink,    s.comm);
    ctrl = ctrl_add (ctrl, "unlink",    comm_rm_symlink, s.comm);
    ctrl = ctrl_add (ctrl, "trigger",   comm_trigger,    s.comm);
    ctrl = ctrl_add (ctrl, "untrigger", comm_rm_trigger, s.comm);
    ctrl = ctrl_add (ctrl, "access",    comm_access,     s.comm);
    ctrl = ctrl_add (ctrl, "unaccess",  comm_rm_access,  s.comm);
    ctrl = ctrl_add (ctrl, "store",     comm_store,      s.comm);
    ctrl = ctrl_add (ctrl, "load",      comm_load,       s.comm);
    ctrl = ctrl_add (ctrl, "rusage",    ctrl_resource,   ctrl);
    ctrl = ctrl_add (ctrl, "noisy",     ctrl_verbosity,  ctrl);
    ctrl = ctrl_add (ctrl, "help",      ctrl_usage,      ctrl);
    s.comm = comm_update (s.comm, ctrl, !dont_defload);

#   ifndef DISCARD_HTTP
    /* initalise the http command interface */
    if (no_http == 0) {
        if ((http = http_init (s.loop, s.lst, slots, ipaddr, port)) == 0) {
            logger (LOG_ERR, "Cannot initialise http commumication system");
            exit (2);
        }
        http_cmdadd (http, "store",  0,      store_cmd, 0);
        http_cmdadd (http, "slots",  0,      slots_cmd, 0);
        http_cmdadd (http,  "load",  0,       load_cmd, 0);
        http_cmdadd (http,  "list", -1,       list_cmd, 0); /* opt. args */
        http_cmdadd (http,   "get",  1,        get_cmd, 0);
        http_cmdadd (http,   "set", -1,        set_cmd, 0); /* opt. args */
        http_cmdadd (http,   "del",  1,        del_cmd, 0);
        http_cmdadd (http, "noisy", -1, http_verbosity, 0);
        http_cmdadd (http,       0,  0,      error_cmd, 0);
    }
#   endif /* DISCARD_HTTP */

    if (setjmp (go_there) != 0)
        goto cleanup_and_leave ;

    signal (SIGINT,  s_go_interrupt);
    signal (SIGQUIT, s_go_interrupt);
    signal (SIGILL,  s_go_endup);
    signal (SIGABRT, s_go_endup);
    signal (SIGKILL, s_go_interrupt);
    signal (SIGTERM, s_go_interrupt);

    setjmp (go_here);

    signal (SIGHUP,  s_go_here);
    signal (SIGUSR1, s_go_here);
    signal (SIGUSR2, s_go_here);
        
    /* accept commands */
    do_action (s.loop, ctrl_fd (ctrl), (void(*)(void*))ctrl_handler, ctrl);

#   ifndef DISCARD_HTTP
    if (http)
        do_action
            (s.loop, http_fd (http), (void(*)(void*))http_accept, http);
#   endif /* DISCARD_HTTP */
        
    for (;;)
        if (do_loop (s.loop, idle_timeout * 1000) < 0 && verbose)
            logger (LOG_INFO, "%s: Error in do_loop(): %s",
                    progname, strerror (errno));

    /*NOTREACHED*/

cleanup_and_leave:
    ctrl_shutdown (ctrl);
    do_loop (s.loop, 0);

    /* release the server control socket */
    ctrl_destroy (ctrl) ;

    /* kill sub processes */
    list_close (s.lst) ;
    
#   ifdef _MAIN_DEBUG
    /* release dispatcher */
    do_destroy (s.loop);
#   endif /* _MAIN_DEBUG */

    exit (0);
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
