/*
 * nvramd - active data base server and flash memory cache
 *
 * Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA
 *
 * Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-Isenburg, Germany
 *
 * $Id: preload.c 2711 2006-06-27 16:42:24Z jordan $
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>

#ifdef STDC_HEADERS
# include <stdlib.h>
# include <stddef.h>
#else
# ifdef HAVE_STDLIB_H
#  include <stdlib.h>
# endif
#endif

#ifdef TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# ifdef HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#ifdef HAVE_CTYPE_H
# include <ctype.h>
#endif

#include "util.h"
#include "flash.h"
#include "buffer.h"
#include "hlst.h"
#include "list.h"
#include "nvalloc.h"
#include "functions.h"
#include "dsafunctions.h"

#define  __NVRAMD_PRELOAD_PRIVATE
#include "preload.h"

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */


void preload_functions (LIST *l) {
    mkfunctions (l);
    mkdsafunctions (l);
}


unsigned preload_defaults (LIST *lst, int def_mode, int io_mode) {

    if (def_mode && (io_mode & import_clear_ok) == 0) {

#       ifdef MAC_TEMPLATE
        const char *p ;
        /* template_ok is allowed not before the mac address is loaded */
#       define DEF_MODE (def_mode & ~import_template_ok)
#       else
#       define DEF_MODE  def_mode
#       endif /* MAC_TEMPLATE */

        /* check whether templates will be applied later on, anyway */
        if (io_mode & import_template_ok)
            def_mode &= ~import_template_ok ;

        list_load (lst, flash_defaults, DEF_MODE);

#       ifdef MAC_TEMPLATE
        if ((p = strchr (mac_template, '=')) != 0) {
            unsigned int x, y, z ;
            if (isxdigit (p [1]) == 0)
                p = strchr (p, ':') ;
            if (p != 0)
                p ++ ;
            if (sscanf (p, "%x:%x:%x", &x, &y, &z) == 3) {
                struct timeval seed ;
                size_t len = strlen (mac_template) ;
                char *q = memcpy (zmalloc (len + 20), mac_template, len);

                /* get the 24 bit private mac prefix */
                long long eth = (((x << 8) | y) << 8) | z ; eth <<= 24;

                /* get 24 random bits for the private mac address */
                gettimeofday (&seed, 0); /* weak random is ok here */
                srand (seed.tv_sec ^ seed.tv_usec) ; 
                eth |= (rand () >> 4) & 0xffffff ;

                sprintf (q + (p - mac_template),
                         "%02x:%02x:%02x:%02x:%02x:%02x",
                         (int) ((eth >> 40) & 0xff),
                         (int) ((eth >> 32) & 0xff),
                         (int) ((eth >> 24) & 0xff),
                         (int) ((eth >> 16) & 0xff),
                         (int) ((eth >>  8) & 0xff),
                         (int) ((eth      ) & 0xff));
                p = q ;
            }
            else
                p = 0 ;
        }

        list_load
            (lst, p ? p : mac_template, def_mode & ~import_clear_ok);

        if (p)
            free ((char*)p) ;
#       endif /* MAC_TEMPLATE */
    }
    
    if (io_mode) {

        const char *d = flash_read ();
    
        list_load (lst, d, io_mode); /* d might be 0 */

        flash_bufclear () ;

        if (d == 0)
            return 0 ;
    }
    
    return 1;
}
    
/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
