/*
 * nvramd - active data base server and flash memory cache
 *
 * Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA
 *
 * Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-Isenburg, Germany
 *
 * $Id: preload.h 2821 2006-07-10 12:30:02Z jordan $
 */

#ifndef __NVRAMD_PRELOAD_H
#define __NVRAMD_PRELOAD_H

#include "list.h"

extern unsigned preload_defaults (LIST*, int def_mode, int io_mode);
extern     void preload_functions (LIST*);

/* avoiding typos using symbols rather than plain text */
#define AV_TRUE      "true"
#define AV_FALSE     "false"
#define AV_DISABLED  "disabled"
#define AV_ENABLED   "enabled"
#define AV_UNSET     "unset"
#define AV_KEEPALIVE "keepalive"

#ifdef __NVRAMD_PRELOAD_PRIVATE

#include "symdump.h"

#define __str(x)              # x
#define str(x)                __str (x)

#define VACCESS_PFX     ACCESS_TRIGGER \
                        str (ACTION_VOLATILE_DATA) \
                        ACTION_FIELD_DELIM

#define FUNCTION_PFX    ACCESS_TRIGGER \
                        str (ACTION_VOLATILE_FUNCTION) \
                        ACTION_FIELD_DELIM

#define TRIGGER_PFX     EXECUTE_TRIGGER \
                        str (ACTION_ENABLE_EXEC_ON_WRITE) \
                        ACTION_FIELD_DELIM "-1" \
                        ACTION_FIELD_DELIM "-1" \
                        ACTION_FIELD_DELIM \
                        ACTION_FIELD_DELIM

#define VOLTRIG_PFX     EXECUTE_TRIGGER \
                        str (ACTION_ENABLE_EXEC_ON_WRITE_VOLATILE) \
                        ACTION_FIELD_DELIM "-1" \
                        ACTION_FIELD_DELIM "-1" \
                        ACTION_FIELD_DELIM \
                        ACTION_FIELD_DELIM

#define SYMLINK_PFX     SYMLINK_TRIGGER \
                        str (ACTION_DELAY_XLINK_ON_WRITE) \
                        ACTION_FIELD_DELIM

#define VOLTLINK_PFX    SYMLINK_TRIGGER \
                        str (ACTION_DELAY_XLINK_ON_WRITE_VOLATILE) \
                        ACTION_FIELD_DELIM

#define WRLINK_TPFX     SYMLINK_TRIGGER \
                        str (ACTION_DELAY_XLINK_ON_WRITE_TMPL) \
                        ACTION_FIELD_DELIM

#define access_vtile(val)           VACCESS_PFX val

#define function_vtile(val)         FUNCTION_PFX val

#define trigger_vtile(script, val)	VOLTRIG_PFX \
                                    "/bin/sh:/usr/share/board_cfg/" \
                                    script ACTION_FIELD_DELIM val

#define trigger_vbin(script, val)	VOLTRIG_PFX \
                                    "/usr/share/board_cfg/" \
                                    script ACTION_FIELD_DELIM val

#define trigger_sh( script, value)	TRIGGER_PFX \
                                    "/bin/sh:/usr/share/board_cfg/" \
                                    script ACTION_FIELD_DELIM value

#define write_link( target, value)	SYMLINK_PFX \
                                    target ACTION_FIELD_DELIM value

#define vlatile_link( target, val)  VOLTLINK_PFX \
                                    target ACTION_FIELD_DELIM val

#define wrlink_templ(  trg, value)  WRLINK_TPFX \
                                    trg    ACTION_FIELD_DELIM value

#define _avpair( var,    val)          var "=" val "\0"
#define _avolt(  var,    val) _avpair (var,  access_vtile      (val))
#define _afunc(  var, fn)     _avpair (var,function_vtile       (fn))
#define _vtrig(  var,exe,val) _avpair (var, trigger_vtile (exe, val))
#define _vbtrig( var,exe,val) _avpair (var, trigger_vbin  (exe, val))
#define _trigger(var,exe,val) _avpair (var, trigger_sh    (exe, val))
#define _vltlnk( var,exe,val) _avpair (var, vlatile_link  (exe, val))
#define _wrlink( var,exe,val) _avpair (var,   write_link  (exe, val))
#define _wrltmpl(var,exe)     _avpair (var,  wrlink_templ (exe, AV_UNSET))

#include "preload-data.h"
	
#endif /* __NVRAMD_PRELOAD_PRIVATE */
#endif /* __NVRAMD_PRELOAD_H */
