/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>

#include "filter.h"
#include "util.h"
#include "settings.h"
#include "logger.h"

#define  __FILTER_DEFAULTS_PRIVATE
#include "filter-defaults.h"

/* ----------------------------------------------------------------------- *
 * private: constants, private data, macros
 * ----------------------------------------------------------------------- */

#define DIM(x)  (sizeof (x) / sizeof (x [0]))

#define __str(x)              # x
#define str(x)                __str (x)

/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */

static
void errmsg (FRUN_SPEC *q, const char *tag, const char *instance) {
    logger (LOG_ERR, "Error setting up default for %s(%s): %s",
            tag, instance, filter_error (q));
}

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

unsigned filter_load_defaults (FILTER *f) {
    int n, used ;
    
    for (n = used = 0; n < DIM (filter_defaults); n ++) {
        const FILTER_DEFAULTS *d = filter_defaults + n ;
        FRUN_SPEC *q ;
        int m = 0 ;

        /* always create a new record */
        if ((q = filter_find (f, d->tag)) != 0)
            filter_delete (q) ;
        q = filter_spec (f, d->tag);

        /* record comand */
        if (d->run.template) {
            filter_tmpl (q, d->run.template, 0) ;
            m ++ ;
        }

        /* compile regular expressions */
        if (d->run.stop) {
            if (filter_stop (q, d->run.stop) == 0)
                errmsg (q, d->tag, MAIN_STOP);
            else
                m ++ ;
        }
        if (d->run.redo) {
            if (filter_redo (q, d->run.redo) == 0)
                errmsg (q, d->tag, MAIN_REDO);
            else
                m ++ ;
        }
        if (d->run.start) {
            if (filter_start (q, d->run.start) == 0)
                errmsg (q, d->tag, MAIN_FIND);
            else
                m ++ ;
        }
        if (d->run.replace) {
            if (filter_replace (q, d->run.replace) == 0)
                errmsg (q, d->tag, MAIN_REPLACE);
            else
                m ++ ;
        }

        if (m) used ++ ;
    }

    return used ;
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
