/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/


#ifndef __FILTER_DEFAULTS_H
#define __FILTER_DEFAULTS_H

unsigned filter_load_defaults (FILTER*);

#ifdef __FILTER_DEFAULTS_PRIVATE

typedef struct _filter_defaults {
    const char *tag ;
    struct {
        const char *template ;
        const char *stop ;
        const char *redo ;
        const char *start ;
        const char *replace ;
    } run ;
} FILTER_DEFAULTS ;


/*
 * pattern flags:      i -- ignore cases
 *                     s -- single line ('\n' is "any" character)
 *
 * replacemet flags:   g -- globally, repeat replacement rule
 *                     x -- extraction mode, discard unmatched data
 *
 * argument variables: $0, $1, ...   -- replace by 1st, 2nd, ... parameter
 *
 * pattern matching:   $0            -- replace by whole pattern
 *                     $1, $2 ...    -- replace by 1st, 2nd ... sub-pattern
 * 
 * default handling:   ${#:-<other>} -- replace by <other> if variable $#
 *                                      (# = 0, 1, 2 ..) is empty or does not
 *                                      exist, otherwise use variable $#
 *                     ${#-<other>}  -- replace by <other> if variable $#
 *                                      (# = 0, 1, 2 ..) exists and is empty,
 *                                      replace by empty string if variable
 *                                      $# does not exist, otherwise use
 *                                      variable $#
 *                     ${#:+<other>} -- replace by <other> if variable $#
 *                                      exists and is not empty, otherwise
 *                                      replace by empty string 
 *                     ${#+<other>}  -- same as  ${#:+<other>} 
 */


/* templates */
#define CM_CLI_SHOW(x) "cm_cli -l <<X\n" "begin\n" x "end\n" "X\n"
#define CM_CLI_CMD(x)  "cm_cli <<X\n"    "begin\n" x "end\n" "X\n"

/* patterns */
#define CM_CLI_ERROR   "/error:\\s+error_code=/i"
// #define CM_CLI_AGAIN   "/error:.*\\sconnecting\\s/i"
#define CM_CLI_AGAIN   "/core\\s+logic\\s+is\\s+not\\s/i"
#define CM_CLI_BLOCK   "/^(>\\s+)?X\\r?$/"       /* ad-hoc string terminator */
#define CM_CLI_VALACK  "/^cli:\\s+value\\s+=\\s/" /* re-prints commands */

/* replacements */
#define CM_CLI_VALUES  "/" "[0-9]\\s+[^=\\s]*=" \
                           "([^[:cntrl:][:blank:]]*)" \
                       "/" "${1:-0}\\n" \
                       "/xg"
#define CM_CLI_SKIP    "/" "(>\\s)?" "(.*[^\\r])" "\\r?$" \
                       "/" "$2"  /* have the request reflected */ \
                       "/g"

static const FILTER_DEFAULTS filter_defaults [] = {

    { "save-now", {
        CM_CLI_CMD

        ("logic:command/save\n"),

        CM_CLI_ERROR,
        CM_CLI_AGAIN,
        "/X/",
        "/.*//xs" }
    },

    { "save-reboot", {
        CM_CLI_CMD

        ("logic:command/save\n"
         "logic:command/reboot\n"),

        CM_CLI_ERROR,
        CM_CLI_AGAIN,
        "/X/",
        "/.*//xs" }
    },
    
    { "restore-defaults", {
        CM_CLI_CMD

        ("logic:command/defaults\n"),

        CM_CLI_ERROR,
        CM_CLI_AGAIN,
        "/X/",
        "/.*//xs" }
    },

#   include   "ofilter-config-defaults.h"
#   include     "route-config-defaults.h"
#   include       "dsl-config-defaults.h"
#   include        "fw-config-defaults.h"
// #include       "rip-config-defaults.h"
// #include    "bridge-filter-defaults.h"
// #include        "ip-config-defaults.h"
// #include      "snmp-config-defaults.h"
// #include      "dhcp-config-defaults.h"
// #include      "wlan-config-defaults.h"
#   include         "wlan-acl-defaults.h"
// #include       "wlan-tuner-defaults.h"
#   include        "wlan-ssid-defaults.h"
#   include      "upnp-config-defaults.h"
#   include   "virtual-config-defaults.h"
#   include     "debug-config-defaults.h"
#   include       "net-config-defaults.h"
#   include "neighbour-config-defaults.h"

};

#endif /* __FILTER_DEFAULTS_PRIVATE */
#endif /* __FILTER_DEFAULTS_H */
