/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/


#ifndef __FILTER_H
#define __FILTER_H

#include <stdarg.h>
#include "buffer.h"

#ifdef __FILTER_PRIVATE
# include "hlst.h"
# include <sys/types.h>
# include <regex.h>

typedef struct _filter_find {
    const char *pattern ;
    regex_t       regex ;
    unsigned match_hits ;
} FILTER_FIND ;

typedef struct _filter_edit {
    FILTER_FIND   match ;
    const char  *format ;
    int      globally:1 ;
    int  matched_only:1 ;
} FILTER_EDIT ;
#endif

typedef struct _filter {
#   ifdef __FILTER_PRIVATE
    HLST          *lookup ;
    unsigned submatch_dim ;
#   else
    char       any ;
#   endif
} FILTER;

typedef struct _frun_spec {
#   ifdef __FILTER_PRIVATE
    const char         *key ;
    IOBUF             handy ;
    unsigned        visited ;
    struct {
        const char  *format ;
        unsigned       argc ;
        const char   **argv ;
    } send ;
    FILTER_FIND        stop ;
    FILTER_FIND        redo ;
    FILTER_FIND        find ;
    FILTER_EDIT     replace ;
    FILTER            *base ;
    const char       *error ;
    regmatch_t submatch [1] ;
    /* open end */
#   else
    char       any ;
#   endif
} FRUN_SPEC;

extern FILTER     *filter_open    (unsigned);
extern void        filter_close   (FILTER*);
extern void        filter_print   (FILTER*,
                                   void(*)(void*,const char*,...),void*);

extern FRUN_SPEC  *filter_find    (FILTER*,const char*);
extern FRUN_SPEC  *filter_spec    (FILTER*,const char*);
extern void        filter_delete  (FRUN_SPEC*);

extern int         filter_run     (FRUN_SPEC*,IOBUF*);
extern int         filter_matches (FRUN_SPEC*,int*,int*,int*,int*,int*);
extern void        filter_push    (FRUN_SPEC*,const char*);
extern const char *filter_tmpl    (FRUN_SPEC*,const char*,int);
extern const char *filter_start   (FRUN_SPEC*,const char*);
extern const char *filter_stop    (FRUN_SPEC*,const char*);
extern const char *filter_redo    (FRUN_SPEC*,const char*);
extern const char *filter_replace (FRUN_SPEC*,const char*);
extern const char *filter_error   (FRUN_SPEC*);

#endif /* __FILTER_H */
