/*
 * nvtools lib - flash data base support tools
 *
 * Copyright (C) 1998-2006 mjh-EDV Beratung - Jordan Hrycaj <jordan@mjh.teddy-net.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA
 *
 * $Id: hlst.h 3128 2006-09-01 07:17:15Z jordan $
 */

#ifndef __SHAT_HLST_H__
#define __SHAT_HLST_H__

#ifdef __HLST_INTERNAL__
typedef struct _hash_defs {     /* hash list parameters */
    unsigned  mod ;             /* number of buckets */
    unsigned  fac ;             /* shift by multiplication */
} HASH_DEFS ;
#endif

typedef struct _hsrch           /* walk through the list */
#ifdef __HLST_INTERNAL__
{
    struct _hlst     *hlist ;	/* current hash list, to walk on */
    unsigned      bucket_id ;	/* current bucket */
    struct _hashqueue *ntry ;	/* pointer to the next entry */
    struct _hsrch     *next ;	/* more such entries */
}
#endif
HSRCH ;

typedef struct _hlst            /* hash list descriptor */
#ifdef __HLST_INTERNAL__
{
    struct _sorter *access;     /* there might be an index on that list */
    int (*sorter_fn)(void*,const char*,unsigned,const char*,unsigned);
    void *sorter_desc ;         /* custom sort of the entries */
    void *clup_state ;
    void (*clup)                /* call back destructor for entries */
    (void*,void*,char*,unsigned);
    HASH_DEFS       z ; 		/* hash parameters for this list */
    HSRCH         *walk ;		/* list: walk through the hash list */
    unsigned total_entries ;	/* number of entries in the list */
    void *bucket [1] ;          /* varable length array of buckets */
    /* varable length, pointer aligned  */
}
#endif
HLST ;

/* open/close management */
extern HLST *create_hlst
(unsigned estimated_size_hint, 
 /* this function is called when non-empty slots are removed, if the
    list is destroyed, the last call will be destroy(state,0,0,0). Apart
    from the last call, the destroy function will always be called with
    a non NULL item, */
 void (*destroy)(void *state, void* item, char* key, unsigned keylen), 
 void* state) ;

extern HLST *copy_hlst 
(HLST *to_be_copied,
 /* leave the estimated_size_hint 0 to use the parameters from the
    list to_be_copied */
 unsigned estimated_size_hint, 

 /* The copy() function is called when an entry is to be copied and
    inserted into the copied slot - leave it empty to copy a list with
    keys but without inserted data, only. This function is only used for
    duplicating the list. Upon copy() returning NULL while errno set to
    a non-NULL value, copying is stopped and the partly allocated list
    is removed using destroy_hlst(). In this case, the copy_hlst() 
    function returns NULL while errno is set as passed from the copy() 
    function. */
 void *(*copy) (void *cpstate, void* item, char *key, unsigned keylen),
 void *cpstate,

 /* The destroy() function is called when non-empty slots are removed,
    if the list is destroyed, the last call will be destroy(state,0,0,0) 
    This function is registered, internally to be used later on.Apart
    from the last call, the detroy function will only be called with a
    non NULL item, */
 void (*destroy)(void *state, void* item, char* key, unsigned keylen), 
 void* state) ;

/* global factor to be applied internally to any estimated_size_hint */
extern unsigned compress_hlst_index (unsigned size_hint_percentage);

extern void destroy_hlst (HLST*);
extern void   flush_hlst 
(HLST*,
 /* can have another destroy function, here, uses the default
    assigned in chreate_hlst, if the destroy argument is NULL */
 void (*destroy)(void*,void*,char*,unsigned), void*state);



/* find an existing slot, len == 0 means: key is ascii string */
extern void** find_hlst (HLST*, const char *key, unsigned len);

/* create a non existing slot, otherwise error
   len == 0 means: key is ascii string */
extern void** make_hlst (HLST*, const char *key, unsigned len);

/* delete slot (-1 == error), len == 0 means: key is ascii string */
extern int delete_hlst (HLST*, const char *key, unsigned len);


/* for a given (void**) ptr value as returned by {find|make}_hlst,
   retrieve the korresponding key and its length */
extern char    *query_key_hlst    (void **);
extern unsigned query_keylen_hlst (void **);

/* returns the number of elements in the argument list (might be NULL) */
extern unsigned query_hlst_size (HLST *);

/* applying a function fn () to all list elements */
extern int for_hlst_do
(HLST *,
 /* if the argunemt function returns non 0 while the algorithm is running,
    the rest of the items not visited yet are skipped and for_hlst_do ()
    returns with the value passed by this function. */
 int (*fn)(void *state, void *, char *key, unsigned keylen),
 void *state) ;


/* searching step wise through all elements */
extern HSRCH* open_hlst_search (HLST*) ;
extern void** next_hlst_search (HSRCH*) ;
extern void  close_hlst_search (HSRCH*) ;


/* sorter stuff */
extern void   sort_hlst (HLST*);
extern void  **inx_hlst (HLST*,unsigned);
extern void unsort_hlst (HLST*);

/* custom sort */
extern int csort_hlst
(HLST*,
 /* custom comparison function for qsort */
 int (*)(void *desc,
         const char  *left_key, unsigned  left_klen,
         const char *right_key, unsigned right_klen),
 void *desc);

/* statistics etc. */
extern int  hlst_buckets (HLST*);
extern void hlst_statistics
(HLST *h,
 /* statistics, fn may be NULL to use the default */
 void (*fn) (void*state, unsigned bucket_fill), void *state) ;

#endif /* __SHAT_HLST_H__ */

