/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/


#ifndef __SHELL_H
#define __SHELL_H

#include <stdio.h>
#include "buffer.h"

typedef struct _shell {
#   ifdef __SHELL_PRIVATE
    IOBUF        rd_cache ;
    IOBUF            data ;
    const char      *pool ;
    pid_t     rclocal_pid ;
    unsigned   rcl_script ;
    unsigned  rclocal_ttd ;
    const char   *rclocal ;
    const char  *rcscript ;
    struct {
        unsigned min_wait_repeat ; /* between to echo requests */
        time_t       wait_repeat ;
        int           keep_alive ;
        size_t           req_len ;
#       ifdef SHELL_ECHO_REPLY
        const char          *req ;
        const char        *reply ;
#       else
        char                *req ;
        size_t           cmd_len ;
#       endif
    } echo ;
    int no_iamode_check:1 ;
    int save_resp_headr:1 ;
    struct {
        const char    *ps ;
        unsigned      len ;
    } prompt ;
    struct {
        enum {
            no_error = 0,
            reinit_error,
            child_error,
            illegal_file_path,
            write_error,
            cannot_open_file,
            cannot_stat_file,
            file_too_large,
            rcfile_too_large,
            echo_too_early,
            dialogue_mode_shell,
            output_jam,
            read_while_no_write,
        } num ;
        int     no ;
        size_t len ;
    } err ;
    struct {
        struct {
            time_t    started ;
            int     rc_sent:1 ;
            int     rc_init:1 ;
            unsigned requests ;
            size_t   outbytes ;
            size_t    inbytes ;
            FILE          *fp ;
            int            fd ;
        } io ;
        FILE     *dump_fp ;
        pid_t         pid ;
        const char **argv ;
    } cmd ;
#   else
    char       any ;
#   endif
} SHELL;

extern SHELL      *shell_open  (SHELL **open_anyway,
                                const char *user,
                                const char *idfile,
                                const char *trg,
                                uint16_t    port,
                                const char *prompt,
                                const char *pool,
                                const char *rclocal,
                                const char *rcscript,
                                int no_iamode_check);
extern void       shell_status (SHELL             *s,
                                time_t       *uptime,
                                size_t           *in,
                                size_t          *out,
                                const char **rc_init,
                                const char **rc_sent,
                                unsigned   *requests,
                                unsigned   *kp_alive);
extern void        shell_close   (SHELL*);
extern void        shell_term    (SHELL*);
extern void        shell_flush   (SHELL*);
extern int         shell_fd      (SHELL*);
extern unsigned    shell_restart (SHELL*);
extern unsigned    shell_contest (SHELL*, int timeout, int force);
extern unsigned    shell_kpalve  (SHELL*, int yes);
extern const char *shell_data    (SHELL*,const char*,size_t,
                                  int (*)(void*,IOBUF*),void*,int);
extern FILE       *shell_iodump  (SHELL*,FILE*);
extern const char *shell_file    (SHELL*,const char*,
                                  int (*)(void*,IOBUF*),void*,int);
extern const char *shell_error   (SHELL*);

#endif /* __SHELL_H */
