/***************************************************************************
 *   Copyright (C) 2005 by Henry Margies                                   *
 *   henry.margies@gmx.de                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <mtd/mtd-user.h>


#include "redconfig.h"

#ifdef DEBUG
#define PDEBUG(fmt, args...) 	printf("[%s:%d] DEBUG: " fmt,	\
							__FILE__,	\
							__LINE__, 	\
							##args)
#else
#define PDEBUG(fmt, argc...)	{}
#endif

int mtd_import_ip(unsigned char* ip)
{
	int a,b,c,d,r;

	r = scanf("Value: %i.%i.%i.%i\n",&a,&b,&c,&d);
	if (r!=4)
		return 0;
	ip[0] = a; ip[1] = b; ip[2] = c; ip[3] = d;
	return 1;
}


int mtd_import_mac(unsigned char* ip)
{
	int r, a,b,c,d,e,f;
	r = scanf("Value: %02x:%02x:%02x:%02x:%02x:%02x\n",
		&a,&b,&c,&d, &e, &f);
	if (r!=6)
		return 0;
	ip[0] = a; ip[1] = b; ip[2] = c; ip[3] = d;
	ip[4] = e; ip[5] = f; ip[6] = 0; ip[7] = 0;

	return 1;
}

int mtd_import_script(char* b)
{
	int r;

	scanf("Value:\n");

	while(1) {
		r = scanf(".. %[^\n]\n", b);
		if (r!=1)
			return 1;
		strcat(b, "\n");
		b += strlen(b);
	}
}

char* mtd_import_value(char* redconfig, char* max, int type)
{
	char buffer[MAX_SIZE];
	int r,b;
	unsigned char ip[8];

	memset(buffer, 0, MAX_SIZE);
	
	switch (type) {
		case CONFIG_BOOL:
		case CONFIG_INT:
			r = scanf("Value: %d\n", &b);
			PDEBUG("Value: %d\n", b);
			memcpy(redconfig, &b, sizeof(int));
			redconfig += sizeof(int);
			break;
		case CONFIG_IP:
			r = mtd_import_ip(ip);
			PDEBUG("Value: %i.%i.%i.%i\n", 
				ip[0],ip[1],ip[2],ip[3]);
			memcpy(redconfig, ip, 4);
			redconfig += 4;
			break;
		case CONFIG_ESA:
			r = mtd_import_mac(ip);
			PDEBUG("Value: %02x:%02x:%02x:%02x:%02x:%02x\n", 
				ip[0],ip[1],ip[2],ip[3],ip[4],ip[5]);
			memcpy(redconfig, ip, 8);
			redconfig += 8;
			break;
		case CONFIG_SCRIPT:
			r = mtd_import_script(buffer);
			PDEBUG("Value:\n%s\n", buffer);
			memcpy(redconfig, buffer, SCRIPT_SIZE);
			redconfig += SCRIPT_SIZE;
			break;
		case CONFIG_STRING:
		case CONFIG_NETPORT:
			r = scanf("Value: %[^\n]\n", buffer);
			PDEBUG("Value: %s\n", buffer);
			memcpy(redconfig, buffer, STRING_SIZE);
			redconfig += STRING_SIZE;
			break;

		default:
			fprintf(stderr, "Unknown type\n");
			return NULL;
	}

	if (r != 1) {
		fprintf(stderr, "Error reading value\n");
		return NULL;
	}
	return redconfig;
}


char* mtd_import_item(char* redconfig, char* max)
{
	int type, name_length, sense, dep_length, ret;
	header_t header;
	char buffer[MAX_SIZE];

	ret = scanf(	"Header.Type: %d\n"
			"Header.NameLength: %d\n"
			"Header.Sense: %d\n"
			"Header.DependencyLength: %d\n",
			&type, &name_length,
			&sense, &dep_length);
	if (ret != 4) {
		int crc32;
		if (scanf("Checksum: %x\n", &crc32) == 1) {
			PDEBUG("Complete\n");
			return max;
		}
		fprintf(stderr, "Error in reading header information\n");
		return NULL;
	}

	PDEBUG(	"Header.Type: %d\n", type);
	PDEBUG(	"Header.NameLength: %d\n", name_length);
	PDEBUG(	"Header.Sense: %d\n", sense);
	PDEBUG(	"Header.DependencyLength: %d\n", dep_length);

	header.type = type;
	header.name_length = name_length;
	header.sense = sense;
	header.dependency_length = dep_length;

	if (redconfig 
		+ name_length 
		+ dep_length 
		+ sizeof(header)
		+ config_value_size[type] > max) {

		fprintf(stderr, "Too much information, filesize to small\n");
		return NULL;
	}
	memcpy(redconfig, &header, sizeof(header));
	redconfig += sizeof(header);

	if (name_length) {
		ret = scanf("Name: %s\n", buffer);
		if (ret != 1) {
			fprintf(stderr, "Error in reading name\n");
			return NULL;
		}
		PDEBUG("Name: %s\n", buffer);
		memcpy(redconfig, buffer, name_length);
		redconfig += name_length;
	}

	if (dep_length) {
		ret = scanf("Dependency: %s\n", buffer);
		if (ret != 1) {
			fprintf(stderr, "Error in reading dependency\n");
			return NULL;
		}
		PDEBUG("Dependency: %s\n", buffer);
		memcpy(redconfig, buffer, dep_length);
		redconfig += dep_length;
	}

	return mtd_import_value(redconfig, max, type);
}


#include "crc32.c"
char* mtd_checksum(char* redconfig, int filesize)
{
	char magic[] = { 0xde, 0xad, 0xde, 0xad };
	int ret;

	memcpy(redconfig+filesize-8, magic, 4);
	ret = cyg_crc32(redconfig, filesize-4);

	memcpy(redconfig+filesize-4, &ret, 4);
	PDEBUG("Checksum: %d\n", ret);
	return redconfig;
}

char* mtd_import()
{
	char* redconfig;
	char* p;
	int filesize;
	int magic;
	
	int ret;

	
	ret = scanf("Filelength: %d\n", &filesize);
	if (ret != 1) {
		fprintf(stderr, "Error reading file size from config\n");
		return NULL;
	}
	PDEBUG("Filelength: %d\n", filesize);

	ret = scanf("Magic: %x\n", &magic);
	if (ret != 1) {
		fprintf(stderr, "Error reading magic number from config\n");
		return NULL;
	}
	PDEBUG("Magic: %x\n", magic);

	redconfig = (char*) malloc(filesize);
	if (!redconfig) {
		perror("Could not allocate memory: ");
		return NULL;
	}
	memset(redconfig, 0, filesize);
	p = redconfig;
	memcpy(p, &filesize, sizeof(filesize));
	p += sizeof(filesize);
	memcpy(p, &magic, sizeof(magic));
	p += sizeof(magic);

	do {
		p = mtd_import_item(p, redconfig+filesize);
	} while(p && p!= redconfig+filesize);	

	if (p)
		return mtd_checksum(redconfig, filesize);
	return NULL;

}


int compare_config() 
{
	int	ret;
	int	fsz;
	int  	fd = 0;
	char* 	new_cfg;
	char*	old_cfg;
	
	/* read config from stdin */
	new_cfg = mtd_import();
	if (NULL == new_cfg)
		return -1;

	/* read config from flash */
	fd = open_mtd_by_name("RedBoot config", O_RDONLY);
	if (-1 == fd) {
		perror("Unable to open device");
		return -1;
	}
	read(fd, &fsz, sizeof(fsz));
	old_cfg = (char*) malloc(fsz);
	memcpy(old_cfg, &fsz, sizeof(fsz));
	ret = read(fd, old_cfg+4, fsz-4);
	if (ret != fsz-4) {
		perror("Error in reading redconfig from flash\n");
		return ret;
	}
	close(fd);

	ret = memcmp(new_cfg, old_cfg, fsz);

	if (ret)
		printf("Configs differ\n");

	return ret;
}


int write_config()
{
	int	ret;
	int	fsz;
	int	fd;
	char* 	new_cfg;

	struct mtd_info_user mtd;
	struct erase_info_user erase;
	
	/* read config from stdin */
	new_cfg = mtd_import();
	if (NULL == new_cfg)
		return -1;

	/* get config size */
	memcpy(&fsz, new_cfg, sizeof(int));


	fd = open_mtd_by_name("RedBoot config", O_WRONLY);
	if (fd < 0) {
		perror("Unable to open device");
		return -1;
	}

	/* get device info */
	ret = ioctl(fd, MEMGETINFO, &mtd);
	if (-1 == ret) {
		perror("could not get mtd info, is this a mtd device?: ");
		close(fd);
		return -1;
	}

	/* check for enough space */
	if (mtd.size < fsz) {
		fprintf(stderr, "mtd device too small, "
				"want to write %d bytes, size is %d",
				fsz, mtd.size);
		close(fd);
		return -1;
	}

	/* calculate blocks to erase */
	erase.start = 0;
	erase.length= mtd.size;

	/* unlock */
	ret = ioctl(fd, MEMUNLOCK, &erase);
	if (ret < 0) {
		perror("could not unlock flash device: ");
		close(fd);
		return -1;
	}

	/* erase */
	ret = ioctl(fd, MEMERASE, &erase);
	if (ret < 0) {
		perror("could not erase flash device: ");
		close(fd);
		return -1;
	}
	
	ret = write(fd, new_cfg, fsz);
	
	/* lock */
	ret = ioctl(fd, MEMLOCK, &erase);
	if (ret < 0) {
		perror("could not lock flash device: ");
		close(fd);
		return -1;
	}

	close(fd);

	return 0;
}

int main(int nargs, char** argv)
{
	char 	params[] = "wch";
	int	i;

	while(1) {
		i = getopt(nargs, argv, params);
		if (i == -1) 
			break;

		switch(i) {

			case 'w':
				return write_config();
	
			case 'c':
				return compare_config();

			case 'h':
				break;

			default:
				fprintf(stderr, "Unknown option %c\n", i);
		}		
	}

	fprintf(stderr, 
		"write_redconfig - Writes new redboot configuration to flash\n"
		"GPL Licensed - Copyright Henry Margies <h.margies@maxina.de>\n\n"
		"This programm reads a configuration created by "
		"dump_redconfig\nfrom stdin and writes it into the "
		"redboot configuration\npartition in flash.\n\n"
		"    -w Write config to flash\n"
		"    -c Compares config from stdin with config in flash\n"
		"    -h This help text\n"
		"\nExample: dump_redconfig > config\n"
		"         write_redconfig -c < config\n\n");
	
	return -1;
}
