/***************************************************************************
 *   Copyright (C) 2005 by Henry Margies <h.margies@maxina.de>		   *
 *   			   Sven Kolberg  <s.kolberg@maxina.de>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


static struct hbx_dsp_config* dsp_config_ec;

/**
 * Just needed to save the config as global var.
 */
static void hbx_dsp_register_ec(struct hbx_dsp_config* cfg)
{
	dsp_config_ec = cfg;
}


void hbx_dsp_request_ec(int* channel)
{
	if (!dsp_config_ec->initialized) {
		*channel = -1;
		return;
	}

	PDEBUG("---> Requesting echo canceller... <---\n");
	*channel = hbx_dsp_request_codec(dsp_config_ec, 
		SRC_CODEC(ECHOC_FMT_ALAW)|DST_CODEC(ECHOC_FMT_ALAW));
	PDEBUG("---> Result of Request is %d      <---\n", *channel);
}

void hbx_dsp_release_ec(int* channel)
{
	PDEBUG("---> Release echo canceller... <---\n");
	
	hbx_dsp_release_codec(dsp_config_ec, *channel);
	*channel = -1;
}

void hbx_dsp_put_ec(int channel, unsigned char* tx, unsigned char* rx)
{
	struct rwbuf *wbuf;
	struct dbuf *dbuf;
	int i;
	unsigned char header[] = { 	LCT_START, 
					16, 0,	/* length in le */
					8, 0, 	/* samples in le */
					LCT_CONTINUE, 
					LCT_CONTINUE, 
					LCT_CONTINUE};

	/* if channel is unconfigured, return */
	if (channel < 0)
		return;

	/* get write ring buffer for channel */
	wbuf = &dsp_config_ec->wbuf[channel];
	/* get data buffer (to write to it) */
	dbuf = &wbuf->dbuf[wbuf->write_idx];

	/* check if buffer is empty */
	if (dbuf->size > 0 )
		return;

	for (i=0; i<8; i++) {
		/* copy voice data to buffer */
		dbuf->data_buf[i] = *(tx++) | (*(rx++)<<8);
	}

	/* set length and samples */
	memcpy(dbuf->info_buf, header,  sizeof(header));

	dbuf->size = 16;
	dbuf->idx  =  0;

	wbuf->write_idx++;
	wbuf->write_idx%=HBX_DSP_RBS;
}

void hbx_dsp_get_ec(int channel, unsigned char* tx, unsigned char* rx)
{
	struct rwbuf *rbuf;
	struct dbuf *dbuf;
	int i;

	/* FIXME just for  testing */
	memset(rx, 0, 8);
	memset(tx, 0, 8);

	/* if no channel was configured */
	if (channel < 0)  {
		dsp_config_ec->c2++;
		return;
	}

	/* get read ring buffer for channel */
	rbuf = &dsp_config_ec->rbuf[channel];

	/* exit if no data available */
	if (rbuf->write_idx == rbuf->read_idx) {
		dsp_config_ec->c2++;
		return;
	}

	/* get data buffer */
	dbuf = &rbuf->dbuf[rbuf->read_idx];

	/* check available data */
//	if (16 != dbuf->size) {
//		printk("wrong size! (%d)\n", dbuf->size);
//	}

	for (i=0; i<8; i++) {
		/* copy voice data to buffer */
		*(tx++) = dbuf->data_buf[i] & 0xff;
		*(rx++) = (dbuf->data_buf[i] >> 8) & 0xff;
	}
	
	dbuf->size = 0;
	dbuf->idx  = 0;

	rbuf->read_idx++;
	rbuf->read_idx%=HBX_DSP_RBS;
}

EXPORT_SYMBOL(hbx_dsp_request_ec);
EXPORT_SYMBOL(hbx_dsp_release_ec);
EXPORT_SYMBOL(hbx_dsp_put_ec);
EXPORT_SYMBOL(hbx_dsp_get_ec);
