/***************************************************************************
 *   Copyright (C) 2005 by Henry Margies <h.margies@maxina.de>             *
 *                                                                         *
 *   IXP425 High Speed Serial Driver for Linux				   *
 * 									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifdef IXP425_HSS_PORT0_LOOPBACK_CLIENT
#error "Loopback not supported on port 1"
#endif

#ifdef IXP425_HSS_PORT1_LOOPBACK_SERVER
#error "Loopback not supported on port 1"
#endif

#if (!defined IXP425_HSS_PORT1_TRANS_FN) && (defined IXP425_HSS_ENABLE_PORT1)
#define IXP425_HSS_PORT1_TRANS_FN ixp425_hss_data_transfer2_cb

static void handle_pkt_hss(IX_OSAL_MBUF* rxBuffer)
{
	IX_OSAL_MBUF* 	rx;
	IX_OSAL_MBUF* 	txBuffer;
	unsigned char* 	rx_ptr;
	unsigned char* 	tx_ptr;
	unsigned char	buffer[32*8];
	struct si3210_hss_config* cfg;

	cfg = &hss_config[1];
		
	for(	rx=rxBuffer; 
		rx != NULL; 
		rx=IX_OSAL_MBUF_NEXT_BUFFER_IN_PKT_PTR(rx)) {

		/* get RX buffers */
		rx_ptr = IX_OSAL_MBUF_MDATA(rx);
		IX_OSAL_CACHE_INVALIDATE(rx_ptr, IX_OSAL_MBUF_MLEN(rx));
			
		/* read and write data */
		if (cfg->transfer[0].func) {

			cfg->transfer[0].func(	rx_ptr, buffer, 
						cfg->transfer[0].data); 
		}
			

	}

	/* return buffer */
	IX_OSAL_MBUF_POOL_PUT_CHAIN(rxBuffer);

	/* get TX buffers */
	txBuffer = IX_OSAL_MBUF_POOL_GET(cfg->poolIdPtr);
	if (!txBuffer)
		return;

	IX_OSAL_MBUF_MLEN(txBuffer) = 32*8;
	tx_ptr = ((unsigned char *)IX_OSAL_MBUF_MDATA(txBuffer));

	memcpy(tx_ptr, buffer, 32*8);

	/* send data */
	IX_OSAL_CACHE_FLUSH(tx_ptr, IX_OSAL_MBUF_MLEN(txBuffer));
	ixHssAccPktPortTx(1, 0, txBuffer);

}


static void rxCallback(	IX_OSAL_MBUF *buffer,
			unsigned numHssErrs,
			IxHssAccPktStatus pktStatus,
			IxHssAccPktUserId rxUserId)
{
#ifdef IXP425_HSS_DEBUG
	if (pktStatus != IX_HSSACC_PKT_OK) {
		printk("Error in rxCallback (%d)\n", pktStatus);
	}
#endif

	handle_pkt_hss(buffer);
}


static void txDoneCallback( IX_OSAL_MBUF *buffer,
			    unsigned numHssErrs,
			    IxHssAccPktStatus pktStatus,
			    IxHssAccPktUserId txDoneUserId)
{
#ifdef IXP425_HSS_DEBUG
	if (pktStatus != IX_HSSACC_PKT_OK) {
		printk("Error txDoneCallback (%d)\n", pktStatus);
	}
#endif
	
	IX_OSAL_MBUF_POOL_PUT_CHAIN(buffer);
}


static void rxFreeLowCallback(IxHssAccPktUserId rxFreeLowUserId)
{
	struct si3210_hss_config* cfg;
	IX_OSAL_MBUF *rxBuffer;
	int ret;

	cfg = &hss_config[1];

	do {
		rxBuffer = IX_OSAL_MBUF_POOL_GET(cfg->poolIdPtr);
	
		IX_OSAL_MBUF_MLEN(rxBuffer) = 32*8;

		ret = ixHssAccPktPortRxFreeReplenish(1, 0, rxBuffer);
		if (IX_SUCCESS != ret)
			IX_OSAL_MBUF_POOL_PUT_CHAIN(rxBuffer);

	} while(rxBuffer && ret==IX_SUCCESS);
}




#endif

