/***************************************************************************
 *   Copyright (C) 2005 by Henry Margies <h.margies@maxina.de>             *
 *                                                                         *
 *   IXP425 High Speed Serial Driver for Linux				   *
 * 									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef IXP425_HSS_ENABLE_PORT0

static IxHssAccPortConfig	hss_port0_config_tx = {
	/* frame sync pulse type (tx/rx) */
//	.frmSyncType	= IX_HSSACC_FRM_SYNC_ACTIVE_LOW,
	.frmSyncType	= IX_HSSACC_FRM_SYNC_ACTIVE_HIGH,
//	.frmSyncType	= IX_HSSACC_FRM_SYNC_FALLINGEDGE,

	/* how the frame sync pulse is used (tx/rx) */
//	.frmSyncIO	= IX_HSSACC_FRM_SYNC_OUTPUT_FALLING,
	.frmSyncIO	= IX_HSSACC_FRM_SYNC_INPUT,

	/* frame sync clock edge type (tx/rx) */
	.frmSyncClkEdge	= IX_HSSACC_CLK_EDGE_RISING,

	/* data clock edge type (tx/rx) */
	.dataClkEdge	= IX_HSSACC_CLK_EDGE_RISING,

	/* clock direction (tx/rx) */
	.clkDirection	= IX_HSSACC_SYNC_CLK_DIR_INPUT,

	/* whether to use the frame sync pulse or not (tx/rx) */
	.frmPulseUsage	= IX_HSSACC_FRM_PULSE_ENABLED,
//	.frmPulseUsage	= IX_HSSACC_FRM_PULSE_DISABLED,
	
	/* data rate in relation to the clock (tx/rx) */
	.dataRate	= IX_HSSACC_CLK_RATE,
//	.dataRate	= IX_HSSACC_HALF_CLK_RATE,

	/* data polarity type (tx/rx) */
	.dataPolarity	= IX_HSSACC_DATA_POLARITY_SAME,
//	.dataPolarity	= IX_HSSACC_DATA_POLARITY_INVERT,

	/* data endianness (tx/rx) */
//	.dataEndianness	= IX_HSSACC_LSB_ENDIAN,
	.dataEndianness	= IX_HSSACC_MSB_ENDIAN,

	/* tx pin open drain mode (tx) */
	.drainMode	= IX_HSSACC_TX_PINS_NORMAL,
//	.drainMode	= IX_HSSACC_TX_PINS_OPEN_DRAIN,

	/* start of frame types (tx/rx) */
//	.fBitUsage	= IX_HSSACC_SOF_FBIT,
	.fBitUsage	= IX_HSSACC_SOF_DATA,

	/* whether or not to drive the data pins (tx) */
	.dataEnable	= IX_HSSACC_DE_DATA,
//	.dataEnable	= IX_HSSACC_DE_TRI_STATE,
	
	/* how to drive the data pins for voice56k type (tx) */
	.voice56kType	= IX_HSSACC_TXSIG_LOW,
//	.voice56kType	= IX_HSSACC_TXSIG_HIGH,
//	.voice56kType	= IX_HSSACC_TXSIG_HIGH_IMP,
	
	/* how to drive the data pins for unassigned type (tx) */
	.unassignedType	= IX_HSSACC_TXSIG_LOW,
//	.unassignedType	= IX_HSSACC_TXSIG_HIGH,
//	.unassignedType	= IX_HSSACC_TXSIG_HIGH_IMP,
	
	/* how to drive the Fbit (tx) */
	.fBitType	= IX_HSSACC_FB_FIFO,
//	.fBitType	= IX_HSSACC_HIGH_IMP,
	
	/* 56k data endianness when using the 56k type (tx) */
	.voice56kEndian	= IX_HSSACC_56KE_BIT_7_UNUSED,
//	.voice56kEndian	= IX_HSSACC_56KE_BIT_0_UNUSED,
	
	/* 56k data transmission type when using the 56k type (tx) */
	.voice56kSel	= IX_HSSACC_56KS_32_8_DATA,
//	.voice56kSel	= IX_HSSACC_56KS_56K_DATA,
	
	/* frame pulse offset in bits wrt the first timeslot (0-1023) (tx/rx)*/
	.frmOffset	= 0,
	
	/* frame size in bits (1-1024) (tx/rx)*/
	.maxFrmSize	= 256,
};


static IxHssAccPortConfig	hss_port0_config_rx = {
	/* frame sync pulse type (tx/rx) */
//	.frmSyncType	= IX_HSSACC_FRM_SYNC_ACTIVE_LOW,
//	.frmSyncType	= IX_HSSACC_FRM_SYNC_ACTIVE_HIGH,
	.frmSyncType	= IX_HSSACC_FRM_SYNC_FALLINGEDGE,

	/* how the frame sync pulse is used (tx/rx) */
	.frmSyncIO	= IX_HSSACC_FRM_SYNC_INPUT,
//	.frmSyncIO	= IX_HSSACC_FRM_SYNC_OUTPUT_FALLING,
//	.frmSyncIO	= IX_HSSACC_FRM_SYNC_OUTPUT_RISING,

	/* frame sync clock edge type (tx/rx) */
	.frmSyncClkEdge	= IX_HSSACC_CLK_EDGE_RISING,
//	.frmSyncClkEdge	= IX_HSSACC_CLK_EDGE_RISING,

	/* data clock edge type (tx/rx) */
	.dataClkEdge	= IX_HSSACC_CLK_EDGE_RISING,
//	.dataClkEdge	= IX_HSSACC_CLK_EDGE_RISING,

	/* clock direction (tx/rx) */
	.clkDirection	= IX_HSSACC_SYNC_CLK_DIR_INPUT,
//	.clkDirection	= IX_HSSACC_SYNC_CLK_DIR_OUTPUT,

	/* whether to use the frame sync pulse or not (tx/rx) */
	.frmPulseUsage	= IX_HSSACC_FRM_PULSE_ENABLED,
//	.frmPulseUsage	= IX_HSSACC_FRM_PULSE_DISABLED,
	
	/* data rate in relation to the clock (tx/rx) */
	.dataRate	= IX_HSSACC_CLK_RATE,
//	.dataRate	= IX_HSSACC_HALF_CLK_RATE,

	/* data polarity type (tx/rx) */
	.dataPolarity	= IX_HSSACC_DATA_POLARITY_SAME,
//	.dataPolarity	= IX_HSSACC_DATA_POLARITY_INVERT,

	/* data endianness (tx/rx) */
//	.dataEndianness	= IX_HSSACC_LSB_ENDIAN,
	.dataEndianness	= IX_HSSACC_MSB_ENDIAN,

	/* tx pin open drain mode (tx) */
	.drainMode	= IX_HSSACC_TX_PINS_NORMAL,
//	.drainMode	= IX_HSSACC_TX_PINS_OPEN_DRAIN,

	/* start of frame types (tx/rx) */
//	.fBitUsage	= IX_HSSACC_SOF_FBIT,
	.fBitUsage	= IX_HSSACC_SOF_DATA,

	/* whether or not to drive the data pins (tx) */
	.dataEnable	= IX_HSSACC_DE_DATA,
//	.dataEnable	= IX_HSSACC_DE_TRI_STATE,
	
	/* how to drive the data pins for voice56k type (tx) */
	.voice56kType	= IX_HSSACC_TXSIG_LOW,
//	.voice56kType	= IX_HSSACC_TXSIG_HIGH,
//	.voice56kType	= IX_HSSACC_TXSIG_HIGH_IMP,
	
	/* how to drive the data pins for unassigned type (tx) */
//	.unassignedType	= IX_HSSACC_TXSIG_LOW,
//	.unassignedType	= IX_HSSACC_TXSIG_HIGH,
	.unassignedType	= IX_HSSACC_TXSIG_HIGH_IMP,
	
	/* how to drive the Fbit (tx) */
	.fBitType	= IX_HSSACC_FB_FIFO,
//	.fBitType	= IX_HSSACC_HIGH_IMP,
	
	/* 56k data endianness when using the 56k type (tx) */
	.voice56kEndian	= IX_HSSACC_56KE_BIT_7_UNUSED,
//	.voice56kEndian	= IX_HSSACC_56KE_BIT_0_UNUSED,
	
	/* 56k data transmission type when using the 56k type (tx) */
	.voice56kSel	= IX_HSSACC_56KS_32_8_DATA,
//	.voice56kSel	= IX_HSSACC_56KS_56K_DATA,
	
	/* frame pulse offset in bits wrt the first timeslot (0-1023) (tx/rx)*/
	.frmOffset	= 0,
	
	/* frame size in bits (1-1024) (tx/rx)*/
	.maxFrmSize	= 256,
};

/* Note: maxFrmSize, numChannelised and clkSpeed are related to each 
 * other. Do not simply change.
 */

/* Configuration for one hss port */
static IxHssAccConfigParams hss_port0_config = {
	.numChannelised		= 8,
	.channelisedIdlePattern	= 0xd5,
#ifdef IXP425_HSS_PORT0_LOOPBACK_CLIENT
	.loopback		= TRUE,
#else
	.loopback		= FALSE,
#endif
	.hssPktChannelCount	= 0,
	.packetizedIdlePattern	= 0x7F7F7F7F,
	.clkSpeed		= IX_HSSACC_CLK_SPEED_2048KHZ,
};

static IxHssAccTdmSlotUsage hss_tdm_config0[IX_HSSACC_TSLOTS_PER_HSS_PORT] =
{
/* TS: 00 */	IX_HSSACC_TDMMAP_VOICE64K,
/* TS: 01 */	IX_HSSACC_TDMMAP_UNASSIGNED,
/* TS: 02 */	IX_HSSACC_TDMMAP_VOICE64K,
/* TS: 03 */	IX_HSSACC_TDMMAP_UNASSIGNED,
/* TS: 04 */	IX_HSSACC_TDMMAP_VOICE64K,
/* TS: 05 */	IX_HSSACC_TDMMAP_UNASSIGNED,
/* TS: 06 */	IX_HSSACC_TDMMAP_VOICE64K,
/* TS: 07 */	IX_HSSACC_TDMMAP_UNASSIGNED,
/* TS: 08 */	IX_HSSACC_TDMMAP_VOICE64K,
/* TS: 09 */	IX_HSSACC_TDMMAP_UNASSIGNED,
/* TS: 10 */	IX_HSSACC_TDMMAP_VOICE64K,
/* TS: 11 */	IX_HSSACC_TDMMAP_UNASSIGNED,
/* TS: 12 */	IX_HSSACC_TDMMAP_VOICE64K,
/* TS: 13 */	IX_HSSACC_TDMMAP_UNASSIGNED,
/* TS: 14 */	IX_HSSACC_TDMMAP_VOICE64K,
/* TS: 15 */	IX_HSSACC_TDMMAP_UNASSIGNED,
/* rest of port are unassigned */
};

#endif
