/***************************************************************************
 *   Copyright (C) 2005 by Henry Margies <h.margies@maxina.de>             *
 *                                                                         *
 *   IXP425 High Speed Serial Driver for Linux				   *
 * 									   *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


/*  proc dir for hss statistics */
static struct proc_dir_entry* ixp425_hss_proc_dir;

/* file and directory names */
#define IXP425_HSS_PROC_DIR		"hss"
#define IXP425_HSS_PROC_FILE		"statistics"

static int ixp425_hss_proc_read(char *page, char **start __unused,
			    off_t off __unused, int count __unused,
			    int *eof __unused, void *data)
{
	int port = (int)data;
	int channels = 0;
	int i;

	for (i=0; i<8; i++) {
		if (hss_config[port].transfer[i].func)
			channels++;
	}

	return sprintf(page, 
			"Registered channels: %d\n"
			"Frame Drops        : %u\n" 
			"Loopback (Success) : %u\n"
			"Loopback (Fails)   : %u\n",
			channels,
			hss_config[port].drops_total,
			hss_config[port].loopback_success,
			hss_config[port].loopback_fails);

}

static void ixp425_hss_proc_register(void)
{
	/* register proc directory */
	ixp425_hss_proc_dir = proc_mkdir(IXP425_HSS_PROC_DIR, NULL);
	if(ixp425_hss_proc_dir == NULL) {
		printk("Error creating proc directory\n");
		return;
	}

	create_proc_read_entry(	IXP425_HSS_PROC_FILE"_0", 0444, 
				ixp425_hss_proc_dir, 
				ixp425_hss_proc_read, (void*)0);

	create_proc_read_entry(	IXP425_HSS_PROC_FILE"_1", 0444, 
				ixp425_hss_proc_dir, 
				ixp425_hss_proc_read, (void*)1);
}
	

static void ixp425_hss_proc_unregister(void)
{
	/* remove proc file */
	remove_proc_entry(IXP425_HSS_PROC_FILE"_1", ixp425_hss_proc_dir);
	remove_proc_entry(IXP425_HSS_PROC_FILE"_0", ixp425_hss_proc_dir);
	remove_proc_entry(IXP425_HSS_PROC_DIR, NULL);
}


