/***************************************************************************
 *   Copyright (C) 2005 by Sven Kolberg  <s.kolberg@maxina.de>		   *
 *   			   Henry Margies <h.margies@maxina.de>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/*
 * TODO:
 * 	- clean up of the code
 * 	- remove bugs
 *
 */

#include <linux/config.h>
#include <linux/init.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/kernel.h>

#include <linux/delay.h>
#include <linux/spinlock.h>

#include "ixp425_spi.h"
#include "si3210_regs.h"

 /* GPIO pins used */
#define IXP425_SDI		3
#define IXP425_SDO		4
#define IXP425_SCLK		2
#define IXP425_CS		5      //Chip-Select for the Si3210
#define IXP425_CSe		12     //Chip-Select for the Si3250
#define IXP425_RESET		13
#define IXP425_INT		11
#define IXP425_GPIO_ACTIVE_LOW	0	/* How to trigger is defined on request_irq */


/* Use macro PDEBUG for printing debug messages */
#undef PDEBUG				// undef it, just in case
#ifdef IXP425_SPI_DEBUG
	#define PDEBUG(fmt, args...) 	\
		printk("[%s:%d] "fmt, __FILE__,__LINE__, ##args)
#else
	#define PDEBUG(fmt, args...)	{}
#endif

/* The Semaphore to provide exclusive access to the SPI-Bus */
spinlock_t ixp425_spi_access_spinlock = SPIN_LOCK_UNLOCKED;

/* The variable indicating a active interrupt */
static unsigned char ixp425_spi_irq_active = 0;

/**
 * Marks a section with active SPI - Interrupt
 */
void SPI_IRQ_Start(void)
{
//	PDEBUG("IRQ Lock on\n");
	ixp425_spi_irq_active = 1;
}

/**
 * Marks a section with inactive SPI - Interrupt
 */
void SPI_IRQ_Stop(void)
{
	ixp425_spi_irq_active = 0;
//	PDEBUG("IRQ Lock off\n");
}

/**
 *  Checks if a SPI-Interrupt is active
 * \returns 0 when out of Interrupt Service Routine
 *          1 when in Interrupt Service Routine
 */
static unsigned char SPI_in_ISR(void)
{
	return ixp425_spi_irq_active;
}


/**
 * Write a byte to the SPI-Bus (The Si321x are attached to)
 *
 * Timing-Diagram:
 *       __    __    __    __    __    __    __    __    ____
 * sclk    \__/  \__/  \__/  \__/  \__/  \__/  \__/  \__/
 *       _                                                ___
 * /cs    \______________________________________________/
 *          _____ _____ _____ _____ _____ _____ _____ ___ 
 * sdi   XXX__d7_X__d6_X__d5_X_d4__X_d3__X_d2__X_1d__X_d0XXXX
 *
 * \param value The Byte to be written
 */
static void ixp425_spi_write(unsigned char value)
{ 
	int i;
	/* set cs to low and sclk to high -> start byte transfer */
	gpio_line_set(IXP425_CS, 0);
	gpio_line_set(IXP425_CSe, 1);
	gpio_line_set(IXP425_SCLK, 1);
	/* set SDI to the MSB of our value */
	gpio_line_set(IXP425_SDI, (int)value&0x80);
	/* wait for at least 25ns */
	ndelay(25); 

	/* mask from MSB to LSB */
	for (i=0x80; i; i>>=1) {

		/* generate falling edge */
		gpio_line_set(IXP425_SCLK, 0);
		/* set current bit */
		gpio_line_set(IXP425_SDI, (value&i)?1:0);
		/* wait for at least 25ns */
		ndelay(25);
		/* generate rising edge */
		gpio_line_set(IXP425_SCLK, 1);
		/* wait for at least 20 ns */
		ndelay(20);
	}

	/* set cs to high -> end of byte transfer */
	gpio_line_set(IXP425_CS, 1);
	/* wait for at least 440ns */
	ndelay(600);
}

/**
 * Write a byte to the SPI-Bus (The Si3050 are attached to)
 *
 * Timing-Diagram:
 *       __    __    __    __    __    __    __    __    ____
 * sclk    \__/  \__/  \__/  \__/  \__/  \__/  \__/  \__/
 *       _                                                ___
 * /cse   \______________________________________________/
 *          _____ _____ _____ _____ _____ _____ _____ ___ 
 * sdi   XXX__d7_X__d6_X__d5_X_d4__X_d3__X_d2__X_1d__X_d0XXXX
 *
 * \param value The Byte to be written
 */
static void ixp425_spi_write_si3050(unsigned char value)
{ 
	int i;
	/* set cs to low and sclk to high -> start byte transfer */
	gpio_line_set(IXP425_CSe, 0);
	gpio_line_set(IXP425_CS, 1);
	gpio_line_set(IXP425_SCLK, 1);
	/* set SDI to the MSB of our value */
	gpio_line_set(IXP425_SDI, (int)value&0x80);
	/* wait for at least 25ns */
	ndelay(25); 

	/* mask from MSB to LSB */
	for (i=0x80; i; i>>=1) {

		/* generate falling edge */
		gpio_line_set(IXP425_SCLK, 0);
		/* set current bit */
		gpio_line_set(IXP425_SDI, (value&i)?1:0);
		/* wait for at least 25ns */
		ndelay(25);
		/* generate rising edge */
		gpio_line_set(IXP425_SCLK, 1);
		/* wait for at least 20 ns */
		ndelay(20);
	}

	/* set cs to high -> end of byte transfer */
	gpio_line_set(IXP425_CSe, 1);
	/* wait for at least 440ns */
	ndelay(600);
}


/**
 * Write a byte to the SPI-Bus (The Si321x are attached to)
 *
 * Timing-Diagram:
 *       __    __    __    __    __    __    __    __    ____
 * sclk    \__/  \__/  \__/  \__/  \__/  \__/  \__/  \__/
 *       _                                                ___
 * /cs    \______________________________________________/
 *          _____ _____ _____ _____ _____ _____ _____ ___ 
 * sdo   XXX__d7_X__d6_X__d5_X_d4__X_d3__X_d2__X_1d__X_d0XXXX
 *
 * \returns The Byte read from the SPI-Bus
 */
static unsigned char ixp425_spi_read(void)
{
	unsigned char b=0;
	int r;
	int i;

	/* set sclk high and cs low to start byte transfer */
	gpio_line_set(IXP425_SCLK, 1);
	gpio_line_set(IXP425_CS, 0);
	gpio_line_set(IXP425_CSe, 1);
	/* wait for at least 25ns */
	ndelay(50);

	for (i=0; i<8; i++) {

		/* generate falling edge and wait at least 25ns*/
		gpio_line_set(IXP425_SCLK, 0);
		ndelay(50);
		
		/* generate rising edge and wait at least 20ns*/
		gpio_line_set(IXP425_SCLK, 1);
		ndelay(50);

		/* read pin */
		gpio_line_get(IXP425_SDO, &r);
		b = (b << 1) | r;
	}

	/* set cs to high, end of byte read and wait for 440ns */
	gpio_line_set(IXP425_CS, 1);
	ndelay(440);
	return b;
}

/**
 * Write a byte to the SPI-Bus (The Si3050 are attached to)
 *
 * Timing-Diagram:
 *       __    __    __    __    __    __    __    __    ____
 * sclk    \__/  \__/  \__/  \__/  \__/  \__/  \__/  \__/
 *       _                                                ___
 * /cse   \______________________________________________/
 *          _____ _____ _____ _____ _____ _____ _____ ___ 
 * sdo   XXX__d7_X__d6_X__d5_X_d4__X_d3__X_d2__X_1d__X_d0XXXX
 *
 * \returns The Byte read from the SPI-Bus
 */
static unsigned char ixp425_spi_read_si3050(void)
{
	unsigned char b=0;
	int r;
	int i;

	/* set sclk high and cs low to start byte transfer */
	gpio_line_set(IXP425_SCLK, 1);
	gpio_line_set(IXP425_CSe, 0);
	gpio_line_set(IXP425_CS, 1);
	/* wait for at least 25ns */
	ndelay(50);

	for (i=0; i<8; i++) {

		/* generate falling edge and wait at least 25ns*/
		gpio_line_set(IXP425_SCLK, 0);
		ndelay(50);
		
		/* generate rising edge and wait at least 20ns*/
		gpio_line_set(IXP425_SCLK, 1);
		ndelay(50);

		/* read pin */
		gpio_line_get(IXP425_SDO, &r);
		b = (b << 1) | r;
	}

	/* set cs to high, end of byte read and wait for 440ns */
	gpio_line_set(IXP425_CSe, 1);
	ndelay(440);
	return b;
}


/**
 * Looks the access to the spi bus and disallows Interrupts 
 * using spin_lock_irq()
 */
static inline void ixp425_spi_lock(void)
{
	if(!SPI_in_ISR()) 
		spin_lock_irq(&ixp425_spi_access_spinlock);
}


/**
 * UnLooks the access to the spi bus and allows Interrupts
 * using spin_unlock_irq()
 */
static inline void ixp425_spi_unlock(void)
{
	if(!SPI_in_ISR())  
		spin_unlock_irq(&ixp425_spi_access_spinlock);
}


/**
 * Reads a Register a from a given SPI-Device chip 
 * The Read operation consist out of three steps:
 *	-select chip
 *	-write the number of the register to be read
 *	-read back the content of the register
 *
 * To make sure no service-Interrupt tries to access the SPI-Bus, we
 * look the access
 *
 * \param chip The Number of the Device
 * \param reg The Register to be read
 * \return The Answer Byte
 */
unsigned char si321x_read_cmd(unsigned char chip, unsigned char reg)
{
	unsigned char b;

	if (chip>1) PDEBUG("Read from non existend card\n");	
	
	ixp425_spi_lock(); // Block Interrupts

	/* select chip and write register number, MSB=1 for read mode */
	ixp425_spi_write(1<<chip);
	ixp425_spi_write(reg | 0x80);

	/* read byte from the SPI-Bus */
	b = ixp425_spi_read();
	
	ixp425_spi_unlock(); // Allow Interrupts
	
	PDEBUG("Reading from chip %d of 3210 reg %d value 0x%x\n", 
			chip, reg, b);

	return b;
}

/**
 * Reads a Register a from a given SPI-Device chip 
 * The Read operation consist out of three steps:
 *	-select chip
 *	-write the number of the register to be read
 *	-read back the content of the register
 *
 * To make sure no service-Interrupt tries to access the SPI-Bus, we
 * look the access
 *
 * \param chip The Number of the Device
 * \param reg The Register to be read
 * \return The Answer Byte
 */
unsigned char si3050_read_cmd(unsigned char chip, unsigned char reg)
{
	unsigned char b;
	
	ixp425_spi_lock(); // Block Interrupts

	/* select chip and write register number, MSB=1 for read mode */
	ixp425_spi_write_si3050(0x40 | 0x20 | chip); //READ | 1 | CID
	ixp425_spi_write_si3050(reg);

	/* read byte from the SPI-Bus */
	b = ixp425_spi_read_si3050();
	
	ixp425_spi_unlock(); // Allow Interrupts
	
	PDEBUG("Reading from chip %d of 3050 reg %d value 0x%x\n", 
			chip, reg, b);

	return b;
}


/**
 * Writes a Register a of a given SPI-Device chip with the Value value
 * The Write Operation consist out of three Steps
 *	-select chip
 *	-write the number of the Register
 *	-write the new content of the Register
 *
 * To make sure no service-Interrupt tries to access the SPI-Bus, we
 * look the access
 *
 * \param chip The Number of the Device
 * \param reg The Register
 * \param value The Value to be writen
 */
void si3050_write_cmd(unsigned char chip,unsigned char reg, unsigned char value)
{
	ixp425_spi_lock();	//Block Interrupts

	PDEBUG("Writing on chip %d of 3050 reg %d to value 0x%x\n", 
			chip, reg, value);

	/* select chip and write register number (MSB=0, indicating a write) */
	ixp425_spi_write_si3050(0x20 | chip); //Write | 1 | CID
	ixp425_spi_write_si3050(reg & 0x3f);
	/* write byte */
	ixp425_spi_write_si3050(value);

	ixp425_spi_unlock(); //Allow Interrupts
}

/**
 * Writes a Register a of a given SPI-Device chip with the Value value
 * The Write Operation consist out of three Steps
 *	-select chip
 *	-write the number of the Register
 *	-write the new content of the Register
 *
 * To make sure no service-Interrupt tries to access the SPI-Bus, we
 * look the access
 *
 * \param chip The Number of the Device
 * \param reg The Register
 * \param value The Value to be writen
 */
void si321x_write_cmd(unsigned char chip,unsigned char reg, unsigned char value)
{
	ixp425_spi_lock();	//Block Interrupts
	if (chip>1) PDEBUG("Read from non existend card\n");	

	PDEBUG("Writing on chip %d of 3210 reg %d to value 0x%x\n", 
			chip, reg, value);

	/* select chip and write register number (MSB=0, indicating a write) */
	ixp425_spi_write(1<<chip);
	ixp425_spi_write(reg & 0x7f);
	/* write byte */
	ixp425_spi_write(value);

	ixp425_spi_unlock(); //Allow Interrupts
}

/** 
 * Writes a indirect register of the Si321x using two special registers
 * \param chip the number of the device
 * \param reg The indirect register number
 * \param value The 16bit value to be writen
 */
void si321x_write_indirect(	unsigned char chip, 
				unsigned char reg, 
				unsigned short value)
{
	/* wait until last write process has been proceeded */
	while( si321x_read_cmd(chip, SI3210_INDIRECT_ADDRESS_STATUS_REG) ){
	}

	/* block interrupts */
	ixp425_spi_lock();
	
	/* write high data byte to indirect data register */
	ixp425_spi_write(1<<chip);
	ixp425_spi_write(SI3210_INDIRECT_DATA_ACCESS_HIGH_REG & 0x7f);
	ixp425_spi_write((unsigned char) ((value>>8) & 0xff));

	/* write low data byte */
	ixp425_spi_write(1<<chip);
	ixp425_spi_write(SI3210_INDIRECT_DATA_ACCESS_LOW_REG & 0x7f);
	ixp425_spi_write((unsigned char) (value & 0xff));

	/* write address */
	ixp425_spi_write(1<<chip);
	ixp425_spi_write(SI3210_INDIRECT_ADDRESS_REG & 0x7f);
	ixp425_spi_write(reg);
	
	ixp425_spi_unlock();
}


/** 
 * Reads indirect register of the Si321x using two special registers
 * \param chip the number of the device
 * \param reg The indirect register number
 * \returns The 16bit value read
 */
unsigned short si321x_read_indirect(unsigned char chip, unsigned char reg)
{
	unsigned char byteh, bytel;
	
	/* wait till the last operation completes */
	while(si321x_read_cmd(chip, SI3210_INDIRECT_ADDRESS_STATUS_REG )){
	}

	/* block interrupts */
	ixp425_spi_lock();

	/* read address */
	ixp425_spi_write(1<<chip);
	ixp425_spi_write(SI3210_INDIRECT_ADDRESS_REG & 0x7f);
	ixp425_spi_write(reg);

	/* read low - Data-Byte */
	ixp425_spi_write(1<<chip);
	ixp425_spi_write(SI3210_INDIRECT_DATA_ACCESS_LOW_REG | 0x80);
	bytel = ixp425_spi_read();

	/* read high - Data-Byte */
	ixp425_spi_write(1<<chip);
	ixp425_spi_write(SI3210_INDIRECT_DATA_ACCESS_HIGH_REG | 0x80);
	byteh = ixp425_spi_read();
		
	ixp425_spi_unlock();

	return (unsigned short) byteh<<8 | bytel;
}


/**
 * Configures GPIO-Pins for the use as SPI and resets all connected devices
 */ 
static int ixp425_spi_init(void) 
{
	/* setup gpio's */
	gpio_line_config(IXP425_RESET	, IXP4XX_GPIO_OUT);
	gpio_line_config(IXP425_SDO	, IXP4XX_GPIO_IN);
	gpio_line_config(IXP425_CS	, IXP4XX_GPIO_OUT);
	gpio_line_config(IXP425_CSe	, IXP4XX_GPIO_OUT);
	gpio_line_config(IXP425_SCLK	, IXP4XX_GPIO_OUT);
	gpio_line_config(IXP425_SDI	, IXP4XX_GPIO_OUT);
	gpio_line_config(IXP425_INT	, IXP4XX_GPIO_IN | 
					  IXP425_GPIO_ACTIVE_LOW);
	/* hold reset*/
	gpio_line_set(IXP425_RESET, 0);
	
	/* set to default state */
	gpio_line_set(IXP425_SCLK, 1);	/* SCLK MUST be 1 during Reset to indicate SPI-Mode for the Si3050 */
	gpio_line_set(IXP425_CS, 1);	
	gpio_line_set(IXP425_CSe, 1);	

	mdelay(250); //wait 250 ms
	
	/* release reset */
	gpio_line_set(IXP425_RESET, 1);
	mdelay(2); /* wait 2ms */

	/* activate daisy chain mode on the Si321x SPI-Bus portion*/	
	ixp425_spi_write(SI3210_SPI_MODE_SELECT_REG);
	ixp425_spi_write(SI3210_SPI_DAISY_CHAIN_MODE_BIT);

	return 0;
}

static void ixp425_spi_release(void) 
{
	/* TODO */
}


MODULE_DESCRIPTION("SPI Driver for Intel IXDP425 Boards");
MODULE_AUTHOR("Henry Margies (h.margies@maxina.de");
MODULE_LICENSE("GPL");


module_init(ixp425_spi_init);
module_exit(ixp425_spi_release);

EXPORT_SYMBOL(si321x_read_cmd);
EXPORT_SYMBOL(si321x_read_indirect);
EXPORT_SYMBOL(si3050_read_cmd);
EXPORT_SYMBOL(si321x_write_cmd);
EXPORT_SYMBOL(si3050_write_cmd);
EXPORT_SYMBOL(si321x_write_indirect);
EXPORT_SYMBOL(SPI_IRQ_Start);
EXPORT_SYMBOL(SPI_IRQ_Stop);
