/***************************************************************************
 *   Copyright (C) 2005 by Henry Margies <h.margies@maxina.de>		   *
 *   			  Sven Kolberg  <s.kolberg@maxina.de>              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <linux/version.h>
#include <linux/config.h>
#include <linux/delay.h>
#include <linux/timer.h>
#include <linux/proc_fs.h>
#include <linux/interrupt.h>
#include <asm/arch/irqs.h>

#if (defined SI3050_USE_LEDMAN) && (defined CONFIG_LEDMAN)
#include <linux/ledman.h>
#else
#define ledman_cmd(a,b)
#endif

#include "ixp425_spi.h"
#include "si3050_driver.h"

/* Use macro PDEBUG for printing debug messages */
#undef PDEBUG				// undef it, just in case
#ifdef SI3050_DEBUG
	#define PDEBUG(fmt, args...) 	\
		printk("[%s:%d] "fmt, __FILE__,__LINE__, ##args)
#else
	#define PDEBUG(fmt, args...)	{}
#endif

#define __unused __attribute__((__unused__))

#define SI3050_PROC_DIR		"si3050"
#define SI3050_PROC_FILE	"status"

/*
 * This struct is used to have lists of default register settings.
 */
struct si3050_reg_list
{
    unsigned char  reg;
    unsigned short val;
};

unsigned char si3050_digital_hybrid_config[] = {
	/* ACIM, HC1, HC2, HC3, HC4, HC5, HC6, HC7, HC8 */
	      3,   0,   0,   0,   0,   0,   0,   0,   0,
	      3,   0,   0,   0,   0,   0,   0,   0,   0,
	      7,   0, 255, 254, 255,   0, 255,   0,   0,
	      3, 255,   0, 255, 254,   1, 255,   0,   0,
	      9,   0, 253, 253,   1, 255,   0,   0,   0,
	      9,   0, 253, 253,   1, 255,   0,   0,   0,
	      5,   1, 249, 254,   3, 253,   1,   0,   0,
	      5,   1, 249, 254,   3, 253,   1,   0,   0,
	      5, 252, 250,   1,   1, 254,   0, 255,   0,
	      9, 254, 251, 255,   1, 254,   0, 255, 255,
	      5,   3, 251, 251,   2, 253, 255, 255, 255,
	      5,   3, 251, 251,   2, 253, 255, 255, 255,
	      8, 253,   2, 244, 255,  10, 244,   3, 253,
	      8, 253,   2, 244, 255,  10, 244,   3, 253,
	     10, 249, 244,   8,  12, 245, 252,   0,   1,
	     10, 249, 244,   8,  12, 245, 252,   0,   1,
};
	

/*
 * List of default values for direct registers.
 */
struct si3050_reg_list si3050_direct_reg_list[] = {
//	{ 7,  0x0},	/* set sample rate to 8kHz */
//	{10,  0x0},	/* no digital loopback */
//	{14,  0x0},	/* set RGDT to active low */
//	{15,  0x0},	/* TX and RX not muted */
	{16,  0x0},	/* International Cntr1 */
	/* TODO: Use FIR or IIR Filter? */
//	{17,  0x8},	/* International Cntr2 */
	{18,  0x0},	/* International Cntr3 */
//					/* Half-Wave Ring Detection */
//	{19,  0x0},	/* International Cntr4 */
//	{20,  0x0},	/* Mute RX AOUT */
//	{21,  0x0},	/* Mute TX AOUT */
	{22, 0x16},	/* RDLY, RMX max Frequ = 83 Hz*/
			
//	{23, 0x2d},	/* RDLY2, RTO, RCC */
	{24, 0x99},	/* RNGV, RAS */
			/* automatic detection + minimum freq = 10Hz */
//	{25,  0x0},     /* auto calibration */
	{26, 0xc2},	/* DC Termination Control */

	{30, 0*2+0},	/* SPECIAL Item */
	/* Configures the digital hybrid with EIA 7 and
	 * first configuration (first of two) */

	{31, 0xab},	/* DAA Control 5 */
	{33,  0x0},	/* Configure PCM */
	{38,  0x4},	/* TX Gain Control 2 */
	{39,  0x0},	/* RX Gain Control 2 */
	{40,  0x0},	/* TX Gain Control 3 */
	{41,  0x0},	/* RX Gain Control 3 */
	{42,  0x0},	/* GCI Control */
	{43,  0x5},	/* CVT */
	{44,  0x6},	/* Line Voltage Treshhold Control*/
	{ 3, 0xfe},	/* activate all interrupts    */
			/* (except polarity reversal) */
	{ 4,  0x0},	/* clear all pending interrupts */
	{ 5,  0x8}, 	/* go on-hook */
	{ 6,  0x0},	/* power up */
};

static struct proc_dir_entry* si3050_proc_dir;
	

static void si3050_configure_digital_hybrid(int chip, int eia)
{
	int i;
	unsigned char* table = si3050_digital_hybrid_config + 9*eia;
		
	si3050_write_cmd(chip, 30, table[0]);
	PDEBUG("Writing to reg %d %d\n", 30, table[0]);

	for (i=0;i<8;i++) {
		si3050_write_cmd(chip, 45+i, table[1+i]);
		PDEBUG("Writing to reg %d %d\n", 45+i, table[i+1]);
	}
}


/**
 * Writes default values to direct registers.
 * \param chip the number of the chip to be initialized [0-7]
 */
static void si3050_initialize_regs(int chip)
{
	int i;
	int entries;

	entries = sizeof(si3050_direct_reg_list) 
		/ sizeof(struct si3050_reg_list);
	
	for(i=0; i<entries; i++) {
    		unsigned char  reg = si3050_direct_reg_list[i].reg;
		unsigned short val = si3050_direct_reg_list[i].val;

		if (reg == 30)
			si3050_configure_digital_hybrid(chip, val);
		else
		/* write value to the corresponding direct register */
			si3050_write_cmd(chip, reg, val);
	}
}


/**
 * Starts pcm transfers on the given chip.
 * \param start_bit for hss configuration
 */
void si3050_start_pcm(int start_bit)
{
	/* configure hss slot */	
	si3050_write_cmd(0, 34, start_bit);
	si3050_write_cmd(0, 35, 0);
	si3050_write_cmd(0, 36, start_bit);
	si3050_write_cmd(0, 37, 0);

	/* configure pcm (+activate) */
#ifdef SI3050_USE_ULAW
	si3050_write_cmd(0, 33, 0x28);
#else
	si3050_write_cmd(0, 33, 0x20);
#endif

	PDEBUG("PCM is active (Freq = Unknown, Startbit = %d)\n", start_bit);
}


/**
 * Stops pcm transfers on the given chip.
 */
void si3050_stop_pcm(void)
{
	/* configure pcm (+activate) */
	si3050_write_cmd(0, 33, 0x0);

	PDEBUG("PCM is inactive now\n");
}
			

/**
 * Update led 
 */
static void si3050_led_update(void)
{
	int ret;
		
	/* off-hook or ringing?  */
	ret = si3050_read_cmd(0, 5);
	if (ret&4 || ret&1) {
		ledman_cmd(LEDMAN_CMD_FLASH, 8);
		return;
	}

	/* online? */

	ret = si3050_read_cmd(0, 29);
	if ((ret & 128) > 5) {
		ledman_cmd(LEDMAN_CMD_ON, 8);
		return;
	}

	ledman_cmd(LEDMAN_CMD_OFF, 8);

}


/**
 * Go off-hook
 */
void si3050_go_offhook()
{
	PDEBUG("Going Off-hook\n");

	/* go off hook */
	si3050_write_cmd(0, 5, 9);

	/* update leds */
	si3050_led_update();
}


/**
 * Go on-hook
 */
void si3050_go_onhook()
{
	PDEBUG("Going On-hook\n");

	/* activate on hook line monitor */
	si3050_write_cmd(0, 5, 8);
	
	/* update leds */
	si3050_led_update();
}


/*
 *
 * Interrupt functions
 *
 */

/*
 * This timer is used to trigger a call to a function from within an 
 * interrupt. The function will be started right after the interrupt.
 */
static struct timer_list cb_timer;
static struct timer_list irq_timer;

/* Prototype of callback function */
typedef void (*si3050_cb_func)(void *);

/* 
 * Use this function to trigger the call of a function.
 * \param func is the callback function you want to start
 * \param data will be the first parameter of the callback function
 */
static void si3050_run_cb(si3050_cb_func func, void* data)
{
	if (!func)	/* no callback function, nothing to do */
		return;

	/* because of some reason.. no timer is used */
	func(data);
	return;
	
	/* XXX somebody should check, if the timer is free */

	init_timer(&cb_timer);
	cb_timer.data = (unsigned long)data;
	cb_timer.expires = jiffies;
	cb_timer.function = (void(*)(unsigned long))func;
	add_timer(&cb_timer);
}


static void si3050_enable_irq(void* data)
{
	PDEBUG("Enable interrupt again...\n");
	enable_irq(IRQ_IXP4XX_GPIO11);
}

static void si3050_isr1(struct si3050_driver_config *config, int status)
{
	int ret;

	/* clear all interrupts */
	si3050_write_cmd(0, 4, 0);

	if (status & 0x80) {
		PDEBUG("*** Ring detect interrupt *** (%lu)\n", jiffies);
		ret = si3050_read_cmd(0, 5);
		if (ret&4) {
			config->status = RINGING;
		}  else {
			config->status = OFF_HOOK;
		}

		/* update leds */
		si3050_led_update();

		si3050_run_cb((si3050_cb_func) config->status_changed,config);
	}

	if (status & 0x40) {
		PDEBUG("Receive overload interrupt\n");
	}

	if (status & 0x20) {
		PDEBUG("Frame detect interrupt (bad!!!)\n");
	}

	if (status & 0x10) {
		PDEBUG("Billing tone detect interrupt\n");
	}

	if (status & 0x08) {
		PDEBUG("Drop out detect interrupt\n");
	}

	if (status & 0x04) {
		PDEBUG("Loop current sense overload interrupt\n");
	}

	if (status & 0x02) {
		PDEBUG("TIP ground detect interrupt\n");
	}

	if (status & 0x01) {
		PDEBUG("Polarity reversal detect interrupt\n");
		si3050_run_cb(
			(si3050_cb_func) config->polarity_changed, config);

//		ret = si3050_read_cmd(0, 29);
//		PDEBUG("Line Voltage is now %c%d\n",
//			(ret&0x80)?'-':'+', ret&0x7f);
	}
}


static irqreturn_t si3050_isr(int irq __unused, void* data, struct pt_regs* regs __unused)
{
	int ret;
	struct si3050_driver_config * config = data;

	PDEBUG("Stopping IRQ's\n");
	disable_irq_nosync(IRQ_IXP4XX_GPIO11);
	irq_timer.expires = jiffies + 50;
	add_timer(&irq_timer);

	SPI_IRQ_Start();

	/* read interrupt status */
	ret = si3050_read_cmd(0, 4);

	if (ret)
		si3050_isr1(config, ret);

	ret = si3050_read_cmd(0, 44);
	if (ret & 8) {
		si3050_write_cmd(0, 44, (ret & ~8));
		if (ret && 8) {
			PDEBUG("Line Voltage Treshhold Crossed\n");
			si3050_led_update();
		}
	}

	SPI_IRQ_Stop();

	return IRQ_HANDLED;
}

static int si3050_proc_read(char *page, char **start __unused,
			    off_t off __unused, int count __unused,
			    int *eof __unused, void *data __unused)
{
	int ret = si3050_read_cmd(0, 29);

	PDEBUG("Online/Offline Status = %d\n", ret);
	
	if ((ret & 127) > 5)
		return sprintf(page, "Online ");
	return sprintf(page, "Offline");

}



/**
 * Initializes si3050 chips.
 * \params config will be set to point to the configuration data.
 * \return 0 on success.
 */
int si3050_initialize(struct si3050_driver_config* config)
{
	int ret;
	char rev[] = "AABCDEGHJIJKLMNO";
		
	/* init config structure */
	memset(config, 0, sizeof(struct si3050_driver_config));

	/* soft reset chip */
	si3050_write_cmd(0, 1, 0x80);
	mdelay(10);
	
	/* initialized registers */
	si3050_initialize_regs(0);

	/* wait until si3019 is reachable */
	ret = 0;
	while( (si3050_read_cmd(0, 11)&0xf0) == 0) {
		mdelay(10);
		if (ret++> 1000) {
			printk("SI3018/9 not reachable\n");
			si3050_release(config);
			return -EIO;
		}
	}

	/* show ids and revisions */
	ret = si3050_read_cmd(0, 11);
	if ( ((ret&0xf0) != 0x10) && ((ret&0xf0) != 0x30)) {
		printk("Wrong line side id (%x)\n", ret>>4);
		si3050_release(config);
		return -EIO;
	}
	
	printk("Found Si3050 (Rev. %c) and Si301%d ",
		rev[ret&0xf], ((ret&0xf0)==1)?8:9);

	ret = si3050_read_cmd(0, 13);
	ret = (ret>>2)&0xf;
	printk("(Rev. %c)\n", rev[ret]);
	
	init_timer(&irq_timer);
	irq_timer.function = (void(*)(unsigned long))si3050_enable_irq;

	/* request interrupt */
	ret = request_irq(	IRQ_IXP4XX_GPIO11, si3050_isr, 
				SA_SHIRQ | SA_TRIGGER_LOW, 
				"si3050_irq", (void*)config);
	if (ret) {
		printk("Error while requesting interrupt for si3050\n");
		si3050_release(config);
		return -1;
	}

	/* enable interrupts */
	si3050_write_cmd(0, 2, 0x87);

	printk("Si3050 initialized\n");

	/* register proc file */
	si3050_proc_dir = proc_mkdir(SI3050_PROC_DIR, NULL);
	if(si3050_proc_dir == NULL) {
		printk("Error creating proc directory\n");
		return 0;
	}
	create_proc_read_entry(SI3050_PROC_FILE, 0444, si3050_proc_dir, si3050_proc_read, NULL);
	
	/* update leds */
	si3050_led_update();

	return 0;
	
}

void si3050_release(struct si3050_driver_config* config __unused)
{
	/* remove interrupt */
	free_irq(IRQ_IXP4XX_GPIO11, (void*)config);
	
	/* soft reset chip */
	si3050_write_cmd(0, 1, 0x80);

	/* remove proc file */
	remove_proc_entry(SI3050_PROC_FILE, si3050_proc_dir);
	remove_proc_entry(SI3050_PROC_DIR, NULL);
}

