/***************************************************************************
 *   Copyright (C) 2005 by Henry Margies <h.margies@maxina.de>		   *
 *   			   Sven Kolberg  <s.kolberg@maxina.de>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* maximum number of chips */
#define SI3050_MAX_CHIPS	8

/* status */
typedef enum {
	ON_HOOK,
	OFF_HOOK,
	RINGING,
} si3050_status;


struct si3050_driver_config {

	/* callbacks */
	void (*status_changed)(struct si3050_driver_config*);
	void (*dtmf_detected) (struct si3050_driver_config*);
	void (*polarity_changed)(struct si3050_driver_config*);
	
	/* current status of chip */
	si3050_status 	status;

	/* true if chip is in use */
	int		in_use;

	/* last dtmf digit detected */
	unsigned int 	dtmf;

	/* chip number [0-7] */
	unsigned int	chip_number;

	int		hss_start_bit;
	
	/* channel for echo canceller */
	int		ec_channel;
};	

/**
 * Generates a ring on the specified chip.
 * \param chip the chip where it should start ringing [0-7]
 */
void si3050_start_ringing(void);

/**
 * Changes device to off-hook.
 * \param chip the chip where it should stop ringing [0-7]
 */
void si3050_go_offhook(void);

/**
 * Changes device to on-hook.
 * \param chip the chip where it should stop ringing [0-7]
 */
void si3050_go_onhook(void);

/**
 * Sets the polarity of the line.
 * \param chip is the chip where to change the polarity
 * \param polarity 1 if inverse polarity
 */
void si3050_set_polarity(int polarity);

/**
 * Starts pcm transfers on the given chip.
 * \param chip chipnumber [0-7]
 */
void si3050_start_pcm(int start_bit);

/**
 * Stops pcm transfers on the given chip.
 * \param chip chipnumber [0-7]
 */
void si3050_stop_pcm(void);

/**
 * Turns off a single chip.
 * \param chip specifies the chip to be initialized [0-7]
 */
void si3050_poweroff(void);

int si3050_initialize(struct si3050_driver_config* config);

/* This function __should__ power off all active chips and release all 
 * resources.
 * \param config points to the driver configuration structure.
 */
void si3050_release(struct si3050_driver_config* config);


/* DEBUG functions */
void si3050_fsk_test_start(void);

