/***************************************************************************
 *   Copyright (C) 2005 by Henry Margies <h.margies@maxina.de>		   *
 *   			   Sven Kolberg  <s.kolberg@maxina.de>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* maximum number of chips */
#define SI3210_MAX_CHIPS	8

/* status */
typedef enum {
	ON_HOOK,
	OFF_HOOK,
} si3210_status;


struct si3210_chip_config {
	/* current status of chip */
	si3210_status 	status;

	/* true if chip is in use */
	int		in_use;

	/* last dtmf digit detected */
	unsigned int 	dtmf;

	/* chip number [0-7] */
	unsigned int	chip_number;

	int		hss_start_bit;

	/* channel for echo canceller */
	int		ec_channel;
};


struct si3210_driver_config {

	/* number of chips */
	int	num_of_chips;
	/* configuration for each */
	struct si3210_chip_config chip[SI3210_MAX_CHIPS];

	/* callbacks */
	void (*status_changed)(struct si3210_chip_config*);
	void (*dtmf_detected)(struct si3210_chip_config*);
};	

/**
 * Generates a ring on the specified chip.
 * \param chip the chip where it should start ringing [0-7]
 */
void si3210_start_ringing(int chip);

/**
 * Changes device to off-hook.
 * \param chip the chip where it should stop ringing [0-7]
 */
void si3210_go_offhook(int chip);

/**
 * Changes device to on-hook.
 * \param chip the chip where it should stop ringing [0-7]
 */
void si3210_go_onhook(int chip);

/**
 * Sets the polarity of the line.
 * \param chip is the chip where to change the polarity
 * \param polarity 1 if inverse polarity
 */
void si3210_set_polarity(int chip, int polarity);

/**
 * Starts pcm transfers on the given chip.
 * \param chip chipnumber [0-7]
 */
void si3210_start_pcm(int chip, int start_bit);

/**
 * Stops pcm transfers on the given chip.
 * \param chip chipnumber [0-7]
 */
void si3210_stop_pcm(int chip);

/**
 * Initializes a single chip.
 * \param chip specifies the chip to be initialized [0-7]
 * \returns zero on success
 */
int si3210_initialize_chip(int chip);

/**
 * Turns off a single chip.
 * \param chip specifies the chip to be initialized [0-7]
 */
void si3210_poweroff_chip(int chip);

/**
 * This function performs a basic setup of all si3210 chips.
 * Call ixp425_hss_init and ixp425_spi_init before. The driver
 * configuration will be initialized, so setting values before 
 * makes no sense.
 * \param config points to the driver configuration structure.
 * \returns zero on success
 */
int si3210_initialize_chips(struct si3210_driver_config* config);

/* This function __should__ power off all active chips and release all 
 * resources.
 * \param config points to the driver configuration structure.
 */
void si3210_release_chips(struct si3210_driver_config* config);


/* DEBUG functions */
void si3210_fsk_test_start(int chip);

