/***************************************************************************
 *   Copyright (C) 2005 by Henry Margies <h.margies@maxina.de>		   *
 *   			   Sven Kolberg  <s.kolberg@maxina.de>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
/*
* Si3210 direct registers' addresses.
*/

/* Setup Registers */
#define SI3210_SPI_MODE_SELECT_REG				0
#define SI3210_PCM_MODE_SELECT_REG				1
#define SI3210_PCM_TX_START_COUNT_LB_REG			2
#define SI3210_PCM_TX_START_COUNT_HB_REG			3
#define SI3210_PCM_RX_START_COUNT_LB_REG			4
#define SI3210_PCM_RX_START_COUNT_HB_REG			5
#define SI3210_DIGITAL_IO_CTRL_REG				6

/* Audio Registers */
#define SI3210_AUDIO_PATH_LOOPBACK_REG				8
#define SI3210_AUDIO_GAIN_CTRL_REG				9
#define SI3210_TWO_WIRE_IMPEDANCE_SYNTH_CTRL_REG		10
#define SI3210_HYBRID_CTRL_REG					11

/* Powerdown Registers */
#define SI3210_POWER_DOWN_CTRL_1_REG				14
#define SI3210_POWER_DOWN_CTRL_2_REG				15

/* Interrupt Registers */
#define SI3210_INTERRUPT_STATUS_1_REG				18
#define SI3210_INTERRUPT_STATUS_2_REG				19
#define SI3210_INTERRUPT_STATUS_3_REG				20
#define SI3210_INTERRUPT_ENABLE_1_REG				21
#define SI3210_INTERRUPT_ENABLE_2_REG				22
#define SI3210_INTERRUPT_ENABLE_3_REG				23
#define SI3210_DECODE_STATUS_REG				24

/* Oscillators Registers */
#define SI3210_OSCIL_1_CTRL_REG					32
#define SI3210_OSCIL_2_CTRL_REG					33
#define SI3210_RINGING_OSCIL_CTRL_REG				34
#define SI3210_PULSE_METERING_OSCIL_CTRL_REG			35
#define SI3210_OSCIL_1_ACTIVE_TIMER_LB_REG			36
#define SI3210_OSCIL_1_ACTIVE_TIMER_HB_REG			37
#define SI3210_OSCIL_1_INACTIVE_TIMER_LB_REG			38
#define SI3210_OSCIL_1_INACTIVE_TIMER_HB_REG			39
#define SI3210_OSCIL_2_ACTIVE_TIMER_LB_REG			40
#define SI3210_OSCIL_2_ACTIVE_TIMER_HB_REG			41
#define SI3210_OSCIL_2_INACTIVE_TIMER_LB_REG			42
#define SI3210_OSCIL_2_INACTIVE_TIMER_HB_REG			43
#define SI3210_PULSE_METERING_OSCIL_ACTIVE_TIMER_LB_REG		44
#define SI3210_PULSE_METERING_OSCIL_ACTIVE_TIMER_HB_REG		45
#define SI3210_PULSE_METERING_OSCIL_INACTIVE_TIMER_LB_REG	46
#define SI3210_PULSE_METERING_OSCIL_INACTIVE_TIMER_HB_REG	47
#define SI3210_RINGING_OSCIL_ACTIVE_TIMER_LB_REG		48
#define SI3210_RINGING_OSCIL_ACTIVE_TIMER_HB_REG		49
#define SI3210_RINGING_OSCIL_INACTIVE_TIMER_LB_REG		50
#define SI3210_RINGING_OSCIL_INACTIVE_TIMER_HB_REG		51
#define SI3210_FSK_DATA_REG52

/* SLIC Registers */
#define SI3210_LOOP_CLOSR_DEB_INTRVL__4_AUTO_RING_REG		63
#define SI3210_LINEFEED_CTRL_REG				64
#define SI3210_EXTERNAL_BIPOLAR_TRANSISTOR_CTRL_REG		65
#define SI3210_BATTERY_FEED_CTRL_REG				66
#define SI3210_AUTO_MANUAL_CONTROL_REG				67
#define SI3210_LOOP_CLOSR_RING_DETECT_REG			68
#define SI3210_LOOP_CLOSURE_DEBOUNCE_INTERVAL_REG		69
#define SI3210_RING_TRIP_DETECT_DEBOUNCE_INTERVAL_REG		70
#define SI3210_LOOP_CURRENT_LIMIT_REG				71
#define SI3210_ONHOOK_LINE_VOLTAGE_REG				72
#define SI3210_COMMON_MODE_VOLTAGE_REG				73
#define SI3210_HIGH_BATTERY_VOLTAGE_REG				74
#define SI3210_LOW_BATTERY_VOLTAGE_REG				75
#define SI3210_POWER_MONITOR_POINTER_REG			76
#define SI3210_LINE_POWER_OUTPUT_MONITOR_REG			77
#define SI3210_LOOP_VOLTAGE_SENSE_REG				78
#define SI3210_LOOP_CURRENT_SENSE_REG				79
#define SI3210_TIP_VOLTAGE_SENSE_REG				80
#define SI3210_RING_VOLTAGE_SENSE_REG				81
#define SI3210_BATTERY_VOLTAGE_SENSE_1_REG			82
#define SI3210_BATTERY_VOLTAGE_SENSE_2_REG			83
#define SI3210_TRANSISTOR_1_CURRENT_SENSE_REG			84
#define SI3210_TRANSISTOR_2_CURRENT_SENSE_REG			85
#define SI3210_TRANSISTOR_3_CURRENT_SENSE_REG			86
#define SI3210_TRANSISTOR_4_CURRENT_SENSE_REG			87
#define SI3210_TRANSISTOR_5_CURRENT_SENSE_REG			88
#define SI3210_TRANSISTOR_6_CURRENT_SENSE_REG			89
#define SI3210_DC_DC_CONVERTER_PWM_PERIOD_REG			92
#define SI3210_DC_DC_CONVERTER_SWITCHING_DELAY_REG		93
#define SI3210_PWM_PULSE_WIDTH_REG				94
#define SI3210_CALIB_CONTROL_STATUS_1_REG			96
#define SI3210_CALIB_CONTROL_STATUS_2_REG			97
#define SI3210_RING_GAIN_MISMATCH_CALIB_RESULT_REG		98
#define SI3210_TIP_GAIN_MISMATCH_CALIB_RESULT_REG		99
#define SI3210_DIFF_LOOP_CURRENT_GAIN_CALIB_RESULT_REG		100
#define SI3210_COMMON_MODE_LOOP_CURRENT_GAIN_CALIB_RES_REG	101
#define SI3210_CURRENT_LIMIT_CALIB_RESULT_REG			102
#define SI3210_MONITOR_ADC_OFFSET_CALIB_RESULT_REG		103
#define SI3210_ANALOG_DAC_ADC_OFFSET_REG			104
#define SI3210_DAC_OFFSET_CALIB_RESULT				105
#define SI3210_COMMON_MODE_BALANCE_CALIB_RESULT_REG		106
#define SI3210_DC_PEAK_VOLTAGE_CALIB_RESULT_REG			107
#define SI3210_ENHANCEMENT_ENABLE_REG				108

/* Scratch pad register addresses in the device */
#define SI3210_SCRATCH_PAD_0					97
#define SI3210_SCRATCH_PAD_1					193
#define SI3210_SCRATCH_PAD_2					194
#define SI3210_SCRATCH_PAD_3					195
#define SI3210_SCRATCH_PAD_4					196
#define SI3210_SCRATCH_PAD_5					197
#define SI3210_SCRATCH_PAD_6					198
#define SI3210_SCRATCH_PAD_7					199
#define SI3210_SCRATCH_PAD_8					200
#define SI3210_SCRATCH_PAD_9					201
#define SI3210_SCRATCH_PAD_10					202
#define SI3210_SCRATCH_PAD_11					203
#define SI3210_SCRATCH_PAD_12					204
#define SI3210_SCRATCH_PAD_13					205
#define SI3210_SCRATCH_PAD_14					206
#define SI3210_SCRATCH_PAD_15					207
#define SI3210_SCRATCH_PAD_16					208
#define SI3210_SCRATCH_PAD_17					209
#define SI3210_SCRATCH_PAD_18					210
#define SI3210_SCRATCH_PAD_19					211

/* Indirect Register Access Registers */
#define SI3210_INDIRECT_DATA_ACCESS_LOW_REG			28
#define SI3210_INDIRECT_DATA_ACCESS_HIGH_REG			29
#define SI3210_INDIRECT_ADDRESS_REG				30
#define SI3210_INDIRECT_ADDRESS_STATUS_REG			31


/*
 * Interrupt enable bits. These bits can be logically ORed in order to
 * define which interrupts are to be enabled
 */
#define SI3210_OSC_1_ACTIVE_TIMER_INT_ENABLE            0x00000001
#define SI3210_OSC_1_INACTIVE_TIMER_INT_ENABLE          0x00000002
#define SI3210_OSC_2_ACTIVE_TIMER_INT_ENABLE            0x00000004
#define SI3210_OSC_2_INACTIVE_TIMER_INT_ENABLE          0x00000008
#define SI3210_RINGING_ACTIVE_TIMER_INT_ENABLE          0x00000010
#define SI3210_RINGING_INACTIVE_TIMER_INT_ENABLE        0x00000020
#define SI3210_PULSE_METERING_ACTIVE_TIMER_INT_ENABLE   0x00000040
#define SI3210_PULSE_METERING_INACTIVE_TIMER_INT_ENABLE 0x00000080
#define SI3210_RING_TRIP_INT_ENABLE                     0x00000100
#define SI3210_LOOP_CLOSURE_TRANS_INT_ENABLE            0x00000200
#define SI3210_POWER_ALARM_Q_1_INT_ENABLE               0x00000400
#define SI3210_POWER_ALARM_Q_2_INT_ENABLE               0x00000800
#define SI3210_POWER_ALARM_Q_3_INT_ENABLE               0x00001000
#define SI3210_POWER_ALARM_Q_4_INT_ENABLE               0x00002000
#define SI3210_POWER_ALARM_Q_5_INT_ENABLE               0x00004000
#define SI3210_POWER_ALARM_Q_6_INT_ENABLE               0x00008000
#define SI3210_DTMF_DETECTED_INT_ENABLE                 0x00010000
#define SI3210_INDIRECT_REG_ACCESS_SERVICED_INT_ENABLE  0x00020000
#define SI3210_COM_MODE_CALIB_ERROR_INT_ENABLE          0x00040000

/*
 * Interrupt pending bits. Status Bits
 */
#define SI3210_OSC_1_ACTIVE_TIMER_INT_PENDING            0x00000001
#define SI3210_OSC_1_INACTIVE_TIMER_INT_PENDING          0x00000002
#define SI3210_OSC_2_ACTIVE_TIMER_INT_PENDING            0x00000004
#define SI3210_OSC_2_INACTIVE_TIMER_INT_PENDING          0x00000008
#define SI3210_RINGING_ACTIVE_TIMER_INT_PENDING          0x00000010
#define SI3210_RINGING_INACTIVE_TIMER_INT_PENDING        0x00000020
#define SI3210_PULSE_METERING_ACTIVE_TIMER_INT_PENDING   0x00000040
#define SI3210_PULSE_METERING_INACTIVE_TIMER_INT_PENDING 0x00000080

#define SI3210_RING_TRIP_INT_PENDING                     0x00000001
#define SI3210_LOOP_CLOSURE_TRANS_INT_PENDING            0x00000002
#define SI3210_POWER_ALARM_Q_1_INT_PENDING               0x00000004
#define SI3210_POWER_ALARM_Q_2_INT_PENDING               0x00000008
#define SI3210_POWER_ALARM_Q_3_INT_PENDING               0x00000010
#define SI3210_POWER_ALARM_Q_4_INT_PENDING               0x00000020
#define SI3210_POWER_ALARM_Q_5_INT_PENDING               0x00000040
#define SI3210_POWER_ALARM_Q_6_INT_PENDING               0x00000080
#define SI3210_POWER_ALARM				 0x000000FC

#define SI3210_DTMF_DETECTED_INT_PENDING                 0x00000001
#define SI3210_INDIRECT_REG_ACCESS_SERVICED_INT_PENDING  0x00000002
#define SI3210_COM_MODE_CALIB_ERROR_INT_PENDING          0x00000004

/* Define the indices of the interrupt vector table */
#define SI3210_SC_T1_ACTIVE_HANDLER_INDEX             0
#define SI3210_SC_T1_INACTIVE_HANDLER_INDEX           1
#define SI3210_SC_T2_ACTIVE_HANDLER_INDEX             2
#define SI3210_SC_T2_INACTIVE_HANDLER_INDEX           3
#define SI3210_SC_RINGING_ACTIVE_HANDLER_INDEX        4
#define SI3210_SC_RINGING_INACTIVE_HANDLER_INDEX      5
#define SI3210_SC_METERING_ACTIVE_HANDLER_INDEX       6
#define SI3210_SC_METERING_INACTIVE_HANDLER_INDEX     7
#define SI3210_SC_RING_TRIP_HANDLER_INDEX             8
#define SI3210_SC_HOOK_SWITCH_HANDLER_INDEX           9
#define SI3210_SC_POWER_ALARM1_HANDLER_INDEX          10
#define SI3210_SC_POWER_ALARM2_HANDLER_INDEX          11
#define SI3210_SC_POWER_ALARM3_HANDLER_INDEX          12
#define SI3210_SC_POWER_ALARM4_HANDLER_INDEX          13
#define SI3210_SC_POWER_ALARM5_HANDLER_INDEX          14
#define SI3210_SC_POWER_ALARM6_HANDLER_INDEX          15
#define SI3210_SC_DTMF_HANDLER_INDEX                  16
#define SI3210_SC_INDIRECT_REG_ACC_COMP_HANDLER_INDEX 17
#define SI3210_SC_CALIBRATION_ERROR_HANDLER           18

/*
 * The largest direct register address in the Si3210 is 108
 * While the largest indirect register address in the Si3210 is 104
 */
#define SI3210_SC_MAX_DIRECT_REG_ADDRESS 108
#define SI3210_SC_MAX_INDIRECT_REG_ADDRESS 104

/*
 * There are 19 interrupts per Si3210 device
 */ 
#define SI3210_SC_MAX_NUM_INTS_PER_DEVICE 19

/*
 * The line onook mask
 */
#define SI3210_SC_ONHOOK_MASK 0x04


#define SI3210_SPI_DAISY_CHAIN_MODE_BIT         0x80
