/*
 * Copyright (C) 2003-2005 Maxina GmbH - Jordan Hrycaj
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 * Author: Jordan Hrycaj <jordan@mjh.teddy.net.com>
 *
 * $Id: ipcalc.h 743 2005-12-10 16:43:51Z jordan $
 */

#ifndef __STUNC_IPCALC_H
#define __STUNC_IPCALC_H

/* quad long */
typedef struct { uint8_t oct [16]; } uint128_t ;

/* ip v4 address length, just to be used similar to ETH_ALEN */
#define IP4_ALEN 4

/* unaligned ip address pointer to ip4addr_t */
#define ip2ip4addr(ip)      ((((((((ip4addr_t)(((uint8_t*)(ip)) [0]))\
	                       << 8) + (ip4addr_t)(((uint8_t*)(ip)) [1]))\
	                       << 8) + (ip4addr_t)(((uint8_t*)(ip)) [2]))\
	                       << 8) + (ip4addr_t)(((uint8_t*)(ip)) [3]))

/* ip4addr_t data value to unaligned address in memory */
#define ip4addr2ip(ptr,ip)  (((uint8_t*)(ptr)) [0] = ((ip) >> 24),\
			                 ((uint8_t*)(ptr)) [1] = ((ip) >> 16),\
			                 ((uint8_t*)(ptr)) [2] = ((ip) >>  8),\
			                 ((uint8_t*)(ptr)) [3] = ((ip)      ))

#if    __BYTE_ORDER == __LITTLE_ENDIAN

# define N16TOH(s) (((( (uint16_t) (s))>>8)&0xff)|\
                    ((( (uint16_t) (s))<<8)&0xff00))

# define n16toh(s) ((((*(uint16_t*)(s))>>8)&0xff)|\
                    (((*(uint16_t*)(s))<<8)&0xff00))

#elif  __BYTE_ORDER == __BIG_ENDIAN
# define N16TOH(s) ( (uint16_t*)(s))
# define n16toh(s) (*(uint16_t*)(s))
#else
# error unknown byte order LITTLE or BIG ENDIAN 
#endif

#define H16TON(s) N16TOH(s)
#define h16ton(s) n16toh(s)

#define ldx16b(p) \
        ((uint16_t)*((const uint8_t*)(p) + 0) << 8 |\
         (uint16_t)*((const uint8_t*)(p) + 1)) 

#define stx16b(p,val) \
        (*((uint8_t*)(p) + 0) = (uint16_t)(val) >> 8,\
         *((uint8_t*)(p) + 1) = (uint16_t)(val)     )

#define ip2l(ip) ntohl (inet_addr (ip))

/* some checksum helpers */
extern uint16_t   ip_cksum (const struct ip*);
extern uint16_t  udp_cksum (const struct ip*, unsigned short max);
extern uint16_t  tcp_cksum (const struct ip*, unsigned short max);
extern uint16_t icmp_cksum (const struct ip*, unsigned short max);

/* pretty print integer pointer values */
extern const char  *ulongp_pp (const      long *);  /* arg may be NULL */
extern const char *ushortp_pp (const     short *);  /* arg may be NULL */
extern const char  *xlongp_pp (const      long *);  /* arg may be NULL */
extern const char  *xquadp_pp (const uint128_t *);

#endif /* __STUNC_IPCALC_H */
