#
# nvram lib - flash data base client library
#
# Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA
#
# Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-Isenburg, Germany
#
# $Id: acinclude.m4 3182 2006-09-25 10:49:27Z jordan $
#

AC_DEFUN([AC_ENABLE_DEBUG],[
  AM_WITH_DMALLOC
  AC_ARG_ENABLE([debug],
    [AC_HELP_STRING([--enable-debug],
    [enable source code debug mode])])
  AM_CONDITIONAL([USE_DEBUG], [test x"${enable_debug-no}" != xno])
  AC_ARG_ENABLE([squeeze],
    [AC_HELP_STRING([--enable-squeeze],
    [squeeze the source code, omit help texts])])
  AM_CONDITIONAL([USE_SQUEEZE], [test x"${enable_squeeze-no}" != xno])
  if test x"${with_dmalloc-no}" != xno; then
    enable_debug=yes
  fi
  if test x"${enable_debug-no}" != xno; then
    CFLAGS=`echo "$CFLAGS" | sed 's/-O@<:@0-9@:>@*//'`
    AC_SUBST([CFLAGS])
    AC_DEFINE([USE_DEBUG], [], [enable source code debugging mode])
  fi
  if test x"${enable_squeeze-no}" != xno; then
    AC_DEFINE([DISCARD_INFO], [],
      [squeeze source, omit description and help line])
    AC_DEFINE([DISCARD_EXTENSIVE_LOGGING], [],
      [squeeze source, omit extensive logging])
  fi
])


AC_DEFUN([AC_DISABLE_HTTP],[
  AC_ARG_ENABLE([http], [AC_HELP_STRING([--disable-http],
      [do not provide the http communication intercface])])
  if test x"${enable_http-yes}" = xno; then
      AC_DEFINE([DISCARD_HTTP],[],
                [do not provide the http communication intercface])
  fi
])


AC_DEFUN([AC_ENABLE_LIBNVRAM],[
  AC_ARG_ENABLE([nvram_runtime_only],
    [AC_HELP_STRING([--enable-nvram-runtime-only],
    [do not install headers and other development files from the nvram lib])])
  AC_ARG_ENABLE([installed_nvram],
    [AC_HELP_STRING([--enable-installed-nvram],
    [use installed nvram lib version when available])])
  AM_CONDITIONAL([INSTALL_NVRAM_LIB],
    [test x"${enable_installed_nvram-no}" = xno])
  nvram_runtime=configured
])


AC_DEFUN([AC_DISABLE_LIBNVRAM],[
  AC_ARG_ENABLE([installed_nvram],
    [AC_HELP_STRING([--disable-installed-nvram],
    [use the local nvram lib version rather than any installed one])])
  AM_CONDITIONAL([LOCAL_NVRAM_LIB], [test x"${enable_installed_nvram-no}" = xno])
  AM_CONDITIONAL([USE_LOCAL_REQUEST_FILE], [test x"${enable_local-no}" != xno])
  nvram_runtime=auto
])


# check whether we should use the installed version of the nvram
AC_DEFUN([AC_CHECK_LIBNVRAM],[
  do_have_libnvram=yes
  dont_have_libnvram=yes
  AC_CHECK_HEADERS([nvram/util.h nvram/nvram-lib.h nvram/buffer.h \
                    nvram/logger.h nvram/base64.h \
                    nvram/tvcalc.h nvram/varstr.h],
                   [dont_have_libnvram=no],[
    if test x"${enable_installed_nvram}" = xyes; then
       AC_MSG_FAILURE([The nvram-lib does not seem to be properly installed for use])
    fi
    do_have_libnvram=no])
  AC_CHECK_LIB([nvram],[nvcl_putcmd],[dont_have_libnvram=no],[
    if test x"${enable_installed_nvram}" = xyes; then
      AC_MSG_FAILURE([The nvram-lib does not seem to be properly installed for linking])
    fi
    do_have_libnvram=no])
  if test x"${enable_installed_nvram-set}" = xset; then
    if test x${do_have_libnvram} = xyes; then
      if test x"${enable_nvram_runtime_only-no}" != xno; then
        AC_MSG_FAILURE([The runtime-only option is not available with the installed nvram library])
      fi
      if test x"${enable_local-no}" != xno; then
        AC_MSG_FAILURE([The local socket option is not available with the installed nvram library])
      fi
      enable_installed_nvram=yes
      ac_configure_args="$ac_configure_args --enable-installed-nvram"
    else
      enable_installed_nvram=no
      ac_configure_args="$ac_configure_args --disable-installed-nvram"
    fi
  fi
  if test x"${enable_installed_nvram-no}" = xno ; then
    if test x"${dont_have_libnvram}" = xyes -a \
            x"${nvram_runtime}"      = xauto; then
      ac_configure_args="$ac_configure_args --enable-nvram-runtime-only"
      enable_nvram_runtime_only=yes
    fi
  else
    AC_DEFINE([HAVE_NVRAM_LIB], [], [have the nvram lib preinstalled])
  fi
  AM_CONDITIONAL([HAVE_NVRAM_LIB], [test x"${enable_installed_nvram-no}" != xno])
  AC_CONFIG_SUBDIRS([libnvram])
])


AC_DEFUN([AC_MSG_LIBNVRAM],[
  if test x"${enable_installed_nvram-no}" = xno ; then
    AC_MSG_NOTICE([Using the local version of the nvram library])
  else
    AC_MSG_NOTICE([Using the installed version of the nvram library])
  fi
])
