/*
 * Copyright (C) 2006 Maxina GmbH - Jordan Hrycaj
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 * Author: Jordan Hrycaj <jordan@mjh.teddy.net.com>
 *
 * $Id: nvalloc.c 3216 2006-10-06 08:59:51Z henry $
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef STDC_HEADERS
# include <stdlib.h>
# include <stddef.h>
#else
# ifdef HAVE_STDLIB_H
#  include <stdlib.h>
# endif
#endif

#ifdef HAVE_STRING_H
# if !defined (STDC_HEADERS) && defined (HAVE_MEMORY_H)
#  include <memory.h>
# endif
# include <string.h>
#endif

#ifdef HAVE_ERRNO_H
# include <errno.h>
#endif

#include "logger.h"
#include "nvalloc.h"
#include "util.h"

/* ----------------------------------------------------------------------- *
 * Definitions ..
 * ----------------------------------------------------------------------- */

#ifndef HAVE_MALLOC
#define HAVE_MALLOC 0
#endif
#ifndef HAVE_REALLOC
#define HAVE_REALLOC 0
#endif

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

#ifndef WITH_DMALLOC

/* secure malloc */
VOID *_malloc (size_t n) {

    VOID *p ;

#   if HAVE_MALLOC == 0
    /* Allocate an N-byte block of memory from the heap.
       If N is zero, allocate a 1-byte block.  */
#   undef malloc
    if (n == 0) n = 1 ;
#   endif
	    
    if ((p = malloc (n)) == 0) {
        logger (LOG_ERR, "%s: malloc (%u) returned NULL (%s)!\n",
                progname, n, strerror (errno));
        exit (2) ;
    }
    return p ;
}


/* secure realloc, does not initialize memory */
VOID *_realloc (void  *s, size_t n) {

    char *p ;

    /* Allocate an N-byte block of memory from the heap. If N is zero,
       allocate a 1-byte block. GNU compatibility says that
       realloc() <=> free() if N is zero which is not wanted, here */

    if (n == 0) n = 1 ;

#   if HAVE_REALLOC == 1
    p = realloc (s, n) ; /* GNU compatible: s == 0 is ok */
#   else
#   undef realloc
    p = (s == 0 ? malloc (n) : realloc (s, n)) ;
#   endif

    if (p == 0) {
        logger (LOG_ERR, "%s: realloc (%#x,%u) returned NULL (%s)!\n",
                progname, s, n, strerror (errno));
        exit (2) ;
    }

    if (s == 0)
        memset (p, 0, n);
        
    return (VOID*)p;
}

#endif /* WITH_DMALLOC */

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
