#
# nvtools lib - flash data base support tools
#
# Copyright (C) 2003-2006 Maxina GmbH - Jordan Hrycaj <jordan@maxina.de>
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA
#
# Ackn: Sources reused from SHAT - Source Hardware Address Translator
#       Jordan Hrycaj <jordan@mjh.teddy-net.com>
#
# $Id: acinclude.m4 2666 2006-06-23 09:53:08Z jordan $
#


AC_DEFUN([AC_ENABLE_SIMPLECTRL],[
  AC_ARG_ENABLE([simple_ctrl],
    [AC_HELP_STRING([--enable-simple-ctrl],
    [ctrl does not use the recvmsg() call and will not provide user acccess control])])
])


AC_DEFUN([AC_ENABLE_LIBNVTOOLS],[
  AC_ARG_ENABLE([local],[AC_HELP_STRING([--enable-local],
    [puts the request socket file into current directory])])
  AC_ARG_ENABLE([nvtools_runtime_only],
    [AC_HELP_STRING([--enable-nvtools-runtime-only],
    [do not install headers and other development files from the nvtools lib])])
  AC_ARG_ENABLE([installed_nvtools],
    [AC_HELP_STRING([--enable-installed-nvtools],
    [use installed nvtools lib version when available])])
  AM_CONDITIONAL([INSTALL_NVTOOLS_LIB], [test x"${enable_installed_nvtools-no}" = xno])
  if test x"${enable_local-no}" != xno; then
    AC_DEFINE([USE_LOCAL_REQUEST_FILE], [],
      [put all the request sockets into the current directory])
  fi
  nvtools_runtime=configured
])


AC_DEFUN([AC_DISABLE_LIBNVTOOLS],[
  AC_ARG_ENABLE([local],[AC_HELP_STRING([--enable-local],
    [puts the request socket file into current directory])])
  AC_ARG_ENABLE([installed_nvtools],
    [AC_HELP_STRING([--disable-installed-nvtools],
    [use the local nvtools lib version rather than any installed one])])
  AM_CONDITIONAL([LOCAL_NVTOOLS_LIB], [test x"${enable_installed_nvtools-no}" = xno])
  nvtools_runtime=auto
])


AC_DEFUN([AC_WITHOUT_OPENSSL],[
  AC_ARG_WITH([ssl], AC_HELP_STRING([--with-ssl=DIR],
    [location of installed OpenSSL libraries/include files,
     or "no" to disable ssl]),
    [case "$withval" in
      no) disable_ssl=yes ;;
     yes) disable_ssl=no ;;
       *) disable_ssl=no
          LIBSSL="-L${withval}/lib"
          INCSSL="-I${withval}/include"
     esac],
    [disable_ssl=check])	    
])



AC_DEFUN([AC_CHECK_SIMPLECTRL],[
  AC_CHECK_TYPES([struct cmsghdr, struct ucred],[],[
    if test x"i$enable_simple_ctrl" = xno; then
      enable_simple_ctrl=forced
    fi],[
#ifdef HAVE_SYS_SOCKET_H
# include <sys/socket.h>
#endif
#ifdef HAVE_LINUX_SOCKET_H
# include <linux/socket.h>
#endif
])
  AC_MSG_CHECKING([whether we can and should use uid/gid check for control sockets])
  case "$enable_simple_ctrl" in
     yes) AC_MSG_RESULT([deconfigured]) ;;
  forced) AC_MSG_RESULT([impossible]) ;;
       *) AC_MSG_RESULT([yes])
          enable_simple_ctrl=no
  esac
  if test x"i$enable_simple_ctrl" != xno; then
    AC_DEFINE([DONT_USE_RECVMSG], [],
        [do not check the sender uid/gid of the request socket])
  fi
])

AC_DEFUN([AC_MSG_SIMPLECTRL],[
  case "$enable_simple_ctrl" in
     yes) AC_MSG_NOTICE([Runtime will check the sender uid/gid of the request sockets]);;
  forced) AC_MSG_NOTICE([OS support missing: no runtime check of the sender uid/gid of the request sockets]);;
  no)  AC_MSG_NOTICE([Runtime will check any sender uid/gid of the request sockets]);
  esac
])



# check whether we should use the installed version of the nvtools
AC_DEFUN([AC_CHECK_LIBNVTOOLS],[
  do_have_libnvtools=yes
  dont_have_libnvtools=yes
  AC_CHECK_HEADERS([nvtools/ctrl.h nvtools/do.h \
                    nvtools/prefix.h nvtools/hlst.h],
                   [dont_have_libnvtools=no],[
    if test x"${enable_installed_nvtools}" = xyes; then
      AC_MSG_FAILURE([The nvtools-lib does not seem to be properly installed for use])
    fi
    do_have_libnvtools=no])
  ac_save_LDFLAGS="$LDFLAGS"
  if test x"${enable_installed_nvram-no}" = xno ; then
    LIBNVRAMPATH=
  else
    LIBNVRAMPATH=-Llibnvram
  fi
  AC_CHECK_LIB([nvtools],[ctrl_init],[dont_have_libnvtools=no],[
    if test x"${enable_installed_nvtools}" = xyes; then
      AC_MSG_FAILURE([The nvtools-lib does not seem to be properly installed for linking])
    fi
    do_have_libnvtools=no],[$LIBNVRAMPATH -lnvram])
  if test x"${enable_installed_nvtools-set}" = xset; then
    if test x${do_have_libnvtools} = xyes; then
      if test x"${enable_nvtools_runtime_only-no}" != xno; then
        AC_MSG_FAILURE([The runtime-only option is not available with the installed nvtools library])
      fi
      if test x"${enable_local-no}" != xno; then
        AC_MSG_FAILURE([The local socket option is not available with the installed nvtools library])
      fi
      enable_installed_nvtools=yes
      ac_configure_args="$ac_configure_args --enable-installed-nvtools"
    else
      enable_installed_nvram=no
      ac_configure_args="$ac_configure_args --disable-installed-nvtools"
    fi
  fi
  if test x"${enable_installed_nvtools-no}" = xno ; then
    if test x"${dont_have_libnvtools}" = xyes -a \
            x"${nvtools_runtime}"      = xauto; then
      ac_configure_args="$ac_configure_args --enable-nvtools-runtime-only"
      enable_nvtools_runtime_only=yes
    fi
  else
    AC_DEFINE([HAVE_NVTOOLS_LIB], [], [have the nvtools lib preinstalled])
  fi
  AM_CONDITIONAL([HAVE_NVTOOLS_LIB], [test x"${enable_installed_nvtools-no}" != xno])
  AC_CONFIG_SUBDIRS([libnvtools])
])


AC_DEFUN([AC_MSG_LIBNVTOOLS],[
  if test x"${enable_installed_nvtools-no}" = xno ; then
    AC_MSG_NOTICE([Using the local version of the nvtools library])
  else
    AC_MSG_NOTICE([Using the installed version of the nvtools library])
  fi 
])

# Check for SSL directory -- taken from stunnel configure.ac
AC_DEFUN([AC_CHECK_OPENSSL],[
  ac_save_LDFLAGS="$LDFLAGS"
  ac_save_CFLAGS="$CFLAGS"
  LDFLAGS="$LDFLAGS $LIBSSL"
  CFLAGS="$CFLAGS $INCSSL"

  AC_CHECK_HEADER(openssl/ssl.h,,
   [case "$disable_ssl" in
    no)    AC_MSG_ERROR([Could not find OpenSSL header files,
                         use --with-ssl option to fix this problem]) ;;
    check) AC_MSG_NOTICE([Could not find OpenSSL header files - disabling ssl])
           disable_ssl=yes
    esac])

  if (test x"$disable_ssl" = xyes); then
    LIBSSL=
    INCSSL=
  else
    AC_CHECK_LIB(crypto,AES_encrypt,[LIBSSL="$LIBSSL -lcrypt"],
     [if test x"$disable_ssl" = xno; then
      AC_MSG_ERROR([Could not find your OpenSSL crypto lib installation,
                    use --with-ssl option to fix this problem])
      fi])
  
    AC_CHECK_LIB(ssl,tls1_new,[LIBSSL="$LIBSSL -lssl"],
     [if test x"$disable_ssl" = xno; then
      AC_MSG_ERROR([Could not find your OpenSSL ssl lib installation, 
                    use --with-ssl option to fix this problem])
      fi])

    disable_ssl=no
    AC_DEFINE(HAVE_OPENSSL, [], [System provides OpenSSL])
  fi

  AC_SUBST([LIBSSL])
  AC_SUBST([INCSSL])

  LDFLAGS="$ac_save_LDFLAGS"
  CFLAGS="$ac_save_CFLAGS"

  AM_CONDITIONAL([USE_OPENSSL], [test x"${disable_ssl}" != xyes])
])


AC_DEFUN([AC_MSG_OPENSSL],[
  if test x"$disable_ssl" = xyes ; then
    AC_MSG_NOTICE([Not using OpenSSL])
  else
    AC_MSG_NOTICE([Using OpenSSL])
  fi 
])

