/*
 * Copyright (C) 2003-2005 Maxina GmbH - Jordan Hrycaj
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 * Author: Jordan Hrycaj <jordan@mjh.teddy.net.com>
 *
 * $Id: pool.h 528 2005-11-22 18:02:23Z jordan $
 */

#ifndef __SHAT_POOL_H
#define __SHAT_POOL_H

#include "util.h"

typedef
struct _ip4_pool {
#   ifdef __POOL_PRIVATE
    int entries ;                  /* number of used links */
    struct _ip4_pool *octet [32] ; /* is not really an octet but a sextet */
#   else
    char any ;
#   endif
} IP4_POOL ;


// typedef
// struct _u16_pool {
// #   ifdef __POOL_PRIVATE
//     int entries ;                  /* number of used links */
//     struct _u16_pool *octet [16] ;
// #   else
//     char any ;
// #   endif
// } U16_POOL ;


// typedef
// struct _mac_pool {
// #   ifdef __POOL_PRIVATE
//     int entries ;                  /* number of used links */
//     struct _mac_pool *octet [64] ; /* is really a septet */
// #   else
//     char any ;
// #   endif
// } MAC_POOL ;


typedef
struct _ip4_walk {
#   ifdef __POOL_PRIVATE
    IP4_POOL *ip ;
    struct {                       /* convenient wrapper for bzero() */
        struct {
            IP4_POOL *ip ;
            unsigned inx ;
        } node [7];
    } cache ;
#   else
    char any ;
#   endif
} IP4_WALK ;


// typedef
// struct _u16_walk {
// #   ifdef __POOL_PRIVATE
//     U16_POOL *u16 ;
//     struct {                       /* convenient wrapper for bzero() */
//         struct {
//             U16_POOL *u16 ;
//             unsigned inx ;
//         } node [4];
//     } cache ;
// #   else
//     char any ;
// #   endif
// } U16_WALK ;



// typedef
// struct _mac_walk {
// #   ifdef __POOL_PRIVATE
//     MAC_POOL *mac ;
//     struct {                       /* convenient wrapper for bzero() */
//         struct {
//             MAC_POOL *mac ;
//             unsigned inx ;
//         } node [8];
//     } cache ;
// #   else
//     char any ;
// #   endif
// } MAC_WALK ;


extern IP4_POOL *ip4_pool_init  (void);
// extern unsigned  ip4_pool_empty (IP4_POOL*);
extern IP4_POOL *ip4_pool_merge (IP4_POOL*, IP4_POOL*);
extern void      ip4_pool_close (IP4_POOL*, void(*)(void*));
extern void      ip4_pool_flush (IP4_POOL*, void(*)(void*));
extern void    **ip4_fetch      (IP4_POOL*, const struct in_addr*);
extern void    **ip4_lookup     (IP4_POOL*, const struct in_addr*);
extern void     *ip4_delete     (IP4_POOL*, const struct in_addr*);

// extern U16_POOL *u16_pool_init  (void);
// extern unsigned  u16_pool_empty (U16_POOL*);
// extern U16_POOL *u16_pool_merge (U16_POOL*, U16_POOL*);
// extern void      u16_pool_close (U16_POOL*, void(*)(void*));
// extern void      u16_pool_flush (U16_POOL*, void(*)(void*));
// extern void    **u16_fetch      (U16_POOL*, uint16_t);
// extern void    **u16_lookup     (U16_POOL*, uint16_t);
// extern void     *u16_delete     (U16_POOL*, uint16_t);

// extern MAC_POOL *mac_pool_init  (void);
// extern void      mac_pool_close (MAC_POOL*, void(*)(void*));
// extern void     *mac_delete     (MAC_POOL*, const struct ether_addr*);
// extern void    **mac_fetch      (MAC_POOL*, const struct ether_addr*);
// extern void    **mac_lookup     (MAC_POOL*, const struct ether_addr*);
//
// extern MAC_WALK *mac_walk_init  (MAC_POOL*, MAC_WALK*);
// extern void      mac_walk_close            (MAC_WALK*);
// extern void     **mac_this                 (MAC_WALK*);
// extern void     **mac_next                 (MAC_WALK*);

extern IP4_WALK *ip4_walk_init (IP4_POOL*, IP4_WALK*);
extern void      ip4_walk_close           (IP4_WALK*);
extern ip4addr_t ip4_inx2ip4addr          (IP4_WALK*);
extern void    **ip4_this                 (IP4_WALK*);
extern void    **ip4_next                 (IP4_WALK*);

// extern U16_WALK *u16_walk_init (U16_POOL*, U16_WALK*);
// extern void      u16_walk_close           (U16_WALK*);
// extern uint16_t  u16_inx2uint16           (U16_WALK*);
// extern void    **u16_this                 (U16_WALK*);
// extern void    **u16_next                 (U16_WALK*);

#endif /* __SHAT_POOL_H */
