/*
 * Copyright (C) 2003-2005 Maxina GmbH - Jordan Hrycaj
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 * Author: Jordan Hrycaj <jordan@mjh.teddy.net.com>
 *
 * $Id: rfc3489.h 714 2005-12-08 18:20:29Z henry $
 */

#ifndef __STUN_RFC3489_H
#define __STUN_RFC3489_H

#include "ctrl.h"
#include "do.h"
#include "pool.h"
#include "stun.h"

/* names from RFC 3489 - 10.1/Figure 2: state diagram */
#define STATE_SYMBOLS(x)   \
    x (Open_Internet)      \
    x (UDP_Blocked)        \
    x (Sym_UDP_Firewall)   \
    x (Full_Cone)          \
    x (Symmetric_NAT)      \
    x (Restricted)         \
    x (Port_Restricted)

#ifdef __RFC3489_PRIVATE

#include "hlst.h"

#define TEST_CHAIN_RFC3489   0x1000 /* test chain identifier */
#define TEST_STATE_A         0x0100
#define TEST_STATE_B         0x0200
#define TEST_STATE_C         0x0300
#define TEST_STATE_D         0x0400
#define TEST_STATE_E         0x0500

typedef enum {
    no_NATSTATE = 0
#   define         __symbols(s) , s
    STATE_SYMBOLS (__symbols)
#   undef          __symbols
} NATSTATE;

typedef enum _action_constants {
    stun_fields_delim =  ' ',
    escape_character  = '\\',
} ACTION_CONSTANTS ;

typedef struct _stun_server {
    uint16_t stun_port ; /* target port for stun testing */

    ADDR         local ;
    
    struct {
        NATSTATE state ;
        ADDR       src ; /* stun source address (auto detected) */
    } nat ;

    const char  *label ; /* stun reference used in the NVRAM */ 
    int        reset:1 ;
} STUN_SERVER ;

#endif /* __RFC3489_PRIVATE */

/* communication through the control socket */
typedef struct _r3489 {
#   ifdef __RFC3489_PRIVATE
    unsigned  timeout ;
    unsigned    recur ;
    time_t start_test ;
    ip4addr_t localip ;
    SREQ          *sp ;
    NVCL          *nv ;

    struct {
        const char *list  ;
        struct {
            const char *s ;
            size_t len    ;
        } pfx, query, type, sipip ;
        IP4_POOL     *lst ;
    } stun ;
    
#   else
    char          any ;
#   endif
} R3489 ;

extern R3489   *rfc3489_open   (DO*,ip4addr_t,const char*,const char*,
                                const char*,const char*,const char*);
extern void     rfc3489_tmo    (R3489*,unsigned,unsigned);
extern void     rfc3489_run    (R3489*,void(*)(void*,const char*,...),void*);
extern void     rfc3489_show   (R3489*,void(*)(void*,const char*,...),void*);
extern void     rfc3489_auto   (R3489*) ;
extern unsigned rfc3489_update (R3489*) ;

#endif /* __STUN_RFC3489_H */
