/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef STDC_HEADERS
# include <stdlib.h>
# include <stddef.h>
#else
# ifdef HAVE_STDLIB_H
#  include <stdlib.h>
# endif
#endif

#include "encodec.h"

/* ----------------------------------------------------------------------- *
 * Private variables and settings
 * ----------------------------------------------------------------------- */

static int wspace_map_ok = 0 ;

/* ----------------------------------------------------------------------- *
 * Private: ascii to alpha encoding
 * ----------------------------------------------------------------------- */

/* starts at 0x20 */
static long ascii2alpha_table [] = {

  142725, /* 0x20 SPACE */
  142726, /* 0x21   !   */
  142727, /* 0x22   "   */
  142728, /* 0x23   #   */
  142729, /* 0x24   $   */
  142730, /* 0x25   %   */
  142731, /* 0x26   &   */
  142732, /* 0x27   '   */
  142733, /* 0x28   (   */
  142734, /* 0x29   )   */
  142742, /* 0x2a   *   */
  142743, /* 0x2b   +   */
  142744, /* 0x2c   ,   */
  142745, /* 0x2d   -   */
  142746, /* 0x2e   .   */
  142747, /* 0x2f   /   */

  25,     /* 0x30   0   */
  26,     /* 0x31   1   */
  27,     /* 0x32   2   */
  28,     /* 0x33   3   */
  29,     /* 0x34   4   */
  30,     /* 0x35   5   */
  31,     /* 0x36   6   */
  32,     /* 0x37   7   */
  33,     /* 0x38   8   */
  34,     /* 0x39   9   */
  142842, /* 0x3a   :   */
  142843, /* 0x3b   ;   */
  142844, /* 0x3c   <   */
  142845, /* 0x3d   =   */
  142846, /* 0x3e   >   */
  142847, /* 0x3f   ?   */

  142925, /* 0x40   @   */
  42,     /* 0x41   A   */
  43,     /* 0x42   B   */
  44,     /* 0x43   C   */
  45,     /* 0x44   D   */
  46,     /* 0x45   E   */
  47,     /* 0x46   F   */
  48,     /* 0x47   G   */
  49,     /* 0x48   H   */
  50,     /* 0x49   I   */
  51,     /* 0x4a   J   */
  52,     /* 0x4b   K   */
  53,     /* 0x4c   L   */
  54,     /* 0x4d   M   */
  55,     /* 0x4e   N   */
  56,     /* 0x4f   O   */

  57,     /* 0x50   P   */
  58,     /* 0x51   Q   */
  59,     /* 0x52   R   */
  60,     /* 0x53   S   */
  61,     /* 0x54   T   */
  62,     /* 0x55   U   */
  63,     /* 0x56   V   */
  64,     /* 0x57   W   */
  65,     /* 0x58   X   */
  66,     /* 0x59   Y   */
  67,     /* 0x5a   Z   */
  143043, /* 0x5b   [   */
  143044, /* 0x5c   \   */
  143045, /* 0x5d   ]   */
  143046, /* 0x5e   ^   */
  143047, /* 0x5f   _   */

  143125, /* 0x60   `   */
  74,     /* 0x61   a   */
  75,     /* 0x62   b   */
  76,     /* 0x63   c   */
  77,     /* 0x64   d   */
  78,     /* 0x65   e   */
  79,     /* 0x66   f   */
  80,     /* 0x67   g   */
  81,     /* 0x68   h   */
  82,     /* 0x69   i   */
  83,     /* 0x6a   j   */
  84,     /* 0x6b   k   */
  85,     /* 0x6c   l   */
  86,     /* 0x6d   m   */
  87,     /* 0x6e   n   */
  88,     /* 0x6f   o   */

  89,     /* 0x70   p   */
  90,     /* 0x71   q   */
  91,     /* 0x72   r   */
  92,     /* 0x73   s   */
  93,     /* 0x74   t   */
  94,     /* 0x75   u   */
  95,     /* 0x76   v   */
  96,     /* 0x77   w   */
  97,     /* 0x78   x   */
  98,     /* 0x79   y   */
  99,     /* 0x7a   z   */

  143243, /* 0x7b   {   */
  143244, /* 0x7c   |   */
  143245, /* 0x7d   }   */
  0,      /* 0x7e   ~   */
};


static
void ascii2alpha (IOBUF *b, int c) {
    data_printf (b, "%u", ascii2alpha_table [c - 0x20]);
}

/* ----------------------------------------------------------------------- *
 * Private: identity encoding
 * ----------------------------------------------------------------------- */

static
void ascii2ascii (IOBUF *b, int c) {
    char_append (b, c);
}

/* ----------------------------------------------------------------------- *
 * Private: string mapper
 * ----------------------------------------------------------------------- */

static
const char *string_encode (IOBUF                *b,
                           const char           *s,
                           int             xmap_ok,
                           void (*a) (IOBUF*, int)) {
    int c, pos = 0 ;

    for (;;) {
        switch (c = s [pos ++]) {
        case '\0':
            return b->data ;
        case  ' ':
        case '\t':
            if (xmap_ok == 0)
                c = '_' ;
            break ;
        case 0x7f:
            c = '?' ;
            break ;
        case '\r':
            switch (c = s [pos ++]) {
            case '\0':
                return b->data;
            case '\n':
                if ((c = s [pos ++]) == '\0')
                    return b->data;
            default:
                (*a) (b, c = '?');
            }
            break ;
        case '\n':
            switch (c = s [pos ++]) {
            case '\0':
                return b->data;
            default:
                /*  c = '?' ; */
                ;
            }
        default:
            if (c < ' ' || (c & ~0x7f))
                c = '?' ;
        }

        (*a) (b, c);
    }
}

/* ----------------------------------------------------------------------- *
 * Public: string map functions
 * ----------------------------------------------------------------------- */

const char *a2nencode (IOBUF *b, const char *s) {
    return string_encode (b, s, wspace_map_ok, ascii2alpha);
}

const char *normalise (IOBUF *b, const char *s) {
    return string_encode (b, s, wspace_map_ok, ascii2ascii);
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
