/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_NETDB_H
# include <netdb.h>
#endif

#ifdef HAVE_ARPA_INET_H
# include <arpa/inet.h>
#endif

#ifdef HAVE_ERRNO_H
# include <errno.h>
#endif

#ifdef HAVE_CTYPE_H
#include <ctype.h>
#endif

#include "helpers.h"

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

ip4addr_t hostip (const char *s) {
    
    extern int h_errno;
    int       e ;
    ip4addr_t ip ;

    struct hostent* h = gethostbyname (s) ;

    if (h != 0) {
        struct in_addr sin_addr;
        sin_addr.s_addr = *(in_addr_t*)h->h_addr;
        return ntohl (sin_addr.s_addr);
    }

    e = h_errno ;
    if ((ip = ntohl (inet_addr (s))) != 0 && ip != -1)
        return ip;
    errno = e ;
    return 0;
}

const char *nvalue (NVCL *nvram, const char *var) {
    const char *val ;

    nvcl_putcmd (nvram, "set");
    nvcl_putavpair (nvram, var, 0, 0);

    if (nvcl_rply (nvram, 5) > 0 &&
        nvcl_getavpair (nvram, &var, &val, 0, 0) != 0)
        return val ;

    return 0;
}

const char *false_is0 (IOBUF *b, const char *s) {
    /* 0 or f or false or disa.* => 0, else 1 */
    switch (tolower (s [0])) {
    case 'f':
        return s [1] == '\0' || strncasecmp (s, "false", 5) == 0
            ? "0" /* off */
            : "1" /* on */
            ;
    case 'd':
        return strncasecmp (s, "disa", 4) == 0
            ? "0" /* off */
            : "1" /* on */
            ;
    case '0':
        return "0";
    default:
        return "1" ;
    }
}

const char *ena_is1 (IOBUF *b, const char *s) {
    /* [1-9].* or t or true or ena.* => 1, else 0 */
    switch (tolower (s [0])) {
    case 't':
        return  s [1] == '\0' || strncasecmp (s, "true", 4) == 0
            ? "1" /* on */
            : "0" /* off */
            ;
    case 'e':
        return strncasecmp (s, "ena", 3) == 0
            ? "1" /* on/enabled */
            : "0" /* off/disabled */
            ;
    default:
        return s [0] != '0' && isdigit (s [0]) ? "1" : "0" ;
    }
}

const char *any_is0      (IOBUF *b, const char *s) {  return "0"; }
const char *any_is1      (IOBUF *b, const char *s) {  return "1"; }
const char *any_is_empty (IOBUF *b, const char *s) {  return "";  }

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
