/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifndef __WPOST_HTTP_H
#define __WPOST_HTTP_H

#include "nvram-lib.h"
#include "buffer.h"

typedef
struct _ht_dial {
#   ifdef __WPOST_HTTP_PRIVATE
    ip4addr_t     ip ;
    uint16_t    port ;
    int      sock_fd ; /* current communication socket */
    int econnreset:1 ; /* peer has dropped the line */
    struct {
        struct {
            unsigned   max_zero ;
            unsigned wait_first ; /* ms */
            unsigned  wait_next ;
        } send ;
        unsigned headers ;
        struct _header {
            const char *var ;
            const char *val ;
        } *header ;
        IOBUF post ;
        IOBUF resp ;
        struct {
            unsigned code ;
            unsigned text ; /* offset into IOBUF resp */
            unsigned data ; /* offset into IOBUF resp */
        } reply ;
    } data ;
    NVCL *nvram ;
#   else
    char any;
#   endif
} HTDIAL ;


extern HTDIAL     *http_open       (NVCL*);
extern void        http_close      (HTDIAL*);
extern void        http_target     (HTDIAL*, ip4addr_t, uint16_t);
extern unsigned    http_connect    (HTDIAL*);
extern void        http_disconnect (HTDIAL*);
extern unsigned    http_send       (HTDIAL*,const char*);
extern unsigned    http_recv       (HTDIAL*);

extern const char *http_val        (HTDIAL*,const char*);
extern const char *http_basic_auth (HTDIAL*,const char*,const char*);
extern void        http_hdr_edit   (HTDIAL*,const char*,const char*,int);

extern void        http_var_flush  (HTDIAL*);
extern void        http_resp_flush (HTDIAL*);
extern void        http_avp_append (HTDIAL*,const char*,const char*);
extern unsigned    http_var_append (HTDIAL*,const char*,const char*,
                                    const char*(*)(IOBUF*,const char*));

#define http_hdr_default(h,v,s)   http_hdr_edit (h, v, s, 0)
#define http_hdr_replace(h,v,s)   http_hdr_edit (h, v, s, 1)
#define http_hdr_append( h,v,s)   http_hdr_edit (h, v, s, 2)

extern unsigned    http_rply_code (HTDIAL*) ;
extern const char *http_rply_text (HTDIAL*) ;
extern const char *http_rply_data (HTDIAL*) ;

#endif /* __WPOST_HTTP_H */
