/*
 * Copyright (C) 2006 Maxina GmbH - Jordan Hrycaj
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 * Author: Jordan Hrycaj <jordan@mjh.teddy.net.com>
 *
 * $Id: do.h 3496 2006-10-31 19:08:13Z jordan $
 */

#ifndef __SHAT_DO_H
#define __SHAT_DO_H

#define DO_LOG_ACTION_REGISTER  0x010 /* log selector flag */
#define DO_LOG_ACTION_RELEASE   0x020
#define DO_LOG_ACTION_EXPIRE    0x040
#define DO_LOG_ACTION_NEXTIDLE  0x080

#ifdef __DO_PRIVATE
#include <sys/select.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>

typedef struct _do_set {
    int              rfd; /* read file handle to listen on */
    void   (*cb) (void*); /* action to take on incoming data */
    void        *cb_data; /* to be passed to cb on activation */
    unsigned     timeout; /* timeout => time to tie */
    time_t        expiry; /* time to tie */
    void   (*xh) (void*); /* expiry handler  */
    void        *xh_data; /* to be passed to xh */
} DO_SET ;
#endif /* __DO_PRIVATE */


typedef struct _do
#ifdef __DO_PRIVATE
{
    void (*idle) (void*); /* action to be taken on idle looping */
    void           *data; /* to be passed to the idle fn */
    int  default_timeout; /* default timeout (used by the rescheduler) */
    int          timeout; /* currently active timeout */
    time_t     time_leap;  /* flag a time change */
    time_t           now; /* event loop system time */
    struct timeval   due; /* idle timer scheduler, next run */
    struct timeval    tv; /* idle time scheduler timeout */
    struct timeval  *tvp; /* ... to be used in the select loop */
    int           max_fd; /* max bit id in fds */
    fd_set           fds; /* bit vector set for active connections */
    int           length; /* total number of set entries */
    int         with_ttd; /* entries in the list with expiry time set */
    DO_SET          *set; /* action handler list */
}
#endif /* __DO_PRIVATE */
DO ;


extern DO    *do_init      (void(*)(void*),void*);
extern time_t do_time_cont (DO*,time_t);
extern time_t do_time      (DO*);
extern void   do_action    (DO*,int,         void(*)(void*),void*);
extern int    do_expire    (DO*,int,unsigned,void(*)(void*),void*);
extern int  undo_action    (DO*,int);
extern int    do_loop      (DO*,int);
extern void   do_resched   (DO*,int);
extern void   do_destroy   (DO*);

#endif /* __SHAT_DO_H */
