/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifndef __WPOST_MAP_WLAN_H
#define __WPOST_MAP_WLAN_H

/* ----------------------------------------------------------------------- *
 * Helper functions
 * ----------------------------------------------------------------------- */

#ifdef __WPOST_PARAMETERS_PRIVATE

#include "encodec.h"
#include "helpers.h"

static /* auth_type */
const char *wlan_nvatpe (IOBUF *b, const char *s) {
    if (strcasecmp (s, "open") == 0)
        return "0";
    if (strcasecmp (s, "shared") == 0)
        return "1";
    return "2"; /* both */
}


static /* privacy type */
const char *wlan_nvptpe (IOBUF *b, const char *s) {
    if (strcasecmp (s, "none") == 0)
        return "0";
    if (strcasecmp (s, "wep") == 0)
        return "1";
    return "2"; /* wpa */
}


static
const char *wlan_nvkid (IOBUF *b, const char *s) {
    switch (tolower (s [0])) {
    case 'b': return "1" ;
    case 'c': return "2" ;
    case 'd': return "3" ;
    }
    return "0";
}

static
const char *wlan_nvklen (IOBUF *b, const char *s) {
    if (strcmp (s, "256") == 0)
        return "29";
    if (strcmp (s, "128") == 0)
        return "13";
    return "5";
}
#endif /* __WPOST_PARAMETERS_PRIVATE */

/* ----------------------------------------------------------------------- *
 * preamble
 * ----------------------------------------------------------------------- */

#define _X0(alpha,inst)            _ ## alpha ## inst
#define _X1(alpha,inst)         var_ ## alpha ## inst
#define _X2(alpha,inst) ap_settings_ ## alpha ## inst

#define _A0(alpha)                    # alpha
#define _A1(alpha)             "var:" # alpha
#define _A2(alpha)     "ap:settings/" # alpha

#define _N0(nvram)                      0
#define _N1(nvram)        "lan_wlan." # nvram           

/* ----------------------------------------------------------------------- *
 * wlan - common
 * ----------------------------------------------------------------------- */

#define _X _X0
#define _A _A0
#define _N _N0

_D (APSecurity, 0, ?)
_D (APEnable,   0, ?)
_D (SSIDName,   0, ?)
_D (uiSecurity, 0, ?)
_D (Channel,    0, ?)

#undef  _X
#define _X _X1 /* var_ */
#undef  _A
#define _A _A1 /* var: */
#undef  _N
#define _N _N1

_D (ssid,      0, ssid)
_D (channel,   0, channel)

#undef  _N
#define _N _N0 /* null */

_D (apenabled, 0, ?)

#undef  _X
#define _X _X2 /* ap_settings_ */
#undef  _A
#define _A _A2 /* ap:settings/ */
#undef  _N
#define _N _N1

_D (ssid,           a2nencode, ssid)
_D (channel_g,              0, channel)
_D (ap_enabled, ena_is1, accesspointEnabled)
_D (privacy_type, wlan_nvptpe, security)

#undef  _N
#define _N _N0 /* null */

_D (user_isolation,   any_is0, ?)

/* ----------------------------------------------------------------------- *
 * wlan - WEP
 * ----------------------------------------------------------------------- */

#undef  _X
#define _X _X2 /* ap_settings_ */
#undef  _A
#define _A _A2 /* ap:settings/ */
#undef  _N
#define _N _N1

_D (wep_auth_type, wlan_nvatpe, authtype)
_D (key_id,         wlan_nvkid, encryptionselection)
_D (key_value1,              0, encryption_A)
_D (key_len1,      wlan_nvklen, cipher_A)
_D (key_value2,              0, encryption_B)
_D (key_len2,      wlan_nvklen, cipher_B)
_D (key_value3,              0, encryption_C)
_D (key_len3,      wlan_nvklen, cipher_C)
_D (key_value4,              0, encryption_D)
_D (key_len4,      wlan_nvklen, cipher_D)

#undef  _N
#define _N _N0 /* null */

_D (wep_privacy,       any_is1, ?)

/* ----------------------------------------------------------------------- *
 * wlan - WPA
 * ----------------------------------------------------------------------- */

#undef  _X
#define _X _X2 /* ap_settings_ */
#undef  _A
#define _A _A2 /* ap:settings/ */
#undef  _N
#define _N _N1

_D (radius_ip,          0, serverip)
_D (radius_port,        0, port)
_D (radius_secret,      0, secret)
_D (interval,           0, groupkeyinterval)
_D (psktype,            0, wpatype)
_D (pskvalue,           0, hex)    /* same alpha symbol */
_E (pskvalue,           0, string) /* same alpha symbol */

/* ----------------------------------------------------------------------- *
 * epilogue
 * ----------------------------------------------------------------------- */

#undef _X
#undef _X0
#undef _X1
#undef _X2

#undef _A
#undef _A0
#undef _A1
#undef _A2

#undef _N
#undef _N0
#undef _N1

#endif /* __WPOST_MAP_WLAN_H */
