/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>

#ifdef STDC_HEADERS
# include <stdlib.h>
# include <stddef.h>
#else
# ifdef HAVE_STDLIB_H
#  include <stdlib.h>
# endif
#endif

#ifdef HAVE_UNISTD_H
# include <unistd.h>
# include <sys/types.h>
# else
# ifdef HAVE_SYS_TYPES_H
#  include <sys/types.h>
# endif
#endif

#ifdef HAVE_STRING_H
# if !defined (STDC_HEADERS) && defined (HAVE_MEMORY_H)
#  include <memory.h>
# endif
# include <string.h>
#endif

#ifdef TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# ifdef HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#define __WPOST_PARSET_PRIVATE
#include "parsets.h"

#include "settings.h"
#include "nvalloc.h"
#include "logger.h"

#include "set-dsl.h"
#include "set-dslmodulation.h"
#include "set-reboot.h"
#include "set-ip.h"
#include "set-dhcp.h"
#include "set-wlan.h"
#include "set-dyndns.h"
#include "set-dmz.h"
#include "set-dns.h"
#include "set-fwoff.h"
#include "set-addssid.h"
#include "set-delssid.h"
#include "set-snmp.h"
#include "set-xwlan.h"
#include "set-rip.h"
#include "set-aprestart.h"

#ifdef HTTP_GET_OK
#include "get-login.h"
#include "get-wlan.h"
#include "get-addssid.h"
#include "get-delssid.h"
#endif

/* ----------------------------------------------------------------------- *
 * Private variables and settings
 * ----------------------------------------------------------------------- */

#define __str(x)   # x
#define to_str(x)  __str (x)

#define DIM(x)     (sizeof (x) / sizeof (x [0]))

static const char default_uri [] = "/cgi-bin/webcm" ;

/* ----------------------------------------------------------------------- *
 * Private list of parameter sets
 * ----------------------------------------------------------------------- */

static
const PARDATA data [] = {

#   ifdef HTTP_GET_OK
#   define _double_entry(sym,alias) {                             \
        {alias, sym ## _get_uri},                                 \
        {sym ## _extra, sym ## _vars, sym ## _help, sym ## _getp} }
#   else
#   define _double_entry _double2_entry
#   endif

#   define _double2_entry(sym,alias) {                            \
        {alias}, {sym ## _extra, sym ## _vars, sym ## _help}      }

#   define _simple_entry(sym)  _double_entry (sym, to_str (sym))

#   define _old_double_entry _double2_entry

    _old_double_entry  (aprestart, "aprestart"),
    _old_double_entry  (dhcp, "dhcp"),
    _old_double_entry  (dmz, "dmz"),
    _old_double_entry  (dns, "dns"),
    _old_double_entry  (dsl, "dsl"),
    _old_double_entry  (dslmodulation, "dslmodulation"),
    _old_double_entry  (dyndns, "dyndns"),
    _old_double_entry  (fwoff, "fwoff"),
    _old_double_entry  (ip, "ip"),
#   ifdef HTTP_GET_OK
    _simple_entry    (login),
#   endif
    _old_double_entry  (snmp, "snmp"),
    _double_entry    (addssid, "+ssid"),
    _double_entry    (delssid, "-ssid"),
    _old_double_entry  (reboot, "reboot"),
    _old_double_entry  (rip, "rip"),
    _simple_entry    (wlan),
    _old_double_entry  (xwlan, "xwlan"),

#   undef _simple_entry
#   undef _double_entry
#   undef _double2_entry
#   undef _old_double_entry
} ;

/* ----------------------------------------------------------------------- *
 * Private parameter and assignment lists
 * ----------------------------------------------------------------------- */

static
struct {
    const char *var ;
    const char *val ;
} headers [] = {
    {"Accept", (""
                // "image/gif,"
                // "image/x-xbitmap,"
                // "image/jpeg,"
                // "image/pjpeg,"
                // "application/msword,"
                // "application/x-shockwave-flash,"
                // "application/vnd.ms-excel,"
                // "application/vnd.ms-powerpoint,"
                "text/html,"
                "text/plain"
                // "*/*"
                "")},
    {"Accept-Language", "en"},
    {"Content-Type",    "application/x-www-form-urlencoded"},
    // {"Accept-Encoding", "gzip, deflate"},
    {"User-Agent",     ("Mozilla/4.0 "
                        "(compatible; MSIE 6.0; Windows NT 5.0)")},
    {"Connection",      "Close" /* "Keep-Alive" */},
    {"Pragma",          "no-cache"},
    {"Cache-Control",   "no-cache"}
} ;


/* ----------------------------------------------------------------------- *
 * Helpers
 * ----------------------------------------------------------------------- */

static
unsigned parset_find (const char *name) {
    int            n =    0 ;
    const PARDATA *p = data ;

    do
        if (strcmp  (name, p->lookup.module) == 0)
            return n + 1;
    while (++ p, ++ n < DIM (data)) ;

    return 0;
}

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

void parset_close (PARSET *ps) {

#   ifdef HTTP_GET_OK
    if (ps->get_uri)
        free ((char*)ps->get_uri);
#   endif

    free (ps);
}


PARSET *parset_open (const char *name, HTDIAL *http) {
    PARSET p ;
    unsigned inx ;

    if ((inx = parset_find (name)) == 0) {
        logger
            (LOG_ERR, "parset: No parameter set available for %s", name);
        return 0 ;
    }

    XZERO (p) ;

    if ((p.http = http) == 0) {
        logger (LOG_ERR, "parset: HTTP subsystem not initialised!");
        return 0 ;
    }
    
    p.param = data + inx - 1 ;
    return zmemdup (&p) ;
}


const char *parset_posturi (PARSET *ps) {

    return ps->param->lookup.post_uri
        ? ps->param->lookup.post_uri
        : default_uri
        ;
}


void parset_list (void(*pr)(const char*,unsigned)) {

    int            n =    0 ;
    const PARDATA *p = data ;

    do
        (*pr) (p->lookup.module, n + 1) ;

    while (++ p, ++ n < DIM (data)) ;
}




const char *parset_help (PARSET *ps, unsigned indent) {
    char b [20] ;
    IOBUF i ;

    XZERO (i) ;
    sprintf (b, "%%-%ds%%-" to_str (PARSET_HELP_FF) "s%%s\n", indent);

    data_printf (&i, b, "", "name:", ps->param->lookup.module);

#   ifdef HTTP_GET_OK
    if (ps->param->post.send)
#   endif
        data_printf (&i, b, "", "post uri:", ps->param->lookup.post_uri
                     ? ps->param->lookup.post_uri : default_uri);

#   ifdef HTTP_GET_OK
    if (ps->param->lookup.get_uri)
        data_printf (&i, b, "", "get uri:", ps->param->lookup.get_uri);
#   endif

    data_append (&i, (*ps->param->post.help) (indent, PARSET_HELP_FF), 0) ;

    unlink_space (&i);
    return i.data ;
}


unsigned parset_header (PARSET *ps) {
    int n;

    for (n = 0; n < DIM (headers) ; n++ )
        http_hdr_append (ps->http, headers [n].var, headers [n].val);

    return 1;
}


unsigned parset_post_xtra (PARSET *ps, const char *inst) {
    if (ps->param->post.xtra == 0)
        return 1;

    http_var_flush (ps->http);
    return (*ps->param->post.xtra) (ps->http, inst);
}


unsigned parset_post_body (PARSET *ps, const char *inst) {
#   ifdef HTTP_GET_OK
    if (ps->param->post.send == 0)
        return 0 ;
#   endif

    http_var_flush (ps->http);
    return (*ps->param->post.send) (ps->http, inst);
}


#ifdef HTTP_GET_OK /* experimental */
const char *parset_geturi (PARSET *ps) {

    return ps->get_uri
        ? ps->get_uri
        : ps->param->lookup.get_uri
        ;
}


const char *parset_filter (PARSET *ps, const char *inst) {
    const char *s ;

    if (ps->param->post.getp == 0)
        return 0;
    
    s = (*ps->param->post.getp) (ps->http, inst) ;

    if (s == 0 || s [0] == '\0')
        return s ;

    /* s might be inst, so dup it now to prevent from vanishing */
    s = zstrdup (s);

    if (ps->get_uri)
        free ((char*)ps->get_uri);
    
    return ps->get_uri = s;
}
#endif

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
