/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>

#ifdef STDC_HEADERS
# include <stdlib.h>
# include <stddef.h>
#else
# ifdef HAVE_STDLIB_H
#  include <stdlib.h>
# endif
#endif

#ifdef HAVE_UNISTD_H
# include <unistd.h>
# include <sys/types.h>
# else
# ifdef HAVE_SYS_TYPES_H
#  include <sys/types.h>
# endif
#endif

#ifdef HAVE_STRING_H
# if !defined (STDC_HEADERS) && defined (HAVE_MEMORY_H)
#  include <memory.h>
# endif
# include <string.h>
#endif

#ifdef TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# ifdef HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#include "buffer.h"
#include "varstr.h"
#include "logger.h"
#include "nvalloc.h"
#include "http.h"
#include "helpers.h"
#include "encodec.h"

#include "set-addssid.h"

/* ----------------------------------------------------------------------- *
 * Private variables and settings
 * ----------------------------------------------------------------------- */

#define NVRAM_PREFIX "lan_mgmt_multiple."

#define DIM(x)  (sizeof (x) / sizeof (x [0]))

/* ----------------------------------------------------------------------- *
 * Private: parameter and assignment lists
 * ----------------------------------------------------------------------- */

struct _intro_avp {
    const char *var ;
    const char *val ;
};

struct _append_vars {
    const char *var ;
    const char *val ;
    const char *(*map)(IOBUF*, const char*);
};

static
const struct _intro_avp intro_avp [] = {
    {"getpage",                   "../html/advanced/wireless_ssid.htm"},
};

static
const struct _intro_avp intro_avp_extra [] = {
    {"Wireless_Management",       "on"},
};

static
const struct _append_vars append_onoff [] = {
    {"ap:settings/multiple_ssid", "hasMultiSsid", ena_is1},
};

static
const struct _append_vars append_vars [] = {
    {"ap:settings/mssid_add",     "ssid.%d",      normalise},
};



/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */

static
unsigned post_vars (HTDIAL *http, const char *instance, int extra) {
    int n, k ;

    int               inx, ip_dim [2] ;
    const struct _intro_avp   *ip [2] ;

    int               iny, ap_dim [2] ;
    const struct _append_vars *ap [2] ;

    unsigned id ;
    IOBUF  i, j ;
   
    if (instance == 0)
        id = 0 ;
    else if (parse_unsigned (&id, instance, 0) == 0) {
        if (extra == 0)
            logger (LOG_ERR,
                    "addssid_vars: illegal instance variable \"%s\"",
                    instance);
        return 0 ;
    }

    inx = iny = 0 ;

    ip_dim [inx] = DIM (intro_avp);
    ip [inx ++]  =      intro_avp ;

    ap_dim [iny] = DIM (append_onoff);
    ap [iny ++]  =      append_onoff ;

    if (extra) {
        ip_dim [inx] = DIM (intro_avp_extra);
        ip [inx ++]  =      intro_avp_extra ;
    }
    else if (id != 0) {
        ap_dim [iny] = DIM (append_vars);
        ap [iny ++]  =      append_vars ;
    }

    for (k = 0; k < inx; k ++)
        for (n = 0; n < ip_dim [k]; n ++)
            http_avp_append (http, ip [k] [n].var, ip [k] [n].val);

    XZERO (i);
    XZERO (j);
    data_append (&j, NVRAM_PREFIX, 0);
    for (k = 0; k < iny; k ++)
        for (n = 0; n < ap_dim [k]; n ++) {
            i.top = 0 ;
            j.top = sizeof (NVRAM_PREFIX) - 1;
            data_append (&j, ap [k] [n].val, 0);
            data_printf (&i, j.data, id);
            http_var_append
                (http, ap [k] [n].var, i.data, ap [k] [n].map);
        }

    kill_space   (&i);
    kill_space   (&j);

    return 1;
}

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

const char *addssid_help (unsigned indent, unsigned ff) {
    char b [20] ;
    IOBUF i ;
    XZERO (i) ;

    sprintf (b, "%%-%ds%%-%ds%%s\n", indent, ff);
        
    data_printf (&i, b, "", "instance:", "[ssid-id]");
    data_printf (&i, b, "", "with:",     "ssid-id as found in the nvram");
    data_printf (&i, b, "", "default:",  "none - enable/disable only");

    unlink_space (&i);
    return i.data ;
}

unsigned addssid_extra (HTDIAL *http, const char *instance) {
    return post_vars (http, instance, 1);
}


unsigned addssid_vars (HTDIAL *http, const char *instance) {
    return post_vars (http, instance, 0);
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
