/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>

#ifdef STDC_HEADERS
# include <stdlib.h>
# include <stddef.h>
#else
# ifdef HAVE_STDLIB_H
#  include <stdlib.h>
# endif
#endif

#ifdef HAVE_UNISTD_H
# include <unistd.h>
# include <sys/types.h>
# else
# ifdef HAVE_SYS_TYPES_H
#  include <sys/types.h>
# endif
#endif

#ifdef HAVE_STRING_H
# if !defined (STDC_HEADERS) && defined (HAVE_MEMORY_H)
#  include <memory.h>
# endif
# include <string.h>
#endif

#ifdef TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# ifdef HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#include "buffer.h"
#include "logger.h"
#include "nvalloc.h"
#include "http.h"
#include "helpers.h"

#include "set-dmz.h"

/* ----------------------------------------------------------------------- *
 * Private variables and settings
 * ----------------------------------------------------------------------- */

#define NVRAM_PREFIX "wan_dmz."

#define DIM(x)  (sizeof (x) / sizeof (x [0]))

/* ----------------------------------------------------------------------- *
 * Private: parameter and assignment lists
 * ----------------------------------------------------------------------- */

struct _intro_avp {
    const char *var ;
    const char *val ;
} ;


static
const struct _intro_avp intro_avp [] = {
    {"getpage", "../html/advanced/adv_dmz.htm"},
};


static
const struct _intro_avp intro_off [] = {
};


static
const struct {
    const char *var ;
    const char *val ;
    const char *(*map)(IOBUF*, const char*);
} append_vars [] = {

    {"fdb:settings/service/dmz/state",	"has_demilitarizedZone",  ena_is1},
    {"fdb:settings/service/dmz/InternalClient",	"ipAddress",  		0},
};

/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */

static
unsigned post_vars (HTDIAL *http, const char *instance, int extra) {
    int n ;
    IOBUF i, j ;
    
    XZERO (i) ;
    XZERO (j) ;

    if (instance != 0) {
            logger (LOG_ERR, "dmz_vars: no instance variable needed");
            return 0 ;
    }

    for (n = 0; n < DIM (intro_avp) ; n++ )
        http_avp_append (http, intro_avp [n].var, intro_avp [n].val);

    if (extra) {
        for (n = 0; n < DIM (intro_off) ; n++ ) {
            i.top = 0 ;
            data_printf (&i, intro_off [n].var, 0);
            http_avp_append (http, i.data, intro_off [n].val);
        }
    }
    else {
        
        j.top = 0 ;
        data_append (&j, NVRAM_PREFIX, 0);
        for (n = 0; n < DIM (append_vars) ; n++ ) {
            i.top = 0 ;
            j.top = sizeof (NVRAM_PREFIX) - 1;
            data_printf (&i, append_vars [n].var, 0);
            data_printf (&j, append_vars [n].val, 0);
            http_var_append (http, i.data, j.data, append_vars [n].map);
        }
    }

    kill_space   (&i);
    kill_space   (&j);

    return 1;
}

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

const char *dmz_help (unsigned indent, unsigned ff) {
    char b [20] ;
    IOBUF i ;
    XZERO (i) ;

    sprintf (b, "%%-%ds%%-%ds%%s\n", indent, ff);

    data_printf (&i, b, "", "instance:", "not needed");

    unlink_space (&i);
    return i.data ;
}


const char *dmz_getp (HTDIAL *http, const char *instance) {
    return 0;
}

unsigned dmz_extra (HTDIAL *http, const char *instance) {
    return post_vars (http, instance, 1);
}


unsigned dmz_vars (HTDIAL *http, const char *instance) {
    return post_vars (http, instance, 0);
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
