/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>

#ifdef STDC_HEADERS
# include <stdlib.h>
# include <stddef.h>
#else
# ifdef HAVE_STDLIB_H
#  include <stdlib.h>
# endif
#endif

#ifdef HAVE_UNISTD_H
# include <unistd.h>
# include <sys/types.h>
# else
# ifdef HAVE_SYS_TYPES_H
#  include <sys/types.h>
# endif
#endif

#ifdef HAVE_STRING_H
# if !defined (STDC_HEADERS) && defined (HAVE_MEMORY_H)
#  include <memory.h>
# endif
# include <string.h>
#endif

#ifdef TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# ifdef HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#include "buffer.h"
#include "logger.h"
#include "nvalloc.h"
#include "http.h"
#include "helpers.h"
#include "varstr.h"

#include "set-dsl.h"

/* ----------------------------------------------------------------------- *
 * Private variables and settings
 * ----------------------------------------------------------------------- */

#define NVRAM_PREFIX "wan_dsl."

#define DIM(x)  (sizeof (x) / sizeof (x [0]))

/* ----------------------------------------------------------------------- *
 * Private: map functions
 * ----------------------------------------------------------------------- */

static
const char *nv40off (IOBUF *b, const char *s) {
    int mtu = atoi (s) ;
    if (mtu <= 40)
        return "0" ;
    kill_space (b);
    data_printf (b, "%d", mtu - 40) ;
    return b->data ;
}

static
const char *min2sec (IOBUF *b, const char *s) {
    int t = atoi (s) ;
    kill_space (b);
    data_printf (b, "%d", t*60) ;
    return b->data ;
}

static
const char *mode2i (IOBUF *b, const char *s) {

    if (!strcmp(s, "disconnect"))
        return "1";
    return "0";
}

#if 0
static
const char *off_is2 (IOBUF *b, const char *s) {
    
    const char *p = ena_is1 (b, s);

    if (p [0] == '1')
        return p ;

    return "2";
}
#endif

/* ----------------------------------------------------------------------- *
 * Private: parameter and assignment lists
 * ----------------------------------------------------------------------- */

struct _intro_avp {
    const char *var ;
    const char *val ;
};


static
const struct _intro_avp intro_any [] = {
    {"getpage",     "../html/home/fpanelNAT.htm"},
    {"var:getpage", "../html/advanced/advPPP.htm"},
};


static
const struct _intro_avp intro_avp [] = {
 
    {"var:Encaps",                                   "pppoe"},
    {"var:change",                                   ""},
    {"var:contype",                                  "pppoe"},
    {"var:preForm",                                  "advanced"},
    {"var:NATstate",                                 "0"},
    {"var:Firewallstate",                            "1"},
    {"var:conid",                                    "encaps%d"},
    {"var:mycon",                                    "connection%d"},
    {"encaps%d:settings/type",                       "rfc2684"},
    {"encaps%d:pvc:settings/traffic_class",          "UBR"},
    {"encaps%d:pvc:settings/pcr",                    "0"},
    {"encaps%d:pvc:settings/scr",                    "0"},
    {"encaps%d:pvc:settings/cdvt",                   "0"},
    {"encaps%d:pvc:settings/mbs",                    "0"},
    {"encaps%d:pvc:settings/auto_vc",                "0"},
    {"encaps%d:rfc2684:settings/encaps",             "0"},
    {"connection%d:settings/type",                   "pppoe"},
    {"connection%d:pppoe:settings/auth_type",        "Auto"},
    {"connection%d:pppoe:settings/maxfail",          "10"},
    {"connection%d:pppoe:settings/debug",            "0"},
    {"connection%d:fwan:settings/private/spi/state", "1"},
    {"connection%d:pppoe:settings/ppp_unnum",        "0"},
    {"connection%d:pppoe:settings/assigned_ip",      "0.0.0.0"},

    /* will be set in append_vars [HM] */
//    {"connection%d:pppoe:settings/defaultroute",     "0"},
         
    // encaps0:pvc:settings/mac=00:e0:a0:a6:66:70
};


static
const struct _intro_avp intro_off [] = {
    {"encaps%d:pvc:settings/active",                 "2"}, /* off */
    {"connection%d:fwan:settings/private/spi/state", "0"},
    {"connection%d:fwan:settings/private/mtu/state", "0"},
};


static
const struct {
    const char *var ;
    const char *val ;
    const char *(*map)(IOBUF*, const char*);
} append_vars [] = {

    {"encaps%d:pvc:settings/vpi",                         "vpi.%d"},
    {"encaps%d:pvc:settings/vci",                         "vci.%d"},
    {"encaps%d:pvc:settings/active",           "virtualCircuit.%d", ena_is1},
    {"connection%d:pppoe:settings/mtu",                   "mtu.%d"},
    {"fdb:settings/service/mtu/mss_value",                "mtu.%d", nv40off},
    {"connection%d:pppoe:settings/mru",                   "mru.%d"},
    {"connection%d:pppoe:settings/username",         "username.%d"},
    {"connection%d:pppoe:settings/password",         "password.%d"},
    {"connection%d:pppoe:settings/defaultroute", "defaultRoute.%d", ena_is1},
    {"connection%d:pppoe:settings/idle",       "disconnectTime.%d", min2sec},
    {"connection%d:pppoe:settings/mode",           "connection.%d", mode2i},
};

/* ----------------------------------------------------------------------- *
 * Helpers
 * ----------------------------------------------------------------------- */

static
void dsl_intro (HTDIAL               *http,
                const struct _intro_avp *p,
                int                    dim,
                IOBUF                   *i,
                IOBUF                   *j,
                int                     id,
                int                    pvc) {
    int n ;

    for (n = 0; n < dim ; n++ ) {
        j->top = i->top = 0 ;
        data_printf (i, p [n].var, pvc);
        data_printf (j, p [n].val, pvc);
        http_avp_append (http, i->data, j->data);
    }
}


static
unsigned post_vars (HTDIAL *http, const char *instance, int extra) {
    int n, id, pvc ;
    unsigned u ;
    IOBUF i, j ;
    
    XZERO (i) ;
    XZERO (j) ;
    
    if (instance == 0)
        instance = "0x01" ;

    if (parse_unsigned (&u, instance, 0) == 0) {
        if (extra == 0)
            logger (LOG_ERR, "dsl_vars: illegal instance variable \"%s\"",
                    instance);
        return 0 ;
    }
    pvc = (u >> 4) & 0xf ;
    id  = (u)      & 0xf ;

    dsl_intro (http, intro_any, DIM (intro_any), &i, &j, id, pvc);

    if (extra) {

        dsl_intro (http, intro_off, DIM (intro_off), &i, &j, id, pvc);
    }
    else {
        
        dsl_intro (http, intro_avp, DIM (intro_avp), &i, &j, id, pvc);

        j.top = 0 ;
        data_append (&j, NVRAM_PREFIX, 0);
        for (n = 0; n < DIM (append_vars) ; n++ ) {
            i.top = 0 ;
            j.top = sizeof (NVRAM_PREFIX) - 1;
            data_printf (&i, append_vars [n].var, pvc);
            data_printf (&j, append_vars [n].val, id);
            http_var_append (http, i.data, j.data, append_vars [n].map);
        }
    }

    kill_space   (&i);
    kill_space   (&j);

    return 1;
}


/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

const char *dsl_help (unsigned indent, unsigned ff) {
    char b [20] ;
    IOBUF i ;
    XZERO (i) ;

    sprintf (b, "%%-%ds%%-%ds%%s\n", indent, ff);

    data_printf (&i, b, "", "instance:", "number");
    data_printf (&i, b, "", "with:",     "pvc id as   \"number / 16\"");
    data_printf (&i, b, "", "",          "nvram id as \"number % 16\"");
    data_printf (&i, b, "", "default:",  "0x01 - pvc 0 at nvram id 1");

    unlink_space (&i);
    return i.data ;
}


const char *dsl_getp (HTDIAL *http, const char *instance) {
    return 0;
}

unsigned dsl_extra (HTDIAL *http, const char *instance) {
    return post_vars (http, instance, 1);
}


unsigned dsl_vars (HTDIAL *http, const char *instance) {
    return post_vars (http, instance, 0);
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
