/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Henning Holtschneider <henning@loca.net>                      *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>

#ifdef STDC_HEADERS
# include <stdlib.h>
# include <stddef.h>
#else
# ifdef HAVE_STDLIB_H
#  include <stdlib.h>
# endif
#endif

#ifdef HAVE_UNISTD_H
# include <unistd.h>
# include <sys/types.h>
# else
# ifdef HAVE_SYS_TYPES_H
#  include <sys/types.h>
# endif
#endif

#ifdef HAVE_STRING_H
# if !defined (STDC_HEADERS) && defined (HAVE_MEMORY_H)
#  include <memory.h>
# endif
# include <string.h>
#endif

#ifdef TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# ifdef HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#include "buffer.h"
#include "varstr.h"
#include "logger.h"
#include "nvalloc.h"
#include "http.h"

#include "set-dslmodulation.h"

/* ----------------------------------------------------------------------- *
 * Private variables and settings
 * ----------------------------------------------------------------------- */

#define DIM(x)  (sizeof (x) / sizeof (x [0]))
#define NVRAM_PREFIX "wan_dsl."

/* ----------------------------------------------------------------------- *
 * Private: parameter and assignment lists
 * ----------------------------------------------------------------------- */

static
struct {
    const char *var ;
    const char *val ;
} intro_avp [] = {
    {"getpage", "../html/advanced/adv_adsl.htm"}
} ;

static
const struct {
    const char *var ;
    const char *val ;
    const char *(*map)(IOBUF*, const char*);
} append_vars [] = {

    {"sar:settings/modulation",             "modulation"}
};

/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */

static
unsigned post_vars (HTDIAL *http, const char *instance, int extra) {
    int n ;
    unsigned id ;
    IOBUF i, j ;
    
    XZERO (i) ;
    XZERO (j) ;


    if (instance == 0)
        id = 0 ;
    else if (parse_unsigned (&id, instance, 0) == 0) {
        if (extra == 0)
            logger (LOG_ERR, "ip_vars: illegal instance variable \"%s\"",
                    instance);
        return 0 ;
    }

    for (n = 0; n < DIM (intro_avp) ; n++ )
        http_avp_append (http, intro_avp [n].var, intro_avp [n].val);

    j.top = 0 ;
    data_append (&j, NVRAM_PREFIX, 0);
    for (n = 0; n < DIM (append_vars) ; n++ ) {
        i.top = 0 ;
        j.top = sizeof (NVRAM_PREFIX) - 1;
        data_printf (&i, append_vars [n].var, id);
        data_printf (&j, append_vars [n].val, id);
        http_var_append (http, i.data, j.data, append_vars [n].map);
    }

    kill_space   (&i);
    kill_space   (&j);

    return 1;
}

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

const char *dslmodulation_help (unsigned indent, unsigned ff) {
    char b [20] ;
    IOBUF i ;
    XZERO (i) ;

    sprintf (b, "%%-%ds%%-%ds%%s\n", indent, ff);

    data_printf (&i, b, "", "instance:", "ignored");

    unlink_space (&i);
    return i.data ;
}

unsigned dslmodulation_extra (HTDIAL *http, const char *instance) {
    return post_vars (http, instance, 1);

    return 1;
}

unsigned dslmodulation_vars (HTDIAL *http, const char *instance) {
    int n ;

    for (n = 0; n < DIM (intro_avp) ; n++ )
        http_avp_append (http, intro_avp [n].var, intro_avp [n].val);

    return 1;
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
