/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>

#ifdef STDC_HEADERS
# include <stdlib.h>
# include <stddef.h>
#else
# ifdef HAVE_STDLIB_H
#  include <stdlib.h>
# endif
#endif

#ifdef HAVE_UNISTD_H
# include <unistd.h>
# include <sys/types.h>
# else
# ifdef HAVE_SYS_TYPES_H
#  include <sys/types.h>
# endif
#endif

#ifdef HAVE_STRING_H
# if !defined (STDC_HEADERS) && defined (HAVE_MEMORY_H)
#  include <memory.h>
# endif
# include <string.h>
#endif

#ifdef TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# ifdef HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#include "buffer.h"
#include "logger.h"
#include "nvalloc.h"
#include "http.h"

#include "set-ip.h"

/* ----------------------------------------------------------------------- *
 * Private variables and settings
 * ----------------------------------------------------------------------- */

#define DIM(x)  (sizeof (x) / sizeof (x [0]))

/* ----------------------------------------------------------------------- *
 * Private: map functions
 * ----------------------------------------------------------------------- */

/* ----------------------------------------------------------------------- *
 * Private: parameter and assignment lists
 * ----------------------------------------------------------------------- */

struct _intro_avp {
    const char *var ;
    const char *val ;
} ;


static
const struct _intro_avp intro_avp [] = {
    {"getpage", "../html/advanced/saveall.htm"},
    {"var:nextpage", "../html/advanced/adv_firewall.htm"},
};

static
const struct _intro_avp intro_off [] = {
    {"fdb:settings/service/dos/state",                   "1"},
    {"fdb:settings/service/dos_syn_flood/state",         "0"},
    {"fdb:settings/service/dos_ping_flood/state",        "0"},
    {"logic:command/save", ""},
};

static
const struct _intro_avp intro_cmds [] = {
    {"fdb:settings/service/dos/state",                   "0"},
    {"fdb:settings/service/dos_syn_flood/state",         "0"},
    {"fdb:settings/service/dos_ping_flood/state",        "0"},
    {"fdb:settings/service/port_scan/state",             "0"},
    {"fdb:settings/service/port_scan_nmap/state",        "0"},
    {"fdb:settings/service/port_scan_xmas_tree/state",   "0"},
    {"fdb:settings/service/port_scan_null_scan/state",   "0"},
    {"fdb:settings/service/port_scan_syn_rst/state",     "0"},
    {"fdb:settings/service/port_scan_syn_fin/state",     "0"},
    {"fdb:settings/service/ping/state",                  "0"},
    {"fdb:settings/service/service_filter_telnet/state", "0"},
    {"fdb:settings/service/service_filter_ftp/state",    "0"},
    {"fdb:settings/service/service_filter_dns/state",    "0"},
    {"fdb:settings/service/service_filter_ike/state",    "0"},
    {"fdb:settings/service/rip/state",                   "0"},
    {"fdb:settings/service/dhcp_relay/state",            "0"},
    {"fdb:settings/service/icmp_from_lan/state",         "0"},
    {"logic:command/save", ""},
};


/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */

static
unsigned post_vars (HTDIAL *http, const char *instance, int extra) {
    int n, dim ;
    const struct _intro_avp *tab ;

    for (n = 0; n < DIM (intro_avp) ; n++ )
        http_avp_append (http, intro_avp [n].var, intro_avp [n].val);

    if (extra) {
        dim = DIM (intro_off) ;
        tab =      intro_off  ;
    }
    else {
        dim = DIM (intro_cmds) ;
        tab =      intro_cmds  ;
    }
    
    for (n = 0; n < dim ; n++ )
        http_avp_append (http, tab [n].var, tab [n].val);

    return 1;
}

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

const char *fwoff_help (unsigned indent, unsigned ff) {
    char b [20] ;
    IOBUF i ;
    XZERO (i) ;

    sprintf (b, "%%-%ds%%-%ds%%s\n", indent, ff);
        
    data_printf (&i, b, "", "instance:", "ignored");

    unlink_space (&i);
    return i.data ;
}


const char *fwoff_getp (HTDIAL *http, const char *instance) {
    return 0;
}

unsigned fwoff_extra (HTDIAL *http, const char *instance) {
    return post_vars (http, instance, 1);
}


unsigned fwoff_vars (HTDIAL *http, const char *instance) {
    return post_vars (http, instance, 0);
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
