/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>

#ifdef STDC_HEADERS
# include <stdlib.h>
# include <stddef.h>
#else
# ifdef HAVE_STDLIB_H
#  include <stdlib.h>
# endif
#endif

#ifdef HAVE_UNISTD_H
# include <unistd.h>
# include <sys/types.h>
# else
# ifdef HAVE_SYS_TYPES_H
#  include <sys/types.h>
# endif
#endif

#ifdef HAVE_STRING_H
# if !defined (STDC_HEADERS) && defined (HAVE_MEMORY_H)
#  include <memory.h>
# endif
# include <string.h>
#endif

#ifdef TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# ifdef HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#include "logger.h"
#include "nvalloc.h"
#include "http.h"
#include "helpers.h"

#include "set-xwlan.h"

/* ----------------------------------------------------------------------- *
 * Private variables and settings
 * ----------------------------------------------------------------------- */

#define NVRAM_PREFIX "lan_performance."

#define DIM(x)  (sizeof (x) / sizeof (x [0]))

/* ----------------------------------------------------------------------- *
 * Private: map functions
 * ----------------------------------------------------------------------- */

static
const char *apw (IOBUF *b, const char *s) {
    if (strcasecmp (s, "50") == 0)
        return "2";
    if (strcasecmp (s, "25") == 0)
        return "3";
    if (strcasecmp (s, "12") == 0)
        return "4";
    if (strcasecmp (s, "6") == 0)
        return "5";
    return "1"; /* full */
}
 
static
const char *borg (IOBUF *b, const char *s) {
    if (strcasecmp (s, "11b") == 0)
        return "1";
    if (strcasecmp (s, "11b+") == 0)
        return "2";
    if (strcasecmp (s, "11g") == 0 ||
        strcasecmp (s, "11g+") == 0)
        return "3";
    return "0" ; /* mixed */
}

/* ----------------------------------------------------------------------- *
 * Private: parameter and assignment lists
 * ----------------------------------------------------------------------- */

struct _intro_avp {
    const char *var ;
    const char *val ;
};

struct _append_vars {
    const char *var ;
    const char *val ;
    const char *(*map)(IOBUF*, const char*);
};

static
const struct _intro_avp intro_avp [] = {
    {"getpage", "../html/advanced/wireless_perf.htm"},
};

static
const struct _intro_avp intro_std [] = {
    {"ap:settings/plus_4x",        "0"},
    {"ap:settings/plus_fs",        "0"},
    {"ap:settings/plus_cat",       "0"},
    {"ap:settings/plus_pb",        "0"},
    {"ap:settings/plus_mm",        "0"},
};

static
const struct _intro_avp intro_gplus [] = {
    {"ap:settings/plus_4x",        "1"},
    {"ap:settings/plus_fs",        "1"},
    {"ap:settings/plus_cat",       "1"},
    {"ap:settings/plus_pb",        "1"},
    {"ap:settings/plus_mm",        "1"},
};

static
const struct _append_vars append_vars [] = {
    {"ap:settings/beacon",         "beaconInterval"},
    {"ap:settings/dtim",           "dtim"},
    {"ap:settings/hidden_ssid",    "has_hiddenSsid", ena_is1},
    {"ap:settings/powerlevel",     "antennaPower",   apw},
    {"ap:settings/rts_threshold",  "rtsThreshold"},
    {"ap:settings/frag_threshold", "fragThreshold"},
    {"ap:settings/bgmode",         "bgMode",         borg},
};

/* ----------------------------------------------------------------------- *
 * Private
 * ----------------------------------------------------------------------- */

static
unsigned post_vars (HTDIAL *http, const char *instance, int extra) {
    int n, dim ;
    IOBUF j ;
    const char *s ;
    const struct _intro_avp *avp ;
        
    for (n = 0; n <DIM (intro_avp); n ++)
        http_avp_append (http, intro_avp [n].var, intro_avp [n].val);

    if (extra)
        return 1 ;

    if ((s = http_val (http, NVRAM_PREFIX "bgMode")) != 0 &&
        strcasecmp (s, "11g+")                       == 0) {
        dim = DIM (intro_gplus) ;
        avp =      intro_gplus  ;
    }
    else {
        dim = DIM (intro_std) ;
        avp =      intro_std  ;
    }
    
    for (n = 0; n < dim; n ++)
        http_avp_append (http, avp [n].var, avp [n].val);

    XZERO (j);
    data_append (&j, NVRAM_PREFIX, 0);
    for (n = 0; n < DIM (append_vars); n ++) {
        j.top = sizeof (NVRAM_PREFIX) - 1;
        data_append (&j, append_vars [n].val, 0);
        http_var_append
            (http, append_vars [n].var, j.data, append_vars [n].map);
    }

    kill_space (&j);

    return 1;
}

/* ----------------------------------------------------------------------- *
 * Public
 * ----------------------------------------------------------------------- */

const char *xwlan_help (unsigned indent, unsigned ff) {
    char b [20] ;
    IOBUF i ;
    XZERO (i) ;

    sprintf (b, "%%-%ds%%-%ds%%s\n", indent, ff);
    data_printf (&i, b, "", "instance:", "n/a (ignored)");
    unlink_space (&i);
    return i.data ;
}


const char *xwlan_getp (HTDIAL *http, const char *instance) {
    return 0;
}

unsigned xwlan_extra (HTDIAL *http, const char *instance) {
    return post_vars (http, instance, 1);
}


unsigned xwlan_vars (HTDIAL *http, const char *instance) {
    return post_vars (http, instance, 0);
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
