/**************************************************************************
*   Copyright (C) 2006 - D-Link (Europe) Ltd., London, UK                 *
*                                                                         *
*   Author: Jordan Hrycaj <jordan@maxina.de> - Maxina GmbH, Neu-          *
*   Isenburg, Germany							  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
*                                                                         *
***************************************************************************/



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>

#define _GNU_SOURCE
#include <getopt.h>
#undef  _GNU_SOURCE

#ifdef STDC_HEADERS
# include <stdlib.h>
# include <stddef.h>
#else
# ifdef HAVE_STDLIB_H
#  include <stdlib.h>
# endif
#endif

#ifdef HAVE_UNISTD_H
# include <unistd.h>
# include <sys/types.h>
# else
# ifdef HAVE_SYS_TYPES_H
#  include <sys/types.h>
# endif
#endif

#ifdef HAVE_STRING_H
# if !defined (STDC_HEADERS) && defined (HAVE_MEMORY_H)
#  include <memory.h>
# endif
# include <string.h>
#endif

#ifdef TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# ifdef HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#ifdef HAVE_FCNTL_H
# include <fcntl.h>
#endif

#ifdef HAVE_ERRNO_H
# include <errno.h>
#endif

#include "logger.h"
#include "util.h"
#include "varstr.h"
#include "nvalloc.h"
#include "helpers.h"
#include "settings.h"

#include "set-wpost-module.h"

#ifdef HTTP_GET_OK
#include "get-wpost-module.h"
#endif

// #ifdef HAVE_OPENSSL
// #include "ssl.h"
// #endif

/* ----------------------------------------------------------------------- *
 * variables and definitions
 * ----------------------------------------------------------------------- */

static int verbose = 0 ;

#ifdef _DEBUG
#define _MAIN_DEBUG
#endif

#ifndef DISABLE_COMPAT_MODE
#define _MAIN_COMPAT_MODE
#endif

#define __arg1of4(a,b,c,d)    a
#define __arg2of4(a,b,c,d)    b
#define __arg3of4(a,b,c,d)    c
#define __arg4of4(a,b,c,d)    d
#define __str(x)              # x
#define str(x)                __str (x)

/* I am fed up with option name and option key management using strings
   and letters at several places - so keep them together -- jordan */

#define opt_name(x)         x (__arg1of4)
#define opt_key(x)          x (__arg2of4)
#define opt_argtype(x)      x (__arg3of4)
#define opt_info(x)         x (__arg4of4)

#ifdef _MAIN_COMPAT_MODE
#define OPT_NOTIMEPFX(x)    x ("no-timestamp",  'n', 0,           "no effect (compatibility option)")
#define OPT_PARINST(x)      x ("post-instance", 'i', "INSTANCE",  "instance of the post module to be used (compatibility option)")
#define OPT_OFF_SSID(x)     x ("-p -ssid",      '?', 0,           "set parameters with module \"-ssid\" (compatibility option)")
#endif

#ifdef HTTP_GET_OK
#define OPT_PARGET(x)       x ("get-data",      'g', 0,           "query parameters from the DSL board")
#define OPT_NO_LOGIN(x)     x ("no-login",      'l', 0,           "do not request (a dummy) login before querying parameters")
#endif

#define OPT_VERSION(x)      x ("version",       'V', 0,           "print version and exit")
#define OPT_VERBOSE(x)      x ("verbose",       'v', 0,           "print diagnostic messages, use -vv etc. for more verbosity")
#define OPT_HELP(x)         x ("help",          'h', 0,           "instead of a manual page")
#define OPT_SYSLOG(x)       x ("syslog",        's', "FACILITY",  "use syslog with the given facility")
#define OPT_DOTIMEPFX(x)    x ("timestamp",     'N', 0,           "prefix the log data by a time stamp")
#define OPT_CHROOT(x)       x ("chroot",        'r', "DIR",       "chroot to DIR after initialisation")
#define OPT_PARSET(x)       x ("post-data",     'p', 0,           "set DSL board parameters from the NVRAM")
#define OPT_NVRAM_HOST(x)   x ("nvram-host",    'A', "ADDRESS",   "address of nvramd")
#define OPT_NVRAM_PORT(x)   x ("nvram-port",    'Y', "PORT",      "port of nvramd")
#define OPT_XTRA_POST(x)    x ("extra-post",    'x', 0,           "send an extra message to a brain dead config server")
#define OPT_NULL(x)         x (0,               0 , 0,            0)

#define LONG_OPTION_RECORD(x,y)  {opt_name (x), y, 0, opt_key (x)}
static const
struct option long_options [] = {
    LONG_OPTION_RECORD (OPT_VERSION,      0),
    LONG_OPTION_RECORD (OPT_VERBOSE,      0),
#   ifdef               OPT_NOTIMEPFX
    LONG_OPTION_RECORD (OPT_NOTIMEPFX,    0),
#   endif
    LONG_OPTION_RECORD (OPT_DOTIMEPFX,    0),
    LONG_OPTION_RECORD (OPT_XTRA_POST,    0),
#   ifdef               OPT_PARINST
    LONG_OPTION_RECORD (OPT_PARINST,      1),
#   endif
#   ifdef               OPT_NO_LOGIN
    LONG_OPTION_RECORD (OPT_NO_LOGIN,     0),
#   endif
#   ifndef DISCARD_INFO
    LONG_OPTION_RECORD (OPT_HELP,         0),
#   endif /* DISCARD_INFO */
    LONG_OPTION_RECORD (OPT_SYSLOG,       1),
    LONG_OPTION_RECORD (OPT_CHROOT,       1),
    LONG_OPTION_RECORD (OPT_PARSET,       0),
#   ifdef               OPT_PARGET
    LONG_OPTION_RECORD (OPT_PARGET,       0),
#   endif
    LONG_OPTION_RECORD (OPT_NVRAM_HOST,   1),
    LONG_OPTION_RECORD (OPT_NVRAM_PORT,   1),
    LONG_OPTION_RECORD (OPT_NULL,         0)
};
#undef LONG_OPTION_RECORD


static const
char short_options [] = {
    '+',
    opt_key (OPT_VERSION),
    opt_key (OPT_VERBOSE),
#   ifdef    OPT_NOTIMEPFX
    opt_key (OPT_NOTIMEPFX),
#   endif
    opt_key (OPT_DOTIMEPFX),
    opt_key (OPT_XTRA_POST),
#   ifdef    OPT_PARINST
    opt_key (OPT_PARINST),      ':',
#   endif
#   ifdef    OPT_NO_LOGIN
    opt_key (OPT_NO_LOGIN),
#   endif
#   ifndef DISCARD_INFO
    opt_key (OPT_HELP),
#   endif /* DISCARD_INFO */
    opt_key (OPT_SYSLOG),       ':',
    opt_key (OPT_CHROOT),       ':',
    opt_key (OPT_PARSET),
#   ifdef    OPT_PARGET
    opt_key (OPT_PARGET),
#   endif
    opt_key (OPT_NVRAM_HOST),   ':',
    opt_key (OPT_NVRAM_PORT),   ':',
    opt_key (OPT_NULL)
} ;


/* ----------------------------------------------------------------------- *
 * helpers
 * ----------------------------------------------------------------------- */

static
void cpry (void) {
    logger (LOG_INFO, "WPOST - main/dsl board configuration client");
    logger (LOG_INFO, "Copyright (c) 2006 - D-Link (Europe) Ltd., London, UK");
    logger (LOG_INFO, "Version " VERSION
            " -- Jordan Hrycaj <jordan@maxina.de>");
#   ifdef HAVE_OPENSSL
    logger (LOG_INFO, "SSL/TLS support: %s", ssl_version ());
#   endif
}

#ifndef DISCARD_INFO
static
void usage_line (const char indent,
                 const  char *name,
                 char          key,
                 const char   *arg,
                 const char  *info) {
    char longopt [100], format [30] ;
    strcpy (longopt, name) ;
    if (arg) { strcat (longopt, "="); strcat (longopt, arg); }
#   ifdef _MAIN_COMPAT_MODE
    if (key == '?') {
        sprintf (format, "%%%us%%-28s %%s\n", indent);
        fprintf (stderr, format, "", longopt, info);
        return ;
    }
#   endif
    sprintf (format, "%%%us-%%c, --%%-22s %%s\n", indent);
    fprintf (stderr, format, "", key, longopt, info);
}
#endif /* DISCARD_INFO */


#ifndef DISCARD_INFO
static
void uri_print (const char *name, unsigned slot) {
    if ((slot % 8) == 1)
        fprintf (stderr, "\n%-17s", "");
    fprintf (stderr, " %s", name);
}
#endif


static
void _vusage (const char *info, const char *opt, va_list ap) {

#   ifndef DISCARD_INFO
    if (info != 0) {
        fprintf (stderr, "\n%s: ERROR: ", progname);
        vfprintf (stderr, info, ap);
        if (opt) fprintf (stderr, " (option arg was %s)", opt);
        fprintf (stderr, "\n");
    }

#   define SHOW_OPTION(indent,opt) usage_line (indent,            \
                                               opt_name    (opt), \
                                               opt_key     (opt), \
                                               opt_argtype (opt), \
                                               opt_info    (opt)  )

    fprintf (stderr,
             "\nUsage: %s [options] module ["
#            ifdef OPT_PARINST
             "[-%c] "
#            endif
             "post-instance]\n", progname
#            ifdef OPT_PARINST
             , opt_key (OPT_PARINST)
#            endif
             );

    fputs ("\nGeneral options:", stderr);
    SHOW_OPTION ( 2, OPT_HELP);

    fputs ("\nLogging options:", stderr);
    SHOW_OPTION ( 2, OPT_VERBOSE);
    SHOW_OPTION (18, OPT_SYSLOG);
    SHOW_OPTION (18, OPT_DOTIMEPFX);
#   ifdef            OPT_NOTIMEPFX
    SHOW_OPTION (18, OPT_NOTIMEPFX);
#   endif

    fputs ("\nProcess control:", stderr);
    SHOW_OPTION ( 2, OPT_CHROOT);
    SHOW_OPTION (18, OPT_PARSET);
#   ifdef            OPT_OFF_SSID
    SHOW_OPTION (18, OPT_OFF_SSID);
#   endif
#   ifdef            OPT_PARINST
    SHOW_OPTION (18, OPT_PARINST);
#   endif
#   ifdef            OPT_PARGET
    SHOW_OPTION (18, OPT_PARGET);
#   endif
#   ifdef            OPT_NO_LOGIN
    SHOW_OPTION (18, OPT_NO_LOGIN);
#   endif
    SHOW_OPTION (18, OPT_XTRA_POST);

    fputs ("\nNvram server:", stderr);
    SHOW_OPTION ( 5, OPT_NVRAM_HOST);
    SHOW_OPTION (18, OPT_NVRAM_PORT);

    fprintf (stderr, "\nModules available:");
    parset_list (uri_print);
    fputs ("\n", stderr);

#   undef SHOW_OPTION
#   endif /* DISCARD_INFO */
}

static
void _usage (const char *info, const char *opt, ...) {
    va_list ap;
    va_start (ap, opt);
    _vusage (info, opt, ap);
    va_end (ap);
}

static
void usage (const char *info, const char *opt, ...) {
    va_list ap;
    va_start (ap, opt);
    _vusage (info, opt, ap);
    va_end (ap);
    exit (1);
}

#ifndef DISCARD_INFO
static
void help (const char *complain, const char *name) {

    fputs ("\n", stderr);
    syslog_init (0, 0); /* no timestamps with the introduction */
    cpry ();

    _usage (complain, 0);

    if (name) {
        fprintf (stderr, "\nModule parameter set info:\n%s",
                 parset_help (parset_open (name, (void*)1), 10));
    }
    else {
        fputs ("\nUse the " opt_name (OPT_PARSET)
               " option to get some details.\n",  stderr);
    }
    exit (1);
}
#endif /* DISCARD_INFO */

/* ----------------------------------------------------------------------- *
 * MAIN
 * ----------------------------------------------------------------------- */

#ifndef _MAIN_DEBUG
#undef main /* in case some debugging redirection applied */
#endif

int main (int argc, char *argv[]) {

    int c, option_inx ;
    MODULE m ;

#   ifdef     OPT_PARGET
    int           get_ok = 0 ;
#   endif
#   ifdef   OPT_NO_LOGIN
    int         no_login = 0 ;
#   endif
    int          post_ok = 0 ;
    int            xcode = 0 ;
    char       *new_root = 0 ;
    char       *facility = 0 ;
    int          timepfx = 0 ;
    int        xtra_post = 0 ;
#   ifndef DISCARD_INFO
    unsigned help_wanted = 0 ;
#   endif

#   ifdef WITH_DMALLOC
    dmalloc_debug_setup ("debug=0x4f47d03,log=/tmp/dmalloc.log");
#   endif

    XZERO (m);
   
    /* progname ::= only the last path segment */
    if ((progname = strrchr (argv [0], '/')) == 0)
        progname = argv [0] ;
    else
        ++ progname ;
    
    while ((c = getopt_long (argc, argv, short_options, 
                             long_options, &option_inx)) != -1)
        switch (c) {
        case opt_key (OPT_CHROOT):
            new_root = optarg;
            continue ;
#       ifdef         OPT_NOTIMEPFX
        case opt_key (OPT_NOTIMEPFX):
            /* compatibility only */
            continue ;
#       endif
#       ifdef         OPT_PARINST
        case opt_key (OPT_PARINST):
            /* compatibility only */
            m.inst = optarg ;
            continue ;
#       endif
        case opt_key (OPT_DOTIMEPFX):
            timepfx = 1 ;
            continue ;
#       ifdef         OPT_PARGET
        case opt_key (OPT_PARGET):
            get_ok ++ ;
            continue ;
#       endif
#       ifdef         OPT_NO_LOGIN
        case opt_key (OPT_NO_LOGIN):
            no_login ++ ;
            continue ;
#       endif
        case opt_key (OPT_PARSET):
            post_ok = optind ;
            continue ;
        case opt_key (OPT_SYSLOG):
#           ifdef OPT_OFF_SSID
            if (post_ok == optind - 1 &&
                strcmp (argv [optind - 1], opt_name (OPT_OFF_SSID)) == 0)
                m.name = opt_name (OPT_OFF_SSID);
            else
#           endif
            facility = optarg ;
            continue ;
        case opt_key (OPT_VERSION):
            cpry ();
            exit (0);
        case opt_key (OPT_VERBOSE):
            verbose ++ ;
            continue ;
        case opt_key (OPT_NVRAM_PORT):
            if (parse_unsigned (& m.v.nvport, optarg, 0) == 0 ||
                (m.v.nvport & ~0xffff)                   != 0)
                usage ("illegal port number", optarg);
            continue ;
        case opt_key (OPT_NVRAM_HOST):
            if ((m.v.nvserver = hostip (optarg)) == 0)
                usage ("illegal ip addess", optarg);
            continue ;
        case opt_key (OPT_XTRA_POST):
            xtra_post = 1;
            continue ;
#       ifndef DISCARD_INFO
        case opt_key (OPT_HELP):
            help_wanted = 1 ;
            continue ;
        case '?':
            usage (0, "?") ; /* no return */
#       endif /* DISCARD_INFO */
        default:
            usage (0,0) ; /* no return */
        }

#   ifdef _MAIN_COMPAT_MODE
    if (m.name == 0) {
#   endif
        if (optind < argc)
            m.name = argv [optind ++] ;

        if (optind < argc) {
#           ifdef OPT_PARINST
            if (argv [optind] [0] == '-' &&
                argv [optind] [1] == opt_key (OPT_PARINST) &&
                argv [optind] [2] == '\0') {
                if (++ optind >= argc)
                    usage ("post-instance data expected", 0);
            }
#           endif
            m.inst = argv [optind ++] ;
        }
#   ifdef _MAIN_COMPAT_MODE
    }
#   endif

#   ifdef OPT_PARGET
#   ifdef OPT_NO_LOGIN
    if (no_login && get_ok == 0)
        usage ("--" opt_name (OPT_PARGET) " is expected when "
               "--" opt_name (OPT_NO_LOGIN) " is present!", 0);
#   endif
#   endif

#   ifndef DISCARD_INFO
    if (help_wanted)
        help (0, m.name); /* no return */
#   endif

    if (m.name == 0)
        usage ("module name expected", 0);

    if (m.inst && post_ok == 0)
        usage ("--" opt_name (OPT_PARSET) " is expected when the "
               "post-instance parameter is present!", 0);

    if (optind < argc) {
        fprintf
            (stderr, "%s: Unexpected non-optional arguments: ", progname);
        while (optind < argc)
            fprintf (stderr, "%s ", argv [optind++]);
        fprintf (stderr, "\n");
        usage (0 ,0);
    }

    if (xtra_post && post_ok == 0)
        usage ("--" opt_name (OPT_PARSET) " is expected if the option "
               "--"  opt_name (OPT_XTRA_POST) " is used", 0);
    
    /* initialise logging subsystem */
    if (facility) {
        int devnull ;
        /* ready for closing< stdin/stdout/stderr */
        if ((devnull = open ("/dev/null", O_RDWR)) < 0)
            usage ("Cannot open /dev/null: %s", 0, strerror (errno));
        if (syslog_init (facility, timepfx) == 0)
            usage ("cannot open the syslog facility %s", 0, facility);
    }
    else {
        syslog_init (0, timepfx);
        close (1) ;
        if (dup2 (2, 1) != 1) {
            logger (LOG_ERR, "Cannot dup stdout to stderr: %s",
                    strerror (errno));
            exit (2);
        }
    }

    switch (verbose) {
    case 4:
        verbosity |= FILTER_LOG_COMP_ERROR ;
        verbosity |= WPOST_LOG_HTTP_RDATA ;
    case 3:
        verbosity |= WPOST_LOG_HTTP_REQU ;
        verbosity |= WPOST_LOG_HTTP_REQURPT ;
        verbosity |= WPOST_LOG_POST_URL ;
        verbosity |= WPOST_LOG_LOGIN_URL ;
        verbosity |= WPOST_LOG_GET_URL ;
    case 2:
        verbosity |= FILTER_LOG_NO_MATCHES ;
        verbosity |= FILTER_PRINT_ALL ;
        verbosity |= WPOST_LOG_HTTP_PCONN ;
        verbosity |= WPOST_LOG_HTTP_GCONN ;
    case 1:
        verbosity |= WPOST_LOG_HTTP_REPLY ;
    case 0:
        break ;
    default:
        verbosity = -1 ;
    }

 
    if (post_ok == 0
#       ifdef OPT_PARGET
        && get_ok == 0
#       endif
        ) {
        const char r [] = ("--" opt_name (OPT_PARSET)
#                          ifdef OPT_PARGET
                           " or --" opt_name (OPT_PARGET)
#                          endif
                           " expected!");
        if (m.name)
            help (r, m.name);
        usage (r, 0) ;
    }

    /* FIXME: basic authentication needs it for the moment */
    wpost_need_nvram (&m.v);

    /* connect to http server */
    if ((m.http = http_open (m.v.nv)) == 0)
        exit (2) ;

    /* get the proper parameter set */
    if ((m.set = parset_open (m.name, m.http)) == 0)
        exit (2);

#   ifdef OPT_PARGET
    if (get_ok) {
        const char *s = parset_geturi (m.set);
        if (s == 0)
            usage ("--" opt_name (OPT_PARGET)
                   " not supported by module \"%s\"", 0, m.name);
        if ((s = wpost_normalise_uri (&m.get,  &m,  s)) != 0)
            usage (s, 0);
    }

    if (post_ok)
#   endif
    {
        const char *s = parset_posturi (m.set);
        if (s == 0)
            usage ("--" opt_name (OPT_PARSET)
                   " not supported by module \"%s\"", 0, m.name);
        if ((s = wpost_normalise_uri (&m.post,  &m,  s)) != 0)
            usage (s, 0);
    }

    http_target     (m.http, m.addr, m.port);
    http_hdr_append (m.http, "Referer", m.post.raw.uri);
    http_hdr_append (m.http, "Authorization",
                     http_basic_auth (m.http, "sys_admin.login",
                                      "sys_admin.passwd"));
    parset_header (m.set);

#   ifdef OPT_PARGET
    if (post_ok)
#   endif
        xcode = wpost_module_set (&m, &m.post, xtra_post);

#   ifdef OPT_PARGET

#   ifdef OPI_NO_LOGIN
#    define login_ok (no_login == 0)
#   else
#    define login_ok 1
#   endif
    
    if (get_ok) {
        int n = wpost_module_get (&m, &m.get, login_ok);

        if (xcode == 0 && n)
            xcode = n ;
    }
#   endif /* OPT_PARGET */

    exit (xcode) ;
}

/* ----------------------------------------------------------------------- *
 * End
 * ----------------------------------------------------------------------- */
